﻿'''<summary>Classe Charge: encapsule toutes les caractéristiques d'une charge: intensite, type, nature, ordonnées, abscisse, etc. ...</summary>
<Serializable()> Public Class Charge
    Private _num As Integer
    Private _P1, _P2, _a, _b, _c, _d, _Ox, _Oy, _Lx, _Ly As Double

    ''' <summary>
    ''' n° de cas de charge
    ''' </summary>
    Public Property Num As Integer
        Get
            Num = _num
        End Get
        Set(value As Integer)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur n° de cas de charge n'est pas numérique.")
            ElseIf value <= 0 Then
                Throw New ArgumentException("La valeur n° de cas de charge doit être positive.")
            Else
                _num = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' charge valeur gauche ou unique - Unité: MN ou MN/m
    ''' </summary>
    Public Property P1 As Double
        Get
            P1 = _P1
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur charge valeur gauche ou unique n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New ArgumentException("La valeur charge valeur gauche ou unique doit être positive.")
            Else
                _P1 = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' charge valeur droite - Unité: MN ou MN/m
    ''' </summary>
    Public Property P2 As Double
        Get
            P2 = _P2
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur charge valeur droite n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New ArgumentException("La valeur charge valeur droite doit être positive ou nulle.")
            Else
                _P2 = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' distance charge depuis nu appui gauche - Unité: m
    ''' </summary>
    Public Property a As Double
        Get
            a = _a
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur distance charge depuis nu appui gauche n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New ArgumentException("La valeur distance charge depuis nu appui gauche doit être positive ou nulle.")
            Else
                _a = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' longueur application charge - Unité: m
    ''' </summary>
    Public Property b As Double
        Get
            b = _b
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur longueur application charge n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New ArgumentException("La valeur longueur application charge doit être positive.")
            Else
                _b = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' distance charge (complémentaire) - Unité: m
    ''' </summary>
    Public Property c As Double
        Get
            c = _c
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur distance charge (complémentaire) n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New ArgumentException("La valeur distance charge (complémentaire) doit être positive.")
            Else
                _c = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' distance charge depuis nu appui droit - Unité: m
    ''' </summary>
    Public Property d As Double
        Get
            d = _d
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur distance charge depuis nu appui droit n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New ArgumentException("La valeur distance charge depuis nu appui droit doit être positive.")
            Else
                _d = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' nature de la charge: 1 Permanente :2 Catégorie A ou B: 3 Exploitation Catégorie C ou D: 4 Exploitation Catégorie E: 5 Exploitation Catégorie F: 6 Exploitation Catégorie G: 7 Exploitation Catégorie H
    ''' : 8 Neige : 9 Vent"
    ''' </summary>
    Public Property Nature As Integer
    ''' <summary>
    ''' schéma de la charge: linéaire, triangulaire, ponctuelle, ...
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property schem As Integer
    ''' <summary>
    ''' Si charge suspendue - =1 charge supérieure; =2 charge inférieure
    ''' </summary>
    Public Property suspendue As Integer
    ''' <summary>
    ''' % de charge permanente pour le calcul de flèche de prédalle
    ''' </summary>
    Public Property taux As Double
    ''' <summary>
    ''' Abscisse du point O depuis le bord supérieur gauche de la dalle
    ''' </summary>
    Public Property Ox As Double
        Get
            Ox = _Ox
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur Abscisse du point O depuis le bord supérieur gauche de la dalle n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New ArgumentException("La valeur Abscisse du point O depuis le bord supérieur gauche de la dalle doit être positive ou nulle.")
            Else
                _Ox = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' Ordonnée du point O depuis le bord supérieur gauche de la dalle
    ''' </summary>
    Public Property Oy As Double
        Get
            Oy = _Oy
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur Ordonnée du point O depuis le bord supérieur gauche de la dalle n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New ArgumentException("La valeur Ordonnée du point O depuis le bord supérieur gauche de la dalle doit être positive ou nulle.")
            Else
                _Oy = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' Dimension de la charge dans le sens X - unité: m
    ''' </summary>
    Public Property Lx As Double
        Get
            Lx = _Lx
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur Dimension de la charge dans le sens X n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New Exception("La valeur Dimension de la charge dans le sens X doit être positive.")
            Else
                _Lx = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' Dimension de la charge dans le sens Y - unité: m
    ''' </summary>
    Public Property Ly As Double
        Get
            Ly = _Ly
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur Dimension de la charge dans le sens Y n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New ArgumentException("La valeur Dimension de la charge dans le sens Y doit être positive.")
            Else
                _Ly = value
            End If
        End Set
    End Property
End Class
