﻿Imports System.Math
''' <summary>
''' Classe encapsulant toutes les caractéristiques d'un plancher
''' </summary>
<Serializable()> Public Class Plancher

    Private _Lx, _Ly, _h, x_a1, x_a2, x_I, x_iv, x_iv1, x_Lcx, x_Lcy, x_VS, x_αa As Double '
    Private x_g_Or_x, x_g_Or_y, _ν_fl, _ν_eff, _Intervalle As Double
    Private _rang As Integer = 12
    Private _taille As Double = 0.2
    Private _modele_calcul As Integer
    Private _Repartition As Integer = 1
    Private _Interval As Double = 2
    Private _IntervalAinf As Double '= 0

#Region "Géométrie dalles - prédalles"
    ''' <summary>
    ''' Nom de la dalle = P1, P2, ...
    ''' </summary>
    Public Property Nom As String
    ''' <summary>
    ''' Longueur entres appuis suivant x de la dalle - unité: m.
    ''' </summary>
    Public Property Lx As Double
        Get
            Lx = _Lx
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur de la distance entres appuis suivant X de la dalle n'est pas numérique.")
            ElseIf value <= 0 Then
                Throw New ArgumentException("La valeur de la distance entres appuis suivant X de la dalle doit être positive.")
            Else
                _Lx = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' Longueur entres appuis suivant y de la dalle - unité: m.
    ''' </summary>
    Public Property Ly As Double
        Get
            Ly = _Ly
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur de la distance entres appuis suivant Y de la dalle n'est pas numérique.")
            ElseIf value <= 0 Then
                Throw New ArgumentException("La valeur de la distance entres appuis suivant Y de la dalle doit être positive.")
            Else
                _Ly = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' Calcul de la travée suivant modèle Isostatique ou pas - Isostatique = True / Hyperstatique = False
    ''' </summary>
    Public Property Isostatique As Boolean
    ''' <summary>
    ''' hauteur totale de la dalle - Unité: m.
    ''' </summary>
    Public Property H As Double
        Get
            H = _h
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur Epaisseur de la dalle n'est pas numérique.")
            ElseIf value <= 0.05 Then
                Throw New ArgumentException("La valeur Epaisseur de la dalle doit être supérieure à 5 cm.")
            Else
                _h = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' Modélisation calcul: 1 = portée dans une direction; 2 = portée dans 2 directions
    ''' </summary>
    Public Property modele_calcul As Integer
        Get
            modele_calcul = _modele_calcul
        End Get
        Set(value As Integer)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur Modèle de calcul n'est pas numérique.")
            ElseIf value <> 1 And value <> 2 Then
                Throw New ArgumentException("La valeur Modèle de calcul doit être égale à 1 ou 2.")
            Else
                _modele_calcul = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' Plancher portant dans 2 directions - Répartition du ferraillage: 1 = uniforme/ 2 = variable
    ''' </summary>
    Public Property Repartition As Integer
        Get
            Repartition = _Repartition
        End Get
        Set(value As Integer)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur Répartition n'est pas numérique.")
            ElseIf value <> 1 And value <> 2 Then
                Throw New ArgumentException("La valeur Répartition doit être égale à 1 ou 2.")
            Else
                _Repartition = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' Plancher portant dans 2 directions - Dimension de l'intervalle pour répartition non uniforme - Unité: m
    ''' </summary>
    Public Property Interval As Double
        Get
            Interval = _Interval
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("La valeur Intervalle n'est pas numérique.")
            ElseIf value < 1 Then
                Throw New ArgumentException("La valeur Intervalle doit être supérieure à 1.")
            Else
                _Interval = value
            End If
        End Set
    End Property

#Region "Données pour prédalle"
    ''' <summary>
    ''' =true ->plancher avec prédalle; =False plancher coulé en place
    ''' </summary>
    Public Property Prefa As Boolean = False
    ''' <summary>
    ''' Hauteur prédalle - unité: m.
    ''' </summary>
    Public Property Prefa_h As Double
    ''' <summary>
    ''' largeur préférentielle de la prédalle - unité: m
    ''' </summary>
    Public Property Prefa_L As Double
    ''' <summary>
    ''' Numéro de la prédalle pour la travée i
    ''' </summary>
    Public Property Prefa_Num As New List(Of Integer)
    ''' <summary>
    ''' largeur de la prédalle - unité: m
    ''' </summary>
    Public Property Prefa_Ly As New List(Of Double)
    ''' <summary>
    ''' longueur de la prédalle, inclue les éventuels repos à gauche et à droite, repos identiques - unité: m
    ''' </summary>
    Public Property Prefa_Lx As New List(Of Double)
    ''' <summary>
    ''' Qualité surface reprise: 1 = lisse/ 2 = Rugueux 
    ''' </summary>
    Public Property Prefa_Reprise As Integer
    ''' <summary>
    ''' Nombre de prédalle dans la travée i
    ''' </summary>
    Public Property Prefa_nb As Integer
    ''' <summary>
    ''' Espacement entre prédalles - unité: m
    ''' </summary>
    Public Property Prefa_esp As Double
    ''' <summary>
    ''' Avec lisse de rive prés des appuis= 1/ Sans lisse de rive + repos suivant Tab A3= 2/ Sans lisse de rive + repos forcé= 3/
    ''' </summary>
    Public Property Prefa_ReposLisse As Integer
    ''' <summary>
    ''' repos minimal prédalle sur appui GAUCHE - unité: m.
    ''' </summary>
    Public Property Prefa_ReposG As Double = 0
    ''' <summary>
    ''' repos minimal prédalle sur appui DROIT - unité: m.
    ''' </summary>
    Public Property Prefa_ReposD As Double = 0
    ''' <summary>
    ''' Pose avec étai =true ; sans étai =False  
    ''' </summary>
    Public Property Prefa_etai As Boolean = True
    ''' <summary>
    ''' temps écoulé entre l'enlèvement des étais et la mise en oeuvre des ouvrages supportés - Unité: jours - valeur de base: 30 jours
    ''' </summary>
    Public Property Prefa_t As Integer = 30
    ''' <summary>Ouvrages supportés fragiles au sens du §3.5.3 du CPT: Vrai =true ; sinon =False</summary>
    Public Property Prefa_fragile As Boolean = True
    ''' <summary>Forçage du calcul des contraintes ELS sous section non fissurée au sens du §3.1 du CPT: Vrai =true ; sinon =False</summary>
    Public Property Prefa_Contrainte As Boolean = False
    ''' <summary>
    ''' distance entre étai - unité: m.
    ''' </summary>
    Public Property Prefa_etai_L As Double
    ''' <summary>
    ''' TS sur joint entre prédalles
    ''' </summary>
    Public Property Prefa_TS_joint As New Ferraillage_TS
    ''' <summary>
    ''' TS sur rive latérale porté
    ''' </summary>
    Public Property Prefa_TS_RiveLaterale As New Ferraillage_TS
    ''' <summary>
    ''' n° de modèle de prédalle pour la prédalle d'index i
    ''' </summary>
    ''' 
    <Xml.Serialization.XmlArray(ElementName:="Prefa_model")>
    <Xml.Serialization.XmlArrayItem("Prefa_model", Type:=GetType(Integer))>
    Public Property Prefa_modele As New List(Of Integer)
#End Region

    ''' <summary>
    ''' Encapsulation de toutes les données pour la trémie de la dalle 
    ''' </summary>
    <Xml.Serialization.XmlArray(ElementName:="_Tremie")>
    <Xml.Serialization.XmlArrayItem("_Tremie", Type:=GetType(Tremie))>
    Public Property _Tremie As New List(Of Tremie)


    ''' <summary>
    ''' Inertie section brute béton
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Inert As Double
        Get
            Inert = x_I
        End Get
    End Property
    Public ReadOnly Property vs As Double
        Get
            vs = x_VS
        End Get
    End Property
    ''' <summary>
    ''' Module élastique vis à vis de la fibre sup - Unité: m3
    ''' </summary>
    Public ReadOnly Property iv As Double
        Get
            iv = x_iv
        End Get
    End Property
    ''' <summary>
    ''' Module élastique vis à vis de la fibre inf - Unité: m3
    ''' </summary>
    Public ReadOnly Property iv1 As Double
        Get
            iv1 = x_iv1
        End Get
    End Property
    ''' <summary>
    ''' Longueur de calcul de la dalle = Leff suivant sens X - Cf EN1992-1-1 A5.3.2.2 - Unité: m.
    ''' </summary>
    Public ReadOnly Property Lcx As Double
        Get
            Lcx = x_Lcx
        End Get
    End Property
    ''' <summary>
    ''' Longueur de calcul de la dalle = Leff suivant sens Y - Cf EN1992-1-1 A5.3.2.2 - Unité: m.
    ''' </summary>
    Public ReadOnly Property Lcy As Double
        Get
            Lcy = x_Lcy
        End Get
    End Property
    ''' <summary>
    ''' distance entre axe gauche et nu appui gauche - A5.3.2.2
    ''' </summary>
    Public ReadOnly Property a1 As Double
        Get
            a1 = x_a1
        End Get
    End Property
    ''' <summary>
    ''' distance entre axe droit et nu appui droit - A5.3.2.2
    ''' </summary>
    Public ReadOnly Property a2 As Double
        Get
            a2 = x_a2
        End Get
    End Property
    ''' <summary>
    ''' Caractéristiques géométriques section brute beton: Aire, Moment quadratique (Inertie), etc...
    ''' </summary>
    Public Sub Geometrie(ByVal n_Trav As Integer)
        ' Caractéristiques géométriques : Aire, Moment quadratique (Inertie), etc...
        'Variables: inert() = inertie brute de la poutre
        '           iv()    = moment statique supérieur
        '           iv1()    = moment statique inférieur
        '           vs() = distance entre le cdg et le bord supérieur de la poutre
        Dim u1, u2 As Double
        'calcul inertie brute de la dalle (sur 1 m de large)
        x_I = 1 * H ^ 3 / 12
        x_VS = H / 2
        x_iv = x_I / x_VS : x_iv1 = x_I / (H - x_VS)

        '2 appuis - Par convention suivant x
        '----->  Appui Gauche
        'Select Case AppuiG 'A5.3.2.2
        '    Case 1, 2 'béton, maçonnerie
        If 0.5 * H < 0.5 * LappG Then
            x_a1 = 0.5 * H
        Else
            x_a1 = 0.5 * LappG
        End If
        '    Case 3 'console
        '        '              x_a1 = 0
        'End Select
        '----->  Appui Droit
        'Select Case AppuiD
        '    Case 1, 2 'béton
        If 0.5 * H < 0.5 * LappD Then
            x_a2 = 0.5 * H
        Else
            x_a2 = 0.5 * LappD
        End If
        '    Case 3 'console
        '        '          x_a2 = 0
        'End Select
        If Prefa_h < 0.04 Then
            'dalle pleine coulée en place
            x_Lcx = Lx + x_a1 + x_a2 'longueur de calcul
            '----->  Appui haut et bas: Lcy
            x_Lcy = Ly   'longueur de calcul
            '----->  Coefficient de dégression horizontal
            x_αa = 0.77 + (3.5 / (Ly * Lx))
            If x_αa >= 1 Then x_αa = 1 'limite 15 m²
            If Aff.degression = False Then x_αa = 1
        Else
            'Plancher à prédalle
            Select Case n_Trav
                Case 1 'appui de rive pour la travée 1
                    x_a1 = Max(x_a1, 0.07) '7cm voir §1.2.2.2 du CPT
                Case 2 'appui de rive pour la dernière travée 
                    x_a2 = Max(x_a2, 0.07) '7cm voir §1.2.2.2 du CPT
            End Select
            x_Lcx = Lx + x_a1 + x_a2 + 0.0000000001 'longueur de calcul, + 0.00000001 pour compenser les arrondis bizarres sur les double
            '----->  Appui haut et bas: Lcy
            x_Lcy = Ly   'longueur de calcul
            '----->  Coefficient de dégression horizontal
            x_αa = 0.77 + (3.5 / (2.5 * Lx ^ 2))
            If x_αa >= 1 Then x_αa = 1 'limite 15 m²
            If x_αa < 0.7875 Then x_αa = 0.77 'limite 200m²
            If Aff.degression = False Then x_αa = 1
        End If
        x_Lcx = CInt(Math.Truncate(x_Lcx * 100)) / 100 'arrondi au cm inférieur
        '
        'Modification de Lcy si dalle porte dans les 2 sens
        If modele_calcul = 2 Then 'dalle sur 4 appuis
            '----->  Appui Haut
            If 0.5 * H < 0.5 * LappH Then
                x_a1 = 0.5 * H
            Else
                x_a1 = 0.5 * LappH
            End If
            '----->  Appui Bas
            If 0.5 * H < 0.5 * LappB Then
                x_a2 = 0.5 * H
            Else
                x_a2 = 0.5 * LappB
            End If
            x_Lcy = Ly + x_a1 + x_a2 + 0.0000000001 'longueur de calcul, + 0.00000001 pour compenser les arrondis bizarres sur les double
            x_Lcy = CInt(Math.Truncate(x_Lcy * 100)) / 100 'arrondi au cm inférieur
            If Prefa_h < 0.04 Then
                'dalle pleine coulée en place
                If x_αa >= 1 Then x_αa = 1 'limite 15 m²
                If x_αa < 0.7875 Then x_αa = 0.77 'limite 200m²
                If Aff.degression = False Then x_αa = 1
            Else
                'Plancher à prédalle
                x_αa = 0.77 + (3.5 / (2.5 * Lx ^ 2))
                If x_αa >= 1 Then x_αa = 1 'limite 15 m²
                If x_αa < 0.7875 Then x_αa = 0.77 'limite 200m²
                If Aff.degression = False Then x_αa = 1
            End If
        End If


    End Sub
#End Region

#Region "Données complémentaires pour le calcul de plaque sur 4 appuis"
    ''' <summary>
    ''' Coefficient de Poisson spécifique au calcul de flèche - unité: aucune
    ''' </summary>
    Public Property ν_fl As Double
        Get
            ν_fl = _ν_fl
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur du coefficient de Poisson n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur du coefficient de Poisson doit être positive.")
            ElseIf value >= 1 Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur du coefficient de Poisson doit être inférieure à 1.")
            Else
                _ν_fl = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' Coefficient de Poisson spécifique au calcul des efforts - unité: aucune
    ''' </summary>
    Public Property ν_eff As Double
        Get
            ν_eff = _ν_eff
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur du coefficient de Poisson n'est pas numérique.")
            ElseIf value < 0 Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur du coefficient de Poisson doit être positive.")
            ElseIf value >= 1 Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur du coefficient de Poisson doit être inférieure à 1.")
            Else
                _ν_eff = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' Précision du calcul demandé - Ne pas dépasser une valeur de 20 - unité: SO
    ''' </summary>
    Public Property Rang As Integer
        Get
            Rang = _rang
        End Get
        Set(value As Integer)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur Précision du calcul demandé n'est pas numérique.")
            ElseIf value < 6 Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur Précision du calcul demandé doit être supérieur à 5.")
            ElseIf value > 20 Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur Précision du calcul demandé doit être inférieure à 21.")
            Else
                _rang = value
            End If
        End Set
    End Property
    ''' <summary>
    ''' Taille de la maille de calcul demandé - Ne pas descendre en dessous d'une valeur de 10cm - unité: m
    ''' </summary>
    Public Property Taille As Double
        Get
            Taille = _taille
        End Get
        Set(value As Double)
            If Not IsNumeric(value) Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur taille de la maille de calcul demandé n'est pas numérique.")
            ElseIf value < 0.1 Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur taille de la maille de calcul demandé doit être supérieur ou égal à 10 cm.")
            ElseIf value > 1 Then
                Throw New ArgumentException("Plancher bidirectionnel: La valeur taille de la maille de calcul demandé doit être inférieur à 1 m.")
            Else
                _taille = value
            End If
        End Set
    End Property
#End Region

#Region "Données pour Appui"

#Region "Appui gauche"
    ''' <summary>
    ''' Type d'appui: 1/béton, 2/maçonnerie, 3/autre
    ''' </summary>
    Public Property AppuiG As Integer = 1
    '''' <summary>
    '''' Résistance du béton pour appui gauche
    '''' </summary>
    'Public Property BappG As Integer = 20
    '''' <summary>
    '''' Résistance maçonnerie pour appui gauche
    '''' </summary>
    'Public Property MappG As Integer
    ''' <summary>
    ''' Largeur de l'appui gauche - unité: m.
    ''' </summary>
    Public Property LappG As Double
#End Region

#Region "Appui droit"
    ''' <summary>
    ''' Type d'appui: 1/béton, 2/maçonnerie, 3/autre
    ''' </summary>
    Public Property AppuiD As Integer = 1
    '''' <summary>
    '''' Résistance du béton pour appui droit
    '''' </summary>>
    'Public Property BappD As Integer = 20
    '''' <summary>
    '''' Résistance maçonnerie pour appui droit
    '''' </summary>
    'Public Property MappD As Integer
    ''' <summary>
    ''' Largeur de l'appui droit - unité: m.
    ''' </summary>
    Public Property LappD As Double
#End Region

#Region "Appui haut"
    ''' <summary>Type d'appui: 1/béton, 2/maçonnerie, 3/autre </summary>
    Public Property AppuiH As Integer = 1
    ''' <summary>Largeur de l'appui haut - unité: m. </summary>
    Public Property LappH As Double

#End Region

#Region "Appui Bas"
    ''' <summary>Type d'appui: 1/béton, 2/maçonnerie, 3/autre </summary>
    Public Property AppuiB As Integer = 1
    ''' <summary> Largeur de l'appui bas - unité: m. </summary>
    Public Property LappB As Double
#End Region

    ''' <summary>Longueur ancrage sur appui non porteur des aciers de répatition - valeur de base: 5cm - unité: m. </summary>
    Public Property LapHA As Double = 0.05
    ''' <summary>
    ''' Indique si l'appui est monolithique avec le plancher - Ne s'applique que pour des appuis bétons - Vrai si Monolithe
    ''' </summary>
    ''' <returns>True si monolithe sinon False; initialisé à False</returns>
    Public Property App_Monolith As Boolean = False
#End Region

#Region "Diverses et variées ..."
    ''' <summary>
    ''' Charge appliquée sur le plancher - unité: daN, daN/m, daN/m².
    ''' </summary>
    <Xml.Serialization.XmlArray(ElementName:="Chge")>
    <Xml.Serialization.XmlArrayItem("Chge", Type:=GetType(Charge))>
    Public Property Chge As New List(Of Charge)

    ''' <summary>
    ''' Angle de la bielle béton d'effort tranchant
    ''' </summary>
    Public ReadOnly Property Θ_Tr As Double = PI / 4 ' PI / 4 'angle bielle = 45°
    ''' <summary>
    ''' angle des armatures d'effort tranchant vis à vis de la fibre moyenne
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property α_tr As Double = PI / 2 'angle cadre effort tranchant = 90°
    ''' <summary>
    ''' valeur du décalage du moment pour épure
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property al As Double
    ''' <summary>
    ''' valeur coefficient δ1 Art. 5.5 EN1992-1-1 
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property δ1 As Double = 0
    ''' <summary>valeur coefficient δ2 Art. 5.5 EN1992-1-1</summary>
    ''' <value></value>
    ''' <returns>prends les valeurs 0.7 pour acier de classe B et C, et 0.8 pour acier de classe A</returns>
    ''' <remarks></remarks>
    Public Property δ2 As Double = 0
#End Region

#Region "Armatures"
    ''' <summary>
    ''' type de ferraillage en travée: 1 = Treillis Soudé, 2 = quadrillage barres HA
    ''' </summary>
    Public Property Type_Acier_Travée As Integer
    ''' <summary>
    ''' type de ferraillage sur appui gauche: 1 = Treillis Soudé, 2 = quadrillage barres HA
    ''' </summary>
    Public Property Type_Acier_Appui_Gauche As Integer
    ''' <summary>
    ''' type de ferraillage sur appui droit: 1 = Treillis Soudé, 2 = quadrillage barres HA
    ''' </summary>
    Public Property Type_Acier_Appui_Droit As Integer
    ''' <summary>
    ''' nombre de lits Maxi en travée
    ''' </summary>
    Public Property MaxnbLitTravee As Integer
    ''' <summary>
    ''' nombre de lits Maxi sur appui
    ''' </summary>
    Public Property MaxnbLitAppui As Integer
    ''' <summary>
    ''' Indique si le 2ième lit en travée couvre l'intégralité de la longueur de la travée: Vrai si couvre la travée / Faux si uniquement que sur une partie de la travée
    ''' </summary>
    Public Property Lit2CouvertureTravee As Boolean = False
    ''' <summary>
    ''' Choix de l'espacement maximal: 1 = zone standard/ 2 = zone sollicitée par charge concentrée/ 3 = choix libre
    ''' </summary>
    Public Property Choix_Esp As Integer = 1
    ''' <summary>
    ''' Espacement Maximum entre barres - Unité: m
    ''' </summary>
    Public Property EspMax As Double
    ''' <summary>
    ''' Espacement minimum entre barres - Unité: m
    ''' </summary>
    Public Property Espmin As Double
    ''' <summary>
    ''' Espacement Maximum entre barres Armatures prinicpales - Unité: m
    ''' </summary>
    Public Property EspMaxPrincipal As Double
    ''' <summary>
    ''' Espacement minimum entre barres Armatures prinicpales - Unité: m
    ''' </summary>
    Public Property EspminPrincipal As Double
    ''' <summary>
    ''' Espacement Maximum entre barres Armatures Secondaires - Unité: m
    ''' </summary>
    Public Property EspMaxSecondaire As Double
    ''' <summary>
    ''' Espacement minimum entre barres Armatures Secondaires - Unité: m
    ''' </summary>
    Public Property EspminSecondaire As Double
    ''' <summary>
    ''' Quantité d'acier nécessaire à la reprise du moment sollicitant sur l'intervalle longitudinalement (sens X) - quantité: m²
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property As_Inf_X As New List(Of Double)
    ''' <summary>
    ''' Quantité d'acier nécessaire à la reprise du moment sollicitant sur l'intervalle Transversalement (sens Y) - quantité: m²
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property As_Inf_Y As New List(Of Double)

#Region "Ferraillage à base de TS"
    ''' <summary>
    ''' caractéristique TS en travée
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property TS_trav As New List(Of Ferraillage_TS)
    ''' <summary>
    ''' caractéristique TS en travée disposé en portefeuille - L'indice correspond au TS rallongé
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property TS_trav_Portefeuille As New List(Of Ferraillage_TS)
    ''' <summary>
    ''' caractéristique TS sur appui Gauche
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property TS_AppG As New List(Of Ferraillage_TS)
    ''' <summary>
    ''' caractéristique TS sur appui Droit
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property TS_AppD As New List(Of Ferraillage_TS)
    ''' <summary>
    ''' caractéristique TS sur appui Haut
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property TS_AppH As New List(Of Ferraillage_TS)
    ''' <summary>
    ''' caractéristique TS sur appui Bas
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property TS_AppB As New List(Of Ferraillage_TS)
#End Region


#Region "Tous les lits de barres HA constituant le ferraillage d'un plancher"
    ''' <summary>
    ''' Armature longitudinale en partie basse de la dalle - sens porteur X
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property LitInf As New List(Of Ferraillage_HA)
    ''' <summary>
    ''' Armature longitudinale en partie basse de la dalle - acier de répartition ou sens porteur Y
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property LitInf_Y As New List(Of Ferraillage_HA)
    ''' <summary>
    ''' Armature longitudinale en partie haute de la dalle: appui gauche - sens porteur X
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property LitSupG As New List(Of Ferraillage_HA)
    ''' <summary>
    ''' Armature longitudinale en partie haute de la dalle: appui gauche - acier de répartition ou sens porteur Y
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property LitSupG_Y As New List(Of Ferraillage_HA)
    ''' <summary>
    ''' Armature longitudinale en partie haute de la dalle: appui droit - sens porteur X
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property LitSupD As New List(Of Ferraillage_HA)
    ''' <summary>
    ''' Armature longitudinale en partie haute de la dalle: appui droit - acier de répartition ou sens porteur Y
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property LitSupD_Y As New List(Of Ferraillage_HA)
    ''' <summary>
    ''' Barres HA en partie haute de la dalle sur appui haut - appui non porteur
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property LitSupH As New List(Of Ferraillage_HA)
    ''' <summary>
    ''' Barres HA en partie haute de la dalle sur appui bas - appui non porteur
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property LitSupB As New List(Of Ferraillage_HA)
    ''' <summary>
    ''' Armature complémentaire pour glissement sur appui gauche
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property LitAppG As New Ferraillage_HA
    ''' <summary>
    ''' Armature complémentaire pour glissement sur appui droit
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    <Xml.Serialization.XmlIgnore> Public Property LitAppD As New Ferraillage_HA
#End Region

    ''' <summary>
    ''' Armature d'effort tranchant
    ''' </summary>
    Public Property HATr As New Ferraillage_HA_Tranchant
    ''' <summary>
    ''' Enrobage génératrice inférieure
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property enrInf As Double
    ''' <summary>
    ''' Enrobage génératrice supérieure
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property enrSup As Double
    ''' <summary>
    ''' Enrobage génératrice latérale
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property enrLat As Double
    ''' <summary>
    ''' bras de levier d pour acier en travée
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property dInf As Double
    ''' <summary>
    ''' bras de levier d pour acier sur appui gauche
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property dSupG As Double
    ''' <summary>
    ''' bras de levier d pour acier sur appui droit
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property dSupD As Double
    Public Sub Init_Ferraillage()
        Dim i As Integer
        'Armatures principales
        'Par hypothèse sens X
        Select Case Choix_Esp
            Case 1 'zone standard
                If 3 * H > 0.4 Then
                    EspMaxPrincipal = 0.4
                Else
                    EspMaxPrincipal = 3 * H
                End If
            Case 2 'zone charge concentrée
                If 2 * H > 0.25 Then
                    EspMaxPrincipal = 0.25
                Else
                    EspMaxPrincipal = 2 * H
                End If
            Case 3 'esp max imposé
                EspMaxPrincipal = EspMax
        End Select
        'Armatures secondaire
        'dans le cas ou le sens Y est aussi un sens porteur, les valeurs calculées ci-dessus s'imposeront.

        Select Case Choix_Esp
            Case 1 'zone standard
                If 3.5 * H > 0.45 Then
                    EspMaxSecondaire = 0.45
                Else
                    EspMaxSecondaire = 3.5 * H
                End If
            Case 2 'zone charge concentrée
                If 3 * H > 0.4 Then
                    EspMaxSecondaire = 0.4
                Else
                    EspMaxSecondaire = 3 * H
                End If
            Case 3 'esp max imposé
                EspMaxSecondaire = EspMax
        End Select
    End Sub
#End Region

#Region "Rdm"

    ''' <summary>
    ''' Effort tranchant enveloppe ELU Maxi (>0)
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property VMaxELU As New Dictionary(Of Integer, Double)
    ''' <summary>
    ''' Effort tranchant enveloppe ELU mini (inférieur à 0)
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public Property VminELU As New Dictionary(Of Integer, Double)
    ''' <summary>
    ''' coefficient de dégression horizontal: valeur de base  = 1
    ''' </summary>
    Public Property αa As Double = 1
#End Region

#Region "Résultats du calcul RdM"
    ''' <summary>
    ''' Classe encapsulant toutes les caractéristiques des efforts résultants d'une combinaison à l'état limite ELU Str
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public ELU_Str As New List(Of Result)
    ''' <summary>
    ''' Classe encapsulant toutes les caractéristiques des efforts résultants de la combinaison enveloppe de l'état limite ELU Str
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public ELU_Str_Env As New Result
    ''' <summary>
    ''' Classe encapsulant toutes les caractéristiques des efforts résultants des combinaisons de l'état limite ELU EQU équilibre statique = non soulèvement
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public ELU_EQ As New List(Of Result)
    ''' <summary>
    ''' Classe encapsulant toutes les caractéristiques des efforts résultants des combinaisons de l'état limite ELS Caractéristiques
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public ELS_Car As New List(Of Result)
    ''' <summary>
    ''' Classe encapsulant toutes les caractéristiques des efforts résultants de la combinaison enveloppe de l'Etat Limite de Service Caractéristique ELS_Car
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public ELS_Car_Env As New Result
    ''' <summary>
    ''' Classe encapsulant toutes les caractéristiques des efforts résultants des combinaisons de l'état limite ELS Quasi-Permanent
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public ELS_QP As New List(Of Result)
    ''' <summary>
    ''' Classe encapsulant toutes les caractéristiques des efforts résultants de la combinaison enveloppe de l'Etat Limite de Service Quasi-Permanent ELS_QP
    ''' </summary>
    <Xml.Serialization.XmlIgnore> Public ELS_QP_Env As New Result

#End Region

#Region "Intermédiaire Calcul BA"
    ''' <summary>
    ''' Plancher portant dans 2 directions - Section d'acier de flexion par intervalle - Unité: m²/ml
    ''' </summary>
    Public Property Interval_Ainf As New List(Of Double)

#End Region

#Region "Classe d'exposition"
    ''' <summary>
    ''' Forçage de l'enrobage: Vrai forçage / Faux pas de forçage
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Enrobage As Boolean = False
    Public Property X0 As Boolean = False
    Public Property XC1 As Boolean = False
    Public Property XC2 As Boolean = False
    Public Property XC3 As Boolean = False
    Public Property XC4 As Boolean = False
    Public Property XD1 As Boolean = False
    Public Property XD2 As Boolean = False
    Public Property XD3 As Boolean = False
    Public Property XS1 As Boolean = False
    Public Property XS2 As Boolean = False
    Public Property XS3 As Boolean = False
    Public Property XF1 As Boolean = False
    Public Property XF2 As Boolean = False
    Public Property XF3 As Boolean = False
    Public Property XF4 As Boolean = False
    Public Property XA1 As Boolean = False
    Public Property XA2 As Boolean = False
    Public Property XA3 As Boolean = False
#End Region

#Region "Feu"
    ''' <summary>
    ''' Classement au feu de la travée - 1 = REI30/ 2=REI60/ 3=REI90/ 4=REI120/ 5=REI180/ 6=REI240
    ''' </summary>
    Public Property Feu_REI As Integer = 0
    ''' <summary>
    ''' epaisseur du revetement incombustible - Unité: m
    ''' </summary>
    Public Property feu_EpCarrelage As Double = 0
#End Region

    Sub New()
        If Me.Interval_Ainf.Count >= 1 Then
            If Me.Interval_Ainf.Item(0) < 1 Then

            End If
        End If

    End Sub
    Private Sub Poubelle()
        'Public Sub GeomBA(ByVal nct As Double, ByVal nlt As Double)
        '    Dim As1, Sctnc, Sltnc, Sctc, Sltc, x_I As Double
        '    As1 = LitInf.Item(0).Aire + LitInf.Item(1).Aire + LitInf.Item(2).Aire

        '    If b0 = 0 Then
        '        'calcul inertie poutre rectangulaire
        '        '-> non fissurée
        '        Sctnc = b * h + nct * As1
        '        x_Ictnc = (b * h ^ 2 / 2 + nct * As1 * dInf) / Sctnc
        '        Sltnc = b * h + nlt * As1
        '        x_Iltnc = (b * h ^ 2 / 2 + nlt * As1 * dInf) / Sltnc
        '        '-> fissurée
        '        Sctc = b * h + nct * As1
        '        x_Ictc = (b * h ^ 2 / 2 + nct * As1 * dInf) / Sctc
        '        Sltc = b * h + nlt * As1
        '        x_Iltc = (b * h ^ 2 / 2 + nlt * As1 * dInf) / Sltc
        '    Else
        '        'calcul inertie poutre en Té
        '        '-> non fissurée
        '        Sctnc = b0 * h0 + b * (h - h0) + nct * As1
        '        x_I=(b0*h0^3/12+(b0*h0)*(h0/2)^2)+(b*(h-h0)^3/12+(b*(h-h0))*(h0+h/2)^2)+()
        '        x_Ictnc = ((b0 * h0 ^ 2 / 2) + ((h0 + (h - h0) / 2) * b * (h - h0)) + nct * As1 * dInf) / Sctnc

        '        Sltnc = b * h + nlt * As1
        '        x_Iltnc = (b * h ^ 2 / 2 + nlt * As1 * dInf) / Sltnc
        '    End If
        'End Sub
        ''' <summary>
        ''' Inertie section homogène béton + acier - section non fissurée - sans fluage
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        'Public ReadOnly Property IshCt As Double
        '    Get
        '        IshCt = x_Ictnc
        '    End Get
        'End Property
        ' ''' <summary>
        ' ''' Inertie section homogène béton + acier - section non fissurée - avec fluage
        ' ''' </summary>
        ' ''' <value></value>
        ' ''' <returns></returns>
        ' ''' <remarks></remarks>
        'Public ReadOnly Property IshLt As Double
        '    Get
        '        IshLt = x_Iltnc
        '    End Get
        'End Property
        ' ''' <summary>
        ' ''' Inertie section homogène réduite béton + acier - section fissurée - sans fluage
        ' ''' </summary>
        ' ''' <value></value>
        ' ''' <returns></returns>
        ' ''' <remarks></remarks>
        'Public ReadOnly Property IshrCt As Double
        '    Get
        '        IshrCt = x_Ictc
        '    End Get
        'End Property
        ' ''' <summary>
        ' ''' Inertie section homogène réduite béton + acier - section fissurée - avec fluage
        ' ''' </summary>
        ' ''' <value></value>
        ' ''' <returns></returns>
        ' ''' <remarks></remarks>
        'Public ReadOnly Property IshrLt As Double
        '    Get
        '        IshrLt = x_Iltc
        '    End Get
        'End Property
    End Sub

End Class

''' <summary>
''' Classe encapsulant toutes les caractéristiques d'une trémie
''' </summary>
<Serializable()> Public Class Tremie
    ''' <summary>
    ''' Trémie: largeur (dimension perpendiculaire au sens de la portée) - unité: m
    ''' </summary>
    Public Property _Ly As Double
    ''' <summary>
    ''' Trémie: longueur (dimension dans le sens de la portée) - unité: m
    ''' </summary>
    Public Property _Lx As Double
    ''' <summary>
    ''' Trémie: position du point A suivant y - unité: m
    ''' </summary>
    Public Property _yA As Double
    ''' <summary>
    ''' Trémie: position du point A suivant x - unité: m
    ''' </summary>
    Public Property _xA As Double
    ''' <summary>
    ''' Armatures disposées sur les cotées de la trémie suivant sens X
    ''' </summary>
    Public Property _HA_Lx As New Ferraillage_HA
    ''' <summary>
    ''' Largeur sur laquelle les Armatures sont disposées sur les cotées de la trémie suivant sens X
    ''' </summary>
    Public Property _b_Lx As Double
    ''' <summary>
    ''' Armatures disposées sur les cotées de la trémie suivant sens Y
    ''' </summary>
    Public Property _HA_Ly As New Ferraillage_HA
    ''' <summary>
    ''' Largeur sur laquelle les Armatures sont disposées sur les cotées de la trémie suivant sens Y
    ''' </summary>
    Public Property _b_Ly As Double
    ''' <summary>
    ''' Trémie: cloison de poids > 3 kN/ml en bordure de trémie - True si vrai / False si faux
    ''' </summary>
    Public Property _Cloison As Boolean = False
    ''' <summary>
    ''' Trémie: indique le type de chevêtre en bordure de trémie - 0 si aucun chevêtre / 1 si modèle 1(aciers sur prédalle) / 2 si modèle 2 (aciers dans prédalle)
    ''' </summary>
    Public Property _type_chevetre As Integer = 2
    ''' <summary>
    ''' Armatures de suspente disposées sur les cotées de la trémie suivant sens X (sens porteur)
    ''' </summary>
    Public Property _HA_Lx_Susp As Ferraillage_HA_Tranchant
    ''' <summary>
    ''' Armatures de suspente disposées sur les cotées de la trémie suivant sens Y (sens perpendiculaire au sens porteur) =  chevêtre
    ''' </summary>
    Public Property _HA_Ly_Susp As Ferraillage_HA_Tranchant
End Class
