﻿Imports System.Math
Imports System.IO

Module RdM_Poutre
    ''COMMENTAIRES :
    ''les moments sont pris positifs lorsque la fibre inférieure de la poutre est tendue, et négatifs lorsque la fibre inférieure de la poutre est comprimée.
    '         EIrotCaquotgg = obtention du produit E(module) x I(Inertie) x rotation par la méthode de CAQUOT pour l'appui gauche
    '         EIrotCaquotdd = obtention du produit E(module) x I(Inertie) x rotation par la méthode de CAQUOT pour l'appui droit
    'Les formules de RDM sont tirés de l'article IA n°68 des Annales de l'ITBTP n°479 de décembre 1989
    Private Dalle As New Plancher
    Private Comb As New Combinaison
    Private chge As Charge
    ''' <summary>Programme général de calcul Rdm de poutre continue suivant la méthode des 3 moments </summary>
    ''' <param name="Miso">Moment isostatique de la travée avec les indices (comb, i1, x) comb = n° de combinaison, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="Mur">Moment Ultime résistant avec les indices (ELU_ELS, i1, j, 1) ELU_ELs = n° de combinaison, i1 = n° de travée, j=1,2 ou 3 entravée, sur appui gauche, sur appui droit, 
    ''' 1 = n° de lit, de la surface vers l'intérieur de la dalle</param>
    '''  <param name="Mmax">Moment enveloppe Maxi de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    '''  <param name="Mmin">Moment enveloppe mini de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="IMmin">indice de la combinaison conduisant à la valeur maximale du moment sur appui droit de la travée i (en valeur absolue) avec les indices (ELU_ELS, i2): ELU_ELS = type d'etat Limite, i2 = n° de travée</param>
    ''' <param name="M"> Valeur du moment M(imax, PlanCont.Count, lc) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    ''' <param name="V">Valeur de l'effort tranchant V(imax, PlanCont.Count, lcMax) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    ''' <param name="verif"> variable prenant la valeur False si une erreur bloquante s'est produite pendant le calcul</param>
    ''' <param name="m1d">moment décalé pour tenir compte de l'effort tranchant en travée</param>
    ''' <param name="m2d">moment décalé pour tenir compte de l'effort tranchant sur appui gauche</param>
    ''' <param name="m3d">moment décalé pour tenir compte de l'effort tranchant sur appui droit</param>
    Public Sub Calcul_RdM_Poutre(ByRef M(,,) As Double, ByRef Mmax(,,) As Double, ByRef Mmin(,,) As Double, ByRef m1d(,,) As Double, ByRef m2d(,,) As Double, ByRef m3d(,,) As Double,
                          ByRef Mur(,,,) As Double, ByRef x1MAX(,) As Integer, ByRef V(,,) As Double, ByRef ΔM(,) As Double, ByRef IMmin(,) As Integer, ByRef EIf(,,) As Double,
                                 ByRef Miso(,,) As Double, ByRef Nbapp As Integer, ByRef Ry(,) As Double, ByRef Verif As Boolean) 'ByRef rtf As To_Rtf.RtfAnsiTextWriter,


        'Variables: inert() = inertie brute de la dalle
        '           iv()    = moment statique supérieur = I/v
        '           iv1()    = moment statique inférieur = I/v'
        '           vs() = distance entre le cdg et le bord supérieur de la poutre
        '           elu_els = 1=elu fondamental, =2: elu accidentel, =3: els
        '           calcul_G = lors du calcul interne et de l'assemblage, indique si le calcul est effectué avec des charges permanentes
        '                      (calcul_G=True) ou si les calculs sont effectués avec des charges d'exploitation (calcul_G=false)

        Dim lcMax, i1, i2, imax As Integer 'i,i3,
        Dim Ind1(4), Ind2(4) As Integer '
        Dim αa As Double
        Dim Mx(), Vx(), EIrotx(), EIfx(), EIrot(,,), Mapp(,) As Double

        lcMax = 0 'détermination de la longueur de portée maximale
        For Each Dalle In PlanCont
            i2 = PlanCont.IndexOf(Dalle) ' = n° de dalle
            Select Case i2 'pour le calcul de la valeur d'appui
                Case 0
                    Dalle.Geometrie(1) 'calcul inertie, Moment statique, aire,etc. ...
                Case PlanCont.Count - 1
                    Dalle.Geometrie(2) 'calcul inertie, Moment statique, aire,etc. ...
                Case Else
                    Dalle.Geometrie(0) 'calcul inertie, Moment statique, aire,etc. ...
            End Select
            If CInt(Truncate(Dalle.Lcx * 100)) > lcMax Then lcMax = CInt(Truncate(Dalle.Lcx * 100))
        Next

        'Détermination des indices de combinaisons
        Ind1(1) = 1
        Ind2(1) = Aff.ELU : imax = Aff.ELU
        If Aff.ELU_EQ > 0 Then 'ELU EQU équilibre statique = non soulèvement
            Ind1(2) = imax + 1
            Ind2(2) = Aff.ELU_EQ
            imax = Aff.ELU_EQ  '+ combMax(2) + combMax(3) + combMax(4)
        End If
        If Aff.ELScar > 0 Then 'ELS caractéristique
            Ind1(3) = imax + 1
            Ind2(3) = Aff.ELScar
            imax = Aff.ELScar
        End If
        If Aff.ELSQP > 0 Then 'ELS quasi-permanent
            Ind1(4) = imax + 1
            Ind2(4) = Aff.ELSQP
            imax = Aff.ELSQP
        End If

        ReDim Miso(imax, PlanCont.Count, lcMax), M(imax, PlanCont.Count, lcMax), V(imax, PlanCont.Count, lcMax)
        ReDim EIrot(imax, PlanCont.Count, lcMax), EIf(imax, PlanCont.Count, lcMax), Mapp(imax, PlanCont.Count + 1)
        ReDim Mmax(4, PlanCont.Count, lcMax), Mmin(4, PlanCont.Count, lcMax)
        ReDim m1d(4, PlanCont.Count, lcMax), m2d(4, PlanCont.Count, lcMax), m3d(4, PlanCont.Count, lcMax)
        ReDim Ry(Aff.ELU + Aff.ELU_EQ, Nbapp)
        '--------------------------------------------------------------------------
        '                            Programme principal
        '---------------------------------------------------------------------------

        For Each Dalle In PlanCont
            For Each Comb In Combinaisons
                i2 = PlanCont.IndexOf(Dalle) ' = n° de dalle
                For Each chge In PlanCont.Item(PlanCont.IndexOf(Dalle)).Chge 'Charges
                    'i3 = PlanCont.Item(PlanCont.IndexOf(Dalle)).Chge.IndexOf(chge) '= Charges.IndexOf(chge) = n° de charges
                    If Comb.Trav = PlanCont.IndexOf(Dalle) + 1 And Comb.Charge = chge.Num Then 'And Comb.Trav = dalle.Chge.Trav  base en 0 et non en 1
                        ': i1 = Comb.Num = n° de combinaison
                        Select Case chge.schem
                            Case 1 To 7
                                αa = 1
                                'Si charge d'exploitation de catégorie A, B, C ou D et charge uniformément répartie
                                ' If Aff.degression = True And (chge.Nature = 2 Or chge.Nature = 3) = True And chge.schem = 1 Then αa = Dalle.αa
                                Call Efforts_internes(1, αa, Mx, Vx, EIrotx, EIfx) 'charge=CP, n°accés=1, nCP(i2, i3)=type de charge
                                Call Assemblage(Comb.Num, i2 + 1, Mx, Vx, EIrotx, EIfx, Miso, V, EIrot, EIf) 'i2+1 pour base à partir de 1 et pas 0
                            Case 8, 9, 10, 12, 13, 14, 15
                                Call Efforts_internes(1, 1, Mx, Vx, EIrotx, EIfx) 'charge CP, accés 1, type de charge= 8,9,...
                                Call Assemblage(Comb.Num, i2 + 1, Mx, Vx, EIrotx, EIfx, Miso, V, EIrot, EIf)
                                Call Efforts_internes(2, 1, Mx, Vx, EIrotx, EIfx) 'charge CP, accés 2, nCP(i2, i3)=type de charge
                                Call Assemblage(Comb.Num, i2 + 1, Mx, Vx, EIrotx, EIfx, Miso, V, EIrot, EIf)
                            Case 11, 16
                                'Call Efforts_internes(i2, i3, 1, chge.schem, pu, P2, Mx, Vx, EIrotx, EIfx) 'charge=CP, n°accés=1, type de charge=11
                                'Call Assemblage(i1, i2 + 1, Mx, Vx, EIrotx, EIfx, Miso, V, EIrot, EIf)
                                'Call Efforts_internes(i2, i3, 2, chge.schem, pu, P2, Mx, Vx, EIrotx, EIfx) 'charge=CP, n°accés=2, type de charge=11
                                'Call Assemblage(i1, i2 + 1, Mx, Vx, EIrotx, EIfx, Miso, V, EIrot, EIf)
                                'Call Efforts_internes(i2, i3, 3, chge.schem, pu, P2, Mx, Vx, EIrotx, EIfx) 'charge=CP, n°accés=3, type de charge=11
                                'Call Assemblage(i1, i2 + 1, Mx, Vx, EIrotx, EIfx, Miso, V, EIrot, EIf)
                            Case 17
                                'Call Efforts_internes(i2, i3, 1, chge.schem, pu, P2, Mx, Vx, EIrotx, EIfx)
                                'Call Assemblage(i1, i2 + 1, Mx, Vx, EIrotx, EIfx, Miso, V, EIrot, EIf)
                                'Call Efforts_internes(i2, i3, 2, chge.schem, pu, P2, Mx, Vx, EIrotx, EIfx)
                                'Call Assemblage(i1, i2 + 1, Mx, Vx, EIrotx, EIfx, Miso, V, EIrot, EIf)
                                'Call Efforts_internes(i2, i3, 3, chge.schem, pu, P2, Mx, Vx, EIrotx, EIfx)
                                'Call Assemblage(i1, i2 + 1, Mx, Vx, EIrotx, EIfx, Miso, V, EIrot, EIf)
                                'Call Efforts_internes(i2, i3, 4, chge.schem, pu, P2, Mx, Vx, EIrotx, EIfx)
                                'Call Assemblage(i1, i2 + 1, Mx, Vx, EIrotx, EIfx, Miso, V, EIrot, EIf)
                        End Select
                    End If
                Next
            Next
        Next

        If PlanCont.Count > 1 Then
            'détermination du nombre d'appui
            If PlanCont.Item(PlanCont.Count - 1).LappD = 0 And PlanCont.Item(0).LappG = 0 Then 'double console à droite et à gauche
                Nbapp = PlanCont.Count - 1
            ElseIf PlanCont.Item(PlanCont.Count - 1).LappD > 0.05 And PlanCont.Item(0).LappG = 0 Then
                Nbapp = PlanCont.Count
            ElseIf PlanCont.Item(PlanCont.Count - 1).LappD < 0.05 And PlanCont.Item(0).LappG > 0.05 Then
                Nbapp = PlanCont.Count
            Else 'aucune console
                Nbapp = PlanCont.Count + 1
            End If

            For i1 = 1 To imax

                Call Moment_sur_appuis(i1, Miso, EIrot, Nbapp, Mapp, Verif)
                If Verif = False Then
                    ' libération de mémoire
                    Erase EIf, EIfx, EIrotx, EIrot, Mapp, Mx, Vx '
                    Exit Sub
                End If
            Next
            'détermination des combinaisons Maxi et mini sur appui n°2 en combinaison ELU Str
            Dim i_combMax, i_Combmin As Integer
            Dim Mapp_Max, Mapp_min As Double
            '-- Initialisation
            Mapp_Max = Abs(Mapp(1, 2)) : i_combMax = 1
            Mapp_min = Abs(Mapp(1, 2)) : i_Combmin = 1
            '-- Recherche
            For i1 = 2 To Aff.ELU
                If Abs(Mapp(i1, 2)) > Mapp_Max Then Mapp_Max = Abs(Mapp(i1, 2)) : i_combMax = i1
                If Abs(Mapp(i1, 2)) < Mapp_min Then Mapp_min = Abs(Mapp(i1, 2)) : i_Combmin = i1
            Next
            'calcul des éléments de réduction
            For i1 = 1 To imax
                Call Eléments_de_réduction(i1, Miso, Mapp, Nbapp, i_combMax, i_Combmin, M, V, EIf)
            Next
        Else
            Dim x As Integer '= abscisse du moment
            For i1 = 1 To imax
                With PlanCont.Item(0)
                    For x = 0 To CInt(Truncate(.Lcx * 100))
                        M(i1, 1, x) = Miso(i1, 1, x)
                    Next
                End With
            Next
            Nbapp = 2
        End If


        Call diagramme_enveloppe(Ind1, Ind2, M, V, Mmax, Mmin, IMmin, lcMax)
        Call Reaction_d_appui(V, Nbapp, Ry)
        'libération de mémoire pour les éléments ne servant plus pour la suite du calcul
        Erase EIfx, EIrotx, EIrot, Mapp, Mx, Vx ', vxecrete, Vecrete
        'Fin de la détermination des efforts Rdm et appel au module de ferraillage
        '  Call Eurocode2(Mmax, Mmin, EIfl, Nbapp, Ry, M, V, IMmin, m1d, m2d, m3d, Mur, x1MAX, ΔM, rtf, Verif)

    End Sub
    Private Sub Efforts_internes(ByVal Tour As Integer, ByVal αa As Double, ByRef Mx() As Double, ByRef Vx() As Double, ByRef EIrotx() As Double, ByRef EIfx() As Double)

        '---------------------------------------------------------------------------
        '   Calcul des Efforts Internes dans la poutre isostatique correspondante
        '---------------------------------------------------------------------------
        ' Attention: les distances sont transformées en distance entre axes et non plus
        '            en distances entre nus d'appui
        'Variables:  
        '            tour     = nbre entier indiquant le nbre d'accés
        '              αa = coefficient de dégression
        '            a        = distance entre l'appui gauche et la charge (PP1)
        '            b        = longueur d'appui de la charge
        '            c        = distance entre l'appui droit et la charge  (PP2)
        '            PP1      = charge (en DaN) sur son bord gauche - pu = chge.P1 * Comb.Pond
        '            PP2      = charge (en DaN) sur son bord droit (=bord gauche si charge continue égale) - P2 = chge.P2 * Comb.Pond

        Dim a, b, c, PP1, PP2 As Double

        With Dalle
            Select Case chge.schem                   '
                Case 1  'charge linéaire uniformément répartie
                    a = .a1 : b = .Lx : c = .a2 : PP1 = αa * chge.P1 * Comb.Pond : PP2 = αa * chge.P1 * Comb.Pond
                Case 2  'charge triangulaire - P à droite
                    a = .a1 : b = .Lx : c = .a2 : PP1 = 0 : PP2 = chge.P1 * Comb.Pond
                Case 3  'charge triangulaire - P à gauche
                    a = .a1 : b = .Lx : c = .a2 : PP1 = chge.P1 * Comb.Pond : PP2 = 0
                Case 4 'charge linéaire partielle
                    a = .a1 + chge.a : b = chge.b : c = .Lcx - a - b : PP1 = chge.P1 * Comb.Pond : PP2 = chge.P1 * Comb.Pond
                Case 5 'charge concentrée
                    PP1 = chge.P1 * Comb.Pond : a = chge.a + .a1
                Case 6 'charge trapézoidale totale
                    a = .a1 : b = .Lx : c = .a2 : PP1 = chge.P1 * Comb.Pond : PP2 = chge.P2 * Comb.Pond
                Case 7 'charge trapézoidale partielle
                    a = .a1 + chge.a : b = chge.b : c = .Lcx - a - b : PP1 = chge.P1 * Comb.Pond : PP2 = chge.P2 * Comb.Pond
                Case 8  'charge linéaire égale partielle
                    If Tour = 1 Then
                        a = .a1
                    Else
                        a = .Lx + .a1 - chge.a
                    End If
                    b = chge.a : c = .Lcx - a - b : PP1 = chge.P1 * Comb.Pond : PP2 = chge.P1 * Comb.Pond
                Case 9 'charge linéaire non égale partielle
                    If Tour = 1 Then
                        a = .a1 : b = chge.a : PP1 = chge.P1 * Comb.Pond : PP2 = chge.P1 * Comb.Pond
                    Else
                        a = .Lx + .a1 - chge.b : b = chge.b : PP1 = chge.P2 * Comb.Pond : PP2 = chge.P2 * Comb.Pond
                    End If
                    c = .Lcx - a - b
                Case 10 'charge répartie non uniforme
                    If Tour = 1 Then
                        a = .a1 : b = chge.a : PP1 = chge.P1 * Comb.Pond : PP2 = chge.P1 * Comb.Pond
                    Else
                        a = .Lcx - .a2 - chge.a : b = .Lx - chge.a : PP1 = chge.P2 * Comb.Pond : PP2 = chge.P2 * Comb.Pond
                    End If
                    c = .Lcx - a - b
                Case 11 'charges réparties non uniforme
                    Select Case Tour
                        Case 1
                            a = .a1 : b = chge.a : PP1 = chge.P2 * Comb.Pond : PP2 = chge.P2 * Comb.Pond
                        Case 2
                            a = .a1 + chge.a : b = chge.b : PP1 = chge.P1 * Comb.Pond : PP2 = chge.P1 * Comb.Pond
                        Case 3
                            a = .a1 + chge.a + chge.b : b = chge.a : PP1 = chge.P2 * Comb.Pond : PP2 = chge.P2 * Comb.Pond
                    End Select
                    c = .Lcx - a - b
                Case 12 'charge triangulaire totale
                    If Tour = 1 Then
                        a = .a1 : b = chge.a : PP1 = 0 : PP2 = chge.P1 * Comb.Pond
                    Else
                        a = .a1 + chge.a : b = .Lx - chge.a : PP1 = chge.P1 * Comb.Pond : PP2 = 0
                    End If
                    c = .Lcx - a - b
                Case 13 'Charge triangulaire opposée
                    If Tour = 1 Then
                        a = .a1 : b = chge.a : PP2 = 0 : PP1 = chge.P1 * Comb.Pond
                    Else
                        a = .a1 + chge.a : b = .Lx - chge.a : PP2 = chge.P1 * Comb.Pond : PP1 = 0
                    End If
                    c = .Lcx - a - b
                Case 14 'charge triangulaire ponctuelle
                    If Tour = 1 Then
                        a = .a1 + chge.a : b = chge.b : PP1 = 0 : PP2 = chge.P1 * Comb.Pond
                    Else
                        a = .a1 + chge.a + chge.b : b = chge.c : PP1 = chge.P1 * Comb.Pond : PP2 = 0
                    End If
                    c = .Lcx - a - b
                Case 15 'charges triangulaires isolées
                    If Tour = 1 Then
                        a = .a1 : b = chge.a : PP1 = chge.P1 * Comb.Pond : PP2 = 0
                    Else
                        a = .a1 + (.Lx - chge.b) : b = chge.b : PP1 = 0 : PP2 = chge.P2 * Comb.Pond
                    End If
                    c = .Lcx - a - b
                Case 16 'charges triangulaires à pentes différentes
                    Select Case Tour
                        Case 1
                            a = .a1 + chge.a : b = chge.b : PP1 = 0 : PP2 = chge.P1 * Comb.Pond
                        Case 2
                            a = .a1 + chge.a + chge.b : b = chge.c : PP1 = chge.P1 * Comb.Pond : PP2 = chge.P2 * Comb.Pond
                        Case 3
                            a = .a1 + chge.a + chge.b + chge.c : b = chge.d : PP1 = chge.P2 * Comb.Pond : PP2 = 0
                    End Select
                    c = .Lcx - a - b
                Case 17 'charges triangulaires type tortue
                    Select Case Tour
                        Case 1
                            a = .a1 : b = chge.a : PP1 = 0 : PP2 = chge.P1 * Comb.Pond
                        Case 2
                            a = .a1 + chge.a : b = chge.a : PP1 = chge.P1 * Comb.Pond : PP2 = 0
                        Case 3
                            a = .a1 + 2 * chge.a : b = chge.a : PP1 = 0 : PP2 = chge.P1 * Comb.Pond
                        Case 4
                            a = .a1 + 3 * chge.a : b = chge.a : PP1 = chge.P1 * Comb.Pond : PP2 = 0
                    End Select
                    c = .Lcx - a - b
            End Select
            Select Case chge.schem   '= type de charge
                Case 1 To 4, 6 To 17
                    If PlanCont.Item(0).LappG < 0.05 And PlanCont.IndexOf(Dalle) = 0 Then
                        '         Call ConsolG_Prépartie(i2, a, b, PP1, PP2, Mx, Vx, EIrotx, EIfx)  ' calcul de M, V, oméga, flèche
                    ElseIf PlanCont.Item(PlanCont.Count - 1).LappD < 0.05 And PlanCont.IndexOf(Dalle) = PlanCont.Count - 1 Then
                        '            Call ConsolD_Prépartie(i2, a, b, PP1, PP2, Mx, Vx, EIrotx, EIfx)
                    Else
                        Call Calcul_trapézoidal_partiel(a, b, c, PP1, PP2, Mx, Vx, EIrotx, EIfx) ' calcul de M, V, oméga, flèche, calcul_G
                    End If
                Case 5
                    If PlanCont.Item(0).LappG < 0.05 And PlanCont.IndexOf(Dalle) = 0 Then
                        '              Call ConsolG_Pconcentré(i2, a, PP1, Mx, Vx, EIrotx, EIfx)
                    ElseIf PlanCont.Item(PlanCont.Count - 1).LappD < 0.05 And PlanCont.IndexOf(Dalle) = PlanCont.Count - 1 Then
                        '              Call ConsolD_Pconcentré(i2, a, PP1, Mx, Vx, EIrotx, EIfx)
                    Else
                        '     Call Calcul_charge_concentré(i2, a, PP1, Mx, Vx, EIrotx, EIfx)
                    End If

            End Select
        End With

    End Sub

    Private Sub Calcul_charge_concentré(ByVal a As Double, ByVal PP1 As Double,
                                        ByRef Mx() As Double, ByRef Vx() As Double, ByRef EIrotx() As Double, ByRef EIfx() As Double)
        '----------------------------------------------------------------------------------------
        ' Travée isostatique : Charge concentrée
        '----------------------------------------------------------------------------------------
        'Variables: x,xx        = abcisse de la poutre en cm (entier), en m (double)
        '           vv          = moitié de la largeur d'appui
        '           Nint, Nint1 = variable entière pour le calcul de x
        '            a          = distance entre l'appui gauche et la charge
        '            bb         = distance entre l'appui droit et la charge
        '            PP1        = charge (en DaN)
        '            VA         = effort tranchant en A (appui gauche)
        '            VD         = effort tranchant en D (appui droit)

        Dim VA, xx As Double 'VD, vv,bb, kk,
        Dim Nint, Nint1, x, tabMax As Integer

        ' une idée d'implémentation de l'article 6.2.3(8)
        '   av=min(a, lc(i2) - a)
        '    If av <0.5*d    Then
        '                kk = 0.25
        '    ElseIf av < 2*d      Then
        '                    kk = av / (2*d)
        '        Else
        '            kk = 1
        '        End If

        With Dalle
            tabMax = CInt(Truncate(.Lcx * 100))
            ReDim Mx(tabMax), Vx(tabMax), EIrotx(tabMax), EIfx(tabMax)
            VA = PP1 * (1 - a / .Lcx)
            'Calcul de Mx, Vx, Rotx, fx pour chaque cm de poutre et pour chaque charge
            Nint = CInt((a * 100)) ': Nint = CInt(Nint)
            For x = 0 To Nint Step 1 ' Mx, Vx, Rotx, fx pour x<a
                xx = x / 100 ' xx en m, x en cm
                Mx(x) = PP1 * xx * (1 - a / .Lcx)
                Vx(x) = VA
                ' vxecrete(x) = kk * VA
                EIrotx(x) = (PP1 / 6) * (1 - a / .Lcx) * (3 * xx ^ 2 - 2 * a * .Lcx + a ^ 2)
                EIfx(x) = PP1 * xx / 6 * (1 - a / .Lcx) * (xx ^ 2 - 2 * a * .Lcx + a ^ 2)
            Next x
            Nint1 = CInt(Truncate(.Lcx * 100))  ': Nint1 = CInt(Nint1)
            For x = Nint To Nint1 Step 1 ' Mx, Vx, Rotx, fx pour x>a
                xx = x / 100 ' xx en m, x en cm
                Mx(x) = PP1 * a * (1 - xx / .Lcx)
                Vx(x) = -PP1 * a / .Lcx
                ' vxecrete(x) = -kk * PP1 * a / plancont.Item(i2).Lc
                EIrotx(x) = PP1 * (a / (6 * .Lcx)) * (6 * .Lcx * xx - 3 * xx ^ 2 - 2 * .Lcx ^ 2 - a ^ 2)
                EIfx(x) = PP1 * a / 6 * (1 - xx / .Lcx) * (a ^ 2 - 2 * xx * .Lcx + xx ^ 2)
            Next x
        End With

    End Sub

    Private Sub Calcul_trapézoidal_partiel(ByVal a As Double, ByVal bb As Double, ByVal c As Double, ByVal PP1 As Double, ByVal PP2 As Double,
                                           ByRef Mx() As Double, ByRef Vx() As Double, ByRef EIrotx() As Double, ByRef EIfx() As Double)
        ') ',ByVal i_charge As Integer,ByVal i2 As Integer,ByVal jj As Integer, ByVal Calcul_G As Boolean
        '----------------------------------------------------------------------------------------
        'Travée isostatique : charge trapézoidale partielle
        '----------------------------------------------------------------------------------------
        'Variables:  jj       = type de charge
        '            ecretage                  
        '            vv       = moitié de la largeur d'appui
        '            a        = distance entre l'appui gauche et la charge (PP1)
        '            bb       = longueur d'appui de la charge
        '            c        = distance entre l'appui droit et la charge  (PP2)
        '            PP1      = charge (en DaN) sur son bord gauche
        '            PP2      = charge (en DaN) sur son bord droit (=bord gauche si charge continue égale)
        '            VA       = effort tranchant en A (appui gauche)
        '            VD       = effort tranchant en D (appui droit)

        Dim EIrot0, EIrotc, VA, VD, xx As Double 'vv,
        Dim Nint, Nint1, x, tabMax As Integer
        With Dalle
            tabMax = CInt(Truncate(.Lcx * 100))
            ReDim Mx(tabMax), Vx(tabMax), EIrotx(tabMax), EIfx(tabMax)

            VA = (bb / (6 * .Lcx)) * (PP1 * (2 * bb + 3 * c) + PP2 * (bb + 3 * c))
            VD = VA - (PP1 + PP2) * bb / 2
            EIrot0 = bb ^ 3 / (120 * .Lcx) * (PP1 * (4 * bb + 15 * c) + PP2 * (bb + 5 * c)) + VD * c ^ 3 / (3 * .Lcx) - VA / (6 * .Lcx) * (a + bb) ^ 2 * (.Lcx + 2 * c)
            EIrotc = -(3 * PP1 + PP2) * bb ^ 3 / 24 + VA * (a + bb) ^ 2 / 2 + EIrot0
            'Calcul de Mx, Vx, Rotx, fx pour chaque cm de poutre et pour chaque charge
            Nint = CInt(Truncate(a * 100))
            For x = 0 To Nint Step 1 ' Mx, Vx, Rotx, fx pour x<a
                xx = x / 100
                Mx(x) = VA * xx
                Vx(x) = VA
                EIrotx(x) = VA * xx ^ 2 / 2 + EIrot0
                EIfx(x) = VA * xx ^ 3 / 6 + EIrot0 * xx
            Next x
            Nint1 = CInt(Truncate((a + bb) * 100))
            For x = Nint To Nint1 Step 1 ' Mx, Vx, Rotx, fx pour a<x<b
                xx = x / 100
                Mx(x) = -PP1 * (xx - a) ^ 2 / 2 - (PP2 - PP1) * (xx - a) ^ 3 / (6 * bb) + VA * xx
                Vx(x) = -PP1 * (xx - a) - (PP2 - PP1) * (xx - a) ^ 2 / (2 * bb) + VA
                EIrotx(x) = -PP1 * (xx - a) ^ 3 / 6 - (PP2 - PP1) * (xx - a) ^ 4 / (24 * bb) + VA * xx ^ 2 / 2 + EIrot0
                EIfx(x) = -PP1 * (xx - a) ^ 4 / 24 - (PP2 - PP1) * (xx - a) ^ 5 / (120 * bb) + VA * xx ^ 3 / 6 + EIrot0 * xx
            Next x

            Nint = CInt(Truncate(.Lcx * 100)) ' : Nint = Nint \ 1
            For x = Nint1 To Nint Step 1 ' Mx, Vx, Rotx, fx pour x>b
                xx = x / 100
                Vx(x) = VD
                Mx(x) = VD * (xx - .Lcx)
                EIrotx(x) = VD * (xx - .Lcx) ^ 2 / 2 + EIrotc - VD * c ^ 2 / 2
                EIfx(x) = VD * (xx - .Lcx) ^ 3 / 6 + (EIrotc - VD * c ^ 2 / 2) * (xx - .Lcx)
            Next x
        End With

#Region "A conserver au cas ou ..."
        'Calcul de l'effort tranchant écrété
        '    Nint = CInt(((5 * h(i2) / 6) + vv * kap(i2, 1)) * 100) ' : Nint = Nint \ 1
        '    ecretage = Nint
        '    Nint = CInt((lc(i2) - (5 * h(i2) / 6) - Lap(i2 + 1 - consolG) * kap(i2, 2)) * 100) ' : Nint = Nint \ 1
        '    x1 = Nint
        '    Nint = CInt(lc(i2) * 100) ' : Nint = Nint \ 1
        '    For x = 0 To Nint Step 1
        '        vxecrete(x) = vx(x)
        '        If jj = 1 And i_charge = 2 Then 'tracée de la courbe enveloppe - commentaire B.6.7,1 BAEL 99
        '            If x > ecretage And x <= l(i2) * 50 + vv * kap(i2, 1) * 100 Then
        '                vxecrete(x) = vxecrete(x) + ((vx(ecretage) / 4) * ((x - ecretage) / (l(i2) * 50 + vv * kap(i2, 1) * 100 - ecretage)))
        '            End If
        '            If x < x1 And x >= l(i2) * 50 + vv * kap(i2, 1) * 100 Then
        '                vxecrete(x) = vxecrete(x) - ((vx(ecretage) / 4) * ((x1 - x) / (x1 - (l(i2) * 50 + vv * kap(i2, 1) * 100))))
        '            End If
        '        End If
        '    Next x
        '    If a < (5 * h(i2) / 6) + vv * kap(i2, 1) Then
        '        Nint = CInt(((5 * h(i2) / 6) + vv * kap(i2, 1)) * 100) ': Nint = Nint \ 1
        '        For x = 0 To Nint Step 1
        '            Nint1 = CInt((((5 * h(i2) / 6) + vv * kap(i2, 1)) * 100)) ' \ 1 ': Nint1 = CInt(Nint1)
        '            vxecrete(x) = vx(Nint1)
        '        Next x
        '    End If
        '    If c < (5 * h(i2) / 6) + Lap(i2 + 1 - consolG) * kap(i2, 2) Then
        '        Nint = CInt(lc(i2) * 100) ' : Nint = Nint \ 1
        '        For x = x1 To Nint Step 1
        '            vxecrete(x) = vx(x1)
        '        Next x
        '    End If

        '    If type_calcul <> 1 Then
        '        If i2 = 1 Or i2 = nbtrav Then
        '            EIrotCaquotgg = EIrotx(0)
        '            EIrotCaquotdd = EIrotx(Nint)
        '        Else
        '            If a + bb < 0.8 * lc(i2) Then
        '                cc = 0.8 * lc(i2) - a - bb
        '                VAcaquot = (bb / (6 * 0.8 * lc(i2))) * (PP1 * (2 * bb + 3 * cc) + PP2 * (bb + 3 * cc))
        '                VDcaquot = VAcaquot - (PP1 + PP2) * bb / 2
        '                EIrotCaquotgg = (bb ^ 3 / (120 * 0.8 * lc(i2))) * (PP1 * (4 * bb + 15 * cc) + PP2 * (bb + 5 * cc)) + VDcaquot * cc ^ 3 / (3 * 0.8 * lc(i2)) - VAcaquot / (6 * 0.8 * lc(i2)) * (a + bb) ^ 2 * (0.8 * lc(i2) + 2 * cc)
        '            Else
        '                If a > 0.8 * lc(i2) Then
        '                    EIrotCaquotgg = 0
        '                Else
        '                    bb1 = 0.8 * lc(i2) - a : PP2caquot = PP1 + ((PP2 - PP1) * bb1 / bb)
        '                    cc = 0
        '                    VAcaquot = (bb1 / (6 * 0.8 * lc(i2))) * (PP1 * (2 * bb1 + 3 * cc) + PP2caquot * (bb1 + 3 * cc))
        '                    VDcaquot = VAcaquot - (PP1 + PP2caquot) * bb1 / 2
        '                    EIrotCaquotgg = (bb1 ^ 3 / (120 * 0.8 * lc(i2))) * (PP1 * (4 * bb1 + 15 * cc) + PP2caquot * (bb1 + 5 * cc)) + VDcaquot * cc ^ 3 / (3 * 0.8 * lc(i2)) - VAcaquot / (6 * 0.8 * lc(i2)) * (a + bb1) ^ 2 * (0.8 * lc(i2) + 2 * cc)
        '                End If
        '            End If
        '            If a > 0.2 * lc(i2) Then
        '                aa = a - 0.2 * lc(i2)
        '                VAcaquot = (bb / (6 * 0.8 * lc(i2))) * (PP1 * (2 * bb + 3 * c) + PP2 * (bb + 3 * c))
        '                VDcaquot = VAcaquot - (PP1 + PP2) * bb / 2
        '                EIrot0caquot = bb ^ 3 / (120 * 0.8 * lc(i2)) * (PP1 * (4 * bb + 15 * c) + PP2 * (bb + 5 * c)) + VDcaquot * c ^ 3 / (3 * 0.8 * lc(i2)) - VAcaquot / (6 * 0.8 * lc(i2)) * (aa + bb) ^ 2 * (0.8 * lc(i2) + 2 * c)
        '                EIrotCaquotdd = -(3 * PP1 + PP2) * bb ^ 3 / 24 + VAcaquot * (aa + bb) ^ 2 / 2 + EIrot0caquot - VDcaquot * c ^ 2 / 2
        '            Else
        '                If a + bb < 0.2 * lc(i2) Then
        '                    EIrotCaquotdd = 0
        '                Else
        '                    aa = 0 : bb1 = a + bb - 0.2 * lc(i2) : PP1caquot = PP1 + ((PP2 - PP1) * bb1 / bb)
        '                    VAcaquot = (bb1 / (6 * 0.8 * lc(i2))) * (PP1caquot * (2 * bb1 + 3 * c) + PP2 * (bb1 + 3 * c))
        '                    VDcaquot = VAcaquot - (PP1caquot + PP2) * bb1 / 2
        '                    EIrot0caquot = bb1 ^ 3 / (120 * 0.8 * lc(i2)) * (PP1caquot * (4 * bb1 + 15 * c) + PP2 * (bb1 + 5 * c)) + VDcaquot * c ^ 3 / (3 * 0.8 * lc(i2)) - VAcaquot / (6 * 0.8 * lc(i2)) * (aa + bb1) ^ 2 * (0.8 * lc(i2) + 2 * c)
        '                    EIrotCaquotdd = -(3 * PP1caquot + PP2) * bb1 ^ 3 / 24 + VAcaquot * (aa + bb1) ^ 2 / 2 + EIrot0caquot - VDcaquot * c ^ 2 / 2
        '                End If
        '            End If
        '        End If
        '    End If
        '    'vérification de St à L/2 en raison de la variation de Q
        '    If i1 <= combMax(1) Then
        '        If choix_effort_tranchant = 1 Then  'i1=n° combinaison, i2=n° travée
        '            If Calcul_G = False Then
        '                If jj = 1 Or jj = 6 Then
        '                    If PP2 > 0 Then PP1 = (PP1 + PP2) / 2
        '                    Vvariable(i1, i2) = Vvariable(i1, i2) + PP1 * lc(i2) / 8
        '                End If
        '            End If
        '        End If
        '    End If
#End Region'    

    End Sub

    Private Sub ConsolG_Pconcentré(ByVal i2 As Integer, ByVal a As Double, ByVal PP1 As Double, ByRef Mx() As Double, ByRef Vx() As Double,
                                        ByRef EIrotx() As Double, ByRef EIfx() As Double)
        '---------------------------------------------------------
        '     Console à gauche : charge concentrée
        '---------------------------------------------------------
        'Variables:  i2 = n° de l'index de poutre = n° de travée - 1 (base en 0 et pas en 1 !!!!)
        Dim xx As Double
        Dim Nint, Nint1, x, tabMax As Integer
        tabMax = CInt(Truncate(PlanCont.Item(i2).Lcx * 100))
        ReDim Mx(tabMax), Vx(tabMax), EIrotx(tabMax), EIfx(tabMax)

        Nint = CInt(a * 100) ' : Nint = Nint \ 1
        For x = 0 To Nint Step 1
            xx = x / 100
            Mx(x) = 0
            Vx(x) = 0 ': vxecrete(x) = Vx(x)
            EIrotx(x) = PP1 * (PlanCont.Item(i2).Lcx - a) ^ 2 / 2
            EIfx(x) = PP1 * (PlanCont.Item(i2).Lcx - a) ^ 3 / 6 - PP1 * (PlanCont.Item(i2).Lcx - a) ^ 2 * (PlanCont.Item(i2).Lcx - xx) / 2
        Next x
        Nint1 = CInt(Truncate(PlanCont.Item(i2).Lcx * 100)) ' \ 1 ': Nint1 = CInt(Nint1)
        For x = Nint To Nint1 Step 1
            xx = x / 100
            Mx(x) = -PP1 * (xx - a)
            Vx(x) = -PP1 ': vxecrete(x) = Vx(x)
            EIrotx(x) = (-PP1 * (PlanCont.Item(i2).Lcx - xx) ^ 2 / 2) + PP1 * (PlanCont.Item(i2).Lcx - a) * (PlanCont.Item(i2).Lcx - xx)
            EIfx(x) = (PP1 * (PlanCont.Item(i2).Lcx - xx) ^ 3 / 6) - (PP1 * (PlanCont.Item(i2).Lcx - a) * (PlanCont.Item(i2).Lcx - xx) ^ 2 / 2)
        Next x
    End Sub

    Private Sub ConsolD_Pconcentré(ByVal i2 As Integer, ByVal a As Double, ByVal PP1 As Double, ByRef Mx() As Double, ByRef Vx() As Double,
                                        ByRef EIrotx() As Double, ByRef EIfx() As Double)
        '---------------------------------------------------------
        ' Console à droite : charge concentrée
        '---------------------------------------------------------
        Dim xx As Double
        Dim Nint, Nint1, x, tabMax As Integer
        tabMax = CInt(Truncate(PlanCont.Item(i2).Lcx * 100))
        ReDim Mx(tabMax), Vx(tabMax), EIrotx(tabMax), EIfx(tabMax)

        Nint = CInt(a * 100) ': Nint = Nint \ 1
        For x = 0 To Nint Step 1
            xx = x / 100
            Mx(x) = PP1 * (xx - a)
            Vx(x) = PP1 ': vxecrete(x) = Vx(x)
            EIrotx(x) = PP1 * xx ^ 2 / 2 - PP1 * a * xx
            EIfx(x) = PP1 * xx ^ 3 / 6 - PP1 * a * xx ^ 2 / 2
        Next x
        Nint1 = CInt(Truncate(PlanCont.Item(i2).Lcx * 100)) '\ 1 ': Nint1 = CInt(Nint1)
        For x = Nint To Nint1 Step 1
            xx = x / 100
            Mx(x) = 0
            Vx(x) = 0 ': vxecrete(x) = Vx(x)
            EIrotx(x) = -PP1 * a ^ 2 / 2
            EIfx(x) = PP1 * a ^ 3 / 6 - PP1 * a ^ 2 * xx / 2
        Next x
    End Sub

    Private Sub ConsolG_Prépartie(ByVal i2 As Integer, ByVal a As Double, ByVal bb As Double, ByVal PP1 As Double, ByVal PP2 As Double, ByRef Mx() As Double,
                                  ByRef Vx() As Double, ByRef EIrotx() As Double, ByRef EIfx() As Double)
        '    '---------------------------------------------------------
        '    ' Console à gauche : charge répartie
        '    '---------------------------------------------------------
        'Variables:  i2 = n° de l'index de poutre = n° de travée - 1 (base en 0 et pas en 1 !!!!)
        Dim aa, fA, fB, MA, M0, qq, rotA, rotB, VA, xx As Double
        Dim Nint, Nint1, x, tabMax As Integer
        tabMax = CInt(Truncate(PlanCont.Item(i2).Lcx * 100))
        ReDim Mx(tabMax), Vx(tabMax), EIrotx(tabMax), EIfx(tabMax)

        qq = PP1 - PP2
        aa = a : a = PlanCont.Item(i2).Lcx - a - bb
        xx = PP2 : PP2 = PP1 : PP1 = xx
        M0 = -PP1 * bb * (a + bb / 2) - qq * (bb / 2) * (a + 2 * bb / 3)
        VA = ((PP1 * bb) + (qq * bb / 2))
        MA = -(PP1 / 2 + qq / 3) * bb ^ 2
        rotA = ((PP1 * bb + qq * bb / 2) * a ^ 2 / 2 + M0 * a)
        fA = (PP1 * bb + qq * bb / 2) * a ^ 3 / 6 + M0 * a ^ 2 / 2
        rotB = (-PP1 * bb ^ 3 / 6 - qq * bb ^ 4 / 24 + VA * bb ^ 2 / 2 + MA * bb + rotA)
        fB = -PP1 * bb ^ 4 / 24 - qq * bb ^ 4 / 120 + VA * bb ^ 3 / 6 + MA * bb ^ 2 / 2 + rotA * bb + fA
        Nint = CInt(aa * 100) ': Nint = Nint \ 1
        For x = 0 To Nint Step 1
            xx = x / 100
            xx = PlanCont.Item(i2).Lcx - xx
            Vx(x) = 0 ': vxecrete(x) = Vx(x)
            Mx(x) = 0
            EIrotx(x) = -rotB
            EIfx(x) = rotB * (xx - a - bb) + fB
        Next x
        Nint1 = CInt(((aa + bb) * 100)) '\ 1 ': Nint1 = CInt(Nint1)
        For x = Nint To Nint1 Step 1
            xx = x / 100
            xx = PlanCont.Item(i2).Lcx - xx
            Vx(x) = VA - PP1 * (xx - a) - qq * ((xx - a) ^ 2) / (2 * bb)
            Vx(x) = -Vx(x) ': vxecrete(x) = Vx(x)
            Mx(x) = -PP1 * (xx - a) ^ 2 / 2 - qq * (xx - a) ^ 3 / (6 * bb) + VA * (xx - a) + MA
            EIrotx(x) = -PP1 * ((xx - a) ^ 3 / 6) - qq * (xx - a) ^ 4 / (24 * bb) + VA * (xx - a) ^ 2 / 2 + MA * (xx - a) + rotA
            EIrotx(x) = -EIrotx(x)
            EIfx(x) = -PP1 * ((xx - a) ^ 4 / 24) - qq * (xx - a) ^ 5 / (120 * bb) + VA * (xx - a) ^ 3 / 6 + MA * (xx - a) ^ 2 / 2 + rotA * (xx - a) + fA
        Next x
        Nint = CInt(Truncate(PlanCont.Item(i2).Lcx * 100)) ': Nint = Nint \ 1
        For x = Nint1 To Nint Step 1
            xx = x / 100
            xx = PlanCont.Item(i2).Lcx - xx
            Vx(x) = -VA ': vxecrete(x) = Vx(x)
            Mx(x) = VA * xx + M0
            EIrotx(x) = -(VA * xx ^ 2 / 2 + M0 * xx)
            EIfx(x) = VA * xx ^ 3 / 6 + M0 * xx ^ 2 / 2
        Next x
    End Sub

    Private Sub ConsolD_Prépartie(ByVal i2 As Integer, ByVal a As Double, ByVal bb As Double, ByVal PP1 As Double, ByVal PP2 As Double, ByRef Mx() As Double,
                                  ByRef Vx() As Double, ByRef EIrotx() As Double, ByRef EIfx() As Double)
        '---------------------------------------------------------
        ' Console à droite : charge répartie
        '---------------------------------------------------------
        'Variables:  i2 = n° de l'index de poutre = n° de travée - 1 (base en 0 et pas en 1 !!!!)
        Dim fA, fB, MA, M0, qq, rotA, rotB, VA, xx As Double
        Dim Nint, Nint1, x, tabMax As Integer
        tabMax = CInt(Truncate(PlanCont.Item(i2).Lcx * 100))
        ReDim Mx(tabMax), Vx(tabMax), EIrotx(tabMax), EIfx(tabMax)

        qq = PP2 - PP1
        M0 = -PP1 * bb * (a + bb / 2) - qq * (bb / 2) * (a + 2 * bb / 3)
        VA = (PP1 * bb) + (qq * bb / 2)
        MA = -(PP1 / 2 + qq / 3) * bb ^ 2
        rotA = (PP1 * bb + qq * bb / 2) * a ^ 2 / 2 + M0 * a
        fA = (PP1 * bb + qq * bb / 2) * a ^ 3 / 6 + M0 * a ^ 2 / 2
        rotB = -PP1 * bb ^ 3 / 6 - qq * bb ^ 4 / 24 + VA * bb ^ 2 / 2 + MA * bb + rotA
        fB = -PP1 * bb ^ 4 / 24 - qq * bb ^ 4 / 120 + VA * bb ^ 3 / 6 + MA * bb ^ 2 / 2 + rotA * bb + fA
        Nint = CInt(a * 100) ': Nint = Nint \ 1
        For x = 0 To Nint Step 1
            xx = x / 100
            Vx(x) = VA
            'vxecrete(x) = VA
            Mx(x) = VA * xx + M0
            EIrotx(x) = VA * xx ^ 2 / 2 + M0 * xx
            EIfx(x) = VA * xx ^ 3 / 6 + M0 * xx ^ 2 / 2
        Next x
        Nint1 = CInt((a + bb) * 100) ' \ 1 ': Nint1 = CInt(Nint1)
        For x = Nint To Nint1 Step 1
            xx = x / 100
            Vx(x) = VA - PP1 * (xx - a) - qq * ((xx - a) ^ 2) / (2 * bb)
            ' vxecrete(x) = Vx(x)
            Mx(x) = -PP1 * (xx - a) ^ 2 / 2 - qq * (xx - a) ^ 3 / (6 * bb) + VA * (xx - a) + MA
            EIrotx(x) = -PP1 * ((xx - a) ^ 3 / 6) - qq * (xx - a) ^ 4 / (24 * bb) + VA * (xx - a) ^ 2 / 2 + MA * (xx - a) + rotA
            EIfx(x) = -PP1 * ((xx - a) ^ 4 / 24) - qq * (xx - a) ^ 5 / (120 * bb) + VA * (xx - a) ^ 3 / 6 + MA * (xx - a) ^ 2 / 2 + rotA * (xx - a) + fA
        Next x
        Nint = CInt(Truncate(PlanCont.Item(i2).Lcx * 100)) ': Nint = Nint \ 1
        For x = Nint1 To Nint Step 1
            xx = x / 100
            Vx(x) = 0
            ' vxecrete(x) = Vx(x)
            Mx(x) = 0
            EIrotx(x) = rotB
            EIfx(x) = rotB * (xx - a - bb) + fB
        Next x
    End Sub
    '''<summary>cAssemblage pour le calcul de </summary>
    ''' <param name="i1">n° combinaison</param>
    ''' <param name="i2">n° de travée.</param>
    Private Sub Assemblage(ByVal i1 As Integer, ByVal i2 As Integer, ByVal Mx() As Double, ByVal Vx() As Double, ByVal EIrotx() As Double, ByVal EIfx() As Double,
                           ByRef Miso(,,) As Double, ByRef V(,,) As Double, ByRef EIrot(,,) As Double, ByRef EIf(,,) As Double)
        '----------------------------------------------------------------------
        '        Assemblage pour poutre isostatique
        '----------------------------------------------------------------------
        'Variables: x,xx  = abcisse de la poutre en cm (entier), en m (double)
        '           i1 = n° de combinaison
        '           i2= n° de travée
        Dim Nint, x As Integer
        Nint = CInt(Truncate(PlanCont.Item(i2 - 1).Lcx * 100))
        For x = 0 To Nint Step 1  'Assemblage de Mx, Vx, Rotx, fx pour chaque cm de poutre
            Miso(i1, i2, x) = Miso(i1, i2, x) + Mx(x)  'i1=n° de combinaison, i2=n° de travée
            V(i1, i2, x) = V(i1, i2, x) + Vx(x)
            ' Vecrete(i1, i2, x) = Vecrete(i1, i2, x) + vxecrete(x)
            EIrot(i1, i2, x) = EIrot(i1, i2, x) + EIrotx(x)
            EIf(i1, i2, x) = EIf(i1, i2, x) + EIfx(x)
        Next x
    End Sub

    Private Sub Moment_sur_appuis(ByVal i1 As Integer, ByVal Miso(,,) As Double, ByVal EIrot(,,) As Double, ByVal nbapp As Integer, ByRef Mapp(,) As Double, ByRef verif As Boolean)
        '----------------------------------------
        '  Moment sur appuis (CLAPEYRON)
        '----------------------------------------
        'Variables: x,xx  = abcisse de la poutre en cm (entier), en m (double)
        '           i1    =  n° de combinaison
        Dim i, i2, j, Nint, nbtrav, consolG As Integer
        nbtrav = PlanCont.Count
        Dim aij(nbapp, nbapp), bij(nbapp), E_omégaD(nbtrav), E_omégaG(nbtrav) As Double
        'rotation à l'extrémité droite de la travée
        For i2 = 1 To nbtrav - 1
            Nint = CInt(Truncate(PlanCont.Item(i2 - 1).Lcx * 100))
            E_omégaD(i2) = EIrot(i1, i2, Nint) / PlanCont.Item(i2 - 1).Inert  'i1=n° de combinaison, i2=n° de travée
        Next i2
        'rotation à l'extrémité gauche de la travée
        For i2 = 2 To nbtrav
            E_omégaG(i2) = EIrot(i1, i2, 0) / PlanCont.Item(i2 - 1).Inert
        Next i2
        'mise à 0 de la matrice
        For i = 1 To nbapp
            For j = 1 To nbapp 'j1
                aij(i, j) = 0
            Next j
        Next i

        'écriture du système d'équations
        '1ière ligne
        aij(1, 1) = 1
        If PlanCont.Item(0).LappG > 0.05 Then
            Mapp(i1, 1) = 0 'pas de console
            consolG = 0
        Else
            Nint = CInt(Truncate(PlanCont.Item(0).Lcx * 100)) ' : Nint = Nint \ 1
            Mapp(i1, 1) = Miso(i1, 1, Nint)
            consolG = 1
        End If
        bij(1) = Mapp(i1, 1)
        'lignes intermédiaires = appuis intermédiaires
        For i = 2 To nbapp - 1
            aij(i, i) = 2 * ((PlanCont.Item(i + consolG - 2).Lcx / PlanCont.Item(i + consolG - 2).Inert) + (PlanCont.Item(i + consolG - 1).Lcx / PlanCont.Item(i + consolG - 1).Inert))
            aij(i, i - 1) = PlanCont.Item(i + consolG - 2).Lcx / PlanCont.Item(i + consolG - 2).Inert
            aij(i, i + 1) = PlanCont.Item(i + consolG - 1).Lcx / PlanCont.Item(i + consolG - 1).Inert
            bij(i) = 6 * (E_omégaG(i + consolG) - E_omégaD(i + consolG - 1)) '2nd membre
        Next i
        'dernière ligne
        If PlanCont.Item(nbtrav - 1).LappD > 0.01 Then
            Mapp(i1, nbapp) = 0
        Else
            Mapp(i1, nbapp) = Miso(i1, nbtrav, 0)
        End If
        aij(nbapp, nbapp) = 1
        bij(nbapp) = Mapp(i1, nbapp)
        'Résolution du système d'équation par Gauss
        Call Gauss_Pivot_Max(nbapp, i1, nbapp, aij, bij, Mapp, verif)

        Erase aij, bij, E_omégaD, E_omégaG
    End Sub

    '''<summary>Détermination de la courbe enveloppe des efforts M, V avec les moments sur appuis</summary>
    '''  <param name="i_combMax ">n° combinaison correspondant au moment Maxi sur appui n°2 à l'ELU Str</param>
    '''  <param name="i_Combmin ">n° combinaison correspondant au moment mini sur appui n°2 à l'ELU Str</param>
    ''' <param name="i1">n° combinaison</param>
    ''' <param name="Miso">Moment isostatique de la travée.</param>
    Private Sub Eléments_de_réduction(ByVal i1 As Integer, ByVal Miso(,,) As Double, ByVal Mapp(,) As Double, ByVal nbapp As Integer, ByVal i_combMax As Integer, ByVal i_Combmin As Integer, ByRef M(,,) As Double, ByRef V(,,) As Double,
                                      ByRef EIf(,,) As Double)
        '----------------------------------------------------------------------------------------
        '        Eléments de réduction pour poutre isostatique ou hyperstatique
        '---------------------------------------------------------------------------
        'Variables:
        '           i1 = n° combinaison
        '           i2 = n° de travée
        Dim i2, k1, k2, Nint, x, consolG As Integer
        Dim δ1, δ2, Δ As Double
        i2 = 0 : k1 = 1 : k2 = PlanCont.Count   'base = 0 et non 1 !!!
        consolG = 0
        'traitement de la console gauche
        If PlanCont.Item(0).LappG <= 0.05 Then 'consolG = 1
            i2 = 1
            Nint = CInt(Truncate(PlanCont.Item(i2).Lcx * 100)) ': Nint = Nint \ 1
            For x = 0 To Nint Step 1 'Assemblage de Mx, Vx, Rotx, fx pour chaque cm de poutre
                M(i1, i2, x) = Miso(i1, i2, x)  'i1=n° de combinaison, i2=n° de travée
                V(i1, i2, x) = V(i1, i2, x)
                ' Vecrete(i1, i2, x) = Vecrete(i1, i2, x)
            Next x
            k1 = 2
            consolG = 1
        End If
        'traitement de la console droite
        If PlanCont.Item(k2 - 1).LappD <= 0.05 Then 'consolD = 1
            i2 = k2 'nbtrav
            Nint = CInt(Truncate(PlanCont.Item(i2).Lcx * 100)) ': Nint = Nint \ 1
            For x = 0 To Nint Step 1 'Assemblage de Mx, Vx, Rotx, fx pour chaque cm de poutre
                M(i1, i2, x) = Miso(i1, i2, x)  'i1=n° de combinaison, i2=n° de travée
                V(i1, i2, x) = V(i1, i2, x)
                '  Vecrete(i1, i2, x) = Vecrete(i1, i2, x)
            Next x
            k2 = k2 - 1
        End If
        Δ = 1
        If Aff.δ < 0.9999999 Then
            If i1 <= Aff.ELU Then
                Δ = Aff.δ
                If i1 = i_combMax Then
                    Call Verification_Delta(1, i1, Mapp, nbapp) 'ELU_ELS
                Else
                    'If Aff.δ * Abs(Mapp(i_combMax, 2)) < Abs(Mapp(i1, 2)) Then
                    '    Δ = Abs(Aff.δ * Mapp(i_combMax, 2) / Mapp(i1, 2))
                    'Else
                    '    Δ = 1
                    'End If
                End If
            ElseIf Aff.Reco_Fr = True Then
                If Aff.ELU_EQ >= 1 Then 'si existence de combinaison ELU Equ
                    If i1 > Aff.ELU_EQ Then 'And Aff.ELScar >= 1)
                        'Suivant Reco professionnelle, redistribution aussi aux ELS  - EN 1992 autorise redistribution uniquement à l'ELU
                        Δ = Aff.δ
                    End If
                ElseIf i1 > Aff.ELU Then 'And Aff.ELScar >= 1)
                    'Suivant Reco professionnelle, redistribution aussi aux ELS  - EN 1992 autorise redistribution uniquement à l'ELU
                    Δ = Aff.δ
                End If
            End If
        End If
        Dim x1 As Double 'pour rendre une formule plus lisible
        For i2 = k1 To k2
            If i2 = 2 And PlanCont.Item(0).LappG <= 0.05 Then δ1 = 1 Else δ1 = Δ
            If i2 = PlanCont.Count - 1 And PlanCont.Item(PlanCont.Count - 1).LappD <= 0.05 Then δ2 = 1 Else δ2 = Δ
            With PlanCont.Item(i2 - 1)
                Nint = CInt(Truncate(.Lcx * 100))
                For x = 0 To Nint Step 1 'Assemblage de Mx, Vx, Rotx, fx pour chaque cm de poutre
                    M(i1, i2, x) = Miso(i1, i2, x) + δ2 * Mapp(i1, i2 - consolG + 1) * (x / Nint) + δ1 * (1 - (x / Nint)) * Mapp(i1, i2 - consolG) 'i1=n° de combinaison, i2=n° de travée
                    V(i1, i2, x) = V(i1, i2, x) + (δ2 * Mapp(i1, i2 - consolG + 1) - δ1 * Mapp(i1, i2 - consolG)) / .Lcx
                    ' Vecrete(i1, i2, x) = Vecrete(i1, i2, x) + (Mapp(i1, i2 - consolG + 1) - Mapp(i1, i2 - consolG)) / lc(i2)
                    x1 = x / 100 'en m
                    EIf(i1, i2, x) = EIf(i1, i2, x) - δ1 * Mapp(i1, i2 - consolG) * (x1 * (.Lcx - x1) * (2 * .Lcx - x1)) / (6 * .Lcx) - δ2 * Mapp(i1, i2 - consolG + 1) * (x1 * (.Lcx - x1) * (.Lcx + x1)) / (6 * .Lcx) 'TI form C2015 equ. 40

                Next x
            End With
        Next i2

    End Sub
    ''' <summary>
    ''' Vérification de la valeur du coefficient de redistribution δ
    ''' </summary>
    Private Sub Verification_Delta(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal Mapp(,) As Double, ByVal Nbapp As Integer)
        'Variables:
        '           i1 = n° combinaison
        '           i2 = n° d'appui
        Dim i2, i3 As Integer
        Dim d, fcd, Mu, µ As Double
        Dim η, λ, α, γs, γc, δi, ECU As Double 'δ1, δ2,
        Dim msg As String

        Select Case ELU_ELS
            Case 1
                γc = bet.γcd : γs = aci.γsd 'combinaison fondamentale
            Case 2
                γc = bet.γca : γs = aci.γsa 'combinaison acidentelle
        End Select
        If bet.fck > 50 Then
            η = 1 - ((bet.fck - 50) / 200) : λ = 0.8 - ((bet.fck - 50) / 400)
        Else
            η = 1 : λ = 0.8
        End If
        fcd = η * bet.fck / γc
        i3 = 1
        For i2 = 2 To Nbapp - 1 'pas de réduction de moment sur un appui en console
            If i2 = Nbapp And PlanCont.Item(PlanCont.Count - 1).LappD <= 0.05 Then Continue For
            d = PlanCont.Item(i3 - 1).H * 0.9 'valeur moyenne
            Mu = Aff.δ * Mapp(i1, i2)
            µ = Abs(Mu) / (1 * d ^ 2 * fcd)
            α = (1 - (1 - 2 * µ) ^ 0.5) / λ
            Select Case aci.classe
                Case 1 '"a"
                    If bet.fck > 50 Then δi = 0.54 + 1.25 * (0.6 + 1.4 / ECU) * α Else δi = 0.44 + 1.25 * α
                    PlanCont.Item(i3 - 1).δ2 = 0.8
                Case 2, 3 '"b", "c"
                    If bet.fck > 50 Then δi = 0.54 + 1.25 * (0.6 + 1.4 / ECU) * α Else δi = 0.44 + 1.25 * α
                    PlanCont.Item(i3 - 1).δ2 = 0.7
            End Select
            If ELU_ELS = 1 And δi > PlanCont.Item(i3 - 1).δ1 Then PlanCont.Item(i3 - 1).δ1 = δi
            If (Aff.δ >= PlanCont.Item(i3 - 1).δ2 And Aff.δ >= PlanCont.Item(i3 - 1).δ1) = False Then
                msg = "La valeur du coefficient de redistribution δ (= " & Aff.δ.ToString("#0.000") & ") ne satisfait pas les conditions de l'article 5.5 de l'EN1992-1-1:" & Environment.NewLine &
                      "  -  δ1 = " & PlanCont.Item(i3 - 1).δ1.ToString("#0.000") & "  eq 5.10" & Environment.NewLine &
                      "  -  δ2 = " & PlanCont.Item(i3 - 1).δ2.ToString("#0.000") & "  pour la classe acier correspondante"
                Aff.msg_Avertiss.Add(msg)
            End If
            i3 = i3 + 1 'n° de travée
        Next
    End Sub

    '''<summary> 
    '''Courbes enveloppes des moments et des efforts tranchants 
    '''</summary>
    ''' <param name="M">Valeur du moment M(imax, PlanCont.Count, lc) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    ''' <param name="V">Valeur de l'effort tranchant V(imax, PlanCont.Count, lcMax) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    ''' <param name="Mmax">moment Maxi à l'abscisse x sous l'Etat Limite j dans travée i</param>
    '''  <param name="Mmin">Moment enveloppe mini de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="IMmin">indice de la combinaison conduisant à la valeur maximale du moment sur appui droit de la travée i (en valeur absolue) avec les indices (ELU_ELS, i2): ELU_ELS = type d'etat Limite, i2 = n° de travée</param>
    ''' <param name="lcmax">longueur de calcul de la travée la plus longue</param>
    Private Sub diagramme_enveloppe(ByVal Ind1() As Integer, ByVal Ind2() As Integer, ByVal M(,,) As Double, ByVal V(,,) As Double, ByRef Mmax(,,) As Double, ByRef Mmin(,,) As Double, ByRef IMmin(,) As Integer,
                                    ByVal LcMax As Integer)
        '----------------------------------------------------------------------------
        '       Courbes enveloppes des moments et des efforts tranchants
        '----------------------------------------------------------------------------
        'Variables:
        '           i1 = n° de combinaison
        Dim ELU_ELS As Integer = 1
        Dim i1, i2 As Integer
        Dim Nint, x, IMmax(3), IVmax(3), IVmaxecrete(3), IVmin(3), IVminecrete(3) As Integer
        Dim Vmax(4, PlanCont.Count, LcMax), Vmin(4, PlanCont.Count, LcMax) As Double
        ReDim IMmin(4, PlanCont.Count + 1)
        'IMmax(ELU_ELS) = 1 :  : IVmax(ELU_ELS) = 1 : IVmin(ELU_ELS) = 1
        'IVmaxecrete(ELU_ELS) = 1 : IVminecrete(ELU_ELS) = 1
        '--->initialisation des MAX et min pour chaque ELU et ELS
        '--->pour l'ELU STR =1, pour l'ELU EQU = 2; pour l'ELS combinaison caractéristique = 3; pour l'ELS combinaison quasi-permanente = 4
        i1 = 1
        For ELU_ELS = 1 To 4
            If Ind2(ELU_ELS) >= 1 Then
                i1 = Ind1(ELU_ELS)
                For i2 = 1 To PlanCont.Count
                    IMmin(ELU_ELS, i2) = 1 ': If PlanCont.Count > 1 Then IMmin(ELU_ELS, i2 + 1) = 1
                    With PlanCont.Item(i2 - 1)
                        Nint = CInt(Truncate(.Lcx * 100))
                        For x = 0 To Nint Step 1
                            If M(i1, i2, x) > 0 Then
                                Mmax(ELU_ELS, i2, x) = M(i1, i2, x) : Mmin(ELU_ELS, i2, x) = 0
                            Else
                                Mmax(ELU_ELS, i2, x) = 0 : Mmin(ELU_ELS, i2, x) = M(i1, i2, x)
                                '    .Mmax.Item(ELU_ELS).M.Add(x, 0) : .Mmin.Item(ELU_ELS).M.Add(x, M(i1, i2, x))
                            End If
                            If V(i1, i2, x) > 0 Then
                                Vmax(ELU_ELS, i2, x) = V(i1, i2, x) : Vmin(ELU_ELS, i2, x) = 0
                                '   .Vmax.Item(ELU_ELS).V.Add(x, V(i1, i2, x)) : .Vmin.Item(ELU_ELS).V.Add(x, 0)
                            Else
                                Vmax(ELU_ELS, i2, x) = 0 : Vmin(ELU_ELS, i2, x) = V(i1, i2, x)
                                '  .Vmax.Item(ELU_ELS).V.Add(x, 0) : .Vmin.Item(ELU_ELS).V.Add(x, M(i1, i2, x))
                            End If
                            ' EIfl(i2, x) = Abs(EIf(i1, i2, x)) 'je ne garde que l'ELS quasi-permanent soit le dernier
                            'If Vecrete(i1, i2, x) > 0 Then
                            '    Vmaxecrete(ELU_ELS, i2, x) = Vecrete(i1, i2, x)
                            '    Vminecrete(ELU_ELS, i2, x) = 0
                            'Else
                            '    Vmaxecrete(ELU_ELS, i2, x) = 0
                            '    Vminecrete(ELU_ELS, i2, x) = Vecrete(i1, i2, x)
                            'End If
                        Next x
                    End With
                Next i2
            End If
        Next

        If (Ind2(1) = 1 And Ind2(2) - Ind1(2) < 2 And Ind2(3) - Ind1(3) < 2 And Ind2(4) - Ind1(4) < 2) = False Then  '1 seule combinaison renseignée par Etat Limite

            '--->diagramme enveloppe pour chaque Etat Limite
            For ELU_ELS = 1 To 4 'Step 2
                If Ind2(ELU_ELS) - Ind1(ELU_ELS) > 1 Then
                    For i2 = 1 To PlanCont.Count
                        With PlanCont.Item(i2 - 1)
                            Nint = CInt(Truncate(.Lcx * 100))
                            For i1 = Ind1(ELU_ELS) + 1 To Ind2(ELU_ELS)
                                For x = 0 To Nint Step 1
                                    '.Mmax.Item(ELU_ELS).M.Add(x, Max(.Mmax.Item(ELU_ELS).M.Item(x), M(i1, i2, x)))
                                    '.Mmin.Item(ELU_ELS).M.Add(x, Min(.Mmin.Item(ELU_ELS).M.Item(x), M(i1, i2, x)))
                                    '.Vmax.Item(ELU_ELS).V.Add(x, Max(.Vmax.Item(ELU_ELS).V.Item(x), V(i1, i2, x)))
                                    '.Vmin.Item(ELU_ELS).V.Add(x, Min(.Vmin.Item(ELU_ELS).V.Item(x), V(i1, i2, x)))
                                    'Mmax(ELU_ELS, i2, x) = Max(M(i1, i2, x), Mmax(ELU_ELS, i2, x))
                                    'Mmin(ELU_ELS, i2, x) = Min(M(i1, i2, x), Mmin(ELU_ELS, i2, x))
                                    If M(i1, i2, x) > 0 Then
                                        Mmax(ELU_ELS, i2, x) = Max(M(i1, i2, x), Mmax(ELU_ELS, i2, x))   ': IMmax(ELU_ELS) = i1
                                    Else
                                        'Select Case x
                                        '    Case 0
                                        '        IMmin(ELU_ELS, i2) = CInt(IIf(M(i1, i2, x) < Mmin(ELU_ELS, i2, x), i1, IMmin(ELU_ELS, i2))) 'n° de combinaison
                                        '    Case Nint
                                        '        IMmin(ELU_ELS, i2 + 1) = CInt(IIf(M(i1, i2, x) < Mmin(ELU_ELS, i2, x), i1, IMmin(ELU_ELS, i2 + 1)))
                                        '        ' IMmin(ELU_ELS, i2 + 1) = i1 'n° de combinaison
                                        'End Select

                                        'détermination de l'indice de la combinaison conduisant à la valeur maximale du moment sur appui droit de la travée i
                                        If x = Nint Then
                                            IMmin(ELU_ELS, i2) = CInt(IIf(M(i1, i2, x) < Mmin(ELU_ELS, i2, x), i1, IMmin(ELU_ELS, i2)))
                                        End If
                                        Mmin(ELU_ELS, i2, x) = Min(M(i1, i2, x), Mmin(ELU_ELS, i2, x))
                                    End If
                                    Vmax(ELU_ELS, i2, x) = Max(V(i1, i2, x), Vmax(ELU_ELS, i2, x))
                                    Vmin(ELU_ELS, i2, x) = Min(V(i1, i2, x), Vmin(ELU_ELS, i2, x))
                                    '   EIfl(i2, x) = Max(Abs(EIf(i1, i2, x)), Abs(EIfl(i2, x)))

                                    'If V(i1, i2, x) > 0 Then
                                    '    Vmax(ELU_ELS, i2, x) = Max(V(i1, i2, x), Vmax(ELU_ELS, i2, x)) ': IVmax(ELU_ELS) = i1
                                    'Else
                                    '    Vmin(ELU_ELS, i2, x) = Min(V(i1, i2, x), Vmin(ELU_ELS, i2, x)) ': IVmin(ELU_ELS) = i1
                                    'End If
                                    'If Vecrete(i1, i2, x) > 0 Then
                                    '    If Vmaxecrete(ELU_ELS, i2, x) < Vecrete(i1, i2, x) Then Vmaxecrete(ELU_ELS, i2, x) = Vecrete(i1, i2, x) : IVmaxecrete(ELU_ELS) = i1
                                    'Else
                                    '    If Vminecrete(ELU_ELS, i2, x) > Vecrete(i1, i2, x) Then Vminecrete(ELU_ELS, i2, x) = Vecrete(i1, i2, x) : IVminecrete(ELU_ELS) = i1
                                    'End If
                                Next x
                            Next i1
                        End With
                    Next i2
                End If
            Next ELU_ELS
        End If
        'Affectation
        For i2 = 1 To PlanCont.Count
            With PlanCont.Item(i2 - 1)
                .VMaxELU.Clear() : .VminELU.Clear()
                Nint = CInt(Truncate(.Lcx * 100))
                For x = 0 To Nint Step 1
                    .VMaxELU.Add(x, Vmax(1, i2, x)) : .VminELU.Add(x, Vmin(1, i2, x))
                Next
            End With
        Next
        Erase Vmax, Vmin
    End Sub

    Private Sub Gauss_Pivot_Max(ByVal nbapp As Integer, ByVal i1 As Integer, ByVal n As Integer, ByVal Akg(,) As Double, ByVal F() As Double, ByRef Mapp(,) As Double, ByRef verif As Boolean)
        ' RESOLUTION DE SYSTEME LINEAIRE par la METHODE DE LA TRIANGULATION DE GAUSS AVEC RECHERCHE DU PIVOT MAXIMUM
        ' "PREMIER MEMBRE DU SYSTEME : MATRICE A"
        '"SECOND  MEMBRE DU SYSTEME : MATRICE B"
        'matrice de dimension nxn
        Dim msg As String
        Dim i, j, L, Lmax, k, ns As Integer
        Dim m, S, m1, A(n, n), B(n), U(n) As Double
        For i = 1 To n
            B(i) = F(i)
            For j = 1 To n
                A(i, j) = Akg(i, j)
            Next
        Next
        ns = n
        For k = 1 To ns - 1
            m = A(k, k)
            Lmax = k
            For L = k + 1 To ns
                If Abs(A(L, k)) >= Abs(m) Then
                    m = A(L, k)
                    Lmax = L
                End If
            Next L
            If Abs(m) < 9.999999E-21 Then 'pivot nul
                msg = "Combinaison n°" & CStr(i1) & " - calcul du terme diagonal n°" & CStr(k) & Environment.NewLine &
                                " matrice singulière"
                MsgBox(msg, MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "RdM:Sub Gauss_Pivot_Max")
                verif = False
                Erase A, B, U, Akg, F
                Exit Sub
            End If
            If Lmax <> k Then
                If Abs(A(Lmax, k)) >= 10 * Abs(A(k, k)) Then
                    For L = k To ns
                        m1 = A(k, L) 'SWAP(A(k, L), A(Lmax, L))
                        A(k, L) = A(Lmax, L)
                        A(Lmax, L) = m1
                    Next
                    m1 = B(k) 'SWAP(B(k), B(Lmax))
                    B(k) = B(Lmax)
                    B(Lmax) = m1
                End If
            End If
            For i = k + 1 To ns
                B(i) = B(i) - A(i, k) / A(k, k) * B(k)
                For j = k + 1 To ns
                    A(i, j) = A(i, j) - A(i, k) / A(k, k) * A(k, j)
                Next j
            Next i
        Next k
        '
        'CALCUL DES SOLUTIONS
        '
        If Abs(A(ns, ns)) < 9.999999E-21 Then 'pivot nul
            msg = "Combinaison n°" & CStr(i1) & " - calcul du dernier terme diagonal" & Environment.NewLine &
                        " matrice  singulière"
            MsgBox(msg, MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "RdM:Sub Gauss_Pivot_Max")
            verif = False
            Erase A, B, U, Akg, F
            Exit Sub
        End If
        U(ns) = B(ns) / A(ns, ns)
        For i = ns - 1 To 1 Step -1
            S = 0
            For k = i + 1 To ns
                S = S + A(i, k) * U(k)
            Next k
            U(i) = (B(i) - S) / A(i, i)
        Next i
        'affectation des résultats
        For i = 1 To nbapp
            Mapp(i1, i) = U(i)
        Next
        Erase A, B, U, Akg, F
    End Sub

    Private Sub Reaction_d_appui(ByVal V(,,) As Double, ByVal Nbapp As Integer, ByRef Ry(,) As Double)
        '------------------------------------------------------------------------------
        'calcul de la réacton d'appui pour chaque appui et pour chaque combinaison
        '------------------------------------------------------------------------------
        Dim i, k, Nint As Integer
        If Aff.ELU_EQ = 0 Then
            Aff.msg_Avertiss.Add("Vérification des réactions d'appui: aucune combinaison ELU Equilibre n'a été renseignée. Un soulèvement d'appui peut se produire.")
            ReDim Ry(Aff.ELU, Nbapp)
        Else
            ReDim Ry(Aff.ELU_EQ, Nbapp)
        End If


        For i = 1 To Aff.ELU
            For k = 1 To Nbapp
                'pour les appuis d'about
                If k = 1 And PlanCont.Item(0).LappG > 0.05 Then Ry(i, k) = V(i, 1, 0)
                If k = 1 And PlanCont.Item(0).LappG <= 0.05 Then Nint = CInt(Truncate(PlanCont.Item(0).Lcx * 100)) : Ry(i, k) = V(i, 2, 0) - V(i, 1, Nint)
                If k = Nbapp And PlanCont.Item(PlanCont.Count - 1).LappD > 0.05 Then Nint = CInt(Truncate(PlanCont.Item(PlanCont.Count - 1).Lcx * 100)) : Ry(i, k) = -V(i, PlanCont.Count, Nint)
                If k = Nbapp And PlanCont.Item(PlanCont.Count - 1).LappD <= 0.05 Then
                    Nint = CInt(Truncate(PlanCont.Item(PlanCont.Count - 2).Lcx * 100)) : Ry(i, k) = V(i, PlanCont.Count - 1, 0) - V(i, PlanCont.Count - 2, Nint)
                End If
                'pour les appuis intermédiaires
                If k > 1 And k < Nbapp Then
                    Nint = CInt(Truncate(PlanCont.Item(k - 1).Lcx * 100)) ' : Nint = Nint \ 1
                    Ry(i, k) = V(i, k, 0) - V(i, k - 1, Nint)
                End If
            Next k
        Next i
        For i = Aff.ELU + 1 To Aff.ELU_EQ
            For k = 1 To Nbapp
                'pour les appuis d'about
                If k = 1 And PlanCont.Item(0).LappG > 0.05 Then Ry(i, k) = V(i, 1, 0)
                If k = 1 And PlanCont.Item(0).LappG <= 0.05 Then Nint = CInt(Truncate(PlanCont.Item(0).Lcx * 100)) : Ry(i, k) = V(i, 2, 0) - V(i, 1, Nint)
                If k = Nbapp And PlanCont.Item(PlanCont.Count - 1).LappD > 0.05 Then Nint = CInt(Truncate(PlanCont.Item(PlanCont.Count - 1).Lcx * 100)) : Ry(i, k) = -V(i, PlanCont.Count, Nint)
                If k = Nbapp And PlanCont.Item(PlanCont.Count - 1).LappD <= 0.05 Then
                    Nint = CInt(Truncate(PlanCont.Item(PlanCont.Count - 2).Lcx * 100)) : Ry(i, k) = V(i, PlanCont.Count - 1, 0) - V(i, PlanCont.Count - 2, Nint)
                End If
                'pour les appuis intermédiaires
                If k > 1 And k < Nbapp Then
                    Nint = CInt(Truncate(PlanCont.Item(k - 1).Lcx * 100)) ' : Nint = Nint \ 1
                    Ry(i, k) = V(i, k, 0) - V(i, k - 1, Nint)
                End If
            Next k
        Next
    End Sub

End Module

Module RdM_Plaque
    ''' <summary>Programme général de calcul Rdm de plaque suivant solution Navier-Lagrange </summary>
    ''' <param name="rtf">fichier sous format rtf permettant l'écriture de la note de calcul</param>
    Public Sub Calcul_Lagrange_Navier(ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim A1, A2, B1, B2, D, EcmLt, nCt, nLt, P0, U33, U4, U5, U6, Rangle_Cumul As Double 'X, Y, Rangle, U1,
        Dim i, i11, index As Integer ' Ind_Dal, Ind_Comb,
        Dim A(,), B(,), Z(17) As Double '
        Dim ϕ, β, αa As Double
        β = 0.5
        aci.Initialisation()
        bet.Initialisation()
        If aci.n_Forcer_Fl = True Then
            'Calcul avec un coefficient n forcé
            nCt = aci.n_Fl : nLt = aci.n_Fl : ϕ = (aci.n_Fl * bet.Ecm / aci.Es) - 1
            EcmLt = bet.Ecm / (1 + ϕ)
            'Call Tab_Initial_Fleche(True, ϕ, β, EcmLt, nCt, nLt, rtf)
        Else
            'Suivant §7.4.3 et suivant article CSTC = conforme à l'EN1992-1-1
            nCt = aci.Es / bet.Ecm
            Call Calcul_Coef_Fluage(Aff.t0_Fl, Aff.RH_Fl, Aff.tinf_Fl, ϕ, rtf)
            EcmLt = bet.Ecm / (1 + ϕ) : nLt = aci.Es / EcmLt
            ' Call Tab_Initial_Fleche(False, ϕ, β, EcmLt, nCt, nLt, rtf)
        End If
        Dim progressForm As New frmTaskProgress()
        progressForm.Show()
        ' La fonction Refresh provoque l'affichage instantané (non publié) de l'étiquette.
        progressForm.Refresh()
        With progressForm.ProgressIndicator
            .Minimum = 0
            .Maximum = PlanCont.Count * Combinaisons.Count

            For Each Dalle As Plancher In PlanCont
                With Dalle
                    '  Debug.WriteLine("----------------- Nouveau calcul  ------------------:")
                    For Each Comb As Combinaison In Combinaisons ' 
                        Rangle_Cumul = 0
                        D = EcmLt * .H ^ 3 / (12 * (1 - .ν_fl ^ 2))
                        ReDim A(.Rang, .Rang), B(.Rang, .Rang)
                        For Each chge As Charge In PlanCont.Item(PlanCont.IndexOf(Dalle)).Chge 'Charges 
                            If Comb.Trav = PlanCont.IndexOf(Dalle) + 1 And Comb.Charge = chge.Num Then
                                Dim R As New Result
                                'Détermination des coefficients constants liés à la charge
                                αa = 1
                                If Aff.degression = True And (chge.Nature = 2 Or chge.Nature = 3) = True And chge.schem = 1 Then αa = 1
                                P0 = αa * chge.P1 * Comb.Pond   'charge en MN/m2 
                                A1 = chge.Ox  'Abscisse début charge (m) 
                                A2 = chge.Ox + chge.Lx 'Abscisse fin charge (m) : 
                                B1 = chge.Oy ' Ordonnée début charge (m) : 
                                B2 = chge.Oy + chge.Ly 'Ordonnée fin charge (m) 
                                U33 = (A1 * PI) / Dalle.Lx : U5 = (A2 * PI) / Dalle.Lx : U4 = (B1 * PI) / Dalle.Ly : U6 = (B2 * PI) / Dalle.Ly
                                'Appel au module de calcul Navier: calcul des efforts sous maille
                                Call Module_Calcul_Navier(Dalle, P0, U33, U4, U5, U6, D, Comb.Type, R)

                                'Rangle_Cumul = Rangle_Cumul + Rangle * (2 * (1 - .ν_eff) / PI ^ 2)
                                Select Case Comb.Type
                                    Case 1 'ELU_Str
                                        If .ELU_Str.Count = 0 Then
                                            .ELU_Str.Add(R)  'initialisation de l'item 0
                                            .ELU_Str.Item(0).R_angle = Rangle_Cumul
                                            .ELU_Str.Item(0).Num = Comb.Num
                                        Else
                                            'ELUStr sert à rentrer les valeurs suivantes d'effort dans la même combinaison
                                            index = 1
                                            For Each ELUStr As Result In .ELU_Str
                                                If ELUStr.Num = Comb.Num Then
                                                    For i = 0 To R.Mx.Count - 1
                                                        ELUStr.Mx.Item(i).M += R.Mx.Item(i).M
                                                        ELUStr.My.Item(i).M += R.My.Item(i).M
                                                        ELUStr.Vx.Item(i).V += R.Vx.Item(i).V
                                                        ELUStr.Vy.Item(i).V += R.Vy.Item(i).V
                                                        ELUStr.Cz.Item(i).Cz += R.Cz.Item(i).Cz
                                                        'ELUStr.Rx.Item(i).Rx += R.Rx.Item(i).Rx
                                                        'ELUStr.Ry.Item(i).Ry += R.Ry.Item(i).Ry
                                                        ELUStr.W_fl.Item(i).wfl += R.W_fl.Item(i).wfl
                                                    Next
                                                    ELUStr.R_angle = Rangle_Cumul
                                                    Exit For
                                                End If
                                                index += 1
                                                If index > .ELU_Str.Count Then
                                                    'Ajout d'une nouvelle combinaison ELU
                                                    .ELU_Str.Add(R)
                                                    .ELU_Str.Item(.ELU_Str.Count - 1).R_angle = Rangle_Cumul
                                                    .ELU_Str.Item(.ELU_Str.Count - 1).Num = Comb.Num
                                                    Exit For
                                                End If
                                            Next
                                        End If

                                    '  Debug.WriteLine("Charge n°:" & Comb.Charge.ToString & "  Mx =" & (.ELU_Str.Item(0).Mx.Item(908).M * 1000).ToString("#0.000") & "kN.m/ml - x= " & .ELU_Str.Item(0).Mx.Item(908).x.ToString("#0.000") & "- y= " & .ELU_Str.Item(0).Mx.Item(908).y.ToString("#0.000"))

                                    Case 2 'ELU EQU équilibre statique = non soulèvement

                                    Case 3 'ELS caractéristique
                                        If .ELS_Car.Count = 0 Then
                                            .ELS_Car.Add(R)  'initialisation de l'item 0
                                            .ELS_Car.Item(0).R_angle = Rangle_Cumul
                                            .ELS_Car.Item(0).Num = Comb.Num
                                        Else
                                            index = 1
                                            For Each ELSCar As Result In .ELS_Car
                                                If ELSCar.Num = Comb.Num Then
                                                    For i = 0 To R.Mx.Count - 1
                                                        ELSCar.Mx.Item(i).M += R.Mx.Item(i).M
                                                        ELSCar.My.Item(i).M += R.My.Item(i).M
                                                        ELSCar.Vx.Item(i).V += R.Vx.Item(i).V
                                                        ELSCar.Vy.Item(i).V += R.Vy.Item(i).V
                                                        ELSCar.Cz.Item(i).Cz += R.Cz.Item(i).Cz
                                                        'ELSCar.Rx.Item(i).Rx += R.Rx.Item(i).Rx
                                                        'ELSCar.Ry.Item(i).Ry += R.Ry.Item(i).Ry
                                                        ELSCar.W_fl.Item(i).wfl += R.W_fl.Item(i).wfl
                                                    Next
                                                    ELSCar.R_angle = Rangle_Cumul
                                                    Exit For
                                                End If
                                                index += 1
                                                If index > .ELS_Car.Count Then
                                                    'initialisation d'un nouvel item
                                                    .ELS_Car.Add(R)
                                                    .ELS_Car.Item(.ELS_Car.Count - 1).R_angle = Rangle_Cumul
                                                    .ELS_Car.Item(.ELS_Car.Count - 1).Num = Comb.Num
                                                    Exit For
                                                End If
                                            Next
                                        End If
                                    Case 4 'ELS_Quasi-Permanent
                                        If .ELS_QP.Count = 0 Then
                                            .ELS_QP.Add(R)  'initialisation de l'item 0
                                            .ELS_QP.Item(0).R_angle = Rangle_Cumul
                                            .ELS_QP.Item(0).Num = Comb.Num
                                        Else
                                            index = 1
                                            For Each ELSqp As Result In .ELS_QP
                                                If ELSqp.Num = Comb.Num Then
                                                    For i = 0 To R.Mx.Count - 1
                                                        ELSqp.Mx.Item(i).M += R.Mx.Item(i).M
                                                        ELSqp.My.Item(i).M += R.My.Item(i).M
                                                        ELSqp.Vx.Item(i).V += R.Vx.Item(i).V
                                                        ELSqp.Vy.Item(i).V += R.Vy.Item(i).V
                                                        ELSqp.Cz.Item(i).Cz += R.Cz.Item(i).Cz
                                                        'ELSqp.Rx.Item(i).Rx += R.Rx.Item(i).Rx
                                                        'ELSqp.Ry.Item(i).Ry += R.Ry.Item(i).Ry
                                                        ELSqp.W_fl.Item(i).wfl += R.W_fl.Item(i).wfl
                                                    Next
                                                    ELSqp.R_angle = Rangle_Cumul
                                                    Exit For
                                                End If
                                                index += 1
                                                If index > .ELS_QP.Count Then
                                                    'initialisation d'un nouvel item
                                                    .ELS_QP.Add(R)
                                                    .ELS_QP.Item(.ELS_QP.Count - 1).R_angle = Rangle_Cumul
                                                    .ELS_QP.Item(.ELS_QP.Count - 1).Num = Comb.Num
                                                    Exit For
                                                End If
                                            Next
                                        End If
                                End Select
                                R = Nothing
                            End If
                        Next
                        progressForm.ProgressIndicator.Style = ProgressBarStyle.Blocks
                        progressForm.ProgressIndicator.Value += 1
                    Next '   --->  FIN DU CALCUL DES CHARGES PONEREES PAR DALLE
                    '
                    'DETERMINATION DES MINI MAXI PAR DALLE ET PAR BANDE
                    '
                    'Affectation des Maxi - mini pour l'ensemble des combinaisons de la travée i
                    'Détermination du nombre de colonnes, le balayage se faisant dans le sens Y
                    Dim Tot_Colonne As Integer = 0
                    Dim i_col(1) As Integer
                    For i = 1 To .ELU_Str.Item(0).Mx.Count
                        If .ELU_Str.Item(0).Mx(i - 1).y = 0 Then
                            Tot_Colonne += 1
                            ReDim Preserve i_col(Tot_Colonne)
                            i_col(Tot_Colonne) = i
                        End If
                    Next


                    'Combinaisons ELU
                    Dim R1 As New M_x : R1.x = 0 : R1.y = 0 : R1.M = 0 : Dim R2 As New M_y : R2.x = 0 : R2.y = 0 : R2.M = 0 : Dim R11 As New M_x : R11.x = 0 : R11.y = 0 : R11.M = 0
                    Dim R12 As New M_y : R12.x = 0 : R12.y = 0 : R12.M = 0
                    Dalle.ELU_Str_Env.Mx_Interval_Max.Add(R1) : Dalle.ELU_Str_Env.Mx_Interval_min.Add(R11) : Dalle.ELU_Str_Env.My_Interval_Max.Add(R2) : Dalle.ELU_Str_Env.My_Interval_min.Add(R12)
                    Dim Rv1 As New V_x : Rv1.x = 0 : Rv1.y = 0 : Rv1.V = 0 : Dim Rv2 As New V_y : Rv2.x = 0 : Rv2.y = 0 : Rv2.V = 0 : Dim Rv11 As New V_x : Rv11.x = 0 : Rv11.y = 0 : Rv11.V = 0
                    Dim Rv12 As New V_y : Rv12.x = 0 : Rv12.y = 0 : Rv12.V = 0
                    Dalle.ELU_Str_Env.Vx_Interval_Max.Add(Rv1) : Dalle.ELU_Str_Env.Vx_Interval_min.Add(Rv11) : Dalle.ELU_Str_Env.Vy_Interval_Max.Add(Rv2) : Dalle.ELU_Str_Env.Vy_Interval_min.Add(Rv12)
                    R1 = Nothing : R2 = Nothing : R11 = Nothing : R12 = Nothing : Rv1 = Nothing : Rv2 = Nothing : Rv11 = Nothing : Rv12 = Nothing
                    Dim i_colonne As Integer
                    Dim Ajout_Interval As Integer = 1
                    For Each ELUStr As Result In .ELU_Str
                        i11 = 1 'n° de tranche
                        Do
                            i = 0    'tous les attributs ont le même nombre d'item
                            i_colonne = 1
                            Do
                                If i11 >= 2 Then
                                    Do While ELUStr.Mx(i).y < Dalle.Interval * (i11 - 1)
                                        i = i + 1
                                        If i = ELUStr.Mx.Count Then Exit Do
                                    Loop
                                End If
                                If i = ELUStr.Mx.Count Then Exit Do

                                Do While ELUStr.Mx(i).y < Dalle.Interval * i11 And ELUStr.Mx(i).y >= Dalle.Interval * (i11 - 1)
                                    'Mx Max >0
                                    With Dalle.ELU_Str_Env.Mx_Interval_Max
                                        If .Item(i11 - 1).M < ELUStr.Mx(i).M Then .Item(i11 - 1).M = ELUStr.Mx(i).M : .Item(i11 - 1).x = ELUStr.Mx(i).x : .Item(i11 - 1).y = ELUStr.Mx(i).y
                                    End With
                                    ''Mx min <0
                                    With Dalle.ELU_Str_Env.Mx_Interval_min
                                        If .Item(i11 - 1).M > ELUStr.Mx(i).M Then .Item(i11 - 1).M = ELUStr.Mx(i).M : .Item(i11 - 1).x = ELUStr.Mx(i).x : .Item(i11 - 1).y = ELUStr.Mx(i).y
                                    End With
                                    'My Max >0
                                    With Dalle.ELU_Str_Env.My_Interval_Max
                                        If .Item(i11 - 1).M < ELUStr.My(i).M Then .Item(i11 - 1).M = ELUStr.My(i).M : .Item(i11 - 1).x = ELUStr.My(i).x : .Item(i11 - 1).y = ELUStr.My(i).y
                                    End With
                                    ''My min <0
                                    With Dalle.ELU_Str_Env.My_Interval_min
                                        If .Item(i11 - 1).M > ELUStr.My(i).M Then .Item(i11 - 1).M = ELUStr.My(i).M : .Item(i11 - 1).x = ELUStr.My(i).x : .Item(i11 - 1).y = ELUStr.My(i).y
                                    End With
                                    'Vx Max >0
                                    With Dalle.ELU_Str_Env.Vx_Interval_Max
                                        If .Item(i11 - 1).V < ELUStr.Vx(i).V Then .Item(i11 - 1).V = ELUStr.Vx(i).V : .Item(i11 - 1).x = ELUStr.Vx(i).x : .Item(i11 - 1).y = ELUStr.Vx(i).y
                                    End With
                                    'Vx min <0
                                    With Dalle.ELU_Str_Env.Vx_Interval_min
                                        If .Item(i11 - 1).V > ELUStr.Vx(i).V Then .Item(i11 - 1).V = ELUStr.Vx(i).V : .Item(i11 - 1).x = ELUStr.Vx(i).x : .Item(i11 - 1).y = ELUStr.Vx(i).y
                                    End With
                                    'Vy Max >0
                                    With Dalle.ELU_Str_Env.Vy_Interval_Max
                                        If .Item(i11 - 1).V < ELUStr.Vy(i).V Then .Item(i11 - 1).V = ELUStr.Vy(i).V : .Item(i11 - 1).x = ELUStr.Vy(i).x : .Item(i11 - 1).y = ELUStr.Vy(i).y
                                    End With
                                    'Vy min <0
                                    With Dalle.ELU_Str_Env.Vy_Interval_min
                                        If .Item(i11 - 1).V > ELUStr.Vy(i).V Then .Item(i11 - 1).V = ELUStr.Vy(i).V : .Item(i11 - 1).x = ELUStr.Vy(i).x : .Item(i11 - 1).y = ELUStr.Vy(i).y
                                    End With
                                    i += 1
                                    If i = ELUStr.Mx.Count Then Exit Do
                                Loop
                                i_colonne += 1
                                If i_colonne > Tot_Colonne Then Exit Do
                                i = i_col(i_colonne)
                            Loop
                            i11 += 1 'tranche suivante
                            If i = ELUStr.Mx.Count Then Exit Do 'condition de sortie de la boucle
                            If Ajout_Interval = 1 Then 'pour éviter d'en créer autant que de combinaison
                                'pour créer une instance supplémentaire correspondant à la tranche suivante
                                Dim R3 As New M_x : R3.x = 0 : R3.y = 0 : R3.M = 0 : Dim R4 As New M_y : R4.x = 0 : R4.y = 0 : R4.M = 0 : Dim R5 As New M_x : R5.x = 0 : R5.y = 0 : R5.M = 0
                                Dim R6 As New M_y : R6.x = 0 : R6.y = 0 : R6.M = 0
                                Dim Rv3 As New V_x : Rv3.x = 0 : Rv3.y = 0 : Rv3.V = 0 : Dim Rv4 As New V_y : Rv4.x = 0 : Rv4.y = 0 : Rv4.V = 0 : Dim Rv5 As New V_x : Rv5.x = 0 : Rv5.y = 0 : Rv5.V = 0
                                Dim Rv6 As New V_y : Rv6.x = 0 : Rv6.y = 0 : Rv6.V = 0
                                Dalle.ELU_Str_Env.Mx_Interval_Max.Add(R3) : Dalle.ELU_Str_Env.Mx_Interval_min.Add(R5) : Dalle.ELU_Str_Env.My_Interval_Max.Add(R4) : Dalle.ELU_Str_Env.My_Interval_min.Add(R6)
                                Dalle.ELU_Str_Env.Vx_Interval_Max.Add(Rv3) : Dalle.ELU_Str_Env.Vx_Interval_min.Add(Rv5) : Dalle.ELU_Str_Env.Vy_Interval_Max.Add(Rv4) : Dalle.ELU_Str_Env.Vy_Interval_min.Add(Rv6)
                                R3 = Nothing : R4 = Nothing : R5 = Nothing : R6 = Nothing : Rv3 = Nothing : Rv4 = Nothing : Rv5 = Nothing : Rv6 = Nothing
                            End If
                        Loop
                        Ajout_Interval = 2
                    Next

                    'Calcul des efforts Maxi
                    With .ELU_Str_Env
                        .Mx_max.M = 0 : .Mx_max.x = 0 : .Mx_max.y = 0 : .Mx_min.M = 0 : .Mx_min.x = 0 : .Mx_min.y = 0
                        .My_Max.M = 0 : .My_Max.x = 0 : .My_Max.y = 0 : .My_min.M = 0 : .My_min.x = 0 : .My_min.y = 0
                        For i = 0 To Dalle.ELU_Str_Env.My_Interval_min.Count - 1 'tous les intervalles Max-min ont le même nombre d'occurences
                            'Mx Max >0
                            If .Mx_max.M < .Mx_Interval_Max.Item(i).M Then .Mx_max.M = .Mx_Interval_Max.Item(i).M : .Mx_max.x = .Mx_Interval_Max.Item(i).x : .Mx_max.y = .Mx_Interval_Max.Item(i).y
                            'Mx Min <0
                            If .Mx_min.M > .Mx_Interval_min.Item(i).M Then .Mx_min.M = .Mx_Interval_min.Item(i).M : .Mx_min.x = .Mx_Interval_min.Item(i).x : .Mx_min.y = .Mx_Interval_min.Item(i).y
                            'My Max >0
                            If .My_Max.M < .My_Interval_Max.Item(i).M Then .My_Max.M = .My_Interval_Max.Item(i).M : .My_Max.x = .My_Interval_Max.Item(i).x : .My_Max.y = .My_Interval_Max.Item(i).y
                            'My Min <0
                            If .My_min.M > .My_Interval_min.Item(i).M Then .My_min.M = .My_Interval_min.Item(i).M : .My_min.x = .My_Interval_min.Item(i).x : .My_min.y = .My_Interval_min.Item(i).y
                            'Vx Max >0
                            If .Vx_Max.V < .Vx_Interval_Max.Item(i).V Then .Vx_Max.V = .Vx_Interval_Max.Item(i).V : .Vx_Max.x = .Vx_Interval_Max.Item(i).x : .Vx_Max.y = .Vx_Interval_Max.Item(i).y
                            'Vx Min <0
                            If .Vx_min.V > .Vx_Interval_min.Item(i).V Then .Vx_min.V = .Vx_Interval_min.Item(i).V : .Vx_min.x = .Vx_Interval_min.Item(i).x : .Vx_min.y = .Vx_Interval_min.Item(i).y
                            'Vy Max >0
                            If .Vy_Max.V < .Vy_Interval_Max.Item(i).V Then .Vy_Max.V = .Vy_Interval_Max.Item(i).V : .Vy_Max.x = .Vy_Interval_Max.Item(i).x : .Vy_Max.y = .Vy_Interval_Max.Item(i).y
                            'Vy Min <0
                            If .Vy_min.V > .Vy_Interval_min.Item(i).V Then .Vy_min.V = .Vy_Interval_min.Item(i).V : .Vy_min.x = .Vy_Interval_min.Item(i).x : .Vy_min.y = .Vy_Interval_min.Item(i).y
                        Next
                    End With

                    'Combinaison ELS caractéristiques
                    Ajout_Interval = 1
                    Dim R1s As New M_x : R1s.x = 0 : R1s.y = 0 : R1s.M = 0 : Dim R2s As New M_y : R2s.x = 0 : R2s.y = 0 : R2s.M = 0 : Dim R11s As New M_x : R11s.x = 0 : R11s.y = 0 : R11s.M = 0
                    Dim R12s As New M_y : R12s.x = 0 : R12s.y = 0 : R12s.M = 0
                    Dalle.ELS_Car_Env.Mx_Interval_Max.Add(R1s) : Dalle.ELS_Car_Env.Mx_Interval_min.Add(R11s) : Dalle.ELS_Car_Env.My_Interval_Max.Add(R2s) : Dalle.ELS_Car_Env.My_Interval_min.Add(R12s)
                    R1s = Nothing : R2s = Nothing : R11s = Nothing : R12s = Nothing
                    For Each ELSCar As Result In .ELS_Car
                        i11 = 1
                        Do
                            i = 0    'tous les attributs ont le même nombre d'item
                            i_colonne = 1
                            Do
                                If i11 >= 2 Then
                                    Do While ELSCar.Mx(i).y < Dalle.Interval * (i11 - 1)
                                        i = i + 1
                                        If i = ELSCar.Mx.Count Then Exit Do
                                    Loop
                                End If
                                If i = ELSCar.Mx.Count Then Exit Do
                                Do While ELSCar.Mx(i).y < Dalle.Interval * i11 And ELSCar.Mx(i).y >= Dalle.Interval * (i11 - 1)
                                    'Mx Max >0
                                    With Dalle.ELS_Car_Env.Mx_Interval_Max
                                        If .Item(i11 - 1).M < ELSCar.Mx(i).M Then .Item(i11 - 1).M = ELSCar.Mx(i).M : .Item(i11 - 1).x = ELSCar.Mx(i).x : .Item(i11 - 1).y = ELSCar.Mx(i).y
                                    End With
                                    'Mx min <0
                                    With Dalle.ELS_Car_Env.Mx_Interval_min
                                        If .Item(i11 - 1).M > ELSCar.Mx(i).M Then .Item(i11 - 1).M = ELSCar.Mx(i).M : .Item(i11 - 1).x = ELSCar.Mx(i).x : .Item(i11 - 1).y = ELSCar.Mx(i).y
                                    End With
                                    'My Max >0
                                    With Dalle.ELS_Car_Env.My_Interval_Max
                                        If .Item(i11 - 1).M < ELSCar.My(i).M Then .Item(i11 - 1).M = ELSCar.My(i).M : .Item(i11 - 1).x = ELSCar.My(i).x : .Item(i11 - 1).y = ELSCar.My(i).y
                                    End With
                                    'My min <0
                                    With Dalle.ELS_Car_Env.My_Interval_min
                                        If .Item(i11 - 1).M > ELSCar.My(i).M Then .Item(i11 - 1).M = ELSCar.My(i).M : .Item(i11 - 1).x = ELSCar.My(i).x : .Item(i11 - 1).y = ELSCar.My(i).y
                                    End With
                                    i += 1
                                    If i = ELSCar.Mx.Count Then Exit Do
                                Loop
                                i_colonne += 1
                                If i_colonne > Tot_Colonne Then Exit Do
                                i = i_col(i_colonne)
                            Loop
                            i11 += 1 'tranche suivante
                            If i = ELSCar.Mx.Count Then Exit Do 'condition de sortie de la boucle
                            If Ajout_Interval = 1 Then 'pour éviter d'en créer autant que de combinaison
                                'pour créer une instance supplémentaire correspondant à la tranche suivante
                                Dim R3S As New M_x : R3S.x = 0 : R3S.y = 0 : R3S.M = 0 : Dim R4S As New M_y : R4S.x = 0 : R4S.y = 0 : R4S.M = 0 : Dim R5S As New M_x : R5S.x = 0 : R5S.y = 0 : R5S.M = 0
                                Dim R6S As New M_y : R6S.x = 0 : R6S.y = 0 : R6S.M = 0
                                Dalle.ELS_Car_Env.Mx_Interval_Max.Add(R3S) : Dalle.ELS_Car_Env.Mx_Interval_min.Add(R5S) : Dalle.ELS_Car_Env.My_Interval_Max.Add(R4S) : Dalle.ELS_Car_Env.My_Interval_min.Add(R6S)
                                R3S = Nothing : R4S = Nothing : R5S = Nothing : R6S = Nothing
                            End If
                        Loop
                        Ajout_Interval = 2
                    Next

                    'Calcul des efforts Maxi  els caractéristiques
                    With Dalle.ELS_Car_Env
                        .Mx_max.M = 0 : .Mx_max.x = 0 : .Mx_max.y = 0 : .Mx_min.M = 0 : .Mx_min.x = 0 : .Mx_min.y = 0
                        .My_Max.M = 0 : .My_Max.x = 0 : .My_Max.y = 0 : .My_min.M = 0 : .My_min.x = 0 : .My_min.y = 0
                        For i = 0 To Dalle.ELS_Car_Env.My_Interval_min.Count - 1
                            'Mx Max >0
                            If .Mx_max.M < .Mx_Interval_Max.Item(i).M Then .Mx_max.M = .Mx_Interval_Max.Item(i).M : .Mx_max.x = .Mx_Interval_Max.Item(i).x : .Mx_max.y = .Mx_Interval_Max.Item(i).y
                            'Mx Min <0
                            If .Mx_min.M > .Mx_Interval_min.Item(i).M Then .Mx_min.M = .Mx_Interval_min.Item(i).M : .Mx_min.x = .Mx_Interval_min.Item(i).x : .Mx_min.y = .Mx_Interval_min.Item(i).y
                            'My Max >0
                            If .My_Max.M < .My_Interval_Max.Item(i).M Then .My_Max.M = .My_Interval_Max.Item(i).M : .My_Max.x = .My_Interval_Max.Item(i).x : .My_Max.y = .My_Interval_Max.Item(i).y
                            'Mx Min <0
                            If .My_min.M > .My_Interval_min.Item(i).M Then .My_min.M = .My_Interval_min.Item(i).M : .My_min.x = .My_Interval_min.Item(i).x : .My_min.y = .My_Interval_min.Item(i).y
                        Next
                    End With

                    'Combinaison ELS Quasi-Permanent
                    Ajout_Interval = 1
                    Dim R1q As New M_x : R1q.x = 0 : R1q.y = 0 : R1q.M = 0 : Dim R2q As New M_y : R2q.x = 0 : R2q.y = 0 : R2q.M = 0 : Dim R11q As New M_x : R11q.x = 0 : R11q.y = 0 : R11q.M = 0
                    Dim R12q As New M_y : R12q.x = 0 : R12q.y = 0 : R12q.M = 0
                    Dalle.ELS_QP_Env.Mx_Interval_Max.Add(R1q) : Dalle.ELS_QP_Env.Mx_Interval_min.Add(R11q) : Dalle.ELS_QP_Env.My_Interval_Max.Add(R2q) : Dalle.ELS_QP_Env.My_Interval_min.Add(R12q)
                    R1q = Nothing : R2q = Nothing : R11q = Nothing : R12q = Nothing
                    For Each ELSqp As Result In .ELS_QP
                        i11 = 1
                        ELSqp.W_fl_Max.wfl = 0 : ELSqp.W_fl_min.wfl = 0
                        Do
                            i = 0    'tous les attributs ont le même nombre d'item
                            i_colonne = 1
                            Do
                                If i11 >= 2 Then
                                    Do While ELSqp.Mx(i).y < Dalle.Interval * (i11 - 1)
                                        i = i + 1
                                        If i = ELSqp.Mx.Count Then Exit Do
                                    Loop
                                End If
                                If i = ELSqp.Mx.Count Then Exit Do
                                Do While ELSqp.Mx(i).y < Dalle.Interval * i11 And ELSqp.Mx(i).y >= Dalle.Interval * (i11 - 1)
                                    'Mx Max >0
                                    With Dalle.ELS_QP_Env.Mx_Interval_Max
                                        If .Item(i11 - 1).M < ELSqp.Mx(i).M Then .Item(i11 - 1).M = ELSqp.Mx(i).M : .Item(i11 - 1).x = ELSqp.Mx(i).x : .Item(i11 - 1).y = ELSqp.Mx(i).y
                                    End With
                                    'Mx min <0
                                    With Dalle.ELS_QP_Env.Mx_Interval_min
                                        If .Item(i11 - 1).M > ELSqp.Mx(i).M Then .Item(i11 - 1).M = ELSqp.Mx(i).M : .Item(i11 - 1).x = ELSqp.Mx(i).x : .Item(i11 - 1).y = ELSqp.Mx(i).y
                                    End With
                                    'My Max >0
                                    With Dalle.ELS_QP_Env.My_Interval_Max
                                        If .Item(i11 - 1).M < ELSqp.My(i).M Then .Item(i11 - 1).M = ELSqp.My(i).M : .Item(i11 - 1).x = ELSqp.My(i).x : .Item(i11 - 1).y = ELSqp.My(i).y
                                    End With
                                    'My min <0
                                    With Dalle.ELS_QP_Env.My_Interval_min
                                        If .Item(i11 - 1).M > ELSqp.My(i).M Then .Item(i11 - 1).M = ELSqp.My(i).M : .Item(i11 - 1).x = ELSqp.My(i).x : .Item(i11 - 1).y = ELSqp.My(i).y
                                    End With
                                    'flèche positive
                                    If ELSqp.W_fl_Max.wfl < ELSqp.W_fl(i).wfl Then ELSqp.W_fl_Max.wfl = ELSqp.W_fl(i).wfl : ELSqp.W_fl_Max.x = ELSqp.W_fl(i).x : ELSqp.W_fl_Max.y = ELSqp.W_fl(i).y
                                    'flèche négative
                                    If ELSqp.W_fl_min.wfl > ELSqp.W_fl(i).wfl Then ELSqp.W_fl_min.wfl = ELSqp.W_fl(i).wfl : ELSqp.W_fl_min.x = ELSqp.W_fl(i).x : ELSqp.W_fl_min.y = ELSqp.W_fl(i).y
                                    i += 1
                                    If i = ELSqp.Mx.Count Then Exit Do
                                Loop
                                i_colonne += 1
                                If i_colonne > Tot_Colonne Then Exit Do
                                i = i_col(i_colonne)
                            Loop
                            i11 += 1
                            If i = ELSqp.Mx.Count Then Exit Do 'condition de sortie de la boucle
                            If Ajout_Interval = 1 Then 'pour éviter d'en créer autant que de combinaison
                                'pour créer une instance supplémentaire correspondant à la tranche suivante
                                Dim R3q As New M_x : R3q.x = 0 : R3q.y = 0 : R3q.M = 0 : Dim R4q As New M_y : R4q.x = 0 : R4q.y = 0 : R4q.M = 0 : Dim R5q As New M_x : R5q.x = 0 : R5q.y = 0 : R5q.M = 0
                                Dim R6q As New M_y : R6q.x = 0 : R6q.y = 0 : R6q.M = 0
                                Dalle.ELS_QP_Env.Mx_Interval_Max.Add(R3q) : Dalle.ELS_QP_Env.Mx_Interval_min.Add(R5q) : Dalle.ELS_QP_Env.My_Interval_Max.Add(R4q) : Dalle.ELS_QP_Env.My_Interval_min.Add(R6q)
                                R3q = Nothing : R4q = Nothing : R5q = Nothing : R6q = Nothing
                            End If
                        Loop
                        Ajout_Interval = 2
                    Next

                    'Calcul des efforts Maxi  els Quasi-Permanent
                    With Dalle.ELS_QP_Env
                        .Mx_max.M = 0 : .Mx_max.x = 0 : .Mx_max.y = 0 : .Mx_min.M = 0 : .Mx_min.x = 0 : .Mx_min.y = 0
                        .My_Max.M = 0 : .My_Max.x = 0 : .My_Max.y = 0 : .My_min.M = 0 : .My_min.x = 0 : .My_min.y = 0
                        For i = 0 To Dalle.ELS_QP_Env.My_Interval_min.Count - 1
                            'Mx Max >0
                            If .Mx_max.M < .Mx_Interval_Max.Item(i).M Then .Mx_max.M = .Mx_Interval_Max.Item(i).M : .Mx_max.x = .Mx_Interval_Max.Item(i).x : .Mx_max.y = .Mx_Interval_Max.Item(i).y
                            'Mx Min <0
                            If .Mx_min.M > .Mx_Interval_min.Item(i).M Then .Mx_min.M = .Mx_Interval_min.Item(i).M : .Mx_min.x = .Mx_Interval_min.Item(i).x : .Mx_min.y = .Mx_Interval_min.Item(i).y
                            'My Max >0
                            If .My_Max.M < .My_Interval_Max.Item(i).M Then .My_Max.M = .My_Interval_Max.Item(i).M : .My_Max.x = .My_Interval_Max.Item(i).x : .My_Max.y = .My_Interval_Max.Item(i).y
                            'Mx Min <0
                            If .My_min.M > .My_Interval_min.Item(i).M Then .My_min.M = .My_Interval_min.Item(i).M : .My_min.x = .My_Interval_min.Item(i).x : .My_min.y = .My_Interval_min.Item(i).y
                        Next
                    End With
                End With
            Next
        End With
        ' Supprime le formulaire à la fin de la tâche.
        progressForm.Hide()
        progressForm.Dispose()
        Call Ecriture_Resultats(αa, rtf)
    End Sub
    '''<summary>Calcul des efforts dans la plaque suivant solution de Navier</summary>
    ''' <param name="dalle">Instance de la dalle objet des calculs.</param>
    ''' <param name="P0">Caractéristique attachée à la charge</param>
    ''' <param name="U33">Caractéristique attachée à la charge</param>
    ''' <param name="U4">Caractéristique attachée à la charge</param>
    ''' <param name="U5">Caractéristique attachée à la charge</param>
    ''' <param name="U6">Caractéristique attachée à la charge</param>
    ''' <param name="D">Caractéristique attachée à la charge</param>
    ''' <param name="Type">n° caractérisant la combinaison: 1=ELU STR, 2=ELU EQU, 3=ELS caract., 5=ELS QP</param>
    ''' <param name="R"> encapsule les résultats du calcul</param>
    Private Sub Module_Calcul_Navier(ByVal dalle As Plancher, ByVal P0 As Double, ByVal U33 As Double, ByVal U4 As Double, ByVal U5 As Double, ByVal U6 As Double, ByVal D As Double, ByVal Type As Integer,
                                    ByRef R As Result)
        'Calcul des efforts sous la charge puis par maillage de 20x20 cm jusqu'à l'appui
        Dim M1, M2, C, T1, T2, U1, U2, U3, X, Y, X1, Y1 As Double
        Dim A(,), B(,) As Double
        Dim i, m, n As Integer
        Dim ν As Double
        With dalle
            Select Case Type
                Case 1, 2 'ELU Str et EQU
                    ν = 0
                Case 3, 4 'ELS Caractéristique et quasi-Permanent
                    ν = .ν_eff
            End Select
            R.Mx.Clear() : R.My.Clear() : R.Vx.Clear() : R.Vy.Clear() : R.Cz.Clear() : R.W_fl.Clear() 'remise à zero
            X = 0
            Do
                Y = 0
                i = 1
                Do
                    Dim wfl As New W_fl
                    X1 = (X * PI) / .Lx : Y1 = (Y * PI) / .Ly
                    M1 = 0 : M2 = 0 : C = 0 : T1 = 0 : T2 = 0
                    ReDim A(.Rang, .Rang), B(.Rang, .Rang)
                    For m = 1 To dalle.Rang
                        For n = 1 To dalle.Rang
                            A(m, n) = (4 * P0 / (m * n * PI ^ 2) * (Cos(m * U33) - Cos(m * U5)) * (Cos(n * U4) - Cos(n * U6))) ' + A(m, n) 'expression équivalente à amn Timoshenko p111
                            U1 = (m ^ 2 / dalle.Lx ^ 2 + n ^ 2 / dalle.Ly ^ 2) ^ 2
                            B(m, n) = (-1 / (D * PI ^ 4)) * (A(m, n) / U1)
                            'Rangle = Rangle + A(m, n) * (m * n / (dalle.Lx * dalle.Ly)) / U1
                            U1 = (m ^ 2 / .Lx ^ 2 + n ^ 2 / .Ly ^ 2) ^ 2
                            U2 = Sin(m * X1) * Sin(n * Y1)
                            U3 = Cos(m * X1) * Cos(n * Y1)
                            wfl.wfl = wfl.wfl + B(m, n) * U2 'équation 130 timoshenko p109 --> calcul de la flèche
                            M1 = M1 + A(m, n) * (m ^ 2 / .Lx ^ 2 + ν * n ^ 2 / .Ly ^ 2) * U2 / U1
                            M2 = M2 + A(m, n) * (n ^ 2 / .Ly ^ 2 + ν * m ^ 2 / .Lx ^ 2) * U2 / U1
                            C = C + A(m, n) * (1 - ν) * ((m * n) / (.Ly * .Lx)) * U3 / U1
                            T1 = T1 + A(m, n) * (m / .Lx) / Sqrt(U1) * Cos(m * X1) * Sin(n * Y1)
                            T2 = T2 + A(m, n) * (n / .Ly) / Sqrt(U1) * Sin(m * X1) * Cos(n * Y1)
                            'R1 = R1 + A(m, n) * (m / .Lx) * (m ^ 2 / .Lx ^ 2 + (2 - .ν) * n ^ 2 / .Ly ^ 2) * Sin(n * Y1) / U1
                            'R2 = R2 + A(m, n) * (n / .Ly) * (n ^ 2 / .Ly ^ 2 + (2 - .ν) * m ^ 2 / .Lx ^ 2) * Sin(n * X1) / U1
                        Next
                    Next
                    Dim mX As New M_x
                    Dim my As New M_y
                    Dim Vx As New V_x
                    Dim Vy As New V_y
                    Dim Cz As New C_z
                    'Dim R_1 As New R_x
                    'Dim R_2 As New R_y
                    mX.M = M1 / (PI ^ 2) : mX.x = X : mX.y = Y ' MCP_X(k1, k2)
                    my.M = M2 / (PI ^ 2) : my.x = X : my.y = Y
                    Cz.Cz = C / (PI ^ 2) : Cz.x = X : Cz.y = Y
                    Vx.V = T1 / PI : Vx.x = X : Vx.y = Y
                    Vy.V = T2 / PI : Vy.x = X : Vy.y = Y
                    ' R_1.Rx = R1 / PI : R_1.y = Y
                    ' R_2.Ry = R2 / PI : R_2.x = X
                    wfl.x = X : wfl.y = Y
                    'Affectation des valeurs
                    R.Mx.Add(mX) : R.My.Add(my) : R.Vx.Add(Vx) : R.Vy.Add(Vy) : R.Cz.Add(Cz) : R.W_fl.Add(wfl) ': R.Rx.Add(R_1) : R.Ry.Add(R_2) 

                    If Y = 0 Then Y = Y + .Taille / 2 Else Y = Y + .Taille 'pour avoir toujours le milieu de la maille
                    mX = Nothing : my = Nothing : Vx = Nothing : Vy = Nothing : Cz = Nothing : wfl = Nothing ': R_1 = Nothing : R_2 = Nothing 'pour asurer la remise à 0
                Loop While Y <= .Ly

                If X = 0 Then X = X + .Taille / 2 Else X = X + .Taille
            Loop While X <= .Lx
        End With
    End Sub
    Private Sub Ecriture_Resultats(ByVal αa As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter) 'ByRef Editeur As Integer, ByRef msgResult As String,
        Dim i, j As Integer

        rtf.WriteStartGroup()
        rtf.NewLine() : rtf.FontSize = 12 : rtf.Bold = True : rtf.Underline = True
        rtf.WriteString("2 - Résultats du calcul Résistance des Matériaux") : rtf.Underline = False : rtf.Bold = False
        rtf.FontSize = 9
        If Aff.degression = True Then rtf.NewLine() : rtf.NewLine() : rtf.WriteString(" Dégression appliquée à la charge αa=" & αa.ToString("#0.000"))
        rtf.NewLine() : rtf.NewLine() : rtf.WriteString(" Valeurs maximales et minimales pour chaque dalle.") : rtf.NewLine()
        i = 0
        For Each Dalle As Plancher In PlanCont
            With Dalle.ELU_Str_Env
                i += 1
                rtf.WriteString(" Dalle n°" & i.ToString & ":") : rtf.NewLine()
                rtf.WriteString(" - Mx Max = " & (.Mx_max.M * 1000).ToString("#0.000") & "kN.m/ml x = " & .Mx_max.x.ToString("#0.00") & "m, y = " & .Mx_max.y.ToString("#0.00") & "m.") : rtf.NewLine()
                rtf.WriteString(" - Mx Min = " & (.Mx_min.M * 1000).ToString("#0.000") & "kN.m/ml x = " & .Mx_min.x.ToString("#0.00") & "m, y = " & .Mx_min.y.ToString("#0.00") & "m.") : rtf.NewLine()
                rtf.WriteString(" - My Max = " & (.My_Max.M * 1000).ToString("#0.000") & "kN.m/ml x = " & .My_Max.x.ToString("#0.00") & "m, y = " & .My_Max.y.ToString("#0.00") & "m.") : rtf.NewLine()
                rtf.WriteString(" - My Min = " & (.My_min.M * 1000).ToString("#0.000") & "kN.m/ml x = " & .My_min.x.ToString("#0.00") & "m, y = " & .My_min.y.ToString("#0.00") & "m.") : rtf.NewLine()
                rtf.WriteString(" - Vx Max = " & (.Vx_Max.V * 1000).ToString("#0.000") & "kN/ml x = " & .Vx_Max.x.ToString("#0.00") & "m, y = " & .Vx_Max.y.ToString("#0.00") & "m.") : rtf.NewLine()
                rtf.WriteString(" - Vx Min = " & (.Vx_min.V * 1000).ToString("#0.000") & "kN/ml x = " & .Vx_min.x.ToString("#0.00") & "m, y = " & .Vx_min.y.ToString("#0.00") & "m.") : rtf.NewLine()
                rtf.WriteString(" - Vy Max = " & (.Vy_Max.V * 1000).ToString("#0.000") & "kN/ml x = " & .Vy_Max.x.ToString("#0.00") & "m, y = " & .Vy_Max.y.ToString("#0.00") & "m.") : rtf.NewLine()
                rtf.WriteString(" - Vy Min = " & (.Vy_min.V * 1000).ToString("#0.000") & "kN/ml x = " & .Vy_min.x.ToString("#0.00") & "m, y = " & .Vy_min.y.ToString("#0.00") & "m.") : rtf.NewLine()
            End With
            With Dalle.ELS_QP
                For j = 1 To .Count
                    rtf.WriteString("Combinaison n°" & .Item(j - 1).Num.ToString) : rtf.NewLine()
                    rtf.WriteString(" - Wfl positif = " & (.Item(j - 1).W_fl_Max.wfl * 1000).ToString("#0.00") & "mm x = " & .Item(j - 1).W_fl_Max.x.ToString("#0.00") & "m, y = " & .Item(j - 1).W_fl_Max.y.ToString("#0.00") & "m.") : rtf.NewLine()
                    rtf.WriteString(" - Wfl négatif = " & (.Item(j - 1).W_fl_min.wfl * 1000).ToString("#0.00") & "mm x = " & .Item(j - 1).W_fl_min.x.ToString("#0.00") & "m, y = " & .Item(j - 1).W_fl_min.y.ToString("#0.00") & "m.") : rtf.NewLine()
                Next
            End With
        Next
        rtf.WriteEndGroup()
        If det_Ndc.Moment_Flexion = 1 Then
            Call Ndc_Affichage_Moments_Flexion(rtf)
        End If
    End Sub
    Private Sub Ndc_Affichage_Moments_Flexion(ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim Val, x, y As Double
        Dim i, k As Integer
        rtf.WriteStartGroup()
        rtf.NewLine() : rtf.FontSize = 12 : rtf.FontSize = 10
        rtf.NewLine() : rtf.NewLine() : rtf.WriteString(" Valeurs maximales et minimales suivant maillage retenu pour chaque bande:")

        i = 0 : k = 0 '    i_trav = 0 :
        For Each Dalle As Plancher In PlanCont
            With Dalle
                rtf.Bold = True : rtf.WriteString("Travée n°" & (PlanCont.IndexOf(Dalle) + 1).ToString("#")) : rtf.Bold = False : rtf.NewLine()
                Dim msg As String
                rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                'Ligne 1
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
                'Ligne 3
                rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString("ELU") : rtf.EndCell() : rtf.EndRow()
                'Ligne 4 - Mx - Max
                For i = 0 To Dalle.ELU_Str_Env.My_Interval_min.Count - 1
                    rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mx - Max") : rtf.EndCell()
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                    With .ELU_Str_Env.Mx_Interval_Max
                        Val = .Item(i).M : x = .Item(i).x : y = .Item(i).y
                    End With
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                    msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                    rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                Next
                'Ligne 5 - Mx - min
                For i = 0 To Dalle.ELU_Str_Env.My_Interval_min.Count - 1
                    rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mx - min") : rtf.EndCell()
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                    With .ELU_Str_Env.Mx_Interval_min
                        Val = .Item(i).M : x = .Item(i).x : y = .Item(i).y
                    End With
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                    msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                    rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                Next
                'Ligne 6 - My - Max
                For i = 0 To Dalle.ELU_Str_Env.My_Interval_min.Count - 1
                    rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("My - Max") : rtf.EndCell()
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                    With .ELU_Str_Env.My_Interval_Max
                        Val = .Item(i).M : x = .Item(i).x : y = .Item(i).y
                    End With
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                    msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                    rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                Next
                'Ligne 7- My - min
                For i = 0 To Dalle.ELU_Str_Env.My_Interval_min.Count - 1
                    rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("My - min") : rtf.EndCell()
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                    With .ELU_Str_Env.My_Interval_min
                        Val = .Item(i).M : x = .Item(i).x : y = .Item(i).y
                    End With
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                    msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                    rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                Next
                ''Ligne 8
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vx - Max") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELU_Str.Count - 1
                '    If Val < .ELU_Str(k).Vx_Max.V Then Val = .ELU_Str(k).Vx_Max.V : x = .ELU_Str(k).Vx_Max.x : y = .ELU_Str(k).Vx_Max.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                ''Ligne 9
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vx - min") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELU_Str.Count - 1
                '    If Val > .ELU_Str(k).Mx_min.M Then Val = .ELU_Str(k).Mx_min.M : x = .ELU_Str(k).Mx_min.x : y = .ELU_Str(k).Mx_min.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                ''Ligne 10
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vy - Max") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELU_Str.Count - 1
                '    If Val < .ELU_Str(k).Vy_Max.V Then Val = .ELU_Str(k).Vy_Max.V : x = .ELU_Str(k).Vy_Max.x : y = .ELU_Str(k).Vy_Max.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                ''Ligne 11
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vy - min") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELU_Str.Count - 1
                '    If Val > .ELU_Str(k).Vy_min.V Then Val = .ELU_Str(k).Vy_min.V : x = .ELU_Str(k).Vy_min.x : y = .ELU_Str(k).Vy_min.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                ''Ligne 3
                'rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString("ELS caractéristiques") : rtf.EndCell() : rtf.EndRow()
                ''Ligne 4
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mx - Max") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELS_Car.Count - 1
                '    '                If Val < .ELS_Car(k).Mx_Max.M Then Val = .ELS_Car(k).Mx_Max.M : x = .ELS_Car(k).Mx_Max.x : y = .ELS_Car(k).Mx_Max.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                ''Ligne 5
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mx - min") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELS_Car.Count - 1
                '    If Val > .ELS_Car(k).Mx_min.M Then Val = .ELS_Car(k).Mx_min.M : x = .ELS_Car(k).Mx_min.x : y = .ELS_Car(k).Mx_min.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                ''Ligne 6
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("My - Max") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELS_Car.Count - 1
                '    '               If Val < .ELS_Car(k).My_Max.M Then Val = .ELS_Car(k).My_Max.M : x = .ELS_Car(k).My_Max.x : y = .ELS_Car(k).My_Max.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                ''Ligne 7
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("My - min") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELS_Car.Count - 1
                '    If Val > .ELS_Car(k).My_min.M Then Val = .ELS_Car(k).My_min.M : x = .ELS_Car(k).My_min.x : y = .ELS_Car(k).My_min.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                ''Ligne 7
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Flèche") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELS_QP.Count - 1
                '    If Val < .ELS_QP(k).W_fl_Max.wfl Then Val = .ELS_QP(k).W_fl_Max.wfl : x = .ELS_QP(k).W_fl_Max.x : y = .ELS_QP(k).W_fl_Max.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.0")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                ''Ligne 3
                'rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString("ELS Quasi-Permanent") : rtf.EndCell() : rtf.EndRow()
                ''Ligne 4
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mx - Max") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELS_QP.Count - 1
                '    '                If Val < .ELS_QP(k).Mx_Max.M Then Val = .ELS_QP(k).Mx_Max.M : x = .ELS_QP(k).Mx_Max.x : y = .ELS_QP(k).Mx_Max.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                ''Ligne 6
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("My - Max") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("KN.m ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELS_QP.Count - 1
                '    '                  If Val < .ELS_QP(k).My_Max.M Then Val = .ELS_QP(k).My_Max.M : x = .ELS_QP(k).My_Max.x : y = .ELS_QP(k).My_Max.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.000")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                ''Ligne 7
                'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Flèche") : rtf.EndCell()
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm ") : rtf.EndCell()
                'Val = 0
                'For k = 0 To .ELS_QP.Count - 1
                '    If Val < .ELS_QP(k).W_fl_Max.wfl Then Val = .ELS_QP(k).W_fl_Max.wfl : x = .ELS_QP(k).W_fl_Max.x : y = .ELS_QP(k).W_fl_Max.y
                'Next
                'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Val * 1000).ToString("#0.0")) : rtf.EndCell()
                'msg = "pour x= " & x.ToString("#0.000") & " m et y= " & y.ToString("#0.000") & " m - comb. n°" & (k + 1).ToString
                'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()



                'rtf.Italic = True : rtf.WriteString("Réactions sur les appuis suivant X: ") : rtf.Italic = False : rtf.NewLine()
                'rtf.WriteString(" ► Maximal: " & (.ELU_Str.Item(k).Rx_Max.Rx * 1000).ToString("#0.00") & " KN/ml pour y = " & (.ELU_Str.Item(k).Rx_Max.y).ToString("#0.00") & " m.") : rtf.NewLine()
                'rtf.WriteString(" ► minimal: " & (.ELU_Str.Item(k).Rx_min.Rx * 1000).ToString("#0.00") & " KN/ml pour y = " & (.ELU_Str.Item(k).Rx_min.y).ToString("#0.00") & " m.") : rtf.NewLine()
                'rtf.Italic = True : rtf.WriteString("Réactions sur les appuis suivant Y:") : rtf.Italic = False : rtf.NewLine()
                'rtf.WriteString(" ► Maximal: " & (.ELU_Str.Item(k).Ry_Max.Ry * 1000).ToString("#0.00") & " KN/ml pour x = " & (.ELU_Str.Item(k).Ry_Max.x).ToString("#0.00") & " m.") : rtf.NewLine()
                'rtf.WriteString(" ► minimal: " & (.ELU_Str.Item(k).Ry_min.Ry * 1000).ToString("#0.00") & " KN/ml pour x = " & (.ELU_Str.Item(k).Ry_min.x).ToString("#0.00") & " m.") : rtf.NewLine()
                'rtf.Italic = True : rtf.WriteString("Flèche:") : rtf.Italic = False : rtf.NewLine()
                'rtf.WriteString(" ► Maximal: " & (.ELU_Str.Item(k).W_fl_Max.wfl * 1000).ToString("#0.0") & " mm pour x = " & (.ELU_Str.Item(k).W_fl_Max.x).ToString("#0.00") & " m et y = " & (.ELU_Str.Item(k).W_fl_Max.y).ToString("#0.00") & " m.") : rtf.NewLine()
                'rtf.WriteString(" ► minimal: " & (.ELU_Str.Item(k).W_fl_min.wfl * 1000).ToString("#0.0") & " mm pour x = " & (.ELU_Str.Item(k).W_fl_min.x).ToString("#0.00") & " m et y = " & (.ELU_Str.Item(k).W_fl_min.y).ToString("#0.00") & " m.") : rtf.NewLine()
                'rtf.NewLine()
                'rtf.WriteString("Valeur de réaction aux angles (soulèvement): " & (.ELU_Str.Item(k).R_angle * 1000).ToString("#0.000") & " KN.") : rtf.NewLine()

            End With
        Next

        rtf.WriteEndGroup()
    End Sub

    Private Sub Poubelle()



        ' End With

        'Select Case .Repartition
        '    Case 1
        '        With ELUStr
        '            For i = 0 To .Mx.Count - 1 'tous les attributs ont le même nombre d'item

        '                If .Mx_min.M > .Mx(i).M Then .Mx_min.M = .Mx(i).M : .Mx_min.x = .Mx(i).x : .Mx_min.y = .Mx(i).y
        '              
        '                If .My_min.M > .My(i).M Then .My_min.M = .My(i).M : .My_min.x = .My(i).x : .My_min.y = .My(i).y
        '                If .Vx_Max.V < .Vx(i).V Then .Vx_Max.V = .Vx(i).V : .Vx_Max.x = .Vx(i).x : .Vx_Max.y = .Vx(i).y
        '                If .Vx_min.V > .Vx(i).V Then .Vx_min.V = .Vx(i).V : .Vx_min.x = .Vx(i).x : .Vx_min.y = .Vx(i).y
        '                If .Vy_Max.V < .Vy(i).V Then .Vy_Max.V = .Vy(i).V : .Vy_Max.x = .Vy(i).x : .Vy_Max.y = .Vy(i).y
        '                If .Vy_min.V > .Vy(i).V Then .Vy_min.V = .Vy(i).V : .Vy_min.x = .Vy(i).x : .Vy_min.y = .Vy(i).y
        '                If .Cz_Max.Cz < .Cz(i).Cz Then .Cz_Max.Cz = .Cz(i).Cz : .Cz_Max.x = .Cz(i).x : .Cz_Max.y = .Cz(i).y
        '                If .Cz_min.Cz > .Cz(i).Cz Then .Cz_min.Cz = .Cz(i).Cz : .Cz_min.x = .Cz(i).x : .Cz_min.y = .Cz(i).y
        '                If .Rx_Max.Rx < .Rx(i).Rx Then .Rx_Max.Rx = .Rx(i).Rx : .Rx_Max.y = .Rx(i).y
        '                If .Rx_min.Rx > .Rx(i).Rx Then .Rx_min.Rx = .Rx(i).Rx : .Rx_min.y = .Rx(i).y
        '                If .Ry_Max.Ry < .Ry(i).Ry Then .Ry_Max.Ry = .Ry(i).Ry : .Ry_Max.x = .Ry(i).x
        '                If .Ry_min.Ry > .Ry(i).Ry Then .Ry_min.Ry = .Ry(i).Ry : .Ry_min.x = .Ry(i).x
        '            Next
        '        End With
        '    Case 2 'répartition NON uniforme = on prend un point par intervalle



        '        If .Mx_min.M > .Mx(i).M Then .Mx_min.M = .Mx(i).M : .Mx_min.x = .Mx(i).x : .Mx_min.y = .Mx(i).y
        '            If .My_Max.M < .My(i).M Then .My_Max.M = .My(i).M : .My_Max.x = .My(i).x : .My_Max.y = .My(i).y
        '            If .My_min.M > .My(i).M Then .My_min.M = .My(i).M : .My_min.x = .My(i).x : .My_min.y = .My(i).y
        '            If .Vx_Max.V < .Vx(i).V Then .Vx_Max.V = .Vx(i).V : .Vx_Max.x = .Vx(i).x : .Vx_Max.y = .Vx(i).y
        '            If .Vx_min.V > .Vx(i).V Then .Vx_min.V = .Vx(i).V : .Vx_min.x = .Vx(i).x : .Vx_min.y = .Vx(i).y
        '            If .Vy_Max.V < .Vy(i).V Then .Vy_Max.V = .Vy(i).V : .Vy_Max.x = .Vy(i).x : .Vy_Max.y = .Vy(i).y
        '            If .Vy_min.V > .Vy(i).V Then .Vy_min.V = .Vy(i).V : .Vy_min.x = .Vy(i).x : .Vy_min.y = .Vy(i).y
        '            If .Cz_Max.Cz < .Cz(i).Cz Then .Cz_Max.Cz = .Cz(i).Cz : .Cz_Max.x = .Cz(i).x : .Cz_Max.y = .Cz(i).y
        '            If .Cz_min.Cz > .Cz(i).Cz Then .Cz_min.Cz = .Cz(i).Cz : .Cz_min.x = .Cz(i).x : .Cz_min.y = .Cz(i).y
        '            If .Rx_Max.Rx < .Rx(i).Rx Then .Rx_Max.Rx = .Rx(i).Rx : .Rx_Max.y = .Rx(i).y
        '            If .Rx_min.Rx > .Rx(i).Rx Then .Rx_min.Rx = .Rx(i).Rx : .Rx_min.y = .Rx(i).y
        '            If .Ry_Max.Ry < .Ry(i).Ry Then .Ry_Max.Ry = .Ry(i).Ry : .Ry_Max.x = .Ry(i).x
        '            If .Ry_min.Ry > .Ry(i).Ry Then .Ry_min.Ry = .Ry(i).Ry : .Ry_min.x = .Ry(i).x
        '        ' Next

        'End Select

        'For i = 0 To .Mx.Count - 1 'tous les attributs ont le même nombre d'item
        '    If .Mx_Max.M < .Mx(i).M Then .Mx_Max.M = .Mx(i).M : .Mx_Max.x = .Mx(i).x : .Mx_Max.y = .Mx(i).y
        '    If .Mx_min.M > .Mx(i).M Then .Mx_min.M = .Mx(i).M : .Mx_min.x = .Mx(i).x : .Mx_min.y = .Mx(i).y
        '    If .My_Max.M < .My(i).M Then .My_Max.M = .My(i).M : .My_Max.x = .My(i).x : .My_Max.y = .My(i).y
        '    If .My_min.M > .My(i).M Then .My_min.M = .My(i).M : .My_min.x = .My(i).x : .My_min.y = .My(i).y
        '    If .Vx_Max.V < .Vx(i).V Then .Vx_Max.V = .Vx(i).V : .Vx_Max.x = .Vx(i).x : .Vx_Max.y = .Vx(i).y
        '    If .Vx_min.V > .Vx(i).V Then .Vx_min.V = .Vx(i).V : .Vx_min.x = .Vx(i).x : .Vx_min.y = .Vx(i).y
        '    If .Vy_Max.V < .Vy(i).V Then .Vy_Max.V = .Vy(i).V : .Vy_Max.x = .Vy(i).x : .Vy_Max.y = .Vy(i).y
        '    If .Vy_min.V > .Vy(i).V Then .Vy_min.V = .Vy(i).V : .Vy_min.x = .Vy(i).x : .Vy_min.y = .Vy(i).y
        '    If .Cz_Max.Cz < .Cz(i).Cz Then .Cz_Max.Cz = .Cz(i).Cz : .Cz_Max.x = .Cz(i).x : .Cz_Max.y = .Cz(i).y
        '    If .Cz_min.Cz > .Cz(i).Cz Then .Cz_min.Cz = .Cz(i).Cz : .Cz_min.x = .Cz(i).x : .Cz_min.y = .Cz(i).y
        '    If .Rx_Max.Rx < .Rx(i).Rx Then .Rx_Max.Rx = .Rx(i).Rx : .Rx_Max.y = .Rx(i).y
        '    If .Rx_min.Rx > .Rx(i).Rx Then .Rx_min.Rx = .Rx(i).Rx : .Rx_min.y = .Rx(i).y
        '    If .Ry_Max.Ry < .Ry(i).Ry Then .Ry_Max.Ry = .Ry(i).Ry : .Ry_Max.x = .Ry(i).x
        '    If .Ry_min.Ry > .Ry(i).Ry Then .Ry_min.Ry = .Ry(i).Ry : .Ry_min.x = .Ry(i).x
        '    If .W_fl_Max.wfl < .W_fl.Item(i).wfl Then .W_fl_Max.wfl = .W_fl.Item(i).wfl : .W_fl_Max.x = .W_fl.Item(i).x : .W_fl_Max.y = .W_fl.Item(i).y
        '    If .W_fl_min.wfl > .W_fl.Item(i).wfl Then .W_fl_min.wfl = .W_fl.Item(i).wfl : .W_fl_Max.x = .W_fl.Item(i).x : .W_fl_Max.y = .W_fl.Item(i).y
        'Next

        'For i = 0 To .Mx.Count - 1 'tous les attributs ont le même nombre d'item
        '    If .Mx_Max.M < .Mx(i).M Then .Mx_Max.M = .Mx(i).M : .Mx_Max.x = .Mx(i).x : .Mx_Max.y = .Mx(i).y
        '    If .Mx_min.M > .Mx(i).M Then .Mx_min.M = .Mx(i).M : .Mx_min.x = .Mx(i).x : .Mx_min.y = .Mx(i).y
        '    If .My_Max.M < .My(i).M Then .My_Max.M = .My(i).M : .My_Max.x = .My(i).x : .My_Max.y = .My(i).y
        '    If .My_min.M > .My(i).M Then .My_min.M = .My(i).M : .My_min.x = .My(i).x : .My_min.y = .My(i).y
        '    If .Vx_Max.V < .Vx(i).V Then .Vx_Max.V = .Vx(i).V : .Vx_Max.x = .Vx(i).x : .Vx_Max.y = .Vx(i).y
        '    If .Vx_min.V > .Vx(i).V Then .Vx_min.V = .Vx(i).V : .Vx_min.x = .Vx(i).x : .Vx_min.y = .Vx(i).y
        '    If .Vy_Max.V < .Vy(i).V Then .Vy_Max.V = .Vy(i).V : .Vy_Max.x = .Vy(i).x : .Vy_Max.y = .Vy(i).y
        '    If .Vy_min.V > .Vy(i).V Then .Vy_min.V = .Vy(i).V : .Vy_min.x = .Vy(i).x : .Vy_min.y = .Vy(i).y
        '    If .Cz_Max.Cz < .Cz(i).Cz Then .Cz_Max.Cz = .Cz(i).Cz : .Cz_Max.x = .Cz(i).x : .Cz_Max.y = .Cz(i).y
        '    If .Cz_min.Cz > .Cz(i).Cz Then .Cz_min.Cz = .Cz(i).Cz : .Cz_min.x = .Cz(i).x : .Cz_min.y = .Cz(i).y
        '    If .Rx_Max.Rx < .Rx(i).Rx Then .Rx_Max.Rx = .Rx(i).Rx : .Rx_Max.y = .Rx(i).y
        '    If .Rx_min.Rx > .Rx(i).Rx Then .Rx_min.Rx = .Rx(i).Rx : .Rx_min.y = .Rx(i).y
        '    If .Ry_Max.Ry < .Ry(i).Ry Then .Ry_Max.Ry = .Ry(i).Ry : .Ry_Max.x = .Ry(i).x
        '    If .Ry_min.Ry > .Ry(i).Ry Then .Ry_min.Ry = .Ry(i).Ry : .Ry_min.x = .Ry(i).x
        '    If .W_fl_Max.wfl < .W_fl.Item(i).wfl Then .W_fl_Max.wfl = .W_fl.Item(i).wfl : .W_fl_Max.x = .W_fl.Item(i).x : .W_fl_Max.y = .W_fl.Item(i).y
        '    If .W_fl_min.wfl > .W_fl.Item(i).wfl Then .W_fl_min.wfl = .W_fl.Item(i).wfl : .W_fl_Max.x = .W_fl.Item(i).x : .W_fl_Max.y = .W_fl.Item(i).y
        'Next
    End Sub

End Module
