﻿Imports System.Math
Imports System.IO
Imports System.Drawing.Imaging

Module Code_Eurocode

#Region "Code BA pour plaque sur 2 appuis"
    Dim V_Nul(0, 0, 0) As Double

    ''' <summary>Programme général de calcul BA suivant les eurocodes pour les plaques sur 2 appuis </summary>
    ''' <param name="Miso">Moment isostatique de la travée avec les indices (comb, i1, x) comb = n° de combinaison, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="Mur">Moment Ultime résistant avec les indices (ELU_ELS, i1, j, 1) ELU_ELs = n° de combinaison, i1 = n° de travée, j=1,2 ou 3 entravée, sur appui gauche, sur appui droit, 
    ''' 1 = n° de lit, de la surface vers l'intérieur de la dalle</param>
    '''  <param name="Mmax">Moment enveloppe Maxi de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    '''  <param name="Mmin">Moment enveloppe mini de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="IMmin">indice de la combinaison conduisant à la valeur maximale du moment sur appui droit de la travée i (en valeur absolue) avec les indices (ELU_ELS, i2): ELU_ELS = type d'etat Limite, i2 = n° de travée</param>
    ''' <param name="M"> Valeur du moment M(imax, PlanCont.Count, lc) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    ''' <param name="V">Valeur de l'effort tranchant V(imax, PlanCont.Count, lcMax) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    '''  <param name="rtf">variable renvoyant le fichier de note de calcul sous format rtf</param>
    ''' <param name="verif"> variable prenant la valeur False si une erreur bloquante s'est produite pendant le calcul</param>
    Public Sub Calcul_EC2_2_Appuis(ByVal Mmax(,,) As Double, ByVal Mmin(,,) As Double, ByVal EIf(,,) As Double, ByVal NbApp As Integer, ByVal Ry(,) As Double,
                         ByVal M(,,) As Double, ByVal V(,,) As Double, ByVal IMmin(,) As Integer, ByVal Miso(,,) As Double,
                         ByRef m1d(,,) As Double, ByRef m2d(,,) As Double, ByRef m3d(,,) As Double, ByRef Mur(,,,) As Double, ByRef x1Max(,) As Integer,
                          ByRef ΔM(,) As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter, ByRef verif As Boolean)
        ' Plancher en béton armé suivant l'Eurocode 2
        'Rappel:
        'Les efforts sont adressés en MN et MN.m
        Dim i, tour, Ind1(4), imax, Ind2(4), Num_Chap As Integer 'CombMax(1),
        ' Dim nbapp As Integer = 2 'provisoire
        Dim RyMax(2), Rymin(2) As Double
        Dim Acier_construction(PlanCont.Count) As Boolean
        Dim Modification_Acier_HA As Boolean = False
        Dim Ainf(3, PlanCont.Count, 4) As Double
        Dim Prefa_Fissuration As Boolean = False
        Dim msg As String = String.Empty
Debut_Programme:  'Etiquette

        Dim Limitation_Contrainte_Beton As Boolean = False
        Dim Maitrise_Fissuration As Boolean = True
        Dim Fleche(PlanCont.Count) As Boolean   'Pas de calcul de flèche à réaliser

        Call Initialisation(Acier_construction, Mur, x1Max, ΔM)
        aci.Initialisation()
        bet.Initialisation()

        For Each dalle As Plancher In PlanCont
            dalle.Init_Ferraillage() 'calcul espacement des aciers
            ' If dalle.Lx > 12 Then Aff.msg_Avertiss.Add("Travée n°" & (PlanCont.IndexOf(dalle) + 1).ToString & ": longueur de la portée principale >12 m: prévoir un redoublement des armatures principales.")
        Next
        'Détermination des indices de combinaisons
        Ind1(1) = 1
        Ind2(1) = Aff.ELU : imax = Aff.ELU
        If Aff.ELU_EQ > 0 Then 'ELU EQU équilibre statique = non soulèvement
            Ind1(2) = imax + 1
            Ind2(2) = Aff.ELU_EQ
            imax = Aff.ELU_EQ  '+ combMax(2) + combMax(3) + combMax(4)
        End If
        If Aff.ELScar > 0 Then 'ELS caractéristique
            Ind1(3) = imax + 1
            Ind2(3) = Aff.ELScar
            imax = Aff.ELScar
        End If
        If Aff.ELSQP > 0 Then 'ELS quasi-permanent
            Ind1(4) = imax + 1
            Ind2(4) = Aff.ELSQP
            imax = Aff.ELSQP
        End If
        ''-------------------------------------------------------------------------------
        ''   PROGRAMME PRINCIPAL :   Calcul des sections d'aciers et des contraintes
        ''--------------------------------------------------------------------------------   
        ''----->ELU Fondamental:
        ''-----> calcul des aciers de flexion (détermination des armatures et épure),
        ''-----> calcul et vérification des efforts sur appui (bielle, acier de glissement et entrainement)
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.NewLine() : rtf.Bold = True
        rtf.Underline = True : rtf.WriteString("2 - Détermination des armatures de flexion") : rtf.Bold = False : rtf.FontSize = 11 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("2 - 1  Calcul des sections d'acier") : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("Les calculs sont réalisés avec les modélisations suivantes:") : rtf.NewLine()
        rtf.WriteString("  - Diagramme simplifié rectangulaire des contraintes de compression du béton suivant art. 3.1.7(3) - figure 3.5") : rtf.NewLine()
        rtf.WriteString("  - Diagramme élasto-plastique parfait pour l'acier suivant art. 3.2.7(2) avec prise en compte éventuelle d'une branche supérieure inclinée suivant " &
                        "valeur  de la déformation de l'acier (voir tableau calcul ci-dessous).")
        If PlanCont.Count > 1 Then
            rtf.NewLine()
            If Aff.Redistrib_limit = False Then
                rtf.WriteString("  - Pas de redistribution limité des moments suivant art. 5.5.") : rtf.NewLine()
            Else
                rtf.WriteString("  - Analyse linéaire avec redistribution limité des moments suivant art. 5.5: δ = " & Aff.δ.ToString("0.00")) : rtf.NewLine()
            End If
            If Aff.Ecretage_Moment = False Then
                rtf.WriteString("  - Pas d'écrêtage des moments sur appui.") : rtf.NewLine()
            Else
                rtf.WriteString("  - Ecrêtage des moments sur appui suivant conditions art. 5.3.2.2(4). Voir valeur ΔM dans tableau.") : rtf.NewLine()
            End If
        End If
        If PlanCont(0).Type_Acier_Travée = 2 Then 'acier HA
            For Each dalle As Plancher In PlanCont
                With dalle
                    Dim EspMaxSecond As Double
                    Select Case dalle.Choix_Esp
                        Case 1 'zone standard
                            If 3 * .H > 0.4 Then .EspMax = 0.4 Else .EspMax = 3 * .H
                            If 3.5 * .H > 0.45 Then EspMaxSecond = 0.45 Else EspMaxSecond = 3.5 * .H
                        Case 2 'zone charge concentrée
                            If 2 * .H > 0.25 Then .EspMax = 0.25 Else .EspMax = 2 * .H
                            If 3 * .H > 0.4 Then EspMaxSecond = 0.4 Else EspMaxSecond = 3 * .H
                        Case 3 'esp max imposé
                            ' .EspMaxPrincipal = .EspMax = déjà renseigné dans le module Ferraillage
                            EspMaxSecond = .EspMax
                    End Select
                    .EspMaxPrincipal = .EspMax : .EspminPrincipal = .Espmin
                    .EspMaxSecondaire = EspMaxSecond : .EspminSecondaire = .Espmin
                End With
            Next
            rtf.WriteString("  - Espacement minimal pour acier principal = " & (PlanCont(0).EspminPrincipal * 100).ToString("#.0") & " cm - Espacement Maximal pour acier principal = " & (PlanCont(0).EspMaxPrincipal * 100).ToString("#.0")) : rtf.NewLine()
            rtf.WriteString("  - Espacement minimal pour acier secondaire = " & (PlanCont(0).EspminSecondaire * 100).ToString("#.0") & " cm - Espacement Maximal pour acier secondaire = " & (PlanCont(0).EspMaxSecondaire * 100).ToString("#.0") & " cm.") : rtf.NewLine()
        End If

        rtf.NewLine()
        rtf.WriteEndGroup()
        Call Calcul_section_ELU_Fondamental(Mmax, Mmin, M, V, Rymin, IMmin, Ainf, Modification_Acier_HA, verif, Acier_construction, m1d, m2d, m3d, Mur, x1Max, ΔM, rtf)
        If verif = False Then Exit Sub
        '______________________________________________________________________________________________________________________________________________________________________________________
        ''--->ELS: vérification  à l'ELS
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.NewLine() : rtf.Bold = True
        rtf.Underline = True : rtf.WriteString("3 - Vérifications à l'Etat Limite de Service") : rtf.Bold = False : rtf.FontSize = 11 : rtf.NewLine() : rtf.NewLine()
        '______________________________________________________________________________________________________________________________________________________________________________________
        '----->Calcul des contraintes
        rtf.WriteString("3 - 1  Limitation des contraintes") : rtf.Underline = False : rtf.NewLine()
        rtf.FontSize = 9 : rtf.NewLine()
        If Aff.ELScar = 0 Then
            rtf.WriteString("Aucune combinaison ELS Caractéristique n'a été renseignée - La verification de la limitation des contraintes n'est pas établie.") : rtf.NewLine() : rtf.NewLine()
            rtf.WriteEndGroup()
        Else
            rtf.WriteString("Vérification des contraintes de flexion sous combinaison ELS caractéristique") : rtf.NewLine()
            For i = 1 To PlanCont.Count
                With PlanCont.Item(i - 1)
                    If .XD1 = True Or .XD2 = True Or .XD3 = True Or .XS1 = True Then
                        rtf.WriteString(" - Travée n°" & i.ToString & ": σb ≤ 0,6.fck - Classe d'exposition XD") : rtf.NewLine()
                    ElseIf .XS2 = True Or .XS3 = True Then
                        rtf.WriteString(" - Travée n°" & i.ToString & ": σb ≤ 0,6.fck - Classe d'exposition XS") : rtf.NewLine()
                    ElseIf .XF1 = True Or .XF2 = True Or .XF3 = True Or .XF4 = True Then
                        rtf.WriteString(" - Travée n°" & i.ToString & ": σb ≤ 0,6.fck - Classe d'exposition XF") : rtf.NewLine()
                    End If
                End With
            Next
            rtf.WriteString("- Toutes travées : σs ≤ 0,8.fyk") : rtf.NewLine()
            rtf.WriteEndGroup()
            Call Calcul_section_ELS(Mmax, Mmin, IMmin, V, ΔM, verif, Acier_construction, m1d, m2d, m3d, Mur, x1Max, Ainf, Modification_Acier_HA, rtf)
            If Modification_Acier_HA = True Then

                GoTo Debut_Programme
            End If
        End If
        '______________________________________________________________________________________________________________________________________________________________________________________
        '----->Maitrise de la fissuration
        rtf.WriteStartGroup()
        rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 11
        rtf.Underline = True : rtf.WriteString("3 - 2  Maitrise de la fissuration") : rtf.Underline = False : rtf.NewLine()
        rtf.WriteEndGroup()

        If Aff.ELSQP = 0 Then
            rtf.WriteStartGroup() : rtf.FontSize = 9 : rtf.NewLine()
            rtf.WriteString("Aucune combinaison ELS quasi-permanente de charges n'a été renseignée - La verification de la fissuration n'est pas établie.") : rtf.NewLine() : rtf.NewLine()
            rtf.WriteEndGroup()
        Else
            Maitrise_Fissuration = False : tour = 1
            For Each dalle As Plancher In PlanCont
                If PlanCont.IndexOf(dalle) = 0 And dalle.LappG < 0.05 Then Continue For 'pas de maitrise de fissuration sur les balcons
                If PlanCont.IndexOf(dalle) = PlanCont.Count - 1 And dalle.LappD < 0.05 Then Continue For 'pas de maitrise de fissuration sur les balcons
                With dalle
                    If .X0 = True Or .XC1 = True Then
                        rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                        If .X0 = True Then msg = "X0" Else msg = "XC1"
                        If PlanCont.IndexOf(dalle) >= 1 Then
                            If PlanCont.Item(PlanCont.IndexOf(dalle) - 1).X0 = True Or PlanCont.Item(PlanCont.IndexOf(dalle) - 1).XC1 = True Then
                                rtf.WriteString("Travée n°" & (PlanCont.IndexOf(dalle) + 1).ToString & ": idem que travée précédente.") : rtf.NewLine()
                            Else
                                rtf.WriteString("Travée n°" & (PlanCont.IndexOf(dalle) + 1).ToString & ":") : rtf.NewLine()
                                rtf.WriteString("Travée classée " & msg & " - pas de calcul de fissuration demandé, les dispositions constructives suffisent à assurer la maitrise de la fissuration.") : rtf.NewLine()
                                rtf.WriteString("Limite de la fissuration Wmax = 0,40 mm.") : rtf.NewLine()
                            End If
                        Else
                            rtf.WriteString("Travée n°" & (PlanCont.IndexOf(dalle) + 1).ToString & ":") : rtf.NewLine()
                            rtf.WriteString("Travée classée " & msg & " - pas de calcul de fissuration demandé, les dispositions constructives suffisent à assurer la maitrise de la fissuration.") : rtf.NewLine()
                            rtf.WriteString("Limite de la fissuration Wmax = 0,40 mm.") : rtf.NewLine()
                        End If
                        rtf.WriteEndGroup()
                    ElseIf .XC2 = True Or .XC3 = True Or .XC4 = True Then
                        For Each chges As Charge In dalle.Chge
                            If chges.Nature = 2 Or chges.Nature = 3 Then 'charges catégories A,B,C et D
                                rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                                If .XC2 = True Then
                                    msg = "XC2"
                                ElseIf .XC3 Then
                                    msg = "XC3"
                                Else
                                    msg = "XC4"
                                End If
                                rtf.WriteString("Travée n°" & (PlanCont.IndexOf(dalle) + 1).ToString & ":") : rtf.NewLine()
                                rtf.WriteString("Travée classée" & msg & " - pas de calcul de fissuration demandé car les charges d'exploitation appartiennent à la catégorie A à D. " &
                                                "Suivant l'Annexe Nationale, les dispositions constructives suffisent à assurer la maitrise de la fissuration.") : rtf.NewLine()
                                rtf.WriteString("Limite de la fissuration Wmax = 0,30 mm.") : rtf.NewLine()
                                rtf.WriteEndGroup()
                                Maitrise_Fissuration = True : Exit For
                            End If
                        Next
                        If Maitrise_Fissuration = False Then

                            rtf.WriteStartGroup() : rtf.FontSize = 9 : rtf.NewLine()
                            rtf.WriteString("Travée n°" & (PlanCont.IndexOf(dalle) + 1).ToString & ":") : rtf.NewLine()
                            rtf.WriteEndGroup()
                            Prefa_Fissuration = False
                            If .Prefa = True Then
                                Call Fissuration_Predalle(dalle, Mmax, Mmin, m1d, m2d, m3d, rtf, Prefa_Fissuration)
                            End If
                            If tour = 1 And Prefa_Fissuration = False Then
                                rtf.WriteString("Détermination de la fissuration par application du schéma suivant:") : rtf.NewLine()
                                rtf.WriteString("1 - Vérification de la section minimale d'armature - Application de l'article 7.3.2") : rtf.NewLine()
                                rtf.WriteString("2 - Vérification de l'ouverture des fissures soit sans calcul direct soit avec calcul direct") : rtf.NewLine()
                            End If
                            rtf.WriteEndGroup()
                            Call Fissuration_verification_forfaitaire(tour, PlanCont.IndexOf(dalle) + 1, rtf, verif, Maitrise_Fissuration)
                            If Maitrise_Fissuration = False Then
                                rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                                rtf.NewLine() : rtf.WriteString("La méthode forfaitaire de maitrise de l'ouverture des fissures n'étant pas concluante, il est procédé au calcul direct de l'ouverture des fissures suivant Art. 7.3.4") : rtf.NewLine() : rtf.NewLine()
                                rtf.WriteEndGroup()
                                Call Fissuration_Calcul_Direct(PlanCont.IndexOf(dalle) + 1, Mmax, Mmin, m1d, m2d, m3d, rtf, verif, Maitrise_Fissuration)
                            End If
                            tour = tour + 1
                        End If
                    ElseIf .XD1 = True Or .XD2 = True Or .XD3 = True Or .XS1 = True Or .XS2 = True Or .XS3 = True Then
                        Prefa_Fissuration = False
                        If .Prefa = True Then
                            Call Fissuration_Predalle(dalle, Mmax, Mmin, m1d, m2d, m3d, rtf, Prefa_Fissuration)
                        End If
                        If Prefa_Fissuration = False Then
                            rtf.WriteStartGroup() : rtf.FontSize = 9 : rtf.NewLine()
                            rtf.WriteString("Détermination de la fissuration par application du schéma suivant:") : rtf.NewLine()
                            rtf.WriteString("1 - Vérification de la section minimale d'armature - Application de l'article 7.3.2") : rtf.NewLine()
                            rtf.WriteString("2 - Vérification de l'ouverture des fissures soit sans calcul direct soit avec calcul direct") : rtf.NewLine()
                            rtf.WriteEndGroup()
                            Call Fissuration_verification_forfaitaire(tour, PlanCont.IndexOf(dalle) + 1, rtf, verif, Maitrise_Fissuration)
                            If Maitrise_Fissuration = False Then
                                rtf.WriteStartGroup()
                                rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                                rtf.NewLine() : rtf.WriteString("La méthode forfaitaire de maitrise de l'ouverture des fissures n'étant pas concluante, il est procédé au calcul direct de l'ouverture des fissures suivant Art. 7.3.4") : rtf.NewLine() : rtf.NewLine()
                                rtf.WriteEndGroup()
                                Call Fissuration_Calcul_Direct(PlanCont.IndexOf(dalle) + 1, Mmax, Mmin, m1d, m2d, m3d, rtf, verif, Maitrise_Fissuration)
                            End If
                        End If
                    End If
                End With
            Next
        End If
        '______________________________________________________________________________________________________________________________________________________________________________________
        '----->Calcul des flèches
        rtf.WriteStartGroup()
        rtf.NewLine()
        rtf.FontSize = 12 : rtf.FontSize = 11
        rtf.Underline = True : rtf.WriteString("3 - 3  Détermination des flèches") : rtf.Underline = False : rtf.NewLine() : rtf.FontSize = 9
        If PlanCont.Item(0).Prefa = True Then
            rtf.WriteString("Le calcul des flèches est réalisé suivant les dispositions de §3.5.1 du CPT. Les limites de flèches sont tirées du §3.5.3 du CPT.") : rtf.NewLine()
            rtf.WriteEndGroup()
            Call Fleche_Predalle(Mmin, rtf)
        Else
            rtf.WriteString("Détermination des conditions limites de flèches par le calcul des élancements suivant les dispositions de l'EN1992-1-1 §7.4.2") : rtf.NewLine()
            rtf.WriteEndGroup()
            Call Dispense_Calcul_Fleche(1, Fleche, rtf, Ainf)
            For i = 1 To PlanCont.Count
                rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                If Fleche(i) = False Then
                    If Aff.ELSQP = 0 Then
                        msg = "Flèche: vérification sur rapport géométrique (EN1992-1-1 Art. 7.4.2) non satisfaisante et calcul de flèche non effectué."
                        Aff.msg_Avertiss.Add(msg)
                        rtf.WriteString("Aucune combinaison ELS quasi-permanente de charges n'a été renseignée - La vérification de la flèche n'est pas établie.") : rtf.NewLine() : rtf.NewLine()
                        rtf.WriteEndGroup()
                    Else
                        rtf.WriteString("Les élancements dépassent les limites autorisées par l'Eurocode, le calcul explicite des flèches est exigé.") : rtf.NewLine()
                        rtf.WriteString("Le calcul est réalisé suivant les dispositions de l'EN1992-1-1 §7.4.3") : rtf.NewLine()
                        rtf.WriteEndGroup()
                        ' 4 = combinaison ELS quasi-permanente, Ind1(4)=1ier numéro de combinaison, Ind2(4)= dernier n° de combinaison
                        Call Calcul_Fleche(4, Ind1, Ind2, M, EIf, rtf) 'calcul flèche poutre
                    End If
                Else
                    rtf.WriteString("Travée n°" & i.ToString & " : les élancements ne dépassent pas les limites autorisées par l'Eurocode, les conditions de flèche sont vérifiées.") : rtf.NewLine()
                    rtf.WriteEndGroup()
                End If
            Next

        End If
        Num_Chap = 4
        '______________________________________________________________________________________________________________________________________________________________________________________
        '----->Vérification du monolithisme pour les planchers à prédalles
        If PlanCont(0).Prefa = True Then
            rtf.WriteStartGroup()
            rtf.FontSize = 12 : rtf.NewLine()
            rtf.Underline = True : rtf.Bold = True : rtf.WriteString("4 - Vérification du monolithisme du plancher") : rtf.Underline = False : rtf.Bold = False : rtf.NewLine()
            rtf.FontSize = 9 : rtf.WriteString("Vérification du monolithisme du plancher suivant les dispositions de §2.3.2 du CPT") : rtf.NewLine()
            rtf.WriteEndGroup()
            Call Monolithisme_Prefa(Num_Chap, rtf)
            Num_Chap = 5
        End If
        '______________________________________________________________________________________________________________________________________________________________________________________
        '----->ELU Effort Tranchant: détermination des cadres
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.NewLine() : rtf.Bold = True
        rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - Dimensionnement de la section de béton et des armatures à l'Effort Tranchant") : rtf.Bold = False : rtf.FontSize = 11 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString(Num_Chap.ToString & " - 1  Clauses générales") : rtf.Underline = False : rtf.NewLine() : rtf.NewLine() : rtf.FontSize = 9
        rtf.WriteString("Hypothèses générales prises pour le calcul de l'ensemble des travées:") : rtf.NewLine()
        rtf.WriteString("  ■ Angle Θ de la bielle béton comprimé: 45° soit Cot(Θ)= 1") : rtf.NewLine()
        rtf.WriteString("  ■ Angle α des armatures d'effort tranchant (si existe): 90° soit Cot(α)= 0") : rtf.NewLine()
        rtf.WriteString("  ■ [EC2, 6.2.3(5)] - Décalage de l'Effort Tranchant: ")
        If Aff.Decalage_ET = False Then
            rtf.WriteString("Non appliqué") : rtf.NewLine()
        Else
            rtf.WriteString("Pris en compte") : rtf.NewLine()
        End If
        rtf.WriteString("  ■ Minoration pour les charges appliquées près de l'appui pour le calcul de l'Effort Tranchant suivant conditions de l'article  6.2.1(8) de l'EN1992-1-1:  ")
        If Aff.MinorationQ_ET = False Then
            rtf.WriteString("Non appliqué") : rtf.NewLine()
        Else
            rtf.WriteString("Non appliqué") : rtf.NewLine() ' rtf.WriteString("Pris en compte") : rtf.NewLine()  provisoire
        End If
        rtf.WriteString("  ■ Espacement 1ier cadre: St/2") : rtf.NewLine()

        rtf.WriteString("  ■ Ø Maxi de l'armature d'effort tranchant =  Ø de l'armature longitudinale") : rtf.NewLine()
        rtf.WriteEndGroup()
        Call Effort_Tranchant_Plancher(Num_Chap, verif, rtf) : If verif = False Then Exit Sub
        '
        '---------------> Vérification des appuis
        Num_Chap = Num_Chap + 1
        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.Bold = True : rtf.Underline = True
        rtf.WriteString(Num_Chap.ToString & " - Vérification des appuis") : rtf.Bold = False : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine()
        rtf.WriteString("Seuls les appuis perpendiculaires au sens directionnel du plancher sont vérifiés.")
        rtf.WriteEndGroup()
        Call verification_appui(NbApp, Ry, M, V, rtf)
        '
        '---------------> Vérification des trémies
        rtf.WriteStartGroup()
        rtf.NewParagraph() : rtf.FontSize = 12 : rtf.Bold = True : rtf.Underline = True : Num_Chap = Num_Chap + 1
        rtf.WriteString(Num_Chap.ToString & " - Vérification des trémies") : rtf.Bold = False : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine()
        i = 0
        For Each dalle As Plancher In PlanCont
            With dalle
                i = i + ._Tremie.Count
            End With
        Next
        If i = 0 Then
            rtf.NewLine() : rtf.WriteString("Aucune trémie") : rtf.NewLine()
            rtf.WriteEndGroup()
        Else
            rtf.WriteEndGroup()
            Select Case PlanCont.Item(0).Prefa
                Case True
                    Call Calcul_Tremie_Predalle(rtf)
                Case False
                    Call Calcul_Tremie_DallePleine(rtf)
            End Select
        End If
        '
        '---------------> calcul au feu
        rtf.WriteStartGroup()
        rtf.NewParagraph() : rtf.FontSize = 12 : rtf.Bold = True : Num_Chap = Num_Chap + 1
        rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - Résistance au feu") : rtf.Bold = False : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteEndGroup()
        Call Calcul_au_Feu(Miso, Mur, rtf) 'Vérification de la stabilité au feu
        '
        '---------------> Vérification phase provisoire - Uniquement pour plancher avec prédalle
        If PlanCont.Item(0).Prefa = True Then
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.Bold = True : rtf.Underline = True : Num_Chap = Num_Chap + 1
            rtf.WriteString(Num_Chap.ToString & " - Vérification phases provisoires") : rtf.Bold = False : rtf.FontSize = 11 : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString(Num_Chap.ToString & " - 1   Vérification des dimensions des repos sur appuis") : rtf.Underline = False : rtf.NewLine()
            rtf.WriteEndGroup()
            Call Verification_Repos_Appui_Predalle(rtf)
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 11
            rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - 2   Vérification du critère de sécurité") : rtf.Underline = False : rtf.NewLine()
            rtf.WriteString(" Vérification établie suivant les conditions de l'article 4.2.1 du CPT.") : rtf.NewLine()
            rtf.WriteEndGroup()
            Call Verification_etaiement(rtf)
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 11
            rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - 3   Vérification du critère d'intégrité") : rtf.Underline = False : rtf.NewLine()
            rtf.WriteString(" Vérification établie suivant les conditions de l'article 4.2.2 du CPT.") : rtf.NewLine()
            rtf.WriteEndGroup()
            Call Verification_integrite(rtf)
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 11
            rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - 4   Vérification du critère de déformation") : rtf.Underline = False : rtf.NewLine()
            rtf.WriteString(" Vérification établie suivant les conditions de l'article 4.2.3 du CPT.") : rtf.NewLine()
            rtf.WriteEndGroup()
            Call Verification_Deformation(rtf)
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 11
            rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - 5   Vérification manutention par boucle") : rtf.Underline = False : rtf.NewLine()
            rtf.WriteString("4 boucles de levage par prédalle en acier de classe B235C et de diamètre minimal 10 mm.") : rtf.NewLine()
            rtf.WriteString("Répartition de l'effort de levage sur 3 appuis: prédalle souple de moins de 8 cm") : rtf.NewLine()
            rtf.WriteString("Angle d'élingage inférieur à 60°.") : rtf.NewLine()
            rtf.WriteEndGroup()
            Call Manutention_boucle(rtf)
        End If
        '
        '---------------> Calcul des aciers complémentaires pour prédalles:
        '------------     - Chapeaux entre joints
        '-----------      - Aciers sur appuis non porteurs
        If PlanCont.Item(0).Prefa = True Then
            Call Armatures_complementaires_predalles()
        End If
        '
        '---------------> calcul du métré
        Call Metre()
        '
        '---------------> Affichage des avertissements
        rtf.WriteStartGroup()
        rtf.NewParagraph() : rtf.FontSize = 12 : rtf.Bold = True : Num_Chap = Num_Chap + 1
        rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - Avertissements") : rtf.Bold = False : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        If Aff.msg_Avertiss.Count > 0 Then
            Dim i1 As Integer
            For i1 = 0 To Aff.msg_Avertiss.Count - 1
                rtf.WriteString(Aff.msg_Avertiss.Item(i1).ToString()) : rtf.NewLine()
            Next
        Else
            rtf.WriteString("Aucun avertissement.") : rtf.NewLine()
        End If
        rtf.NewParagraph() : rtf.FontSize = 12 : rtf.Bold = True : Num_Chap = Num_Chap + 1
        rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - Plans") : rtf.Bold = False : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("Vous trouverez ci-après les plans d'exécution des ouvrages établis suivant la note de calcul.") : rtf.NewLine()
        rtf.WriteString("Attention: ces plans ne sont pas à l'échelle. Ils servent simplement à illustrer la note de calcul.") : rtf.NewLine()
        rtf.WriteString("Les plans à l'échelle doivent être tirés directement depuis le logiciel via la commande imprimer.") : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("Le dossier de plans se compose de:") : rtf.NewLine() : rtf.NewLine()
        If PlanCont.Item(0).Prefa = True Then

            rtf.WriteString("   → Plan de calepinage des prédalles.") : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("   → Plan des prédalles.") : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("   → Plan ferraillage inférieur sur prédalles") : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("   → Plan ferraillage supérieur sur prédalles") : rtf.NewLine()
        Else
            rtf.WriteString("   → Plan Ferraillage - Armatures basses") : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("   → Plan Ferraillage - Armatures hautes") : rtf.NewLine() : rtf.NewLine()
        End If
        rtf.WriteEndGroup()
        '
        'affichage des différents plans
        rtf.WriteStartGroup()

        Try
            Dim myImage As Image
            Dim g As Graphics
            '
            'Si plancher préfa:
            '   --->   dessin du positionnement des prédalles
            '   --->   dessin des prédalles
            '
            If PlanCont.Item(0).Prefa = True Then
                '
                '--->   dessin du positionnement des prédalles
                '_______________________________________________________________________________________________________________________________________________________________
                myImage = My.Resources.Test1250_890 'Image.FromStream(Img)
                g = Graphics.FromImage(myImage)
                Call Affichage_Coffrage_Ferraillage.Affichage_Dalle_Pleine(g, 2, 3, 1250, 890)
                myImage.Save(Directory.GetCurrentDirectory.ToString() & "\test1.bmp") ', ImageFormat.Bmp)
                g.Dispose()
                myImage.RotateFlip(RotateFlipType.Rotate90FlipNone)
                rtf.InsertImage(myImage)
                myImage.Dispose()
                '
                '--->   dessin des prédalles
                '_______________________________________________________________________________________________________________________________________________________________

                For i = 1 To PlanCont.Count
                    Dim NumPredal, modele As Integer
                    NumPredal = 0 : modele = 1
                    Do
                        Dim Titre As String = String.Empty
                        ' Call Parcours_Predalle(True)
                        With PlanCont.Item(i - 1)
                            For j = 1 To .Prefa_nb
                                If .Prefa_modele.Item(j - 1) = modele Then
                                    If Titre = String.Empty Then
                                        Titre = "Prédalle(s) n°" & (i * 10 + j).ToString("#")
                                        NumPredal = j
                                    Else
                                        Titre = Titre & " - " & (i * 10 + j).ToString("#")
                                    End If
                                End If
                            Next
                        End With
                        'test de sortie
                        If NumPredal = 0 Then Exit Do
                        'Dessin de la prédalle
                        myImage = My.Resources.Test1250_890 'Image.FromStream(Img)
                        g = Graphics.FromImage(myImage)
                        Call Affichage_Coffrage_Ferraillage.Predalle(g, 2, 1250, 890, i, NumPredal, Titre)
                        myImage.Save(Directory.GetCurrentDirectory.ToString() & "\test1.bmp") ', ImageFormat.Bmp)
                        g.Dispose()
                        myImage.RotateFlip(RotateFlipType.Rotate90FlipNone)
                        rtf.InsertImage(myImage)
                        myImage.Dispose()
                        modele += 1 : NumPredal = 0
                    Loop

                Next
            End If

            '
            'dessin ferraillage inférieur
            myImage = My.Resources.Test1250_890 'Image.FromStream(Img)
            g = Graphics.FromImage(myImage)
            If PlanCont.Item(0).Prefa = True Then
                'dessin des armatures inférieures complémentaires sur les prédalles
                Call Affichage_Coffrage_Ferraillage.Affichage_Dalle_Pleine(g, 2, 5, 1250, 890)
            Else
                'dessin du ferraillage des armatures inférieures
                Call Affichage_Coffrage_Ferraillage.Affichage_Dalle_Pleine(g, 2, 1, 1250, 890)
            End If
            myImage.Save(Directory.GetCurrentDirectory.ToString() & "\test1.bmp") ', ImageFormat.Bmp)
            g.Dispose()
            myImage.RotateFlip(RotateFlipType.Rotate90FlipNone)
            rtf.InsertImage(myImage)
            myImage.Dispose()
            '
            'dessin ferraillage supérieur
            ' msg = "\Images\Test1100_890.png"
            '  Img = New FileStream(Directory.GetCurrentDirectory.ToString() & msg, FileMode.Open)
            myImage = My.Resources.Test1250_890 'Image.FromStream(Img)
            g = Graphics.FromImage(myImage)
            Call Affichage_Coffrage_Ferraillage.Affichage_Dalle_Pleine(g, 2, 2, 1250, 890)
            myImage.Save(Directory.GetCurrentDirectory.ToString() & "\test1.bmp") ', ImageFormat.Bmp)
            g.Dispose()
            myImage.RotateFlip(RotateFlipType.Rotate90FlipNone)
            rtf.InsertImage(myImage)
            ''myImage = Image.FromFile(Directory.GetCurrentDirectory.ToString() & msg)
            'Img.Close()
            My.Computer.FileSystem.DeleteFile(Directory.GetCurrentDirectory.ToString() & "\test1.bmp") 'effacement du fichier
        Catch ex As Exception
            msg = "Le logiciel n'a pas pu charger l'image correspondante." & Environment.NewLine & "Erreur: " & ex.Message & ex.StackTrace & Environment.NewLine & "Note de calcul sans image"
            MessageBox.Show(msg, "Plancher BA EC2", MessageBoxButtons.AbortRetryIgnore, MessageBoxIcon.Exclamation)
        End Try
        rtf.WriteEndGroup()
    End Sub
    Private Sub Initialisation(ByRef acier_construction() As Boolean, ByRef Mur(,,,) As Double, ByRef x1Max(,) As Integer, ByRef ΔM(,) As Double)
        '----------------------------------------------------------------------------
        '                    Mise à zéro et initialisation des variables
        '----------------------------------------------------------------------------
        Dim i As Integer ' 
        ' m1d, m2d, m3d: remis à 0 dans module RDM
        ReDim Mur(1, PlanCont.Count, 3, 4), x1Max(3, PlanCont.Count), ΔM(PlanCont.Count, 3)
        'aciers de construction
        For i = 1 To PlanCont.Count
            acier_construction(i) = True ': construction(i1) = 0
        Next

        'aciers longitudinaux
        For i = 0 To PlanCont.Count - 1
            With PlanCont.Item(i)
                .TS_trav.Clear() : .TS_AppB.Clear() : .TS_AppD.Clear() : .TS_AppG.Clear() : .TS_AppH.Clear()
                .LitInf.Clear() : .LitInf_Y.Clear() : .LitSupD.Clear() : .LitSupD_Y.Clear() : .LitSupG.Clear() : .LitSupG_Y.Clear()
                .TS_trav_Portefeuille.Clear()
            End With
        Next i

    End Sub
    Private Sub Valeur_Decalage_Moment(ByRef i1 As Integer)
        With PlanCont.Item(i1 - 1)
            .al = 0.9 * PlanCont.Item(i1 - 1).H
            .al = Truncate(.al * 100) / 100 'arrondi au cm inférieur
        End With
    End Sub
    Private Sub ancrage_acier_console(ByVal i1 As Integer, ByVal φHA As Double, ByRef l1 As Double, ByRef l2 As Double, ByRef l3 As Double)
        '---------------------------------------
        ' ancrage des aciers en bout de console
        '---------------------------------------
        'variables: i1 = n° de travée
        '           i2 = indice du diamètre du lit d'acier
        Dim angl, r As Double
        angl = PI / 2 'conversion de l'angle en radian
        r = aci.dmL(aci.Ø_Norm.IndexOf(φHA)) / 2 + φHA
        l2 = angl * r 'l2=longueur développé sur partie courbe
        l3 = aci.dmL(aci.Ø_Norm.IndexOf(φHA)) / 2 + φHA
        l1 = PlanCont.Item(i1 - 1).H - PlanCont.Item(i1 - 1).enrSup - l3 - Aff.Consol_a '  = a = distance de l'arrêt
    End Sub
    ''' <summary>Calcul des aciers supérieurs et inférieurs en flexion simple - dalle rectangulaire    ''' </summary>
    ''' <param name="ELU_ELS">indique la combinaison: 1=ELU Str/ </param>
    '''  <param name="i1">n° de travée</param>
    '''  <param name="j">indique l'armature de flexion: 1=en travée/ 2=appui gauche/ 3=appui droit</param>
    ''' <param name="mu">Moment Maxi sollicitant pour la combinaison considérée</param>
    ''' <param name="d"> bras de levier</param>
    ''' <param name="Ainf">Section théorique d'accier pour reprendre l'effort - indice (ELU_ELS, i1, j) avec limites =(3, PlanCont.Count, 4): ELU_ELS=1 à 3(ELU Str, ELU Eq, ELS car); i1= n° travée; j=position armature = 1=position centrale, 2=appui gauche, 3=appui droit, 4=aciers supérieurs sur appuis haut et bas  </param>
    ''' <param name="Ainfc"> </param>
    ''' <param name="fydC"> </param>
    '''  <param name="str">variable permettant de renvoyer divers éléments vers le fichier de note de calcul</param>
    ''' <param name="verif"> variable prenant la valeur False si une erreur bloquante s'est produite pendant le calcul</param>
    ''' <param name="message">variable permettant de renvoyer divers éléments vers le fichier de note de calcul</param>
    Private Sub Calcul_ELU_armature_flexion_poutre_rectangulaire(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal j As Integer, ByVal Mu As Double, ByVal d As Double,
                                                                 ByRef Ainf(,,) As Double, Optional ByRef AinfC As Double = 0, Optional ByRef fydC As Double = 0, Optional ByRef str() As Double = Nothing,
                                                                 Optional ByRef message As Integer = 0, Optional ByRef verif As Boolean = True)

        Dim λ, αu, αAB, εspL, εud, εs, εc, εcu3, εc3, µu, µlimu, εuk, γs, γc As Double 'η, µlim, σcLim,
        Dim fyd, fcd, fud, k, zu As Double
        Select Case ELU_ELS
            Case 1
                γc = bet.γcd : γs = aci.γsd 'combinaison fondamentale
            Case 2
                γc = bet.γca : γs = aci.γsa 'combinaison acidentelle
        End Select
        'détermination paramètrique pour le béton
        λ = 0.8
        εcu3 = 3.5 : εc3 = 1.75
        fcd = bet.fck / γc
        'détermination paramétrique pour l'acier
        fyd = aci.fyk / γs : str(11) = fyd
        Select Case aci.classe
            Case 1 'A
                k = 1.05 : εuk = 25' valeur en pour mille
            Case 2 'B
                k = 1.08 : εuk = 50
            Case 3 'C
                k = 1.15 : εuk = 75
        End Select
        Ainf(ELU_ELS, i1, j) = 0
        With PlanCont.Item(i1 - 1)
            'Variables générales avant le calcul par pivot
            µu = Mu / (1 * d ^ 2 * fcd) : str(2) = µu 'largeur de base = 1m
            εud = εuk * 0.9 'EN1992-1-1 Art. 3.2.7
            εspL = fyd / aci.Es : εspL = εspL * 1000
            fud = fyd + fyd * (k - 1) * (εud - εspL) / (εuk - εspL) 'contrainte limite de l'acier sur branche inclinée
            αAB = εcu3 / (εcu3 + εud) ' Voir cours INSA Rennes, identique pour εcu2 car εcu3=εcu2
            µlimu = λ * αAB * (1 - (λ / 2) * αAB) : str(1) = µlimu 'Moment réduit limite entre pivot A et pivot B
            'Calcul suivant pivot
            If µu < µlimu Then
                'Pivot A
                εs = εud
                αu = (1 - (1 - 2 * µu) ^ 0.5) / λ
                zu = d * (1 - λ * αu / 2)
                Ainf(ELU_ELS, i1, j) = Mu / (zu * fud)
                εc = εud * αu / (1 - αu) 'déformation sur la fibre béton
                If εc < εc3 Then
                    message = 22 ' σc =   fcd * εc / εc3                    'section béton mal utilisé = béton surabondant
                Else
                    ' σc =   fcd
                End If
                AinfC = -1000 'Paour éviter des pb dans la suite du logiciel
                str(10) = fud
                fydC = fud 'contrainte qui s'applique sur l'armature
            Else
                'Pivot B
                αu = (1 - (1 - 2 * µu) ^ 0.5) / λ
                zu = d * (1 - λ * αu / 2)
                Ainf(ELU_ELS, i1, j) = Mu / (zu * fyd)
                εs = εcu3 * (1 - αu) / αu

                If εs < εspL Then
                    'calcul avec acier comprimé
                    message = 21 ' rtf.WriteString("εs < εspL - La poutre nécessite des armatures comprimés - Calcul arrêté !") : rtf.NewLine()
                    MessageBox.Show("Le plancher nécessite des armatures comprimés." & Environment.NewLine & "Calcul arrêté !", "EC2 - module calcul flexion simple ELU:", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                    verif = False
                ElseIf εs > εud Then
                    AinfC = -1000 'Pas de correction - palier plastique
                    fydC = fyd : str(10) = fydC
                Else
                    fydC = fyd + fyd * (k - 1) * (εs - εspL) / (εuk - εspL)
                    AinfC = Mu / (zu * fydC) : str(10) = fydC
                End If
            End If

            'vérification section maximale
            If Ainf(1, i1, j) > 0.04 * (1 * .H) Then
                Dim msg As String
                msg = "Vérification section maximale acier: La section d'acier demandée (" & (Ainf(1, i1, j) * 10000).ToString("#,##0.00") & " cm²) pour la travée " & i1.ToString &
                Environment.NewLine & "est beaucoup trop importante ! Modifiez la largeur ou la hauteur de la dalle."
                Aff.msg_Avertiss.Add(msg)
                Ainf(1, i1, j) = 0.04 * (1 * .H)
            End If
        End With
        str(3) = αu : str(4) = zu : str(5) = Ainf(ELU_ELS, i1, j) : str(6) = εc : str(7) = εs : str(8) = εspL : str(9) = AinfC

    End Sub
    Private Sub Tableau_ELU_Armature_Flexion(ByVal Mu As Double, ByVal ΔM(,) As Double, ByVal M2d As Double, ByVal M3d As Double, ByVal i1 As Integer, ByVal j As Integer,
                                             ByVal x As Double, ByVal d As Double, ByVal str() As Double, ByVal Message As Integer, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        'str(1) = µlimu moment réduit limite
        Dim msg As String
        Dim Amin As Double
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9

        If Message = 21 Then
            msg = "εs < εspL - Calcul en pivot B - Le plancher nécessite des armatures comprimées -Non calculées dans la suite du programme."
            rtf.WriteString(msg) : rtf.NewLine()
            Aff.msg_Avertiss.Add(msg)
        End If
        If Message = 22 Then
            msg = "Calcul en pivot A mais section béton surabondante: εc < 0.35%"
            rtf.WriteString(msg) : rtf.NewLine() ' & σcLim.ToString("#0.00") & "Mpa > " & (0.6 * bet.fck).ToString("#0.00") & " MPa") : rtf.NewLine()
        End If
        rtf.NewParagraph()
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.Bold = True : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.Bold = False : rtf.EndCell() : rtf.EndRow()
        rtf.BeginRow() : rtf.DeclareCell(8600) : rtf.BeginCell(True) : rtf.WriteString("Sens porteur suivant X") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        Select Case j
            Case 1
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True)
                rtf.WriteString("moment Maxi à " & x.ToString & " cm de l'appui gauche") : rtf.EndCell() : rtf.EndRow()
            Case 2
                If Aff.Ecretage_Moment = False Then
                    rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                    rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                    rtf.DeclareCell(5500) : rtf.BeginCell(True)
                    If i1 = 1 And Aff.Reprise_M0_1_porteur = True Then
                        rtf.WriteString(" = 0,15.Mt Max") : rtf.EndCell() : rtf.EndRow()
                    Else
                        rtf.WriteString("moment Maxi sur appui gauche") : rtf.EndCell() : rtf.EndRow()
                    End If
                Else
                    If i1 = 1 And Aff.Reprise_M0_1_porteur = True Then
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" = 0,15.Mt Max") : rtf.EndCell() : rtf.EndRow()
                    Else
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mus") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((M2d * 1000).ToString("#,##0.0")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True)
                        rtf.WriteString("moment Maxi sur appui gauche avant écrétage") : rtf.EndCell() : rtf.EndRow()
                        'ligne suivante
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("ΔM") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((ΔM(i1, j) * 1000).ToString("#,##0.0")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True)
                        rtf.WriteString("Valeur de l'écrêtage") : rtf.EndCell() : rtf.EndRow()
                        'ligne suivante
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mur") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True)
                        If PlanCont.Item(i1 - 1).App_Monolith = False Then
                            rtf.WriteString("Moment retenu: Mur = Mus - ΔM") : rtf.EndCell() : rtf.EndRow()
                        Else
                            rtf.WriteString("Moment retenu: Mur = Mus - ΔM ou Mu au nu appui") : rtf.EndCell() : rtf.EndRow()
                        End If
                    End If
                End If
            Case 3
                If Aff.Ecretage_Moment = False Then
                    rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                    rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                    rtf.DeclareCell(5500) : rtf.BeginCell(True)
                    If i1 = PlanCont.Count And Aff.Reprise_M0_1_porteur = True Then
                        rtf.WriteString(" = 0,15.Mt Max") : rtf.EndCell() : rtf.EndRow()
                    ElseIf PlanCont.Item(i1 - 1).App_Monolith = True Then
                        rtf.WriteString("moment Maxi au nu de l'appui droit") : rtf.EndCell() : rtf.EndRow()
                    Else
                        rtf.WriteString("moment Maxi sur appui droit") : rtf.EndCell() : rtf.EndRow()
                    End If
                Else
                    If i1 = PlanCont.Count And Aff.Reprise_M0_1_porteur = True Then
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" = 0,15.Mt Max") : rtf.EndCell() : rtf.EndRow()
                    Else
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mus") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((M3d * 1000).ToString("#,##0.0")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True)
                        rtf.WriteString("moment Maxi sur appui droit avant écrétage") : rtf.EndCell() : rtf.EndRow()
                        'ligne suivante
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("ΔM") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((ΔM(i1, j) * 1000).ToString("#,##0.0")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True)
                        rtf.WriteString("Valeur de l'écrêtage") : rtf.EndCell() : rtf.EndRow()
                        'ligne suivante
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mur") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True)
                        If PlanCont.Item(i1 - 1).App_Monolith = False Then
                            rtf.WriteString("Moment retenu: Mur = Mus - ΔM") : rtf.EndCell() : rtf.EndRow()
                        Else
                            If x = CInt(Truncate(PlanCont.Item(i1 - 1).Lcx * 100)) Then
                                rtf.WriteString("Moment retenu: Mur = Mus - ΔM")
                            Else
                                rtf.WriteString("Moment retenu: Mur = Mu au nu appui")
                            End If
                            rtf.EndCell() : rtf.EndRow()
                        End If
                    End If
                End If
            Case 4 'armature sur appuis haut et bas
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" = 0,15.Mt Maxi") : rtf.EndCell() : rtf.EndRow()
        End Select
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("d") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((d * 100).ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("hauteur utile") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("µ limite") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(1).ToString("#0.0000")) : rtf.EndCell() 'µlimu
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("moment réduit limite AB") : rtf.EndCell() : rtf.EndRow()
        'Ligne 5
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("µ") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(2).ToString("#0.0000")) : rtf.EndCell() '(µu)
        If str(2) < str(1) Then 'µu < µlimu
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" < µ limite AB - Calcul en pivot A") : rtf.EndCell() : rtf.EndRow()
        Else
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" > µ limite - Calcul en pivot B") : rtf.EndCell() : rtf.EndRow()
        End If
        'Ligne 6
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("α") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(3).ToString("#0.0000")) : rtf.EndCell() '(αu)
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Rapport x/d (axe neutre/hauteur utile)") : rtf.EndCell() : rtf.EndRow()
        'Ligne 7
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Zu") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(4) * 100).ToString("#0.0")) : rtf.EndCell() 'zu 
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("bras de levier ") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        Select Case j
            Case 1
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As inf") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature inférieure ") : rtf.EndCell() : rtf.EndRow()
                Call Pourcent_mini_acier_flexion(i1, d, Amin) ' % minimun longitudinal des aciers de flexion dans une dalle
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As min") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Amin * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section d'acier minimale") : rtf.EndCell() : rtf.EndRow()
            Case 2
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As sup G") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature supérieure sur appui gauche") : rtf.EndCell() : rtf.EndRow()
            Case 3
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As sup Dr") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature supérieure sur appui droit") : rtf.EndCell() : rtf.EndRow()
            Case 4 'Appuis haut et bas
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As sup ") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature supérieure sur appui Haut et/ou Bas") : rtf.EndCell() : rtf.EndRow()
        End Select
        'Ligne 10
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("εc") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("‰") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(6).ToString("#,##0.00")) : rtf.EndCell() '(εc)
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("déformation maximale béton") : rtf.EndCell() : rtf.EndRow()
        'Ligne 11
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("εs") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("‰") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(7).ToString("#0.00")) : rtf.EndCell() '(εs)
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("déformation maximale acier - Limite plasticité = " & str(8).ToString("#0.00") & "‰") : rtf.EndCell() : rtf.EndRow() 'εspL
        If str(2) < str(1) Then 'calcul en pivot A
            'Ligne 12
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("σs") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(10).ToString("#,##0.00")) : rtf.EndCell() 'fydC
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("modélisation avec palier incliné ( à comparer à " & str(11).ToString("#0.0") & "MPa)") : rtf.EndCell() : rtf.EndRow() 'fyd
        ElseIf str(9) = -1000 And str(2) >= str(1) Then 'µu >= µlimuAB AinfC
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As corrigé") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("S.O.") : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("εs > εud - utilisation de la branche horizontale") : rtf.EndCell() : rtf.EndRow()
        Else
            'Ligne 12
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("σs") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(10).ToString("#,##0.00")) : rtf.EndCell() 'fydC
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("modélisation avec palier incliné ( à comparer à " & str(11).ToString("#0.0") & "MPa)") : rtf.EndCell() : rtf.EndRow() 'fyd
            'Ligne 13
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As corrigé") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(9) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'AinfC
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature optimisée") : rtf.EndCell() : rtf.EndRow()
        End If
        If ((i1 = 1 And j = 2 And Aff.Reprise_M0_1_porteur = True) = True Or (i1 = PlanCont.Count And j = 3 And Aff.Reprise_M0_1_porteur = True) = True) = False Then
            If ((j = 1) Or (j = 2 Or j = 3 And Aff.HA_Trans_Appui = True)) = True Then
                'A réaliser uniquement pour les aciers reprenant les moments sur appui - les armatures reprenant 0.15M0 sont exclues de cette vérification
                'ligne
                rtf.BeginRow() : rtf.DeclareCell(8600) : rtf.BeginCell(True) : rtf.WriteString("Sens NON porteur suivant Y") : rtf.EndCell() : rtf.EndRow()
                'ligne
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As requis") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(12) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature inférieure suivant Y (=20%)") : rtf.EndCell() : rtf.EndRow()
            End If
        End If
        rtf.WriteEndGroup()
    End Sub
    Private Sub Harmonisation_Barre(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal Ainf(,,) As Double, ByVal m2d(,,) As Double, ByVal m3d(,,) As Double,
                                    ByVal courbe_moment As Integer, ByVal x0(,,) As Integer, ByVal x1min(,) As Integer, ByVal σsd As Double,
                                 ByRef MuR(,,,) As Double, ByRef acier_construction() As Boolean, ByRef Verif As Boolean)
        '   Dim d, Mu As Double
        If Ainf(ELU_ELS, i1, 2) > Ainf(ELU_ELS, i1 - 1, 3) Then
            MsgBox("Sub Harmonisation barres" & Environment.NewLine & "section acier appui gauche gauche > section acier appui gauche droit travée suivante - non implémenté")
            'With PoutCont.Item(i1 - 1)
            '    Select Case .LitSupG_nblit 'inf(i1, 2)
            '        Case 1 '1 seul lit
            '            PoutCont.Item(i1 - 2).LitSupD.Item(0).φHA = .LitSupG.Item(0).φHA : PoutCont.Item(i1 - 2).LitSupD.Item(0).nbar = .LitSupG.Item(0).nbar
            '            PoutCont.Item(i1 - 2).LitSupD.Item(1).φHA = 0 : PoutCont.Item(i1 - 2).LitSupD.Item(1).nbar = 0
            '            PoutCont.Item(i1 - 2).LitSupD.Item(2).φHA = 0 : PoutCont.Item(i1 - 2).LitSupD.Item(2).nbar = 0
            '        Case 2 '2 lits
            '            PoutCont.Item(i1 - 2).LitSupD.Item(0).φHA = .LitSupG.Item(0).φHA : PoutCont.Item(i1 - 2).LitSupD.Item(0).nbar = .LitSupG.Item(0).nbar
            '            PoutCont.Item(i1 - 2).LitSupD.Item(1).φHA = .LitSupG.Item(1).φHA : PoutCont.Item(i1 - 2).LitSupD.Item(1).nbar = .LitSupG.Item(1).nbar
            '            PoutCont.Item(i1 - 2).LitSupD.Item(2).φHA = 0 : PoutCont.Item(i1 - 2).LitSupD.Item(2).nbar = 0
            '        Case 3 ' 3 lits
            '            PoutCont.Item(i1 - 2).LitSupD.Item(0).φHA = .LitSupG.Item(0).φHA : PoutCont.Item(i1 - 2).LitSupD.Item(0).nbar = .LitSupG.Item(0).nbar
            '            PoutCont.Item(i1 - 2).LitSupD.Item(1).φHA = .LitSupG.Item(1).φHA : PoutCont.Item(i1 - 2).LitSupD.Item(1).nbar = .LitSupG.Item(1).nbar
            '            PoutCont.Item(i1 - 2).LitSupD.Item(2).φHA = .LitSupG.Item(2).φHA : PoutCont.Item(i1 - 2).LitSupD.Item(2).nbar = .LitSupG.Item(2).nbar
            '    End Select
            '    PoutCont.Item(i1 - 2).LitSupD_nblit = .LitSupG_nblit
            '    Mu = -m3d(ELU_ELS, i1 - 1, CInt(.a1 + .L + .a2 * 100))
            '    d = PoutCont.Item(i1 - 2).dSupD
            'End With

            'Call Moment_resistant_Lit_Acier(ELU_ELS, i1 - 1, 3, Mu, d, Ainf(ELU_ELS, i1, 2), MuR) 'calcul du moment repris par chaque lit d'acier
            'Call Epure_lit_acier_appui_droit(Verif, ELU_ELS, i1 - 1, courbe_moment, x0, x1min, σsd, MuR, m3d, acier_construction) ' : If Verif = False Then Exit Sub
            ' Call Entrainement_armature_haute(i1 - 1, Verif, 3) : If Verif = False Then Exit Sub 'vérification de l'entrainement des armatures hautes sur appuis pour les poutres Hyperstatiques
        Else
            MsgBox("Sub Harmonisation barres" & Environment.NewLine & "section acier appui gauche gauche < section acier appui gauche droit travée suivante - non implémenté")
            'With PoutCont.Item(i1 - 2)
            '    Select Case .LitSupD_nblit
            '        Case 1 '1 seul lit
            '            PoutCont.Item(i1 - 1).LitSupG.Item(0).φHA = .LitSupD.Item(0).φHA : PoutCont.Item(i1 - 1).LitSupG.Item(0).nbar = .LitSupD.Item(0).nbar
            '            PoutCont.Item(i1 - 1).LitSupG.Item(1).φHA = 0 : PoutCont.Item(i1 - 1).LitSupG.Item(1).nbar = 0
            '            PoutCont.Item(i1 - 1).LitSupG.Item(2).φHA = 0 : PoutCont.Item(i1 - 1).LitSupG.Item(2).nbar = 0
            '        Case 2 '2 lits
            '            PoutCont.Item(i1 - 1).LitSupG.Item(0).φHA = .LitSupD.Item(0).φHA : PoutCont.Item(i1 - 1).LitSupG.Item(0).nbar = .LitSupD.Item(0).nbar
            '            PoutCont.Item(i1 - 1).LitSupG.Item(1).φHA = .LitSupD.Item(1).φHA : PoutCont.Item(i1 - 1).LitSupG.Item(1).nbar = .LitSupD.Item(1).nbar
            '            PoutCont.Item(i1 - 1).LitSupG.Item(2).φHA = 0 : PoutCont.Item(i1 - 1).LitSupG.Item(2).nbar = 0
            '        Case 3 ' 3 lits
            '            PoutCont.Item(i1 - 1).LitSupG.Item(0).φHA = .LitSupD.Item(0).φHA : PoutCont.Item(i1 - 1).LitSupG.Item(0).nbar = .LitSupD.Item(0).nbar
            '            PoutCont.Item(i1 - 1).LitSupG.Item(1).φHA = .LitSupD.Item(1).φHA : PoutCont.Item(i1 - 1).LitSupG.Item(1).nbar = .LitSupD.Item(1).nbar
            '            PoutCont.Item(i1 - 1).LitSupG.Item(2).φHA = .LitSupD.Item(2).φHA : PoutCont.Item(i1 - 1).LitSupG.Item(2).nbar = .LitSupD.Item(2).nbar
            '    End Select
            '    PoutCont.Item(i1 - 1).LitSupG_nblit = .LitSupD_nblit
            '    d = PoutCont.Item(i1 - 1).dSupG
            'End With

            'Mu = -m2d(ELU_ELS, i1, 0) 'Calcul de l'acier pour le moment maxi sur appui gauche
            'Call Moment_resistant_Lit_Acier(ELU_ELS, i1, 2, Mu, d, Ainf(ELU_ELS, i1 - 1, 3), MuR)  'calcul du moment repris par chaque lit d'acier
        End If
    End Sub

    Private Sub Epure_lit_acier_en_travee_HA(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal Mur(,,,) As Double, ByVal M1d(,,) As Double, ByVal σsd As Double)
        '-------------------------------------------------------------------------------
        ' détermination de l'épure des lits d'acier en travée -  ELU  ELS
        '---------------------------------------------------------------------------------
        'Rappel: L'origine de X1acier(i1, j, ikk) et X1acierLS(i1, j, ikk) étant le nu d'appui et l'origine des x pour les moments 
        '       étant variables suivant le type d'appui (prise en compte de l'appui si maçonnerie et exclusion si béton), il est 
        '       retranché à  X1acier() et  X1acierLS() la valeur (kap(i1, 1) * Lap(i1 - consolG) / 2)
        Dim appui, i2, ikk, x3, x4 As Integer
        Dim Xmur, la(2) As Double
        ' enrLat = PoutCont.Item(i1 - 1).enrLat

        'Pour les autres lits
        With PlanCont.Item(i1 - 1).LitInf
            If .Count = 2 Then
                '2 lits d'aciers
                appui = 1 : ikk = 2 : i2 = Indice_Acier(.Item(1).φHA) 'n° du diamètre de l'acier du lit 2
                If .Item(1).L13acier = 0 Then 'le lit n°2 n'est pas ancré sur l'appui 1 et/ou 2
                    Call Point_intersection_moment_positif(ELU_ELS, i1, ikk, appui, Mur, M1d, Xmur) 'recherche du point d'intersection côté appui gauche
                    .Item(1).X1acier = Xmur
                    .Item(1).X1acierLS = Xmur + Truncate(Lscel_droit(1, 1, i1, ELU_ELS, σsd, False) * 100) / 100
                    Call Interception(ELU_ELS, i1, ikk, appui, Mur, M1d, x3, x4) 'vérif de non interception entre moment enveloppe et courbe enveloppe des moments résistants des aciers
                    .Item(1).X1acierLS = .Item(1).X1acierLS - (x4 / 100) - PlanCont.Item(i1 - 1).a1
                    .Item(1).X1acier = .Item(1).X1acier - (x4 / 100) - PlanCont.Item(i1 - 1).a1
                    If .Item(1).X1acierLS < 0 Then .Item(1).X1acierLS = 0 'rappel: origine X1acierLS = nu d'appui
                    If .Item(1).X1acier < 0 Then .Item(1).X1acier = 0 'rappel: origine X1acier = nu d'appui
                End If
                appui = 2
                If .Item(1).L23acier = 0 Then 'le lit n°2 n'est pas ancré sur l'appui 2
                    Call Point_intersection_moment_positif(ELU_ELS, i1, ikk, appui, Mur, M1d, Xmur) 'recherche du point d'intersection côté appui droit
                    .Item(1).X2acier = Xmur
                    .Item(1).X2acierLS = Xmur - Truncate(Lscel_droit(1, 1, i1, ELU_ELS, σsd, False) * 100) / 100
                    If .Item(1).X2acierLS < 0 Then .Item(1).X2acierLS = 0 'Pour éviter une valeur négative
                    Call Interception(ELU_ELS, i1, ikk, appui, Mur, M1d, x3, x4) 'vérif de non interception entre moment enveloppe et courbe enveloppe des moments résistants des aciers
                    .Item(1).X2acierLS = .Item(1).X2acierLS + (x4 / 100) - PlanCont.Item(i1 - 1).a1
                    .Item(1).X2acier = .Item(1).X2acier + (x4 / 100) - PlanCont.Item(i1 - 1).a1
                    If .Item(1).X2acierLS > PlanCont.Item(i1 - 1).Lx Then .Item(1).X2acierLS = PlanCont.Item(i1 - 1).Lx 'rappel: origine X2acierLS = nu d'appui
                    If .Item(1).X2acierLS > PlanCont.Item(i1 - 1).Lx Then .Item(1).X2acierLS = PlanCont.Item(i1 - 1).Lx
                End If
            End If

        End With

    End Sub
    Private Sub Epure_lit_acier_en_travee_TS(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal Mur(,,,) As Double, ByVal M1d(,,) As Double, ByVal σsd As Double)
        '-------------------------------------------------------------------------------
        ' détermination de l'épure des lits d'acier en travée -  ELU  ELS
        '---------------------------------------------------------------------------------
        'Rappel: L'origine de X1acier(i1, j, ikk) et X1acierLS(i1, j, ikk) étant le nu d'appui et l'origine des x pour les moments 
        '       étant variables suivant le type d'appui (prise en compte de l'appui si maçonnerie et exclusion si béton), il est 
        '       retranché à  X1acier() et  X1acierLS() la valeur (kap(i1, 1) * Lap(i1 - consolG) / 2)
        Dim appui, i, ikk, x3, x4 As Integer 'i2,
        Dim Xmur, la(2) As Double
        ' enrLat = PoutCont.Item(i1 - 1).enrLat

        'Pour les autres lits
        With PlanCont.Item(i1 - 1)
            If .TS_trav.Count = 2 Then
                With .TS_trav
                    '2 lits d'aciers
                    appui = 1 : ikk = 2 ': i2 = Indice_Acier(.Item(1).φHA) 'n° du diamètre de l'acier du lit 2
                    If .Item(1).L13acier = 0 Then 'le lit n°2 n'est pas ancré sur l'appui 1 et/ou 2
                        Call Point_intersection_moment_positif(ELU_ELS, i1, ikk, appui, Mur, M1d, Xmur) 'recherche du point d'intersection côté appui gauche
                        .Item(1).X1acier = Xmur
                        .Item(1).X1acierLS = Xmur + Truncate(Lscel_droit(1, 1, i1, ELU_ELS, σsd, False) * 100) / 100
                        Call Interception(ELU_ELS, i1, ikk, appui, Mur, M1d, x3, x4) 'vérif de non interception entre moment enveloppe et courbe enveloppe des moments résistants des aciers
                        .Item(1).X1acierLS = .Item(1).X1acierLS - (x4 / 100) - PlanCont.Item(i1 - 1).a1
                        .Item(1).X1acier = .Item(1).X1acier - (x4 / 100) - PlanCont.Item(i1 - 1).a1
                        If .Item(1).X1acierLS < 0 Then .Item(1).X1acierLS = 0 'rappel: origine X1acierLS = nu d'appui
                        If .Item(1).X1acier < 0 Then .Item(1).X1acier = 0 'rappel: origine X1acier = nu d'appui
                    End If
                    appui = 2
                    If .Item(1).L23acier = 0 Then 'le lit n°2 n'est pas ancré sur l'appui 2
                        Call Point_intersection_moment_positif(ELU_ELS, i1, ikk, appui, Mur, M1d, Xmur) 'recherche du point d'intersection côté appui droit
                        .Item(1).X2acier = Xmur
                        .Item(1).X2acierLS = Xmur - Truncate(Lscel_droit(1, 1, i1, ELU_ELS, σsd, False) * 100) / 100
                        If .Item(1).X2acierLS < 0 Then .Item(1).X2acierLS = 0 'Pour éviter une valeur négative
                        Call Interception(ELU_ELS, i1, ikk, appui, Mur, M1d, x3, x4) 'vérif de non interception entre moment enveloppe et courbe enveloppe des moments résistants des aciers
                        .Item(1).X2acierLS = .Item(1).X2acierLS + (x4 / 100) - PlanCont.Item(i1 - 1).a1
                        .Item(1).X2acier = .Item(1).X2acier + (x4 / 100) - PlanCont.Item(i1 - 1).a1
                        If .Item(1).X2acierLS > PlanCont.Item(i1 - 1).Lx Then .Item(1).X2acierLS = PlanCont.Item(i1 - 1).Lx 'rappel: origine X2acierLS = nu d'appui
                        If .Item(1).X2acierLS > PlanCont.Item(i1 - 1).Lx Then .Item(1).X2acierLS = PlanCont.Item(i1 - 1).Lx
                    End If
                End With
            End If


        End With

    End Sub
    ''' <summary>
    ''' détermination de l'épure du lit d'acier supérieur sur  appui haut et bas dans le cas d'un encastrement partiel
    ''' </summary>
    ''' <param name="ELU_ELS">type de combinaison appliquée: ELU fondamental (Str), Accidentel,  </param>
    ''' <param name="i1">n° de travée</param>
    ''' <param name="σsd">Taux de Contrainte sur l'armature</param>
    Private Sub Epure_lit_acier_appui_Haut_Bas(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal σsd As Double)

        Dim lappui As Double ' 
        With PlanCont.Item(i1 - 1)
            If .LappH >= 0.05 Then 'reprise 0,15.M0
                lappui = .LappH - .enrLat
                Call Ancrage_lit(0, i1, 1, lappui, 4, ELU_ELS, σsd, 36, , True) 'ancrage total  sur l'appui haut

            End If
            If .LappB >= 0.05 Then 'reprise 0,15.M0
                lappui = .LappB - .enrLat
                Call Ancrage_lit(0, i1, 1, lappui, 4, ELU_ELS, σsd, 36,, True) 'ancrage total  sur l'appui bas
            End If
        End With
    End Sub
    ''' <summary>
    ''' détermination de l'épure des lits d'acier supérieurs sur l'appui gauche  -  ELU  ELS
    ''' </summary>
    ''' <param name="Verif"> </param>
    ''' <param name="ELU_ELS">type de combinaison appliquée: ELU fondamental (Str), Accidentel,  </param>
    ''' <param name="i1">n° de travée</param>
    ''' <param name="courbe_moment">type de courbes de moment</param>
    ''' <param name="x0">abcsisse pour laquelle le moment est nul</param>
    ''' <param name="x1min">abcsisse pour laquelle  </param>
    ''' <param name="σsd"></param>
    ''' <param name="Mur">Moment ultime résistant développé par les armatures</param>
    ''' <param name="m2d">Enveloppe des Moments Maxi développé depuis l'appui gauche en intégrant le décalage de l'effort tranchant (négatif)</param>
    ''' <param name="acier_construction"></param>
    Private Sub Epure_lit_acier_appui_gauche_HA(ByRef verif As Boolean, ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal courbe_moment As Integer,
                                             ByVal x0(,,) As Integer, ByVal x1min(,) As Integer, ByVal σsd As Double, ByVal Mur(,,,) As Double,
                                            ByVal M2d(,,) As Double, ByRef acier_construction() As Boolean)
        '-------------------------------------------------------------------------------
        ' détermination de l'épure des lits d'acier supérieurs sur l'appui gauche  -  ELU  ELS
        '---------------------------------------------------------------------------------
        'Variables: i1 = n° de travée
        '           j = position acier
        '           courbe_moment = type de courbe de moment
        '           x0(,,) = abcsisse pour laquelle le moment est nul
        Dim i, i2, ikk, x4 As Integer 'x3,i2,
        Dim lappui, l1, l2, l3, Xmur As Double ' 
        Dim msg As String
        With PlanCont.Item(i1 - 1)
            If i1 = 1 And .LappG >= 0.05 And Aff.Reprise_M0_1_porteur = True Then 'reprise 0,15.M0
                lappui = .LappG - .enrLat
                Call Ancrage_lit(1, i1, 1, lappui, 2, ELU_ELS, σsd, 9, 0) 'ancrage total  sur l'appui gauche
            Else
                If i1 = PlanCont.Count And .LappD < 0.05 Then
                    'appui = 2
                    For ikk = 1 To .LitSupG.Count
                        Select Case ikk
                            Case 1
                                If .LitSupG.Item(0).X2acier > .Lx Then 'lc(i1) Then 'soulèvement de la dernière travée sur un cas de charge
                                    i2 = Indice_Acier(.LitSupG.Item(0).φHA)
                                    Call ancrage_acier_console(i1, i2, l1, l2, l3)
                                    .LitSupG.Item(0).X2acier = .Lx - .enrLat - l3
                                    .LitSupG.Item(0).X2acierLS = .Lx - .enrLat - l3
                                    msg = "Travée n°" & i1.ToString & " - Epure lit acier appui gauche: Problème de courbe de moment dans module Epure_lit_acier_appui_gauche"
                                    Aff.msg_Avertiss.Add(msg)
                                Else
                                    .LitSupG.Item(0).X1acier = 0
                                    .LitSupG.Item(0).X1acierLS = 0
                                    i2 = Indice_Acier(.LitSupG.Item(0).φHA)
                                    If Aff.Consol_Equerre = True Then
                                        Call ancrage_acier_console(i1, i2, l1, l2, l3)
                                    Else
                                        l1 = 0 : l2 = 0 : l3 = 0
                                    End If
                                    .LitSupG.Item(0).L21acier = l1
                                    .LitSupG.Item(0).L23acier = l3
                                    .LitSupG.Item(0).L22acier = l2
                                    .LitSupG.Item(0).X2acier = .Lx - .enrLat - l3
                                    .LitSupG.Item(0).X2acierLS = .Lx - .enrLat - l3 - .al
                                    If .LitSupG.Item(0).X2acierLS < 0 Then .LitSupG.Item(0).X2acierLS = 0
                                End If
                            Case 2, 3
                                Call Point_intersection_moment_negatif(ELU_ELS, i1, 2, ikk, Mur, courbe_moment, x0, x1min, M2d, Xmur)   'recherche du point d'intersection
                                i2 = Indice_Acier(.LitSupG.Item(ikk - 1).φHA)
                                .LitSupG.Item(ikk - 1).X1acier = 0
                                .LitSupG.Item(ikk - 1).X1acierLS = 0
                                .LitSupG.Item(ikk - 1).X2acier = Truncate(Xmur * 100) / 100 'x2mur                           
                                .LitSupG.Item(ikk - 1).X2acierLS = Truncate((Xmur - Lscel_droit(ikk - 1, 2, i1, ELU_ELS, σsd, True)) * 100) / 100 'x2mur
                                Call Interception_appui_Gauche(ELU_ELS, i1, ikk, Mur, M2d, 0, x4)
                                .LitSupG.Item(ikk - 1).X2acierLS = .LitSupG.Item(ikk - 1).X2acierLS + (x4 / 100)
                                .LitSupG.Item(ikk - 1).X2acier = .LitSupG.Item(ikk - 1).X2acier + (x4 / 100)
                                If .LitSupG.Item(ikk - 2).X2acierLS < .LitSupG.Item(ikk - 1).X2acier Then
                                    .LitSupG.Item(ikk - 2).X2acierLS = .LitSupG.Item(ikk - 1).X2acierLS + (x4 / 100)
                                    .LitSupG.Item(ikk - 2).X2acier = .LitSupG.Item(ikk - 1).X2acier + (x4 / 100)
                                End If
                        End Select
                    Next ikk
                Else
                    For ikk = 1 To .LitSupG.Count
                        If ikk = 1 Then
                            .LitSupG.Item(0).X1acier = 0 : .LitSupG.Item(0).X1acierLS = 0
                            Select Case courbe_moment
                                Case 1 'la courbe des moments n'intercepte pas l'axe des abscisses = négatif sur toute la travée
                                    .LitSupG.Item(0).X2acier = (Truncate(x1min(ELU_ELS, i1) + 100 * .al)) / 100
                                Case Else 'la courbe des moments  intercepte  l'axe des abscisses
                                    .LitSupG.Item(0).X2acier = (Truncate(x0(ELU_ELS, i1, 1) + 100 * .al)) / 100
                                    If .LitSupG.Item(0).X2acier > .a1 + .Lx + .a2 Then 'soulèvement de la dernière travée sur un cas de charge
                                        If i1 = PlanCont.Count Then
                                            'Call Longueur_d_appui(i1, la, i2)
                                            'Call Ancrage_total_lit(2, i1, i2, la, j, 1)
                                            'acier_construction(nbtrav) = False
                                        Else
                                            msg = "Problème de courbe de moment dans module Epure_lit_acier_appui_gauche" & Environment.NewLine &
                                                                                        "Revoir programmation - Les dimensions aciers fausses!"
                                            i = MsgBox(msg, MsgBoxStyle.Information Or MsgBoxStyle.OkCancel, "Sub Epure_lit_acier_appui_gauche")

                                            Stop
                                        End If
                                        courbe_moment = 6
                                    End If

                            End Select
                            If courbe_moment < 6 Then
                                If .LitSupG.Item(0).X2acier - Lscel_droit(0, 2, i1, 1, σsd, True) < 0 Then .LitSupG.Item(0).X2acier = Lscel_droit(0, 2, i1, 1, σsd, True) + 0.01
                                .LitSupG.Item(0).X2acierLS = Truncate((.LitSupG.Item(0).X2acier - Lscel_droit(0, 2, i1, 1, σsd, True)) * 100) / 100
                                Call Interception_appui_Gauche(ELU_ELS, i1, ikk, Mur, M2d, 0, x4)
                                .LitSupG.Item(0).X2acierLS = .LitSupG.Item(0).X2acierLS + (x4 / 100) - .a1
                                .LitSupG.Item(0).X2acier = .LitSupG.Item(0).X2acier + (x4 / 100) - .a1
                            End If
                        Else
                            Call Point_intersection_moment_negatif(ELU_ELS, i1, 2, ikk, Mur, courbe_moment, x0, x1min, M2d, Xmur) 'recherche du point d'intersection
                            .LitSupG.Item(ikk - 1).X1acier = 0
                            .LitSupG.Item(ikk - 1).X1acierLS = 0
                            .LitSupG.Item(ikk - 1).X2acier = CInt(Xmur * 100) / 100 'x2mur
                            If .LitSupG.Item(ikk - 1).X2acier - Lscel_droit(ikk - 1, 2, i1, 1, σsd, True) < 0 Then .LitSupG.Item(ikk - 1).X2acier = Lscel_droit(ikk - 1, 2, i1, 1, σsd, True) + 0.01
                            .LitSupG.Item(ikk - 1).X2acierLS = .LitSupG.Item(ikk - 1).X2acier - Lscel_droit(ikk - 1, 2, i1, 1, σsd, True)
                            .LitSupG.Item(ikk - 1).X2acierLS = CInt(.LitSupG.Item(ikk - 1).X2acierLS * 100) / 100
                            Call Interception_appui_Gauche(ELU_ELS, i1, ikk, Mur, M2d, 0, x4)
                            .LitSupG.Item(ikk - 1).X2acierLS = .LitSupG.Item(ikk - 1).X2acierLS + (x4 / 100) - .a1
                            .LitSupG.Item(ikk - 1).X2acier = .LitSupG.Item(ikk - 1).X2acier + (x4 / 100) - .a1
                            'bloc testé: pose problème - à voir avec d'autres poutres
                            'If X2acierLS(i1, j, ikk - 1) < X2acier(i1, j, ikk) Then
                            '    X2acierLS(i1, j, ikk - 1) = X2acierLS(i1, j, ikk - 1) + (x4 / 100)
                            '    X2acier(i1, j, ikk - 1) = X2acier(i1, j, ikk - 1) + (x4 / 100)
                            'End If
                            '
                        End If
                    Next ikk
                End If
            End If
        End With

    End Sub
    Private Sub Epure_lit_acier_appui_gauche_TS(ByRef verif As Boolean, ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal courbe_moment As Integer,
                                             ByVal x0(,,) As Integer, ByVal x1min(,) As Integer, ByVal σsd As Double, ByVal Mur(,,,) As Double,
                                            ByVal M2d(,,) As Double, ByRef acier_construction() As Boolean)
        '-------------------------------------------------------------------------------
        ' détermination de l'épure des lits d'acier supérieurs sur l'appui gauche  -  ELU  ELS
        '--------------------------------------------------------------------------------- ByVal j As Integer,
        'Variables: i1 = n° de travée
        '           j = position acier
        '           courbe_moment = type de courbe de moment
        '           x0(,,) = abcsisse pour laquelle le moment est nul
        Dim i, ikk, x4 As Integer 'x3,i2,i2,
        Dim lappui, Xmur As Double ' l1, l2, l3,
        Dim msg As String
        With PlanCont.Item(i1 - 1)
            If i1 = 1 And .LappG >= 0.05 And Aff.Reprise_M0_1_porteur = True Then 'reprise 0,15.M0
                lappui = .LappG - .enrLat
                Call Ancrage_lit(1, i1, 1, lappui, 2, ELU_ELS, σsd, 9, , True) 'ancrage total  sur l'appui gauche
            Else
                If i1 = PlanCont.Count And .LappD < 0.05 Then
                    'appui = 2
                    MessageBox.Show("Epure_lit_acier_appui_gauche_TS - Console droite Non implémenté")
                Else
                    For ikk = 1 To .TS_AppG.Count
                        If ikk = 1 Then
                            .TS_AppG.Item(0).X1acier = 0 : .TS_AppG.Item(0).X1acierLS = 0
                            Select Case courbe_moment
                                Case 1 'la courbe des moments n'intercepte pas l'axe des abscisses = négatif sur toute la travée
                                    .TS_AppG.Item(0).X2acier = (Truncate(x1min(ELU_ELS, i1) + 100 * .al)) / 100
                                Case Else 'la courbe des moments  intercepte  l'axe des abscisses
                                    .TS_AppG.Item(0).X2acier = (Truncate(x0(ELU_ELS, i1, 1) + 100 * .al)) / 100
                                    If .TS_AppG.Item(0).X2acier > .a1 + .Lx + .a2 Then 'soulèvement de la dernière travée sur un cas de charge
                                        If i1 = PlanCont.Count Then
                                            'Call Longueur_d_appui(i1, la, i2)
                                            'Call Ancrage_total_lit(2, i1, i2, la, j, 1)
                                            'acier_construction(nbtrav) = False
                                        Else
                                            msg = "Problème de courbe de moment dans module Epure_lit_acier_appui_gauche" & Environment.NewLine &
                                                                                        "Revoir programmation - Les dimensions aciers fausses!"
                                            i = MsgBox(msg, MsgBoxStyle.Information Or MsgBoxStyle.OkCancel, "Sub Epure_lit_acier_appui_gauche")

                                            Stop
                                        End If
                                        courbe_moment = 6
                                    End If

                            End Select
                            If courbe_moment < 6 Then
                                If .TS_AppG.Item(0).X2acier - Lscel_droit(0, 2, i1, 1, σsd, True) < 0 Then .TS_AppG.Item(0).X2acier = Lscel_droit(0, 2, i1, 1, σsd, True) + 0.01
                                .TS_AppG.Item(0).X2acierLS = Truncate((.TS_AppG.Item(0).X2acier - Lscel_droit(0, 2, i1, 1, σsd, True)) * 100) / 100
                                Call Interception_appui_Gauche(ELU_ELS, i1, ikk, Mur, M2d, 0, x4)
                                .TS_AppG.Item(0).X2acierLS = .TS_AppG.Item(0).X2acierLS + (x4 / 100) - .a1
                                .TS_AppG.Item(0).X2acier = .TS_AppG.Item(0).X2acier + (x4 / 100) - .a1
                            End If
                        Else
                            Call Point_intersection_moment_negatif(ELU_ELS, i1, 2, ikk, Mur, courbe_moment, x0, x1min, M2d, Xmur) 'recherche du point d'intersection
                            .TS_AppG.Item(ikk - 1).X1acier = 0
                            .TS_AppG.Item(ikk - 1).X1acierLS = 0
                            .TS_AppG.Item(ikk - 1).X2acier = CInt(Xmur * 100) / 100 'x2mur
                            If .TS_AppG.Item(ikk - 1).X2acier - Lscel_droit(ikk - 1, 2, i1, 1, σsd, True) < 0 Then .TS_AppG.Item(ikk - 1).X2acier = Lscel_droit(ikk - 1, 2, i1, 1, σsd, True) + 0.01
                            .TS_AppG.Item(ikk - 1).X2acierLS = .TS_AppG.Item(ikk - 1).X2acier - Lscel_droit(ikk - 1, 2, i1, 1, σsd, True)
                            .TS_AppG.Item(ikk - 1).X2acierLS = CInt(.TS_AppG.Item(ikk - 1).X2acierLS * 100) / 100
                            Call Interception_appui_Gauche(ELU_ELS, i1, ikk, Mur, M2d, 0, x4)
                            .TS_AppG.Item(ikk - 1).X2acierLS = .TS_AppG.Item(ikk - 1).X2acierLS + (x4 / 100) - .a1
                            .TS_AppG.Item(ikk - 1).X2acier = .TS_AppG.Item(ikk - 1).X2acier + (x4 / 100) - .a1
                            'bloc testé: pose problème - à voir avec d'autres poutres
                            'If X2acierLS(i1, j, ikk - 1) < X2acier(i1, j, ikk) Then
                            '    X2acierLS(i1, j, ikk - 1) = X2acierLS(i1, j, ikk - 1) + (x4 / 100)
                            '    X2acier(i1, j, ikk - 1) = X2acier(i1, j, ikk - 1) + (x4 / 100)
                            'End If                        '
                        End If
                    Next ikk
                End If
            End If
        End With

    End Sub
    Private Sub Epure_lit_acier_appui_droit_HA(ByRef verif As Boolean, ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal courbe_moment As Integer,
                                            ByVal x0(,,) As Integer, ByVal x1min(,) As Integer, ByVal σsd As Double, ByVal Mur(,,,) As Double,
                                            ByVal M3d(,,) As Double, ByRef acier_construction() As Boolean)
        '------------------------------------------------------------------------------- 
        ' détermination de l'épure des lits d'acier supérieurs sur l'appui droit  -  ELU  ELS
        '---------------------------------------------------------------------------------
        'premier passage dans ce sous-programme à l'elu, puis deuxième passage si nécessaire à l'els fissuration 
        'préjudiciable ou trés préjudiciable
        Dim i, i2, Nint, ikk, x4 As Integer 'x3,
        Dim lappui, l1, l2, l3, Xmur As Double '
        Dim msg As String
        ELU_ELS = 1 'provisoire
        With PlanCont.Item(i1 - 1)
            If i1 = PlanCont.Count And .LappD > 0.05 Then
                'Reprise de 0,15.M0
                lappui = .LappD - .enrLat
                Call Ancrage_lit(2, i1, 1, lappui, 3, ELU_ELS, σsd, 9, 0) 'ancrage total

            Else
                If i1 = 1 And .LappG < 0.05 Then
                    '1ière travée en console
                    For ikk = 1 To .LitSupD.Count
                        Select Case ikk
                            Case 1
                                If .LitSupD.Item(0).X1acier < 0 Then 'soulèvement de la première travée sur un cas de charge
                                    i2 = Indice_Acier(.LitSupD.Item(0).φHA)
                                    Call ancrage_acier_console(i1, i2, l1, l2, l3)
                                    .LitSupD.Item(0).X1acier = .enrLat + l3
                                    .LitSupD.Item(0).X1acierLS = .enrLat + l3
                                    Nint = CInt(Truncate((l2 + l1 + 0.01) * 100)) ': Nint = Nint \ 1
                                    msg = "Travée n°" & i1.ToString & " - Epure lit acier appui droit: Problème de courbe de moment dans module Epure_lit_acier_appui_droit" & Environment.NewLine &
                                            "Revoir programmation - Les dimensions aciers fausses!"
                                    Aff.msg_Avertiss.Add(msg)
                                Else
                                    .LitSupD.Item(0).X2acier = .Lx
                                    .LitSupD.Item(0).X2acierLS = .Lx
                                    i2 = Indice_Acier(.LitSupD.Item(0).φHA)
                                    If Aff.Consol_Equerre = True Then
                                        Call ancrage_acier_console(i1, i2, l1, l2, l3)
                                    Else
                                        l1 = 0 : l2 = 0 : l3 = 0
                                    End If
                                    .LitSupD.Item(0).L11acier = l1
                                    .LitSupD.Item(0).L13acier = l3
                                    .LitSupD.Item(0).L12acier = l2
                                    .LitSupD.Item(0).X1acier = .enrLat + l3
                                    .LitSupD.Item(0).X1acierLS = .enrLat + l3 + .al
                                    If .LitSupD.Item(0).X1acierLS > .Lx Then .LitSupD.Item(0).X1acierLS = .Lx
                                End If
                            Case 2, 3
                                Call Point_intersection_moment_negatif(ELU_ELS, i1, 3, ikk, Mur, courbe_moment, x0, x1min, M3d, Xmur) 'recherche du point d'intersection
                                .LitSupD.Item(ikk - 1).X2acier = .Lx
                                .LitSupD.Item(ikk - 1).X2acierLS = .Lx
                                .LitSupD.Item(ikk - 1).X1acier = Truncate(Xmur * 100) / 100
                                i2 = Indice_Acier(.LitSupD.Item(ikk - 1).φHA)
                                .LitSupD.Item(ikk - 1).X1acierLS = Xmur + Lscel_droit(ikk - 1, 3, i1, ELU_ELS, σsd, True) 'x1mur
                                Nint = CInt(.LitSupD.Item(ikk - 1).X1acierLS * 100) ': Nint = Nint \ 1
                                .LitSupD.Item(ikk - 1).X1acierLS = Nint / 100
                                Call Interception_appui_Droit(ELU_ELS, i1, ikk, Mur, M3d, 0, x4)
                                .LitSupD.Item(ikk - 1).X1acierLS = .LitSupD.Item(ikk - 1).X1acierLS - (x4 / 100)
                                .LitSupD.Item(ikk - 1).X1acier = .LitSupD.Item(ikk - 1).X1acier - (x4 / 100)
                                If .LitSupD.Item(ikk - 2).X1acierLS > .LitSupD.Item(ikk - 1).X1acier Then
                                    .LitSupD.Item(ikk - 2).X1acierLS = .LitSupD.Item(ikk - 1).X1acierLS - (x4 / 100)
                                    .LitSupD.Item(ikk - 2).X1acier = .LitSupD.Item(ikk - 1).X1acier - (x4 / 100)
                                End If
                        End Select
                    Next ikk
                Else
                    For ikk = 1 To .LitSupD.Count
                        If ikk = 1 Then
                            .LitSupD.Item(0).X2acier = .Lx
                            .LitSupD.Item(0).X2acierLS = .Lx
                            Select Case courbe_moment
                                Case 1 'pas d'interception avec l'axe des abscisses = courbe négative sur toute la portée de la poutre
                                    .LitSupD.Item(0).X1acier = CInt(Truncate((x1min(ELU_ELS, i1) - 100 * .al) * 100)) / 100
                                    If .LitSupD.Item(0).X1acier < 0 Then .LitSupD.Item(0).X1acier = 0
                                Case Else
                                    .LitSupD.Item(0).X1acier = CInt(Truncate((x0(ELU_ELS, i1, 2) - -100 * .al) * 100)) / 100
                                    If .LitSupD.Item(0).X1acier < 0 Then 'soulèvement de la première travée sur un cas de charge
                                        i2 = Indice_Acier(.LitSupD.Item(0).φHA)
                                        If i1 = 1 Then
                                            'appui = 1
                                            MsgBox("voir ligne 1391 - Eurocode")
                                            Stop
                                            'Call Longueur_d_appui(i1, la, i2)
                                            'Call Ancrage_total_lit(1, i1, i2, la, j, 1)
                                            'acier_construction(1) = False
                                        Else

                                            msg = "Problème de courbe de moment dans module Epure_lit_acier_appui_droit" & Environment.NewLine &
                                                    "Revoir programmation - Les dimensions aciers fausses!"
                                            i = MsgBox(msg, MsgBoxStyle.Information Or MsgBoxStyle.OkCancel, "Sub Epure_lit_acier_appui_droit")

                                            Stop
                                        End If
                                        courbe_moment = 6
                                    End If
                            End Select
                            If courbe_moment < 6 Then
                                If .LitSupD.Item(0).X1acier + Lscel_droit(0, 3, i1, 1, σsd, True) > .Lx Then .LitSupD.Item(0).X1acier = .Lx - Lscel_droit(0, 3, i1, 1, σsd, True) - 0.01
                                Nint = CInt(Truncate((.LitSupD.Item(0).X1acier + Lscel_droit(0, 3, i1, 1, σsd, True)) * 100))
                                .LitSupD.Item(0).X1acierLS = Nint / 100
                                Call Interception_appui_Droit(1, i1, ikk, Mur, M3d, 0, x4)
                                .LitSupD.Item(0).X1acierLS = .LitSupD.Item(0).X1acierLS - (x4 / 100) - .a1
                                .LitSupD.Item(0).X1acier = .LitSupD.Item(0).X1acier - (x4 / 100) - .a1
                            End If
                        Else
                            Call Point_intersection_moment_negatif(ELU_ELS, i1, 3, ikk, Mur, courbe_moment, x0, x1min, M3d, Xmur) 'recherche du point d'intersection
                            .LitSupD.Item(ikk - 1).X2acier = .Lx
                            .LitSupD.Item(ikk - 1).X2acierLS = .Lx
                            .LitSupD.Item(ikk - 1).X1acier = Xmur 'x1mur
                            If Xmur + Lscel_droit(ikk - 1, 3, i1, 1, σsd, True) > .Lx Then .LitSupD.Item(ikk - 1).X1acier = .Lx - Lscel_droit(ikk - 1, 3, i1, 1, σsd, True) - 0.01
                            .LitSupD.Item(ikk - 1).X1acierLS = .LitSupD.Item(ikk - 1).X1acier + Lscel_droit(ikk - 1, 3, i1, 1, σsd, True)
                            Nint = CInt(.LitSupD.Item(ikk - 1).X1acierLS * 100) ' : Nint = Nint \ 1
                            .LitSupD.Item(ikk - 1).X1acierLS = Nint / 100
                            Call Interception_appui_Droit(ELU_ELS, i1, ikk, Mur, M3d, 0, x4)
                            .LitSupD.Item(ikk - 1).X1acierLS = .LitSupD.Item(ikk - 1).X1acierLS - (x4 / 100) - .a1
                            .LitSupD.Item(ikk - 1).X1acier = .LitSupD.Item(ikk - 1).X1acier - (x4 / 100) - .a1
                            'bloc testé: pose problème - à voir avec d'autres poutres
                            'If X1acierLS(i1, j, ikk - 1) > X1acier(i1, j, ikk) Then
                            '    X1acierLS(i1, j, ikk - 1) = X1acierLS(i1, j, ikk - 1) - (x4 / 100)
                            '    X1acier(i1, j, ikk - 1) = X1acier(i1, j, ikk - 1) - (x4 / 100)
                            'End If
                        End If
                    Next ikk

                End If
            End If
        End With

    End Sub
    Private Sub Epure_lit_acier_appui_droit_TS(ByRef verif As Boolean, ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal courbe_moment As Integer,
                                            ByVal x0(,,) As Integer, ByVal x1min(,) As Integer, ByVal σsd As Double, ByVal Mur(,,,) As Double,
                                            ByVal M3d(,,) As Double, ByRef acier_construction() As Boolean)
        '------------------------------------------------------------------------------- 
        ' détermination de l'épure des lits d'acier supérieurs sur l'appui droit  -  ELU  ELS
        '---------------------------------------------------------------------------------
        'premier passage dans ce sous-programme à l'elu, puis deuxième passage si nécessaire à l'els fissuration 
        'préjudiciable ou trés préjudiciable
        Dim i, Nint, ikk, x4 As Integer 'x3,i2,
        Dim lappui, l1, l2, l3, Xmur As Double '
        Dim msg As String
        ELU_ELS = 1 'provisoire
        With PlanCont.Item(i1 - 1)
            '--> armature sur appui d'about droit
            If i1 = PlanCont.Count And .LappD > 0.05 And Aff.Reprise_M0_1_porteur = True Then
                'Reprise de 0,15.M0
                lappui = .LappD - .enrLat
                Call Ancrage_lit(2, i1, 1, lappui, 3, ELU_ELS, σsd, 9, , True) 'ancrage total
                '
                '--> armature sur appui droit si la 1ière travée est une console
            ElseIf i1 = 1 And .LappG < 0.05 Then
                '1ière travée en console
                For ikk = 1 To .TS_AppD.Count
                    Select Case ikk
                        Case 1
                            If .TS_AppD.Item(0).X1acier < 0 Then 'soulèvement de la première travée sur un cas de charge
                                'i2 = Indice_Acier(.TS_AppD.Item(0).φHA)
                                'Call ancrage_acier_console(i1, aci.Ølong, l1, l2, l3)
                                '.TS_AppD.Item(0).X1acier = .enrLat + l3
                                '.TS_AppD.Item(0).X1acierLS = .enrLat + l3
                                'Nint = CInt(Truncate((l2 + l1 + 0.01) * 100)) ': Nint = Nint \ 1
                                msg = "Travée n°" & i1.ToString & " soulèvement de la travée sur un cas de charge - Epure lit acier appui droit: Problème de courbe de moment dans module Epure_lit_acier_appui_droit" & Environment.NewLine &
                                        "Revoir programmation - Les dimensions aciers fausses!"
                                Aff.msg_Avertiss.Add(msg)
                            Else
                                .TS_AppD.Item(0).X2acier = .Lx
                                .TS_AppD.Item(0).X2acierLS = .Lx
                                ' i2 = Indice_Acier(.TS_AppD.Item(0).φHA)
                                If Aff.Consol_Equerre = True Then
                                    '   Call ancrage_acier_console(i1, i2, l1, l2, l3)
                                Else
                                    l1 = 0 : l2 = 0 : l3 = 0
                                End If
                                .TS_AppD.Item(0).L11acier = l1
                                .TS_AppD.Item(0).L13acier = l3
                                .TS_AppD.Item(0).L12acier = l2
                                .TS_AppD.Item(0).X1acier = .enrLat + l3
                                .TS_AppD.Item(0).X1acierLS = .enrLat + l3 + .al
                                If .TS_AppD.Item(0).X1acierLS > .Lx Then .TS_AppD.Item(0).X1acierLS = .Lx
                            End If
                        Case 2, 3
                            Call Point_intersection_moment_negatif(ELU_ELS, i1, 3, ikk, Mur, courbe_moment, x0, x1min, M3d, Xmur) 'recherche du point d'intersection
                            .TS_AppD.Item(ikk - 1).X2acier = .Lx
                            .TS_AppD.Item(ikk - 1).X2acierLS = .Lx
                            .TS_AppD.Item(ikk - 1).X1acier = Truncate(Xmur * 100) / 100
                            '  i2 = Indice_Acier(.TS_AppD.Item(ikk - 1).φHA)
                            .TS_AppD.Item(ikk - 1).X1acierLS = Xmur + Lscel_droit(ikk - 1, 3, i1, ELU_ELS, σsd, True) 'x1mur
                            Nint = CInt(.TS_AppD.Item(ikk - 1).X1acierLS * 100) ': Nint = Nint \ 1
                            .TS_AppD.Item(ikk - 1).X1acierLS = Nint / 100
                            Call Interception_appui_Droit(ELU_ELS, i1, ikk, Mur, M3d, 0, x4)
                            .TS_AppD.Item(ikk - 1).X1acierLS = .TS_AppD.Item(ikk - 1).X1acierLS - (x4 / 100)
                            .TS_AppD.Item(ikk - 1).X1acier = .TS_AppD.Item(ikk - 1).X1acier - (x4 / 100)
                            If .TS_AppD.Item(ikk - 2).X1acierLS > .TS_AppD.Item(ikk - 1).X1acier Then
                                .TS_AppD.Item(ikk - 2).X1acierLS = .TS_AppD.Item(ikk - 1).X1acierLS - (x4 / 100)
                                .TS_AppD.Item(ikk - 2).X1acier = .TS_AppD.Item(ikk - 1).X1acier - (x4 / 100)
                            End If
                    End Select
                Next ikk
                '
                '--> armature sur appui droit - Cas général
            Else
                For ikk = 1 To .TS_AppD.Count
                    If ikk = 1 Then
                        .TS_AppD.Item(0).X2acier = .Lx
                        .TS_AppD.Item(0).X2acierLS = .Lx
                        Select Case courbe_moment
                            Case 1 'pas d'interception avec l'axe des abscisses = courbe négative sur toute la portée de la poutre
                                .TS_AppD.Item(0).X1acier = CInt(Truncate((x1min(ELU_ELS, i1) - 100 * .al))) / 100
                                If .TS_AppD.Item(0).X1acier < 0 Then .TS_AppD.Item(0).X1acier = 0
                            Case Else
                                .TS_AppD.Item(0).X1acier = CInt(Truncate((x0(ELU_ELS, i1, 2) - -100 * .al))) / 100
                                If .TS_AppD.Item(0).X1acier < 0 Then 'soulèvement de la première travée sur un cas de charge
                                    'i2 = Indice_Acier(.TS_AppD.Item(0).φHA)
                                    If i1 = 1 Then
                                        'appui = 1
                                        MsgBox("voir ligne 1391 - Eurocode")
                                        Stop
                                        'Call Longueur_d_appui(i1, la, i2)
                                        'Call Ancrage_total_lit(1, i1, i2, la, j, 1)
                                        'acier_construction(1) = False
                                    Else

                                        msg = "Problème de courbe de moment dans module Epure_lit_acier_appui_droit" & Environment.NewLine &
                                                "Revoir programmation - Les dimensions aciers fausses!"
                                        i = MsgBox(msg, MsgBoxStyle.Information Or MsgBoxStyle.OkCancel, "Sub Epure_lit_acier_appui_droit")

                                        Stop
                                    End If
                                    courbe_moment = 6
                                End If
                        End Select
                        If courbe_moment < 6 Then
                            If .TS_AppD.Item(0).X1acier + Lscel_droit(0, 3, i1, 1, σsd, True) > .Lx Then .TS_AppD.Item(0).X1acier = .Lx - Lscel_droit(0, 3, i1, 1, σsd, True) - 0.01
                            Nint = CInt(Truncate((.TS_AppD.Item(0).X1acier + Lscel_droit(0, 3, i1, 1, σsd, True)) * 100))
                            .TS_AppD.Item(0).X1acierLS = Nint / 100
                            Call Interception_appui_Droit(1, i1, ikk, Mur, M3d, 0, x4)
                            .TS_AppD.Item(0).X1acierLS = .TS_AppD.Item(0).X1acierLS - (x4 / 100) - .a1
                            .TS_AppD.Item(0).X1acier = .TS_AppD.Item(0).X1acier - (x4 / 100) - .a1
                        End If
                    Else
                        Call Point_intersection_moment_negatif(ELU_ELS, i1, 3, ikk, Mur, courbe_moment, x0, x1min, M3d, Xmur) 'recherche du point d'intersection
                        .TS_AppD.Item(ikk - 1).X2acier = .Lx
                        .TS_AppD.Item(ikk - 1).X2acierLS = .Lx
                        .TS_AppD.Item(ikk - 1).X1acier = Xmur 'x1mur
                        If Xmur + Lscel_droit(ikk - 1, 3, i1, 1, σsd, True) > .Lx Then .TS_AppD.Item(ikk - 1).X1acier = .Lx - Lscel_droit(ikk - 1, 3, i1, 1, σsd, True) - 0.01
                        .TS_AppD.Item(ikk - 1).X1acierLS = .TS_AppD.Item(ikk - 1).X1acier + Lscel_droit(ikk - 1, 3, i1, 1, σsd, True)
                        Nint = CInt(.TS_AppD.Item(ikk - 1).X1acierLS * 100) ' : Nint = Nint \ 1
                        .TS_AppD.Item(ikk - 1).X1acierLS = Nint / 100
                        Call Interception_appui_Droit(ELU_ELS, i1, ikk, Mur, M3d, 0, x4)
                        .TS_AppD.Item(ikk - 1).X1acierLS = .TS_AppD.Item(ikk - 1).X1acierLS - (x4 / 100) - .a1
                        .TS_AppD.Item(ikk - 1).X1acier = .TS_AppD.Item(ikk - 1).X1acier - (x4 / 100) - .a1
                        'bloc testé: pose problème - à voir avec d'autres poutres
                        'If X1acierLS(i1, j, ikk - 1) > X1acier(i1, j, ikk) Then
                        '    X1acierLS(i1, j, ikk - 1) = X1acierLS(i1, j, ikk - 1) - (x4 / 100)
                        '    X1acier(i1, j, ikk - 1) = X1acier(i1, j, ikk - 1) - (x4 / 100)
                        'End If
                    End If
                Next ikk
            End If
        End With

    End Sub
    ''' <summary>
    ''' calcul des aciers de flexion (détermination des armatures et épure) - 
    ''' calcul et vérification des efforts sur appui (bielle, acier de glissement et entrainement)
    ''' </summary>
    ''' <param name="Mur">Moment Ultime résistant avec les indices (ELU_ELS, i1, j, 1) ELU_ELs = n° de combinaison, i1 = n° de travée, j=1,2 ou 3 entravée, sur appui gauche, sur appui droit, 
    ''' 1 = n° de lit, de la surface vers l'intérieur de la dalle</param>
    '''  <param name="Mmax">Moment enveloppe Maxi de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    '''  <param name="Mmin">Moment enveloppe mini de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="IMmin">indice de la combinaison conduisant à la valeur maximale du moment sur appui droit de la travée i (en valeur absolue) avec les indices (ELU_ELS, i2): ELU_ELS = type d'etat Limite, i2 = n° de travée</param>
    ''' <param name="M"> Valeur du moment M(imax, PlanCont.Count, lc) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    ''' <param name="V">Valeur de l'effort tranchant V(imax, PlanCont.Count, lcMax) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    '''  <param name="rtf">variable renvoyant le fichier de note de calcul sous format rtf</param>
    ''' <param name="verif"> variable prenant la valeur False si une erreur bloquante s'est produite pendant le calcul</param>
    ''' <param name="Rymin"></param>
    ''' <param name="Ainf">Section théorique d'accier pour reprendre l'effort - indice (ELU_ELS, i1, j) avec limites =(3, PlanCont.Count, 4): ELU_ELS=1 à 3(ELU Str, ELU Eq, ELS car); i1= n° travée; j=position armature = 1=position centrale, 2=appui gauche, 3=appui droit, 4=aciers supérieurs sur appuis haut et bas  </param>
    ''' <param name="Modification_Acier_HA"></param>
    ''' <param name="acier_construction"></param>
    ''' <param name="m1d">Moment enveloppe Maxi positif en travée avec décalage effort tranchant. Indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="m2d">Moment enveloppe mini négatif sur appui gauche avec décalage effort tranchant. Indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="m3d">Moment enveloppe mini négatif sur appui droit avec décalage effort tranchant. Indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="x1Max"></param>
    ''' <param name="ΔM"></param>
    Private Sub Calcul_section_ELU_Fondamental(ByVal Mmax(,,) As Double, ByVal Mmin(,,) As Double, ByVal M(,,) As Double, ByVal V(,,) As Double, ByVal Rymin() As Double, ByVal IMmin(,) As Integer, ByVal Ainf(,,) As Double,
                                               ByRef Modification_Acier_HA As Boolean, ByRef verif As Boolean, ByRef acier_construction() As Boolean, ByRef m1d(,,) As Double, ByRef m2d(,,) As Double, ByRef m3d(,,) As Double,
                                               ByRef Mur(,,,) As Double, ByRef x1Max(,) As Integer, ByRef ΔM(,) As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        '----->ELU Fondamental:
        '-----> calcul des aciers de flexion (détermination des armatures et épure),
        '-----> calcul et vérification des efforts sur appui (bielle, acier de glissement et entrainement)
        'variables: i1 = n° de travée
        '           courbe_moment = type de courbe du moment enveloppe
        '           x0(ELU_ELS, nbtrav, 2) = point d'intersection entre la courbe enveloppe et l'axe des abscisses
        '                                    indique la valeur de l'abscisse pour un moment nul
        Dim Amin, Asteel, d, dd, Mu, M2dTab, M3dTab As Double '
        Dim Mt_Courbe(PlanCont.Count), i, i1, j, x, x0(1, PlanCont.Count, 2), ELU_ELS, Message As Integer 'j1,
        Dim x1min(1, PlanCont.Count) As Integer
        Dim AinfC(PlanCont.Count, 4), al(PlanCont.Count), str(12), σsd(PlanCont.Count, 4) As Double
        Dim msg As String
        ''Dim ferraillage_sup(nbtrav) As Integer
        Dim Fer As New Ferraillage_HA
        ELU_ELS = 1 ' = ELU STR
        For i1 = 1 To PlanCont.Count
            Call Valeur_Decalage_Moment(i1)
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.Underline = True : rtf.WriteString("- Travée n°" & i1.ToString) : rtf.Underline = False : rtf.NewLine() : rtf.NewLine()
            With PlanCont.Item(i1 - 1)
                If Aff.Redistrib_limit = True Then
                    Select Case i1
                        Case 1
                            If .LappG > 0.05 Then
                                rtf.WriteString(" valeur des coefficients limites de redistribution - Appui droit:") : rtf.NewLine()
                                If Aff.δ >= .δ1 Then
                                    rtf.WriteString("  - δlim eq 5.10: δ ≥ δ1 = " & .δ1.ToString("#0.000") & "   -> OK") : rtf.NewLine()
                                Else
                                    rtf.WriteString("  - δlim eq 5.10: δ < δ1 = " & .δ1.ToString("#0.000") & "  -> NS") : rtf.NewLine()
                                End If
                                If Aff.δ >= .δ2 Then
                                    rtf.WriteString("  - δlim pour la classe acier correspondante: δ ≥ δ2 = " & .δ2.ToString("#0.000") & "  -> OK") : rtf.NewLine()
                                Else
                                    rtf.WriteString("  - δlim pour la classe acier correspondante: δ < δ2 = " & .δ2.ToString("#0.000") & "  -> NS") : rtf.NewLine()
                                End If

                            End If
                        Case PlanCont.Count
                            If .LappD > 0.05 Then
                                rtf.WriteString(" valeur des coefficients de redistribution limites - Appui gauche:") : rtf.NewLine()
                                If Aff.δ >= PlanCont.Item(i1 - 2).δ1 Then
                                    rtf.WriteString("  - δlim eq 5.10: δ ≥ δ1 = " & PlanCont.Item(i1 - 2).δ1.ToString("#0.000") & "  -> OK") : rtf.NewLine()
                                Else
                                    rtf.WriteString("  - δlim eq 5.10: δ < δ1 = " & PlanCont.Item(i1 - 2).δ1.ToString("#0.000") & " -> NS") : rtf.NewLine()
                                End If
                                If Aff.δ >= PlanCont.Item(i1 - 2).δ2 Then
                                    rtf.WriteString("  - δlim pour la classe acier correspondante: δ ≥ δ2 = " & PlanCont.Item(i1 - 2).δ2.ToString("#0.000") & "  -> OK") : rtf.NewLine()
                                Else
                                    rtf.WriteString("  - δlim pour la classe acier correspondante: δ < δ2 = " & PlanCont.Item(i1 - 2).δ2.ToString("#0.000") & "  -> NS") : rtf.NewLine()
                                End If
                            End If
                        Case Else
                            rtf.WriteString(" valeur des coefficients de redistribution limites - Appui gauche:") : rtf.NewLine()
                            If Aff.δ >= PlanCont.Item(i1 - 2).δ1 Then
                                rtf.WriteString("  - δlim eq 5.10: δ ≥ δ1 = " & PlanCont.Item(i1 - 2).δ1.ToString("#0.000") & "  -> OK") : rtf.NewLine()
                            Else
                                rtf.WriteString("  - δlim eq 5.10: δ < δ1 = " & PlanCont.Item(i1 - 2).δ1.ToString("#0.000") & "  -> NS") : rtf.NewLine()
                            End If
                            If Aff.δ >= PlanCont.Item(i1 - 2).δ2 Then
                                rtf.WriteString("  - δlim pour la classe acier correspondante: δ ≥ δ2 = " & PlanCont.Item(i1 - 2).δ2.ToString("#0.000") & "   -> OK") : rtf.NewLine()
                            Else
                                rtf.WriteString("  - δlim pour la classe acier correspondante: δ < δ2 = " & PlanCont.Item(i1 - 2).δ2.ToString("#0.000") & "   -> NS") : rtf.NewLine()
                            End If
                            rtf.WriteString(" valeur des coefficients de redistribution limites - Appui droit:") : rtf.NewLine()
                            If Aff.δ >= .δ1 Then
                                rtf.WriteString("  - δlim eq 5.10: δ ≥ δ1 = " & PlanCont.Item(i1 - 1).δ1.ToString("#0.000") & "  -> OK") : rtf.NewLine()
                            Else
                                rtf.WriteString("  - δlim eq 5.10: δ < δ1 = " & PlanCont.Item(i1 - 1).δ1.ToString("#0.000") & "  -> NS") : rtf.NewLine()
                            End If
                            If Aff.δ >= .δ2 Then
                                rtf.WriteString("  - δlim pour la classe acier correspondante: δ ≥ δ2 = " & PlanCont.Item(i1 - 1).δ2.ToString("#0.000") & "   -> OK") : rtf.NewLine()
                            Else
                                rtf.WriteString("  - δlim pour la classe acier correspondante: δ < δ2 = " & PlanCont.Item(i1 - 1).δ2.ToString("#0.000") & "   -> NS") : rtf.NewLine()
                            End If
                    End Select
                End If
                'affichage du monolithisme de l'appui
                If .App_Monolith = True Then
                    rtf.WriteString("La dalle forme un ensemble monolithique avec les appuis béton.") : rtf.NewLine()
                End If
                rtf.WriteEndGroup()
                Call Decalage_moment(ELU_ELS, Mmax, Mmin, i1, Mt_Courbe(i1), x0, x1Max, x1min, m1d, m2d, m3d) ', ferraillage_sup)
                For j = 1 To 4
                    If i1 = 1 And .LappG <= 0.05 Then j = 3 'la poutre continue sur sa 1ière travée comporte une console gauche
                    If i1 = PlanCont.Count And .LappD <= 0.05 Then j = 2 'la poutre continue sur sa dernière travée comporte une console droite
                    Select Case j
                        Case 1 'acier inférieur longitudinal en travée
                            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                            rtf.Bold = True : rtf.WriteString("  ■ Armatures inférieures longitudinales en travée: ") : rtf.Bold = False : rtf.NewLine()
                            'x1Max(ELU_ELS, i1) = Position du moment maxi en travée
                            Mu = Mmax(ELU_ELS, i1, x1Max(ELU_ELS, i1))    'Calcul de l'acier pour le moment maxi en travée
                            If Mu = 0 Then 'And frmDon.MessagesInformationToolStripMenuItem.Checked = False
                                msg = "Travée n°" & i1.ToString("##") & Environment.NewLine & "Pas de moment positif en travée. La dalle" & Environment.NewLine &
                                "est soulevée sur toute sa longueur."
                                MessageBox.Show(msg, "EC2: Sub Calcul_section_ELU_Fondamental", MessageBoxButtons.OK, MessageBoxIcon.Information)
                                rtf.WriteString("Pas de moment positif en travée. la poutre est soulevée sur toute sa longueur.") : rtf.NewLine()
                                rtf.WriteEndGroup() : verif = False : Exit Sub
                            Else
                                x = CInt(Truncate(x1Max(ELU_ELS, i1) - .a1 * 100))
                            End If
                        Case 2 ' acier supérieur longitudinal sur appui gauche
                            If i1 = 1 And .LappG < 0.05 Then Continue For
                            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                            rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures longitudinales sur appui gauche:") : rtf.Bold = False : rtf.NewLine()
                            Call Determination_Mu_Appui_Gauche(1, i1, IMmin, Mmax, Mmin, V, x1Max, Mu, ΔM, x, M2dTab)
                            If i1 = 1 And Aff.Reprise_M0_1_porteur = True Then rtf.WriteString("Reprise moment forfaitaire = 0,15.Mt Max Travée") : rtf.NewLine() 'traitement des appuis d'about
                        Case 3 ' acier supérieur longitudinal sur appui droit
                            If i1 = PlanCont.Count And .LappD < 0.05 Then Exit For
                            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                            rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures longitudinales sur appui droit:") : rtf.Bold = False : rtf.NewLine()
                            Call Determination_Mu_Appui_Droit(1, i1, IMmin, Mmax, Mmin, V, x1Max, Mu, ΔM, x, M3dTab)
                            If i1 = PlanCont.Count And Aff.Reprise_M0_1_porteur = True Then rtf.WriteString("Reprise moment forfaitaire = 0,15.M0") : rtf.NewLine() 'traitement des appuis d'about
                        Case 4 'aciers supérieurs sur appuis haut et bas
                            Mu = 0
                            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                            If Aff.EncastPartiel_1_porteur = True Then
                                If .LappH < 0.05 And .LappB < 0.05 Then
                                    rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui haut et appui bas:") : rtf.Bold = False : rtf.NewLine()
                                    rtf.WriteString("Bords libres pour les 2 appuis - Armatures suivant disposition §9.3.1.4") : rtf.NewLine()
                                ElseIf .LappH > 0.05 And .LappB > 0.05 Then
                                    rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui haut et appui bas:") : rtf.Bold = False : rtf.NewLine()
                                    rtf.WriteString("Moment développé identique pour les 2 appuis.") : rtf.NewLine()
                                    Mu = 0.15 * Mmax(ELU_ELS, i1, x1Max(ELU_ELS, i1))
                                ElseIf .LappB < 0.05 And .LappH > 0.05 Then
                                    rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui bas:") : rtf.Italic = False : rtf.NewLine()
                                    rtf.WriteString("Bord libre - Armatures suivant disposition §9.3.1.4") : rtf.NewLine() : rtf.NewLine()
                                    rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui haut:") : rtf.Bold = False : rtf.NewLine()
                                    Mu = 0.15 * Mmax(ELU_ELS, i1, x1Max(ELU_ELS, i1))
                                ElseIf .LappB > 0.05 And .LappB < 0.05 Then
                                    rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui haut:") : rtf.Italic = False : rtf.NewLine()
                                    rtf.WriteString("Bord libre - Armatures suivant disposition §9.3.1.4") : rtf.NewLine() : rtf.NewLine()
                                    rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui bas:") : rtf.Bold = False : rtf.NewLine()
                                    Mu = 0.15 * Mmax(ELU_ELS, i1, x1Max(ELU_ELS, i1))
                                End If
                            Else
                                rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui haut et appui bas:") : rtf.Bold = False : rtf.NewLine()
                                rtf.WriteString("Sans objet, aucun moment n'est développé.") : rtf.NewLine()
                            End If

                    End Select
                    rtf.WriteEndGroup()
                    If Mu > 0.00001 Then
                        If j = 1 Then
                            d = .H - .enrInf : i = 1 '  d=d(i1, j) 'les aciers porteurs sont au plus prés de la face
                        Else
                            d = .H - .enrSup : i = 1 '  d=d(i1, j) 
                        End If
                        Do
                            msg = String.Empty
                            dd = d
                            Call Calcul_ELU_armature_flexion_poutre_rectangulaire(ELU_ELS, i1, j, Mu, d, Ainf, AinfC(i1, j), σsd(i1, j), str, Message, verif)
                            If verif = False Then Exit Sub

                            Dim Acier_impose_par_projeteur As Boolean = False  'Provisoire
                            If Acier_impose_par_projeteur = False Or j = 4 Then
                                'aciers calculés par le logiciel = phiProj(i1, j, 1)
                                'prise en compte ou pas de la section d'acier corrigé si prise en compte de la branche incliné
                                If AinfC(i1, j) = -1000 Then Asteel = Ainf(ELU_ELS, i1, j) Else Asteel = AinfC(i1, j)
                                'Prise en compte de la section d'acier minimale vis à vis du risque de rupture fragile
                                If (j = 4) = False And ((i1 = 1 And j = 2 And Aff.Reprise_M0_1_porteur = True) = True Or (i1 = PlanCont.Count And j = 3 And Aff.Reprise_M0_1_porteur = True) = True) = False Then
                                    'A réaliser uniquement pour les aciers reprenant les moments sur appuis intermédiaires - les armatures reprenant 0.15M0 sont exclues de cette vérification
                                    Call Pourcent_mini_acier_flexion(i1, d, Amin) ' % minimun longitudinal des aciers de flexion dans une dalle
                                    If Asteel < Amin Then
                                        Select Case j
                                            Case 1
                                                msg = "Travée " & i1.ToString & " - armatures en travée: "
                                            Case 2
                                                msg = "Travée " & i1.ToString & " - armatures sur appui gauche: "
                                            Case 3
                                                msg = "Travée " & i1.ToString & " - armatures sur appui droit: "
                                        End Select
                                        msg = msg & "la section d'acier calculée  (" & (Asteel * 10000).ToString("#0.00") & " cm²)  est inférieure au pourcentage minimal " &
                                    "(" & (Amin * 10000).ToString("#0.00") & " cm²)."
                                        Asteel = Amin
                                    End If
                                End If
                                Select Case j
                                    Case 1
                                        Select Case .Type_Acier_Travée
                                            Case 1 'TS
                                                Call Aciers_longitudinaux_par_lit_TS(ELU_ELS, i1, j, Asteel, d, str, verif)
                                            Case 2 'quadrillage barres HA
                                                Call Aciers_longitudinaux_par_lit_HA(ELU_ELS, i1, j, Asteel, d, str, verif)
                                        End Select
                                        If verif = False Then Exit Sub
                                    Case 2
                                        Select Case .Type_Acier_Appui_Gauche
                                            Case 1 'TS
                                                Call Aciers_longitudinaux_par_lit_TS(ELU_ELS, i1, j, Asteel, d, str, verif)
                                            Case 2 'quadrillage barres HA
                                                Call Aciers_longitudinaux_par_lit_HA(ELU_ELS, i1, j, Asteel, d, str, verif)
                                        End Select
                                        If verif = False Then Exit Sub
                                    Case 3
                                        Select Case .Type_Acier_Appui_Droit
                                            Case 1 'TS
                                                Call Aciers_longitudinaux_par_lit_TS(ELU_ELS, i1, j, Asteel, d, str, verif)
                                            Case 2 'quadrillage barres HA
                                                Call Aciers_longitudinaux_par_lit_HA(ELU_ELS, i1, j, Asteel, d, str, verif)
                                        End Select
                                        If verif = False Then Exit Sub
                                    Case 4 'pour les armatures sur les appuis non porteurs
                                        Select Case .Type_Acier_Appui_Droit
                                            Case 1 'TS
                                                Call Aciers_longitudinaux_par_lit_TS(ELU_ELS, i1, j, Asteel, d, str, verif)
                                            Case 2 'quadrillage barres HA
                                                Call Aciers_longitudinaux_par_lit_HA(ELU_ELS, i1, j, Asteel, d, str, verif)
                                        End Select
                                        If verif = False Then Exit Sub
                                End Select

                            Else
                                ''aciers imposés par le projeteur 
                                'If (j = 1 And .LitInf.Item(1).phiProj = 0) Or (j = 2 And .LitSupG.Item(1).phiProj = 0) Or (j = 3 And .LitSupD.Item(1).phiProj = 0) Then
                                '    '1 seul lit d'acier
                                '    'vérification pour ce cas que la section d'acier soit suffisante sinon risque plantage logiciel
                                '    Select Case j
                                '        Case 1
                                '            If Ainf(1, i1, j) > nbarMax * PI * .LitInf.Item(0).phiProj ^ 2 / 4 Then
                                '                msg = "Travée " & i1.ToString & ": section d'acier est insuffisante - " &
                                '                    ((nbarMax * PI * .LitInf.Item(0).phiProj ^ 2 / 4) * 1000).ToString("#0.00") & " cm² pour " &
                                '                    (Ainf(1, i1, j) * 10000).ToString("#0.00") & " cm² requis" &  Environment.NewLine &
                                '                    "Modifiez le nombre maximum de barres par lit  ou le diamètre de l'acier ou le nombre de lits."
                                '                MessageBox.Show(msg, "EC2: calcul section ELU fondamental", MessageBoxButtons.OK, MessageBoxIcon.Information)
                                '                verif = False
                                '            End If
                                '        Case 2
                                '            If Ainf(1, i1, j) > nbarMax * PI * .LitSupG.Item(0).phiProj ^ 2 / 4 Then
                                '                msg = "Travée " & i1.ToString & ": section d'acier est insuffisante - " &
                                '                    ((nbarMax * PI * .LitSupG.Item(0).phiProj ^ 2 / 4) * 1000).ToString("#0.00") & " cm² pour " &
                                '                    (Ainf(1, i1, j) * 10000).ToString("#0.00") & " cm² requis" & vbCrLf &
                                '                    "Modifiez le nombre maximum de barres par lit  ou le diamètre de l'acier ou le nombre de lits."
                                '                MessageBox.Show(msg, "EC2: calcul section ELU fondamental", MessageBoxButtons.OK, MessageBoxIcon.Information)
                                '                verif = False
                                '            End If
                                '        Case 3
                                '            If Ainf(1, i1, j) > nbarMax * PI * .LitSupD.Item(0).phiProj ^ 2 / 4 Then
                                '                msg = "Travée " & i1.ToString & ": section d'acier est insuffisante - " &
                                '                    ((nbarMax * PI * .LitSupD.Item(0).phiProj ^ 2 / 4) * 1000).ToString("#0.00") & " cm² pour " &
                                '                    (Ainf(1, i1, j) * 10000).ToString("#0.00") & " cm² requis" & vbCrLf &
                                '                    "Modifiez le nombre maximum de barres par lit  ou le diamètre de l'acier ou le nombre de lits."
                                '                MessageBox.Show(msg, "EC2: calcul section ELU fondamental", MessageBoxButtons.OK, MessageBoxIcon.Information)
                                '                verif = False
                                '            End If
                                '    End Select
                                '    If verif = False Then Exit Sub
                                'End If
                                'If Modification_Acier_HA = True Then
                                '    'indique un 2Ième passage car As ELU < As ELS
                                '    If Ainf(1, i1, j) < Ainf(3, i1, j) Then Asteel = Ainf(3, i1, j) Else Asteel = Ainf(1, i1, j)
                                'Else
                                '    Asteel = Ainf(1, i1, j)
                                'End If
                                'Verification_Aciers_longitudinaux_par_lit_HA(i1, j, Asteel, nbarMax, d, verif)
                                'If verif = False Then Exit Sub
                                'Exit Do 'pour éviter de boucler sur des aciers imposés
                            End If
                            If verif = False Then Exit Sub
                            i = i + 1 : If i > 25 Then Exit Do 'sortie de l'itération sinon bouclage sans fin(les diamètres ne sont pas linéaires)
                        Loop While Abs(d - dd) / d > 0.01
                        'pour éviter de multiple remplissage de la pile des averrtissements et pour éviter tout bagottement autour des valeurs de Amin dépendantes de d
                        'entrée de l'avertissement
                        If msg <> String.Empty Then Aff.msg_Avertiss.Add(msg)
                        Select Case j
                            Case 1
                                .dInf = d
                                Call Tableau_ELU_Armature_Flexion(Mu, ΔM, M2dTab, M3dTab, i1, j, x, d, str, Message, rtf)
                                Call Ferraillage_Pratique(i1, j, rtf)
                            Case 2
                                .dSupG = d
                                If i1 > 1 Then
                                    Select Case .Type_Acier_Appui_Gauche
                                        Case 1
                                            If aci.nomTS(.TS_AppG.Item(0).indTS) = aci.nomTS(PlanCont(i1 - 2).TS_AppD.Item(0).indTS) And .TS_AppG.Count = PlanCont(i1 - 2).TS_AppD.Count Then
                                                rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                                                rtf.WriteString("Ferraillage identique avec le calcul de l'appui droit de la travée précédente car données identiques - Pour les détails du calcul, voir tableau de la travée précédente.") : rtf.NewLine()
                                                rtf.WriteEndGroup()
                                            Else
                                                Call Tableau_ELU_Armature_Flexion(Mu, ΔM, M2dTab, M3dTab, i1, j, x, d, str, Message, rtf)
                                                Call Ferraillage_Pratique(i1, j, rtf)
                                            End If
                                        Case 2
                                            If .LitSupG.Item(0).nbar = PlanCont(i1 - 2).LitSupD.Item(0).nbar And .LitSupG.Item(0).φHA = PlanCont(i1 - 2).LitSupD.Item(0).φHA Then
                                                rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                                                rtf.WriteString("Ferraillage identique avec le calcul de l'appui droit de la travée précédente car données identiques - Pour les détails du calcul, voir tableau de la travée précédente.") : rtf.NewLine()
                                                rtf.WriteEndGroup()
                                            Else
                                                Call Tableau_ELU_Armature_Flexion(Mu, ΔM, M2dTab, M3dTab, i1, j, x, d, str, Message, rtf)
                                                Call Ferraillage_Pratique(i1, j, rtf)
                                            End If
                                    End Select

                                Else
                                    Call Tableau_ELU_Armature_Flexion(Mu, ΔM, M2dTab, M3dTab, i1, j, x, d, str, Message, rtf)
                                    Call Ferraillage_Pratique(i1, j, rtf)
                                End If
                            Case 3
                                .dSupD = d
                                Call Tableau_ELU_Armature_Flexion(Mu, ΔM, M2dTab, M3dTab, i1, j, x, d, str, Message, rtf)
                                Call Ferraillage_Pratique(i1, j, rtf)
                            Case 4
                                If i1 = 1 Then
                                    'idem que les aciers sur appuis de rives
                                    rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                                    rtf.WriteString("Calcul identique à celui de l'appui gauche - Pour les détails du calcul, voir tableau précédent.") : rtf.NewLine()
                                    rtf.WriteEndGroup()
                                ElseIf i1 = PlanCont.Count Then
                                    'idem que les aciers sur appuis de rives
                                    rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                                    rtf.WriteString("Calcul identique à celui de l'appui droit - Pour les détails du calcul, voir tableau précédent.") : rtf.NewLine()
                                    rtf.WriteEndGroup()
                                Else
                                    Call Tableau_ELU_Armature_Flexion(Mu, ΔM, M2dTab, M3dTab, i1, j, x, d, str, Message, rtf)
                                End If
                                Call Ferraillage_Pratique(i1, j, rtf)
                        End Select

                    End If
                Next
            End With
        Next
        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.Underline = True
        rtf.WriteString("2 - 2  Epure des armatures") : rtf.NewLine() : rtf.FontSize = 9 : rtf.NewLine()
        rtf.WriteString("- Clause générale") : rtf.Underline = False : rtf.NewLine()
        rtf.WriteString("EN1992-1-1 A9.2.1.4(2) modifié par AN et commission EC2: FE= [Ved].a1/z + Med/z  (Ned = 0 en flexion simple).") : rtf.NewLine()
        rtf.WriteString("Pour les appuis d'extrémités, le 1ier lit inférieur est ancré totalement quelque soit la valeur de Ved (EN1992-1-1 A9.2.1.4(3)). ") : rtf.NewLine()
        rtf.WriteEndGroup()
        For i1 = 1 To PlanCont.Count
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.Bold = True
            rtf.Underline = True : rtf.WriteString("- Travée n°" & i1.ToString) : rtf.Bold = False : rtf.Underline = False : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("Valeur du décalage de la courbe enveloppe du moment sollicitant: al = " & PlanCont.Item(i1 - 1).al.ToString("#0.00") & " m (eq. 9.2 de l'EN1992-1-1).") : rtf.NewLine()
            rtf.WriteEndGroup()
            With PlanCont.Item(i1 - 1)
                For j = 1 To 4
                    Select Case j 'détermination de l'épure des armatures et divers suivant j
                        Case 1 'acier en travée
                            If i1 = 1 And .LappG <= 0.05 Then Continue For
                            If i1 = PlanCont.Count And .LappD <= 0.05 Then Continue For
                            Mu = Mmax(ELU_ELS, i1, x1Max(ELU_ELS, i1))
                            If AinfC(i1, j) = -1000 Then Asteel = Ainf(ELU_ELS, i1, j) Else Asteel = AinfC(i1, j)
                            Call Moment_resistant_Lit_Acier(ELU_ELS, i1, j, Mu, .dInf, Asteel, Mur) 'calcul du moment repris par chaque lit d'acier
                            Select Case .Type_Acier_Travée
                                Case 1
                                    Call Verification_Acier_sur_appui_TS(1, i1, .dInf, M, V, rtf)
                                    Call Epure_lit_acier_en_travee_TS(ELU_ELS, i1, Mur, m1d, σsd(i1, j))
                                Case 2
                                    Call Verification_Acier_sur_appui_HA(1, i1, .dInf, M, V, rtf)
                                    Call Epure_lit_acier_en_travee_HA(ELU_ELS, i1, Mur, m1d, σsd(i1, j))
                            End Select
                            Call Tab_Epure(i1, j, Mur, rtf)
                        Case 2 'acier sur appui gauche
                            'on ne traite qu'à partir de la travée 2
                            If i1 > 1 Then
                                If .Type_Acier_Appui_Gauche = 2 Then
                                    If (.LitSupG.Item(0).φHA <> PlanCont.Item(i1 - 2).LitSupD.Item(0).φHA) Or
                                        (.LitSupG.Count = 2 And PlanCont.Item(i1 - 2).LitSupD.Count = 2 AndAlso .LitSupG.Item(1).φHA <> PlanCont.Item(i1 - 2).LitSupD.Item(1).φHA) Or
                                        (.LitSupG.Item(0).nbar <> PlanCont.Item(i1 - 2).LitSupD.Item(0).nbar) Or
                                        (.LitSupG.Count = 2 And PlanCont.Item(i1 - 2).LitSupD.Count = 2 AndAlso .LitSupG.Item(1).nbar <> PlanCont.Item(i1 - 2).LitSupD.Item(1).nbar) Then
                                        If i1 > 2 And Mmax(ELU_ELS, i1 - 1, x1Max(ELU_ELS, i1 - 1)) = 0 Then 'm1MAX(ELU_ELS, i1 - 1)
                                            'la travée précédente était soulevé sur toute sa longueur
                                            'les diamètres et le nombre de barres sont déterminées par la travée précédente
                                            msg = "la travée précédente " & (i1 - 1).ToString & " soulevé sur toute sa longueur - diamètre et le nombre de barre appui gauche pour " _
                                                 & "travée " & i1.ToString & " sont déterminées par la travée précédente"
                                            Aff.msg_Avertiss.Add(msg)
                                            MessageBox.Show("Cas non implémenté !" & Environment.NewLine & msg)
                                            'If PlanCont.Item(i1 - 2).LitSupD.Item(1).X2acierLS = PlanCont.Item(i1 - 2).Lx Then
                                            '    'épure calculée sur 2 lits
                                            '    .LitSupG.Item(0).φHA = PlanCont.Item(i1 - 2).LitSupD.Item(0).φHA : .LitSupG.Item(0).nbar = PlanCont.Item(i1 - 2).LitSupD.Item(0).nbar
                                            '    .LitSupG.Item(1).φHA = PlanCont.Item(i1 - 2).LitSupD.Item(1).φHA : .LitSupG.Item(1).nbar = PlanCont.Item(i1 - 2).LitSupD.Item(1).nbar
                                            '    .LitSupG.Item(2).φHA = 0 : .LitSupG.Item(2).nbar = 0
                                            '    .LitSupG_nblit = 2
                                            'Else
                                            '    'l'épure ne sera calculée que sur 1 lit
                                            '    .LitSupG.Item(0).φHA = PlanCont.Item(i1 - 2).LitSupD.Item(0).φHA : .LitSupG.Item(0).nbar = PlanCont.Item(i1 - 2).LitSupD.Item(0).nbar
                                            '    .LitSupG.Item(1).φHA = 0 : .LitSupG.Item(1).nbar = 0
                                            '    .LitSupG.Item(2).φHA = 0 : .LitSupG.Item(2).nbar = 0
                                            '    .LitSupG_nblit = 1
                                            'End If
                                        Else
                                            Call Harmonisation_Barre(ELU_ELS, i1, Ainf, m2d, m3d, Mt_Courbe(i1 - 1), x0, x1min, σsd(i1 - 1, 3), Mur, acier_construction, verif)
                                            'If verif = False Then Exit Sub
                                        End If
                                    End If
                                End If
                            End If
                            If i1 = 1 And .LappG <= 0.05 Then
                                Continue For
                            ElseIf .LitSupG.Count >= 1 Or .TS_AppG.Count >= 1 Then
                                Call Determination_Mu_Appui_Gauche(1, i1, IMmin, Mmax, Mmin, V, x1Max, Mu, ΔM, x, M2dTab)
                                If AinfC(i1, j) = -1000 Then Asteel = Ainf(ELU_ELS, i1, j) Else Asteel = AinfC(i1, j)
                                Call Moment_resistant_Lit_Acier(ELU_ELS, i1, j, Mu, .dSupD, Asteel, Mur) 'calcul du moment repris par chaque lit d'acier
                                Select Case .Type_Acier_Appui_Gauche
                                    Case 1
                                        Call Epure_lit_acier_appui_gauche_TS(verif, ELU_ELS, i1, Mt_Courbe(i1), x0, x1min, σsd(i1, j), Mur, m2d, acier_construction) ' : If verif = False Then Exit Sub
                                    Case 2
                                        Call Epure_lit_acier_appui_gauche_HA(verif, ELU_ELS, i1, Mt_Courbe(i1), x0, x1min, σsd(i1, j), Mur, m2d, acier_construction) ' : If verif = False Then Exit Sub
                                End Select
                                '                Call Entrainement_armature_haute(i1, verif, j)  'vérification de l'entrainement des armatures hautes sur appuis pour les poutres Hyperstatiques
                                Call Tab_Epure(i1, j, Mur, rtf)
                            End If
                            If i1 = PlanCont.Count And .LappD < 0.05 Then Exit For
                        Case 3 'acier sur appui droit
                            ' i = i1 + 1 - consolG '= n° d'appui
                            ' If i > 1 And i < nbapp And m1MAX(ELU_ELS, i1) = 0 Then 'travée soulevée
                            'les soulèvement d'appui sur 1ier et dernier appui sont traité par ailleurs
                            'Call Prolongement_Barre(i1, verif, courbe_moment, x0, acier_construction)
                            'If verif = False Then Exit Sub
                            ' Else
                            If i1 = PlanCont.Count And .LappD <= 0.05 Then
                                Exit For
                            ElseIf .LitSupD.Count >= 1 Or .TS_AppD.Count >= 1 Then
                                Call Determination_Mu_Appui_Droit(1, i1, IMmin, Mmax, Mmin, V, x1Max, Mu, ΔM, x, M3dTab)
                                If AinfC(i1, j) = -1000 Then Asteel = Ainf(ELU_ELS, i1, j) Else Asteel = AinfC(i1, j)
                                Call Moment_resistant_Lit_Acier(ELU_ELS, i1, j, Mu, .dSupD, Asteel, Mur) 'calcul du moment repris par chaque lit d'acier
                                Select Case .Type_Acier_Appui_Droit
                                    Case 1
                                        Call Epure_lit_acier_appui_droit_TS(verif, ELU_ELS, i1, Mt_Courbe(i1), x0, x1min, σsd(i1, j), Mur, m3d, acier_construction) ' : If verif = False Then Exit Sub
                                    Case 2
                                        Call Epure_lit_acier_appui_droit_HA(verif, ELU_ELS, i1, Mt_Courbe(i1), x0, x1min, σsd(i1, j), Mur, m3d, acier_construction) ' : If verif = False Then Exit Sub
                                End Select
                                Call Tab_Epure(i1, j, Mur, rtf)
                            End If
                        Case 4 'armatures sur appuis haut et bas
                            If Aff.EncastPartiel_1_porteur = True Then
                                If .LappH < 0.05 And .LappB < 0.05 Then
                                    'on fait rien
                                Else
                                    Call Epure_lit_acier_appui_Haut_Bas(ELU_ELS, i1, σsd(i1, j))
                                    ' rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui haut et appui bas:") : rtf.Bold = False : rtf.NewLine()
                                    '    ' rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui haut:") : rtf.Italic = False : rtf.NewLine()
                                    '    'rtf.WriteString("Bord libre - Armatures suivant disposition §9.3.1.4") : rtf.NewLine() : rtf.NewLine()
                                    '    'rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui bas:") : rtf.Bold = False : rtf.NewLine()

                                End If

                            Else
                                rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                                rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui haut et appui bas:") : rtf.Bold = False : rtf.NewLine()
                                rtf.WriteString("Sans objet, aucun moment n'est développé.") : rtf.NewLine()
                                rtf.WriteEndGroup()
                            End If
                    End Select

                Next j
                rtf.WriteStartGroup()
                rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                rtf.WriteString("Les Abscisses marquées par une * correspondent aux aciers totalement ancrés sur appui.") : rtf.NewLine()
                rtf.WriteEndGroup()
                rtf.WriteStartGroup()
                Try
                    ' Create Image Element
                    Dim myImage As Image
                    myImage = My.Resources.Test530_330 'Image.FromFile(Directory.GetCurrentDirectory.ToString() & msg)
                    Dim g As Graphics = Graphics.FromImage(myImage)
                    Call Affichage_Courbes_Dessin.Affichage_Moment(g, i1, 2, 0, 530, 330, 1, M, Mmax, Mmin, m1d, m2d, m3d, Mur, x1Max, ΔM)
                    myImage.Save(Directory.GetCurrentDirectory.ToString() & "\test1.bmp") '  myImage.Save(Directory.GetCurrentDirectory.ToString() & "\test1.png", ImageFormat.Bmp)
                    g.Dispose()
                    'myImage.RotateFlip(RotateFlipType.Rotate90FlipNone)
                    rtf.InsertImage(myImage)
                    My.Computer.FileSystem.DeleteFile(Directory.GetCurrentDirectory.ToString() & "\test1.bmp") 'effacement du fichier
                Catch ex As Exception
                    msg = "Le logiciel n'a pas pu chargé l'image correspondante." & Environment.NewLine & "Erreur: " & ex.Message & ex.StackTrace & Environment.NewLine & "Note de calcul sans image"
                    MessageBox.Show(msg, "EC2: calcul section ELU fondamental", MessageBoxButtons.OK, MessageBoxIcon.Information)
                End Try
                rtf.WriteEndGroup()
            End With
        Next i1
        Erase str
    End Sub
    ''' <summary>Programme de calcul du moment Maxi sur Appui Gauche avec prise en compte ou pas de l'écrêtage et du nu d'appui</summary>
    ''' <param name="i1">n° de travée</param>
    ''' <param name="ELU_ELS">indique le type d'Etat Limite: 1=ELU Str, 2= ELU Equilibre, 3=ELS caractéristique, 4=ELS quasi-permanent</param>
    '''  <param name="Mmax">Moment enveloppe Maxi de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    '''  <param name="Mmin">Moment enveloppe mini de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="IMmin">indice de la combinaison conduisant à la valeur maximale du moment sur appui droit de la travée i (en valeur absolue) avec les indices (ELU_ELS, i2): ELU_ELS = type d'etat Limite, i2 = n° de travée</param>
    ''' <param name="V">, V(imax, PlanCont.Count, lcMax) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    Private Sub Determination_Mu_Appui_Gauche(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal IMmin(,) As Integer, ByVal Mmax(,,) As Double, ByVal Mmin(,,) As Double,
                                           ByVal V(,,) As Double, ByVal x1Max(,) As Integer, ByRef Mu As Double, ByRef ΔM(,) As Double, Optional ByRef x As Integer = 0,
                                           Optional ByRef M2dTab As Double = 0) 'ByVal m2d(,,) As Double,

        With PlanCont.Item(i1 - 1)
            Dim nu_app As Integer = CInt(Truncate(.a1 * 100))
            x = 0 'Position du moment maxi sur appui gauche = 0
            If i1 = 1 Then 'traitement des appuis d'about
                If Aff.Reprise_M0_1_porteur = True Or Aff.EncastPartiel_1_porteur = True Then
                    Mu = 0.15 * Mmax(ELU_ELS, i1, x1Max(ELU_ELS, i1))   ': boucle = 0
                Else
                    Mu = 0.000001 ' 1 N.m,pour éviter trop de modification, ne change rien au calcul
                End If
            ElseIf i1 = PlanCont.Count And .LappD < 0.05 Then
                'calcul aciers sur console
                Mu = -Mmin(ELU_ELS, i1, 0) '-m2d(ELU_ELS, i1, 0) 'Calcul de l'acier pour le moment maxi sur appui gauche
            Else
                If Aff.Ecretage_Moment = True Then
                    Call Calcul_Ecretage_Moment(IMmin, 1, i1, V, ELU_ELS, ΔM(i1, 2)) : M2dTab = -Mmin(ELU_ELS, i1, 0) '-m2d(ELU_ELS, i1, 0)
                    If .App_Monolith = True Then
                        Mu = Min(-Mmin(ELU_ELS, i1, 0) - ΔM(i1, 2), -Mmin(ELU_ELS, i1, nu_app)) 'Min(-m2d(ELU_ELS, i1, 0) - ΔM(i1, 2), -Mmin(ELU_ELS, i1, nu_app))
                    Else
                        Mu = -Mmin(ELU_ELS, i1, 0) - ΔM(i1, 2) ' Min(-m2d(ELU_ELS, i1, 0) - ΔM(i1, 2), -m2d(ELU_ELS, i1, 0))
                    End If
                Else
                    If .App_Monolith = True Then
                        Mu = -Mmin(ELU_ELS, i1, nu_app)
                    Else
                        Mu = -Mmin(ELU_ELS, i1, 0) '-m2d(ELU_ELS, i1, 0) 'Calcul de l'acier pour le moment maxi sur appui gauche
                    End If

                End If
            End If
        End With

    End Sub
    ''' <summary>Programme de calcul du moment Maxi sur Appui Droit avec prise en compte ou pas de l'écrêtage et du nu d'appui</summary>
    ''' <param name="i1">n° de travée</param>
    ''' <param name="ELU_ELS">indique le type d'Etat Limite: 1=ELU Str, 2= ELU Equilibre, 3=ELS caractéristique, 4=ELS quasi-permanent</param>
    '''  <param name="Mmax">Moment enveloppe Maxi de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    '''  <param name="Mmin">Moment enveloppe mini de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="IMmin">indice de la combinaison conduisant à la valeur maximale du moment sur appui droit de la travée i (en valeur absolue) avec les indices (ELU_ELS, i2): ELU_ELS = type d'etat Limite, i2 = n° de travée</param>
    ''' <param name="V">, V(imax, PlanCont.Count, lcMax) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    Private Sub Determination_Mu_Appui_Droit(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal IMmin(,) As Integer, ByVal Mmax(,,) As Double, ByVal Mmin(,,) As Double,
                                             ByVal V(,,) As Double, ByVal x1Max(,) As Integer, ByRef Mu As Double, ByRef ΔM(,) As Double, Optional ByRef x As Integer = 0,
                                             Optional ByRef M3dTab As Double = 0) 'ByVal m3d(,,) As Double,
        With PlanCont.Item(i1 - 1)
            x = CInt(Truncate(.Lcx * 100)) 'Position du moment maxi sur appui droit = lc(i1)
            '  x = CInt(Int(.Lcx * 100))
            Dim nu_app As Integer = CInt(Truncate((.Lcx - .a2) * 100))
            If i1 = 1 And .LappG < 0.05 Then
                'calcul aciers sur console
                Mu = -Mmin(ELU_ELS, i1, x) '-m3d(ELU_ELS, i1, x) 'Calcul de l'acier pour le moment maxi sur appui droit
            ElseIf i1 = PlanCont.Count Then 'traitement des appuis d'about
                If Aff.Reprise_M0_1_porteur = True Or Aff.EncastPartiel_1_porteur = True Then 'traitement des appuis d'about
                    Mu = 0.15 * Mmax(ELU_ELS, i1, x1Max(ELU_ELS, i1))  ': boucle = 0
                Else
                    Mu = 0.000001 ' 1 N.m
                End If
            Else
                If Aff.Ecretage_Moment = True Then
                    Call Calcul_Ecretage_Moment(IMmin, 2, i1, V, ELU_ELS, ΔM(i1, 3)) : M3dTab = -Mmin(ELU_ELS, i1, x) ''-m3d(ELU_ELS, i1, x)
                    If .App_Monolith = True Then
                        If -Mmin(ELU_ELS, i1, x) - ΔM(i1, 3) < -Mmin(ELU_ELS, i1, nu_app) Then
                            Mu = -Mmin(ELU_ELS, i1, x) - ΔM(i1, 3) : x = CInt(Truncate(.Lcx * 100))
                        Else
                            Mu = -Mmin(ELU_ELS, i1, nu_app) : x = nu_app
                        End If
                        ' Mu = Min(-Mmin(ELU_ELS, i1, x) - ΔM(i1, 3), -Mmin(ELU_ELS, i1, nu_app)) 'Min(-m3d(ELU_ELS, i1, x) - ΔM(i1, 3), -Mmin(ELU_ELS, i1, nu_app))
                    Else
                        Mu = -Mmin(ELU_ELS, i1, x) - ΔM(i1, 3) : x = CInt(Truncate(.Lcx * 100)) 'Min(-m3d(ELU_ELS, i1, x) - ΔM(i1, 3), -m3d(ELU_ELS, i1, x))
                    End If
                Else
                    If .App_Monolith = True Then
                        Mu = -Mmin(ELU_ELS, i1, nu_app) : x = nu_app
                    Else
                        Mu = -Mmin(ELU_ELS, i1, x) : x = CInt(Truncate(.Lcx * 100)) '-m3d(ELU_ELS, i1, x) 'Calcul de l'acier pour le moment maxi sur appui droit
                    End If
                End If
            End If
        End With

    End Sub
    ''' <summary>Programme de calcul de la valeur de l'écrêtage sur appui</summary>
    ''' <param name="i1">n° de travée</param>
    '''     <param name="appui">n° d'appui: 1=appui gauche; 2=appui droit</param>
    ''' <param name="ELU_ELS">indique le type d'Etat Limite: 1=ELU Str, 2= ELU Equilibre, 3=ELS caractéristique, 4=ELS quasi-permanent</param>
    '''  <param name="ΔM">valeur de la réduction du moment sur appui grace à l'écrêtage</param>
    ''' <param name="IMmin">indice de la combinaison conduisant à la valeur maximale du moment sur appui droit de la travée i (en valeur absolue) avec les indices (ELU_ELS, i2): ELU_ELS = type d'etat Limite, i2 = n° de travée</param>
    ''' <param name="V">V(imax, PlanCont.Count, lcMax) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    Private Sub Calcul_Ecretage_Moment(ByVal IMmin(,) As Integer, ByVal appui As Integer, ByVal i1 As Integer, ByVal V(,,) As Double, ByVal ELU_ELS As Integer, ByRef ΔM As Double)
        Dim Nint As Integer
        With PlanCont.Item(i1 - 1)
            Select Case appui
                Case 1 'appui gauche
                    Nint = CInt(Truncate(PlanCont.Item(i1 - 2).Lcx * 100))
                    ΔM = (Abs(V(IMmin(ELU_ELS, i1 - 1), i1 - 1, Nint)) + Abs(V(IMmin(ELU_ELS, i1 - 1), i1, 0))) * .LappG / 8
                Case 2 'appui droit
                    Nint = CInt(Truncate(.Lcx * 100))
                    ΔM = (Abs(V(IMmin(ELU_ELS, i1), i1, Nint)) + Abs(V(IMmin(ELU_ELS, i1), i1 + 1, 0))) * .LappD / 8
            End Select
        End With
    End Sub
    Private Sub Ferraillage_Pratique(ByVal i1 As Integer, ByVal j As Integer, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim Tot, totTr As Double
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9
        With PlanCont.Item(i1 - 1)
            rtf.NewLine() : rtf.WriteString("Ferraillage retenu (de la génératrice extérieure vers l'intérieur de la dalle):") : rtf.NewLine()
            Select Case j
                Case 1 'en travée 
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            Tot = aci.As_long(.TS_trav.Item(0).indTS)
                            rtf.WriteString("Lit n°1: TS " & aci.nomTS(.TS_trav.Item(0).indTS))
                            rtf.WriteString(" - section = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            totTr = aci.As_trans(.TS_trav.Item(0).indTS)
                            'pour la partie d'armature sur les appuis non porteurs
                            .TS_trav.Item(0).L33acier = .LapHA : .TS_trav.Item(0).L33acier = .LapHA
                            If aci.Position_Portefeuille = True And .Lx + 0.5 > aci.Long_TS(.TS_trav.Item(0).indTS) Then '.Ly > aci.LcommercialHA And 
                                Dim LitTS As New Ferraillage_TS
                                LitTS.indTS = .TS_trav.Item(0).indTS
                                .TS_trav_Portefeuille.Clear() : .TS_trav_Portefeuille.Add(LitTS)
                            End If
                            If .TS_trav.Count = 2 Then
                                Tot = Tot + aci.As_long(.TS_trav.Item(1).indTS)
                                rtf.WriteString("Lit n°2: TS " & aci.nomTS(.TS_trav.Item(1).indTS) & " - section = " & (aci.As_long(.TS_trav.Item(1).indTS) * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                                rtf.WriteString("Soit une section totale = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                                totTr = totTr + aci.As_trans(.TS_trav.Item(1).indTS)
                                'pour la partie d'armature sur les appuis non porteurs
                                .TS_trav.Item(1).L33acier = .LapHA : .TS_trav.Item(1).L33acier = .LapHA
                            End If
                            If .Prefa = True AndAlso .Prefa_Lx.Item(0) > 5.8 Then 'toutes les prédalles ont même longueur
                                rtf.WriteString("La portée dépasse la longueur commerciale disponible, le lit n°1 sera doublé et disposé en portefeuille (voir plan de ferraillage).") : rtf.NewLine()
                            End If
                            rtf.WriteString("Total section transversale (suivant Y):  " & (totTr * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                        Case 2 'HA
                            rtf.WriteString("Longitudinal porteur (suivant X): " & .LitInf.Item(0).nbar.ToString & " HA" & (.LitInf.Item(0).φHA * 1000).ToString("#") & " - ")
                            rtf.WriteString("s = " & (.LitInf(0).Scourant * 100).ToString("#0.0") & " cm ")
                            Tot = (.LitInf.Item(0).Aire / .Ly) : rtf.WriteString(" soit Aire totale = " & (.LitInf(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            rtf.WriteString("Transversal NON porteur (suivant Y):  " & .LitInf_Y.Item(0).nbar.ToString & " HA" & (.LitInf_Y.Item(0).φHA * 1000).ToString("#") & " - ")
                            rtf.WriteString("s = " & (.LitInf_Y(0).Scourant * 100).ToString("#0.0") & " cm ")
                            Tot = (.LitInf_Y.Item(0).Aire / .Lx) : rtf.WriteString(" soit Aire totale = " & (.LitInf_Y(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            If aci.Position_Portefeuille = True Then '.Ly > aci.LcommercialHA And 
                                .LitInf_Y.Item(0).L13acier = .LapHA : .LitInf_Y.Item(0).Lacier2 = aci.HA_Lg_Max - .LitInf_Y.Item(0).L13acier : .LitInf_Y.Item(0).Lacier1 = aci.HA_Lg_Max
                                .LitInf_Y.Item(0).X1acier = 0 : .LitInf_Y.Item(0).X1acier = aci.HA_Lg_Max - .LitInf_Y.Item(0).L13acier
                                .LitInf_Y.Item(1).nbar = .LitInf_Y.Item(0).nbar : .LitInf_Y.Item(1).φHA = .LitInf_Y.Item(0).φHA : .LitInf_Y(1).Scourant = .LitInf_Y(0).Scourant : .LitInf_Y(1).S1 = .LitInf_Y(0).S1
                                .LitInf_Y(1).S2 = .LitInf_Y(0).S2 : .LitInf_Y.Item(1).φDM1 = .LitInf_Y.Item(0).φDM1 : .LitInf_Y.Item(1).φDM2 = .LitInf_Y.Item(0).φDM2
                                .LitInf_Y.Item(1).L23acier = .LapHA : .LitInf_Y.Item(1).L22acier = .LitInf_Y.Item(0).L22acier : .LitInf_Y.Item(1).L21acier = .LitInf_Y.Item(0).L21acier
                                .LitInf_Y.Item(1).L13acier = 0 : .LitInf_Y.Item(1).L12acier = 0 : .LitInf_Y.Item(1).L11acier = 0
                                .LitInf_Y.Item(1).Lacier2 = .Ly - .LitInf_Y.Item(0).Lacier2 + 1.5 * Lscel_droit(0, 7, i1, 1, 500 / 1.15, False) 'la dernière partie de l'expression correspondat à la longueur de recouvrement
                                .LitInf_Y.Item(1).Lacier2 = (Truncate(.LitInf_Y.Item(1).Lacier2 * 100) + 1) / 100 'arrondi au cm supérieur
                                .LitInf_Y.Item(1).X1acier = .LitInf_Y.Item(1).Lacier2
                                .LitInf_Y.Item(1).Lacier1 = .LitInf_Y.Item(1).Lacier2 + .LitInf_Y.Item(1).L23acier
                                rtf.WriteString("La longueur de barre suivant Y dépasse la longueur commerciale disponible, les barres sont doublées et disposées en portefeuille (voir plan de ferraillage).") : rtf.NewLine()

                            Else
                                'pour le cas d'un seul acier
                                .LitInf_Y.Item(0).L13acier = .LapHA : .LitInf_Y.Item(0).L23acier = .LapHA : .LitInf_Y.Item(0).Lacier1 = .Ly + 2 * .LapHA : .LitInf_Y.Item(0).Lacier2 = .Ly
                            End If
                    End Select
                Case 2 'sur appui gauche
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1 'TS
                            Tot = aci.As_long(.TS_AppG.Item(0).indTS)
                            rtf.WriteString("Lit n°1: TS " & aci.nomTS(.TS_AppG.Item(0).indTS))
                            rtf.WriteString(" - section = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            totTr = aci.As_trans(.TS_AppG.Item(0).indTS)
                            If .TS_AppG.Count = 2 Then
                                Tot = Tot + aci.As_long(.TS_AppG.Item(1).indTS)
                                rtf.WriteString("Lit n°2: TS " & aci.nomTS(.TS_AppG.Item(1).indTS) & " - section = " & (aci.As_long(.TS_AppG.Item(1).indTS) * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                                rtf.WriteString("Soit une section totale = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                                totTr = totTr + aci.As_trans(.TS_AppG.Item(1).indTS)
                            End If
                        Case 2 'HA
                            rtf.WriteString("Longitudinal porteur (suivant X): Lit n°1: " & .LitSupG.Item(0).nbar.ToString & " HA" & (.LitSupG.Item(0).φHA * 1000).ToString("#") & " - ")
                            rtf.WriteString("s = " & (.LitSupG(0).Scourant * 100).ToString("#0.0") & " cm ")
                            Tot = (.LitSupG.Item(0).Aire / .Ly) : rtf.WriteString(" soit Aire totale = " & (.LitSupG(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            If .LitSupG_Y.Count >= 1 Then
                                rtf.WriteString("Transversal NON porteur (suivant Y): Lit n°2: " & .LitSupG_Y.Item(0).nbar.ToString & " HA" & (.LitSupG_Y.Item(0).φHA * 1000).ToString("#") & "/ - ")
                                ' s = 1 / .LitSupG_Y.Item(0).nbar : rtf.WriteString("s = " & (s * 100).ToString("#0.0") & " cm ")
                                Tot = (.LitSupG_Y.Item(0).Aire) : rtf.WriteString(" soit Aire totale = " & (.LitSupG_Y(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            End If
                    End Select
                Case 3 'sur appui droit
                    Select Case .Type_Acier_Appui_Droit
                        Case 1 'TS
                            Tot = aci.As_long(.TS_AppD.Item(0).indTS)
                            rtf.WriteString("Lit n°1: TS " & aci.nomTS(.TS_AppD.Item(0).indTS))
                            rtf.WriteString(" - section = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            totTr = aci.As_trans(.TS_AppD.Item(0).indTS)
                            If .TS_AppD.Count = 2 Then
                                Tot = Tot + aci.As_long(.TS_AppD.Item(1).indTS)
                                rtf.WriteString("Lit n°2: TS " & aci.nomTS(.TS_AppD.Item(1).indTS) & " - section = " & (aci.As_long(.TS_AppD.Item(1).indTS) * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                                rtf.WriteString("Soit une section totale = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                                totTr = totTr + aci.As_trans(.TS_AppD.Item(1).indTS)
                            End If
                        Case 2 'HA
                            rtf.WriteString("Longitudinal porteur (suivant X): Lit n°1: " & .LitSupD.Item(0).nbar.ToString & " HA" & (.LitSupD.Item(0).φHA * 1000).ToString("#") & " - ")
                            rtf.WriteString("s = " & (.LitSupD(0).Scourant * 100).ToString("#0.0") & " cm ")
                            Tot = (.LitSupD.Item(0).Aire / .Ly) : rtf.WriteString(" soit Aire totale = " & (.LitSupD(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            If .LitSupD_Y.Count >= 1 Then
                                rtf.WriteString("Transversal NON porteur (suivant Y): Lit n°2: " & .LitSupD_Y.Item(0).nbar.ToString & " HA" & (.LitSupD_Y.Item(0).φHA * 1000).ToString("#") & " - ")
                                ' s = 1 / .LitSupD_Y.Item(0).nbar : rtf.WriteString("s = " & (s * 100).ToString("#0.0") & " cm ")
                                Tot = (.LitSupD_Y.Item(0).Aire) : rtf.WriteString(" soit Aire totale = " & (.LitSupD_Y(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            End If
                    End Select
                Case 4 'Aciers supérieurs sur appuis non porteurs
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            Tot = aci.As_long(.TS_AppH.Item(0).indTS)
                            rtf.WriteString("Lit appui haut: TS " & aci.nomTS(.TS_AppH.Item(0).indTS) & " - section = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            rtf.WriteString("Lit appui bas: TS " & aci.nomTS(.TS_AppB.Item(0).indTS) & " - section = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                        Case 2 'HA
                            If .LappH > 0.05 And Aff.EncastPartiel_1_porteur = True Then
                                rtf.WriteString("sur appui Haut " & .LitSupH.Item(0).nbar.ToString & " HA" & (.LitSupH.Item(0).φHA * 1000).ToString("#") & " - ")
                                rtf.WriteString("s = " & (.LitSupH.Item(0).Scourant * 100).ToString("#0.0") & " cm ")
                                Tot = (.LitSupH.Item(0).Aire / .Lx) : rtf.WriteString(" soit Aire totale = " & (.LitSupH.Item(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            End If
                            If .LappB > 0.05 Then
                                If .LappH > 0.05 And Aff.EncastPartiel_1_porteur = True Then
                                    rtf.WriteString("sur appui Bas: identique à l'appui haut. ") : rtf.NewLine()
                                Else
                                    rtf.WriteString("sur appui Bas " & .LitSupB.Item(0).nbar.ToString & " HA" & (.LitSupB.Item(0).φHA * 1000).ToString("#") & " - ")
                                    rtf.WriteString("s = " & (.LitSupB.Item(0).Scourant * 100).ToString("#0.0") & " cm ")
                                    Tot = .LitSupB.Item(0).Aire / .Lx : rtf.WriteString(" soit Aire totale = " & (.LitSupB.Item(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                                End If
                            End If
                    End Select
            End Select
        End With
        rtf.WriteEndGroup()
    End Sub
    ''' <summary> Affichage du tableau d'épure</summary>
    ''' <param name="Mur">Moment Ultime résistant avec les indices (ELU_ELS, i1, j, 1) ELU_ELs = n° de combinaison, i1 = n° de travée, j=1,2 ou 3 entravée, sur appui gauche, sur appui droit, 
    ''' 1 = n° de lit, de la surface vers l'intérieur de la dalle</param>
    '''  <param name="i1">n° de travéed</param>
    '''  <param name="j">j= indique la position de l'acier, =1 en travée,2 = sur appui gauche, ou 3 = sur appui droit</param>
    ''' <param name="rtf"> stream pour le renvoi du fichier rtf</param>
    Private Sub Tab_Epure(ByVal i1 As Integer, ByVal j As Integer, ByVal MuR(,,,) As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim i, total_Lit As Integer ', k
        Dim A_s(3), x1(3), x1s(3), x2(3), x2s(3) As Double
        Dim msg As String
        With PlanCont.Item(i1 - 1)
            Select Case j
                Case 1
                    Select Case .Type_Acier_Travée
                        Case 1
                            With .TS_trav
                                A_s(1) = aci.As_long(.Item(0).indTS)
                                If .Count = 2 Then A_s(2) = aci.As_long(.Item(1).indTS)
                                For i = 1 To .Count
                                    x1(i) = .Item(i - 1).X1acier : x1s(i) = .Item(i - 1).X1acierLS : x2(i) = .Item(i - 1).X2acier
                                    x2s(i) = .Item(i - 1).X2acierLS
                                Next
                                total_Lit = .Count
                            End With
                        Case 2
                            With .LitInf
                                For i = 1 To .Count
                                    A_s(i) = .Item(i - 1).Aire : x1(i) = .Item(i - 1).X1acier : x1s(i) = .Item(i - 1).X1acierLS : x2(i) = .Item(i - 1).X2acier
                                    x2s(i) = .Item(i - 1).X2acierLS
                                Next
                                total_Lit = .Count
                            End With
                    End Select

                Case 2
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1
                            With .TS_AppG
                                A_s(1) = aci.As_long(.Item(0).indTS)
                                If .Count = 2 Then A_s(2) = aci.As_long(.Item(1).indTS)
                                For i = 1 To .Count
                                    x1(i) = .Item(i - 1).X2acier : x1s(i) = .Item(i - 1).X2acierLS : x2(i) = 0 : x2s(i) = 0
                                Next
                                total_Lit = .Count
                            End With
                        Case 2
                            With .LitSupG
                                For i = 1 To .Count
                                    A_s(i) = .Item(i - 1).Aire : x1(i) = .Item(i - 1).X2acier : x1s(i) = .Item(i - 1).X2acierLS : x2(i) = 0 : x2s(i) = 0
                                Next
                                total_Lit = .Count
                            End With
                    End Select
                Case 3
                    Select Case .Type_Acier_Appui_Droit
                        Case 1
                            With .TS_AppD
                                A_s(1) = aci.As_long(.Item(0).indTS)
                                If .Count = 2 Then A_s(2) = aci.As_long(.Item(1).indTS)
                                For i = 1 To .Count
                                    x1(i) = .Item(i - 1).X1acier : x1s(i) = .Item(i - 1).X1acierLS : x2(i) = 0 : x2s(i) = 0
                                Next
                                total_Lit = .Count
                            End With
                        Case 2
                            With .LitSupD
                                For i = 1 To .Count
                                    A_s(i) = .Item(i - 1).Aire : x1(i) = .Item(i - 1).X1acier : x1s(i) = .Item(i - 1).X1acierLS : x2(i) = 0 : x2s(i) = 0
                                Next
                                total_Lit = .Count
                            End With
                    End Select
            End Select
        End With

        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9
        rtf.NewParagraph()

        If j = 1 OrElse (i1 = 1 And j = 3 And PlanCont.Item(0).LappG < 0.05) = True OrElse (i1 = PlanCont.Count And j = 2 And PlanCont.Item(PlanCont.Count - 1).LappD < 0.05) = True Then
            rtf.Bold = True : rtf.WriteString("  ■ Epure des aciers") : rtf.Bold = False : rtf.NewLine()
            rtf.WriteString("L'origine de l'abscisse est le nu de l'appui gauche (ou l'extrémité de la console gauche pour la 1ière travée dans le cas d'une console).") : rtf.NewLine()
            rtf.NewParagraph()
            rtf.FontSize = 12 : rtf.FontSize = 9
            'Ligne 1
            rtf.BeginRow() : rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Lit n°1") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Lit n°2") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Lit n°3") : rtf.EndCell() : rtf.EndRow()
        End If

        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.Bold = True
        Select Case j
            Case 1
                rtf.WriteString("Section en travée")
            Case 2
                rtf.WriteString("Section sur appui gauche")
            Case 3
                rtf.WriteString("Section sur appui droit")
        End Select
        rtf.Bold = False : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()

        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((A_s(1) * 10000 / PlanCont.Item(i1 - 1).Ly).ToString("#0.00")) : rtf.EndCell()
        If total_Lit > 1 Then
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((A_s(2) * 10000 / PlanCont.Item(i1 - 1).Ly).ToString("#0.00")) : rtf.EndCell()
        End If
        rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Cumul Moment Ultime Résistant") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((MuR(1, i1, j, 1) * 1000).ToString("#0.0")) : rtf.EndCell()
        If total_Lit > 1 Then
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((MuR(1, i1, j, 2) * 1000).ToString("#0.0")) : rtf.EndCell()
        End If
        rtf.EndRow()
        'Ligne 4
        rtf.BeginRow() : rtf.DeclareCell(6000) : rtf.BeginCell(True)
        If (i1 = 1 And j = 2) OrElse (i1 = PlanCont.Count And j = 3) And Aff.Reprise_M0_1_porteur = True Then
            rtf.WriteString("Calcul sur moment forfaitaire - Ancrage sur longueur forfaitaire") : rtf.EndCell()
        Else
            rtf.WriteString("Interception courbe enveloppe moment sollicitant décalé") : rtf.EndCell()
        End If
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
        Select Case j
            Case 1, 3
                If j = 1 Then msg = " *" Else msg = ""
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x1s(1).ToString("#0.00") & msg) : rtf.EndCell()
                If total_Lit > 1 Then
                    If x1s(2) = 0 And j = 1 Then msg = " *" Else msg = ""
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x1s(2).ToString("#0.00") & msg) : rtf.EndCell()
                End If
            Case 2
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x2s(1).ToString("#0.00") & msg) : rtf.EndCell()
                If total_Lit > 1 Then
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x2s(2).ToString("#0.00") & msg) : rtf.EndCell()
                End If
        End Select
        rtf.EndRow()
        'Ligne 5
        rtf.BeginRow() : rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Abscisse origine de l'ancrage") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
        Select Case j
            Case 1, 3
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x1(1).ToString("#0.00")) : rtf.EndCell()
                If total_Lit > 1 Then
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x1(2).ToString("#0.00")) : rtf.EndCell()
                End If
            Case 2
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x2(1).ToString("#0.00")) : rtf.EndCell()
                If total_Lit > 1 Then
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x2(2).ToString("#0.00")) : rtf.EndCell()
                End If
        End Select
        rtf.EndRow()
        If j = 1 Then
            'Ligne 6
            rtf.BeginRow() : rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Interception courbe enveloppe moment sollicitant décalé (2ième point)") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x2s(1).ToString("#0.00") & " *") : rtf.EndCell()
            If total_Lit > 1 Then
                If x2s(2) = 0 Then msg = " *" Else msg = ""
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x2s(2).ToString("#0.00") & msg) : rtf.EndCell()
            End If
            rtf.EndRow()
            'Ligne 7
            rtf.BeginRow() : rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Abscisse origine de l'ancrage (2ième point)") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x2(1).ToString("#0.00")) : rtf.EndCell()
            If total_Lit > 1 Then
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x2(2).ToString("#0.00")) : rtf.EndCell()
            End If
            rtf.EndRow()
        End If
        rtf.WriteEndGroup()
    End Sub
    Private Sub Calcul_section_ELU_Accidentel(ByRef verif As Boolean)
        '----->ELU Accidentel:
        '-->Pour effectuer les vérifications à l'ELU Accidentel, il est préférable de
        '-->procéder d'abord à la vérification ELS dans le cas ou il est exigé une
        '-->fissuration préjudiciable ou trés préjudiciable, ces deux dernières étant
        '-->généralement dimensionnante. Le logiciel ajoute les aciers complémentaires
        '-->nécessaire pour satisfaire les critères de fissuration.
        '--> pour la vérification au feu , celle-ci est faite directement dans le module
        '--> "calcul au feu"; pour la vérification au seisme,

        'Dim numero, testOK3, testOK4 As Integer
        'testOK3 = 0 : testOK4 = 0
        'For numero = 1 To combMax(2)
        '    If sol(combMax(1) + numero) > 2 Then 'pour le feu, la vérification est faite dans le module "Calcul au feu"
        '        If sol(numero) = 3 Then 'vérification si une comb est sismique
        '            If testOK3 = 0 Then   'si oui, a-t-elle été vérifiée?
        '                ELU_ELS = 2 : testOK3 = 1
        '                Call Moment_resistant_ELU(verif)  'vérification que le moment sollicitant sismique
        '                If verif = False Then Exit Sub 'reste inférieur au moment ELU Fondamental
        '            End If
        '        End If
        '        If sol(numero) = 4 Then
        '            If testOK4 = 0 Then
        '                ELU_ELS = 4 : testOK4 = 1
        '                Call Moment_resistant_ELU(verif) 'vérification que le moment sollicitant accidentel
        '                If verif = False Then Exit Sub 'reste inférieur au moment ELU Fondamental
        '            End If
        '        End If
        '    End If
        'Next numero

    End Sub
    Public Sub Calcul_Coef_Fluage(ByVal t0p As Integer, ByVal RHp As Integer, ByVal tinfp As Integer, ByRef φinf As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        ' calcul du coefficient n suivant annexe B
        Dim φ0, βH, βc, α, h0, t, t0, T1, T2 As Double
        Dim msg As String
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewParagraph()
        rtf.WriteString("Détail du calcul du coefficient n:") : rtf.NewLine()
        'calcul de h0 sur la 1ière travée uniquement, les autres travées étant généralement peu différente de la 1ière
        With PlanCont.Item(0)
            h0 = 2 * (.Ly * .H) / (2 * .Ly)
        End With
        h0 = h0 * 1000 'en mm
        'modification de t0 par équation B9
        Select Case bet.Classe
            Case 0 'classe S
                α = -1
            Case 1 'classe N
                α = 0
            Case 2 'classe R
                α = 1
        End Select
        t0 = t0p * ((9 / (2 + t0p ^ 1.2)) + 1) ^ α '(équation B9)
        If t0 < 0.5 Then t0 = 0.5
        T2 = (16.8 / bet.fcm ^ 0.5) * (1 / (0.1 + t0 ^ 0.2))
        Select Case bet.fcm
            Case Is > 35
                T1 = (1 + (((1 - RHp / 100) / (0.1 * h0 ^ (1 / 3))) * (35 / bet.fcm) ^ 0.7)) * (35 / bet.fcm) ^ 0.2
                βH = 1.5 * (1 + (0.012 * RHp) ^ 18) * h0 + (250 * (35 / bet.fcm) ^ 0.5)
                If βH > 1500 * (35 / bet.fcm) ^ 0.5 Then βH = 1500 * (35 / bet.fcm) ^ 0.5
            Case Else
                T1 = 1 + ((1 - RHp / 100) / (0.1 * h0 ^ (1 / 3)))
                βH = 1.5 * (1 + (0.012 * RHp) ^ 18) * h0 + 250
                If βH > 1500 Then βH = 1500
        End Select
        φ0 = T1 * T2
        msg = "Calcul du coefficient d'équivalence n: la durée d'exploitation de la structure est inférieure à la durée du chargement. Modifiez la durée de chargement."
        Select Case tinfp
            Case 0 '10 ans
                t = 10 * 365
            Case 1 '25 ans
                t = 25 * 365
                If tinfp > Aff.Duree Then Aff.msg_Avertiss.Add(msg)
            Case 2 '50 ans
                t = 50 * 365
                If tinfp > Aff.Duree Then Aff.msg_Avertiss.Add(msg)
            Case 3 '100 ans
                t = 100 * 365
                If tinfp > Aff.Duree Then Aff.msg_Avertiss.Add(msg)
        End Select
        βc = ((t - t0) / (βH + t - t0)) ^ 0.3
        φinf = φ0 * βc 'eq B1
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        'Ligne 10
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("h0") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(h0.ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("rayon moyen eq. B.6") : rtf.EndCell() : rtf.EndRow()
        'Ligne 10
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("α") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(α.ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("facteur correctif type ciment eq. B.9") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("t0") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString("jours") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(t0p.ToString("#,##0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Age du béton au moment du chargement") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("t0") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString("jours") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(t0.ToString("#,##0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Age du béton corrigé par le type de ciment Eq. B.9") : rtf.EndCell() : rtf.EndRow()
        'Ligne 4
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("fcm ") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString("Mpa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(bet.fcm.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("résistance moyenne du béton à 28 jours") : rtf.EndCell() : rtf.EndRow()
        'Ligne 4
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("ϕRH ") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(T1.ToString(" #0.000")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Eq B3") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("β(fcm)") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((16.8 / bet.fcm ^ 0.5).ToString(" #0.000")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Eq B4") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("β(t0)") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((1 / (0.1 + t0 ^ 0.2)).ToString(" #0.000")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Eq B5") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("βH") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(βH.ToString(" #0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Eq B8a-b") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("βc(t,t0)") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(βc.ToString(" #0.000")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Eq B7") : rtf.EndCell() : rtf.EndRow()
        'Ligne 4
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("ϕ0 ") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(φ0.ToString(" #0.000")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Eq B2") : rtf.EndCell() : rtf.EndRow()
        'Ligne 4
        rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("ϕ(t,t0) ") : rtf.EndCell()
        rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Aff.φinf.ToString(" #0.000")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Eq B1 coef fluage à l'age de chargement t") : rtf.EndCell() : rtf.EndRow()
        ''Ligne 4
        'rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Eceff") : rtf.EndCell()
        'rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString("Mpa") : rtf.EndCell()
        'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Eceff.ToString("#,##0")) : rtf.EndCell()
        'rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Module élastique effectif du béton") : rtf.EndCell() : rtf.EndRow()
        ''Ligne 4
        'rtf.BeginRow() : rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("n") : rtf.EndCell()
        'rtf.DeclareCell(700) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(aci.n.ToString("#0.00")) : rtf.EndCell()
        'rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("coef. d'équivalence Acier/Béton") : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
    End Sub
    ''' <summary>Programme de calcul des contraintes sur acier et béton en ELS</summary>
    ''' <param name="Mur">Moment Ultime résistant avec les indices (ELU_ELS, i1, j, 1) ELU_ELs = n° de combinaison, i1 = n° de travée, j=1,2 ou 3 entravée, sur appui gauche, sur appui droit, 
    ''' 1 = n° de lit, de la surface vers l'intérieur de la dalle</param>
    '''  <param name="Mmax">Moment enveloppe Maxi de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    '''  <param name="Mmin">Moment enveloppe mini de la travée avec les indices (ELU_ELS, i1, x) ELU_ELS = type d'etat Limite, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="IMmin">indice de la combinaison conduisant à la valeur maximale du moment sur appui droit de la travée i (en valeur absolue) avec les indices (ELU_ELS, i2): ELU_ELS = type d'etat Limite, i2 = n° de travée</param>
    ''' <param name="V">, V(imax, PlanCont.Count, lcMax) avec les indices (I1,I2,I3): i1= n° de combinaison de 1 à imax, i2=n° de travée de 1 à PlanCont.Count, i3 = valeur de l'abscisse en cm</param>
    '''  <param name="rtf">variable renvoyant le fichier de note de calcul sous format rtf</param>
    ''' <param name="verif"> variable prenant la valeur False si une erreur bloquante s'est produite pendant le calcul</param>
    Private Sub Calcul_section_ELS(ByVal Mmax(,,) As Double, ByVal Mmin(,,) As Double, ByVal IMmin(,) As Integer, ByVal V(,,) As Double, ByVal ΔM(,) As Double, ByRef verif As Boolean, ByRef acier_construction() As Boolean,
                                  ByRef m1d(,,) As Double, ByRef m2d(,,) As Double, ByRef m3d(,,) As Double, ByRef Mur(,,,) As Double,
                                                ByRef x1Max(,) As Integer, ByRef Ainf(,,) As Double, ByRef Modification_Acier_HA As Boolean, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        '--->ELS: vérification des aciers de flexion à l'ELS et recalcul si nécessaire
        Dim boucle, courbe_moment, i1, j, x, ELU_ELS As Integer
        Dim A1, As_ELS(PlanCont.Count, 2), As_ELU, Ifiss, Mu, MsQP, n, X1(3), y, σbc, σsc, σs, φeff As Double '
        Dim x0(3, PlanCont.Count, 2), x1min(3, PlanCont.Count) As Integer

        Dim msg1 As String = String.Empty
        ELU_ELS = 3 'Combinaison ELS caractéristiques
        'le calcul se fait en 2 passes:
        ' passe 1 = Ok, le logiciel ecrit directement la note de calcul
        ' si passe 1 = not OK:
        '   * contrainte béton trop élevé: section insuffisante le logiciel adresse un avertissement et ecrit directement la note de calcul
        '  * contrainte acier trop élevée: le logiciel redimensionne les aciers de flexion
        Modification_Acier_HA = False 'initialisation
        If PlanCont.Item(0).Prefa = True Then
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
            If PlanCont.Item(0).Prefa_Contrainte = True Then
                msg1 = "Plancher à base de prédalles: possibilité de calculer les contraintes sous inertie homogénéisé conformément à l'article 3.1 du CPT sous réserve que σfl  < fctmfl  "
            Else
                msg1 = "Plancher à base de prédalles: calcul des contraintes sous inertie fissurée."
            End If
            rtf.WriteString(msg1) : rtf.NewLine() : rtf.WriteEndGroup()
        End If
        For i1 = 1 To PlanCont.Count
            Call Decalage_moment(ELU_ELS, Mmax, Mmin, i1, courbe_moment, x0, x1Max, x1min, m1d, m2d, m3d) ', ferraillage_sup)
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.WriteString("Travée " & i1.ToString & " :") : rtf.NewLine()
            With PlanCont.Item(i1 - 1)
                For j = 1 To 3
                    Dim msg As String = String.Empty
                    boucle = 1
                    If i1 = 1 And .LappG < 0.05 Then j = 3 'la poutre continue sur sa 1ière travée comporte une console gauche
                    If i1 = PlanCont.Count And .LappD < 0.05 Then Exit For 'la poutre continue sur sa dernière travée comporte une console droite

                    Select Case j
                        Case 1 'acier inférieur longitudinal en travée
                            'Position du moment maxi en travée
                            Mu = Mmax(ELU_ELS, i1, x1Max(ELU_ELS, i1)) 'Calcul de l'acier pour le moment maxi en travée
                            MsQP = Mmax(4, i1, x1Max(ELU_ELS, i1))
                        Case 2
                            If i1 = 1 Then 'pas de calcul de contrainte sur les aciers de reprise
                                boucle = 0    'du moment 0.15xM0
                            Else
                                If i1 > 1 And PlanCont(i1 - 1).H = PlanCont(i1 - 2).H Then
                                    rtf.WriteString("Travée " & i1.ToString & "   - sur appui gauche..: Voir appui droit travée précédente") : rtf.NewLine()
                                    boucle = 0
                                Else
                                    Call Determination_Mu_Appui_Gauche(3, i1, IMmin, Mmax, Mmin, V, x1Max, Mu, ΔM) ' x, ByRef ΔM(,) As Double,ByRef M2dTab As Double)
                                    ' Mu = -m2d(ELU_ELS, i1, 0) 'Calcul de l'acier pour le moment maxi sur appui gauche
                                    Call Determination_Mu_Appui_Gauche(4, i1, IMmin, Mmax, Mmin, V, x1Max, MsQP, ΔM) ' x, ByRef ΔM(,) As Double,ByRef M2dTab As Double)
                                    ' MsQP = -m2d(4, i1, 0)
                                    x = 0
                                End If
                            End If
                        Case 3
                            x = CInt(Truncate(.Lcx * 100))
                            If i1 = PlanCont.Count Then 'pas de calcul de contrainte sur les aciers de
                                boucle = 0      'reprise du moment 0.15xM0
                            Else
                                Call Determination_Mu_Appui_Droit(3, i1, IMmin, Mmax, Mmin, V, x1Max, Mu, ΔM) ' x, ByRef (,) As Double,ByRef M2dTab As Double)
                                Call Determination_Mu_Appui_Droit(4, i1, IMmin, Mmax, Mmin, V, x1Max, MsQP, ΔM) ' x, ByRef ΔM(,) As Double,ByRef M2dTab As Double)
                            End If
                    End Select
                    If boucle = 1 Then 'Vérification à l'ELS
                        'Les diverses sections d'aciers de la poutre sont toujours calculées  en première partie à l'ELU puis sont vérifiés à l'ELS. 
                        'Si la vérification montre que la section d'acier est insuffisante, le logiciel recalcule les nouvelles sections d'aciers nécessaire
                        If .Prefa = True Then
                            n = 15
                        Else
                            Select Case aci.n_Forcer_Cont
                                Case True
                                    If aci.n_Cont < 2 Then
                                        aci.n_Cont = 15
                                        Aff.msg_Avertiss.Add("Coefficient d'équivalence n: non renseigné - Pris égal à 15 par défaut")
                                    End If
                                    n = aci.n_Cont
                                Case False
                                    Dim Eceff, φinf As Double
                                    Call Calcul_Coef_Fluage(Aff.t0, Aff.RH, Aff.tinf, φinf, rtf)
                                    φeff = φinf * (MsQP / Mu) 'suivant Recommandations Professionnelles
                                    Eceff = bet.Ecm / (1 + φeff) 'eq 7.20
                                    n = 200000 / Eceff
                            End Select
                        End If
                        'calcul des contraintes à l'ELS caractéristiques
                        Call Verification_armature_flexion_ELS(i1, j, Mu, n, y, Ifiss, σbc, σsc, σs, msg)
                        If msg <> String.Empty Then
                            If j = 1 Then msg1 = "   - en travée.........: "
                            If j = 2 Then msg1 = "   - sur appui gauche..: "
                            If j = 3 Then msg1 = "   - sur appui droit...: "
                            rtf.WriteString(msg1 & msg) : rtf.NewLine()
                        End If
                        If σs > 0.8 * aci.fyk Then
                            Calcul_armature_flexion_ELS(i1, j, n, Mu, As_ELS(i1, j - 1))
                            Select Case j
                                Case 1
                                    Select Case .Type_Acier_Travée
                                        Case 1 'TS
                                            As_ELU = aci.As_long(.TS_trav.Item(0).indTS)
                                            If .TS_trav.Count = 2 Then As_ELU = As_ELU + aci.As_long(.TS_trav.Item(1).indTS)
                                        Case 2 'barres HA
                                            As_ELU = .LitInf.Item(0).Aire / .Ly
                                            If .LitInf.Count = 2 Then As_ELU += .LitInf.Item(1).Aire / .Ly
                                    End Select
                                Case 2
                                    Select Case .Type_Acier_Travée
                                        Case 1 'TS
                                            As_ELU = aci.As_long(.TS_AppG.Item(0).indTS)
                                            If .TS_AppG.Count = 2 Then As_ELU = As_ELU + aci.As_long(.TS_AppG.Item(1).indTS)
                                        Case 2 'barres HA
                                            As_ELU = .LitSupG.Item(0).Aire / .Ly
                                            If .LitSupG.Count = 2 Then As_ELU += .LitSupG.Item(1).Aire / .Ly
                                    End Select
                                Case 3
                                    Select Case .Type_Acier_Travée
                                        Case 1 'TS
                                            As_ELU = aci.As_long(.TS_AppD.Item(0).indTS)
                                            If .TS_AppD.Count = 2 Then As_ELU = As_ELU + aci.As_long(.TS_AppD.Item(1).indTS)
                                        Case 2 'barres HA
                                            As_ELU = .LitSupD.Item(0).Aire / .Ly
                                            If .LitSupD.Count = 2 Then As_ELU += .LitSupD.Item(1).Aire / .Ly
                                    End Select
                            End Select
                            If As_ELS(i1, j - 1) > As_ELU Then
                                '  Modification_Acier_HA = True : Ainf(3, i1, j) = As_ELS
                                'nouvel assemblage d'armatures   
                                'Select Case j
                                '    Case 1
                                '        'Call Aciers_longitudinaux_par_lit_HA(ELU_ELS, i1, j, As_ELS, .dInf)
                                '        '.LitInf.Item(0).phiProj = .LitInf.Item(0).φHA : .LitInf.Item(1).phiProj = .LitInf.Item(1).φHA : .LitInf.Item(2).phiProj = .LitInf.Item(2).φHA
                                '        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                                '        rtf.WriteString(" - travée" & i1.ToString & " - armatures en travée:  As(ELU) = " & (As_ELU * 10000).ToString("#0.00") & " cm² < " &
                                '                        (As_ELS(i1, j - 1) * 10000).ToString("#0.00") & " cm² - Modification des lits d'armatures:") : rtf.NewLine()
                                '        'rtf.WriteString("    - Lit n°1: impose HA" & (.LitInf.Item(0).phiProj * 1000).ToString("#")) : rtf.NewLine()
                                '        'If .LitInf.Item(1).phiProj > 0.005 Then
                                '        '    rtf.WriteString("    - Lit n°2: impose HA" & (.LitInf.Item(1).phiProj * 1000).ToString("#")) : rtf.NewLine()
                                '        '    If .LitInf.Item(2).phiProj > 0.005 Then
                                '        '        rtf.WriteString("    - Lit n°3: impose HA" & (.LitInf.Item(2).phiProj * 1000).ToString("#")) : rtf.NewLine()
                                '        '    End If
                                '        'End If
                                '        rtf.WriteEndGroup()
                                '        msg = " - armatures en travée - "
                                '    Case 2
                                '        msg = " - armatures sur appui gauche - "
                                '        'Call Aciers_longitudinaux_par_lit_HA(1, i1, j, As_ELS, .dSupG)
                                '        '.LitSupG.Item(0).phiProj = .LitSupG.Item(0).φHA : .LitSupG.Item(1).phiProj = .LitSupG.Item(1).φHA : .LitSupG.Item(2).phiProj = .LitSupG.Item(2).φHA
                                '        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                                '        rtf.WriteString(" - travée" & i1.ToString & " - armatures sur appui gauche:  As(ELU) = " & (As_ELU * 10000).ToString("#0.00") & " cm² < " &
                                '                        (As_ELS(i1, j - 1) * 10000).ToString("#0.00") & " cm² - Modification des lits d'armatures:") : rtf.NewLine()
                                '        'rtf.WriteString("    - Lit n°1: impose HA" & (.LitSupG.Item(0).phiProj * 1000).ToString("#")) : rtf.NewLine()
                                '        'If .LitSupG.Item(1).phiProj > 0.005 Then
                                '        '    rtf.WriteString("    - Lit n°2: impose HA" & (.LitSupG.Item(1).phiProj * 1000).ToString("#")) : rtf.NewLine()
                                '        '    If .LitSupG.Item(2).phiProj > 0.005 Then
                                '        '        rtf.WriteString("    - Lit n°3: impose HA" & (.LitSupG.Item(2).phiProj * 1000).ToString("#")) : rtf.NewLine()
                                '        '    End If
                                '        'End If
                                '        rtf.WriteEndGroup()
                                '    Case 3
                                '        msg = " - armatures sur appui droit - "
                                '        'Call Aciers_longitudinaux_par_lit_HA(ELU_ELS, i1, j, As_ELS, .dSupD)
                                '        '.LitSupD.Item(0).phiProj = .LitSupD.Item(0).φHA : .LitSupD.Item(1).phiProj = .LitSupD.Item(1).φHA : .LitSupD.Item(2).phiProj = .LitSupD.Item(2).φHA
                                '        ''Call Moment_resistant_Lit_Acier(ELU_ELS, i1, j, 0, .dSupD, As_ELS, Mur) 'calcul du moment repris par chaque lit d'acier  σsd(i1, j),
                                '        ''Call Epure_lit_acier_appui_droit(verif, ELU_ELS, i1, courbe_moment, x0, x1min, 0.8 * aci.fyk, Mur, m3d, acier_construction) : If verif = False Then Exit Sub
                                '        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                                '        rtf.WriteString(" - travée" & i1.ToString & " - armatures sur appui droit:  As(ELU) = " & (As_ELU * 10000).ToString("#0.00") & " cm² < " &
                                '                         (As_ELS(i1, j - 1) * 10000).ToString("#0.00") & " cm² - Modification des lits d'armatures:") : rtf.NewLine()
                                '        'rtf.WriteString("    - Lit n°1: HA" & (.LitSupD.Item(0).phiProj * 1000).ToString("#")) : rtf.NewLine()
                                '        'If .LitSupD.Item(1).phiProj > 0.005 Then
                                '        '    rtf.WriteString("    - Lit n°2: impose HA" & (.LitSupD.Item(1).phiProj * 1000).ToString("#")) : rtf.NewLine()
                                '        '    If .LitSupD.Item(2).phiProj > 0.005 Then
                                '        '        rtf.WriteString("    - Lit n°3: impose HA" & (.LitSupD.Item(2).phiProj * 1000).ToString("#")) : rtf.NewLine()
                                '        '    End If
                                '        'End If
                                '        rtf.WriteEndGroup()
                                'End Select
                                msg = "Calcul section ELS - travée" & i1.ToString & msg & " : σs > 0,8.fyk car As(ELS) (= " & (As_ELS(i1, j - 1) * 10000.0).ToString("#0.00") & " cm²) > As(ELU)(= " &
                                     (As_ELU * 10000.0).ToString("#0.00") & " cm²)"
                                Aff.msg_Avertiss.Add(msg)
                            Else
                                Select Case j
                                    Case 1
                                        msg = " - armatures en travée - "
                                    Case 2
                                        msg = " - armatures sur appui gauche - "
                                    Case 3
                                        msg = " - armatures sur appui droit - "
                                End Select
                                msg = "Calcul section ELS - travée" & i1.ToString & msg & "incohérence dans le calcul: σs > 0,8.fyk et As(ELU)<As(ELS) - voir module de calcul Private Sub Calcul_section_ELS"
                                Aff.msg_Avertiss.Add(msg)
                            End If
                        End If
                    End If
                Next j
            End With
            rtf.WriteEndGroup()
        Next i1
        'si modification d'armature, les calculs sont automatiquement repris dans leur intégralité avec les nouvelles sections. 
        'Le logiciel va donc revenir au début du programme d'ou l'utilisation du Goto
        '  If Modification_Acier_HA = True Then Exit Sub

        'Passe 2 - Ecriture de la note de calcul
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        For i1 = 1 To PlanCont.Count
            Dim msg As String = String.Empty
            '   Call Decalage_moment(ELU_ELS, Mmax, Mmin, i1, courbe_moment, x0, x1Max, x1min, m1d, m2d, m3d) ', ferraillage_sup)
            With PlanCont.Item(i1 - 1)
                For j = 1 To 3
                    boucle = 1
                    If i1 = 1 And .LappG < 0.05 Then j = 3 'la poutre continue sur sa 1ière travée comporte une console gauche
                    ' If i1 = PoutCont.Count And .LappD = 0 Then j = 2 'la poutre continue sur sa dernière ytravée comporte une console droite
                    Select Case j
                        Case 1 'acier inférieur longitudinal en travée
                            rtf.BeginRow() : rtf.DeclareCell(8750) : rtf.BeginCell(True) : rtf.Bold = True : rtf.WriteString("Travée n°" & i1.ToString) : rtf.Bold = False : rtf.EndCell() : rtf.EndRow()
                            rtf.BeginRow() : rtf.DeclareCell(8750) : rtf.BeginCell(True) : rtf.WriteString("Vérification contraintes de flexion en travée") : rtf.EndCell() : rtf.EndRow()
                            'Position du moment maxi en travée
                            Mu = Mmax(ELU_ELS, i1, x1Max(ELU_ELS, i1)) 'Calcul de l'acier pour le moment maxi en travée
                            If Mu = 0 Then 'And frmDon.MessagesInformationToolStripMenuItem.Checked = False
                                msg = "Travée n°" & i1.ToString("##") & Environment.NewLine & "Pas de moment positif en travée. la poutre" & Environment.NewLine & "est soulevée sur toute sa longueur."
                                Aff.msg_Avertiss.Add("EC2: Sub Calcul_section_ELS - " & msg)
                                rtf.BeginRow() : rtf.DeclareCell(8750) : rtf.BeginCell(True) : rtf.WriteString("Pas de moment positif en travée. la poutre est soulevée sur toute sa longueur.") : rtf.EndCell() : rtf.EndRow()
                                'rtf.WriteEndGroup() : verif = False : Exit Sub
                            Else
                                'Ligne 2
                                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ms") : rtf.EndCell()
                                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                                rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.00")) : rtf.EndCell()
                                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("moment Maxi") : rtf.EndCell() : rtf.EndRow()
                                'Ligne 3
                                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("x") : rtf.EndCell()
                                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
                                rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString(((x1Max(ELU_ELS, i1) - .a1 * 100) / 100).ToString("#0.00")) : rtf.EndCell()
                                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("abscisse depuis nu appui gauche du moment Maxi") : rtf.EndCell() : rtf.EndRow()
                            End If
                        Case 2
                            If i1 = 1 Then 'pas de calcul de contrainte sur les aciers de reprise
                                boucle = 0    'du moment 0.15xM0
                            Else
                                rtf.BeginRow() : rtf.DeclareCell(8750) : rtf.BeginCell(True)
                                If PlanCont(i1 - 1).H = PlanCont(i1 - 2).H Then
                                    rtf.WriteString("Vérification contraintes de flexion sur appui gauche - Voir appui droit travée précédente") : rtf.EndCell() : rtf.EndRow()
                                Else
                                    'rtf.WriteString("Vérification contraintes de flexion sur appui gauche ") : rtf.EndCell() : rtf.EndRow()
                                    'Mu = -m2d(ELU_ELS, i1, 0) 'Calcul de l'acier pour le moment maxi sur appui gauche
                                    'x = 0
                                    ''Ligne 2
                                    'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ms") : rtf.EndCell()
                                    'rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                                    'rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.00")) : rtf.EndCell()
                                    'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("moment Maxi sous combinaisons caractéristiques") : rtf.EndCell() : rtf.EndRow()
                                End If
                            End If
                        Case 3
                            If i1 = 1 And .LappG <= 0.05 Then
                                rtf.BeginRow() : rtf.DeclareCell(8750) : rtf.BeginCell(True) : rtf.Bold = True : rtf.WriteString("Travée n°" & i1.ToString) : rtf.Bold = False : rtf.EndCell() : rtf.EndRow()
                            End If

                            If i1 = PlanCont.Count Then 'pas de calcul de contrainte sur les aciers de
                                boucle = 0      'reprise du moment 0.15xM0
                            Else
                                rtf.BeginRow() : rtf.DeclareCell(8750) : rtf.BeginCell(True) : rtf.WriteString("Vérification contraintes de flexion sur appui droit") : rtf.EndCell() : rtf.EndRow()
                                ' Mu = -m3d(ELU_ELS, i1, x) 'Calcul de l'acier pour le moment maxi sur appui droit
                                Call Determination_Mu_Appui_Droit(3, i1, IMmin, Mmax, Mmin, V, x1Max, Mu, ΔM, x)
                                'Ligne 2
                                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ms") : rtf.EndCell()
                                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                                rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.00")) : rtf.EndCell()
                                rtf.DeclareCell(5500) : rtf.BeginCell(True)
                                If Aff.Ecretage_Moment = False Then
                                    If PlanCont.Item(i1 - 1).App_Monolith = False Then
                                        rtf.WriteString("moment Maxi à l'axe de l'appui")
                                    Else
                                        rtf.WriteString("moment Maxi au nu de l'appui")
                                    End If
                                Else
                                    If PlanCont.Item(i1 - 1).App_Monolith = False Then
                                        rtf.WriteString("moment Maxi à l'axe de l'appui après écrêtage")
                                    Else
                                        If x = CInt(Truncate(.Lcx * 100)) Then
                                            rtf.WriteString("moment Maxi à l'axe de l'appui après écrêtage")
                                        Else
                                            rtf.WriteString("moment Maxi au nu de l'appui")
                                        End If
                                    End If
                                End If

                                rtf.EndCell() : rtf.EndRow()
                            End If
                    End Select
                    If boucle = 1 Then 'Vérification à l'ELS
                        'Les diverses sections d'aciers de la poutre sont toujours calculées  en première partie à l'ELU puis sont vérifiés à l'ELS. 
                        'Si la vérification montre que la section d'acier est insuffisante, le logiciel recalcule les nouvelles sections d'aciers nécessaire
                        Dim I_non_Fiss As Boolean = False
                        Call Verification_armature_flexion_ELS(i1, j, Mu, n, y, Ifiss, σbc, σsc, σs,,,,, I_non_Fiss)
                        Select Case j
                            Case 1
                                'Ligne 4
                                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("d") : rtf.EndCell()
                                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
                                rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((.dInf * 100).ToString("#,##0.0")) : rtf.EndCell()
                                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("hauteur utile") : rtf.EndCell() : rtf.EndRow()
                                'Ligne 5
                                Select Case .Type_Acier_Travée
                                    Case 1 'TS
                                        A1 = aci.As_long(.TS_trav.Item(0).indTS)
                                        If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                                    Case 2 'barres HA
                                        A1 = (.LitInf.Item(0).Aire) / .Ly
                                        If .LitInf.Count = 2 Then A1 += (.LitInf.Item(1).Aire) / .Ly
                                End Select
                                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As") : rtf.EndCell()
                                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                                rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((A1 * 10000).ToString("#,##0.00")) : rtf.EndCell()
                                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier répartie  ELU armature inférieure") : rtf.EndCell() : rtf.EndRow()
                            Case 2
                                If PlanCont(i1 - 1).H = PlanCont(i1 - 2).H Then
                                    'aucun affichage - Voir appui droit travée précédente 
                                    Continue For
                                Else
                                    'Ligne 4
                                    rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("d") : rtf.EndCell()
                                    rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
                                    rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((.dSupG * 100).ToString("#,##0.0")) : rtf.EndCell()
                                    rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("hauteur utile") : rtf.EndCell() : rtf.EndRow()
                                    'Ligne 5
                                    Select Case .Type_Acier_Travée
                                        Case 1 'TS
                                            A1 = aci.As_long(.TS_AppG.Item(0).indTS)
                                            If .TS_AppG.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppG.Item(1).indTS)
                                        Case 2 'barres HA
                                            A1 = (.LitSupG.Item(0).Aire) / .Ly
                                            If .LitSupG.Count = 2 Then A1 += (.LitSupG.Item(1).Aire) / .Ly
                                    End Select
                                    rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As") : rtf.EndCell()
                                    rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                                    rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((A1 * 10000).ToString("#,##0.00")) : rtf.EndCell()
                                    rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier répartie ELU armature supérieure") : rtf.EndCell() : rtf.EndRow()
                                End If

                            Case 3
                                'Ligne 4
                                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("d") : rtf.EndCell()
                                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
                                rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((.dSupD * 100).ToString("#,##0.0")) : rtf.EndCell()
                                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("hauteur utile") : rtf.EndCell() : rtf.EndRow()
                                'Ligne 5
                                Select Case .Type_Acier_Travée
                                    Case 1 'TS
                                        A1 = aci.As_long(.TS_AppD.Item(0).indTS)
                                        If .TS_AppD.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppD.Item(1).indTS)
                                    Case 2 'barres HA
                                        A1 = (.LitSupD.Item(0).Aire) / .Ly
                                        If .LitSupD.Count = 2 Then A1 += (.LitSupD.Item(1).Aire) / .Ly
                                End Select
                                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As") : rtf.EndCell()
                                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                                rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((A1 * 10000).ToString("#,##0.00")) : rtf.EndCell()
                                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier répartie ELU armature supérieure") : rtf.EndCell() : rtf.EndRow()
                        End Select
                        'Ligne 6
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("y") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
                        rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((y * 100).ToString("#0.0")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("axe neutre") : rtf.EndCell() : rtf.EndRow()
                        'Ligne 7
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("If") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("dm4") : rtf.EndCell()
                        rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((Ifiss * 10000).ToString("#0.0000")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True)
                        If I_non_Fiss = True Then
                            rtf.WriteString("inertie de section non fissurée") : rtf.EndCell() : rtf.EndRow()
                        Else
                            rtf.WriteString("inertie de section fissurée") : rtf.EndCell() : rtf.EndRow()
                        End If

                        'Ligne 8
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("σb") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
                        rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString(σbc.ToString("#0.00 ")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True)
                        If σbc > 0.6 * bet.fck Then
                            If .XD1 = True Or .XD2 = True Or .XD3 = True Or .XS1 = True Or .XS2 = True Or .XS3 = True Or .XF1 = True Or .XF2 = True Or .XF3 = True _
                                 Or .XF4 = True Then
                                rtf.WriteString("contrainte béton - σb > 0,6.fck = " & (0.6 * bet.fck).ToString("#0.00") & " MPa - NS") : rtf.EndCell() : rtf.EndRow()
                                msg = "Vérification des contraintes à l'ELS - Travée " & i1.ToString & " : contrainte béton trop forte, modifiez la section."
                                Aff.msg_Avertiss.Add(msg)
                            Else
                                rtf.WriteString("contrainte béton - σb > 0,6.fck = " & (0.6 * bet.fck).ToString("#0.00") & " MPa ") : rtf.EndCell() : rtf.EndRow()
                            End If
                        Else
                            rtf.WriteString("contrainte béton - σb < 0,6.fck = " & (0.6 * bet.fck).ToString("#0.00") & " MPa - OK") : rtf.EndCell() : rtf.EndRow()
                        End If
                        'Ligne 9
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("σs") : rtf.EndCell()
                        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
                        rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString(σs.ToString("#0.00 ")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True)
                        If σs > 0.8 * aci.fyk Then
                            rtf.WriteString("σ acier - σs > 0,8.fyk = " & (0.8 * aci.fyk).ToString("#0.0") & " MPa - NS (As min = " & (As_ELS(i1, j - 1) * 10000).ToString("#0.00") & " cm²)") : rtf.EndCell() : rtf.EndRow()
                        Else
                            rtf.WriteString("σ acier - σs < 0,8.fyk = " & (0.8 * aci.fyk).ToString("#0.0") & " MPa - OK") : rtf.EndCell() : rtf.EndRow()
                        End If
                    End If
                Next j
            End With
        Next i1
        rtf.WriteEndGroup()
    End Sub
    Private Sub Interception(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal ikk As Integer, ByVal appui As Integer, ByVal Mur(,,,) As Double,
                            ByVal M1d(,,) As Double, ByRef x3 As Integer, ByRef x4 As Integer)
        '--------------------------------------------------------------------------
        'Vérification de non-interception entre la courbe des moments et le diagramme enveloppe des moments résistants
        '--------------------------------------------------------------------------
        'Variables:
        'ELU_ELS =  =1 fondamental,  = 2 Accidentel, ...
        'i1 = n° de travée
        'ikk = n° de lit
        'Appui: = 1 appui gauche, =2 appui droit
        'Mur = Moment ultime résistant
        Dim Nint, Nint1, x As Integer
        Dim murr As Double
        'lit d'acier en travée
        With PlanCont.Item(i1 - 1)
            If appui = 1 Then 'vérification de non-interception sur le côté gauche de la courbe
                x4 = 0 : x3 = 0
                Select Case .Type_Acier_Travée
                    Case 1
                        Nint = CInt(Truncate(.TS_trav.Item(ikk - 1).X1acier * 100))
                        Nint1 = CInt(Truncate(.TS_trav.Item(ikk - 1).X1acierLS * 100))
                    Case 2
                        Nint = CInt(Truncate(.LitInf.Item(ikk - 1).X1acier * 100))
                        Nint1 = CInt(Truncate(.LitInf.Item(ikk - 1).X1acierLS * 100))
                End Select

                For x = Nint To Nint1 Step 2
                    murr = (Mur(ELU_ELS, i1, 1, ikk) - Mur(ELU_ELS, i1, 1, ikk - 1)) / (Nint1 - Nint)
                    murr = murr * (x - Nint) + Mur(ELU_ELS, i1, 1, ikk - 1)
                    If Abs(murr) < 0.000000000000001 Then murr = 0
                    If murr < M1d(ELU_ELS, i1, x) Then
                        x3 = x
                        Do
                            x3 = x3 - 1
                        Loop While murr < M1d(ELU_ELS, i1, x3)
                        x3 = x - x3
                        If x3 > x4 Then x4 = x3
                    End If
                Next x
            Else 'vérification de non-interception sur le côté droit de la courbe
                x4 = 0 : x3 = 0
                Select Case .Type_Acier_Travée
                    Case 1
                        Nint = CInt(Truncate(.TS_trav.Item(ikk - 1).X2acierLS * 100))
                        Nint1 = CInt(Truncate(.TS_trav.Item(ikk - 1).X2acier * 100))
                    Case 2
                        Nint = CInt(Truncate(.LitInf.Item(ikk - 1).X2acierLS * 100))
                        Nint1 = CInt(Truncate(.LitInf.Item(ikk - 1).X2acier * 100))
                End Select

                For x = Nint To Nint1 Step 2
                    murr = (Mur(ELU_ELS, i1, 1, ikk) - Mur(ELU_ELS, i1, 1, ikk - 1)) / (Nint1 - Nint)
                    murr = Mur(ELU_ELS, i1, 1, ikk) - murr * (x - Nint)
                    If Abs(murr) < 0.000000000000001 Then murr = 0
                    If murr < M1d(ELU_ELS, i1, x) Then
                        x3 = x
                        Do
                            x3 = x3 + 1
                        Loop While murr < M1d(ELU_ELS, i1, x3)
                        x3 = x3 - x
                        If x3 > x4 Then x4 = x3
                    End If
                Next x
            End If
        End With

    End Sub
    Private Sub Interception_appui_Gauche(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal ikk As Integer, ByVal Mur(,,,) As Double,
                         ByVal M2d(,,) As Double, ByRef x3 As Integer, ByRef x4 As Integer)
        '--------------------------------------------------------------------------
        'Vérification de non-interception entre la courbe des moments et le diagramme enveloppe des moments résistants
        '--------------------------------------------------------------------------
        'Variables:
        'ELU_ELS =  =1 fondamental,  = 2 Accidentel, ...
        'i1 = n° de travée
        'j = type d'acier, entravée, sur appui gauche, sur appui droit
        'ikk = n° de lit
        'Mur = Moment ultime résistant
        Dim Nint, Nint1, x As Integer
        Dim murr As Double
        'lit d'acier sur appui gauche
        x4 = 0 : x3 = 0
        With PlanCont.Item(i1 - 1)
            Select Case .Type_Acier_Appui_Gauche
                Case 1
                    Nint = CInt(.TS_AppG.Item(ikk - 1).X2acierLS * 100) ': Nint = Nint \ 1
                    Nint1 = CInt(.TS_AppG.Item(ikk - 1).X2acier * 100)
                Case 2
                    Nint = CInt(.LitSupG.Item(ikk - 1).X2acierLS * 100) ': Nint = Nint \ 1
                    Nint1 = CInt(.LitSupG.Item(ikk - 1).X2acier * 100) ': Nint1 = Nint1 \ 1
            End Select
        End With
        For x = Nint To Nint1 Step 2
            murr = (Mur(ELU_ELS, i1, 2, ikk) - Mur(ELU_ELS, i1, 2, ikk - 1)) / (Nint1 - Nint)
            murr = Mur(ELU_ELS, i1, 2, ikk) - murr * (x - Nint)
            If Abs(murr) < 0.000000000000001 Then murr = 0
            If murr < -M2d(ELU_ELS, i1, x) Then
                x3 = x
                Do
                    x3 = x3 + 1
                Loop While murr < -M2d(ELU_ELS, i1, x3)
                x3 = x3 - x
                If x3 > x4 Then x4 = x3
            End If
        Next x
    End Sub
    Private Sub Interception_appui_Droit(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal ikk As Integer, ByVal Mur(,,,) As Double,
                             ByVal M3d(,,) As Double, ByRef x3 As Integer, ByRef x4 As Integer)
        '--------------------------------------------------------------------------
        'Vérification de non-interception entre la courbe des moments et le diagramme enveloppe des moments résistants
        '--------------------------------------------------------------------------
        'Variables:
        'ELU_ELS =  =1 fondamental,  = 2 Accidentel, ...
        'i1 = n° de travée
        'j = type d'acier, entravée, sur appui gauche, sur appui droit
        'ikk = n° de lit
        'Mur = Moment ultime résistant
        Dim Nint, Nint1, x As Integer
        Dim murr As Double
        'lit d'acier sur appui droit
        x4 = 0 : x3 = 0
        With PlanCont.Item(i1 - 1)
            Select Case .Type_Acier_Appui_Droit
                Case 1
                    Nint = CInt(.TS_AppD.Item(ikk - 1).X1acier * 100) ' : Nint = Nint \ 1
                    Nint1 = CInt(.TS_AppD.Item(ikk - 1).X1acierLS * 100) ' : Nint1 = Nint1 \ 1
                Case 2
                    Nint = CInt(.LitSupD.Item(ikk - 1).X1acier * 100) ' : Nint = Nint \ 1
                    Nint1 = CInt(.LitSupD.Item(ikk - 1).X1acierLS * 100) ' : Nint1 = Nint1 \ 1
            End Select
        End With
        For x = Nint To Nint1 Step 2
            murr = (Mur(ELU_ELS, i1, 3, ikk) - Mur(ELU_ELS, i1, 3, ikk - 1)) / (Nint1 - Nint)
            murr = murr * (x - Nint) + Mur(ELU_ELS, i1, 3, ikk - 1)
            If Abs(murr) < 0.000000000000001 Then murr = 0
            If murr < -M3d(ELU_ELS, i1, x) Then
                x3 = x
                Do
                    x3 = x3 - 1
                Loop While murr < -M3d(ELU_ELS, i1, x3)
                x3 = x - x3
                If x3 > x4 Then x4 = x3
            End If
        Next x

    End Sub
    Private Sub Moment_resistant_Lit_Acier(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal j As Integer, ByVal Mu As Double, ByVal d As Double, ByVal Ainf As Double,
                                 ByRef MuR(,,,) As Double)
        '---------------------------------------------------------------------------------
        ' Calcul des moment résistants par lit d'acier  -  ELU  ELS
        '-------------------------------------------------------------------------------
        Dim u1, u2, Cinf, MrELU As Double
        Dim i As Integer
        u1 = 0 : u2 = 0
        With PlanCont.Item(i1 - 1)
            Select Case j
                Case 1
                    If .Type_Acier_Travée = 1 Then
                        u1 = aci.As_long(.TS_trav.Item(0).indTS)
                        If .TS_trav.Count = 2 Then u2 = aci.As_long(.TS_trav.Item(1).indTS)
                        i = .TS_trav.Count
                    Else
                        u1 = .LitInf.Item(0).Aire / .Ly : u2 = 0 : i = .LitInf.Count
                    End If
                Case 2
                    If .Type_Acier_Appui_Gauche = 1 Then
                        u1 = aci.As_long(.TS_AppG.Item(0).indTS)
                        If .TS_AppG.Count = 2 Then u2 = aci.As_long(.TS_AppG.Item(1).indTS)
                        i = .TS_AppG.Count
                    Else
                        u1 = .LitSupG.Item(0).Aire / .Ly : u2 = 0 : i = .LitSupG.Count
                    End If

                Case 3
                    If .Type_Acier_Appui_Droit = 1 Then
                        u1 = aci.As_long(.TS_AppD.Item(0).indTS)
                        If .TS_AppD.Count = 2 Then u2 = aci.As_long(.TS_AppD.Item(1).indTS)
                        i = .TS_AppD.Count
                    Else
                        u1 = .LitSupD.Item(0).Aire / .Ly : u2 = 0 : i = .LitSupD.Count
                    End If

            End Select
        End With
        If Abs(Mu) < 0.000001 Then 'si Mu est nul = calcul à l'ELS
            'Utilisation de cette partie de l'algorithme lors d'une augmentation de la section d'acier après calcul à l'ELS si la contrainte sur l'armature
            'dépasse 0,8.fyd
            'calcul du moment résistant d'une poutre rectangulaire: utilisation de la méthode présentée par Mr Paillé "Calcul des structures en béton" page 177
            u1 = 0 : u2 = 0
            With PlanCont.Item(i1 - 1)
                Select Case j
                    Case 1
                        For i = 1 To 4 : MuR(1, i1, j, i) = 0 : Next 'ré-initialisation
                        Cinf = .H - .dInf
                    Case 2
                        Cinf = .H - .dSupG
                    Case 3
                        Cinf = .H - .dSupD
                End Select
                Select Case i
                    Case 1 '1 lit
                        Call Moment_Resistant(1, .H, Cinf, u1, MrELU)
                        MuR(1, i1, j, 1) = MrELU
                        MuR(1, i1, j, 4) = MrELU
                    Case 2 '2 lits
                        Call Moment_Resistant(1, .H, Cinf, u1 + u2, MrELU)
                        MuR(1, i1, j, 4) = MrELU
                        MuR(1, i1, j, 2) = MuR(1, i1, j, 4)
                        MuR(1, i1, j, 1) = MuR(1, i1, j, 4) * (u1) / (u1 + u2)
                End Select
            End With
            'Attention: il y aura peut-être un pb d'harmonisation de barres si j=2 ou j=3 - A voir en test
        Else
            '
            'modification des lits d'aciers dans le cas de lits imposés par le projeteur
            'If ELU_ELS = 3 And phiProj(i1, j, 1) > 0.0001 Then
            '    If u1 > Ainf(ELU_ELS, i1, j) Then
            '        phiProj(i1, j, 2) = 0 : nbar(i1, j, 2) = 0 : phiProj(i1, j, 3) = 0 : nbar(i1, j, 3) = 0:      inf(i1, j) = 1 : u2 = 0 : u3 = 0
            '        Dim F_ainf, F_i1, F_S As String
            '        F_ainf = Format(Ainf(ELU_ELS, i1, j) * 10000, "###0.0"): F_i1 = Format(i1, "##"):  F_S = Format(u1 * 10000, "###0.0")
            '        msg = "Travée " & F_i1 & msg1 & vbCrLf & _
            '        "la section d'acier du premier lit est suffisante " & vbCrLf &    F_S & " cm² pour " & F_ainf & " cm² requis" & vbCrLf & _
            '            "Le nombre de lit est ramené à 1."
            '        MsgBox(msg, vbOKOnly Or MsgBoxStyle.Information, "Module BAEL: moment résistant")
            '    ElseIf u1 + u2 > Ainf(ELU_ELS, i1, j) Then
            '        phiProj(i1, j, 3) = 0 : nbar(i1, j, 3) = 0:     inf(i1, j) = 2 : u3 = 0
            '        Dim F_ainf, F_i1, F_S As String
            '        F_ainf = Format(Ainf(ELU_ELS, i1, j) * 10000, "###0.0"):  F_i1 = Format(i1, "##"):F_S = Format((u1 + u2) * 10000, "###0.0")
            '        msg = "Travée " & F_i1 & msg1 & vbCrLf & _
            '        "la section d'acier du premier et deuxième lit est suffisante " & vbCrLf &       F_S & " cm² pour " & F_ainf & " cm² requis" & vbCrLf & _
            '            "Le nombre de lit est ramené à 2."
            '        MsgBox(msg, vbOKOnly Or MsgBoxStyle.Information, "Module BAEL: moment résistant")
            '    End If
            'End If
            MuR(ELU_ELS, i1, j, 4) = Mu * (u1 + u2) / Ainf  'j=en travée ou sur appui
            Select Case i 'inf(i1, j)
                Case 1
                    MuR(ELU_ELS, i1, j, 3) = 0
                    MuR(ELU_ELS, i1, j, 2) = 0
                    MuR(ELU_ELS, i1, j, 1) = MuR(ELU_ELS, i1, j, 4)
                Case 2
                    MuR(ELU_ELS, i1, j, 3) = 0
                    MuR(ELU_ELS, i1, j, 2) = Mu * (u1 + u2) / Ainf
                    MuR(ELU_ELS, i1, j, 1) = Mu * (u1) / Ainf
            End Select
        End If
    End Sub
    ''' <summary>Calcul du moment résistant par algorithme numérique</summary>
    ''' <param name="i1">n° de travée - 1</param>
    ''' <param name="Mode">indique le branchement: 3 = calcul étaiement</param>
    Private Sub Moment_Resistant(ByVal bw As Double, ByVal h As Double, ByVal Cinf As Double, ByVal Ainf As Double, ByRef MrELU As Double,
                                       Optional ByVal Csup As Double = 0, Optional ByVal Asup As Double = 0, Optional ByVal i1 As Integer = 0, Optional ByVal Mode As Integer = 0) ' ByRef note_de_calcul As String)
        Dim d, fcd, fyd, y, y1, ymin, MrELUmin As Double
        Dim σsc, σs, σc, α, εs, εsc, εc, Δ, Δmin As Double
        Dim tour As Integer = 0
        fcd = bet.fck / bet.γcd : fyd = aci.fyk / aci.γsd
        'fcd = 15.3 pour l'exemple Paillé
        σsc = fyd : σs = fyd : σc = fcd 'valeurs de départ
        d = h - Cinf
        y = (Ainf * σs - Asup * σsc) / (0.8 * bw * σc)
        y1 = y : Δmin = 1
        Do
            tour += 1 'incrémentation
            y = (y + y1) / 2
            α = y / d
            εc = 0.0035
            Select Case aci.classe
                Case 1 'A
                    εs = 0.0225
                    If α > 0.135 Then
                        'Pivot B
                        εsc = εc * (y - Csup) / (d - y)
                        εs = εc * (1 - α) / α
                    Else
                        'Pivot A
                        εsc = εs * (y - Csup) / (d - y)
                    End If
                    If εsc > 0.00217 Then
                        σsc = fyd + 1111 * (εsc - 0.00217)
                    Else
                        σsc = 200000 * εsc
                    End If
                    If εs > 0.00217 Then
                        σs = fyd + 1111 * (εs - 0.00217)
                    Else
                        σs = 200000 * εs
                    End If
                Case 2 'B
                    εs = 0.045
                    If α > 0.072 Then
                        'Pivot B
                        εsc = εc * (y - Csup) / (d - y)
                        εs = εc * (1 - α) / α
                    Else
                        'Pivot A
                        εsc = εs * (y - Csup) / (d - y)
                    End If
                    If εsc > 0.00217 Then
                        σsc = fyd + 842 * (εsc - 0.00217)
                    Else
                        σsc = 200000 * εsc
                    End If
                    If εs > 0.00217 Then
                        σs = fyd + 1111 * (εs - 0.00217)
                    Else
                        σs = 200000 * εs
                    End If
                Case 3
                    εs = 0.0675
                    If α > 0.0493 Then
                        'Pivot B
                        εsc = εc * (y - Csup) / (d - y)
                    Else
                        'Pivot A
                        εsc = εs * (y - Csup) / (d - y)
                    End If
            End Select
            y1 = (Ainf * σs - Asup * σsc) / (0.8 * bw * σc)
            Δ = Abs(y - y1) / y1
            If Δ < Δmin Then
                Δmin = Δ
                ymin = (y + y1) / 2
                MrELUmin = 0.8 * bw * y * σc * (d - 0.4 * y) + Asup * σsc * (d - Csup)
            End If
            'Pour éviter de boucler si la précision n'arrive pas à être atteinte
            If tour = 100 Then
                Dim msg As String = String.Empty
                Select Case Mode
                    Case 1
                    Case 2
                    Case 3
                        msg = "Vérification etaiement - travée n°" & i1.ToString & " : calcul du moment résistant - sortie de l'algorithme à 100 itération car précision non atteinte -  Δ = " & (Δ * 100).ToString("#0.00") & "%." & Environment.NewLine &
                              "   - meilleure précision atteinte :  Δ = " & (Δmin * 100).ToString("#0.00") & "%." & Environment.NewLine &
                              "   -  Y (axe neutre) = " & (ymin * 100).ToString("#0.0") & " cm." & vbCrLf &
                              "   -  Mt résistant ELU = " & (MrELUmin * 1000).ToString("#0.00") & " kN.m."
                End Select
                Aff.msg_Avertiss.Add(msg)
                Exit Do
            End If
        Loop Until Δ < 0.01 'précision de 1%
        y = (y + y1) / 2
        MrELU = 0.8 * bw * y * σc * (d - 0.4 * y) + Asup * σsc * (d - Csup)
    End Sub
    Private Sub Calcul_armature_flexion_ELS(ByVal i1 As Integer, ByVal j As Integer, ByVal n As Double, ByVal Mu As Double, ByRef As_ELS As Double)
        '-------------------------------------------------------------------------------
        '               Flexion ELS - calcul des armatures
        ' Méthode développée par J.PERCHAT et J.ROUX "Pratique du BAEL 91" pages 131 et
        ' suivantes et H.THONIER "Le projet de béton armé" pour la méthode itérative
        '-------------------------------------------------------------------------------

        Dim alpha1, Mrb, sigmaB, Zb, di As Double
        Dim msg, msg1 As String
        sigmaB = 0.6 * bet.fck        'ELS limitation contr. béton
        With PlanCont.Item(i1 - 1)
            'section rectangulaire
            alpha1 = sigmaB / (0.8 * aci.fyk / n + sigmaB)
            Select Case j
                Case 1
                    di = .dInf
                Case 2
                    di = .dSupG
                Case 3
                    di = .dSupD
            End Select
            Mrb = 1 * di ^ 2 * sigmaB * alpha1 * (1 - alpha1 / 3) / 2
            If Mu > (Mrb * 1.01) Then
                If j = 1 Then msg1 = "en travée: " : If j = 2 Then msg1 = "sur appui gauche: " : If j = 3 Then msg1 = "sur appui droit: "
                msg = "ELS - Vérification des contraintes - Travée n°" & i1.ToString & " - moment " & msg1 & "section de poutre insuffisante pour équilibrer le moment ! " &
                        "Modifiez la géométrie de la poutre."
                Aff.msg_Avertiss.Add(msg)
                Mu = 0 : Zb = 1
            Else
                Zb = di * (1 - alpha1 / 3) ' méthode approchée
                '     ß = mu / (b(i1) * d(i1, j) ^ 2) 'méthode itérative
                '     alpha = 0
                '     Do
                '       alpha1 = alpha
                '       alpha = (90 * ß / ss(i1)) * (1 - alpha) / (3 - alpha)
                '       alpha = Sqr(alpha)
                '     Loop While Abs(alpha1 - alpha) > 0.0001
                '     Zb = d(i1, j) * (1 - alpha / 3)
            End If
            As_ELS = Mu / (Zb * 0.8 * aci.fyk)

        End With

    End Sub
    Private Sub Point_intersection_moment_negatif(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal j As Integer, ByVal ikk As Integer, ByVal MuR(,,,) As Double,
                                                ByVal courbe_moment As Integer, ByVal x0(,,) As Integer, ByVal x1Min(,) As Integer, ByVal Mid(,,) As Double,
                                                  ByRef Xmur As Double)
        '--------------------------------------------------------------------------
        'Recherche du point d'intersection Moment négatif
        '--------------------------------------------------------------------------
        'Variables: i1 = n° de travée
        '           j = position de l'acier
        '           ikk = n° de l'acier soit 2 ou 3, le lit n°1 n'étant pas concerné car si la poutre ne comporte qu'un seul lit, ce
        '           dernier doit être la courbe enveloppe du moment des efforts
        '           k = indice de valeur 1 ou 2 indiquant de quel côté doit être chercher la valeur Xmur
        '           Xmur = point d'intersection entre la courbe enveloppe des moments agissants et celle du moment résistant du lit ikk
        Dim xx, Nint As Integer
        With PlanCont.Item(i1 - 1)
            Select Case j
                Case 2
                    If courbe_moment = 1 Then x0(ELU_ELS, i1, 1) = x1Min(ELU_ELS, i1)
                    For xx = 0 To x0(ELU_ELS, i1, 1) Step 1
                        If MuR(ELU_ELS, i1, j, ikk - 1) > -Mid(ELU_ELS, i1, xx) Then 'Mid = M2d
                            Xmur = xx / 100 'x2mur
                            Exit Sub
                        End If
                    Next xx
                    Xmur = .al + 0.01 'x2mur
                Case 3
                    If courbe_moment = 1 Then x0(ELU_ELS, i1, 2) = x1Min(ELU_ELS, i1)
                    Nint = CInt(Truncate((.a1 + .Lcx + .a2) * 100)) ': Nint = Nint \ 1
                    For xx = x0(ELU_ELS, i1, 2) To Nint Step 1
                        If MuR(ELU_ELS, i1, j, ikk - 1) < -Mid(ELU_ELS, i1, xx) Then 'M3d = M2d
                            Xmur = (xx - 1) / 100 'x1mur
                            Exit Sub
                        End If
                    Next xx
                    Xmur = (.a1 + .Lcx + .a2) - .al - 0.01  'x1mur
            End Select
        End With
        Erase Mid
    End Sub
    Private Sub Point_intersection_moment_positif(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal ikk As Integer,
                                                ByVal appui As Integer, ByVal MuR(,,,) As Double, ByVal m1d(,,) As Double, ByRef Xmur As Double)
        '--------------------------------------------------------------------------
        'Recherche du point d'intersection Moment positif
        '--------------------------------------------------------------------------
        'Variables: i1 = n° de travée
        '           j = position de l'acier
        '           ikk = n° de l'acier soit 2 ou 3, le lit n°1 n'étant pas concerné car si la poutre ne comporte qu'un seul lit, ce
        '           dernier doit être la courbe enveloppe du moment des efforts
        '           appui = indice de valeur 1 ou 2 indiquant de quel côté doit être chercher la valeur Xmur
        '           Xmur = point d'intersection entre la courbe enveloppe des moments agissants et celle du moment résistant du lit ikk
        Dim Nint, xx, k0 As Integer
        k0 = 1 ' demarrage de la recherche depuis l'appui gauche
        Nint = CInt(Truncate(PlanCont.Item(i1 - 1).Lcx * 100))
        If appui = 1 Then Xmur = 0 Else Xmur = Nint / 100 'initialisation des variables 
        For xx = 0 To Nint Step 1
            Select Case k0  '
                Case 1 'recherche de la première interception
                    If MuR(ELU_ELS, i1, 1, ikk - 1) < m1d(ELU_ELS, i1, xx) Then
                        Xmur = (xx - 1) / 100
                        If appui = 1 Then
                            If Xmur < 0 Then Xmur = 0 'pour éviter de renvoyer une valeur négative
                            Exit Sub
                        End If

                        k0 = 2
                    End If
                Case 2 'recherche de la deuxième interception
                    If MuR(ELU_ELS, i1, 1, ikk - 1) > m1d(ELU_ELS, i1, xx) Then
                        Xmur = xx / 100
                        Exit Sub
                    End If
            End Select
        Next xx
    End Sub
    ''' <summary>Flexion ELS - Verification des contraintes par le calcul du moment d'inertie - Méthode développée par J.PERCHAT et J.ROUX "Pratique du BAEL 91" pages 141 et suivantes</summary>
    ''' <param name="i1">n° de travée</param>
    ''' <param name="j">j = position de l'acier, 1 = en partie inférieure, 2 = appui gauche, 3=appui droit</param>
    ''' <param name="Izz">inertie de la section (homogénéisé ou homogénéisé réduite)</param>
    ''' <param name="I_non_Fiss">prend la valeur True si inertie de la section non fissurée</param>
    Private Sub Verification_armature_flexion_ELS(ByVal i1 As Integer, ByVal j As Integer, ByVal Mu As Double, ByVal n As Double, ByRef Optional y1 As Double = 0, ByRef Optional Izz As Double = 0,
                                               ByRef Optional σbc As Double = 0, ByRef Optional σsc As Double = 0, ByRef Optional σs As Double = 0, ByRef Optional msg As String = "",
                                                  ByRef Optional fctmfl As Double = 0, ByRef Optional σfl As Double = 0, ByVal Optional Orientation As Integer = 1, ByRef Optional I_non_Fiss As Boolean = False)
        '------------------------------------------------------------------------------
        ' Flexion ELS - Verification des contraintes par le calcul du moment d'inertie
        ' Méthode développée par J.PERCHAT et J.ROUX "Pratique du BAEL 91" pages 141 et
        ' suivantes
        '---------------------------------------------------------------------
        'Variables: i1 = n° de travée
        '            j = position de l'acier, 1 = en partie inférieure, 

        Dim A1, A2, c, d, delta, XX1, XX2, XX3 As Double 'bb, bb0,hh0,
        Dim Calcul_modele As Integer = 1
        A2 = 0   ' ama(sup(i1, j), i1, j) le logiciel ne comprends pas le module de calcul avec des aciers comprimés
        A1 = 0
        With PlanCont.Item(i1 - 1)
            Select Case j
                Case 1
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            If Orientation = 1 Then
                                A1 = aci.As_long(.TS_trav.Item(0).indTS)
                                If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                            Else
                                A1 = aci.As_trans(.TS_trav.Item(0).indTS)
                                If .TS_trav.Count = 2 Then A1 = A1 + aci.As_trans(.TS_trav.Item(1).indTS)
                            End If
                        Case 2 'barres HA
                            If Orientation = 1 Then
                                If .LitInf.Count = 2 Then A1 = (.LitInf.Item(0).Aire + .LitInf.Item(1).Aire) / .Ly Else A1 = .LitInf.Item(0).Aire / .Ly
                            Else
                                If .LitInf_Y.Count = 2 Then A1 = (.LitInf_Y.Item(0).Aire + .LitInf_Y.Item(1).Aire) / .Lx Else A1 = .LitInf_Y.Item(0).Aire / .Lx
                            End If
                    End Select
                    d = .dInf
                Case 2
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1 'TS
                            A1 = aci.As_long(.TS_AppG.Item(0).indTS)
                            If .TS_AppG.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppG.Item(1).indTS)
                        Case 2 'barres HA
                            If .LitSupG.Count = 2 Then A1 = (.LitSupG.Item(0).Aire + .LitSupG.Item(1).Aire) / .Ly Else A1 = .LitSupG.Item(0).Aire / .Ly
                    End Select
                    d = .dSupG
                Case 3
                    Select Case .Type_Acier_Appui_Droit
                        Case 1 'TS
                            A1 = aci.As_long(.TS_AppD.Item(0).indTS)
                            If .TS_AppD.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppD.Item(1).indTS)
                        Case 2 'barres HA
                            If .LitSupD.Count = 2 Then A1 = (.LitSupD.Item(0).Aire + .LitSupD.Item(1).Aire) / .Ly Else A1 = .LitSupD.Item(0).Aire / .Ly
                    End Select
                    d = .dSupD
            End Select
            If A2 < 0.00001 Then c = 0 'Pas de calcul avec acier comprimé
            ' section rectangulaire
            If .Prefa = True Then
                If .Prefa_Contrainte = True Then
                    'calcul Inertie Section homogénéisée
                    y1 = 1 * .H ^ 2 / 2 + n * (A2 * c + A1 * d)
                    y1 = y1 / (1 * .H + n * (A2 + A1))
                    Izz = 1 * .H ^ 3 / 12 + 1 * .H * (.H / 2 - y1) ^ 2 + n * (A2 * (y1 - c) ^ 2 + A1 * (d - y1) ^ 2) 'Inertie Section homogénéisée
                    σfl = Mu / (Izz / (.H - y1))
                    fctmfl = Max((1.6 - .H / 1000) * bet.fctm, bet.fctm)
                    If σfl > fctmfl Then
                        Calcul_modele = 1
                        msg = "σfl (= " & σfl.ToString("#0.00") & "MPa) > fctmfl (= " & fctmfl.ToString("#0.00") & "MPa) - NS"
                    Else
                        msg = "σfl (= " & σfl.ToString("#0.00") & "MPa) < fctmfl (= " & fctmfl.ToString("#0.00") & "MPa) - OK"
                        σbc = Mu * y1 / Izz
                        σs = n * Mu * (d - y1) / Izz
                        Calcul_modele = 2
                        I_non_Fiss = True
                    End If
                Else
                    Calcul_modele = 1
                End If
            End If
            If Calcul_modele = 1 Then
                I_non_Fiss = False
                XX1 = 1 / 2
                XX2 = n * (A1 + A2)
                XX3 = -n * (A1 * d + A2 * c)
                delta = XX2 ^ 2 - 4 * XX1 * XX3 : delta = Sqrt(delta)
                y1 = (-XX2 + delta) / (2 * XX1)
                Izz = 1 * y1 ^ 3 / 3 + n * A2 * (y1 - c) ^ 2 + n * A1 * (d - y1) ^ 2 'Inertie Section homogénéisée réduite
                σbc = Mu * y1 / Izz
                If A2 > 0.0001 Then σsc = n * Mu * (y1 - c) / Izz
                σs = n * Mu * (d - y1) / Izz
            End If
        End With
    End Sub
    Private Sub Decalage_moment(ByVal ELU_ELS As Integer, ByVal Mmax(,,) As Double, ByVal Mmin(,,) As Double, ByVal i1 As Integer,
                                 Optional ByRef courbe_moment As Integer = Nothing, Optional ByRef x0(,,) As Integer = Nothing, Optional ByRef x1Max(,) As Integer = Nothing, Optional ByRef x1min(,) As Integer = Nothing,
                                 Optional ByRef m1d(,,) As Double = Nothing, Optional ByRef m2d(,,) As Double = Nothing, Optional ByRef m3d(,,) As Double = Nothing) ', _
        'ByRef ferraillage_sup() As Integer)
        '--------------------------------------------------------------------------
        'Recherche des moments MAX en travée et sur appui et décalage des moments  
        '--------------------------------------------------------------------------
        'Variables: i1=n° de travée
        '           Mmax(ELU_ELS, i1, x) = courbe enveloppe des moments positifs de toutes les combinaisons 
        '                                      par Etat Limite
        '           Mmin(ELU_ELS, i1, x) = courbe enveloppe des moments négatifs de toutes les combinaisons 
        '                                      par Etat Limite

        Dim m1, m2, m11, m12 As Double '
        Dim Nint, x, xd, Xlimit1, Xlimit2 As Integer
        Dim Interception As Boolean
        'recherche de l'abscisse et du moment maximun positif
        Nint = CInt(Truncate(PlanCont.Item(i1 - 1).Lcx * 100)) ' : Nint = Nint \ 1
        Mmax(ELU_ELS, i1, Nint) = 0
        m1 = 0
        For x = 0 To Nint - 1 Step 1
            If Mmax(ELU_ELS, i1, x + 1) > m1 Then
                x1Max(ELU_ELS, i1) = x + 1
                m1 = Mmax(ELU_ELS, i1, x + 1)
            End If
        Next x
        'sélection de l'allure des courbes de moment négatif
        courbe_moment = 1
        If i1 = 1 Then courbe_moment = 2 'pour la travée 1 d'une poutre continue, la courbe des moments négatifs est d'abord nulle avant de s'élever progressivement vers une valeur négative culminant sur l'appui
        For x = 1 To Nint Step 1         ' la valeur commence à 1 car il peut y avoir des problèmes d'initialisation de variable
            Select Case courbe_moment  'Pour les moments négatifs
                Case 1 'Recherche de l'abscisse du premier moment nul
                    'courbe_moment = 1 si la courbe des moments négatifs n'intercepte pas l'axe des abscisses
                    If Mmin(ELU_ELS, i1, x) = 0 Then
                        x0(ELU_ELS, i1, 1) = x
                        If i1 = PlanCont.Count And PlanCont.Item(i1 - 1).LappD = 0 Then courbe_moment = 5 : Exit For
                        courbe_moment = courbe_moment + 1
                        If i1 = PlanCont.Count Then Exit For
                    End If
                Case 2 'Recherche de l'abscisse du deuxième moment nul
                    'courbe_moment = 2 si la courbe des moments intercepte une fois l'axe des abscisses
                    If Mmin(ELU_ELS, i1, x) < 0 Then
                        x0(ELU_ELS, i1, 2) = x
                        If i1 = 1 And PlanCont.Item(i1 - 1).LappG < 0.05 Then courbe_moment = 4 : Exit For 'poutre console
                        courbe_moment = courbe_moment + 1 : Exit For
                    End If
            End Select
        Next x
        'pour la courbe_moment = 1, la travée peut être soulevée sur sa longueur entière entre appui, le test ci-aprés 
        'va rechercher le minimun négatif
        If courbe_moment = 1 Then   'recherche de l'abscisse et du plus grand moment négatif
            m1 = Mmin(ELU_ELS, i1, 0)
            For x = 0 To Nint - 1 Step 1 'recherche de l'abscisse et du moment
                If Mmin(ELU_ELS, i1, x + 1) > m1 Then ' minimun négatif
                    x1min(ELU_ELS, i1) = x + 1
                    m1 = Mmin(ELU_ELS, i1, x + 1)      ' en travée
                End If
            Next x
            'ferraillage_sup(i1) = 2
        End If
        'calcul des décalages des moments
        For x = 0 To Nint Step 1
            'décalage des moments positifs en travée
            m1 = Mmax(ELU_ELS, i1, x)
            Select Case x
                Case Is < CInt(x1Max(ELU_ELS, i1) - PlanCont.Item(i1 - 1).al * 100)
                    xd = CInt(x + PlanCont.Item(i1 - 1).al * 100)
                    m11 = Mmax(ELU_ELS, i1, xd)
                    If m11 >= m1 Then
                        m1d(ELU_ELS, i1, x) = m11
                    Else
                        m1d(ELU_ELS, i1, x) = m1
                    End If
                Case CInt(x1Max(ELU_ELS, i1) - PlanCont.Item(i1 - 1).al * 100) To CInt(x1Max(ELU_ELS, i1) + PlanCont.Item(i1 - 1).al * 100)
                    m1d(ELU_ELS, i1, x) = Mmax(ELU_ELS, i1, x1Max(ELU_ELS, i1)) ' m1MAX(ELU_ELS, i1)
                Case Is > CInt(x1Max(ELU_ELS, i1) + PlanCont.Item(i1 - 1).al * 100)
                    xd = CInt(x - PlanCont.Item(i1 - 1).al * 100)
                    m11 = Mmax(ELU_ELS, i1, xd)
                    If m11 >= m1 Then
                        m1d(ELU_ELS, i1, x) = m11
                    Else
                        m1d(ELU_ELS, i1, x) = m1
                    End If
            End Select
            'décalage des moments négatifs en travée
            m2 = Mmin(ELU_ELS, i1, x)   'le moment MAX de m2 est sur l'appui

            Select Case courbe_moment
                Case 1
                    'courbe_moment = 1 si la courbe des moments n'intercepte pas l'axe des abscisses
                    Select Case x
                        Case Is <= CInt(x1min(ELU_ELS, i1))
                            xd = CInt(Int(x - PlanCont.Item(i1 - 1).al * 100))
                            If xd < 0 Then xd = 0
                            m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                            If m12 < m2 Then
                                m2d(ELU_ELS, i1, x) = m12
                            Else
                                m2d(ELU_ELS, i1, x) = m2
                            End If
                            m3d(ELU_ELS, i1, x) = 0
                        Case Is > CInt(x1min(ELU_ELS, i1))
                            xd = CInt(x + PlanCont.Item(i1 - 1).al * 100)
                            If xd > Nint Then xd = Nint
                            m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                            If m12 < m2 Then
                                m3d(ELU_ELS, i1, x) = m12
                            Else
                                m3d(ELU_ELS, i1, x) = m2
                            End If
                            m2d(ELU_ELS, i1, x) = 0
                    End Select
                Case 2
                    'courbe_moment = 2 si la courbe des moments intercepte une fois l'axe des abscisses
                    If i1 = PlanCont.Count Then
                        xd = CInt(x - PlanCont.Item(i1 - 1).al * 100)
                        If xd < 0 Then xd = 0
                        m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                        m2d(ELU_ELS, i1, x) = m12
                        m3d(ELU_ELS, i1, x) = 0
                    End If
                    If i1 < PlanCont.Count Then
                        'ferraillage_sup(i1) = 2
                        Xlimit1 = CInt(x0(ELU_ELS, i1, 1) - PlanCont.Item(i1 - 1).al * 100)
                        Xlimit2 = CInt(x0(ELU_ELS, i1, 1) + PlanCont.Item(i1 - 1).al * 100)
                        Select Case x
                            Case Is < Xlimit1 ' x0(ELU_ELS, i1, 1) + 80 * h(i1)
                                xd = CInt(x - PlanCont.Item(i1 - 1).al * 100)
                                If xd < 0 Then xd = 0
                                m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                                If m12 < m2 Then
                                    m2d(ELU_ELS, i1, x) = m12
                                Else
                                    m2d(ELU_ELS, i1, x) = m2
                                End If
                                m3d(ELU_ELS, i1, x) = 0
                            Case Xlimit1 To Xlimit2 'x0(ELU_ELS, i1, 1) + 80 * h(i1) To x0(ELU_ELS, i1, 2) - 80 * h(i1)
                                xd = CInt(x - PlanCont.Item(i1 - 1).al * 100)
                                If xd < 0 Then xd = 0
                                m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                                If m12 < m2 Then
                                    m2d(ELU_ELS, i1, x) = m12
                                Else
                                    m2d(ELU_ELS, i1, x) = m2
                                End If
                                xd = CInt(x + PlanCont.Item(i1 - 1).al * 100)
                                If xd > Nint Then xd = Nint
                                m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                                If m12 < m2 Then
                                    m3d(ELU_ELS, i1, x) = m12
                                Else
                                    m3d(ELU_ELS, i1, x) = m2
                                End If
                            Case Is > Xlimit2 'x0(ELU_ELS, i1, 2) - 80 * h(i1)
                                xd = CInt(x + PlanCont.Item(i1 - 1).al * 100)
                                If xd > Nint Then xd = Nint
                                m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                                If m12 < m2 Then
                                    m3d(ELU_ELS, i1, x) = m12
                                Else
                                    m3d(ELU_ELS, i1, x) = m2
                                End If
                                m2d(ELU_ELS, i1, x) = 0
                        End Select
                    End If
                Case 3
                    'courbe_moment = 3 si la courbe des moments intercepte 2 fois l'axe des abscisses
                    If x0(ELU_ELS, i1, 1) + PlanCont.Item(i1 - 1).al * 100 > x0(ELU_ELS, i1, 2) - PlanCont.Item(i1 - 1).al * 100 Then
                        Xlimit1 = CInt(x0(ELU_ELS, i1, 2) - PlanCont.Item(i1 - 1).al * 100)
                        Xlimit2 = CInt(x0(ELU_ELS, i1, 1) + PlanCont.Item(i1 - 1).al * 100)
                        Interception = True  'interception entre les 2 courbes de décalage des moments
                    Else
                        Xlimit1 = CInt(x0(ELU_ELS, i1, 1) + PlanCont.Item(i1 - 1).al * 100)
                        Xlimit2 = CInt(x0(ELU_ELS, i1, 2) - PlanCont.Item(i1 - 1).al * 100)
                        Interception = False 'pas d'interception entre les 2 courbes
                    End If
                    Select Case x
                        Case Is < Xlimit1 ' x0(ELU_ELS, i1, 1) + 80 * h(i1)
                            xd = CInt(x - PlanCont.Item(i1 - 1).al * 100)
                            If xd < 0 Then xd = 0
                            m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                            m2d(ELU_ELS, i1, x) = m12
                            m3d(ELU_ELS, i1, x) = 0
                        Case Xlimit1 To Xlimit2 'x0(ELU_ELS, i1, 1) + 80 * h(i1) To x0(ELU_ELS, i1, 2) - 80 * h(i1)
                            If Interception = False Then
                                m2d(ELU_ELS, i1, x) = 0
                                m3d(ELU_ELS, i1, x) = 0
                            Else
                                xd = CInt(x - PlanCont.Item(i1 - 1).al * 100)
                                If xd < 0 Then xd = 0
                                m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                                m2d(ELU_ELS, i1, x) = m12
                                xd = CInt(x + PlanCont.Item(i1 - 1).al * 100)
                                If xd > Nint Then xd = Nint
                                m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                                m3d(ELU_ELS, i1, x) = m12
                            End If
                        Case Is > Xlimit2 'x0(ELU_ELS, i1, 2) - 80 * h(i1)
                            xd = CInt(x + PlanCont.Item(i1 - 1).al * 100)
                            If xd > Nint Then xd = Nint
                            m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                            m3d(ELU_ELS, i1, x) = m12
                            m2d(ELU_ELS, i1, x) = 0
                    End Select
                Case 4 'console à gauche et donc moment sur appui droit
                    xd = CInt(x + PlanCont.Item(i1 - 1).al * 100)
                    If xd > Nint Then xd = Nint
                    m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                    m3d(ELU_ELS, i1, x) = m12
                    m2d(ELU_ELS, i1, x) = 0
                Case 5 'console à droite et donc moment sur appui gauche
                    xd = CInt(x - PlanCont.Item(i1 - 1).al * 100)
                    If xd < 0 Then xd = 0
                    m12 = Mmin(ELU_ELS, i1, xd)  'rappel: les moments sont négatifs
                    m2d(ELU_ELS, i1, x) = m12
                    m3d(ELU_ELS, i1, x) = 0
            End Select
        Next x
    End Sub

    ''' <summary>
    ''' Calcul des aciers longitudinaux sous forme de TS
    ''' </summary>
    ''' <param name="ELU_ELS">=1 pour ELU, 3 pour ELS</param>
    ''' <param name="i1">N° de travée</param>
    ''' <param name="j">1 = acier en travée; 2 = acier sur appui gauche; 3 = acier sur appui droit</param>
    ''' <param name="Ainf">section d'acier minimale requise</param>
    ''' <param name="d">bras de levier</param>
    ''' <remarks>Dim i() As Integer = {2, 3, 4}
    ''' clGestionErreur.VerifierDgv(Me.dgvLstRappel, i, "Onglet Rappels - Liste des rappels - Certaines saisies sont requisent.", Me.tabRappel) </remarks>
    Private Sub Aciers_longitudinaux_par_lit_TS(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal j As Integer, ByVal Ainf As Double, ByRef d As Double, ByRef str() As Double, ByRef Verif As Boolean)
        '-------------------------------------------------------------------------
        '   Détermination des aciers longitudinaux par lits de TS
        '            nécessaire à la reprise des efforts
        '-------------------------------------------------------------------------
        'Variables: i1 = n° de travée (et non n° d'index !!!!!!!!!!!!!!!!!!!!!)
        '           j = localisation du moment, 1->en travée, 2->sur appui gauche, 3->sur appui droit

        Dim i, i2, i3, inf As Integer
        Dim NLit_TS1 As New Ferraillage_TS
        Dim NLit_TS2 As New Ferraillage_TS
        Dim NLit_TS3 As New Ferraillage_TS
        Dim ama(3), da, ev, phiS(3, 3), u1, u2, u3, u4, u5 As Double 'Ac,enrobage,, φn
        Dim Aminx, Aminy, Ac As Double
        Dim Esp As Boolean = True
        Dim sol_Lit As Boolean = False 'indique si une solution a été trouvée
        ev = Max(bet.dg + 0.005, 0.02) 'initialisation espacement vertical entre lits
        Ac = Ainf
        With PlanCont.Item(i1 - 1)

            Select Case .Prefa
                Case False 'dalle pleine
                    Call Pourcent_mini_acier_flexion(i1, d, Aminx) ' % minimun longitudinal des aciers de flexion dans une Dalle
                    Aminy = 0.2 * Aminx 'provisoire
                Case True 'prédalle
                    'on applique la même règle que pour la dalle pleine
                    Call Pourcent_mini_acier_flexion(i1, d, Aminx) ' % minimun longitudinal des aciers de flexion dans une Dalle
                    Aminy = 0.2 * Aminx 'provisoire
            End Select
            Select Case j
                Case 1
                    If Ac < Aminx Then Ac = Aminx : Aminy = 0.2 * Aminx '% mini
                    '% mini dans le sens non porteur
                    If Aminy < Ainf * 0.2 Then Aminy = Ainf * 0.2
                    str(12) = Aminy
                Case 2 'reprise de 0,15M0
                    If i1 = 1 Then Aminy = 0 'aminy n'a pas de signification physique dans ce cas là
                'Pour les autres cas, la section de constructions des TS Adets suffit
                Case 3
                    If i1 = PlanCont.Count Then Aminy = 0 'aminy n'a pas de signification physique dans ce cas là
                'Pour les autres cas, la section de constructions des TS Adets suffit
                Case 4 'acier sur appuis latéraux non porteur
                    Aminy = 0 'aminy n'a pas de signification physique dans ce cas là
            End Select

            ama(1) = 0 : ama(2) = 0 : inf = 1
            'calcul sur 1 lit                  
            i2 = -1
            Do
                i2 = i2 + 1 'base 0
                If aci.TS.Item(i2) = True Then
                    ama(1) = aci.As_long(i2)
                    u3 = aci.As_trans(i2)
                End If
            Loop Until (ama(1) > Ac And u3 > Aminy) Or i2 + 1 = aci.nomTS.Count  'fin de la table
            If ama(1) < Ac Or u3 < Aminy Then
                If (j = 1 And .MaxnbLitTravee = 1) Or (j = 2 Or j = 3 And .MaxnbLitAppui = 1) Then
                    Dim msg As String = String.Empty
                    If j = 1 Then .MaxnbLitTravee = 2 : msg = "Travée n°" & i1.ToString & " - armatures inférieures"
                    If j = 2 Or j = 3 Then .MaxnbLitAppui = 2 : msg = "Travée n°" & i1.ToString & " - armatures supérieures"
                    msg = msg & "vous avez spécifié une solution à 1 lit, or, aprés calcul, 2 lits de TS sont nécessaires. Calcul effectué avec 2 lits. "
                    Aff.msg_Avertiss.Add(msg)
                End If
                NLit_TS1.indTS = -1 'aucune valeur n'est affecté à Nlit_TS1
            ElseIf ama(1) > Ac And u3 > Aminy Then
                NLit_TS1.indTS = i2 'n° de treillis suivant nomenclature ADETS : inf(i1, j) = 1
            End If
            'calcul de As avec 2 lits
            If (j = 1 And .MaxnbLitTravee = 2) Or (j = 2 Or j = 3 And .MaxnbLitAppui = 2) Then
                i2 = 0 : ama(2) = 0.1 'en m²
                Do
                    i2 = i2 + 1
                    If aci.TS.Item(i2) = True Then
                        u1 = aci.As_long(i2)
                        u4 = aci.As_trans(i2)
                    End If
                    i3 = i2 - 2
                    Do
                        i3 = i3 + 1
                        If aci.TS.Item(i2) = True Then
                            u2 = aci.As_long(i3)
                            u5 = aci.As_trans(i3)
                        End If
                    Loop Until (u1 + u2 > Ac And u4 + u5 > Aminy) Or i3 = i2
                    If u1 + u2 > Ac And u1 > Aminx And u4 > Aminy Then
                        sol_Lit = True
                        If u1 + u2 < ama(2) Then 'recherche de la quantité d'acier la + petite
                            ama(2) = u1 + u2
                            NLit_TS2.indTS = i2 : NLit_TS3.indTS = i3 '    NLit_TS(2) = i2 : NLit_TS(3) = i3
                        End If
                    End If
                Loop While i2 + 1 < aci.nomTS.Count
                If sol_Lit = False Then
                    Dim msg As String
                    If u4 + u5 < Aminy Then Ac = Aminy
                    msg = "La section d'acier requise (" & (Ac * 10000).ToString("#,###.00") & " cm²) est trop importante et ne peut être couverte avec 2 lits d'acier."
                    MessageBox.Show(msg, "Module de calcul", MessageBoxButtons.OK, MessageBoxIcon.Error)
                    Verif = False
                    Exit Sub
                End If
                ' comparaison de As avec 2 lits et de As avec 1 lit
                If NLit_TS1.indTS = -1 Then
                    'la solution à 1 seul lit n'est pas suffisante
                    inf = 2
                    'TS le + important le + prés de la face pour la solution à 2 TS
                    If aci.As_long(NLit_TS3.indTS) > aci.As_long(NLit_TS2.indTS) Then
                        i = NLit_TS2.indTS : NLit_TS2.indTS = NLit_TS3.indTS : NLit_TS3.indTS = i 'inversion pour avoir le TS le plus gros en bas
                    End If
                Else
                    'Solution avec 1 seul ou 2 lits possibles
                    If ama(1) > ama(2) Then inf = 2 Else inf = 1
                    'TS le + important le + prés de la face pour la solution à 2 TS
                    If inf = 2 And aci.As_long(NLit_TS3.indTS) > aci.As_long(NLit_TS2.indTS) Then
                        i = NLit_TS2.indTS : NLit_TS2.indTS = NLit_TS3.indTS : NLit_TS3.indTS = i 'inversion pour avoir le TS le plus gros en bas
                    End If
                End If
            End If

            Select Case inf   ' détermination du nouveau bras de levier avec l'acier réel
                Case 1 'solution à 1 seul lit
                    da = aci.Ølong(NLit_TS1.indTS) / 2
                    Select Case j
                        Case 1
                            .TS_trav.Clear() 'remise à zero
                            .TS_trav.Add(NLit_TS1)
                        Case 2
                            .TS_AppG.Clear() 'remise à zero
                            .TS_AppG.Add(NLit_TS1)
                        Case 3
                            .TS_AppD.Clear() 'remise à zero
                            .TS_AppD.Add(NLit_TS1)
                        Case 4 'acier sur appuis latéraux non porteur
                            .TS_AppH.Clear() 'remise à zero
                            .TS_AppH.Add(NLit_TS1)
                            .TS_AppB.Clear() 'remise à zero
                            .TS_AppB.Add(NLit_TS1)
                    End Select
                Case 2 'solution à 2 lits
                    'de base, les TS sont collés les uns aux autres, pas de cales de réglages entre les TS
                    da = ((aci.Ølong(NLit_TS2.indTS) / 2 * aci.As_long(NLit_TS2.indTS)) + ((aci.Ølong(NLit_TS2.indTS) + aci.Øtrans(NLit_TS2.indTS) + aci.Ølong(NLit_TS3.indTS) / 2) * aci.As_long(NLit_TS3.indTS))) / (aci.As_long(NLit_TS2.indTS) + aci.As_long(NLit_TS3.indTS))
                    Select Case j
                        Case 1
                            .TS_trav.Clear() 'remise à zero
                            .TS_trav.Add(NLit_TS2)   '  .TS_trav.Item(0).indTS = NLit_TS(2) 'indice du TS retenu
                            .TS_trav.Add(NLit_TS3)   '   .TS_trav.Item(1).indTS = NLit_TS(3) 'indice du TS retenu
                        Case 2
                            .TS_AppG.Clear() 'remise à zero
                            .TS_AppG.Add(NLit_TS2)   ' .TS_AppG.Item(0).indTS = NLit_TS(2) 'indice du TS retenu
                            .TS_AppG.Add(NLit_TS3)  ' .TS_AppG.Item(1).indTS = NLit_TS(3) 'indice du TS retenu
                        Case 3
                            .TS_AppD.Clear() 'remise à zero
                            .TS_AppD.Add(NLit_TS2)    ' .TS_AppD.Item(0).indTS = NLit_TS(2) 'indice du TS retenu
                            .TS_AppD.Add(NLit_TS3)    '  .TS_AppD.Item(1).indTS = NLit_TS(3) 'indice du TS retenu
                            'Case 4 'acier sur appuis latéraux non porteur
                            '    .TS_AppH.Add(NLit_TS2) : .TS_AppH.Add(NLit_TS3)
                            '    .TS_AppB.Add(NLit_TS2) : .TS_AppB.Add(NLit_TS3)
                            '    Dim msg As String
                            '    msg = "Travée n°" & i1.ToString & " Calcul des TS sur appuis Haut et Bas (appuis non porteurs) effectué avec 2 lits. "
                            '    Aff.msg_Avertiss.Add(msg)
                    End Select
            End Select
            'Calcul du nouveau bras de levier
            Select Case j
                Case 1
                    d = .H - .enrInf - da   '0.05 correspond au diamètre de l'acier du cadre et aux erreurs de calage 'd(i1, j)
                Case 2, 3
                    d = .H - .enrSup - da   '0.015 correspond au diamètre de l'acier du cadre et aux erreurs de calage 'd(i1, j)
            End Select

        End With
        NLit_TS1 = Nothing : NLit_TS2 = Nothing : NLit_TS3 = Nothing 'suppression de la classe
    End Sub
    Private Sub Aciers_longitudinaux_par_lit_HA(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal j As Integer, ByVal Ainf As Double, ByRef d As Double, ByRef str() As Double, ByRef verif As Boolean)
        '-------------------------------------------------------------------------
        '   Détermination des aciers longitudinaux par lits de barres HA
        '            nécessaire à la reprise des efforts
        '-------------------------------------------------------------------------
        'Variables: i1 = n° de travée (et non n° d'index !!!!!!!!!!!!!!!!!!!!!)
        '           j = localisation du moment, 1->en travée, 2->sur appui gauche, 3->sur appui droit

        Dim i, i2, n1, n2 As Integer 'nbarS(3, 3),i22, i222, i4, i5, ip1,, nblit, inf
        Dim ama, aminx, aminy, ac, ac_Y, Lcalcul, u1 As Double 'da,ev, phiS(3, 3),Ac,enrobage,, φn
        Dim Esp As Boolean = True
        Dim NLit_HA1 As New Ferraillage_HA
        '   Dim NLit_HA2 As New Ferraillage_HA
        Dim msg As String

        ac = Ainf

        With PlanCont.Item(i1 - 1)
            '% mini dans le sens non porteur
            ac_Y = Ainf * 0.2 : str(12) = ac_Y
            'calcul dans le sens porteur
            Call Pourcent_mini_acier_flexion(i1, d, aminx)
            aminy = 0.2 * aminx 'provisoire

            Select Case j
                Case 1
                    If ac < aminx Then
                        ac = aminx '% mini
                        ac_Y = aminy : str(12) = ac_Y
                    End If
                Case 2 'reprise de 0,15M0
                    If i1 = 1 Then aminy = 0 'aminy n'a pas de signification physique dans ce cas là
                Case 3
                    If i1 = PlanCont.Count Then aminy = 0 'aminy n'a pas de signification physique dans ce cas là
                Case 4 'aciers sur appuis non porteurs
                    aminy = 0 'aminy n'a pas de signification physique dans ce cas là
            End Select
            If j <= 3 Then
                n1 = CInt(Round(.Ly / .EspMaxPrincipal)) : n2 = CInt(Int(.Ly / .EspminPrincipal))   'même valeur pour tous les lits
                Lcalcul = .Ly
            Else
                n1 = CInt(Round(.Lx / .EspMaxSecondaire)) : n2 = CInt(Int(.Lx / .EspminSecondaire))   'même valeur pour tous les lits
                Lcalcul = .Lx
            End If
            n1 += 1 'sinon risque effet de bord - déjà testé!!!!
            ama = 0.1   '1000 cm²
            'recherche de la combinaison des diamètres d'acier la plus proche de la section calculée
            'calcul pour la section x - calcul dans le sens porteur
            For i = n1 To n2
                i2 = -1
                Do
                    i2 = i2 + 1
                    If aci.Ha_Lg(i2) = True Then u1 = i * PI * aci.Ø_Norm(i2) ^ 2 / 4
                    If i2 + 1 = aci.Ha_Lg.Count Then Exit Do
                Loop While u1 < ac * Lcalcul
                If u1 < ama Then
                    NLit_HA1 = New Ferraillage_HA 'réinitialisation
                    ama = u1
                    With NLit_HA1
                        .nbar = i : .φHA = aci.Ø_Norm.Item(i2)
                        Call Calcul_Espacement(PlanCont.Item(i1 - 1).Ly, i, .S1, .S2, .Scourant)
                    End With
                    'affectation
                    Select Case j
                        Case 1
                            .LitInf.Clear() 'en raison de plusieurs accès successifs
                            .LitInf.Add(NLit_HA1)
                        Case 2
                            .LitSupG.Clear()
                            .LitSupG.Add(NLit_HA1)
                        Case 3
                            .LitSupD.Clear()
                            .LitSupD.Add(NLit_HA1)
                        Case 4
                            .LitSupH.Clear() : .LitSupB.Clear()
                            If .LappH > 0.05 And Aff.EncastPartiel_1_porteur = True Then
                                With NLit_HA1
                                    Call Calcul_Espacement(PlanCont.Item(i1 - 1).Lx, i, .S1, .S2, .Scourant)
                                End With
                                .LitSupH.Add(NLit_HA1)
                            End If
                            If .LappB > 0.05 And Aff.EncastPartiel_1_porteur = True Then
                                With NLit_HA1
                                    Call Calcul_Espacement(PlanCont.Item(i1 - 1).Lx, i, .S1, .S2, .Scourant)
                                End With
                                .LitSupB.Add(NLit_HA1)
                            End If
                    End Select
                End If
            Next i
            'vérification qu'un résultat ait été trouvé
            If ama = 0.1 Then
                u1 = n2 * aci.Ø_Norm.Item(i2) ^ 2 / 4 * PI
                msg = "Section" & (ac * 10000).ToString("#,##0.0") & "cm2 Impossible à satisfaire avec " & n2.ToString & " Barres de " & (aci.Ø_Norm.Item(i2) * 10000).ToString("#") &
                    " mm ( " & (u1 * 10000).ToString("#,##0.0") & " cm2)." & Environment.NewLine & "Cliquez sur la touche OK"
                MessageBox.Show(msg, "Plancher BA : module de calcul", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1)
                verif = False
                Exit Sub
            End If
            'Mise à jour du bras de levier
            Select Case j
                Case 1
                    d = .H - .enrInf - NLit_HA1.φHA / 2   ' détermination du nouveau bras de levier avec l'acier réel
                Case 2, 3
                    d = .H - .enrSup - NLit_HA1.φHA / 2   ' détermination du nouveau bras de levier avec l'acier réel
            End Select

            'vérification de la mise en place de ce type d'armatures pour les aciers sur appuis
            Select Case j
                Case 2
                    If i1 = 1 Or Aff.HA_Trans_Appui = False Then NLit_HA1 = Nothing : Exit Sub
                Case 3
                    If i1 = PlanCont.Count Or Aff.HA_Trans_Appui = False Then NLit_HA1 = Nothing : Exit Sub ' .LitSupD_Y.Item(0).nbar = 0 : .LitSupD_Y.Item(0).φHA = 0
                Case 4
                    NLit_HA1 = Nothing : Exit Sub 'Pas de signification pour ce type d'acier
            End Select

            'calcul dans le sens non porteur
            'calcul pour la section aminy (sens de la portée non porteuse)
            ama = 0.1   '1000 cm²
            n1 = CInt(Round(.Lx / .EspMaxSecondaire)) : n1 = n1 + 1
            n2 = CInt((Int(.Lx / .EspminSecondaire)))
            For i = n1 To n2
                i2 = -1 'le diamètre peut commencer à 6 mm
                Do
                    i2 = i2 + 1
                    If aci.Ha_Lg(i2) = True Then u1 = i * PI * aci.Ø_Norm(i2) ^ 2 / 4
                    If i2 >= Indice_Acier(NLit_HA1.φHA) Then Exit Do
                Loop While u1 < ac_Y * .Lx
                If u1 < ama Then
                    NLit_HA1 = New Ferraillage_HA 'pour remise à 0 systématique
                    ama = u1
                    Select Case j
                        Case 1
                            With NLit_HA1
                                .nbar = i : .φHA = aci.Ø_Norm.Item(i2)
                                Call Calcul_Espacement(PlanCont.Item(i1 - 1).Lx, i, .S1, .S2, .Scourant)
                                If PlanCont.Item(i1 - 1).Ly + .S1 + .S2 > aci.HA_Lg_Max Then
                                    aci.Position_Portefeuille = True ' : .LitInf_Y_nblit = 2
                                Else
                                    aci.Position_Portefeuille = False ': .LitInf_Y_nblit = 1
                                End If
                            End With
                            .LitInf_Y.Clear() 'en raison de plusieurs accès successifs
                            .LitInf_Y.Add(NLit_HA1)
                            If aci.Position_Portefeuille = True Then
                                Dim NLit_HA2 As New Ferraillage_HA
                                .LitInf_Y.Add(NLit_HA2)
                                .LitInf_Y.Item(1).S1 = .LitInf_Y.Item(0).S1 : .LitInf_Y.Item(1).S2 = .LitInf_Y.Item(0).S2 : .LitInf_Y.Item(1).Scourant = .LitInf_Y.Item(0).Scourant
                                .LitInf_Y.Item(1).nbar = .LitInf_Y.Item(0).nbar : .LitInf_Y.Item(1).φHA = .LitInf_Y.Item(0).φHA : .LitInf_Y.Item(1).Angle = .LitInf_Y.Item(0).Angle
                                NLit_HA2 = Nothing
                            End If
                        Case 2, 3
                            If Aff.HA_Trans_Appui = True Then
                                msg = "Module acier_longitudinaux_par_lit_HA - aciers transversaux pour armatures supérieures - calcul non implémenté"
                                Aff.msg_Avertiss.Add(msg)
                            End If
                    End Select
                End If
            Next i

            NLit_HA1 = Nothing
        End With
    End Sub
    Private Sub Calcul_Espacement(ByVal Longueur As Double, ByVal nbar As Integer, ByRef S1 As Double, ByRef S2 As Double, ByRef S_courant As Double)
        Dim restant As Double ', S_courant1
        Dim i As Integer
        S_courant = Round((Longueur * 100) / (nbar - 1))
        i = 0
        Do
            i += 1
            Do
                restant = (Longueur * 100) - (nbar - 1) * S_courant
                If restant >= 0 Then Exit Do Else S_courant = S_courant - 0.1 'recherche par pas de 0.1cm car je ne peux pas changer le nombre de barres
                i += 1
                If i > 1000 Then MessageBox.Show("Pb calcul espacement", "Module calcul_espacement", MessageBoxButtons.OK, MessageBoxIcon.Exclamation) : Exit Sub 'pour éviter de boucler à l'infni.
            Loop
            S1 = Truncate(restant / 2)
            S2 = (Longueur * 100) - S1 - (nbar - 1) * S_courant
            If S1 < S_courant And S2 < S_courant Then Exit Do Else S_courant = S_courant + 0.1
        Loop
        S_courant = S_courant / 100 : S1 = S1 / 100 : S2 = S2 / 100 'basculement cm -> m
    End Sub
    ''' <summary>
    ''' Retourne le φ de l'acier si celui-ci a été validé dans le formulaire Ferraillage
    ''' </summary>
    Private Function dioL(ByVal i As Integer) As Double
        With aci
            If .Ha_Lg.Item(i) = True Then dioL = .Ø_Norm(i) Else dioL = 0
        End With
    End Function
    Private Sub Verification_Acier_sur_appui_TS(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal d As Double, ByVal M(,,) As Double,
                                             ByVal V(,,) As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        'Variables: i1 = n° de travée ( et non n° d'index !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!)
        '           appui = 1->gauche, 2->droit
        '           j = précise comment se prolonge les aciers sur appui
        '               j=5-> ajout d'acier supplémentaires
        '               j=4->
        '           Amanquant = section d'acier manquant pour satisfaire aux conditions d'appui
        '           cgl = 1,2,3,4  -> caractéristique des aciers sur appui pour la vérification des conditions d'entrainement
        '           utot = somme totale de l'aire des lits d'aciers arrivant sur l'appui
        '           utot1 = (utot) -(somme des lits d'aciers ancrés totalement sur l'appui)

        Dim msg As String
        Dim Appui As Integer

        Dim agl, angle, FE(2), fyd, Lappui(2), Med(2), Vapp(2), Z, γs, CotΘ(2) As Double ', CotΘ(2) , espu1, u2, u3,Amanquant,angle,
        'Call Verification_bielle_appui(verif, i1, la) : If verif = False Then Exit Sub

        Call Effort_Traction_Appui(ELU_ELS, i1, d, M, V, CotΘ, FE, Vapp, Med) 'calcul de l'effort horizontal repris par l'appui
        Select Case ELU_ELS
            Case 1
                γs = aci.γsd 'combinaison fondamentale   γc = bet.γcd : γc = bet.γca :
            Case 2
                γs = aci.γsa 'combinaison acidentelle
        End Select
        fyd = aci.fyk / γs
        With PlanCont.Item(i1 - 1).TS_trav
            'le lit 1 est toujours ancré sur l'appui par mesure de sécurité en cours de construction
            .Item(0).X1acier = 0 : .Item(0).X1acierLS = 0
            .Item(0).X2acier = PlanCont.Item(i1 - 1).Lx : .Item(0).X2acierLS = PlanCont.Item(i1 - 1).Lx
            Z = 0.9 * PlanCont.Item(i1 - 1).dInf
            If .Count = 2 Then msg = "2" & " lits inférieurs." Else msg = "1 seul" & " lit inférieur."
            rtf.WriteStartGroup()
            rtf.NewParagraph()
            rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.Bold = True : rtf.WriteString("  ■ Vérification de l'ancrage des aciers sur appui") : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("La travée comporte " & msg) : rtf.NewLine()
            rtf.NewParagraph()
            'Ligne 1
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
            rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
            'Ligne 2
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As1") : rtf.EndCell()
            rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((aci.As_long(.Item(0).indTS) * 10000).ToString("#0.00")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Aire répartie du lit inférieur n°1") : rtf.EndCell() : rtf.EndRow()
            If .Count = 2 Then
                'Ligne 2
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As2") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((aci.As_long(.Item(1).indTS) * 10000).ToString("#0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Aire répartie du lit inférieur n°2") : rtf.EndCell() : rtf.EndRow()
            End If
            'Ligne 3
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("z") : rtf.EndCell()
            rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Z.ToString("#0.000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" z = 0.9.d") : rtf.EndCell() : rtf.EndRow()
            rtf.WriteEndGroup()
            For Appui = 1 To 2
                rtf.WriteStartGroup()
                rtf.FontSize = 12 : rtf.FontSize = 9
                'initialisation
                If Appui = 1 Then msg = "Appui gauche" Else msg = "Appui droit"
                'Ligne 5
                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                'Ligne 2
                angle = Atan(1 / CotΘ(Appui)) * 180 / PI
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Θ'") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("°") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(angle.ToString("#0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("angle de la bielle moyenne d'appui") : rtf.EndCell() : rtf.EndRow()
                'Ligne 5
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("|Ved|") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
                If Vapp(Appui) < 0 Then Vapp(Appui) = -Vapp(Appui)
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Vapp(Appui) * 1000).ToString("#,##0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("au nu de l'appui - En valeur absolue.") : rtf.EndCell() : rtf.EndRow()
                If PlanCont.Count > 1 And ((i1 = 1 And Appui = 2) = True Or (i1 = PlanCont.Count And Appui = 1) = True Or (i1 > 1 And i1 < PlanCont.Count) = True) = True Then
                    'Ligne 4
                    rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Med") : rtf.EndCell()
                    rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Med(Appui) * 1000).ToString("#0.00")) : rtf.EndCell()
                    rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Moment sur appui - En valeur algébrique.") : rtf.EndCell() : rtf.EndRow()
                End If
                'Ligne 6
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("FE") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "<0" Else msg = (FE(Appui) * 1000).ToString("#,##0.00")
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "Uniquement disposition constructive applicable (voir ci-dessous)" Else msg = " Effort de traction à ancrer sur l'appui"
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                'Ligne 7
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Agl") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
                If FE(Appui) < 0 Then
                    msg = "S.O." : agl = 0
                Else
                    agl = FE(Appui) / fyd
                    msg = (agl * 10000.0).ToString("#,##0.00")
                End If
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "1ier lit prolongé sur appui (disp. constructive)" Else msg = "Section minimale pour reprise effort de traction"
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                rtf.WriteEndGroup()
                'ancrage total lit 1 quelque soit le cas de figure
                Select Case i1
                    Case 1
                        If PlanCont.Count = 1 Then
                            Lappui(1) = PlanCont.Item(i1 - 1).LappG - PlanCont.Item(i1 - 1).enrLat
                            Lappui(2) = PlanCont.Item(i1 - 1).LappD - PlanCont.Item(i1 - 1).enrLat
                            Call Ancrage_appui_About_TS(i1, Appui, ELU_ELS, agl, fyd, FE, Lappui, d, rtf)
                        Else
                            If Appui = 1 Then
                                Lappui(1) = PlanCont.Item(i1 - 1).LappG - PlanCont.Item(i1 - 1).enrLat
                                Call Ancrage_appui_About_TS(i1, Appui, ELU_ELS, agl, fyd, FE, Lappui, d, rtf)
                            Else
                                Lappui(2) = PlanCont.Item(i1 - 1).LappD
                                Call Ancrage_appui_Intermediaire(i1, Appui, agl, Lappui, FE, rtf)
                            End If
                        End If
                    Case PlanCont.Count
                        If Appui = 1 Then
                            Lappui(1) = PlanCont.Item(i1 - 1).LappG
                            Call Ancrage_appui_Intermediaire(i1, Appui, agl, Lappui, FE, rtf)
                        Else
                            Lappui(2) = PlanCont.Item(i1 - 1).LappD - PlanCont.Item(i1 - 1).enrLat
                            Call Ancrage_appui_About_TS(i1, Appui, ELU_ELS, agl, fyd, FE, Lappui, d, rtf)
                        End If
                    Case Else
                        Lappui(1) = PlanCont.Item(i1 - 1).LappG : Lappui(2) = PlanCont.Item(i1 - 1).LappD
                        Call Ancrage_appui_Intermediaire(i1, Appui, agl, Lappui, FE, rtf)
                End Select
            Next Appui
        End With
    End Sub
    Private Sub Verification_Acier_sur_appui_HA(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal d As Double, ByVal M(,,) As Double,
                                             ByVal V(,,) As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        'Variables: i1 = n° de travée ( et non n° d'index !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!)
        '           appui = 1->gauche, 2->droit
        '           j = précise comment se prolonge les aciers sur appui
        '               j=5-> ajout d'acier supplémentaires
        '               j=4->
        '           Amanquant = section d'acier manquant pour satisfaire aux conditions d'appui
        '           cgl = 1,2,3,4  -> caractéristique des aciers sur appui pour la vérification des conditions d'entrainement
        '           utot = somme totale de l'aire des lits d'aciers arrivant sur l'appui
        '           utot1 = (utot) -(somme des lits d'aciers ancrés totalement sur l'appui)

        Dim msg As String
        Dim Appui As Integer

        Dim agl, angle, FE(2), fyd, Lappui(2), Med(2), Vapp(2), Z, γs, CotΘ(2) As Double ', CotΘ(2) , espu1, u2, u3,Amanquant,angle,
        'Call Verification_bielle_appui(verif, i1, la) : If verif = False Then Exit Sub

        Call Effort_Traction_Appui(ELU_ELS, i1, d, M, V, CotΘ, FE, Vapp, Med) 'calcul de l'effort horizontal repris par l'appui
        Select Case ELU_ELS
            Case 1
                γs = aci.γsd 'combinaison fondamentale   γc = bet.γcd : γc = bet.γca :
            Case 2
                γs = aci.γsa 'combinaison acidentelle
        End Select
        fyd = aci.fyk / γs
        With PlanCont.Item(i1 - 1)
            'le lit 1 est toujours ancré sur l'appui par mesure de sécurité en cours de construction
            .LitInf.Item(0).X1acier = 0 : .LitInf.Item(0).X1acierLS = 0
            .LitInf.Item(0).X2acier = .Lx : .LitInf.Item(0).X2acierLS = .Lx
            Z = 0.9 * .dInf
            If .LitInf.Count = 2 Then msg = "2" & " lits inférieurs." Else msg = "1 seul" & " lit inférieur."
            rtf.WriteStartGroup()
            rtf.NewParagraph()
            rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.Bold = True : rtf.WriteString("  ■ Vérification de l'ancrage des aciers sur appui") : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("La travée comporte " & msg) : rtf.NewLine()
            rtf.NewParagraph()
            'Ligne 1
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
            rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
            'Ligne 2
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As1") : rtf.EndCell()
            rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True)
            Select Case .Type_Acier_Travée
                Case 1
                    rtf.WriteString((aci.As_long(.TS_trav.Item(0).indTS)).ToString("#0.00"))
                Case 2
                    rtf.WriteString((.LitInf.Item(0).Aire * 10000.0 / .Ly).ToString("#0.00"))
            End Select
            rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Aire répartie du lit inférieur n°1") : rtf.EndCell() : rtf.EndRow()
            If .LitInf.Count = 2 Then
                'Ligne 2
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As2") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True)
                Select Case .Type_Acier_Travée
                    Case 1
                        rtf.WriteString((aci.As_long(.TS_trav.Item(1).indTS)).ToString("#0.00"))
                    Case 2
                        rtf.WriteString((.LitInf.Item(1).Aire * 10000.0 / .Ly).ToString("#0.00"))
                End Select
                rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Aire répartie du lit inférieur n°2") : rtf.EndCell() : rtf.EndRow()
            End If
            'Ligne 3
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("z") : rtf.EndCell()
            rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Z.ToString("#0.000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" z = 0.9.d") : rtf.EndCell() : rtf.EndRow()
            rtf.WriteEndGroup()
            For Appui = 1 To 2
                rtf.WriteStartGroup()
                rtf.FontSize = 12 : rtf.FontSize = 9
                'initialisation
                If Appui = 1 Then msg = "Appui gauche" Else msg = "Appui droit"
                'Ligne 5
                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                'Ligne 2
                angle = Atan(1 / CotΘ(Appui)) * 180 / PI
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Θ'") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("°") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(angle.ToString("#0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("angle de la bielle moyenne d'appui") : rtf.EndCell() : rtf.EndRow()
                'Ligne 5
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("|Ved|") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
                If Vapp(Appui) < 0 Then Vapp(Appui) = -Vapp(Appui)
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Vapp(Appui) * 1000).ToString("#,##0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("au nu de l'appui - En valeur absolue.") : rtf.EndCell() : rtf.EndRow()
                If PlanCont.Count > 1 And ((i1 = 1 And Appui = 2) = True Or (i1 = PlanCont.Count And Appui = 1) = True Or (i1 > 1 And i1 < PlanCont.Count) = True) = True Then
                    'Ligne 4
                    rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Med") : rtf.EndCell()
                    rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Med(Appui) * 1000).ToString("#0.00")) : rtf.EndCell()
                    rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Moment sur appui - En valeur algébrique.") : rtf.EndCell() : rtf.EndRow()
                End If
                'Ligne 6
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("FE") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "<0" Else msg = (FE(Appui) * 1000).ToString("#,##0.00")
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "Uniquement disposition constructive applicable (voir ci-dessous)" Else msg = " Effort de traction à ancrer sur l'appui"
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()

                'Ligne 7
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Agl") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
                If FE(Appui) < 0 Then
                    msg = "S.O." : agl = 0
                Else
                    agl = FE(Appui) / fyd
                    msg = (agl * 10000.0).ToString("#,##0.00")
                End If
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "1ier lit prolongé sur appui (disp. constructive)" Else msg = "Section minimale pour reprise effort de traction"
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                rtf.WriteEndGroup()
                'ancrage total lit 1 quelque soit le cas de figure
                Select Case i1
                    Case 1
                        If PlanCont.Count = 1 Then
                            Lappui(1) = .LappG - .enrLat : Lappui(2) = .LappD - .enrLat
                            Call Ancrage_appui_About_HA(i1, Appui, ELU_ELS, agl, fyd, FE, Lappui, d, rtf)
                        Else
                            If Appui = 1 Then
                                Lappui(1) = .LappG - .enrLat
                                Call Ancrage_appui_About_HA(i1, Appui, ELU_ELS, agl, fyd, FE, Lappui, d, rtf)
                            Else
                                Lappui(2) = .LappD
                                Call Ancrage_appui_Intermediaire(i1, Appui, agl, Lappui, FE, rtf)
                            End If
                        End If
                    Case PlanCont.Count
                        If Appui = 1 Then
                            Lappui(1) = .LappG
                            Call Ancrage_appui_Intermediaire(i1, Appui, agl, Lappui, FE, rtf)
                        Else
                            Lappui(2) = .LappD - .enrLat
                            Call Ancrage_appui_About_HA(i1, Appui, ELU_ELS, agl, fyd, FE, Lappui, d, rtf)
                        End If
                    Case Else
                        Lappui(1) = .LappG : Lappui(2) = .LappD
                        Call Ancrage_appui_Intermediaire(i1, Appui, agl, Lappui, FE, rtf)
                End Select
            Next Appui
        End With
    End Sub
    Private Sub Ancrage_appui_About_TS(ByVal i1 As Integer, ByVal appui As Integer, ByVal ELU_ELS As Integer, ByVal agl As Double, ByVal fyd As Double, ByVal FE() As Double,
                                    ByVal Lappui() As Double, ByVal d As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        'Dim i4, i5 As Integer
        Dim msg As String
        Dim LacierTot, u, Amin As Double 'Amanquant,
        rtf.WriteStartGroup()
        'rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        With PlanCont.Item(i1 - 1)
            Call Pourcent_mini_acier_flexion(i1, d, Amin)
            If agl < Amin Then agl = Amin
            u = aci.As_long(.TS_trav.Item(0).indTS)
            If agl <= u Then
                fyd = FE(appui) / aci.As_long(.TS_trav.Item(0).indTS)
                Call Ancrage_lit(appui, i1, 1, Lappui(appui), 1, ELU_ELS, fyd, 0, LacierTot)
                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Agl < As1  -> Lit n°1 totalement ancré- Prise en compte section mini (= " & (Amin * 10000).ToString("#0.00") & " cm²)") : rtf.EndCell() : rtf.EndRow()
            ElseIf .TS_trav.Count > 1 Then
                If agl < aci.As_long(.TS_trav.Item(0).indTS) + aci.As_long(.TS_trav.Item(1).indTS) Then
                    'ancrage total du premier lit sur la limite élastique
                    Call Ancrage_lit(appui, i1, 1, Lappui(appui), 1, ELU_ELS, fyd, 0, LacierTot)
                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(" Agl < As1 + As2 - Ancrage des 2 lits n°1 et n°2 - Prise en compte section mini (= " & (Amin * 10000).ToString("#0.00") & " cm²)") : rtf.EndCell() : rtf.EndRow()
                    'ancrage lit 2  
                    fyd = (FE(appui) - aci.As_long(.TS_trav.Item(0).indTS) * fyd) / aci.As_long(.TS_trav.Item(1).indTS)
                    Call Ancrage_lit(appui, i1, 2, Lappui(appui), 1, ELU_ELS, fyd, 0, LacierTot)
                    '___________________________________________________________________________________________________________________________________________
                    ' "Ancrage appui Intermediaire travée n° i1 
                    ' L'ancrage sur appui intemédiaire - nécessite des armatures complémentaires. Disposition non implémentée dans cette version logicielle!"
                End If
            Else
                Dim k As Integer = 0
                Select Case .TS_trav.Count
                    Case 1 '1 lit
                        'pour permettre la suite du calcul
                        If appui = 1 Then
                            LacierTot = Truncate(Lappui(1) * 50) / 100
                            .TS_trav.Item(k).X1acier = 0 : .TS_trav.Item(k).X1acierLS = 0
                            .TS_trav.Item(k).L13acier = LacierTot : .TS_trav.Item(k).L12acier = 0 : .TS_trav.Item(k).L11acier = 0
                        Else
                            LacierTot = Truncate(Lappui(2) * 50) / 100
                            .TS_trav.Item(k).X2acier = .Lx : .TS_trav.Item(k).X2acierLS = .Lx
                            .TS_trav.Item(k).L23acier = LacierTot : .TS_trav.Item(k).L22acier = 0 : .TS_trav.Item(k).L21acier = 0
                        End If
                        msg = " Agl > As1 et 1 seul lit d'armature dimensionné! "
                    Case 2 ' 2 Lits
                        'pour permettre la suite du calcul
                        If appui = 1 Then
                            LacierTot = Truncate(Lappui(1) * 50) / 100
                            For k = 0 To 1
                                .TS_trav.Item(k).X1acier = 0 : .TS_trav.Item(k).X1acierLS = 0
                                .TS_trav.Item(k).L13acier = LacierTot : .TS_trav.Item(k).L12acier = 0 : .TS_trav.Item(k).L11acier = 0
                            Next
                        Else
                            LacierTot = Truncate(Lappui(2) * 50) / 100
                            For k = 0 To 1
                                .TS_trav.Item(k).X2acier = .Lx : .TS_trav.Item(k).X2acierLS = .Lx
                                .TS_trav.Item(k).L23acier = LacierTot : .TS_trav.Item(k).L22acier = 0 : .TS_trav.Item(k).L21acier = 0
                            Next
                        End If
                        msg = " Agl > As1 + As2 et 2 lits d'armature dimensionnés! "
                End Select
                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                Select Case appui
                    Case 1
                        msg = "gauche"
                    Case 2
                        msg = "droit"
                End Select
                msg = "Ancrage appui About travée n°" & i1.ToString & " :" & Environment.NewLine &
                    " L'ancrage sur appui de rive " & msg & " nécessite soit des armatures complémentaires ou soit l'ancrage du 2ième lit. Disposition non implémentée dans cette version logicielle!"
                Aff.msg_Avertiss.Add(msg)
                '        rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(" Agl > As1 + As2 - Ancrage des 3 lits inférieurs ") : rtf.EndCell() : rtf.EndRow()
                '        'demande au projeteur s'il veut rajouter un lit d'acier ou quitter le programme
                '        If appui = 1 Then msg = "gauche" Else msg = "droit"
                '        Amanquant = agl - .LitInf.Item(0).Aire - .LitInf.Item(1).Aire - .LitInf.Item(2).Aire
                '        i4 = 1 : i5 = Indice_Acier(.LitInf.Item(0).φHA)
                '        Do
                '            u = .LitInf.Item(0).nbar * PI * dio(i4) ^ 2 / 4
                '            If u > Amanquant Then
                '                Select Case appui
                '                    Case 1
                '                        .LitAppG.φHA = dio(i4)
                '                        .LitAppG.Lacier2 = Lscel_droit(0, 4, i1, ELU_ELS, fyd, True) 'pour la partie se trouvant dans la travée
                '                        Call Ancrage_total_lit(appui, i1, 1, Lappui(appui), 5, ELU_ELS, fyd, 0, LacierTot) 'pour la partie se trouvant sur appui
                '                        .LitAppG.Lacier2 = .LitAppG.Lacier2 + LacierTot
                '                    Case 2
                '                        .LitAppD.φHA = dio(i4)
                '                        .LitAppD.Lacier2 = Lscel_droit(0, 5, i1, ELU_ELS, fyd, True) 'pour la partie se trouvant dans la travée
                '                        Call Ancrage_total_lit(appui, i1, 1, Lappui(appui), 6, ELU_ELS, fyd, 0, LacierTot) 'pour la partie se trouvant sur appui
                '                        .LitAppD.Lacier2 = .LitAppD.Lacier2 + LacierTot
                '                End Select
                '                Exit Do
                '            End If
                '            i4 = i4 + 1
                '        Loop While i4 < i5 + 2 '1
                '        If i4 = i5 + 2 Then
                '            msg = "Le rajout de barres sur appui travée " & i1.ToString & " s'avère insuffisant ! - Modifier la géométrie de la poutre !"
                '            Aff.msg_Avertiss.Add(msg)
                '        Else
                '            'Ligne 8
                '            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Acomp") : rtf.EndCell()
                '            rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
                '            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((u * 10000.0).ToString("#,##0.00")) : rtf.EndCell()
                '            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Acier complémentaire glissement ") : rtf.EndCell() : rtf.EndRow()
                '            'cgl = 4
                '        End If
            End If
        End With
        rtf.WriteEndGroup()
    End Sub
    Private Sub Ancrage_appui_About_HA(ByVal i1 As Integer, ByVal appui As Integer, ByVal ELU_ELS As Integer, ByVal agl As Double, ByVal fyd As Double, ByVal FE() As Double,
                                    ByVal Lappui() As Double, ByVal d As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        'Dim i4, i5 As Integer
        Dim msg As String = String.Empty
        Dim LacierTot, Amin As Double 'Amanquant,
        rtf.WriteStartGroup()
        'rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        With PlanCont.Item(i1 - 1)
            Call Pourcent_mini_acier_flexion(i1, d, Amin)
            If agl < Amin Then agl = Amin
            If agl <= .LitInf.Item(0).Aire / .Ly Then
                fyd = FE(appui) / (.LitInf.Item(0).Aire / .Ly)
                Call Ancrage_lit(appui, i1, 1, Lappui(appui), 1, ELU_ELS, fyd, 0, LacierTot)
                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Agl < As1  -> Lit n°1 totalement ancré - Prise en compte section mini (= " & (Amin * 10000).ToString("#0.00") & " cm²)") : rtf.EndCell() : rtf.EndRow()
            ElseIf .LitInf.Count > 1 Then
                If agl < .LitInf.Item(0).Aire / .Ly + .LitInf.Item(1).Aire / .Ly Then
                    'ancrage total du premier lit sur la limite élastique
                    Call Ancrage_lit(appui, i1, 1, Lappui(appui), 1, ELU_ELS, fyd, 0, LacierTot)
                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(" Agl < As1 + As2 - Ancrage des 2 lits n°1 et n°2 - Prise en compte section mini (= " & (Amin * 10000).ToString("#0.00") & " cm²)") : rtf.EndCell() : rtf.EndRow()
                    'ancrage lit 2  
                    fyd = (FE(appui) - (.LitInf.Item(0).Aire / .Ly) * fyd) / (.LitInf.Item(1).Aire / .Ly)
                    Call Ancrage_lit(appui, i1, 2, Lappui(appui), 1, ELU_ELS, fyd, 0, LacierTot)
                End If
                '___________________________________________________________________________________________________________________________________________
                ' "Ancrage appui Intermediaire travée n° i1 
                ' L'ancrage sur appui intemédiaire - nécessite des armatures complémentaires. Disposition non implémentée dans cette version logicielle!"
            Else
                Dim k As Integer = 0
                Select Case .LitInf.Count
                    Case 1 '1 lit
                        'pour permettre la suite du calcul
                        If appui = 1 Then
                            LacierTot = Truncate(Lappui(1) * 50) / 100
                            .LitInf.Item(k).X1acier = 0 : .LitInf.Item(k).X1acierLS = 0
                            .LitInf.Item(k).L13acier = LacierTot : .LitInf.Item(k).L12acier = 0 : .LitInf.Item(k).L11acier = 0
                        Else
                            LacierTot = Truncate(Lappui(2) * 50) / 100
                            .LitInf.Item(k).X2acier = .Lx : .LitInf.Item(k).X2acierLS = .Lx
                            .LitInf.Item(k).L23acier = LacierTot : .LitInf.Item(k).L22acier = 0 : .LitInf.Item(k).L21acier = 0
                        End If
                        msg = " Agl > As1 et 1 seul lit d'armature dimensionné! "
                    Case 2 ' 2 Lits
                        'pour permettre la suite du calcul
                        If appui = 1 Then
                            LacierTot = Truncate(Lappui(1) * 50) / 100
                            For k = 0 To 1
                                .LitInf.Item(k).X1acier = 0 : .LitInf.Item(k).X1acierLS = 0
                                .LitInf.Item(k).L13acier = LacierTot : .LitInf.Item(k).L12acier = 0 : .LitInf.Item(k).L11acier = 0
                            Next
                        Else
                            LacierTot = Truncate(Lappui(2) * 50) / 100
                            For k = 0 To 1
                                .LitInf.Item(k).X2acier = .Lx : .LitInf.Item(k).X2acierLS = .Lx
                                .LitInf.Item(k).L23acier = LacierTot : .LitInf.Item(k).L22acier = 0 : .LitInf.Item(k).L21acier = 0
                            Next
                        End If
                        msg = " Agl > As1 + As2 et 2 lits d'armature dimensionnés! "
                End Select
                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                Select Case appui
                    Case 1
                        msg = "gauche"
                    Case 2
                        msg = "droit"
                End Select
                msg = "Ancrage appui About travée n°" & i1.ToString & " :" & Environment.NewLine &
                    " L'ancrage sur appui de rive " & msg & " nécessite soit des armatures complémentaires ou soit l'ancrage du 2ième lit. Disposition non implémentée dans cette version logicielle!"
                Aff.msg_Avertiss.Add(msg)
                '        rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(" Agl > As1 + As2 - Ancrage des 3 lits inférieurs ") : rtf.EndCell() : rtf.EndRow()
                '        'demande au projeteur s'il veut rajouter un lit d'acier ou quitter le programme
                '        If appui = 1 Then msg = "gauche" Else msg = "droit"
                '        Amanquant = agl - .LitInf.Item(0).Aire - .LitInf.Item(1).Aire - .LitInf.Item(2).Aire
                '        i4 = 1 : i5 = Indice_Acier(.LitInf.Item(0).φHA)
                '        Do
                '            u = .LitInf.Item(0).nbar * PI * dio(i4) ^ 2 / 4
                '            If u > Amanquant Then
                '                Select Case appui
                '                    Case 1
                '                        .LitAppG.φHA = dio(i4)
                '                        .LitAppG.Lacier2 = Lscel_droit(0, 4, i1, ELU_ELS, fyd, True) 'pour la partie se trouvant dans la travée
                '                        Call Ancrage_total_lit(appui, i1, 1, Lappui(appui), 5, ELU_ELS, fyd, 0, LacierTot) 'pour la partie se trouvant sur appui
                '                        .LitAppG.Lacier2 = .LitAppG.Lacier2 + LacierTot
                '                    Case 2
                '                        .LitAppD.φHA = dio(i4)
                '                        .LitAppD.Lacier2 = Lscel_droit(0, 5, i1, ELU_ELS, fyd, True) 'pour la partie se trouvant dans la travée
                '                        Call Ancrage_total_lit(appui, i1, 1, Lappui(appui), 6, ELU_ELS, fyd, 0, LacierTot) 'pour la partie se trouvant sur appui
                '                        .LitAppD.Lacier2 = .LitAppD.Lacier2 + LacierTot
                '                End Select
                '                Exit Do
                '            End If
                '            i4 = i4 + 1
                '        Loop While i4 < i5 + 2 '1
                '        If i4 = i5 + 2 Then
                '            msg = "Le rajout de barres sur appui travée " & i1.ToString & " s'avère insuffisant ! - Modifier la géométrie de la poutre !"
                '            Aff.msg_Avertiss.Add(msg)
                '        Else
                '            'Ligne 8
                '            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Acomp") : rtf.EndCell()
                '            rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
                '            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((u * 10000.0).ToString("#,##0.00")) : rtf.EndCell()
                '            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Acier complémentaire glissement ") : rtf.EndCell() : rtf.EndRow()
                '            'cgl = 4
                '        End If
            End If
        End With
        rtf.WriteEndGroup()
    End Sub
    Private Sub Ancrage_appui_Intermediaire(ByVal i1 As Integer, ByVal appui As Integer, ByVal agl As Double, ByVal Lappui() As Double, ByVal FE() As Double,
                                      ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim As_Lit1, l3, fyd, LacierTot As Double
        Dim k As Integer
        Dim msg As String = String.Empty
        fyd = aci.fyk / aci.γsd 'ELU Str = γs
        rtf.WriteStartGroup()
        'rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        With PlanCont.Item(i1 - 1)
            Select Case .Type_Acier_Travée
                Case 1
                    As_Lit1 = aci.As_long(.TS_trav.Item(0).indTS)
                Case 2
                    As_Lit1 = .LitInf.Item(0).Aire / .Ly
            End Select
            If agl < As_Lit1 Then
                k = 0
                Select Case appui
                    Case 1 'Appui Gauche
                        Select Case .Type_Acier_Travée
                            Case 1
                                .TS_trav.Item(k).X1acier = 0 : .TS_trav.Item(k).X1acierLS = 0
                                If agl <= 0 Then
                                    'Disposition constructive seulement

                                    l3 = Truncate(10 * aci.Ølong(.TS_trav.Item(0).indTS) * 100) / 100
                                    If l3 > Lappui(appui) Then l3 = Lappui(appui)
                                    .TS_trav.Item(k).L13acier = l3 : .TS_trav.Item(k).L12acier = 0 : .TS_trav.Item(k).L11acier = 0
                                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Ancrage du Lit n°1 sur valeur constructive (10.Ø)") : rtf.EndCell() : rtf.EndRow()
                                Else
                                    fyd = FE(appui) / aci.As_long(.TS_trav.Item(0).indTS)
                                    Call Ancrage_lit(appui, i1, k + 1, Lappui(appui), 1, 1, fyd, 0, LacierTot)
                                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Agl < As1  -> Lit n°1 totalement ancré") : rtf.EndCell() : rtf.EndRow()
                                End If
                            Case 2
                                .LitInf.Item(k).X1acier = 0 : .LitInf.Item(k).X1acierLS = 0
                                If agl <= 0 Then
                                    'Disposition constructive seulement
                                    l3 = Truncate(10 * .LitInf.Item(k).φHA * 100) / 100
                                    If l3 > Lappui(appui) Then l3 = Lappui(appui)
                                    .LitInf.Item(k).L13acier = l3 : .LitInf.Item(k).L12acier = 0 : .LitInf.Item(k).L11acier = 0
                                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Ancrage du Lit n°1 sur valeur constructive (10.Ø)") : rtf.EndCell() : rtf.EndRow()
                                Else
                                    fyd = FE(appui) / (.LitInf.Item(0).Aire / .Ly)
                                    Call Ancrage_lit(appui, i1, k + 1, Lappui(appui), 1, 1, fyd, 0, LacierTot)
                                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Agl < As1  -> Lit n°1 totalement ancré") : rtf.EndCell() : rtf.EndRow()
                                End If
                        End Select
                    Case 2 'Appui Droit
                        Select Case .Type_Acier_Travée
                            Case 1
                                If agl <= 0 Then
                                    'Disposition constructive seulement
                                    l3 = Truncate(10 * aci.Ølong(.TS_trav.Item(0).indTS) * 100) / 100
                                    If l3 > Lappui(appui) Then l3 = Lappui(appui)
                                    .TS_trav.Item(k).L23acier = l3 : .TS_trav.Item(k).L22acier = 0 : .TS_trav.Item(k).L21acier = 0 : LacierTot = l3
                                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Ancrage du Lit n°1 sur valeur constructive (10.Ø)") : rtf.EndCell() : rtf.EndRow()
                                Else
                                    fyd = FE(appui) / aci.As_long(.TS_trav.Item(0).indTS)
                                    Call Ancrage_lit(appui, i1, k + 1, Lappui(appui), 1, 1, fyd, 0, LacierTot)
                                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Agl < As1  -> Lit n°1 totalement ancré") : rtf.EndCell() : rtf.EndRow()
                                End If
                            Case 2
                                If agl <= 0 Then
                                    'Disposition constructive seulement
                                    l3 = Truncate(10 * .LitInf.Item(k).φHA * 100) / 100
                                    If l3 > Lappui(appui) Then l3 = Lappui(appui)
                                    .LitInf.Item(k).L23acier = l3 : .LitInf.Item(k).L22acier = 0 : .LitInf.Item(k).L21acier = 0 : LacierTot = l3
                                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Ancrage du Lit n°1 sur valeur constructive (10.Ø)") : rtf.EndCell() : rtf.EndRow()
                                Else
                                    fyd = FE(appui) / (.LitInf.Item(0).Aire / .Ly)
                                    Call Ancrage_lit(appui, i1, k + 1, Lappui(appui), 1, 1, fyd, 0, LacierTot)
                                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Agl < As1  -> Lit n°1 totalement ancré") : rtf.EndCell() : rtf.EndRow()
                                End If
                        End Select

                End Select

                '___________________________________________________________________________________________________________________________________________
                ' "Ancrage appui Intermediaire travée n° i1 
                ' L'ancrage sur appui intemédiaire - nécessite des armatures complémentaires. Disposition non implémentée dans cette version logicielle!"
            Else
                Dim nblit As Integer
                Select Case .Type_Acier_Travée
                    Case 1
                        nblit = .TS_trav.Count
                    Case 2
                        nblit = .LitInf.Count
                End Select
                Select Case nblit
                    Case 1 '1 lit
                        'pour permettre la suite du calcul
                        Select Case .Type_Acier_Travée
                            Case 1
                                If appui = 1 Then
                                    l3 = Truncate(Lappui(1) * 50) / 100
                                    .TS_trav.Item(k).X1acier = 0 : .TS_trav.Item(k).X1acierLS = 0
                                    .TS_trav.Item(k).L13acier = l3 : .TS_trav.Item(k).L12acier = 0 : .TS_trav.Item(k).L11acier = 0
                                Else
                                    l3 = Truncate(Lappui(2) * 50) / 100
                                    .TS_trav.Item(k).X2acier = .Lx : .TS_trav.Item(k).X2acierLS = .Lx
                                    .TS_trav.Item(k).L23acier = l3 : .TS_trav.Item(k).L22acier = 0 : .TS_trav.Item(k).L21acier = 0
                                End If
                            Case 2
                                If appui = 1 Then
                                    l3 = Truncate(Lappui(1) * 50) / 100
                                    .LitInf.Item(k).X1acier = 0 : .LitInf.Item(k).X1acierLS = 0
                                    .LitInf.Item(k).L13acier = l3 : .LitInf.Item(k).L12acier = 0 : .LitInf.Item(k).L11acier = 0
                                Else
                                    l3 = Truncate(Lappui(2) * 50) / 100
                                    .LitInf.Item(k).X2acier = .Lx : .LitInf.Item(k).X2acierLS = .Lx
                                    .LitInf.Item(k).L23acier = l3 : .LitInf.Item(k).L22acier = 0 : .LitInf.Item(k).L21acier = 0
                                End If
                        End Select

                        msg = " Agl > As1 et 1 seul lit d'armature dimensionné! "
                    Case 2 ' 2 Lits
                        'pour permettre la suite du calcul
                        Select Case .Type_Acier_Travée
                            Case 1
                                If appui = 1 Then
                                    l3 = Truncate(Lappui(1) * 50) / 100
                                    For k = 0 To 1
                                        .TS_trav.Item(k).X1acier = 0 : .TS_trav.Item(k).X1acierLS = 0
                                        .TS_trav.Item(k).L13acier = l3 : .TS_trav.Item(k).L12acier = 0 : .TS_trav.Item(k).L11acier = 0
                                    Next
                                Else
                                    l3 = Truncate(Lappui(2) * 50) / 100
                                    For k = 0 To 1
                                        .TS_trav.Item(k).X2acier = .Lx : .TS_trav.Item(k).X2acierLS = .Lx
                                        .TS_trav.Item(k).L23acier = l3 : .TS_trav.Item(k).L22acier = 0 : .TS_trav.Item(k).L21acier = 0
                                    Next
                                End If
                            Case 2
                                If appui = 1 Then
                                    l3 = Truncate(Lappui(1) * 50) / 100
                                    For k = 0 To 1
                                        .LitInf.Item(k).X1acier = 0 : .LitInf.Item(k).X1acierLS = 0
                                        .LitInf.Item(k).L13acier = l3 : .LitInf.Item(k).L12acier = 0 : .LitInf.Item(k).L11acier = 0
                                    Next
                                Else
                                    l3 = Truncate(Lappui(2) * 50) / 100
                                    For k = 0 To 1
                                        .LitInf.Item(k).X2acier = .Lx : .LitInf.Item(k).X2acierLS = .Lx
                                        .LitInf.Item(k).L23acier = l3 : .LitInf.Item(k).L22acier = 0 : .LitInf.Item(k).L21acier = 0
                                    Next
                                End If
                        End Select
                        msg = " Agl > As1 + As2 et 2 lits d'armature dimensionnés! "
                End Select
                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                Select Case appui
                    Case 1
                        msg = "gauche"
                    Case 2
                        msg = "droit"
                End Select
                msg = "Ancrage appui Intermediaire travée n°" & i1.ToString & " :" & Environment.NewLine &
                    " L'ancrage sur appui intemédiaire " & msg & " nécessite soit des armatures complémentaires ou soit l'ancrage du 2ième lit. Disposition non implémentée dans cette version logicielle!"
                Aff.msg_Avertiss.Add(msg)
            End If
        End With
        rtf.WriteEndGroup()
    End Sub
    Private Sub Frettage(ByVal verif As Boolean, ByVal i1 As Integer, ByVal appui As Integer, ByVal la() As Double, ByVal v1 As Double,
                         ByVal v2 As Double, ByVal gamaS As Single)
        'Suivant art. A.5.1,315 et DAVIDOVICI (bielle élargie p266) 
        'rajout de boucles HA sur hauteur de la poutre
        'Variables: HaFrettage(12, 2)= diamètre des aciers de frettage
        '           LFrettage(12, 2)= longueur de scellement du frettage
        '           LTFrettage(12, 2) = longueur totale du frettage
        '           nbBoucleFrettage(12,2) = nombre de boucles de frettage par appui
        'Dim i, i2, j As Integer
        'Dim VuFrettage, L_appui, As_Frettage, pas, V, v3 As Double
        'Dim msg As String
        'If appui = 1 Then V = v1 Else V = v2
        'L_appui = 2 * V / (b(i1) * TauAppB(ELU_ELS))
        'If L_appui > 2 * la(appui) / Sqrt(2) Then
        '    Dim F_L_appui, F_L_appui1 As String
        '    F_L_appui = Format(L_appui * 100, "##0.0")
        '    F_L_appui1 = Format((2 * la(appui) / Sqrt(2)) * 100, "##0.0")
        '    msg = "La largeur d'appui élargie (= " & F_L_appui1 & " cm) est insuffisante pour reprendre" & vbCrLf & _
        '    "l'effort tranchant d'appui. Celui-ci nécessiterait une longueur minimale de " & F_L_appui & "cm." & vbCrLf & _
        '    "Cliquez sur OK pour arrêter le calcul sinon cliquez sur Annuler"
        '    Dim reponse As MsgBoxResult
        '    reponse = MsgBox(msg, MsgBoxStyle.Critical Or MsgBoxStyle.OkCancel, "Module de calcul BAEL: Frettage")
        '    If reponse = MsgBoxResult.Ok Then verif = False
        '    Exit Sub
        'End If
        ''calcul de l'acier de frettage
        ''ce dernier sera placé à la(appui)+2cm au dessus du premier lit d'acier longitudinal
        'VuFrettage = 2 * V / 3
        'As_Frettage = VuFrettage / (fe / gamaS)
        'i2 = Indice_Acier(phi(i1, 1, 1))
        'For j = 1 To i2 - 1
        '    If As_Frettage < 2 * PI * dio(j) ^ 2 / 4 Then
        '        nbBoucleFrettage(i1, appui) = 1
        '        Exit For
        '    End If
        'Next j
        'If j = i2 Then
        '    For j = 1 To i2 - 1
        '        If As_Frettage < 4 * PI * dio(j) ^ 2 / 4 Then
        '            nbBoucleFrettage(i1, appui) = 2
        '            Exit For
        '        End If
        '    Next j
        'End If
        'HaFrettage(i1, appui) = dio(j)
        'LFrettage(i1, appui) = (Int((0.04 + ls(j)) * 100)) / 100
        'LTFrettage(i1, appui) = LFrettage(i1, appui) * 2 + (la(appui) + 0.02) * 2 + b(i1) - 2 * enr(i1)
        ' ''si calcul par la méthode Davidovici
        ' ''augmentation de la largeur par pas de 5 cm
        ''i = 1 : pas = 0.05
        ''Do
        ''    L_appui = la(appui) + (i * pas / Sqrt(2))
        ''    sigBapp(ELU_ELS, i1, appui) = 2 * V / (b(i1) * L_appui)
        ''    If sigBapp(ELU_ELS, i1, appui) < TauAppB(ELU_ELS) Then
        ''        nbBoucleFrettage(i1, appui) = i
        ''        v3 = TauAppB(ELU_ELS) * (b(i1) * la(appui)) / 2
        ''        VuFrettage = v3 - V
        ''        As_Frettage = VuFrettage / (fe / gamaS)
        ''        For j = 1 To 4
        ''            If As_Frettage < 2 * PI * dio(j) ^ 2 / 4 Then Exit For
        ''        Next j
        ''        HaFrettage(i1, appui) = dio(j)
        ''        LFrettage(i1, appui) = (Int((la(appui) + 0.02 + ls(j)) * 100)) / 100
        ''        LTFrettage(i1, appui) = LFrettage(i1, appui) * 2 + b(i1) - 2 * enr(i1)
        ''        Exit Sub
        ''    End If
        ''    i = i + 1
        ''Loop While i * pas < la(appui)

    End Sub
    ''' <summary>Longueur de scellement droit de l'acier</summary>
    ''' <param name="nLit">nlit = index du n° de lit de l'armature = n° de lit - 1 car base = 0 et non 1 !!!!</param>
    ''' <param name="ELU_ELS">type de combinaison appliquée: ELU fondamental (Str), Accidentel,  </param>
    ''' <param name="i_trav">n° de travée</param>
    ''' <param name="j">type de lit (inf =1, sup gauche =2, sup droit =3, appui haut et bas=4, acier de glissement appui gauche =5, acier de glissement appui droit =6, inf acier transversal = 7)</param>
    ''' <param name="σsd">taux de contrainte appliqué sur l'armature</param>
    ''' <param name="Partie_Courante">Permet de déterminer la valeur du coef α4 pour les TS, réduit si vrai sinon pris égal à 1 </param>
    Private Function Lscel_droit(ByVal nLit As Integer, ByVal j As Integer, ByVal i_trav As Integer, ByVal ELU_ELS As Integer, ByVal σsd As Double, ByVal Partie_courante As Boolean) As Double
        '-----------------------------------------------------------------------
        '   longueurs d'ancrage droit total - EN 1992-1-1 A8.4.1 à A8.4.4 -  'longueur de scellement droit des aciers de 6 à 40 mm de diamètre
        '-----------------------------------------------------------------------
        'nlit = index du n° de lit de l'armature = n° de lit - 1 car base = 0 et non 1 !!!!
        'i_trav = n° de travée
        'ELU_ELS = 1 si ELU fondamental, =2 si Accidentel
        'σsd = contrainte sur l'acier, peut être supérieur à fyk / γs si utilisation branche inclinée
        'h = hauteur totale de la dalle
        'h1 = hauteur préfa de la dalle = prédalle - sert à déterminer qualité adhérence
        'enrLat = enrobage latéral
        'enrinf = enrobage inférieur ou supérieur
        'esp = espacement entre armature
        'K_position: pour la détermination de k - =0 si pas de prise en compte du coef α3, =1 pour k=0.1; =2 si k=0.05 voir figure 8.4
        'Partie_courante = Vrai partie courante de la poutre = prise en compte des aciers d'effort tranchant, Faux = sur appui
        Dim Cd, esp, fbd, fctk, K, Lbd, Lbmin, Lbrqd, α1, α2, α3, α4, η1, η2, γc, φbarre As Double ', φbarreTrans

        With PlanCont.Item(i_trav - 1)
            Select Case j
                Case 1 'armature en travée
                    If .Type_Acier_Travée = 1 Then 'TS
                        φbarre = aci.Ølong(.TS_trav.Item(nLit).indTS)
                    Else 'armatures sous forme de barres HA
                        φbarre = .LitInf.Item(nLit).φHA
                    End If
                    'condition de bétonnage
                    If .Prefa_h = 0 Then
                        If .H > 0.25 Then η1 = 0.7 Else η1 = 1
                    ElseIf .Prefa_h <= 0.25 Then
                        η1 = 1
                    Else
                        η1 = 0.7
                    End If
                Case 2  'armature sur appui gauche
                    If .Type_Acier_Appui_Gauche = 1 Then 'TS
                        φbarre = aci.Ølong(.TS_AppG.Item(nLit).indTS)
                    Else 'armatures sous forme de barres HA
                        φbarre = .LitSupG.Item(nLit).φHA
                        ' If .LitSupG_Y.Count >= 1 Then φbarreTrans = .LitSupG_Y.Item(nLit).φHA Else φbarreTrans = 0
                    End If
                    η1 = 1 'conditions d'adhérence toujours bonne
                Case 3 'armature sur appui droit
                    If .Type_Acier_Appui_Droit = 1 Then 'TS
                        φbarre = aci.Ølong(.TS_AppD.Item(nLit).indTS)
                    Else 'armatures sous forme de barres HA
                        φbarre = .LitSupD.Item(nLit).φHA
                        '  If .LitSupD_Y.Count >= 1 Then φbarreTrans = .LitSupD_Y.Item(nLit).φHA Else φbarreTrans = 0
                    End If
                    η1 = 1 'conditions d'adhérence toujours bonne
                Case 4 'armatures sur appui haut ou bas
                    If .Type_Acier_Travée = 1 Then 'TS
                        If .TS_AppH.Count >= 1 Then φbarre = aci.Ølong(.TS_AppH.Item(0).indTS)
                        If .TS_AppB.Count >= 1 Then φbarre = aci.Ølong(.TS_AppB.Item(0).indTS)
                    Else
                        If .LitSupH.Count >= 1 Then φbarre = .LitSupH.Item(0).φHA
                        If .LitSupB.Count >= 1 Then φbarre = .LitSupB.Item(0).φHA
                    End If
                    η1 = 1 'conditions d'adhérence toujours bonne
                Case 5 'acier de glissement appui gauche
                    φbarre = .LitAppG.φHA
                    If .Prefa_h = 0 Then
                        If .H > 0.25 Then η1 = 0.7 Else η1 = 1
                    ElseIf .Prefa_h <= 0.25 Then
                        η1 = 1
                    Else
                        η1 = 0.7
                    End If
                Case 6 'acier de glissement appui droit
                    φbarre = .LitAppD.φHA
                    If .Prefa_h = 0 Then
                        If .H > 0.25 Then η1 = 0.7 Else η1 = 1
                    ElseIf .Prefa_h <= 0.25 Then
                        η1 = 1
                    Else
                        η1 = 0.7
                    End If
                Case 7 'armature en travée - acier transversal
                    If .Type_Acier_Travée = 1 Then 'TS
                        φbarre = aci.Øtrans(.TS_trav.Item(nLit).indTS)
                    Else 'armatures sous forme de barres HA
                        φbarre = .LitInf_Y.Item(nLit).φHA
                    End If
                    'condition de bétonnage
                    If .Prefa_h = 0 Then
                        If .H > 0.25 Then η1 = 0.7 Else η1 = 1
                    ElseIf .Prefa_h <= 0.25 Then
                        η1 = 1
                    Else
                        η1 = 0.7
                    End If
            End Select
            If φbarre > 0.032 Then
                η2 = (132 - (φbarre * 1000)) / 100
            Else
                η2 = 1
            End If
            fctk = bet.fctk05
            If bet.fctk05 > 3.1 Then fctk = 3.1
            Select Case ELU_ELS
                Case 1
                    γc = bet.γcd ' : γs = aci.γsd 'combinaison fondamentale
                Case 2
                    γc = bet.γca ' : γs = aci.γsa 'combinaison acidentelle
            End Select
            fbd = 2.25 * η1 * η2 * fctk / γc
            Lbrqd = (φbarre / 4) * (σsd / fbd)
            'forme des barres
            α1 = 1 'scellement droit
            'enrobage
            Select Case j
                Case 1 'armature en travée
                    Cd = .enrInf 'les aciers porteurs sont au plus prés de la face
                    If .Type_Acier_Travée = 1 Then 'TS
                        esp = aci.Esp_Long(.TS_trav.Item(nLit).indTS)
                    Else
                        esp = .LitInf(0).Scourant
                    End If

                Case 2  'armature sur appui
                    Cd = .enrSup + 0.01 'les aciers porteurs sont au plus prés de la face
                    If .Type_Acier_Appui_Gauche = 1 Then 'TS
                        esp = aci.Esp_Long(.TS_AppG.Item(nLit).indTS)
                    Else
                        esp = .LitSupG.Item(nLit).Scourant
                    End If
                Case 3 'armature sur appui
                    Cd = .enrSup + 0.01 'les aciers porteurs sont au plus prés de la face
                    If .Type_Acier_Appui_Droit = 1 Then 'TS
                        esp = aci.Esp_Long(.TS_AppD.Item(nLit).indTS)
                    Else
                        esp = .LitSupD.Item(nLit).Scourant
                    End If
                Case 4 'armature sur appui haut bas n
                    Cd = Min(.enrSup + 0.01, .enrLat + 0.01)
                    If .Type_Acier_Travée = 1 Then 'TS
                        If .LappH > 0.05 Then
                            esp = aci.Esp_Long(.TS_AppH.Item(nLit).indTS)
                        Else
                            esp = aci.Esp_Long(.TS_AppB.Item(nLit).indTS)
                        End If
                    Else
                        If .LappH > 0.05 Then
                            esp = .LitSupH.Item(nLit).Scourant
                        Else
                            esp = .LitSupB.Item(nLit).Scourant
                        End If
                    End If
                Case 5, 6 'Acier glissement appui gauche
                    Cd = .enrLat + 0.01 : esp = 200
                Case 7 'armature en travée - acier transversal
                    Cd = .enrInf + 0.01
                    If .Type_Acier_Travée = 1 Then 'TS
                        esp = aci.Esp_trans(.TS_trav.Item(nLit).indTS)
                    Else
                        esp = .LitInf_Y(0).Scourant
                    End If
            End Select
            Cd = Min(esp / 2, Cd)
            α2 = 1 - 0.15 * (Cd - φbarre) / φbarre
            If α2 < 0.7 Then α2 = 0.7
            If α2 > 1 Then α2 = 1
            'Confinement des armatures
            α3 = 1 'armatures non soudès : 
            α4 = 1 'de base, éventuellement modifié si TS 
            'Confinement des armatures soudès
            Select Case j
                Case 1
                    If .Type_Acier_Travée = 1 Then 'TS
                        Select Case Partie_courante
                            Case True 'en partie courante
                                α4 = 0.7 'méthode forfaitaire
                            Case False 'sur appui
                                α4 = 1
                        End Select
                    End If
                Case 2
                    If .Type_Acier_Appui_Gauche = 1 Then 'TS
                        Select Case Partie_courante
                            Case True 'sur appui avec aciers transversaux non coupés
                                α4 = 0.7 'méthode forfaitaire
                            Case False 'sur appui avec aciers transversaux  coupés
                                α4 = 1
                        End Select
                    End If
                Case 3
                    If .Type_Acier_Appui_Droit = 1 Then 'TS
                        Select Case Partie_courante
                            Case True 'sur appui avec aciers transversaux non coupés
                                α4 = 0.7 'méthode forfaitaire
                            Case False 'sur appui avec aciers transversaux  coupés
                                α4 = 1
                        End Select
                    End If
                Case 4 'pour les aciers sur appuis haut et bas
                    If .Type_Acier_Travée = 1 Then 'TS
                        Select Case Partie_courante
                            Case True ''sur appui avec aciers transversaux non coupés
                                α4 = 0.7 'méthode forfaitaire
                            Case False 'sur appui avec aciers transversaux  coupés
                                α4 = 1
                        End Select
                    End If
            End Select

            Lbd = α1 * α2 * α3 * α4 * Lbrqd '(eq 8.4)

        End With

        Lbmin = Max(0.3 * Lbrqd, 10 * φbarre)
        If Lbmin < 0.1 Then Lbmin = 0.1
        If Lbd < Lbmin Then Lbd = Lbmin
        Lscel_droit = Truncate(Lbd * 100) / 100 'élimination des chiffres aprés le cm

    End Function
    Private Sub Ancrage_courbe(ByVal lappui As Double, ByVal nlit As Integer, ByVal j As Integer, ByVal i_trav As Integer, ByVal ELU_ELS As Integer, ByVal σsd As Double,
                                   ByRef l1 As Double, ByRef l2 As Double, ByRef l3 As Double, ByRef Verif As Boolean, Optional ByVal appui As Integer = 1)

        '----------------------------------------------------------------------------
        '   ancrage  courbe sur appui
        '----------------------------------------------------------------------------
        'j = type d'acier : 1 acier inf, 2 acier sup gauche, 3 acier sup droit
        'nlit = numéro de lit de l'acier
        'i_trav = n° de travée
        'ELU_ELS = 1 si ELU fondamental, =2 si Accidentel
        'σsd = contrainte sur l'acier, peut être supérieur à fyk / γs si utilisation branche inclinée
        'h = hauteur totale de la poutre
        'h1 = hauteur préfa de la poutre - sert à déterminer qualité adhérence
        'enrLat = enrobage latéral
        'enrinf = enrobage inférieur ou supérieur
        'esp = espacement entre armature
        Dim a01, angl, Cd, esp, Fu, fbt, fed, fbd, fcd, fctk, Lbd, Lbmin, Lbrqd, r, α1, α2, α3, η1, η2, γc, φm1, φbarre, φbarreTrans As Double ' φm2,a02,
        Dim index As Integer
        With PlanCont.Item(i_trav - 1)
            Select Case j
                Case 1 'armature en travée
                    If .Type_Acier_Travée = 1 Then 'TS
                        φbarre = aci.Ølong(.TS_trav.Item(0).indTS)
                    Else 'armatures sous forme de barres HA
                        φbarre = .LitInf.Item(nlit).φHA
                        φbarreTrans = .LitInf_Y.Item(nlit).φHA
                        If appui > 2 Then φbarre = .LitInf_Y.Item(nlit).φHA
                    End If
                        'condition de bétonnage
                        If .Prefa_h = 0 Then
                        If .H > 0.25 Then η1 = 0.7 Else η1 = 1
                    ElseIf .Prefa_h <= 0.25 Then
                        η1 = 1
                    Else
                        η1 = 0.7
                    End If
                Case 2  'armature sur appui gauche
                    If .Type_Acier_Appui_Gauche = 1 Then 'TS
                        φbarre = aci.Ølong(.TS_AppG.Item(0).indTS)
                    Else 'armatures sous forme de barres HA
                        φbarre = .LitSupG.Item(nlit).φHA
                        If .LitSupG_Y.Count >= 1 Then φbarreTrans = .LitSupG_Y.Item(nlit).φHA Else φbarreTrans = 0
                    End If
                    η1 = 1 'conditions d'adhérence toujours bonne
                Case 3 'armature sur appui droit
                    If .Type_Acier_Appui_Droit = 1 Then 'TS
                        φbarre = aci.Ølong(.TS_AppD.Item(0).indTS)
                    Else 'armatures sous forme de barres HA
                        φbarre = .LitSupD.Item(nlit).φHA
                        If .LitSupD_Y.Count >= 1 Then φbarreTrans = .LitSupD_Y.Item(nlit).φHA Else φbarreTrans = 0
                    End If
                    η1 = 1 'conditions d'adhérence toujours bonne
                Case 4 'armatures sur appui haut ou bas
                    If .Type_Acier_Travée = 1 Then 'TS
                        If .TS_AppH.Count >= 1 Then φbarre = aci.Ølong(.TS_AppH.Item(0).indTS)
                        If .TS_AppB.Count >= 1 Then φbarre = aci.Ølong(.TS_AppB.Item(0).indTS)
                    Else
                        If .LitSupH.Count >= 1 Then φbarre = .LitSupH.Item(0).φHA
                        If .LitSupB.Count >= 1 Then φbarre = .LitSupB.Item(0).φHA
                    End If
                    η1 = 1 'conditions d'adhérence toujours bonne
                Case 5 'acier de glissement appui gauche
                    φbarre = .LitAppG.φHA
                    If .Prefa_h = 0 Then
                        If .H > 0.25 Then η1 = 0.7 Else η1 = 1
                    ElseIf .Prefa_h <= 0.25 Then
                        η1 = 1
                    Else
                        η1 = 0.7
                    End If
                Case 6 'acier de glissement appui droit
                    φbarre = .LitAppD.φHA
                    If .Prefa_h = 0 Then
                        If .H > 0.25 Then η1 = 0.7 Else η1 = 1
                    ElseIf .Prefa_h <= 0.25 Then
                        η1 = 1
                    Else
                        η1 = 0.7
                    End If
            End Select
            If φbarre > 0.032 Then
                η2 = (132 - (φbarre * 1000)) / 100
            Else
                η2 = 1
            End If
            fctk = bet.fctk05
            If bet.fctk05 > 3.1 Then fctk = 3.1
            Select Case ELU_ELS
                Case 1
                    γc = bet.γcd ' : γs = aci.γsd 'combinaison fondamentale
                    fcd = bet.fck / γc 'pour calcul de φm
                Case 2
                    γc = bet.γca ' : γs = aci.γsa 'combinaison acidentelle
                    fcd = bet.fck / γc 'pour calcul de φm
            End Select
            If fcd > 55 / γc Then fcd = 55 / γc 'limitation à classe C55/67
            fbd = 2.25 * η1 * η2 * fctk / γc
            Lbrqd = (φbarre / 4) * (σsd / fbd)
            'enrobage
            Select Case j
                Case 1 'armature en travée
                    If .Type_Acier_Travée = 1 Then 'TS
                        esp = aci.Esp_Long(.TS_trav.Item(0).indTS)
                    Else
                        esp = 1 - 2 * 0.01 - 2 * .enrLat - .LitInf.Item(nlit).nbar * .LitInf.Item(nlit).φHA
                        esp = esp / (.LitInf.Item(nlit).nbar - 1)
                    End If
                Case 2  'armature sur appui
                    If .Type_Acier_Appui_Gauche = 1 Then 'TS
                        esp = aci.Esp_Long(.TS_AppG.Item(0).indTS)
                    Else
                        esp = 1 - 2 * 0.01 - 2 * .enrLat - .LitSupG.Item(nlit).nbar * .LitSupG.Item(nlit).φHA
                        esp = esp / (.LitSupG.Item(nlit).nbar - 1)
                    End If
                Case 3 'armature sur appui
                    If .Type_Acier_Appui_Droit = 1 Then 'TS
                        esp = aci.Esp_Long(.TS_AppD.Item(0).indTS)
                    Else
                        esp = 1 - 2 * 0.01 - 2 * .enrLat - .LitSupD.Item(nlit).nbar * .LitSupD.Item(nlit).φHA
                        esp = esp / (.LitSupD.Item(nlit).nbar - 1)
                    End If
                Case 4
                    If .Type_Acier_Travée = 1 Then 'TS
                        If .TS_AppH.Count >= 1 Then esp = aci.Esp_Long(.TS_AppH.Item(0).indTS)
                        If .TS_AppB.Count >= 1 Then esp = aci.Esp_Long(.TS_AppB.Item(0).indTS)
                    Else
                        If .LitSupH.Count >= 1 Then
                            esp = 1 - 2 * 0.01 - 2 * .enrLat - .LitSupH.Item(0).nbar * .LitSupH.Item(0).φHA
                            esp = esp / (.LitSupH.Item(0).nbar - 1)
                        End If
                        If .LitSupB.Count >= 1 Then
                            esp = 1 - 2 * 0.01 - 2 * .enrLat - .LitSupB.Item(0).nbar * .LitSupB.Item(0).φHA
                            esp = esp / (.LitSupB.Item(0).nbar - 1)
                        End If
                    End If
                Case 5, 6 'Acier glissement appui gauche
                    esp = 200
            End Select
            Cd = Min(esp / 2, .enrLat + 0.01)
            'forme des barres
            If Cd > 3 * φbarre Then α1 = 0.7 Else α1 = 1
            'enrobage
            α2 = 1 - 0.15 * (Cd - 3 * φbarre) / φbarre
            If α2 < 0.7 Then α2 = 0.7
            If α2 > 1 Then α2 = 1
            α3 = 1 'pas de cadre sur appui donc pas de confinement
            ' α4 = 1
            'Confinement par compression transversale
            '  α5 = 1 - 0.04 * σcm
            Lbd = α1 * α2 * Lbrqd '(eq 8.4)
            Lbmin = Max(0.3 * Lbrqd, 10 * φbarre)
            If Lbmin < 0.1 Then Lbmin = 0.1
            If Lbd < Lbmin Then Lbd = Lbmin
            If .Type_Acier_Travée = 1 Then 'TS
                l3 = lappui - aci.dmTS.Item(.TS_trav.Item(0).indTS) / 2 - φbarre 'l3=longueur développé sur partie rectiligne aprés partie courbe
            Else
                l3 = lappui - aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre)) / 2 - φbarre
            End If
            If l3 < 0 Then
                Aff.msg_Avertiss.Add("Pb dimension crochet: travée n°" & i_trav.ToString & " - appui n°" & lappui.ToString & ", type acier j = " & j.ToString & " n° de lit: " & nlit.ToString & " - La longueur d'appui n'est pas suffisante pour réaliser le crochet !!! - Il manque " &
                                (Abs(l3 * 100 + 1)).ToString("#") & " cm. Modifiez la longeur d'appui")
                'MessageBox.Show(, "Module calcul ancrage courbe", MessageBoxButtons.OK, MessageBoxIcon.Error)
                'Verif = False : Exit Sub
                l3 = 0
            End If
            angl = aci.angl_Arma * PI / 180 'conversion de l'angle en radian
            Select Case j
                Case 1 'armature en travée
                    If .Type_Acier_Travée = 1 Then 'TS
                        r = (aci.dmTS.Item(.TS_trav.Item(nlit).indTS) / 2 + φbarre)
                        '    If appui = 1 Then .TS_trav.Item(nlit).φDM1 = 2 * r Else .TS_trav.Item(nlit).φDM2 = 2 * r
                        Select Case appui
                            Case 1
                                .TS_trav.Item(nlit).φDM1 = aci.dmTS.Item(.TS_trav.Item(nlit).indTS)
                            Case 2
                                .TS_trav.Item(nlit).φDM2 = aci.dmTS.Item(.TS_trav.Item(nlit).indTS)
                            Case 3
                                .TS_trav.Item(nlit).φDM1 = aci.dmTS.Item(.TS_trav.Item(nlit).indTS)
                            Case 4
                                .TS_trav.Item(nlit).φDM1 = aci.dmTS.Item(.TS_trav.Item(nlit).indTS)
                        End Select
                    Else
                        r = (aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre)) / 2 + φbarre) 'Le calcul se fait sur l'axe
                        '   If appui = 1 Then .LitInf.Item(nlit).φDM1 = 2 * r Else .LitInf.Item(nlit).φDM2 = 2 * r
                        Select Case appui
                            Case 1
                                .LitInf.Item(nlit).φDM1 = aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre))
                            Case 2
                                .LitInf.Item(nlit).φDM2 = aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre))
                            Case 3
                                .LitInf.Item(nlit).φDM1 = aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre))
                            Case 4
                                .LitInf.Item(nlit).φDM1 = aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre))
                        End Select
                    End If

                Case 2 'armature sur appui gauche
                    If .Type_Acier_Appui_Gauche = 1 Then 'TS
                        r = (aci.dmTS.Item(.TS_AppG.Item(0).indTS) / 2 + φbarre) 'Le calcul se fait sur l'axe
                        .TS_AppG.Item(nlit).φDM1 = 2 * r
                    Else
                        r = (aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre)) / 2 + φbarre) 'Le calcul se fait sur l'axe
                        .LitSupG.Item(nlit).φDM1 = aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre)) '2 * r
                    End If

                Case 3 'armature sur appui
                    If .Type_Acier_Appui_Droit = 1 Then 'TS
                        r = (aci.dmTS.Item(.TS_AppD.Item(0).indTS) / 2 + φbarre) 'Le calcul se fait sur l'axe
                        .TS_AppD.Item(nlit).φDM1 = aci.dmTS.Item(.TS_AppD.Item(0).indTS) ' 2 * r
                    Else
                        r = (aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre)) / 2 + φbarre) 'Le calcul se fait sur l'axe
                        .LitSupD.Item(nlit).φDM1 = aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre)) '2 * r
                    End If

                Case 4 'armature sur appui secondaire = même type d'armature qu'en travée
                    If .Type_Acier_Travée = 1 Then 'TS
                        If .TS_AppH.Count >= 1 Then
                            r = (aci.dmTS.Item(.TS_AppH.Item(0).indTS) / 2 + φbarre) 'Le calcul se fait sur l'axe
                            .TS_AppH.Item(0).φDM1 = aci.dmTS.Item(.TS_AppH.Item(0).indTS) '2 * r
                        End If
                        If .TS_AppB.Count >= 1 Then
                            r = (aci.dmTS.Item(.TS_AppB.Item(0).indTS) / 2 + φbarre) 'Le calcul se fait sur l'axe
                            .TS_AppB.Item(0).φDM1 = aci.dmTS.Item(.TS_AppB.Item(0).indTS) ' 2 * r
                        End If
                    Else
                        If .LitSupH.Count >= 1 Then
                            r = (aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre)) / 2 + φbarre) 'Le calcul se fait sur l'axe
                            .LitSupH.Item(0).φDM1 = aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre)) '2 * r
                        End If
                        If .LitSupB.Count >= 1 Then
                            r = (aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre)) / 2 + φbarre) 'Le calcul se fait sur l'axe
                            .LitSupB.Item(0).φDM1 = aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre)) '2 * r
                        End If
                    End If

            End Select

            l2 = angl * r 'l2=longueur développé sur partie courbe
            If l2 + l3 > Lbd Then
                'dans ce cas, l1 est pris égal à une valeur forfaitaire, avec une valeur mini de 5cm
                l1 = 0.05
            Else
                l1 = Lbd - l2 - l3
                l1 = (Truncate(100 * l1 + 1)) / 100 'arrondi au cm
                If l1 > 5 * φbarre Then
                    Select Case j
                        Case 1 'armature en travée
                            If .Type_Acier_Travée = 1 Then 'TS
                                esp = aci.Esp_Long(.TS_trav.Item(0).indTS)
                                fed = σsd * aci.As_long(.TS_trav.Item(0).indTS)
                                Fu = PI * l3 * aci.nbre_fils_N(.TS_trav.Item(0).indTS) * aci.Ølong(.TS_trav.Item(0).indTS) * fbd
                                fbt = (fed - Fu) / aci.nbre_fils_N(.TS_trav.Item(0).indTS)
                            Else
                                fed = σsd * (.LitInf.Item(nlit).φHA ^ 2 / 4) 'pour une barre
                                Fu = PI * l3 * .LitInf.Item(nlit).φHA * fbd 'pour une barre
                                esp = .LitInf.Item(0).Scourant 'pour une barre
                                fbt = (fed - Fu) 'pour une barre
                            End If

                        Case 2  'armature sur appui
                            If .Type_Acier_Appui_Gauche = 1 Then 'TS
                                esp = aci.Esp_Long(.TS_AppG.Item(0).indTS)
                                fed = σsd * aci.As_long(.TS_AppG.Item(0).indTS)
                                Fu = PI * l3 * aci.nbre_fils_N(.TS_AppG.Item(0).indTS) * aci.Ølong(.TS_AppG.Item(0).indTS) * fbd
                                fbt = (fed - Fu) / aci.nbre_fils_N(.TS_AppG.Item(0).indTS)
                            Else
                                esp = .LitSupG.Item(0).Scourant
                                fed = σsd * PI * .LitSupG.Item(nlit).φHA ^ 2 / 4 'pour une barre
                                Fu = PI * l3 * .LitSupG.Item(nlit).φHA * fbd 'pour une barre
                                fbt = (fed - Fu) 'pour une barre
                            End If

                        Case 3 'armature sur appui
                            If .Type_Acier_Appui_Droit = 1 Then 'TS
                                esp = aci.Esp_Long(.TS_AppD.Item(0).indTS)
                                fed = σsd * aci.As_long(.TS_AppD.Item(0).indTS)
                                Fu = PI * l3 * aci.nbre_fils_N(.TS_AppD.Item(0).indTS) * aci.Ølong(.TS_AppD.Item(0).indTS) * fbd
                                fbt = (fed - Fu) / aci.nbre_fils_N(.TS_AppD.Item(0).indTS)
                            Else
                                esp = .LitSupD.Item(0).Scourant
                                fed = σsd * PI * .LitSupD.Item(nlit).φHA ^ 2 / 4 'pour une barre
                                Fu = PI * l3 * .LitSupD.Item(nlit).φHA * fbd 'pour une barre
                                fbt = (fed - Fu) 'pour une barre
                            End If
                        Case 4 'armature sur appui haut et/ou bas
                            If .Type_Acier_Appui_Droit = 1 Then 'TS
                                esp = aci.Esp_Long(.TS_AppH.Item(0).indTS)
                                fed = σsd * aci.As_long(.TS_AppH.Item(0).indTS)
                                Fu = PI * l3 * aci.nbre_fils_N(.TS_AppH.Item(0).indTS) * aci.Ølong(.TS_AppH.Item(0).indTS) * fbd
                                fbt = (fed - Fu) / aci.nbre_fils_N(.TS_AppH.Item(0).indTS)
                            Else
                                esp = .LitSupB.Item(0).Scourant
                                fed = σsd * PI * .LitSupB.Item(0).φHA ^ 2 / 4 'pour une barre
                                Fu = PI * l3 * .LitSupB.Item(0).φHA * fbd 'pour une barre
                                fbt = (fed - Fu) 'pour une barre
                            End If
                        Case 5, 6 'Acier glissement appui gauche
                            esp = 200
                    End Select

                    a01 = esp / 2 'moitié de l'espace entre barres
                    ' a02 = a01
                    φm1 = (1 / a01 + 1 / (2 * φbarre)) * (fbt / fcd)
                    '    φm2 = (1 / a02 + 1 / (2 * φbarre)) * (fbt / fcd)
                    If aci.dmL.Item(aci.Ø_Norm.IndexOf(φbarre)) < φm1 Then 'Max(φm1, φm2)
                        Select Case j
                            Case 1 'armature en travée
                                Select Case appui
                                    Case 1 'appui gauche
                                        Select Case .Type_Acier_Travée
                                            Case 1

                                            Case 2
                                                index = 0 : .LitInf.Item(nlit).φDM1 = 10
                                                Do
                                                    If φm1 < aci.dmR.Item(index) Then .LitInf.Item(nlit).φDM1 = aci.dmR.Item(index) : Exit Do
                                                    index = index + 1
                                                Loop While index < aci.dmR.Count
                                                If .LitInf.Item(nlit).φDM1 = 10 Then
                                                    MessageBox.Show("Pb sur diamètre mandrin", "Module calcul ancrage courbe", MessageBoxButtons.OK, MessageBoxIcon.Error)
                                                    Verif = False : Exit Sub
                                                End If
                                        End Select
                                    Case 2 'appui droit
                                        Select Case .Type_Acier_Travée
                                            Case 1

                                            Case 2
                                                index = 0 : .LitInf.Item(nlit).φDM2 = 10
                                                Do
                                                    If φm1 < aci.dmR.Item(index) Then .LitInf.Item(nlit).φDM2 = aci.dmR.Item(index) : Exit Do
                                                    index = index + 1
                                                Loop While index < aci.dmR.Count
                                                If .LitInf.Item(nlit).φDM2 = 10 Then
                                                    MessageBox.Show("Pb sur diamètre mandrin", "Module calcul ancrage courbe", MessageBoxButtons.OK, MessageBoxIcon.Error)
                                                    Verif = False : Exit Sub
                                                End If
                                        End Select

                                End Select
                                'Egalisation des Dm sur même barre entre appui droit et gauche
                                If appui = 2 Then
                                    Select Case .Type_Acier_Travée
                                        Case 1

                                        Case 2
                                            'si DM<> et existence d'un crochet à droite et à gauche
                                            If .LitInf.Item(nlit).φDM2 <> .LitInf.Item(nlit).φDM1 And .LitInf.Item(nlit).L21acier > 0.01 And .LitInf.Item(nlit).L11acier > 0.01 Then
                                                .LitInf.Item(nlit).φDM1 = Max(.LitInf.Item(nlit).φDM1, .LitInf.Item(nlit).φDM2) : .LitInf.Item(nlit).φDM2 = .LitInf.Item(nlit).φDM1
                                                'modification l1 suivant diamètre mandrin - appui gauche
                                                .LitInf.Item(nlit).L11acier = .LitInf.Item(nlit).L11acier - ((.LitInf.Item(nlit).φDM1 - aci.dmR.Item(index)) / 2)
                                                r = (.LitInf.Item(nlit).φDM1 + φbarre) / 2
                                                l2 = angl * r 'l2=longueur développé sur partie courbe
                                                l1 = Lbd - l2 - l3
                                                l1 = (Truncate(100 * l1 + 1)) / 100 'arrondi au cm
                                                .LitInf.Item(nlit).L13acier = l3
                                                .LitInf.Item(nlit).L12acier = l2
                                                'les modifications sur appui droit se font directement dans le prog appelant
                                            End If
                                    End Select
                                End If
                            Case 2  'armature sur appui
                                Select Case .Type_Acier_Appui_Gauche
                                    Case 1

                                    Case 2
                                        index = 0 : .LitSupG.Item(nlit).φDM1 = 10
                                        Do
                                            If φm1 < aci.dmR.Item(index) Then .LitSupG.Item(nlit).φDM1 = aci.dmR.Item(index) : Exit Do
                                            index = index + 1
                                        Loop While index < aci.dmR.Count
                                        If .LitSupG.Item(nlit).φDM1 = 10 Then
                                            MessageBox.Show("Pb sur diamètre mandrin", "Module calcul ancrage courbe", MessageBoxButtons.OK, MessageBoxIcon.Error)
                                            Verif = False : Exit Sub
                                        End If
                                End Select
                            'les modifications sur appui gauche se font directement dans le prog appelant
                            Case 3 'armature sur appui
                                Select Case .Type_Acier_Appui_Gauche
                                    Case 1

                                    Case 2
                                        index = 0 : .LitSupD.Item(nlit).φDM1 = 10
                                        Do
                                            If φm1 < aci.dmR.Item(index) Then .LitSupD.Item(nlit).φDM1 = aci.dmR.Item(index) : Exit Do
                                            index = index + 1
                                        Loop While index < aci.dmR.Count
                                        If .LitSupD.Item(nlit).φDM1 = 10 Then
                                            MessageBox.Show("Pb sur diamètre mandrin", "Module calcul ancrage courbe", MessageBoxButtons.OK, MessageBoxIcon.Error)
                                            Verif = False : Exit Sub
                                        End If
                                End Select
                            'les modifications sur appui droit se font directement dans le prog appelant
                            Case 4 'armature sur appui haut ou bas - Appuis non porteurs
                                Select Case .Type_Acier_Travée
                                    Case 1 'TS

                                    Case 2 'HA
                                        If .LitSupH.Count >= 1 Then
                                            index = 0 : .LitSupH.Item(0).φDM1 = 10
                                            Do
                                                If φm1 < aci.dmR.Item(index) Then .LitSupH.Item(0).φDM1 = aci.dmR.Item(index) : Exit Do
                                                index = index + 1
                                            Loop While index < aci.dmR.Count
                                            If .LitSupH.Item(0).φDM1 = 10 Then
                                                MessageBox.Show("Pb sur diamètre mandrin", "Module calcul ancrage courbe", MessageBoxButtons.OK, MessageBoxIcon.Error)
                                                Verif = False : Exit Sub
                                            End If
                                        End If
                                        If .LitSupB.Count >= 1 Then
                                            index = 0 : .LitSupB.Item(0).φDM1 = 10
                                            Do
                                                If φm1 < aci.dmR.Item(index) Then .LitSupB.Item(0).φDM1 = aci.dmR.Item(index) : Exit Do
                                                index = index + 1
                                            Loop While index < aci.dmR.Count
                                            If .LitSupB.Item(0).φDM1 = 10 Then
                                                MessageBox.Show("Pb sur diamètre mandrin", "Module calcul ancrage courbe", MessageBoxButtons.OK, MessageBoxIcon.Error)
                                                Verif = False : Exit Sub
                                            End If
                                        End If
                                End Select

                            Case 5, 6 'Acier glissement appui gauche
                                esp = 200
                        End Select
                    End If
                End If
                If l1 < 0.05 Then l1 = 0.05
            End If
        End With

    End Sub
    Private Function DM(ByVal φbarre As Double, Optional ByVal ntrav As Integer = 0) As Double
        'Détermination du diamètre du mandrin
        'With PlanCont.Item(i_trav - 1)
        '    Select Case Aff.Option_Mandrin
        '  Case 1
        '    If .Type_Acier_Travée = 1 Then 'TS
        '        Select Case φbarre
        '            Case Is <= 0.016 ' Ø=<16mm
        '                DM = 4 * φbarre
        '            Case Else ' Ø>16mm
        '                DM = 7 * φbarre
        '        End Select
        '    Else 'armatures sous forme de barres HA
        '        Select Case φbarre
        '            Case Is <= 0.016 ' Ø=<16mm
        '                DM = 4 * φbarre
        '            Case Else ' Ø>16mm
        '                DM = 7 * φbarre
        '        End Select
        '    End If

        'Case Else 'i=2
        '    DM = 10 * φbarre
        '    Aff.msg_Avertiss.Add("Calcul du diamètre du mandrin suivant eq 8.1 non implémenté")
        '    End Select
        'End With

    End Function

    ''' <summary>
    ''' ancrage total du lit  -  ELU 
    ''' </summary>
    ''' <param name="ELU_ELS">type de combinaison appliquée: ELU fondamental (Str), Accidentel,  </param>
    ''' <param name="i1">n° de travée</param>
    ''' <param name="appui">appui=1->appuigauche, 2->droit</param>
    ''' <param name="j">type de lit (inf, sup gauche, sup dro</param>
    ''' <param name="calcul_Ancrage">indique si des aciers supplémentaires sont rajoutées pour satisfaire aux conditions d'appui (5) ou si des aciers des lits en travée sont simplement prolongés sur l'appui  </param>
    ''' <param name="σsd">taux de contrainte appliqué sur l'armature</param>
    ''' <param name="LacierTot">longueur totale de l'armature</param>
    ''' <param name="Partie_Courante">Permet de déterminer la valeur du coef α4, réduit si vrai sinon pris égal à 1 </param>
    Private Sub Ancrage_lit(ByVal appui As Integer, ByVal i1 As Integer, ByVal calcul_Ancrage As Integer, ByVal lappui As Double, ByVal j As Integer,
                            ByVal ELU_ELS As Integer, ByVal σsd As Double, ByVal additif As Integer, Optional ByRef LacierTot As Double = 0, Optional ByVal Partie_Courante As Boolean = False)
        '------------------------------------------------------------------
        '                    ancrage total du lit
        '------------------------------------------------------------------
        'Variables: appui=1->appuigauche, 2->droit
        '           i1=n° de travée ( et non n° d'index !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!)
        '           nlit = calcul_Ancrage = numéro de lit
        '           j = type de lit (inf, sup gauche, sup droit)
        '           calcul_Ancrage = indique si des aciers supplémentaires sont rajoutées pour satisfaire aux conditions d'appui (5)  
        '                             ou si des aciers des lits en travée sont simplement prolongés sur l'appui
        Dim l1, l2, l3 As Double
        l1 = 0 : l2 = 0 : l3 = 0
        If Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, Partie_Courante) > lappui Then '---------------->  SI ANCRAGE COURBE
            Call Ancrage_courbe(lappui, calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, l1, l2, l3, True)  'rapatrie les valeurs l1,l2,l3
        Else  '---------------->  ALORS ANCRAGE DROIT
            l1 = 0 : l2 = 0 : l3 = -10
        End If
        'détermination des abscisses des armatures
        With PlanCont.Item(i1 - 1)
            Select Case j + additif
                Case 1 ', 2, 3 'ancrage total du premier, deuxieme,troisième lit en travée
                    If appui = 1 Then
                        Select Case .Type_Acier_Travée
                            Case 1
                                .TS_trav.Item(calcul_Ancrage - 1).X1acier = 0 : .TS_trav.Item(calcul_Ancrage - 1).X1acierLS = 0
                                If l3 = -10 Then
                                    .TS_trav.Item(calcul_Ancrage - 1).L13acier = Lscel_droit(calcul_Ancrage - 1, 1, i1, ELU_ELS, σsd, False)
                                Else
                                    .TS_trav.Item(calcul_Ancrage - 1).L13acier = l3
                                End If
                                .TS_trav.Item(calcul_Ancrage - 1).L12acier = l2 : .TS_trav.Item(calcul_Ancrage - 1).L11acier = l1
                                LacierTot = .TS_trav.Item(calcul_Ancrage - 1).L13acier + .TS_trav.Item(calcul_Ancrage - 1).L12acier + .TS_trav.Item(calcul_Ancrage - 1).L11acier
                            Case 2
                                .LitInf.Item(calcul_Ancrage - 1).X1acier = 0 : .LitInf.Item(calcul_Ancrage - 1).X1acierLS = 0
                                If l3 = -10 Then
                                    .LitInf.Item(calcul_Ancrage - 1).L13acier = Lscel_droit(calcul_Ancrage - 1, 1, i1, ELU_ELS, σsd, False)
                                Else
                                    .LitInf.Item(calcul_Ancrage - 1).L13acier = l3
                                End If
                                .LitInf.Item(calcul_Ancrage - 1).L12acier = l2 : .LitInf.Item(calcul_Ancrage - 1).L11acier = l1
                                LacierTot = .LitInf.Item(calcul_Ancrage - 1).L13acier + .LitInf.Item(calcul_Ancrage - 1).L12acier + .LitInf.Item(calcul_Ancrage - 1).L11acier
                        End Select
                    Else
                        Select Case .Type_Acier_Travée
                            Case 1
                                With .TS_trav
                                    .Item(calcul_Ancrage - 1).X2acier = PlanCont.Item(i1 - 1).Lx
                                    .Item(calcul_Ancrage - 1).X2acierLS = PlanCont.Item(i1 - 1).Lx
                                    If l3 = -10 Then
                                        .Item(calcul_Ancrage - 1).L23acier = Lscel_droit(calcul_Ancrage - 1, 1, i1, ELU_ELS, σsd, False)
                                    Else
                                        .Item(calcul_Ancrage - 1).L23acier = l3
                                    End If
                                    .Item(calcul_Ancrage - 1).L22acier = l2 : .Item(calcul_Ancrage - 1).L21acier = l1
                                    LacierTot = .Item(calcul_Ancrage - 1).L23acier + .Item(calcul_Ancrage - 1).L22acier + .Item(calcul_Ancrage - 1).L21acier
                                End With
                            Case 2
                                With .LitInf
                                    .Item(calcul_Ancrage - 1).X2acier = PlanCont.Item(i1 - 1).Lx
                                    .Item(calcul_Ancrage - 1).X2acierLS = PlanCont.Item(i1 - 1).Lx
                                    If l3 = -10 Then
                                        .Item(calcul_Ancrage - 1).L23acier = Lscel_droit(calcul_Ancrage - 1, 1, i1, ELU_ELS, σsd, False)
                                    Else
                                        .Item(calcul_Ancrage - 1).L23acier = l3
                                    End If
                                    .Item(calcul_Ancrage - 1).L22acier = l2 : .Item(calcul_Ancrage - 1).L21acier = l1
                                    LacierTot = .Item(calcul_Ancrage - 1).L23acier + .Item(calcul_Ancrage - 1).L22acier + .Item(calcul_Ancrage - 1).L21acier
                                End With
                        End Select
                    End If
                Case 11 'Ancrage Armature supérieure = ancrage total du premier lit sur appui gauche pour la travée 1
                    'appui gauche légèrement encastré ->0,15xM0
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1
                            With .TS_AppG
                                .Item(calcul_Ancrage - 1).X1acier = 0 : .Item(calcul_Ancrage - 1).X1acierLS = 0
                                .Item(calcul_Ancrage - 1).X2acier = Truncate(0.2 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100 'Art 9.3.1.2(2) EN1992-1-1
                                .Item(calcul_Ancrage - 1).X2acierLS = .Item(calcul_Ancrage - 1).X2acier - Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True)
                                If l3 = -10 Then
                                    .Item(calcul_Ancrage - 1).L13acier = Truncate(Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True) * 100) / 100
                                Else
                                    .Item(calcul_Ancrage - 1).L13acier = Truncate(l3 * 100) / 100 'en mètre
                                End If
                                .Item(calcul_Ancrage - 1).L12acier = Truncate(l2 * 100) / 100 : .Item(calcul_Ancrage - 1).L11acier = Truncate(l1 * 100) / 100
                            End With
                        Case 2
                            .LitSupG.Item(calcul_Ancrage - 1).X1acier = 0
                            .LitSupG.Item(calcul_Ancrage - 1).X1acierLS = 0
                            .LitSupG.Item(calcul_Ancrage - 1).X2acier = Truncate(0.2 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100 'Art 9.3.1.2(2) EN1992-1-1
                            .LitSupG.Item(calcul_Ancrage - 1).X2acierLS = .LitSupG.Item(calcul_Ancrage - 1).X2acier - Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, False)
                            If l3 = -10 Then
                                .LitSupG.Item(calcul_Ancrage - 1).L13acier = Truncate(Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True) * 100) / 100
                            Else
                                .LitSupG.Item(calcul_Ancrage - 1).L13acier = Truncate(l3 * 100) / 100 'en mètre
                            End If
                            .LitSupG.Item(calcul_Ancrage - 1).L12acier = Truncate(l2 * 100) / 100 : .LitSupG.Item(calcul_Ancrage - 1).L11acier = Truncate(l1 * 100) / 100
                    End Select
                Case 12 'Ancrage Armature supérieure = ancrage total du premier lit sur appui droit pour la dernière travée
                    'appui droit légèrement encastré ->0,15xM0
                    Select Case .Type_Acier_Appui_Droit
                        Case 1
                            With .TS_AppD
                                .Item(calcul_Ancrage - 1).X2acier = PlanCont.Item(i1 - 1).Lx  '+ 0.02 + la(2)
                                .Item(calcul_Ancrage - 1).X2acierLS = PlanCont.Item(i1 - 1).Lx
                                .Item(calcul_Ancrage - 1).X1acier = Truncate(0.8 * PlanCont.Item(i1 - 1).Lx * 100 - 1) / 100 'Art 9.3.1.2(2) EN1992-1-1
                                .Item(calcul_Ancrage - 1).X1acierLS = .Item(calcul_Ancrage - 1).X1acier + Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True)
                                If l3 = -10 Then
                                    .Item(calcul_Ancrage - 1).L23acier = Truncate(Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True) * 100) / 100
                                Else
                                    .Item(calcul_Ancrage - 1).L23acier = Truncate(l3 * 100) / 100 'en mètre
                                End If
                                .Item(calcul_Ancrage - 1).L22acier = Truncate(l2 * 100) / 100 : .Item(calcul_Ancrage - 1).L21acier = Truncate(l1 * 100) / 100
                            End With
                        Case 2
                            .LitSupD.Item(calcul_Ancrage - 1).X2acier = .Lx  '+ 0.02 + la(2)
                            .LitSupD.Item(calcul_Ancrage - 1).X2acierLS = .Lx
                            .LitSupD.Item(calcul_Ancrage - 1).X1acier = Truncate(0.8 * PlanCont.Item(i1 - 1).Lx * 100 - 1) / 100 'Art 9.3.1.2(2) EN1992-1-1
                            .LitSupD.Item(calcul_Ancrage - 1).X1acierLS = .LitSupD.Item(calcul_Ancrage - 1).X1acier + Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, False)
                            If l3 = -10 Then
                                .LitSupD.Item(calcul_Ancrage - 1).L23acier = Truncate(Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True) * 100) / 100
                            Else
                                .LitSupD.Item(calcul_Ancrage - 1).L23acier = Truncate(l3 * 100) / 100 'en mètre
                            End If
                            .LitSupD.Item(calcul_Ancrage - 1).L22acier = Truncate(l2 * 100) / 100
                            .LitSupD.Item(calcul_Ancrage - 1).L21acier = Truncate(l1 * 100) / 100
                    End Select
                Case 5 'acier de glissement supplémentaire appui gauche
                    With .LitAppG
                        .X1acier = 0
                        .X1acierLS = 0
                        If l3 = -10 Then
                            .L13acier = Lscel_droit(0, 5, i1, ELU_ELS, σsd, False)
                        Else
                            .L13acier = l3
                        End If
                        .L12acier = l2
                        .L11acier = l1
                        LacierTot = .L13acier
                    End With
                Case 6 'acier de glissement supplémentaire appui droit
                    With .LitAppD
                        .X2acier = PlanCont.Item(i1 - 1).Lx
                        .X2acierLS = PlanCont.Item(i1 - 1).Lx
                        If l3 = -10 Then
                            .L23acier = Lscel_droit(0, 6, i1, ELU_ELS, σsd, False)
                        Else
                            .L23acier = l3
                        End If
                        .L22acier = l2
                        .L21acier = l1
                        LacierTot = .L23acier
                    End With
                Case 40, 41 ' aciers sur appuis haut et bas
                    If j = 4 Then
                        Select Case .Type_Acier_Travée
                            Case 1
                                If .TS_AppH.Count >= 1 Then
                                    With .TS_AppH.Item(0)
                                        .X2acier = Truncate(0.2 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100 'Art 9.3.1.2(2) EN1992-1-1
                                        .X2acierLS = Truncate((.X2acier - Lscel_droit(0, j, i1, ELU_ELS, σsd, True)) * 100) / 100
                                        If l3 = -10 Then
                                            .L13acier = Lscel_droit(0, j, i1, ELU_ELS, σsd, True) 'en m
                                        Else
                                            .L13acier = Truncate((l3 - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        End If
                                        .L12acier = Truncate(l2 * 100) / 100 'en m
                                        .L11acier = Truncate(l1 * 100) / 100 'en m
                                    End With
                                End If
                                If .TS_AppB.Count >= 1 Then
                                    With .TS_AppB.Item(0)
                                        .X2acier = Truncate(0.2 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100 'Art 9.3.1.2(2) EN1992-1-1
                                        .X2acierLS = Truncate((.X2acier - Lscel_droit(0, j, i1, ELU_ELS, σsd, True)) * 100) / 100
                                        If l3 = -10 Then
                                            .L13acier = Lscel_droit(0, j, i1, ELU_ELS, σsd, True) 'en m
                                        Else
                                            .L13acier = Truncate((l3 - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        End If
                                        .L12acier = Truncate(l2 * 100) / 100 'en m
                                        .L11acier = Truncate(l1 * 100) / 100 'en m
                                    End With
                                End If
                            Case 2 'HA
                                If .LitSupH.Count >= 1 Then
                                    With .LitSupH.Item(0)
                                        .X2acier = Truncate(0.2 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100 'Art 9.3.1.2(2) EN1992-1-1
                                        .X2acierLS = Truncate((.X2acier - Lscel_droit(0, j, i1, ELU_ELS, σsd, False)) * 100) / 100
                                        If l3 = -10 Then
                                            .L13acier = Truncate((Lscel_droit(0, 4, i1, 1, 0.8 * aci.fyk, False) - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        Else
                                            .L13acier = Truncate((l3 - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        End If
                                        .L12acier = Truncate(l2 * 100) / 100 'en m
                                        .L11acier = Truncate(l1 * 100) / 100 'en m
                                    End With
                                End If
                                If .LitSupB.Count >= 1 Then
                                    With .LitSupB.Item(0)
                                        .X2acier = Truncate(0.2 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100 'Art 9.3.1.2(2) EN1992-1-1
                                        .X2acierLS = Truncate((.X2acier - Lscel_droit(0, j, i1, ELU_ELS, σsd, False)) * 100) / 100
                                        If l3 = -10 Then
                                            .L13acier = Truncate((Lscel_droit(0, 4, i1, 1, 0.8 * aci.fyk, False) - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        Else
                                            .L13acier = Truncate((l3 - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        End If
                                        .L12acier = Truncate(l2 * 100) / 100 'en m
                                        .L11acier = Truncate(l1 * 100) / 100 'en m
                                    End With
                                End If
                        End Select
                        '.HAConstruc.Type= 3 'aciers de construction insuffisants pour reprendre 0.15M0 - ajout d'aciers
                        'ElseIf j = 2 And appui = 1 Then  'si appui gauche
                        '    '.LitSupG.Item(0).X1acier = 0 : .LitSupG.Item(0).X1acierLS = 0
                        '    'If l3 = 0 Then
                        '    '    .LitSupG.Item(0).L13acier = Truncate((Lscel_droit(0, 2, i1, 1, 0.8 * aci.fyk, False) - (.enrSup + 0.01) * Cotgt(.Θ_Tr)) * 100) / 100 'en m
                        '    '    .LitSupG.Item(0).L12acier = 0 'en m
                        '    '    .LitSupG.Item(0).L11acier = 0 'en m
                        '    'Else
                        '    '    '    .LitSupG.Item(0).L13acier = Truncate((l3 - (.enrSup + 0.01) * Cotgt(.Θ_Tr)) * 100) / 100 'en m
                        '    '    '    .LitSupG.Item(0).L12acier = Truncate(l2 * 100) / 100 'en m
                        '    '    '    .LitSupG.Item(0).L11acier = Truncate(l1 * 100) / 100 'en m
                        '    '    '    .LitSupG.Item(0).L23acier = 0 : .LitSupG.Item(0).L22acier = 0 : .LitSupG.Item(0).L21acier = 0
                        '    'End If
                        '    '.LitSupG.Item(0).Lacier2 = .LitSupG.Item(0).L11acier + .LitSupG.Item(0).L12acier + .LitSupG.Item(0).L13acier
                        'ElseIf j = 3 And appui = 2 Then  'si appui droit
                        '    If l3 = 0 Then
                        '        .LitSupD.Item(0).X2acierLS = .Lx
                        '        .LitSupD.Item(0).L23acier = Truncate((Lscel_droit(0, 3, i1, 1, 0.8 * aci.fyk, False) - (.enrSup + 0.01) * Cotgt(.Θ_Tr)) * 100) / 100 'en m
                        '        .LitSupD.Item(0).Lacier2 = .LitSupD.Item(0).L21acier + .LitSupD.Item(0).L22acier + .LitSupD.Item(0).L23acier
                        '    Else
                        '        '    .LitSupD.Item(0).X1acier = Lscel_droit(0, 3, i1, 1, 0.8 * aci.fyk, True)
                        '        '    .LitSupD.Item(0).L23acier = Truncate((l3 - (.enrSup + 0.01) * Cotgt(.Θ_Tr)) * 100) / 100 'en m
                        '        '    .LitSupD.Item(0).L11acier = 0 : .LitSupD.Item(0).L12acier = 0 : .LitSupD.Item(0).L13acier = 0
                        '        '    .LitSupD.Item(0).Lacier2 = .LitSupD.Item(0).L11acier + .LitSupD.Item(0).L12acier + .LitSupD.Item(0).L13acier
                        '    End If
                        '    '    .LitSupD.Item(0).X2acier = .Lx
                        '    '    .LitSupD.Item(0).L22acier = Truncate(l2 * 100) / 100 'en m
                        '    '    .LitSupD.Item(0).L21acier = Truncate(l1 * 100) / 100 'en m
                    End If
                    'Case 41 ' aciers de construction
                    '        'les aciers de construction sont insuffisants, un jeu de barres a été rajouté au-dessus plus le jeu de barre
                    '        'de l'acier de construction ci-dessous
                    '        If appui = 1 Then  'si appui gauche
                    '            X1acier(i1, 4, 1) = 0
                    '            X1acierLS(i1, 4, 1) = 0
                    '        Else
                    '            X2acier(i1, 4, 1) = l(i1)
                    '            X2acierLS(i1, 4, 1) = l(i1)
                    '        End If
                    '        L3acier(i1, 4, 1, appui) = (l3 + 0.02) * 100
                    '        L3acier(i1, 4, 1, appui) = CInt(L3acier(i1, 4, 1, appui)) / 100  'en mètre
                    '        L2acier(i1, 4, 1, appui) = l2 * 100
                    '        L2acier(i1, 4, 1, appui) = CInt(L2acier(i1, 4, 1, appui)) / 100
                    '        L1acier(i1, 4, 1, appui) = l1 * 100
                    '        L1acier(i1, 4, 1, appui) = CInt(L1acier(i1, 4, 1, appui)) / 100
                    '        LacierTotal(i1, 4, 1, appui) = L1acier(i1, 4, 1, appui) + L2acier(i1, 4, 1, appui) + L3acier(i1, 4, 1, appui)
            End Select
        End With
    End Sub
    Private Sub verification_appui(ByVal NbApp As Integer, ByVal Ry(,) As Double, ByVal M(,,) As Double, ByVal V(,,) As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim i, i1, k, ELU_ELS As Integer
        ELU_ELS = 1 'provisoire
        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
        rtf.Underline = True : rtf.Bold = True : rtf.WriteString("5 - 1  Dimensions des appuis") : rtf.Underline = False : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
        rtf.WriteEndGroup()
        For i1 = 1 To PlanCont.Count
            With PlanCont.Item(i1 - 1)
                Select Case i1
                    Case 1
                        If PlanCont.Count = 1 Then
                            Call Appui_About(i1, 1, .dInf, rtf) 'about gauche
                            Call Appui_About(i1, 2, .dInf, rtf) 'about droit
                        Else
                            If .LappG < 0.05 Then
                                Call Appui_Intermediaire(i1, .dSupD, PlanCont.Item(i1).dInf, V, rtf)
                            Else
                                Call Appui_About(i1, 1, .dInf, rtf)
                                Call Appui_Intermediaire(i1, .dInf, PlanCont.Item(i1).dInf, V, rtf)
                            End If
                        End If
                    Case PlanCont.Count - 1
                        If PlanCont.Item(i1).LappD < 0.05 Then
                            Call Appui_Intermediaire(i1, .dInf, PlanCont.Item(i1).dSupG, V, rtf)
                        End If
                    Case PlanCont.Count
                        If .LappD > 0.05 Then
                            Call Appui_About(i1, 2, .dInf, rtf)
                        End If
                    Case Else
                        Call Appui_Intermediaire(i1, .dInf, PlanCont.Item(i1).dInf, V, rtf)
                End Select

            End With
        Next
        '------------------------------------------------------------------------------
        'Affichage de la réacton d'appui pour chaque appui et pour chaque combinaison ELU
        '------------------------------------------------------------------------------
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9
        rtf.Underline = True : rtf.Bold = True : rtf.WriteString("5 - 2  Valeurs des réactions d'appuis") : rtf.Underline = False : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
        rtf.NewParagraph()
        For k = 1 To NbApp
            'ELU STR
            'Ligne 1
            rtf.BeginRow() : rtf.DeclareCell(8800) : rtf.BeginCell(True) : rtf.WriteString("Appui n°" & k.ToString) : rtf.EndCell() : rtf.EndRow()
            For i = 1 To Aff.ELU
                'Ligne 2
                rtf.BeginRow() : rtf.DeclareCell(2200) : rtf.BeginCell(True) : rtf.WriteString("Comb. n° " & i.ToString & " - ELU STR") : rtf.EndCell()
                rtf.DeclareCell(3000) : rtf.BeginCell(True) : rtf.WriteString("Réaction d'appui = " & (Ry(i, k) * 1000).ToString("#0.00") & " KN.") : rtf.EndCell()
                rtf.DeclareCell(3600) : rtf.BeginCell(True)
                If Ry(i, k) > 0 Then
                    rtf.WriteString(" ")
                Else
                    rtf.WriteString(" Ry < 0 - Soulèvement de l'appui ")
                    Aff.msg_Avertiss.Add("Soulèvement de l'appui n°" & k.ToString & " sous la combinaison n°" & i.ToString)
                End If
                rtf.EndCell() : rtf.EndRow()
            Next
            'ELU EQU
            For i = Aff.ELU + 1 To Aff.ELU_EQ
                'Ligne 2
                rtf.BeginRow() : rtf.DeclareCell(2200) : rtf.BeginCell(True) : rtf.WriteString("Comb. n° " & i.ToString & " - ELU EQU") : rtf.EndCell()
                rtf.DeclareCell(3000) : rtf.BeginCell(True) : rtf.WriteString("Réaction d'appui = " & (Ry(i, k) * 1000).ToString("#0.00") & " KN.") : rtf.EndCell()
                rtf.DeclareCell(3600) : rtf.BeginCell(True)
                If Ry(i, k) > 0 Then
                    rtf.WriteString(" ")
                Else
                    rtf.WriteString(" Ry < 0 - Soulèvement de l'appui ")
                    Aff.msg_Avertiss.Add("Soulèvement de l'appui n°" & k.ToString & " sous la combinaison n°" & i.ToString)
                End If
                rtf.EndCell() : rtf.EndRow()
            Next
        Next
        rtf.WriteEndGroup()
        Erase Ry
    End Sub
    Private Sub Appui_About(ByVal i1 As Integer, ByVal appui As Integer, ByVal d As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim a1, a2, Comp, FEd, Lbielle, LappuiMin, Z As Double
        Dim CtΘ, Θa, σRd1, σRd2, σRdMax As Double
        Dim Nint, Nint1 As Integer

        rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
        If appui = 1 Then rtf.Underline = True : rtf.WriteString("Appui gauche - travée " & i1.ToString & " :") : rtf.Underline = False : rtf.NewLine()
        If appui = 2 Then rtf.Underline = True : rtf.WriteString("Appui droit - travée " & i1.ToString & " :") : rtf.Underline = False : rtf.NewLine()
        rtf.Bold = True
        rtf.WriteString("  ■ Détermination de l'angle de la bielle d'about") : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("calcul de l'angle suivant disposition EN1992-1- Art.9.2.1.4(2)") : rtf.NewLine()
        Z = 0.9 * d
        With PlanCont.Item(i1 - 1)
            Nint = CInt(Truncate((.a1) * 100))
            Nint1 = CInt(Truncate((.Lcx - .a2) * 100))
            Select Case appui
                Case 1
                    CtΘ = (.LappG + Z * Cotgt(.Θ_Tr) - .enrLat) / (2 * Z)
                Case 2
                    CtΘ = (.LappD + Z * Cotgt(.Θ_Tr) - .enrLat) / (2 * Z)
            End Select
            Θa = Atan(1 / CtΘ)
            rtf.WriteString("Angle de la bielle d'about Θ': " & (Θa * 180 / PI).ToString("#0.00") & "° (suivant eq.9.17 de l'AN)") : rtf.NewLine()
            rtf.Bold = True : rtf.WriteString("  ■ Vérification de la bielle en zone courante") : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("Application de la clause 6.5.2(2) de l'EN1992-1-1") : rtf.NewLine()
            σRdMax = 0.6 * (1 - bet.fck / 250) * bet.fck / bet.γcd
            rtf.WriteString("Contrainte maximale autorisée: σRdMax = 0.6.ν'.fcd (eq 6.56) soit: " & σRdMax.ToString("#0.00") & " MPa.") : rtf.NewLine()
            Select Case appui
                Case 1
                    FEd = .VMaxELU(Nint) * CtΘ
                    Comp = Sqrt(FEd ^ 2 + .VMaxELU(Nint) ^ 2)
                Case 2
                    FEd = Abs(.VminELU(Nint1)) * CtΘ
                    Comp = Sqrt(FEd ^ 2 + .VminELU(Nint1) ^ 2)
            End Select
            Lbielle = Comp / 1 / σRdMax
            rtf.WriteString("Effort de compression dans la bielle : " & (Comp * 1000).ToString("#,##0.00") & " kN.") : rtf.NewLine()
            rtf.WriteString("Dimension transversale minimale de la bielle : " & (Lbielle * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
            rtf.Bold = True : rtf.WriteString("  ■ Vérification du noeud") : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("Application de la clause 6.5.4(4).b) de l'EN1992-1-1: appui avec compression et traction et tirant dans une seule direction.") : rtf.NewLine()
            rtf.WriteString("La charge s'applique sur 1 m d'appui.") : rtf.NewLine()
            σRdMax = 0.85 * (1 - bet.fck / 250) * bet.fck / bet.γcd
            rtf.WriteString("Contrainte maximale autorisée: σRdMax = 0.85.ν'.fcd (eq 6.61) soit: " & σRdMax.ToString("#0.00") & " MPa.") : rtf.NewLine()
            Select Case appui
                Case 1 'appui gauche
                    rtf.WriteString("Effort vertical maximal: " & (.VMaxELU(Nint) * 1000).ToString("#,##0.00") & " kN ( soit VEd au nu de l'appui - voir § Effort Tranchant).") : rtf.NewLine()
                    a1 = .LappG - .enrLat  'fig 9.102 Pr NF EN 1992-1-1/NA: 2015
                    a2 = (a1 + Cotgt(Θa) * (.enrInf + 0.01)) * Sin(Θa)
                    rtf.WriteString("longueur d'appui a1: " & (a1 * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                    rtf.WriteString("longueur d'appui a2: " & (a2 * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                    σRd1 = .VMaxELU(Nint) / (1 * a1) : σRd2 = .VMaxELU(Nint) / (1 * a2 * Sin(Θa))
                    rtf.WriteString("Facette appui: σRd1 (=Ved/bw.a1): " & σRd1.ToString("#0.00") & " MPa ")
                    If σRd1 > σRdMax Then
                        rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                        LappuiMin = .VMaxELU(Nint) / 1 / σRdMax
                        rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                    Else
                        rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
                    End If
                    rtf.WriteString("Facette poutre: σRd2 (=Ved/bw.a2.SinΘ'): " & σRd2.ToString("#0.00") & " MPa ")
                    If σRd2 > σRdMax Then
                        rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                        LappuiMin = .VMaxELU(Nint) / 1 / σRdMax
                        rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                    Else
                        rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
                    End If
                Case 2 'appui droit
                    rtf.WriteString("Effort vertical maximal: " & Abs(.VminELU(Nint1) * 1000).ToString("#,##0.00") & " kN ( soit VEd au nu de l'appui - voir § Effort Tranchant).") : rtf.NewLine()
                    a1 = .LappD - .enrLat 'fig 9.102 Pr NF EN 1992-1-1/NA: 2015
                    a2 = (a1 + Cotgt(Θa) * (.enrInf + 0.01)) * Sin(Θa)
                    rtf.WriteString("longueur d'appui a1: " & (a1 * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                    rtf.WriteString("longueur d'appui a2: " & (a2 * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                    σRd1 = Abs(.VminELU(Nint1)) / (1 * a1) : σRd2 = Abs(.VminELU(Nint1)) / (1 * a2 * Sin(Θa))
                    rtf.WriteString("Facette appui: σRd1 (=Ved/bw.a1): " & σRd1.ToString("#0.00") & " MPa ")
                    If σRd1 > σRdMax Then
                        rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                        LappuiMin = Abs(.VminELU(Nint1)) / 1 / σRdMax
                        rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                    Else
                        rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
                    End If
                    rtf.WriteString("Facette poutre: σRd2 (=Ved/bw.a2.SinΘ'): " & σRd2.ToString("#0.00") & " MPa ")
                    If σRd2 > σRdMax Then
                        rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                        LappuiMin = Abs(.VminELU(Nint1)) / 1 / σRdMax
                        rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                    Else
                        rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
                    End If
            End Select
        End With
        rtf.NewLine()
        rtf.WriteEndGroup()
    End Sub
    Private Sub Appui_Intermediaire(ByVal i1 As Integer, ByVal d1 As Double, ByVal d2 As Double,
                                    ByVal V(,,) As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim a1, a2, a3, Comp, FEd, Lbielle, LappuiMin, Z As Double
        Dim CtΘ, Θa1, Θa2, σRd1, σRd2, σRd3, σRdMax, VgMax, VdMax As Double
        Dim Nint, Nint1, appui, i, i_gauche, i_droit, message As Integer

        rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
        rtf.Underline = True : rtf.WriteString("Appui intermédiaire entre travée " & i1.ToString & " et travée " & (i1 + 1).ToString & " :") : rtf.Underline = False : rtf.NewLine()
        rtf.Bold = True
        rtf.WriteString("  ■ Détermination de l'angle des bielles") : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("calcul de l'angle suivant disposition EN1992-1- Art.9.2.1.4(2)") : rtf.NewLine()

        With PlanCont.Item(i1 - 1)
            Nint = CInt(Truncate((.Lcx - .a2) * 100)) 'côté gauche
            Nint1 = CInt(Truncate((PlanCont.Item(i1).a1) * 100)) 'côté droit
            For appui = 1 To 2
                Select Case appui
                    Case 1
                        Z = 0.9 * d1
                        CtΘ = (.LappD + Z * Cotgt(.Θ_Tr) - .enrLat) / (2 * Z)
                        Θa1 = Atan(1 / CtΘ)
                        rtf.WriteString("Angle Θ'de la bielle côté gauche de l'appui : " & (Θa1 * 180 / PI).ToString("#0.00") & "° (suivant eq.9.17 de l'AN)") : rtf.NewLine()
                    Case 2
                        Z = 0.9 * d2
                        CtΘ = (.LappD + Z * Cotgt(.Θ_Tr) - .enrLat) / (2 * Z)
                        Θa2 = Atan(1 / CtΘ)
                        rtf.WriteString("Angle Θ'de la bielle côté droit de l'appui: " & (Θa2 * 180 / PI).ToString("#0.00") & "° (suivant eq.9.17 de l'AN)") : rtf.NewLine()
                End Select

            Next
            rtf.Bold = True : rtf.WriteString("  ■ Vérification de la bielle en zone courante") : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("Application de la clause 6.5.2(2) de l'EN1992-1-1") : rtf.NewLine()
            σRdMax = 0.6 * (1 - bet.fck / 250) * bet.fck / bet.γcd
            rtf.WriteString("Contrainte maximale autorisée: σRdMax = 0.6.ν'.fcd (eq 6.56) soit: " & σRdMax.ToString("#0.00") & " MPa.") : rtf.NewLine()
            For appui = 1 To 2
                Select Case appui
                    Case 1

                        FEd = Abs(.VminELU(Nint)) * Cotgt(Θa1)
                        Comp = Sqrt(FEd ^ 2 + .VminELU(Nint) ^ 2)
                        Lbielle = Comp / 1 / σRdMax
                        rtf.WriteString("Bielle côté gauche: - effort de compression: " & (Comp * 1000).ToString("#,##0.00") & " kN.") : rtf.NewLine()
                        rtf.WriteString("                    - dimension transversale minimale : " & (Lbielle * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                    Case 2
                        FEd = .VMaxELU(Nint1) * Cotgt(Θa2)
                        Comp = Sqrt(FEd ^ 2 + PlanCont.Item(i1).VMaxELU(Nint1) ^ 2)
                        Lbielle = Comp / 1 / σRdMax
                        rtf.WriteString("Bielle côté droit: - effort de compression: " & (Comp * 1000).ToString("#,##0.00") & " kN.") : rtf.NewLine()
                        rtf.WriteString("                    - dimension transversale minimale : " & (Lbielle * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                End Select
            Next
            rtf.Bold = True : rtf.WriteString("  ■ Vérification du noeud") : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("Application de la clause 6.5.4(4).a) de l'EN1992-1-1: appui avec compression et sans tirant.") : rtf.NewLine()
            rtf.WriteString("La largeur de l'appui est, à minima, égale à la largeur de la poutre.") : rtf.NewLine()
            σRdMax = 0.85 * (1 - bet.fck / 250) * bet.fck / bet.γcd
            rtf.WriteString("Contrainte maximale autorisée: σRdMax = 0.85.ν'.fcd (eq 6.61) soit: " & σRdMax.ToString("#0.00") & " MPa.") : rtf.NewLine()
            a1 = .LappD - .enrLat 'fig 9.102 Pr NF EN 1992-1-1/NA: 2015
            a2 = (a1 + Cotgt(Θa1) * (.enrInf + 0.01)) * Sin(Θa1)
            a3 = (a1 + Cotgt(Θa2) * (.enrInf + 0.01)) * Sin(Θa2)
            rtf.WriteString("longueur d'appui a1: " & (a1 * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
            rtf.WriteString("longueur d'appui a2: " & (a2 * 100).ToString("#0.0") & " cm (côté gauche).") : rtf.NewLine()
            rtf.WriteString("longueur d'appui a3: " & (a3 * 100).ToString("#0.0") & " cm (côté droit).") : rtf.NewLine()
            For appui = 1 To 2
                Select Case appui
                    Case 1 'appui gauche
                        VgMax = Abs(V(1, i1, Nint)) : i_gauche = 1
                        For i = 2 To Aff.ELU
                            If VgMax < Abs(V(i, i1, Nint)) Then VgMax = Abs(V(i, i1, Nint)) : i_gauche = i
                        Next
                        rtf.WriteString("Combinaison conduisant à l'effort vertical maximal à gauche de l'appui: " & i_gauche.ToString) : rtf.NewLine()
                        rtf.WriteString("Effort vertical maximal: " & (VgMax * 1000).ToString("#,##0.00") & " kN ( soit VEd au nu de l'appui côté gauche).") : rtf.NewLine()
                        rtf.WriteString("Effort vertical côté droit de l'appui pour la même combinaison: " & Abs(V(i, i1 + 1, Nint1) * 1000).ToString("#,##0.00") & " kN.") : rtf.NewLine()
                        σRd1 = (VgMax + Abs(V(i, i1 + 1, Nint1))) / (1 * a1) : σRd2 = VgMax / (1 * a2 * Sin(Θa1))
                        σRd3 = Abs(V(i, i1 + 1, Nint1)) / (1 * a3 * Sin(Θa2))
                        rtf.WriteString("Facette appui: σRd1 (=(Vedg+Vedd)/bw.a1): " & σRd1.ToString("#0.00") & " MPa.") : rtf.NewLine()
                        If σRd1 > σRdMax Then
                            rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                            LappuiMin = (VgMax + Abs(V(i, i1 + 1, Nint1))) / 1 / σRdMax : message = 1
                            rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                        Else
                            rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
                        End If
                        rtf.WriteString("Facette poutre côté gauche: σRd2 (=Vedg/bw.a2.SinΘg'): " & σRd2.ToString("#0.00") & " MPa ")
                        If σRd2 > σRdMax Then
                            rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                            LappuiMin = VgMax / 1 / σRdMax
                            rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                        Else
                            rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
                        End If
                        rtf.WriteString("Facette poutre côté droit: σRd3 (=Vedd/bw.a3.SinΘd'): " & σRd3.ToString("#0.00") & " MPa ")
                        If σRd3 > σRdMax Then
                            rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                            LappuiMin = Abs(V(i, i1 + 1, Nint1)) / 1 / σRdMax
                            rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                        Else
                            rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
                        End If
                    Case 2 'appui droit
                        VdMax = Abs(V(1, i1 + 1, Nint1)) : i_droit = 1
                        For i = 2 To Aff.ELU
                            If VdMax < Abs(V(i, i1 + 1, Nint1)) Then VdMax = Abs(V(i, i1 + 1, Nint1)) : i_droit = i
                        Next
                        rtf.NewLine()
                        rtf.WriteString("Combinaison conduisant à l'effort vertical maximal à droite de l'appui: " & i_droit.ToString) : rtf.NewLine()
                        If i_droit = i_gauche Then
                            rtf.WriteString("Même combinaison que pour le côté gauche - Voir vérification ci-dessus.") : rtf.NewLine()
                        Else
                            rtf.WriteString("Effort vertical maximal: " & (VdMax * 1000).ToString("#,##0.00") & " kN ( soit VEd au nu de l'appui côté droit).") : rtf.NewLine()
                            rtf.WriteString("Effort vertical côté gauche de l'appui pour la même combinaison: " & Abs(V(i, i1, Nint) * 1000).ToString("#,##0.00") & " kN.") : rtf.NewLine()
                            σRd1 = (VdMax + Abs(V(i, i1, Nint))) / (1 * a1) : σRd2 = Abs(V(i, i1, Nint)) / (1 * a2 * Sin(Θa1))
                            σRd3 = VdMax / (1 * a3 * Sin(Θa2))
                            rtf.WriteString("Facette appui: σRd1 (=(Vedg+Vedd)/bw.a1): " & σRd1.ToString("#0.00") & " MPa.")
                            If σRd1 > σRdMax Then
                                rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                                LappuiMin = (VdMax + Abs(V(i, i1, Nint))) / 1 / σRdMax : message = 1
                                rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                            Else
                                rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
                            End If
                            rtf.WriteString("Facette poutre côté gauche: σRd2 (=Vedg/bw.a2.SinΘg'): " & σRd2.ToString("#0.00") & " MPa ")
                            If σRd2 > σRdMax Then
                                rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                                LappuiMin = Abs(V(i, i1, Nint)) / 1 / σRdMax
                                rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                            Else
                                rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
                            End If
                            rtf.WriteString("Facette poutre côté droit: σRd3 (=Vedd/bw.a3.SinΘd'): " & σRd3.ToString("#0.00") & " MPa ")
                            If σRd3 > σRdMax Then
                                rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                                LappuiMin = VdMax / 1 / σRdMax
                                rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                            Else
                                rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
                            End If
                        End If
                End Select
            Next

        End With
        If message = 1 Then rtf.WriteString("Attention!!! Cette longueur minimale de l'appui est théorique et doit prendre en compte les nécessités d'ancrage du 1ier lit d'armature.") : rtf.NewLine()

        rtf.WriteEndGroup()
    End Sub
    Private Sub Effort_Traction_Appui(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal d As Double, ByVal M(,,) As Double, ByVal V(,,) As Double, ByRef CotΘ() As Double, ByRef FE() As Double,
                                      ByRef VEd() As Double, ByRef MEd() As Double)
        Dim Nint, Nint1, i3 As Integer
        Dim VEdi, FEdi, MEdi, z, Lappui(2) As Double ''
        'Θ_Tr = 45° angle de la bielle:  α_tr = 90° acier effort tranchant verticaux
        z = 0.9 * d
        'a1 = d (§9.3.1(4)) soit  a1/z = 1/0.9
        With PlanCont.Item(i1 - 1)
            Nint1 = CInt(Truncate((.a1) * 100))
            Nint = CInt(Truncate((.Lcx - .a2) * 100))
            Select Case i1
                Case 1  'travée n°1
                    ' appui gauche
                    Lappui(1) = .LappG - .enrLat
                    CotΘ(1) = (Lappui(1) + z * Cotgt(.Θ_Tr)) / (2 * z)
                    VEd(1) = Max(Abs(.VMaxELU(Nint1)), Abs(.VminELU(Nint1)))
                    FE(1) = Abs(VEd(1)) * CotΘ(1)   'Effort de traction développé sur l'appui
                    If FE(1) < 0 Then FE(1) = 0 'pour éviter toute valeur négative
                    If PlanCont.Count = 1 Then  'plancher isostatique
                        Lappui(2) = .LappD - .enrLat
                        'voir pr NFEN1992-1-1/NA 2015 remplace expression  a1 = z * (Cotgt(.Θ_Tr) - Cotgt(.α_tr)) / 2
                        CotΘ(2) = (Lappui(2) + z * Cotgt(.Θ_Tr)) / (2 * z)
                        FE(2) = Abs(VEd(2)) * CotΘ(2)
                    Else
                        ' appui droit
                        VEd(2) = V(1, i1, Nint) : MEd(2) = M(1, i1, Nint)
                        Lappui(2) = .LappD  'car appui intermédiaire
                        CotΘ(2) = (Lappui(2) + z * Cotgt(.Θ_Tr)) / (2 * z)
                        FE(2) = Abs(VEd(2)) * CotΘ(2) + MEd(2) / z 'Effort de traction développé sur l'appui

                        If Aff.ELU > 1 Then     'vérification pour toutes les combinaisons de celle
                            For i3 = 2 To Aff.ELU  'la plus défavorable
                                VEdi = V(i3, i1, Nint) : MEdi = M(i3, i1, Nint) 'i3=n° de combinaison, i1=n° de travée
                                FEdi = Abs(VEdi) * CotΘ(2) + MEdi / z
                                If FEdi > FE(2) Then
                                    FE(2) = FEdi : VEd(2) = VEdi : MEd(2) = MEdi
                                End If
                            Next i3
                        End If
                    End If
                Case Is < PlanCont.Count
                    ' appui gauche
                    VEd(1) = V(1, i1, Nint1) : MEd(1) = M(1, i1, Nint1)
                    Lappui(1) = .LappG  'car appui intermédiaire
                    CotΘ(1) = (Lappui(1) + z * Cotgt(.Θ_Tr)) / (2 * z)
                    FE(1) = Abs(VEd(1)) * CotΘ(1) + MEd(1) / z  'Effort de traction développé sur l'appui
                    If Aff.ELU > 1 Then     'vérification pour toutes les combinaisons de celle
                        For i3 = 2 To Aff.ELU  'la plus défavorable
                            VEdi = V(i3, i1, Nint1) : MEdi = M(i3, i1, Nint1) 'i3=n° de combinaison, i1=n° de travée
                            FEdi = Abs(VEdi) * CotΘ(1) + MEdi / z
                            If FEdi > FE(1) Then
                                FE(1) = FEdi : VEd(1) = VEdi : MEd(1) = MEdi
                            End If
                        Next i3
                    End If
                    ' appui droit
                    VEd(2) = V(1, i1, Nint) : MEd(2) = M(1, i1, Nint)
                    Lappui(2) = .LappD  'car appui intermédiaire
                    CotΘ(2) = (Lappui(2) + z * Cotgt(.Θ_Tr)) / (2 * z)
                    FE(2) = Abs(VEd(2)) * CotΘ(2) + MEd(2) / z  'Effort de traction développé sur l'appui

                    If Aff.ELU > 1 Then     'vérification pour toutes les combinaisons de celle
                        For i3 = 2 To Aff.ELU  'la plus défavorable
                            VEdi = V(i3, i1, Nint) : MEdi = M(i3, i1, Nint) 'i3=n° de combinaison, i1=n° de travée
                            FEdi = Abs(VEdi) * CotΘ(2) + MEdi / z
                            If FEdi > FE(2) Then
                                FE(2) = FEdi : VEd(2) = VEdi : MEd(2) = MEdi
                            End If
                        Next i3
                    End If
                Case PlanCont.Count
                    ' appui gauche
                    VEd(1) = V(1, i1, Nint1) : MEd(1) = M(1, i1, Nint1)
                    Lappui(1) = .LappG  'car appui intermédiaire
                    CotΘ(1) = (Lappui(1) + z * Cotgt(.Θ_Tr)) / (2 * z)
                    FE(1) = Abs(VEd(1)) * CotΘ(1) + MEd(1) / z  'Effort de traction développé sur l'appui

                    If Aff.ELU > 1 Then     'vérification pour toutes les combinaisons de celle
                        For i3 = 2 To Aff.ELU  'la plus défavorable
                            VEdi = V(i3, i1, Nint1) : MEdi = M(i3, i1, Nint1) 'i3=n° de combinaison, i1=n° de travée
                            FEdi = Abs(VEdi) * CotΘ(1) + MEdi / z
                            If FEdi > FE(1) Then
                                FE(1) = FEdi : VEd(1) = VEdi : MEd(1) = MEdi
                            End If
                        Next i3
                    End If
                    ' appui droit
                    Lappui(2) = .LappD - .enrLat
                    CotΘ(2) = (Lappui(2) + z * Cotgt(.Θ_Tr)) / (2 * z)
                    VEd(2) = Max(Abs(.VMaxELU(Nint)), Abs(.VminELU(Nint)))
                    FE(2) = Abs(VEd(2)) * CotΘ(2)  'Effort de traction développé sur l'appui
            End Select
        End With
    End Sub

    ''' <summary>
    ''' section mini des aciers longitudinaux tendus (EN1992-1-1 §9.2.1.1(1))  
    ''' </summary>
    ''' <param name="d">distance entre le cdg des aciers et la fibre extrème comprimée</param>
    ''' <param name="i1">n° de travée</param>
    ''' <param name="amini">section minimale armature acier</param>
    Private Sub Pourcent_mini_acier_flexion(ByVal i1 As Integer, ByVal d As Double, ByRef Amini As Double)
        '-----------------------------------------------------------------------------
        '                % minimun longitudinal des aciers de flexion 
        '-------------------------------------------------------------------------------
        'Variables: 
        Dim u1, u2 As Double
        u1 = (0.26 * bet.fctm / aci.fyk) * 1 * d   'Article 9.2.1  largeur 1 m
        u2 = 0.0013 * 1 * d              'Article 9.2.1 largeur 1 m
        If u1 > u2 Then Amini = u1 Else Amini = u2
    End Sub
    Private Sub Verification_abscence_croisement_acier()
        'Le programme vérifie l'absence de croisement entre les aciers en chapeaux
        'Dim i As Integer
        'If nbtrav < 3 Then Exit Sub 'sans objet
        'For i = 2 To nbtrav - 1
        '    If X2acier(i, 2, 1) > X1acier(i, 3, 1) Then

        '    End If
        'Next
    End Sub
    ''' <summary>Programme de vérification du monolithisme du plancher suivant les prescriptions du §2.3.2 du CPT</summary>
    ''' <param name="Num_Chap">N° de tête de chapitre</param>
    '''  <param name="rtf">variable renvoyant le fichier de note de calcul sous format rtf</param>
    Private Sub Monolithisme_Prefa(ByVal Num_Chap As Integer, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim i1, x1, x2 As Integer
        Dim β, µ, σn, A, c, d, Fctd, V1, V2, VEdMax, VEdi, VRdi, VRdiMax As Double
        Dim msg As String

        For i1 = 1 To PlanCont.Count
            rtf.WriteStartGroup()
            rtf.FontSize = 11 : rtf.Underline = True
            rtf.WriteString(Num_Chap.ToString & " - " & (i1 + 1).ToString & "  Travée n°" & i1.ToString) : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine()
            rtf.WriteEndGroup()

            β = 1 'provisoire
            With PlanCont.Item(i1 - 1)
                'Détermination de l'effort tranchant Maximum
                If .LappG <= 0.05 Then
                    d = .dSupD
                ElseIf .LappD <= 0.05 Then
                    d = .dSupG
                Else
                    d = .dInf
                End If
                x1 = CInt(Truncate(.a1 * 100)) : x2 = CInt(Truncate((.Lcx - .a2) * 100))
                V1 = Max(Abs(.VMaxELU.Item(x1)), Abs(.VminELU.Item(x1))) 'au nu d'appui gauche
                V2 = Max(Abs(.VMaxELU.Item(x2)), Abs(.VminELU.Item(x2))) 'au nu d'appui droit
                VEdMax = Max(V1, V2) 'effort tranchant maximum

                VEdi = β * VEdMax / (0.9 * d * 1)
                VRdiMax = 0.5 * 0.6 * (1 - bet.fck / 250) * bet.fck / bet.γcd
                If VEdi > VRdiMax Then
                    'Pas d'interruption du calcul mais affichage d'un avertissement dans la note de calcul
                    msg = "Plancher préfabriquée - vérification au cisaillement le long de la surface de reprise - travée n°" & i1.ToString
                    msg = msg + Environment.NewLine + "  β = 1 à titre conservatoire"
                    msg = msg + Environment.NewLine + " Vedi (= " & (VEdi * 1000).ToString("#,##0.0") & " KN) > VrdMax = 0,5.ν.fcd (= " & (VRdiMax * 1000).ToString("#,##0.0") & " KN)."
                    msg = msg + Environment.NewLine + "La contrainte de cisaillement dépasse la limite autorisée par L'EN1992-1-1 eq6.23. Augmenter la surface géométrique ou les performances du béton."
                    Aff.msg_Avertiss.Add(msg)
                End If
                Select Case .Prefa_Reprise
                    Case 1  'lisse
                        c = 0.2 : µ = 0.6
                    Case 2 'rugueux
                        c = 0.45 : µ = 0.7
                End Select
                σn = 0
                Fctd = bet.fctk05 / bet.γcd
                For Each chges As Charge In PlanCont.Item(i1 - 1).Chge 'voir bouquin Paillé page 436
                    If chges.Nature = 1 And chges.schem = 1 Then 'sur travée i1, permanente et uniformément répartie
                        Select Case chges.suspendue
                            Case 1 'supérieure
                                σn = 1.35 * (chges.P1 - 0.025 * .Prefa_h) / 1 'poids de la dalle coulée en oeuvre
                                Exit For
                            Case 2 'inférieure =  suspendue
                                σn = -1.35 * chges.P1 / 1
                                c = 0
                        End Select
                    End If
                Next
                VRdi = c * bet.fctk05 / bet.γcd - µ * σn
                A = 0
                If VEdi > VRdi Then
                    A = (VEdi - c * bet.fctk05 / bet.γcd - µ * σn) * (1 * 1) 'pour m²/ml
                    A = A / (µ * aci.fyk / aci.γsd)
                End If
                Call Tab_Cisaillement_Reprise(β, d, V1, V2, VEdi, VRdiMax, c, Fctd, µ, σn, VRdi, A, rtf)
                If .LappG <= 0.05 Or .LappD <= 0.05 Then
                    rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                    rtf.WriteString("Bord libre sur cette travée: prévoir armatures longitudinales et transversales comme définies au §9.3.1,4 de l'EN1992-1-1") : rtf.NewLine()
                    rtf.WriteString("Disposition non implémentée dans cette version logicielle.") : rtf.NewLine()
                    rtf.WriteEndGroup()
                End If
            End With
        Next
    End Sub
    Private Sub Tab_Cisaillement_Reprise(ByVal β As Double, ByVal d As Double, ByVal V1 As Double, ByVal V2 As Double, ByVal VEdi As Double, ByVal VRdiMax As Double,
                                        ByVal c As Double, ByVal Fctd As Double, ByVal µ As Double, ByVal σn As Double, ByVal VRdi As Double, ByVal A As Double,
                                         ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim msg1 As String = String.Empty
        Dim msg2 As String = String.Empty
        If V2 > V1 Then msg2 = " - valeur retenue pour VEdi" Else msg1 = " - valeur retenue pour VEdi"
        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        'Ligne 10
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("z") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((0.9 * d * 100).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("bras de levier des forces internes de la section composite eq. 6.24") : rtf.EndCell() : rtf.EndRow()
        'Ligne 10
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("β") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(β.ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("facteur correctif eq. 6.24") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ved_G") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((V1 * 1000).ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("║Ved║ au nu de l'appui gauche" & msg1) : rtf.EndCell() : rtf.EndRow()
        'Ligne 5
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ved_D") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((V2 * 1000).ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("║Ved║ au nu de l'appui droit" & msg2) : rtf.EndCell() : rtf.EndRow()
        'Ligne 7
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString(" VEdi") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(VEdi.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Contrainte de cisaillement à l'interface eq 6.24") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vrdi Max") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(VRdiMax.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Borne haute cisaillement résistant (eq 6.25) limité à 0,15 MPa si fonction diaphragme (cf §10.9.3(12) ") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("c") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(c.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Coefficient eq 6.25") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Fctd") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Fctd.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("suivant §3.1.6(2)P") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("µ") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(µ.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Coefficient eq 6.25") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("σn") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(σn.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Contrainte force normale exercée sur surface de reprise") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vrdi") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(VRdi.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Contrainte résistante de calcul eq. 6.25") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(8800) : rtf.BeginCell(True)
        If VEdi > VRdi Then
            rtf.WriteString(" VEdi > Vrdi -> nécessité armature de couture sur surface de reprise")
            'Ligne 8
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As") : rtf.EndCell()
            rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm²/m²") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((A * 10000.0).ToString("#0.00")) : rtf.EndCell()
            rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Acier de couture sur le 1/3 de la prédalle") : rtf.EndCell() : rtf.EndRow()
        Else
            rtf.WriteString(" VEdi ≤ Vrdi -> Plancher sans armature de couture à la surface de reprise")
        End If
        rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
    End Sub

#Region "Calcul effort tranchant"
    Private Sub Effort_Tranchant_Plancher(ByVal Num_Chap As Integer, ByRef verif As Boolean, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim Crdc, d, fcd, fywd, k, V1, V2, Vrdc, Vrdc1, Vrdcmin, VEdMax, ν, ν1, νmin, ρ1, ρwmin As Double
        ' Dim At, AtSt, Atstmin, Stmax, Stmaxx, VrdMax, xx As Double
        Dim i1, x1, x2 As Integer 'j,i,imin, iMax,i2,i3,ik, ik1, ik2, phiDiamMax,sism, X,

        'détermination des variables globales à l'ensemble des travées pour le calcul à l'effort tranchant
        fywd = aci.fyk / aci.γsd
        Crdc = 0.18 / bet.γcd
        ν = 0.6 * (1 - bet.fck / 250)
        ν1 = ν
        fcd = bet.fck / bet.γcd
        ρwmin = (0.08 * bet.fck ^ 0.5) / aci.fyk

        For i1 = 1 To PlanCont.Count
            rtf.WriteStartGroup()
            rtf.FontSize = 12 : rtf.Underline = True
            rtf.WriteString(Num_Chap.ToString & " - " & (i1 + 1).ToString & "  Travée n°" & i1.ToString) : rtf.NewLine() : rtf.NewLine()
            rtf.FontSize = 9 : rtf.Bold = True
            rtf.WriteString("Vérification de l'effort tranchant résistant de calcul") : rtf.NewLine() : rtf.Underline = False : rtf.Bold = False
            rtf.WriteEndGroup()
            'Vérification de la compression des bielles
            With PlanCont.Item(i1 - 1)
                'Détermination de l'effort tranchant Maximum
                If .LappG <= 0.05 Then
                    d = .dSupD
                    If .Type_Acier_Appui_Droit = 1 Then ρ1 = aci.As_long(.TS_AppD.Item(0).indTS) Else ρ1 = .LitSupD.Item(0).Aire / .Ly
                ElseIf .LappD <= 0.05 Then
                    d = .dSupG
                    If .Type_Acier_Appui_Gauche = 1 Then ρ1 = aci.As_long(.TS_AppG.Item(0).indTS) Else ρ1 = .LitSupG.Item(0).Aire / .Ly
                Else
                    d = .dInf
                    If .Type_Acier_Travée = 1 Then ρ1 = aci.As_long(.TS_trav.Item(0).indTS) Else ρ1 = .LitInf.Item(0).Aire / .Ly
                End If
                x1 = CInt(Truncate(.a1 * 100)) : x2 = CInt(Truncate((.Lcx - .a2) * 100))
                V1 = Max(Abs(.VMaxELU.Item(x1)), Abs(.VminELU.Item(x1))) 'au nu d'appui gauche
                V2 = Max(Abs(.VMaxELU.Item(x2)), Abs(.VminELU.Item(x2))) 'au nu d'appui droit
                VEdMax = Max(V1, V2) 'effort tranchant maximum
                k = 1 + (200 / (d * 1000)) ^ 0.5
                If k > 2 Then k = 2
                νmin = (0.34 / 1.5) * bet.fck ^ 0.5
                ρ1 = ρ1 / (1 * d)
                If ρ1 > 0.02 Then ρ1 = 0.02
                Vrdcmin = (νmin * 1000 * d * 1000) / 1000.0 'unité: N -> KN
                Vrdc1 = (Crdc * k * (100 * ρ1 * bet.fck) ^ 1 / 3 * 1000 * (d * 1000)) / 1000 'unité: N -> KN
                Vrdc = Max(Vrdcmin, Vrdc1)
                Call Tab_Tranchant(i1, V1, V2, d, Vrdc1, Vrdcmin, rtf)

                'Try
                '    rtf.WriteStartGroup()
                '    rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.Underline = True : rtf.Bold = True
                '    rtf.WriteString("Graphe") : rtf.Underline = False : rtf.Bold = False : rtf.NewLine()
                '    ' Create Image Element
                '    Dim myImage As Image
                '    msg = "\Images\Test530_330.png"
                '    myImage = Image.FromFile(Directory.GetCurrentDirectory.ToString() & msg)
                '    Dim g As Graphics = Graphics.FromImage(myImage)
                '    Call FrmCourbe.Affichage_Tranchant(g, i1, 2)
                '    myImage.Save(Directory.GetCurrentDirectory.ToString() & "\Images\test1.png", ImageFormat.Bmp)
                '    g.Dispose()
                '    rtf.InsertImage(myImage)
                'Catch ex As Exception
                '    msg = "Courbe effort tranchant: le logiciel n'a pas pu chargé l'image correspondante." & vbCrLf & "Erreur: " & ex.Message & ex.StackTrace & vbCrLf & "Note de calcul sans image"
                '    MsgBox(msg, MsgBoxStyle.Exclamation Or MsgBoxStyle.OkOnly, "Poutre BA EC2")
                'Finally
                '    rtf.WriteEndGroup()
                'End Try

            End With
        Next

        'VvariableMax = Abs(Vvariable(1, 1))
        'If choix_effort_tranchant = 1 Then   'vérification de St à L/2 - variation de Q
        '    For i5 = 1 To combMax(1)
        '        If VvariableMax < Abs(Vvariable(i5, i1)) Then VvariableMax = Abs(Vvariable(i5, i1))
        '    Next i5
        '    tt3 = VvariableMax / (b(i1) * d_Tr)
        '    aAtst = (tt3 - 0.3 * kr(i1) * ft1) * b(i1) * gamaS / (0.9 * fe)
        '    If st(i1, 1, k1(i1, 1)) > st(i1, 2, k1(i1, 2)) Then
        '        Stmaxx = st(i1, 1, k1(i1, 1)) / 100
        '    Else
        '        Stmaxx = st(i1, 2, k1(i1, 2)) / 100
        '    End If
        '    If (nbarre * PI * phiAt(i1) ^ 2 / 4) / Stmaxx < aAtst Then
        '        msg = "Problème de cadres en milieu de travée pour un effort tranchant variable"
        '        MsgBox(msg, vbOKOnly Or vbInformation, "VBpoutre : module de calcul BAEL")
        '        verif = False
        '        Exit Sub
        '    End If
        'End If




    End Sub
    Private Sub Tab_Tranchant(ByVal i1 As Integer, ByVal V1 As Double, ByVal V2 As Double, ByVal d As Double, ByVal Vrdc1 As Double, ByVal Vrdcmin As Double,
                                ByRef rtf As To_Rtf.RtfAnsiTextWriter)

        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        'Ligne 10
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("d") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((d * 100).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("bras de levier pour calcul eq. 6.2a 6.2b et armatures effort tranchant") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ved_G") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((V1 * 1000).ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("║Ved║ au nu de l'appui gauche") : rtf.EndCell() : rtf.EndRow()
        'Ligne 5
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ved_D") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((V2 * 1000).ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("║Ved║ au nu de l'appui droit") : rtf.EndCell() : rtf.EndRow()
        'Ligne 7
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vrdc min") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Vrdcmin.ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Effort tranchant résistant de calcul - valeur minimale Vrdc eq 6.2b") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vrdc eq6.2a") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Vrdc1.ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Effort tranchant résistant de calcul - Vrdc suivant eq 6.2a") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vrdc") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Max(Vrdc1, Vrdcmin)).ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Valeur retenue pour Effort tranchant résistant de calcul Vrdc") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(8800) : rtf.BeginCell(True)
        If V1 * 1000 > Max(Vrdc1, Vrdcmin) Or V2 * 1000 > Max(Vrdc1, Vrdcmin) Then
            rtf.WriteString(" ║Ved║ > Vrdc -> nécessité armature d'effort tranchant")
        Else
            rtf.WriteString(" ║Ved║ < Vrdc - Plancher sans armature d'effort tranchant")
        End If
        rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
    End Sub
#End Region

#Region "Calcul au feu"
    '''<summary>calcul au feu: module d'orientation général</summary>
    ''' <param name="Miso">Moment isostatique de la travée avec les indices (comb, i1, x) comb = n° de combinaison, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="Mur">Moment Ultime résistant avec les indices (ELU_ELS, i1, j, 1) ELU_ELs = n° de combinaison, i1 = n° de travée, j=1,2 ou 3 entravée, sur appui gauche, sur appui droit, 
    ''' 1 = n° de lit, de la surface vers l'intérieur de la dalle</param>
    Private Sub Calcul_au_Feu(ByVal Miso(,,) As Double, ByVal Mur(,,,) As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        '        Vérification au feu de la dalle suivant la stabilité au feu demandée
        'Variables: verification_feu(i2) = True
        Dim i1, message As Integer
        Dim verification_feu As Boolean
        ' calcul au feu sur les dalles
        ' --> dispositions constructives
        For i1 = 1 To PlanCont.Count
            With PlanCont.Item(i1 - 1)
                If .Feu_REI >= 1 Then
                    rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9                    ' 
                    rtf.WriteString(" ► Dispositions constructives:") : rtf.NewLine()
                    rtf.WriteString("Disposition contre les éclatements: la teneur en eau du béton doit être inférieure à 3%.") : rtf.NewLine()
                    If .Prefa = True Then
                        'espace entre prédalles
                        rtf.WriteString("L'espace entre prédalles ne doit pas dépasser 2cm: actuellement prévue " & (.Prefa_esp * 100).ToString("#0.0") & " cm ")
                        If .Prefa_esp > 0.02 Then
                            rtf.WriteString("→ OK") : rtf.NewLine()
                        Else
                            rtf.WriteString("→ NS") : rtf.NewLine()
                        End If
                        'profondeur du joint
                        rtf.WriteString("La profondeur du joint ne doit pas dépasser la moitié de la hauteur de la dalle: actuellement prévue " & (.Prefa_h * 100).ToString("#0.0") & " cm ")
                        If .Prefa_h > .H / 2 Then
                            rtf.WriteString("→ NS") : rtf.NewLine()
                        Else
                            rtf.WriteString("→ OK") : rtf.NewLine()
                        End If
                    End If
                    rtf.NewLine() : rtf.WriteEndGroup()
                    Exit For 'je ne l'écris qu'une fois
                End If
            End With
        Next
        '--> vérification structurale
        message = 0
        For i1 = 1 To PlanCont.Count
            'pas de vérification sur les consoles
            With PlanCont.Item(i1 - 1)
                If .Feu_REI <= 0 Then
                    rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9                    ' 
                    rtf.WriteString("- Travée " & i1.ToString & ": aucune stabilité au feu demandée.") : rtf.NewLine() : rtf.WriteEndGroup()
                    Continue For 'pour passer à l'itération suivante sans prendre en compte ce qui se trouve ci-après
                End If
                If message = 0 Then
                    rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                    rtf.WriteString(" ► Vérifications structurales:") : rtf.NewLine()
                    If Aff.δ < 0.85 Then
                        rtf.WriteString(" Calcul de stabilité au feu établi suivant méthode du §5.7.3(2) de l'AN à l'EN1992-1-2.")
                    Else
                        rtf.WriteString(" Calcul de stabilité au feu établi suivant les valeurs du tableau 5.8 de l'EN1992-1-2.")
                    End If
                    rtf.NewLine() : rtf.WriteEndGroup()
                    message = 1 'pour éviter de répéter le message dans la note de calcul
                End If
                Select Case i1
                    Case 1
                        If PlanCont.Count = 1 Then
                            '1 travée 
                            Calcul_au_Feu_dalle_regle_simple(True, 1, Miso, Mur, verification_feu, rtf) '-->vérification par les règles simples
                        Else
                            If .LappG > 0.05 Then
                                If PlanCont.Count = 2 And PlanCont.Item(1).LappD < 0.05 Then
                                    '2 travées avec porte-a-faux à droite
                                    Calcul_au_Feu_dalle_regle_simple(True, 1, Miso, Mur, verification_feu, rtf)
                                Else
                                    Calcul_au_Feu_dalle_regle_simple(False, 1, Miso, Mur, verification_feu, rtf)
                                End If
                            End If
                        End If
                    Case 2
                        If PlanCont.Item(0).LappG < 0.05 And PlanCont.Count = 2 And PlanCont.Item(1).LappD > 0.05 Then
                            '2 travées avec porte-a-faux à gauche
                            Calcul_au_Feu_dalle_regle_simple(True, 2, Miso, Mur, verification_feu, rtf)
                        ElseIf PlanCont.Item(0).LappG < 0.05 And PlanCont.Count = 3 And PlanCont.Item(2).LappD < 0.05 Then
                            '3 travées avec porte-a-faux à gauche et à droite
                            Calcul_au_Feu_dalle_regle_simple(True, 2, Miso, Mur, verification_feu, rtf)
                        Else
                            Calcul_au_Feu_dalle_regle_simple(False, 2, Miso, Mur, verification_feu, rtf) '-->vérification par les règles simples
                        End If
                    Case PlanCont.Count
                        If PlanCont.Item(PlanCont.Count - 1).LappD > 0.05 Then
                            Calcul_au_Feu_dalle_regle_simple(False, i1, Miso, Mur, verification_feu, rtf) '-->vérification par les règles simples
                        End If
                    Case Else
                        Calcul_au_Feu_dalle_regle_simple(False, i1, Miso, Mur, verification_feu, rtf)
                End Select
            End With
        Next i1
    End Sub
    '''<summary>calcul au feu suivant les règles simples</summary>
    ''' <param name="Miso">Moment isostatique de la travée avec les indices (comb, i1, x) comb = n° de combinaison, i1 = n° de travée, x = abscisse en cm</param>
    ''' <param name="Mur">Moment Ultime résistant avec les indices (ELU_ELS, i1, j, 1) ELU_ELs = n° de combinaison, i1 = n° de travée, j=1,2 ou 3 entravée, sur appui gauche, sur appui droit, 
    ''' 1 = n° de lit, de la surface vers l'intérieur de la dalle</param>
    Private Sub Calcul_au_Feu_dalle_regle_simple(ByVal ISOSTATIQUE As Boolean, ByVal i1 As Integer, ByVal Miso(,,) As Double, ByVal Mur(,,,) As Double, ByRef verification_feu As Boolean, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        'Variables: i1 = n° de travée
        '           i1 = n° de combinaison
        Dim message As Integer = 0
        Dim Tot_Lit As Integer = 0
        Dim da, phiS(3), hmin, enrfeu As Double
        '    If Protection > 1 Then Call Protection_incendie(epfeu) Else epfeu = 0
        Dim Moment_Flechissant_Appui As Boolean = False
        With PlanCont.Item(i1 - 1)
            If ISOSTATIQUE = True Then
                'un seul sens porteur
                Select Case .Feu_REI  'tableau 5.8 EN1992-1-2
                    Case 1 '0.5h
                        hmin = 0.06 : enrfeu = 0.01
                    Case 2 '1h
                        hmin = 0.08 : enrfeu = 0.02
                    Case 3 '1.5h
                        hmin = 0.1 : enrfeu = 0.03
                    Case 4 '2h
                        hmin = 0.12 : enrfeu = 0.04
                    Case 5 '3h
                        hmin = 0.15 : enrfeu = 0.055
                    Case 6 '4h
                        hmin = 0.175 : enrfeu = 0.065
                End Select
            Else
                'dalle continue
                If Aff.δ >= 0.85 Then
                    Select Case .Feu_REI  'tableau 5.8 EN1992-1-2 - colonnes 2 et 4
                        Case 1 '0.5h
                            hmin = 0.06 : enrfeu = 0.01
                        Case 2 '1h
                            hmin = 0.08 : enrfeu = 0.01
                        Case 3 '1.5h
                            hmin = 0.1 : enrfeu = 0.015
                        Case 4 '2h
                            hmin = 0.12 : enrfeu = 0.02
                        Case 5 '3h
                            hmin = 0.15 : enrfeu = 0.03
                        Case 6 '4h
                            hmin = 0.175 : enrfeu = 0.04
                    End Select
                Else
                    hmin = hA_feu(i1)
                    Select Case .Feu_REI  'tableau 5.8 EN1992-1-2
                        Case 1 '0.5h
                            enrfeu = 0.01
                        Case 2 '1h
                            enrfeu = 0.015
                        Case 3 '1.5h
                            enrfeu = 0.02
                        Case 4 '2h
                            enrfeu = 0.025
                        Case 5 '3h
                            enrfeu = 0.04
                    End Select
                    Moment_Flechissant_Appui = True 'Condition à vérifier
                End If
            End If
            rtf.WriteStartGroup()
            rtf.FontSize = 12 : rtf.FontSize = 10 : rtf.Underline = True
            rtf.WriteString("- Travée " & i1.ToString & ": ") : rtf.FontSize = 9 : rtf.Underline = False : rtf.NewLine()
            '--> Vérification espace entre prédalles
            If .Prefa_esp > 0.02 Then
                rtf.WriteString("  - espace entre prédalles doit être inférieur à 2 cm.") : rtf.NewLine()
                Aff.msg_Avertiss.Add("Calcul au feu: pb sur travée " & i1.ToString) : message = 1 'pour éviter de répéter le message dans la note de calcul
            End If
            '--> Vérification de la condition d'épaisseur de dalle
            If (.H + .feu_EpCarrelage) < hmin And Moment_Flechissant_Appui = False Then
                rtf.WriteString("  - hauteur dalle insuffisante, doit être supérieure à = " & (hmin * 100).ToString("#0.0") & " cm minimum (l'épaisseur d'un revêtement de sol incombustible peut compter) - NS") : rtf.NewLine()
                If message = 0 Then Aff.msg_Avertiss.Add("Calcul au feu: pb sur travée " & i1.ToString) : message = 1 'pour éviter de répéter le message dans la note de calcul
            ElseIf (.H + .feu_EpCarrelage) >= hmin And Moment_Flechissant_Appui = False Then
                rtf.WriteString("  - hauteur dalle suffisante (+ ep carrelage éventuel), h minimal = " & (hmin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
            End If
            If .H < hmin And Moment_Flechissant_Appui = True Then
                rtf.WriteString("  - hauteur dalle insuffisante, doit être supérieure à = " & (hmin * 100).ToString("#0.0") & " cm minimum (l'épaisseur d'un revêtement de sol incombustible peut compter) - NS") : rtf.NewLine()
                If message = 0 Then Aff.msg_Avertiss.Add("Calcul au feu - hauteur dalle insuffisant: pb sur travée " & i1.ToString) : message = 1 'pour éviter de répéter le message dans la note de calcul
            ElseIf .H >= hmin And Moment_Flechissant_Appui = True Then
                rtf.WriteString("  - hauteur dalle suffisante (+ ep carrelage éventuel), h minimal = " & (hmin * 100).ToString("#0.0") & " cm - OK") : rtf.NewLine()
            End If
            '--> Vérification de la condition de distance d'entr'axe
            da = .H - .dInf 'Quelque soit le type d'acier TS ou HA
            If da < enrfeu Then
                rtf.WriteString("  - distance à l'axe de l'armature inférieure insuffisante, doit être supérieure à = " & (enrfeu * 100).ToString("#0.0") & " cm minimum - NS") : rtf.NewLine()
                If message = 0 Then Aff.msg_Avertiss.Add("Calcul au feu - distance à l'axe de l'armature inférieure: pb sur travée " & i1.ToString) : message = 1 'pour éviter de répéter le message dans la note de calcul
            Else
                rtf.WriteString("  - distance à l'axe de l'armature inférieure suffisante, distance minimale = " & (enrfeu * 100).ToString("#0.0") & " cm - OK") : rtf.NewLine()
            End If
            '--> Vérification de la condition de moment - Uniquement dalle continue et Aff.δ < 0.85
            If Moment_Flechissant_Appui = True Then

                Select Case i1
                    Case 1
                        If .Type_Acier_Travée = 1 Then
                            Tot_Lit = .TS_AppD.Count
                        Else
                            Tot_Lit = .LitSupD.Count
                        End If
                        'appui droit
                        If Mur(1, i1, 3, Tot_Lit) < 0.5 * Miso_Max(i1, Miso) Then
                            rtf.WriteString("  - le ferraillage sur appui droit n'équilibre pas 50% du moment isostatique en travée: " & (Mur(1, i1, 3, Tot_Lit) * 1000).ToString("#,##0.0") & " < " &
                                            (0.5 * Miso_Max(i1, Miso) * 1000).ToString("#,##0.0") & " kN.m - NS") : rtf.NewLine()
                            If message = 0 Then Aff.msg_Avertiss.Add("Calcul au feu - moment résistant sur appui droit insuffisant: pb sur travée " & i1.ToString) : message = 1 'pour éviter de répéter le message dans la note de calcul
                        Else
                            rtf.WriteString("  - le ferraillage sur appui droit > 50% du moment isostatique en travée: " & (Mur(1, i1, 3, Tot_Lit) * 1000).ToString("#,##0.0") & " < " &
                                            (0.5 * Miso_Max(i1, Miso) * 1000).ToString("#,##0.0") & " kN.m - OK") : rtf.NewLine()
                        End If
                    Case PlanCont.Count
                        If .Type_Acier_Travée = 1 Then
                            Tot_Lit = .TS_AppG.Count
                        Else
                            Tot_Lit = .LitSupG.Count
                        End If
                        'appui gauche
                        If Mur(1, i1, 2, Tot_Lit) < 0.5 * Miso_Max(i1, Miso) Then
                            rtf.WriteString("  - le ferraillage sur appui gauche n'équilibre pas 50% du moment isostatique en travée: " & (Mur(1, i1, 2, Tot_Lit) * 1000).ToString("#,##0.0") & " < " &
                                            (0.5 * Miso_Max(i1, Miso) * 1000).ToString("#,##0.0") & " kN.m - NS") : rtf.NewLine()
                            If message = 0 Then Aff.msg_Avertiss.Add("Calcul au feu - moment résistant sur appui gauche insuffisant: pb sur travée " & i1.ToString) : message = 1 'pour éviter de répéter le message dans la note de calcul
                        Else
                            rtf.WriteString("  - le ferraillage sur appui gauche > 50% du moment isostatique en travée: " & (Mur(1, i1, 2, Tot_Lit) * 1000).ToString("#,##0.0") & " < " &
                                            (0.5 * Miso_Max(i1, Miso) * 1000).ToString("#,##0.0") & " kN.m - OK") : rtf.NewLine()
                        End If
                    Case Else
                        If .Type_Acier_Travée = 1 Then
                            Tot_Lit = .TS_AppG.Count
                        Else
                            Tot_Lit = .LitSupG.Count
                        End If
                        'appui gauche
                        If Mur(1, i1, 2, Tot_Lit) < 0.5 * Miso_Max(i1, Miso) Then
                            rtf.WriteString("  - le ferraillage sur appui gauche n'équilibre pas 50% du moment isostatique en travée: " & (Mur(1, i1, 2, Tot_Lit) * 1000).ToString("#,##0.0") & " < " &
                                            (0.5 * Miso_Max(i1, Miso) * 1000).ToString("#,##0.0") & " kN.m - NS") : rtf.NewLine()
                            If message = 0 Then Aff.msg_Avertiss.Add("Calcul au feu - moment résistant sur appui gauche insuffisant: pb sur travée " & i1.ToString) : message = 1 'pour éviter de répéter le message dans la note de calcul
                        Else
                            rtf.WriteString("  - le ferraillage sur appui gauche > 50% du moment isostatique en travée: " & (Mur(1, i1, 2, Tot_Lit) * 1000).ToString("#,##0.0") & " < " &
                                            (0.5 * Miso_Max(i1, Miso) * 1000).ToString("#,##0.0") & " kN.m - OK") : rtf.NewLine()
                        End If
                        'appui droit
                        If .Type_Acier_Travée = 1 Then
                            Tot_Lit = .TS_AppD.Count
                        Else
                            Tot_Lit = .LitSupD.Count
                        End If
                        If Mur(1, i1, 3, Tot_Lit) < 0.5 * Miso_Max(i1, Miso) Then
                            rtf.WriteString("  - le ferraillage sur appui droit n'équilibre pas 50% du moment isostatique en travée: " & (Mur(1, i1, 3, Tot_Lit) * 1000).ToString("#,##0.0") & " < " &
                                            (0.5 * Miso_Max(i1, Miso) * 1000).ToString("#,##0.0") & " kN.m - NS") : rtf.NewLine()
                            If message = 0 Then Aff.msg_Avertiss.Add("Calcul au feu - moment résistant sur appui droit insuffisant: pb sur travée " & i1.ToString)
                        Else
                            rtf.WriteString("  - le ferraillage sur appui droit > 50% du moment isostatique en travée: " & (Mur(1, i1, 3, Tot_Lit) * 1000).ToString("#,##0.0") & " < " &
                                            (0.5 * Miso_Max(i1, Miso) * 1000).ToString("#,##0.0") & " kN.m - OK") : rtf.NewLine()
                        End If
                End Select
            End If
            rtf.WriteEndGroup()
        End With
    End Sub
    '''<summary>calcul de la hauteur h minimale suivant équation A de l'ANF</summary>
    ''' <param name="i1">n° de travée.</param>
    Private Function hA_feu(ByVal i1 As Integer) As Double
        Dim a0, b0, h0, ΩR, L1, L2, h1, h2 As Double

        With PlanCont.Item(i1 - 1)
            Select Case .Feu_REI  'tableau de l'ANF
                Case 1 '0.5h
                    a0 = -1.81 : b0 = 0.882 : h0 = 0.0564
                Case 2 '1h
                    a0 = -2.67 : b0 = 1.289 : h0 = 0.0715
                Case 3 '1.5h
                    a0 = -3.64 : b0 = 1.868 : h0 = 0.1082
                Case 4 '2h
                    a0 = -5.28 : b0 = 3.097 : h0 = 0.186
                Case 5 '3h
                    a0 = -40.2 : b0 = 105.74 : h0 = 2.224
            End Select
            Select Case .Type_Acier_Travée
                Case 1 'TS
                    ΩR = 0.08
                Case 2 'quadrillage barres HA
                    Select Case aci.classe
                        Case 1 'classe A
                            ΩR = 0.1
                        Case 2, 3 'Classe B ou C
                            ΩR = 0.25
                    End Select
            End Select
            h1 = 0 : h2 = 0
            Select Case i1
                Case 1
                    If PlanCont.Count > 2 Then
                        L1 = (.Lx + 0.8 * PlanCont.Item(i1).Lx) / 2 'appui droit
                    Else
                        L1 = (.Lx + PlanCont.Item(i1).Lx) / 2 'appui droit
                    End If
                    h1 = -h0 + b0 / ((100 * ΩR / L1) - a0)
                Case 2
                    If PlanCont.Count = 2 Then
                        L1 = (.Lx + PlanCont.Item(i1 - 2).Lx) / 2 'appui gauche
                        h1 = -h0 + b0 / ((100 * ΩR / L1) - a0)
                    Else
                        L2 = (0.8 * .Lx + PlanCont.Item(i1 - 2).Lx) / 2 'appui gauche
                        h2 = -h0 + b0 / ((100 * ΩR / L2) - a0)
                        If PlanCont.Count > 3 Then
                            L1 = (0.8 * .Lx + 0.8 * PlanCont.Item(i1).Lx) / 2 'appui droit
                            h1 = -h0 + b0 / ((100 * ΩR / L1) - a0)
                        Else
                            L1 = (0.8 * .Lx + PlanCont.Item(i1).Lx) / 2 'appui droit
                            h1 = -h0 + b0 / ((100 * ΩR / L1) - a0)
                        End If
                    End If
                Case PlanCont.Count
                    L1 = (.Lx + 0.8 * PlanCont.Item(i1 - 2).Lx) / 2 'appui gauche
                    h1 = -h0 + b0 / ((100 * ΩR / L1) - a0)
                Case Else
                    L1 = (0.8 * .Lx + 0.8 * PlanCont.Item(i1).Lx) / 2 'appui droit
                    h1 = -h0 + b0 / ((100 * ΩR / L1) - a0)
                    L2 = (0.8 * .Lx + 0.8 * PlanCont.Item(i1 - 2).Lx) / 2 'appui gauche
                    h2 = -h0 + b0 / ((100 * ΩR / L2) - a0)
            End Select
        End With
        hA_feu = Max(h1, h2)
    End Function
    '''<summary>Recherche du moment isostatique Maximun</summary>
    ''' <param name="i1">n° de travée.</param>
    ''' <param name="Miso">Moment isostatique de la travée avec les indices (comb, i1, x) comb = n° de combinaison, i1 = n° de travée, x = abscisse en cm</param>
    Private Function Miso_Max(ByVal i1 As Integer, ByVal Miso(,,) As Double) As Double
        Dim comb, x As Integer
        Miso_Max = 0 'initialisation
        For comb = 1 To Aff.ELU
            For x = 0 To CInt(Truncate(PlanCont.Item(i1 - 1).Lcx * 100)) Step 1 'recherche de l'abscisse et du moment maximun positif
                If Miso(comb, i1, x) > Miso_Max Then Miso_Max = Miso(comb, i1, x) ' : Xmax = X
            Next x
        Next
    End Function
    Private Sub Protection_incendie(ByRef epfeu As Double)
        ''suivant commentaires du chapitre 8 des règles DTU Feu-Béton (FB 87)
        'Select Case Protection
        '    Case 2 'mortier
        '        epfeu = 0.67 * EpProtect
        '    Case 3 'vermiculite
        '        epfeu = 2.5 * EpProtect
        '    Case 4 'fibre de roche
        '        epfeu = 2.5 * EpProtect
        '    Case 5 'plâtre ordinaire
        '        epfeu = 2.5 * EpProtect
        '    Case 6 'Autre
        '        epfeu = EpProtect
        'End Select
    End Sub
#End Region

#Region "Maitrise de la fissuration"
    Private Sub Fissuration_Predalle(ByVal dalle As Plancher, ByVal Mmax(,,) As Double, ByVal Mmin(,,) As Double, ByVal m1d(,,) As Double, ByVal m2d(,,) As Double, ByVal m3d(,,) As Double,
                                     ByRef rtf As To_Rtf.RtfAnsiTextWriter, ByRef Prefa_Fissuration As Boolean) '
        Dim ELU_ELS As Integer = 4 'ELS quasi permanent
        Dim x1Max(4, PlanCont.Count) As Integer
        Dim x0(4, PlanCont.Count, 2), x1min(4, PlanCont.Count) As Integer
        Dim j As Integer
        ' Dim msg As String = String.Empty
        Dim fctmfl, MS, σbc, σfl, σs As Double
        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
        rtf.WriteString("Vérification de la maitrise de la fissuration pour les planchers à prédalles suivant les conditions de l'article 3.4.2.1 du CPT.") : rtf.NewLine()

        With dalle
            Call Decalage_moment(ELU_ELS, Mmax, Mmin, PlanCont.IndexOf(dalle) + 1, 0, x0, x1Max, x1min, m1d, m2d, m3d)
            For j = 1 To 3
                If PlanCont.IndexOf(dalle) + 1 = 1 And .LappG < 0.05 Then j = 3
                If PlanCont.IndexOf(dalle) + 1 = PlanCont.Count And .LappD < 0.05 Then Exit For
                rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9

                Select Case j
                    Case 1 'acier inférieur longitudinal en travée
                        rtf.WriteString("En travée:") : rtf.NewLine()
                        MS = Mmax(ELU_ELS, PlanCont.IndexOf(dalle) + 1, x1Max(ELU_ELS, PlanCont.IndexOf(dalle) + 1)) 'Calcul de l'acier pour le moment maxi en travée
                        Verification_armature_flexion_ELS(PlanCont.IndexOf(dalle) + 1, j, MS, 15,  ,  , σbc,  , σs,  , fctmfl, σfl)
                        If σfl > fctmfl Then
                            ' Calcul_modele = 1
                            rtf.WriteString("σfl (= " & σfl.ToString("#0.00") & "MPa) > fctmfl (= " & fctmfl.ToString("#0.00") & "MPa) - NS") : rtf.NewLine()
                            rtf.WriteString("La vérification de la maitrise de la fissuration doit être conduite suivant les dispositions du §7.3 de l'EN1992-1-1.") : rtf.NewLine()
                            rtf.WriteEndGroup()
                            Prefa_Fissuration = False : Exit Sub
                        Else
                            rtf.WriteString("σfl (= " & σfl.ToString("#0.00") & "MPa) < fctmfl (= " & fctmfl.ToString("#0.00") & "MPa) - OK") : rtf.NewLine()
                            rtf.WriteString("La vérification de la maitrise de la fissuration est assurée suivant les dispositions du §3.4 du CPT") : rtf.NewLine()
                            rtf.WriteString("Fibre comprimé béton: σbc (= " & σbc.ToString("#0.00") & "MPa - Traction sur armature: " & σs.ToString("#0.00") & "MPa") : rtf.NewLine()
                            Prefa_Fissuration = True
                        End If
                    Case 2
                        If PlanCont.IndexOf(dalle) = 0 Then 'pas de calcul de contrainte sur les aciers de reprise
                            'du moment 0.15xM0
                        Else
                            rtf.WriteString("Sur appui gauche:") : rtf.NewLine()
                            MS = -m2d(ELU_ELS, PlanCont.IndexOf(dalle) + 1, 0) 'Calcul de l'acier pour le moment maxi sur appui gauche
                            Verification_armature_flexion_ELS(PlanCont.IndexOf(dalle) + 1, j, MS, 15,  ,  , σbc,  , σs,  , fctmfl, σfl)
                            If σfl > fctmfl Then
                                ' Calcul_modele = 1
                                rtf.WriteString("σfl (= " & σfl.ToString("#0.00") & "MPa) > fctmfl (= " & fctmfl.ToString("#0.00") & "MPa) - NS") : rtf.NewLine()
                                rtf.WriteString("La vérification de la maitrise de la fissuration doit être conduit suivant les dispositions du §7.3 de l'EN1992-1-1.") : rtf.NewLine()
                                rtf.WriteEndGroup()
                                Prefa_Fissuration = False : Exit Sub
                            Else
                                rtf.WriteString("σfl (= " & σfl.ToString("#0.00") & "MPa) < fctmfl (= " & fctmfl.ToString("#0.00") & "MPa) - OK") : rtf.NewLine()
                                rtf.WriteString("La vérification de la maitrise de la fissuration est assurée suivant les dispositions du §3.4 du CPT") : rtf.NewLine()
                                rtf.WriteString("Fibre comprimé béton: σbc (= " & σbc.ToString("#0.00") & "MPa - Traction sur armature: " & σs.ToString("#0.00") & "MPa") : rtf.NewLine()
                                Prefa_Fissuration = True
                            End If
                        End If
                    Case 3
                        If PlanCont.IndexOf(dalle) = PlanCont.Count - 1 Then 'pas de calcul de contrainte sur les aciers de
                            rtf.WriteEndGroup()  'reprise du moment 0.15xM0
                        Else
                            rtf.WriteString("Sur appui droit:") : rtf.NewLine()
                            MS = -m3d(ELU_ELS, PlanCont.IndexOf(dalle) + 1, CInt(Truncate(.Lcx * 100))) 'Calcul de l'acier pour le moment maxi sur appui droit
                            Verification_armature_flexion_ELS(PlanCont.IndexOf(dalle) + 1, j, MS, 15,  ,  , σbc,  , σs,  , fctmfl, σfl)
                            If σfl > fctmfl Then
                                ' Calcul_modele = 1
                                rtf.WriteString("σfl (= " & σfl.ToString("#0.00") & "MPa) > fctmfl (= " & fctmfl.ToString("#0.00") & "MPa) - NS") : rtf.NewLine()
                                rtf.WriteString("La vérification de la maitrise de la fissuration doit être conduit suivant les dispositions du §7.3 de l'EN1992-1-1.") : rtf.NewLine()
                                rtf.WriteEndGroup()
                                Prefa_Fissuration = False : Exit Sub
                            Else
                                rtf.WriteString("σfl (= " & σfl.ToString("#0.00") & "MPa) < fctmfl (= " & fctmfl.ToString("#0.00") & "MPa) - OK") : rtf.NewLine()
                                rtf.WriteString("La vérification de la maitrise de la fissuration est assurée suivant les dispositions du §3.4 du CPT") : rtf.NewLine()
                                rtf.WriteString("Fibre comprimé béton: σbc (= " & σbc.ToString("#0.00") & "MPa - Traction sur armature: " & σs.ToString("#0.00") & "MPa") : rtf.NewLine()
                                Prefa_Fissuration = True
                            End If
                        End If
                End Select
            Next
        End With
    End Sub
    Private Sub Fissuration_verification_forfaitaire(ByVal tour As Integer, ByVal i1 As Integer, ByRef rtf As To_Rtf.RtfAnsiTextWriter, ByRef verif As Boolean, ByRef Maitrise_Fissuration As Boolean)
        ',
        Dim i, j, j1, ELU_ELS, x1Max(4, PlanCont.Count), boucle As Integer ' x, 
        Dim Asmin, A1, Act, d, Fcteff, k, kc, Lim_Fiss, σs As Double

        Dim msg As String

        boucle = 0
        With PlanCont.Item(i1 - 1)
            ELU_ELS = 4 'ELS quasi permanent
            Fcteff = bet.fctm

            If .XD1 = True Or .XD2 = True Or .XD3 = True Or .XS1 = True Or .XS2 = True Or .XS3 = True Then
                Lim_Fiss = 0.0002
            ElseIf .XC2 = True Or .XC3 = True Or .XC4 = True Then
                Lim_Fiss = 0.0003
            Else
                Lim_Fiss = 0.0004
            End If
            Dim σsLim() As Integer = {160, 200, 240, 280, 320, 360, 400, 450}
            If .H <= 0.2 Then
                σs = aci.fyk 'suivant ANF
            Else
                Select Case Lim_Fiss
                    Case 0.0004 '0,4mm
                        Dim φLim() As Double = {0.04, 0.032, 0.02, 0.016, 0.012, 0.01, 0.008, 0.006}
                        i = 0
                        Select Case .Type_Acier_Travée
                            Case 1 'TS
                                Select Case j
                                    Case 1
                                        Do Until aci.Ølong(.TS_trav.Item(0).indTS) >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                    Case 2
                                        Do Until aci.Ølong(.TS_AppG.Item(0).indTS) >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                    Case 3
                                        Do Until aci.Ølong(.TS_AppD.Item(0).indTS) >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                End Select
                            Case 2 'barre HA
                                Select Case j
                                    Case 1
                                        Do Until .LitInf.Item(0).φHA >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                    Case 2
                                        Do Until .LitSupG.Item(0).φHA >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                    Case 3
                                        Do Until .LitSupD.Item(0).φHA >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                End Select
                        End Select
                        'basé uniquement sur le tableau 7.2
                        σs = σsLim(i)
                    Case 0.0003 '0,3mm
                        Dim φLim() As Double = {0.032, 0.025, 0.016, 0.012, 0.01, 0.008, 0.006, 0.005}
                        Dim EspLim() As Double = {0.3, 0.25, 0.2, 0.15, 0.1, 0.05}
                        i = 0 : j1 = 0
                        Select Case .Type_Acier_Travée
                            Case 1 'TS
                                Select Case j
                                    Case 1
                                        'tableau 7.2
                                        Do Until aci.Ølong(.TS_trav.Item(0).indTS) >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        'tableau 7.3
                                        Do Until aci.Esp_Long(.TS_trav.Item(0).indTS) >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                    Case 2
                                        Do Until aci.Ølong(.TS_AppG.Item(0).indTS) >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        Do Until aci.Esp_Long(.TS_AppG.Item(0).indTS) >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                    Case 3
                                        Do Until aci.Ølong(.TS_AppD.Item(0).indTS) >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        Do Until aci.Esp_Long(.TS_AppD.Item(0).indTS) >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                End Select
                            Case 2 'barre HA
                                Select Case j
                                    Case 1
                                        'tableau 7.2
                                        Do Until .LitInf.Item(0).φHA >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        'tableau 7.3
                                        Do Until .LitInf.Item(0).Scourant >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                    Case 2
                                        Do Until .LitSupG.Item(0).φHA >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        'tableau 7.3
                                        Do Until .LitSupG.Item(0).Scourant >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                    Case 3
                                        Do Until .LitSupD.Item(0).φHA >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        'tableau 7.3
                                        Do Until .LitSupD.Item(0).Scourant >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                End Select
                        End Select
                        'maxi entre tableau 7.2 et 7.3
                        σs = Max(σsLim(i), σsLim(j1))
                    Case 0.0002 '0,2 mm
                        Dim φLim() As Double = {0.025, 0.016, 0.012, 0.008, 0.006, 0.005, 0.004}
                        Dim EspLim() As Double = {0.3, 0.25, 0.2, 0.15, 0.1, 0.05}
                        i = 0 : j1 = 0
                        Select Case .Type_Acier_Travée
                            Case 1 'TS
                                Select Case j
                                    Case 1
                                        'tableau 7.2
                                        Do Until aci.Ølong(.TS_trav.Item(0).indTS) >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        'tableau 7.3
                                        Do Until aci.Esp_Long(.TS_trav.Item(0).indTS) >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                    Case 2
                                        Do Until aci.Ølong(.TS_AppG.Item(0).indTS) >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        Do Until aci.Esp_Long(.TS_AppG.Item(0).indTS) >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                    Case 3
                                        Do Until aci.Ølong(.TS_AppD.Item(0).indTS) >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        Do Until aci.Esp_Long(.TS_AppD.Item(0).indTS) >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                End Select
                            Case 2 'barre HA
                                Select Case j
                                    Case 1
                                        'tableau 7.2
                                        Do Until .LitInf.Item(0).φHA >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        'tableau 7.3
                                        Do Until .LitInf.Item(0).Scourant >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                    Case 2
                                        Do Until .LitSupG.Item(0).φHA >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        'tableau 7.3
                                        Do Until .LitSupG.Item(0).Scourant >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                    Case 3
                                        Do Until .LitSupD.Item(0).φHA >= φLim(i)
                                            i = i + 1
                                            If i = UBound(φLim) Then Exit Do
                                        Loop
                                        'tableau 7.3
                                        Do Until .LitSupD.Item(0).Scourant >= EspLim(j1)
                                            j1 = j1 + 1
                                            If j1 = UBound(EspLim) Then Exit Do
                                        Loop
                                End Select
                        End Select
                        'maxi entre tableau 7.2 et 7.3
                        σs = Max(σsLim(i), σsLim(j1))
                End Select
            End If
            rtf.WriteStartGroup()
            rtf.NewParagraph()
            rtf.FontSize = 12 : rtf.FontSize = 9
            If tour = 1 Then 'pour éviter de l'écrire chaque fois
                'Ligne 1
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
                rtf.DeclareCell(1100) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
                rtf.DeclareCell(5700) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
                'Ligne 2
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Fct,eff") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
                rtf.DeclareCell(1100) : rtf.BeginCell(True) : rtf.WriteString(Fcteff.ToString("#0.00")) : rtf.EndCell()
                rtf.DeclareCell(5700) : rtf.BeginCell(True) : rtf.WriteString("valeur moyenne de la résistance en traction du béton") : rtf.EndCell() : rtf.EndRow()

            End If
            '1 - Vérification de la section minimale d'armature - Application de l'article 7.3.2
            '2 - Vérification de l'ouverture des fissures soit sans calcul direct soit avec calcul direct
            For j = 1 To 3
                If i1 = 1 And .LappG <= 0.05 Then j = 3 'la poutre continue sur sa 1ière travée comporte une console gauche
                If i1 = 1 And j = 2 Then j = 3 'pas de vérification sur le 1ier appui
                If i1 = PlanCont.Count And .LappD <= 0.05 Then j = 2 'la poutre continue sur sa dernière travée comporte une console droite
                If i1 = PlanCont.Count And j = 3 Then Exit For 'pas de vérification sur le dernier appui
                ' 
                kc = 0.4
                k = Max(0.65, 1.21 - 0.7 * .H)
                k = Min(k, 1)
                Act = 1 * .H / 2 'section transversale par ml - par approximation sécuritaire suivant cours Thonier
                Asmin = kc * k * Fcteff * Act / σs
                Select Case j
                    Case 1
                        Select Case .Type_Acier_Travée
                            Case 1 'TS
                                A1 = aci.As_long(.TS_trav.Item(0).indTS)
                                If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                            Case 2 'barres HA
                                If .LitInf.Count = 2 Then A1 = (.LitInf.Item(0).Aire + .LitInf.Item(1).Aire) / .Ly Else A1 = .LitInf.Item(0).Aire / .Ly
                        End Select
                        d = .dInf
                        rtf.BeginRow() : rtf.DeclareCell(8900) : rtf.BeginCell(True) : rtf.WriteString("  - Vérification en travée") : rtf.EndCell() : rtf.EndRow()
                    Case 2
                        Select Case .Type_Acier_Travée
                            Case 1 'TS
                                A1 = aci.As_long(.TS_AppG.Item(0).indTS)
                                If .TS_AppG.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppG.Item(1).indTS)
                            Case 2 'barres HA
                                If .LitSupG.Count = 2 Then A1 = (.LitSupG.Item(0).Aire + .LitSupG.Item(1).Aire) / .Ly Else A1 = .LitSupG.Item(0).Aire / .Ly
                        End Select
                        d = .dSupG
                        rtf.BeginRow() : rtf.DeclareCell(8900) : rtf.BeginCell(True) : rtf.WriteString("  - Vérification sur appui gauche") : rtf.EndCell() : rtf.EndRow()
                    Case 3
                        Select Case .Type_Acier_Travée
                            Case 1 'TS
                                A1 = aci.As_long(.TS_AppD.Item(0).indTS)
                                If .TS_AppD.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppD.Item(1).indTS)
                            Case 2 'barres HA
                                If .LitSupD.Count = 2 Then A1 = (.LitSupD.Item(0).Aire + .LitSupD.Item(1).Aire) / .Ly Else A1 = .LitSupD.Item(0).Aire / .Ly
                        End Select
                        d = .dSupD
                        rtf.BeginRow() : rtf.DeclareCell(8900) : rtf.BeginCell(True) : rtf.WriteString("  - Vérification sur appui droit") : rtf.EndCell() : rtf.EndRow()

                End Select
                'Ligne 15
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("wmax") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
                rtf.DeclareCell(1100) : rtf.BeginCell(True) : rtf.WriteString((Lim_Fiss * 1000).ToString("#0.0 ")) : rtf.EndCell()
                rtf.DeclareCell(5700) : rtf.BeginCell(True) : rtf.WriteString("Limite d'ouverture de fissure") : rtf.EndCell() : rtf.EndRow()
                'Ligne 3
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("σs") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
                rtf.DeclareCell(1100) : rtf.BeginCell(True) : rtf.WriteString(σs.ToString("#0.0")) : rtf.EndCell()
                If .H <= 0.2 Then
                    msg = " suivant condition ANF épaisseur plancher < 200mm."
                Else
                    If Lim_Fiss = 0.0004 Then
                        msg = " suivant tableau 7.2NF"
                    Else
                        'fissure= 0.0002 ou 0.0003
                        If σsLim(i) > σsLim(j1) Then
                            msg = " suivant tableau 7.2NF"
                        Else
                            msg = " suivant tableau 7.3NF"
                        End If
                    End If
                End If
                rtf.DeclareCell(5700) : rtf.BeginCell(True) : rtf.WriteString("σs maxi admise" & msg) : rtf.EndCell() : rtf.EndRow()
                'Ligne 4
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("kc") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
                rtf.DeclareCell(1100) : rtf.BeginCell(True) : rtf.WriteString(kc.ToString(" #0.000")) : rtf.EndCell()
                rtf.DeclareCell(5700) : rtf.BeginCell(True) : rtf.WriteString("coefficient de répartition des contraintes") : rtf.EndCell() : rtf.EndRow()
                'Ligne 5
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("k") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
                rtf.DeclareCell(1100) : rtf.BeginCell(True) : rtf.WriteString(kc.ToString(" #0.000")) : rtf.EndCell()
                rtf.DeclareCell(5700) : rtf.BeginCell(True) : rtf.WriteString("coefficient fonction des contraintes non uniformes auto-équilibrées") : rtf.EndCell() : rtf.EndRow()
                'Ligne 6
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Act") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
                rtf.DeclareCell(1100) : rtf.BeginCell(True) : rtf.WriteString((Act * 10000).ToString("#,##0.0")) : rtf.EndCell()
                rtf.DeclareCell(5700) : rtf.BeginCell(True) : rtf.WriteString("section droite de béton tendu") : rtf.EndCell() : rtf.EndRow()
                'Ligne 7
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Asmin") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
                rtf.DeclareCell(1100) : rtf.BeginCell(True) : rtf.WriteString((Asmin * 10000).ToString("#,##0.00")) : rtf.EndCell()
                rtf.DeclareCell(5700) : rtf.BeginCell(True) : rtf.WriteString("section minimale d'armature dans la zone tendue") : rtf.EndCell() : rtf.EndRow()
                'Ligne 9
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As ELU") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
                rtf.DeclareCell(1100) : rtf.BeginCell(True) : rtf.WriteString((A1 * 10000).ToString("#,##0.00")) : rtf.EndCell()
                If Asmin < A1 Then
                    rtf.DeclareCell(5700) : rtf.BeginCell(True) : rtf.WriteString("As ELU > Asmin -> Vérification sans calcul direct admis - OK") : rtf.EndCell() : rtf.EndRow()
                    Maitrise_Fissuration = True
                Else
                    rtf.DeclareCell(5700) : rtf.BeginCell(True) : rtf.WriteString("As ELU < Asmin -> Vérification sans calcul direct NON admis - NS ") : rtf.EndCell() : rtf.EndRow()
                    Maitrise_Fissuration = False
                End If
                If j = 1 And .H >= 0.5 Then
                    'Application de l'article A.7.3.3(3)
                    rtf.BeginRow() : rtf.DeclareCell(8900) : rtf.BeginCell(True)
                    rtf.WriteString("Hauteur dalle > 50cm - Application A.7.3.3(3) acier de peau") : rtf.EndCell() : rtf.EndRow()
                    '.As_Peau = 0.4 * 0.5 * Fcteff * Act / aci.fyk 'voir cours Thonier
                    '.Ypeau = y1
                    'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As peau") : rtf.EndCell()
                    'rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
                    'rtf.DeclareCell(1100) : rtf.BeginCell(True) : rtf.WriteString((.As_Peau * 10000).ToString("#,##0.00")) : rtf.EndCell()
                    'rtf.DeclareCell(5700) : rtf.BeginCell(True) : rtf.WriteString("A répartir sur les 2 joues entre acier en traction et axe neutre") : rtf.EndCell() : rtf.EndRow()
                End If
                If i1 = PlanCont.Count And j = 2 Then Exit For 'inutile de tester sur j=3
                If PlanCont.Count = 1 Then Exit For 'uniquement en travée
            Next
        End With

        rtf.WriteEndGroup()
    End Sub
    Private Sub Fissuration_Calcul_Direct(ByVal i1 As Integer, ByVal Mmax(,,) As Double, ByVal Mmin(,,) As Double, ByVal m1d(,,) As Double, ByVal m2d(,,) As Double, ByVal m3d(,,) As Double,
                                                     ByRef rtf As To_Rtf.RtfAnsiTextWriter, ByRef verif As Boolean, ByRef Maitrise_Fissuration As Boolean)
        Dim EcmLt, Fcteff, kt, Lim_Fiss, MS, nLt, Srmax, wk, x, αe, ϕ, σs, εsm_cm, ρpeff As Double '
        Dim j, boucle, ELU_ELS, x1 As Integer '
        Dim x0(4, PlanCont.Count, 2), x1min(4, PlanCont.Count), x1Max(4, PlanCont.Count) As Integer
        ELU_ELS = 4
        Fcteff = bet.fctm
        ELU_ELS = 4 'ELS quasi permanent

        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()

        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Fct,eff") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Fcteff.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("valeur moyenne de la résistance en traction du béton") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("k1 - K2 - K4") : rtf.EndCell()
        rtf.DeclareCell(1600) : rtf.BeginCell(True) : rtf.WriteString("= 0.8 :  = 0.5 :  = 0.425") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("valeurs k1, K2 et K4 pour eq. 7.11") : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
        Call Decalage_moment(ELU_ELS, Mmax, Mmin, i1, 0, x0, x1Max, x1min, m1d, m2d, m3d) 'al, courbe_moment, x0, x1Max, x1min, m1d, m2d, m3d) ', ferraillage_sup)
        With PlanCont.Item(i1 - 1)
            If .XD1 = True Or .XD2 = True Or .XD3 = True Or .XS1 = True Or .XS2 = True Or .XS3 = True Then
                Lim_Fiss = 0.0002
            ElseIf .XC2 = True Or .XC3 = True Or .XC4 = True Then
                Lim_Fiss = 0.0003
            Else
                Lim_Fiss = 0.0004
            End If
            'calcul du coefficient d'équivalence Acier/béton n
            If aci.n_Forcer_Fl = True Then
                'Calcul avec un coefficient n forcé
                nLt = aci.n_Fl '  : ϕ = (aci.n_Fl * bet.Ecm / aci.Es) - 1EcmLt = bet.Ecm / (1 + ϕ)
            Else
                'Suivant §7.4.3 et suivant article CSTC = conforme à l'EN1992-1-1
                Calcul_Coef_Fluage(Aff.t0_Fl, Aff.RH_Fl, Aff.tinf_Fl, ϕ, rtf)
                EcmLt = bet.Ecm / (1 + ϕ) : nLt = aci.Es / EcmLt     '  
            End If
            For j = 1 To 3
                If i1 = 1 And .LappG < 0.05 Then j = 3
                If i1 = PlanCont.Count And .LappD < 0.05 Then Exit For
                rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                boucle = 1
                Select Case j
                    Case 1 'acier inférieur longitudinal en travée
                        rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Travée n°" & i1.ToString & ": ouverture fissures de flexion en travée") : rtf.EndCell() : rtf.EndRow()
                        'Position du moment maxi en travée
                        MS = Mmax(ELU_ELS, i1, x1Max(ELU_ELS, i1)) 'Calcul de l'acier pour le moment maxi en travée
                        x1 = CInt(Truncate(x1Max(ELU_ELS, i1) - .a1))
                        If MS = 0 Then 'And frmDon.MessagesInformationToolStripMenuItem.Checked = False
                            rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Sans objet - Moment nul") : rtf.EndCell() : rtf.EndRow()
                            boucle = 0
                        Else
                            Call Verification_armature_flexion_ELS(i1, j, MS, nLt, x, 0, 0, 0, σs)
                            SrmaxEQ711_714(i1, j, x, ρpeff, Srmax, rtf) 'calcul de Srmax suivant EQ7.11 ou  Eq7.14
                        End If
                    Case 2
                        If i1 = 1 Then 'pas de calcul de contrainte sur les aciers de reprise
                            boucle = 0    'du moment 0.15xM0
                        Else
                            rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Travée n°" & i1.ToString & ": ouverture fissures de flexion sur appui gauche") : rtf.EndCell() : rtf.EndRow()
                            MS = -m2d(ELU_ELS, i1, 0) 'Calcul de l'acier pour le moment maxi sur appui gauche
                            x1 = 0
                            Call Verification_armature_flexion_ELS(i1, j, MS, nLt, x, 0, 0, 0, σs)
                            SrmaxEQ711_714(i1, j, x, ρpeff, Srmax, rtf) 'calcul de Srmax suivant EQ7.11 ou  Eq7.14
                        End If
                    Case 3
                        If i1 = PlanCont.Count Then 'pas de calcul de contrainte sur les aciers de
                            boucle = 0      'reprise du moment 0.15xM0
                        Else
                            rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Travée n°" & i1.ToString & ": ouverture fissures de flexion sur appui droit") : rtf.EndCell() : rtf.EndRow()
                            x1 = CInt(Truncate(.Lcx * 100))
                            MS = -m3d(ELU_ELS, i1, x1) 'Calcul de l'acier pour le moment maxi sur appui droit
                            Call Verification_armature_flexion_ELS(i1, j, MS, nLt, x, 0, 0, 0, σs)
                            SrmaxEQ711_714(i1, j, x, ρpeff, Srmax, rtf) 'calcul de Srmax suivant EQ7.11 ou  Eq7.14
                        End If
                End Select
                rtf.WriteEndGroup()
                If boucle = 1 Then
                    kt = 0.4 'charge de longue durée
                    αe = aci.Es / bet.Ecm
                    εsm_cm = (σs - kt * (Fcteff / ρpeff) * (1 + αe * ρpeff)) / 200000
                    If εsm_cm < 0.6 * σs / 200000 Then εsm_cm = 0.6 * σs / 200000
                    wk = Srmax * εsm_cm
                    Call Tab_Ouverture_Fissure2(kt, MS, wk, σs, εsm_cm, αe, Lim_Fiss, rtf)
                End If
            Next
        End With

    End Sub
    Private Sub SrmaxEQ711_714(ByVal i1 As Integer, ByVal j As Integer, ByVal x As Double, ByRef ρpeff As Double, ByRef Srmax As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter,
                               ByVal Optional Orientation As Integer = 1)
        Dim A1, Aceff, c, esp, espMax, hmin, k1, k2, k3, k4, Srmax1, Srmax2, φeq As Double
        k1 = 0.8 : k2 = 0.5 : k4 = 0.425
        With PlanCont.Item(i1 - 1)
            Select Case j
                Case 1
                    c = .enrInf
                    If c <= 0.025 Then
                        k3 = 3.4
                    Else
                        k3 = 3.4 * (25 / (c * 1000)) ^ (2 / 3)
                    End If

                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            If Orientation = 1 Then
                                esp = aci.Esp_Long(.TS_trav.Item(0).indTS)
                                espMax = 5 * (.enrInf + aci.Ølong(.TS_trav.Item(0).indTS) / 2)
                                A1 = aci.As_long(.TS_trav.Item(0).indTS)
                                φeq = aci.Ølong(.TS_trav.Item(0).indTS)
                                If .TS_trav.Count = 2 Then
                                    A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                                    φeq = aci.nbre_fils_N(.TS_trav.Item(0).indTS) * aci.Ølong(.TS_trav.Item(0).indTS) ^ 2 + aci.nbre_fils_N(.TS_trav.Item(1).indTS) * aci.Ølong(.TS_trav.Item(1).indTS) ^ 2
                                    φeq = φeq / (aci.nbre_fils_N(.TS_trav.Item(0).indTS) * aci.Ølong(.TS_trav.Item(0).indTS) + aci.nbre_fils_N(.TS_trav.Item(1).indTS) * aci.Ølong(.TS_trav.Item(1).indTS))
                                End If
                            Else
                                esp = aci.Esp_trans(.TS_trav.Item(0).indTS)
                                espMax = 5 * (.enrInf + aci.Øtrans(.TS_trav.Item(0).indTS) / 2)
                                A1 = aci.As_trans(.TS_trav.Item(0).indTS)
                                φeq = aci.Øtrans(.TS_trav.Item(0).indTS)
                                If .TS_trav.Count = 2 Then
                                    A1 = A1 + aci.As_trans(.TS_trav.Item(1).indTS)
                                    φeq = aci.nbre_fils_ntrans(.TS_trav.Item(0).indTS) * aci.Øtrans(.TS_trav.Item(0).indTS) ^ 2 + aci.nbre_fils_ntrans(.TS_trav.Item(1).indTS) * aci.Øtrans(.TS_trav.Item(1).indTS) ^ 2
                                    φeq = φeq / (aci.nbre_fils_ntrans(.TS_trav.Item(0).indTS) * aci.Øtrans(.TS_trav.Item(0).indTS) + aci.nbre_fils_ntrans(.TS_trav.Item(1).indTS) * aci.Øtrans(.TS_trav.Item(1).indTS))
                                End If
                            End If
                        Case 2 'HA
                            If Orientation = 1 Then
                                esp = .LitInf.Item(0).Scourant
                                espMax = 5 * (.enrInf + .LitInf.Item(0).φHA / 2)
                                A1 = .LitInf.Item(0).Aire / .Ly
                                φeq = .LitInf.Item(0).φHA
                                If .LitInf.Count = 2 Then
                                    A1 = A1 + .LitInf.Item(2).Aire / .Ly
                                    φeq = (.LitInf.Item(0).nbar * .LitInf.Item(0).φHA ^ 2 + .LitInf.Item(1).nbar * .LitInf.Item(1).φHA ^ 2) / (.LitInf.Item(0).nbar * .LitInf.Item(0).φHA + .LitInf.Item(1).nbar * .LitInf.Item(1).φHA)
                                End If
                            Else
                                esp = .LitInf_Y.Item(0).Scourant
                                espMax = 5 * (.enrInf + .LitInf_Y.Item(0).φHA / 2)
                                A1 = .LitInf_Y.Item(0).Aire / .Ly
                                φeq = .LitInf_Y.Item(0).φHA
                                If .LitInf_Y.Count = 2 Then
                                    A1 = A1 + .LitInf_Y.Item(2).Aire / .Ly
                                    φeq = (.LitInf_Y.Item(0).nbar * .LitInf_Y.Item(0).φHA ^ 2 + .LitInf_Y.Item(1).nbar * .LitInf_Y.Item(1).φHA ^ 2) / (.LitInf_Y.Item(0).nbar * .LitInf_Y.Item(0).φHA + .LitInf_Y.Item(1).nbar * .LitInf_Y.Item(1).φHA)
                                End If
                            End If
                    End Select
                    hmin = Min((.H - x) / 3, 0.5 * .H)
                    hmin = Min(hmin, 2.5 * (.H - .dInf))
                Case 2
                    c = .enrSup
                    If c <= 0.025 Then
                        k3 = 3.4
                    Else
                        k3 = 3.4 * (25 / (c * 1000)) ^ (2 / 3)
                    End If
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1 'TS
                            esp = aci.Esp_Long(.TS_AppG.Item(0).indTS)
                            espMax = 5 * (.enrSup + aci.Ølong(.TS_AppG.Item(0).indTS) / 2)
                            A1 = aci.As_long(.TS_AppG.Item(0).indTS)
                            φeq = aci.Ølong(.TS_AppG.Item(0).indTS)
                            If .TS_AppG.Count = 2 Then
                                A1 = A1 + aci.As_long(.TS_AppG.Item(1).indTS)
                                φeq = aci.nbre_fils_N(.TS_AppG.Item(0).indTS) * aci.Ølong(.TS_AppG.Item(0).indTS) ^ 2 + aci.nbre_fils_N(.TS_AppG.Item(1).indTS) * aci.Ølong(.TS_AppG.Item(1).indTS) ^ 2
                                φeq = φeq / (aci.nbre_fils_N(.TS_AppG.Item(0).indTS) * aci.Ølong(.TS_AppG.Item(0).indTS) + aci.nbre_fils_N(.TS_AppG.Item(1).indTS) * aci.Ølong(.TS_AppG.Item(1).indTS))
                            End If
                        Case 2 'HA
                            esp = .LitSupG.Item(0).Scourant
                            espMax = 5 * (.enrSup + .LitSupG.Item(0).φHA / 2)
                            A1 = .LitSupG.Item(0).Aire / .Ly
                            φeq = .LitSupG.Item(0).φHA
                            If .LitSupG.Count = 2 Then
                                A1 = A1 + .LitSupG.Item(2).Aire / .Ly
                                φeq = (.LitSupG.Item(0).nbar * .LitSupG.Item(0).φHA ^ 2 + .LitSupG.Item(1).nbar * .LitSupG.Item(1).φHA ^ 2) / (.LitSupG.Item(0).nbar * .LitSupG.Item(0).φHA + .LitSupG.Item(1).nbar * .LitSupG.Item(1).φHA)
                            End If
                    End Select
                    hmin = Min((.H - x) / 3, 0.5 * .H)
                    hmin = Min(hmin, 2.5 * (.H - .dSupG))
                Case 3
                    c = .enrSup
                    If c <= 0.025 Then
                        k3 = 3.4
                    Else
                        k3 = 3.4 * (25 / (c * 1000)) ^ (2 / 3)
                    End If
                    Select Case .Type_Acier_Appui_Droit
                        Case 1 'TS
                            esp = aci.Esp_Long(.TS_AppD.Item(0).indTS)
                            espMax = 5 * (.enrSup + aci.Ølong(.TS_AppD.Item(0).indTS) / 2)
                            A1 = aci.As_long(.TS_AppD.Item(0).indTS)
                            φeq = aci.Ølong(.TS_AppD.Item(0).indTS)
                            If .TS_AppD.Count = 2 Then
                                A1 = A1 + aci.As_long(.TS_AppD.Item(1).indTS)
                                φeq = aci.nbre_fils_N(.TS_AppD.Item(0).indTS) * aci.Ølong(.TS_AppD.Item(0).indTS) ^ 2 + aci.nbre_fils_N(.TS_AppD.Item(1).indTS) * aci.Ølong(.TS_AppD.Item(1).indTS) ^ 2
                                φeq = φeq / (aci.nbre_fils_N(.TS_AppD.Item(0).indTS) * aci.Ølong(.TS_AppD.Item(0).indTS) + aci.nbre_fils_N(.TS_AppD.Item(1).indTS) * aci.Ølong(.TS_AppD.Item(1).indTS))
                            End If
                        Case 2 'HA
                            esp = .LitSupD.Item(0).Scourant
                            espMax = 5 * (.enrSup + .LitSupD.Item(0).φHA / 2)
                            A1 = .LitSupD.Item(0).Aire / .Ly
                            φeq = .LitSupD.Item(0).φHA
                            If .LitSupD.Count = 2 Then
                                A1 = A1 + .LitSupD.Item(2).Aire / .Ly
                                φeq = (.LitSupD.Item(0).nbar * .LitSupD.Item(0).φHA ^ 2 + .LitSupD.Item(1).nbar * .LitSupD.Item(1).φHA ^ 2) / (.LitSupD.Item(0).nbar * .LitSupD.Item(0).φHA + .LitSupD.Item(1).nbar * .LitSupD.Item(1).φHA)
                            End If
                    End Select
                    hmin = Min((.H - x) / 3, 0.5 * .H)
                    hmin = Min(hmin, 2.5 * (.H - .dSupD))
            End Select
            Aceff = 1 * hmin
            ρpeff = A1 / Aceff
            Srmax2 = 1.3 * (.H - x) 'eq 7.14
            Srmax1 = k3 * c + k1 * k2 * k4 * φeq / ρpeff 'eq 7.11
            'suivant conditions ANF
            If esp < espMax Then
                Srmax = Srmax1
            ElseIf esp > espMax Then
                If Srmax1 > Srmax2 Then
                    Srmax = Srmax1
                Else
                    Srmax = Srmax2
                End If
            End If
            Call Tab_Ouverture_Fissure1(A1, Aceff, Srmax, k3, c, esp, espMax, hmin, x, φeq, ρpeff, Srmax1, Srmax2, rtf)
        End With
    End Sub
    Private Sub Tab_Ouverture_Fissure1(ByVal A1 As Double, ByVal Aceff As Double, ByVal Srmax As Double, ByVal k3 As Double, ByVal c As Double,
                                      ByVal esp As Double, ByVal espMax As Double, ByVal hmin As Double, ByVal x As Double,
                                       ByVal φeq As Double, ByVal ρpeff As Double, ByVal Srmax1 As Double, ByVal Srmax2 As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim msg As String
        Dim j As Integer = 1
        If esp > espMax Then
            If Srmax1 > Srmax2 Then
                j = 1 '   Srmax = Srmax1
            Else
                j = 2 '   Srmax = Srmax2
            End If
        End If
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 7
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("esp Max") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((espMax * 100).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Espacement maximal entre barres suivant 7.3.4(3)") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("esp") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((esp * 100).ToString("#0.000")) : rtf.EndCell()
        If esp < espMax Then msg = "esp < esp Max -> utilisation de l'eq. 7.11 pour le calcul de Srmax" Else msg = "esp > esp Max -> Application eq. 7.14 ou 7.11 suivant ANF"
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
        If j = 1 Then
            'Ligne 9
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("c") : rtf.EndCell()
            rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((c * 100).ToString("#0.0")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True)
            If c > 0.025 Then
                rtf.WriteString("enrobage φ longitudinal - c>2,5cm -> k3 suivant AN") : rtf.EndCell() : rtf.EndRow()
            Else
                rtf.WriteString("enrobage φ longitudinal - c<=2,5cm -> k3 = 3,4") : rtf.EndCell() : rtf.EndRow()
            End If
            'Ligne 10
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("k3") : rtf.EndCell()
            rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(k3.ToString("#0.00")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("eq. 7.11") : rtf.EndCell() : rtf.EndRow()
            'Ligne 11
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("φeq") : rtf.EndCell()
            rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((φeq * 1000).ToString("#0.0")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("eq. 7.12") : rtf.EndCell() : rtf.EndRow()
        End If


        'Ligne 12
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((A1 * 10000).ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section d'acier tendue") : rtf.EndCell() : rtf.EndRow()
        'Ligne 14
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("x") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((x * 100).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("hauteur du béton comprimé") : rtf.EndCell() : rtf.EndRow()
        'Ligne 15
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("hcef") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((hmin * 100).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("hauteur de l'aire de béton autour des aciers - figure 7.1") : rtf.EndCell() : rtf.EndRow()
        'Ligne 16
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Aceff") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Aceff * 10000).ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section effective de béton autour des aciers tendus - figure 7.1") : rtf.EndCell() : rtf.EndRow()
        'Ligne 17
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("ρpeff") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(ρpeff.ToString("#0.000")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("eq. 7.10") : rtf.EndCell() : rtf.EndRow()
        'Ligne 18
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Srmax") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Srmax * 1000).ToString("#0.00")) : rtf.EndCell()
        If esp < espMax Then
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Espacement maximal des fissures - eq. 7.11") : rtf.EndCell() : rtf.EndRow()
        ElseIf esp > espMax Then
            If Srmax1 > Srmax2 Then
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Esp. maximal des fissures - eq. 7.11 > eq. 7.14 (=" & (Srmax2 * 1000).ToString("#0.00")) : rtf.EndCell() : rtf.EndRow()
            Else
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Espacement maximal des fissures - eq. 7.14") : rtf.EndCell() : rtf.EndRow()
            End If
        End If


        rtf.WriteEndGroup()
    End Sub
    Private Sub Tab_Ouverture_Fissure2(ByVal kt As Double, ByVal MS As Double, ByVal wk As Double, ByVal σs As Double, ByVal εsm_cm As Double, ByVal αe As Double,
                                      ByVal Lim_Fiss As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9

        'Ligne 13
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ms") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("MN.m ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(MS.ToString("#0.0000")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Moment maxi sous combinaison quasi-permanente") : rtf.EndCell() : rtf.EndRow()
        'Ligne 19
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("σs") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(σs.ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("contrainte de traction sur l'acier") : rtf.EndCell() : rtf.EndRow()
        'Ligne 20
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("kt") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(kt.ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("eq. 7.9 - charge de longue durée") : rtf.EndCell() : rtf.EndRow()
        'Ligne 21
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("αe") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(αe.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("eq. 7.9 - Rapport Es/Ecm") : rtf.EndCell() : rtf.EndRow()
        'Ligne 22
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("0.6 x σs/Es") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("  ‰") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(((0.6 * σs / 200000) * 1000).ToString("#0.000")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("2nd membre de l'eq. 7.9 - A multiplier par 1000") : rtf.EndCell() : rtf.EndRow()
        'Ligne 23
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("εsm_cm") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("  ‰") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((εsm_cm * 1000).ToString("#0.000")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("eq. 7.9 - A multiplier par 1000") : rtf.EndCell() : rtf.EndRow()
        'Ligne 24
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("wk") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("mm ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((wk * 1000).ToString("#0.000")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Ouverture conventionnelle des fissures - eq. 7.9") : rtf.EndCell() : rtf.EndRow()
        'Select Case Lim_Fiss
        '    Case 0.0004 '0,4mm

        '    Case 0.0003 '0,3mm

        '    Case 0.0002 '0,2 mm

        'End Select
        'Ligne 25
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("wmax") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Lim_Fiss * 1000).ToString("#0.0 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Limite d'ouverture de fissure") : rtf.EndCell() : rtf.EndRow()
        'Ligne 26
        rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True)
        If wk < Lim_Fiss Then
            rtf.WriteString("wk < wmax -> maitrise fissuration OK.") : rtf.EndCell() : rtf.EndRow()
        Else
            rtf.WriteString("wk > wmax -> maitrise fissuration non satisfaisante.") : rtf.EndCell() : rtf.EndRow()
            Aff.msg_Avertiss.Add("Maitrise de la fissuration: wk > wmax -> maitrise fissuration non satisfaisante. ")
        End If
        rtf.WriteEndGroup()
    End Sub
#End Region

#Region "Calcul de la flèche"
    ''' <summary>Vérification de la dispense de flèche par application des conditions de l'article 7.4.2 de l'EN1992-1-1 complété par les précisions apportées par livre Paillé page 360 </summary>
    '''  <param name="Modele">type de dalle: 1=sur 2 appuis; 2=sur 4 appuis</param>
    '''  <param name="rtf">variable renvoyant le fichier de note de calcul sous format rtf</param>
    ''' <param name="Fleche"> variable prenant la valeur False si un calcul explicite de flèche est à réaliser, fleche(i) avec i n° de travée</param>
    ''' <param name="Ainf">Section théorique d'accier pour reprendre l'effort - indice (ELU_ELS, i1, j) avec limites =(3, PlanCont.Count, 4): ELU_ELS=1 à 3(ELU Str, ELU Eq, ELS car); i1= n° travée; j=position armature = 1=position centrale, 2=appui gauche, 3=appui droit, 4=aciers supérieurs sur appuis haut et bas  </param>
    Private Sub Dispense_Calcul_Fleche(ByVal Modele As Integer, ByRef Fleche() As Boolean, ByRef rtf As To_Rtf.RtfAnsiTextWriter, Optional ByVal Ainf(,,) As Double = Nothing)
        Dim i1 As Integer
        Dim A1, As_maj, L_d, K, ρ, ρ1, ρ0 As Double
        ρ0 = Sqrt(bet.fck) * 0.001
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("ρ0") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("%") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((ρ0 * 100).ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Pourcentage d'armature de référence") : rtf.EndCell() : rtf.EndRow()
        For i1 = 1 To PlanCont.Count
            With PlanCont.Item(i1 - 1)
                Select Case i1
                    Case 1
                        If .LappG < 0.05 Then
                            Select Case .Type_Acier_Appui_Droit
                                Case 1 'TS
                                    A1 = aci.As_long(.TS_AppD.Item(0).indTS)
                                    If .TS_AppD.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppD.Item(1).indTS)
                                    ρ = A1 / (.dSupD)
                                Case 2 'barres HA
                                    If .LitSupD.Count = 2 Then A1 = (.LitSupD.Item(0).Aire + .LitSupD.Item(1).Aire) Else A1 = .LitSupD.Item(0).Aire
                                    ρ = A1 / (.Ly * .dSupD) : A1 = A1 / .Ly
                            End Select
                        Else
                            Select Case .Type_Acier_Travée
                                Case 1 'TS
                                    A1 = aci.As_long(.TS_trav.Item(0).indTS)
                                    If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                                    ρ = A1 / (.dInf)
                                Case 2 'barres HA
                                    If .LitInf.Count = 2 Then A1 = (.LitInf.Item(0).Aire + .LitInf.Item(1).Aire) Else A1 = .LitInf.Item(0).Aire
                                    ρ = A1 / (.Ly * .dInf) : A1 = A1 / .Ly
                            End Select
                        End If
                    Case PlanCont.Count
                        If .LappD < 0.05 Then
                            Select Case .Type_Acier_Appui_Gauche
                                Case 1 'TS
                                    A1 = aci.As_long(.TS_AppG.Item(0).indTS)
                                    If .TS_AppG.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppG.Item(1).indTS)
                                    ρ = A1 / (.dSupG)
                                Case 2 'barres HA
                                    If .LitSupG.Count = 2 Then A1 = (.LitSupG.Item(0).Aire + .LitSupG.Item(1).Aire) Else A1 = .LitSupG.Item(0).Aire
                                    ρ = A1 / (.Ly * .dSupG) : A1 = A1 / .Ly
                            End Select
                        Else
                            Select Case .Type_Acier_Travée
                                Case 1 'TS
                                    A1 = aci.As_long(.TS_trav.Item(0).indTS)
                                    If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                                    ρ = A1 / (.dInf)
                                Case 2 'barres HA
                                    If .LitInf.Count = 2 Then A1 = (.LitInf.Item(0).Aire + .LitInf.Item(1).Aire) Else A1 = .LitInf.Item(0).Aire
                                    ρ = A1 / (.Ly * .dInf) : A1 = A1 / .Ly
                            End Select
                        End If
                    Case Else
                        Select Case .Type_Acier_Travée
                            Case 1 'TS
                                A1 = aci.As_long(.TS_trav.Item(0).indTS)
                                If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                                ρ = A1 / (.dInf)
                            Case 2 'barres HA
                                If .LitInf.Count = 2 Then A1 = (.LitInf.Item(0).Aire + .LitInf.Item(1).Aire) Else A1 = .LitInf.Item(0).Aire
                                ρ = A1 / (.Ly * .dInf) : A1 = A1 / .Ly
                        End Select
                End Select
                'calcul coef majorateur (Eq 7.17)
                Select Case Modele
                    Case 1
                        If Ainf(1, i1, 1) > A1 Then
                            'optimisation importante
                            As_maj = 1
                        Else
                            As_maj = 500 / (aci.fyk * Ainf(1, i1, 1) / A1)
                        End If
                    Case 2
                        As_maj = 1
                End Select

                Select Case ρ
                    Case Is <= 0.005 'béton faiblement sollicité
                        Select Case i1
                            Case 1
                                If PlanCont.Count = 1 Then
                                    K = Interpolation_K(1, ρ, ρ0) '  dalle sur appui simple
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(0).LappG = 0 Then
                                    K = Interpolation_K(4, ρ, ρ0) 'console
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(1).LappD = 0 Then
                                    K = Interpolation_K(1, ρ, ρ0) 'dalle sur appui simple
                                Else
                                    K = Interpolation_K(2, ρ, ρ0) 'poutre de rive d'une poutre continue
                                End If
                            Case 2
                                If PlanCont.Count = 2 And PlanCont.Item(0).LappG = 0 Then
                                    K = Interpolation_K(1, ρ, ρ0) 'poutre sur appui simple  
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(1).LappD = 0 Then
                                    K = Interpolation_K(4, ρ, ρ0) 'console
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(0).LappG = 0 And PlanCont.Item(1).LappD = 0 Then
                                    K = Interpolation_K(1, ρ, ρ0) 'poutre sur appui simple
                                ElseIf PlanCont.Count = 2 Then
                                    K = Interpolation_K(2, ρ, ρ0) 'poutre de rive d'une poutre continue
                                Else
                                    K = Interpolation_K(3, ρ, ρ0) 'travée intermédiaire d'une poutre continue
                                End If
                            Case PlanCont.Count
                                If PlanCont.Item(i1 - 1).LappD = 0 Then
                                    K = Interpolation_K(4, ρ, ρ0) 'console
                                Else
                                    K = Interpolation_K(2, ρ, ρ0) 'poutre de rive d'une poutre continue
                                End If
                            Case Else
                                K = Interpolation_K(3, ρ, ρ0) 'travée intermédiaire d'une poutre continue
                        End Select
                    Case Is >= 0.015 'béton fortement sollicité
                        Select Case i1
                            Case 1
                                If PlanCont.Count = 1 Then
                                    K = Interpolation_K(1, ρ, ρ0) 'poutre sur appui simple
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(0).LappG = 0 Then
                                    K = Interpolation_K(4, ρ, ρ0) 'console
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(1).LappD = 0 Then
                                    K = Interpolation_K(1, ρ, ρ0) 'poutre sur appui simple
                                Else
                                    K = Interpolation_K(2, ρ, ρ0) 'poutre de rive d'une poutre continue
                                End If
                            Case 2
                                If PlanCont.Count = 2 And PlanCont.Item(0).LappG = 0 Then
                                    K = Interpolation_K(1, ρ, ρ0) 'poutre sur appui simple  
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(1).LappD = 0 Then
                                    K = Interpolation_K(4, ρ, ρ0) 'console
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(0).LappG = 0 And PlanCont.Item(1).LappD = 0 Then
                                    K = Interpolation_K(1, ρ, ρ0) 'poutre sur appui simple
                                ElseIf PlanCont.Count = 2 Then
                                    K = Interpolation_K(2, ρ, ρ0) 'poutre de rive d'une poutre continue
                                Else
                                    K = Interpolation_K(3, ρ, ρ0) 'travée intermédiaire d'une poutre continue
                                End If
                            Case PlanCont.Count
                                If PlanCont.Item(i1 - 1).LappD = 0 Then
                                    K = Interpolation_K(4, ρ, ρ0) 'console
                                Else
                                    K = Interpolation_K(2, ρ, ρ0) 'poutre de rive d'une poutre continue
                                End If
                            Case Else
                                K = Interpolation_K(3, ρ, ρ0) 'travée intermédiaire d'une poutre continue
                        End Select
                    Case Else
                        Select Case i1
                            Case 1
                                If PlanCont.Count = 1 Then
                                    K = Interpolation_K(1, ρ, ρ0) 'poutre sur appui simple
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(0).LappG = 0 Then
                                    K = Interpolation_K(4, ρ, ρ0) 'console
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(1).LappD = 0 Then
                                    K = Interpolation_K(1, ρ, ρ0) 'poutre sur appui simple
                                Else
                                    K = Interpolation_K(2, ρ, ρ0) 'poutre de rive d'une poutre continue
                                End If
                            Case 2
                                If PlanCont.Count = 2 And PlanCont.Item(0).LappG = 0 Then
                                    K = Interpolation_K(1, ρ, ρ0) 'poutre sur appui simple  
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(1).LappD = 0 Then
                                    K = Interpolation_K(4, ρ, ρ0) 'console
                                ElseIf PlanCont.Count = 2 And PlanCont.Item(0).LappG = 0 And PlanCont.Item(1).LappD = 0 Then
                                    K = Interpolation_K(1, ρ, ρ0) 'poutre sur appui simple
                                ElseIf PlanCont.Count = 2 Then
                                    K = Interpolation_K(2, ρ, ρ0) 'poutre de rive d'une poutre continue
                                Else
                                    'interpolation - travée intermédiaire d'une poutre continue
                                    K = Interpolation_K(3, ρ, ρ0)
                                End If
                            Case PlanCont.Count
                                If PlanCont.Item(i1 - 1).LappD = 0 Then
                                    K = Interpolation_K(4, ρ, ρ0) 'console
                                Else
                                    K = Interpolation_K(2, ρ, ρ0) 'poutre de rive d'une poutre continue
                                End If
                            Case Else
                                'interpolation - travée intermédiaire d'une poutre continue
                                K = Interpolation_K(3, ρ, ρ0)
                        End Select
                End Select
                'ρ compris entre les bornes du tableau 7.4NF de l'AN, les valeurs L/d étant bornées
                If ρ <= 0.005 Then
                    ρ1 = 0.005
                ElseIf ρ >= 0.015 Then
                    ρ1 = 0.015
                Else
                    ρ1 = ρ
                End If
                'Choix de l'eq. 7-16
                If ρ <= ρ0 Then
                    L_d = As_maj * K * (11 + 1.5 * (bet.fck ^ 0.5) * ρ0 / ρ1 + 3.2 * bet.fck ^ 0.5 * (ρ0 / ρ1 - 1) ^ 1.5)
                Else
                    L_d = As_maj * K * (11 + 1.5 * (bet.fck ^ 0.5) * ρ0 / ρ1)
                End If

                'ligne 3
                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Travée n°" & i1.ToString & ": ") : rtf.EndCell() : rtf.EndRow()
                'Ligne 4
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("ρ") : rtf.EndCell()
                rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("%") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((ρ * 100).ToString("#0.00")) : rtf.EndCell()
                If (i1 = 1 And .LappG < 0.05) = True Or (i1 = PlanCont.Count And .LappD < 0.05) = True Then
                    rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Pourcentage d'armature de traction sur appui (console)") : rtf.EndCell() : rtf.EndRow()
                Else
                    rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Pourcentage d'armature de traction à mi-portée") : rtf.EndCell() : rtf.EndRow()
                End If

                'Ligne 5
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("K") : rtf.EndCell()
                rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(K.ToString("#0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("coef. eq 7.16") : rtf.EndCell() : rtf.EndRow()
                'Ligne 5
                Select Case Modele
                    Case 1 'sur 2 appuis
                        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As maj") : rtf.EndCell()
                        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
                        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(As_maj.ToString("#0.00")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("coef. majorateur issue de l'eq 7.17") : rtf.EndCell() : rtf.EndRow()
                    Case 2 'sur 4 appuis

                End Select

                'Ligne 6
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("lim(l/d)") : rtf.EndCell()
                rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(L_d.ToString("#0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("valeur limite du rapport portée/hauteur") : rtf.EndCell() : rtf.EndRow()
                If (Modele = 1 And .Lcx > 7) Or (Modele = 2 And .Lx > 7) Then
                    Select Case Modele
                        Case 1 'sur 2 appuis
                            L_d = (7 / .Lcx) * L_d
                        Case 2
                            L_d = (7 / .Lx) * L_d
                    End Select
                    'Ligne 6
                    rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("lim(l/d)") : rtf.EndCell()
                    rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
                    rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(L_d.ToString("#0.00")) : rtf.EndCell()
                    rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("réduction du rapport L/d par 7/Lcx pour portée > 7m") : rtf.EndCell() : rtf.EndRow()
                End If
                'Ligne 7
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("L") : rtf.EndCell()
                rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True)
                If Modele = 1 Then rtf.WriteString(.Lcx.ToString("#0.00")) Else rtf.WriteString(.Lx.ToString("#0.00"))
                rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Portée de la dalle") : rtf.EndCell() : rtf.EndRow()
                'Ligne 8
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("d") : rtf.EndCell()
                rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(.dInf.ToString("#0.000")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("hauteur utile de la dalle") : rtf.EndCell() : rtf.EndRow()
                'Ligne 9
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("l/d") : rtf.EndCell()
                rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True)
                Select Case Modele
                    Case 1 'sur 2 appuis
                        rtf.WriteString((.Lcx / .dInf).ToString("#0.00")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True)
                        If .Lcx / .dInf < L_d Then
                            rtf.WriteString("L/d < limite(l/d) - Rapport L/d satisfaisant - Calcul de flèche inutile") : rtf.EndCell() : rtf.EndRow()
                            Fleche(i1) = True 'calcul de flèche à NE PAS réaliser
                        Else
                            Fleche(i1) = False 'calcul de flèche à réaliser
                            rtf.WriteString("L/d > limite(l/d) - Rapport L/d Non satisfaisant - Calcul de flèche nécessaire") : rtf.EndCell() : rtf.EndRow()
                        End If
                    Case 2 'sur 4 appuis
                        rtf.WriteString((.Lx / .dInf).ToString("#0.00")) : rtf.EndCell()
                        rtf.DeclareCell(5500) : rtf.BeginCell(True)
                        If .Lx / .dInf < L_d Then
                            rtf.WriteString("L/d < limite(l/d) - Rapport L/d satisfaisant - Calcul de flèche inutile") : rtf.EndCell() : rtf.EndRow()
                            Fleche(i1) = True 'calcul de flèche à NE PAS réaliser
                        Else
                            Fleche(i1) = False 'calcul de flèche à réaliser
                            rtf.WriteString("L/d > limite(l/d) - Rapport L/d Non satisfaisant - Calcul de flèche nécessaire") : rtf.EndCell() : rtf.EndRow()
                        End If
                End Select


            End With

        Next
        rtf.WriteEndGroup()
    End Sub
    ''' <summary>interpolation de K de l'eq. 7.16a ou b. les valeurs interpolation_k sont tirées du livre Paillé tableau page 361 </summary>
    '''  <param name="cas">type de dalle: 1=dalle sur appui simple; 2=travée de rive d'une dalle continue; 3=travée intermédiaire d'une dalle continue; 4=dalle en console</param>
    '''  <param name="ρ">% d'acier</param>
    ''' <param name="ρ0">% de référence d'acier</param>
    Private Function Interpolation_K(ByVal cas As Integer, ByVal ρ As Double, ByVal ρ0 As Double) As Double
        'Voir livre Paillé page 361 pour valeurs de K
        Select Case cas
            Case 1 'dalle sur appui simple
                If ρ <= 0.5 / 100 Then
                    Interpolation_K = 1.5
                ElseIf ρ >= 1.5 / 100 Then
                    Interpolation_K = 1.79
                Else
                    Interpolation_K = 1.5 + 0.29 * (ρ - 0.005)
                End If
            Case 2 'travée de rive d'une dalle continue
                If ρ <= 0.5 / 100 Then
                    Interpolation_K = 1.75
                ElseIf ρ >= 1.5 / 100 Then
                    Interpolation_K = 2.14
                Else
                    Interpolation_K = 1.75 + 0.39 * (ρ - 0.005)
                End If
            Case 3 'travée intermédiaire d'une dalle continue
                If ρ <= 0.5 / 100 Then
                    Interpolation_K = 2
                ElseIf ρ >= 1.5 / 100 Then
                    Interpolation_K = 2.5
                Else
                    Interpolation_K = 2 + 0.5 * (ρ - 0.005)
                End If
            Case 4 'dalle en console
                If ρ <= 0.5 / 100 Then
                    Interpolation_K = 0.6
                ElseIf ρ >= 1.5 / 100 Then
                    Interpolation_K = 0.71
                Else
                    Interpolation_K = 0.6 + 0.11 * (ρ - 0.005)
                End If
        End Select
    End Function
    Private Sub Calcul_Fleche(ByVal ELU_ELS As Integer, ByVal Ind1() As Integer, ByVal Ind2() As Integer, ByVal M(,,) As Double, ByVal EIf(,,) As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        'Calcul de la flèche suivant les conditions de l'article 7.4.3.  
        Dim i1, x_Fl, Comb As Integer
        Dim A1, delta, EcmLt, d, EIFlMax, fctmFl, Fl_Tot, FlI, FlII, Iltnc, Ictnc, Iltc, Ictc, Imc, Mc, Mqp, nCt, nLt, S, XX1, XX2, XX3, x_Ictnc, x_Iltnc, x_Ictc, x_Iltc, xn, ϕ, β, ξ As Double
        β = 0.5
        If aci.n_Forcer_Fl = True Then
            'Calcul avec un coefficient n forcé
            nCt = aci.n_Fl : nLt = aci.n_Fl : ϕ = (aci.n_Fl * bet.Ecm / aci.Es) - 1
            EcmLt = bet.Ecm / (1 + ϕ)
            Call Tab_Initial_Fleche(True, ϕ, β, EcmLt, nCt, nLt, rtf)
        Else
            'Suivant §7.4.3 et suivant article CSTC = conforme à l'EN1992-1-1
            nCt = aci.Es / bet.Ecm
            Calcul_Coef_Fluage(Aff.t0_Fl, Aff.RH_Fl, Aff.tinf_Fl, ϕ, rtf)
            EcmLt = bet.Ecm / (1 + ϕ) : nLt = aci.Es / EcmLt
            Call Tab_Initial_Fleche(False, ϕ, β, EcmLt, nCt, nLt, rtf)
        End If

        For i1 = 1 To PlanCont.Count
            With PlanCont.Item(i1 - 1)
                If i1 = 1 And .LappG < 0.05 Then
                    Select Case .Type_Acier_Appui_Droit
                        Case 1 'TS
                            A1 = aci.As_long(.TS_AppD.Item(0).indTS)
                            If .TS_AppD.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppD.Item(1).indTS)
                        Case 2 'barres HA
                            If .LitSupD.Count = 2 Then A1 = (.LitSupD.Item(0).Aire + .LitSupD.Item(1).Aire) / .Ly Else A1 = .LitSupD.Item(0).Aire / .Ly
                    End Select
                    d = .dSupD
                ElseIf i1 = PlanCont.Count And .LappD < 0.05 Then
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1 'TS
                            A1 = aci.As_long(.TS_AppG.Item(0).indTS)
                            If .TS_AppG.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppG.Item(1).indTS)
                        Case 2 'barres HA
                            If .LitSupG.Count = 2 Then A1 = (.LitSupG.Item(0).Aire + .LitSupG.Item(1).Aire) / .Ly Else A1 = .LitSupG.Item(0).Aire / .Ly
                    End Select
                    d = .dSupG
                Else
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            A1 = aci.As_long(.TS_trav.Item(0).indTS)
                            If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                        Case 2 'barres HA
                            If .LitInf.Count = 2 Then A1 = (.LitInf.Item(0).Aire + .LitInf.Item(1).Aire) / .Ly Else A1 = .LitInf.Item(0).Aire / .Ly
                    End Select
                    d = .dInf
                End If
                'plancher sur une largeur de 1 m.
                '-> non fissurée
                S = 1 * .H + (nCt - 1) * A1
                x_Ictnc = (1 * .H ^ 2 / 2 + (nCt - 1) * A1 * d) / S
                Ictnc = 1 * .H ^ 3 / 12 + 1 * .H * (.H / 2 - x_Ictnc) ^ 2 + (nCt - 1) * A1 * (d - x_Ictnc) ^ 2
                S = 1 * .H + (nLt - 1) * A1
                x_Iltnc = (1 * .H ^ 2 / 2 + (nLt - 1) * A1 * d) / S
                Iltnc = 1 * .H ^ 3 / 12 + 1 * .H * (.H / 2 - x_Iltnc) ^ 2 + (nLt - 1) * A1 * (d - x_Iltnc) ^ 2
                '->  fissurée
                XX1 = 1 / 2
                XX2 = nCt * A1
                XX3 = -nCt * A1 * d
                delta = XX2 ^ 2 - 4 * XX1 * XX3 : delta = Sqrt(delta)
                x_Ictc = (-XX2 + delta) / (2 * XX1)
                Ictc = 1 * x_Ictc ^ 3 / 3 + nCt * A1 * (d - x_Ictc) ^ 2
                XX2 = nLt * A1
                XX3 = -nLt * A1 * d
                delta = XX2 ^ 2 - 4 * XX1 * XX3 : delta = Sqrt(delta)
                x_Iltc = (-XX2 + delta) / (2 * XX1)
                Iltc = 1 * x_Iltc ^ 3 / 3 + nLt * A1 * (d - x_Iltc) ^ 2

                fctmFl = Max((1.6 - .H) * bet.fctm, bet.fctm)
                'calcul de Imc suivant commentaires Paillé page 362 sauf pour la formule qui est fausse (voir CSTC)
                xn = (1 * .H ^ 2 / 2 + nLt * A1 * d) / (1 * .H + nLt * A1)
                Imc = 1 * .H ^ 3 / 12 + 1 * .H * (.H / 2 - xn) ^ 2 + nLt * A1 * (d - xn) ^ 2
                Mc = fctmFl * Imc / (.H - xn)
                Call Fleche_Max(ELU_ELS, Ind1, Ind2, i1, EIf, Comb, x_Fl, EIFlMax)
                Mqp = M(Comb, i1, x_Fl)
                If Mqp > Mc Then
                    ξ = 1 - β * (Mc / Mqp) ^ 2
                Else
                    ξ = 0
                End If
                Select Case β
                    Case 1 'chargement unique de courte durée
                        FlII = EIFlMax / (bet.Ecm * Ictc) : FlI = EIFlMax / (bet.Ecm * Ictnc)
                    Case 0.5 'chargement prolongé
                        FlII = EIFlMax / (EcmLt * Iltc) : FlI = EIFlMax / (EcmLt * Iltnc)
                End Select
                Fl_Tot = ξ * FlII + (1 - ξ) * FlI
                Call Tab_Intermediaire_Fleche(i1, Ictnc, x_Ictnc, Iltnc, x_Iltnc, Ictc, x_Ictc, Iltc, x_Iltc, Mc, Mqp, x_Fl, Comb, ξ, FlI, FlII, Fl_Tot, .Lcx, rtf)

            End With
        Next

    End Sub
    Private Sub Fleche_Max(ByVal ELU_ELS As Integer, ByVal Ind1() As Integer, ByVal Ind2() As Integer, ByVal i1 As Integer, ByVal EIf(,,) As Double, ByRef Comb As Integer, ByRef x_Fl As Integer,
                           ByRef EIFlMax As Double)
        Dim Nint, x, i2 As Integer
        Nint = CInt(Truncate(PlanCont.Item(i1 - 1).Lcx * 100))
        x_Fl = 0 : EIFlMax = 0
        For i2 = Ind1(4) To Ind2(4)
            For x = 0 To Nint - 1 Step 1
                If Abs(EIf(i2, i1, x)) > EIFlMax Then
                    x_Fl = x : Comb = i2 : EIFlMax = Abs(EIf(i2, i1, x))
                End If
            Next
        Next

    End Sub
    Public Sub Tab_Initial_Fleche(ByVal n_Forcé As Boolean, ByVal ϕ As Double, ByVal β As Double, ByVal EcmLt As Double, ByVal nCt As Double, ByVal nLt As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim msg As String
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString(" ϕ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(ϕ.ToString("#0.00")) : rtf.EndCell()
        If n_Forcé = True Then msg = "Valeur de n forcée - ϕ calculé à partir de la valeur de n" Else msg = "fluage global estimé"
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("β") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(β.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True)
        Select Case β
            Case 1 'chargement unique de courte durée
                rtf.WriteString("chargement unique de courte durée") : rtf.EndCell() : rtf.EndRow()
            Case 0.5
                rtf.WriteString("chargement longue durée") : rtf.EndCell() : rtf.EndRow()
        End Select
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ecmct") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(bet.Ecm.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Module d'élasticité béton court terme") : rtf.EndCell() : rtf.EndRow()
        'Ligne 4
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("EcmLt") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(EcmLt.ToString("#0.00")) : rtf.EndCell()
        If n_Forcé = True Then msg = " avec valeur de n forcée" Else msg = " "
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Module d'élasticité béton long terme" & msg) : rtf.EndCell() : rtf.EndRow()
        'Ligne 5
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("nct") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(nCt.ToString("#0.00")) : rtf.EndCell()
        If n_Forcé = True Then msg = "= valeur de n forcée" Else msg = "Coef équivalence acier/béton court terme "
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
        'Ligne 6
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("nLt") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(nLt.ToString("#0.00")) : rtf.EndCell()
        If n_Forcé = True Then msg = "= valeur de n forcée" Else msg = "Coef équivalence acier/béton long terme "
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
    End Sub
    ''' <summary>
    ''' Affichage de valeurs intermédiaires du calcul de flèche
    ''' </summary>
    ''' <param name="i1">n° de travée  </param>
    Private Sub Tab_Intermediaire_Fleche(ByVal i1 As Integer, ByVal Ictnc As Double, ByVal x_Ictnc As Double, ByVal Iltnc As Double, ByVal x_Iltnc As Double,
                                         ByVal Ictc As Double, ByVal x_Ictc As Double, ByVal Iltc As Double, ByVal x_Iltc As Double, ByVal Mc As Double,
                                         ByVal Mqp As Double, ByVal x_Fl As Integer, ByVal Comb As Integer, ByVal ξ As Double, ByVal FlI As Double, ByVal FlII As Double,
                                         ByVal Fl_Tot As Double, ByVal L As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim msg As String
        Dim Ne_pas_Afficher As Boolean = False
        If x_Fl < -1 Then Ne_pas_Afficher = True 'données provenant d'une dalle bidirectionnelle
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        With PlanCont.Item(i1 - 1)
            'Ligne 7
            rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString("Travée n°" & i1.ToString) : rtf.EndCell() : rtf.EndRow()
            If Ne_pas_Afficher = False Then
                'Ligne 7
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ictnc") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("dm4") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Ictnc * 10000.0).ToString("#0.0000")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Inertie court terme non fissurée") : rtf.EndCell() : rtf.EndRow()
                'Ligne 8
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("xn") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((x_Ictnc * 100).ToString("#0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("axe neutre court terme non fissurée") : rtf.EndCell() : rtf.EndRow()
            End If

            'Ligne 9
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Iltnc") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("dm4") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Iltnc * 10000.0).ToString("#0.0000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Inertie long terme non fissurée") : rtf.EndCell() : rtf.EndRow()
            'Ligne 10
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("xn") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((x_Iltnc * 100).ToString("#0.0")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("axe neutre long terme non fissurée") : rtf.EndCell() : rtf.EndRow()
            If Ne_pas_Afficher = False Then
                'Ligne 11
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ictc") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("dm4") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Ictc * 10000.0).ToString("#0.0000")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Inertie court terme fissurée") : rtf.EndCell() : rtf.EndRow()
                'Ligne 12
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("xn") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((x_Ictc * 100).ToString("#0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("axe neutre court terme fissurée") : rtf.EndCell() : rtf.EndRow()
            End If

            'Ligne 13
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Iltc") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("dm4") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Iltc * 10000.0).ToString("#0.0000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Inertie long terme fissurée") : rtf.EndCell() : rtf.EndRow()
            'Ligne 14
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("xn") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((x_Iltc * 100).ToString("#0.0")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("axe neutre long terme fissurée") : rtf.EndCell() : rtf.EndRow()
            'Ligne 15
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mc") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("MN.m") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Mc.ToString("#0.0000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("moment de fissuration") : rtf.EndCell() : rtf.EndRow()
            If Ne_pas_Afficher = False Then
                'Ligne 16
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("x fl") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(((x_Fl - PlanCont.Item(i1 - 1).a1 * 100) / 100).ToString("#0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("abscisse de la flèche maximale depuis nu d'appui gauche") : rtf.EndCell() : rtf.EndRow()
            Else
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Fl rdm") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((.ELS_QP(Comb).W_fl_min.wfl * 1000).ToString("#0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Flèche QP calculée par solution de Navier - x = " & (.ELS_QP(Comb).W_fl_min.x * 100).ToString("#0.0") & " cm - y= " &
                                                            (.ELS_QP(Comb).W_fl_min.y * 100).ToString("#0.0") & " cm") : rtf.EndCell() : rtf.EndRow()
            End If
            'Ligne 17
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mqp") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("MN.m") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Mqp.ToString("#0.0000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Mt quasi-permanent pour les mêmes coordonnées et même combinaison") : rtf.EndCell() : rtf.EndRow()

            'Ligne 18
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("ξ") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(ξ.ToString("#0.000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("coefficient de distribution") : rtf.EndCell() : rtf.EndRow()
            'Ligne 19
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Fl_I") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((FlI * 1000).ToString("#0.0")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("flèche section non fissurée") : rtf.EndCell() : rtf.EndRow()
            'Ligne 20
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Fl_II") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((FlII * 1000).ToString("#0.0")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("flèche section fissurée") : rtf.EndCell() : rtf.EndRow()
            'Ligne 21
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Fl totale") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Fl_Tot * 1000).ToString("#0.0")) : rtf.EndCell()
            If L / 250 > Fl_Tot Then msg = " < L/250 (= " & ((L / 250) * 1000).ToString("#0.0") & " mm) -> OK." Else msg = " > L/250 (= " & ((L / 250) * 1000).ToString("#0.0") & " mm) -> NS."
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("flèche totale" & msg) : rtf.EndCell() : rtf.EndRow()
        End With
        rtf.WriteEndGroup()
    End Sub
    Private Sub Fleche_Predalle(ByVal Mmin(,,) As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim a, A1, d, delta, Ecef, fqp, fqp1, fa, ft, fs, f1, f2, G12v, Gap, Gq, Iuc, Ifc, Qdominant, Qtot, M0, Mgvga, Mcr, QΨ0_M0, QΨ0, QΨ2, xn, XX1, XX2, XX3, ξi, ξia, α, δw, δe, Ψ As Double '
        Dim Ndominant As Integer
        Dim Ψ2() As Double = {0.3, 0.6, 0.8, 0.6, 0.3, 0}
        Dim Ψ0() As Double = {0.7, 0.7, 1, 0.7, 0.7, 0}
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
        For Each dalle As Plancher In PlanCont
            With dalle
                If PlanCont.IndexOf(dalle) = 0 And .LappG < 0.05 Then
                    'acier sur appui droit pour une console gauche
                    Select Case .Type_Acier_Appui_Droit
                        Case 1 'TS
                            A1 = aci.As_long(.TS_AppD.Item(0).indTS)
                            If .TS_AppD.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppD.Item(1).indTS)
                        Case 2 'barres HA
                            If .LitSupD.Count = 2 Then A1 = (.LitSupD.Item(0).Aire + .LitSupD.Item(1).Aire) / .Ly Else A1 = .LitSupD.Item(0).Aire / .Ly
                    End Select
                    d = .dSupD
                ElseIf PlanCont.IndexOf(dalle) = PlanCont.Count - 1 And .LappD < 0.05 Then
                    'acier sur appui gauche pour une console droite
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1 'TS
                            A1 = aci.As_long(.TS_AppG.Item(0).indTS)
                            If .TS_AppG.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppG.Item(1).indTS)
                        Case 2 'barres HA
                            If .LitSupG.Count = 2 Then A1 = (.LitSupG.Item(0).Aire + .LitSupG.Item(1).Aire) / .Ly Else A1 = .LitSupG.Item(0).Aire / .Ly
                    End Select
                    d = .dSupG
                Else
                    'sinon acier en travée
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            A1 = aci.As_long(.TS_trav.Item(0).indTS)
                            If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                        Case 2 'barres HA
                            If .LitInf.Count = 2 Then A1 = (.LitInf.Item(0).Aire + .LitInf.Item(1).Aire) / .Ly Else A1 = .LitInf.Item(0).Aire / .Ly
                    End Select
                    d = .dInf
                End If
                'calcul de I  suivant commentaires Paillé page 362 sauf pour la formule qui est fausse (voir CSTC)
                '->  non fissurée
                xn = (1 * .H ^ 2 / 2 + 15 * A1 * d) / (1 * .H + 15 * A1)
                Iuc = 1 * .H ^ 3 / 12 + 1 * .H * (.H / 2 - xn) ^ 2 + 15 * A1 * (d - xn) ^ 2 'Inertie Section homogénéisée
                Mcr = bet.fctm * Iuc / (.H - xn)
                '->  fissurée
                XX1 = 1 / 2
                XX2 = 15 * A1
                XX3 = -15 * A1 * d
                delta = XX2 ^ 2 - 4 * XX1 * XX3 : delta = Sqrt(delta)
                xn = (-XX2 + delta) / (2 * XX1)
                Ifc = 1 * xn ^ 3 / 3 + 15 * A1 * (d - xn) ^ 2 'Inertie Section homogénéisée réduite
                G12v = 0 : Gap = 0 : Gq = 0 : Qdominant = 0 : Ndominant = 0
                For Each chge As Charge In PlanCont.Item(PlanCont.IndexOf(dalle)).Chge
                    If chge.Nature = 1 And chge.schem = 1 Then G12v = G12v + chge.P1 * chge.taux
                    If chge.Nature = 1 And chge.schem = 1 Then Gap = Gap + chge.P1 * (1 - chge.taux)
                    If chge.Nature > 1 And chge.Nature <= 7 And chge.schem = 1 Then
                        Gq = Gq + chge.P1 * chge.taux 'part permanente de la charge d'exploitation
                        'même si lelogiciel ne prends pas en compte le concept de charge dominante, pour le calcul de la flèche, je prends la charge la plus importante en dominante.
                        If chge.P1 > Qdominant Then Qdominant = chge.P1 : Ndominant = PlanCont.Item(PlanCont.IndexOf(dalle)).Chge.IndexOf(chge)
                    End If
                Next
                QΨ0 = 0 : QΨ0_M0 = 0 : QΨ2 = 0 : Qtot = 0
                For Each chge As Charge In PlanCont.Item(PlanCont.IndexOf(dalle)).Chge
                    If chge.Nature > 1 And chge.Nature <= 7 And chge.schem = 1 Then 'toutes catégories sauf permanent, neige et vent
                        If PlanCont.Item(PlanCont.IndexOf(dalle)).Chge.IndexOf(chge) <> Ndominant Then
                            QΨ0 = QΨ0 + chge.P1 * Ψ0(chge.Nature - 2) * (1 - chge.taux)
                            QΨ0_M0 = QΨ0_M0 + chge.P1 * Ψ0(chge.Nature - 2)
                        End If
                        QΨ2 = QΨ2 + chge.P1 * Ψ2(chge.Nature - 2)
                        Qtot = Qtot + chge.P1
                    End If
                Next
                M0 = (G12v + Gap + Qdominant + QΨ0_M0) * .Lx ^ 2 / 8 'Moment isostatique sous charge caractéristique
                If M0 <= Mcr Then
                    ξi = 0
                Else
                    ξi = 1 - Sqrt(Mcr / M0)
                End If
                Ecef = bet.Ecm / 3
                α = Qtot / (G12v + Gap + Qtot)
                M0 = (G12v + Gap + QΨ2) * .Lx ^ 2 / 8 'Moment isostatique sous charge QP
                'Moment à l'ELS caractéristique pour être cohérent avec M0
                Select Case PlanCont.IndexOf(dalle)
                    Case 0
                        δe = 0 : δw = Abs(Mmin(4, PlanCont.IndexOf(dalle) + 1, CInt(Truncate(.Lcx * 100))) / M0)
                    Case PlanCont.Count - 1
                        δe = Abs(Mmin(4, PlanCont.IndexOf(dalle) + 1, 0) / M0) : δw = 0
                    Case Else
                        If PlanCont.Count > 1 Then
                            δe = Abs(Mmin(4, PlanCont.IndexOf(dalle) + 1, 0) / M0) : δw = Abs(Mmin(4, PlanCont.IndexOf(dalle) + 1, CInt(Truncate(.Lcx * 100))) / M0)
                        Else
                            δe = 0 : δw = 0
                        End If
                End Select
                If PlanCont.Count = 1 Then
                    a = 1
                ElseIf PlanCont.IndexOf(dalle) = 0 And .LappG < 0.05 Then
                    a = 1
                ElseIf PlanCont.IndexOf(dalle) = PlanCont.Count - 1 And .LappD < 0.05 Then
                    a = 1
                Else
                    a = 1 - 1.2 * ((δe + δw) / 2 - 0.3 * α)
                End If
                '--> calcul de la flèche quasi-permanente
                fqp1 = (.Lx ^ 2 / (8 * Ecef)) * ((1 - ξi) / Iuc + ξi / Ifc) * (G12v + Gap + QΨ2) * (a * .Lx ^ 2 / 9.6)
                fqp = fqp1 + 0.00035 * .Lx ^ 2 / (8 * d)
                '
                '--> calcul de la flèche active
                ft = (.Lx ^ 2 / (8 * Ecef)) * ((1 - ξi) / Iuc + ξi / Ifc) * (G12v + Gap + Gq + (Qdominant + QΨ0 - Gq) / 3) * (a * .Lx ^ 2 / 9.6)
                ft = ft + 0.00035 * .Lx ^ 2 / (8 * d)
                Mgvga = (G12v + Gap) * .Lx ^ 2 / 8
                If Mgvga <= Mcr Then
                    ξia = 0
                Else
                    ξia = 1 - Sqrt(Mcr / Mgvga)
                End If
                f1 = (.Lx ^ 2 / (8 * bet.Ecm)) * ((1 - ξia) / Iuc + ξia / Ifc) * (G12v + Gap) * (a * .Lx ^ 2 / 9.6)
                f1 = f1 + (2 / 5) * 0.00035 * .Lx ^ 2 / (8 * d)
                f2 = (.Lx ^ 2 / (8 * Ecef)) * ((1 - ξia) / Iuc + ξia / Ifc) * (G12v + Gap / 3) * (a * .Lx ^ 2 / 9.6)
                f2 = f2 + 0.00035 * .Lx ^ 2 / (8 * d)
                If .Prefa_t <= 90 Then
                    Ψ = 0.5 * .Prefa_t / 90
                Else
                    Ψ = 0.5
                End If
                fs = f1 + Ψ * (f2 - f1)
                fa = ft - fs

                Tab_Fleche_Predal(PlanCont.IndexOf(dalle), G12v, Gap, QΨ2, ξi, Iuc, Ifc, δe, δw, α, a, fqp1, fqp - fqp1, fqp, .Lx, Qdominant, Gq, ft, ξia, f1, f2, Ψ, fs, fa, Mcr, rtf)
            End With
        Next
    End Sub
    Private Sub Tab_Fleche_Predal(ByVal i1 As Integer, ByVal G12v As Double, ByVal Gap As Double, ByVal QΨ2 As Double, ByVal ξi As Double, ByVal Iuc As Double,
                                  ByVal Ifc As Double, ByVal δe As Double, ByVal δw As Double, ByVal α As Double, ByVal a As Double, ByVal fi As Double, ByVal fii As Double,
                                 ByVal fqp As Double, ByVal L As Double, ByVal Qdominant As Double, ByVal Gq As Double, ByVal ft As Double, ByVal ξia As Double,
                                  ByVal f1 As Double, ByVal f2 As Double, ByVal Ψ As Double, ByVal fs As Double, ByVal fa As Double, ByVal Mcr As Double,
                                  ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim msg As String
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString("Travée n°" & (i1 + 1).ToString("#")) : rtf.EndCell() : rtf.EndRow()
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString("Vérification de la flèche sous combinaison Quasi-Permanente avec limite L/250 suivant §3.5.3 du CPT") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("G1+G2+Gv") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("daN/m²") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((G12v * 100000.0).ToString("#,##0.0 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("PP predalle+ PP dalle/predalle + CP avant mise en oeuvre") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Gap") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("daN/m²") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Gap * 100000.0).ToString("#,##0.0 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Ga + Gp = CP après mise en oeuvre") : rtf.EndCell() : rtf.EndRow()
        'Ligne 4
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Σ Q.Ψ2") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("daN/m²") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((QΨ2 * 100000.0).ToString("#,##0.0 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Somme charges d'exploitation sous ELS QP") : rtf.EndCell() : rtf.EndRow()
        'Ligne 4
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mcr") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mcr * 1000).ToString("#,##0.00 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Moment de fissuration") : rtf.EndCell() : rtf.EndRow()
        'Ligne 5
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("ξi") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(ξi.ToString("#0.00 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("coef. répartition ") : rtf.EndCell() : rtf.EndRow()
        'Ligne 6
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Iuc") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("dm4") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Iuc * 10000).ToString("#,##0.0000")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Moment d'inertie en section non fissurée homogénéisée") : rtf.EndCell() : rtf.EndRow()
        'Ligne 7
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ifc") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("dm4") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Ifc * 10000).ToString("#,##0.0000")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Moment d'inertie en section fissurée homogénéisée réduite") : rtf.EndCell() : rtf.EndRow()
        If PlanCont.Count = 1 Or (i1 = 0 And PlanCont.Item(0).LappG < 0.05) Or (i1 = PlanCont.Count - 1 And PlanCont.Item(i1).LappD < 0.05) Then
            'on affiche rien car non pertinent
        Else
            'Ligne 8
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString(" δe") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(δe.ToString(" #0.000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Rapport Me/M0") : rtf.EndCell() : rtf.EndRow()
            'Ligne 9
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString(" δw") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(δw.ToString(" #0.000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Rapport Mw/M0") : rtf.EndCell() : rtf.EndRow()
            'Ligne 10
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString(" α") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(α.ToString(" #0.000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Chge Exploit./ Chge totale") : rtf.EndCell() : rtf.EndRow()
            'Ligne 11
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString(" a") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(a.ToString(" #0.000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Prise en compte de la continuité") : rtf.EndCell() : rtf.EndRow()
        End If
        'Ligne 12
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Fl charge") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((fi * 1000).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("flèche sous charge") : rtf.EndCell() : rtf.EndRow()
        'Ligne 13
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Fl retrait") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((fii * 1000).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("flèche due au retrait") : rtf.EndCell() : rtf.EndRow()
        'Ligne 14
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Fqp") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((fqp * 1000).ToString("#0.0")) : rtf.EndCell()
        If L / 250 > fqp Then msg = " < L/250 (= " & ((L / 250) * 1000).ToString("#0.0") & " mm) -> OK." Else msg = " > L/250 (= " & ((L / 250) * 1000).ToString("#0.0") & " mm) -> NS."
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("flèche totale sous ELS Quasi-Permanent" & msg) : rtf.EndCell() : rtf.EndRow()
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString("Vérification de la flèche active avec limite L/500 ou plus suivant §3.5.3 du CPT") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Gq") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("daN/m²") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Gq * 100000.0).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Part permanente des charges d'exploitation") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Q") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("daN/m²") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Qdominant * 100000.0).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Charge d'exploitation dominante") : rtf.EndCell() : rtf.EndRow()
        'Ligne 4
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("ft") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((ft * 1000).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("flèche totale") : rtf.EndCell() : rtf.EndRow()
        'Ligne 5
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("ξ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(ξia.ToString("#0.000 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("coef. répartition ") : rtf.EndCell() : rtf.EndRow()
        'Ligne 6
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("f1") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((f1 * 1000).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("flèche si éléments fragile posés immédiatement après désétaiement") : rtf.EndCell() : rtf.EndRow()
        'Ligne 7
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("f2") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((f2 * 1000).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("flèche si éléments fragile posés longtemps après désétaiement") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("t") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("jours") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(PlanCont.Item(0).Prefa_t.ToString("#0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("temps après désétaiement") : rtf.EndCell() : rtf.EndRow()
        'Ligne 9
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString(" Ψ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Ψ.ToString("#0.000 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("coef. répartition ") : rtf.EndCell() : rtf.EndRow()
        'Ligne 10
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("fs") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((fs * 1000).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("flèche avant pose éléments fragile") : rtf.EndCell() : rtf.EndRow()
        'Ligne 21
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("fa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((fa * 1000).ToString("#0.0")) : rtf.EndCell()
        Select Case PlanCont.Item(0).Prefa_fragile
            Case True
                If L > 7 Then
                    If (L / 1000 + 0.007) > fa Then msg = " < L / 1000 + 0.007 (= " & ((L / 1000 + 0.007) * 1000).ToString("#0.0") & " mm) -> OK." Else msg = " > L / 1000 + 0.007 (= " & ((L / 1000 + 0.007) * 1000).ToString("#0.0") & " mm) -> NS."
                Else
                    If L / 500 > fa Then msg = " < L/500 (= " & ((L / 500) * 1000).ToString("#0.0") & " mm) -> OK." Else msg = " > L/500 (= " & ((L / 500) * 1000).ToString("#0.0") & " mm) -> NS."
                End If
            Case False
                If L > 7 Then
                    If (L / 700 + 0.01) > fa Then msg = " < L / 700 + 0.01 (= " & ((L / 700 + 0.01) * 1000).ToString("#0.0") & " mm) -> OK." Else msg = " > L / 700 + 0.01 (= " & ((L / 700 + 0.01) * 1000).ToString("#0.0") & " mm) -> NS."
                Else
                    If L / 350 > fa Then msg = " < L/350 (= " & ((L / 350) * 1000).ToString("#0.0") & " mm) -> OK." Else msg = " > L/350 (= " & ((L / 350) * 1000).ToString("#0.0") & " mm) -> NS."
                End If
        End Select
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("flèche active" & msg) : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()

    End Sub
#End Region

#Region "Calcul phase provisoire Prédalle"
    ''' <summary>Verification du Repos nominal sur Appui des Predalles: Calcul établi suivant l'annexe A du CPT</summary>
    Private Sub Verification_Repos_Appui_Predalle(ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        For Each dalle As Plancher In PlanCont
            With dalle
                rtf.WriteStartGroup()
                rtf.FontSize = 12 : rtf.FontSize = 10 : rtf.Underline = True
                rtf.WriteString("- Travée " & (PlanCont.IndexOf(dalle) + 1).ToString & ": ") : rtf.FontSize = 9 : rtf.Underline = False : rtf.NewLine()
                Select Case .Prefa_ReposLisse
                    Case 1 'avec lisse de rive prés des appuis
                        rtf.WriteString(" Sans objet car mise en place d'une lisse de rive. ") : rtf.NewLine()
                    Case 2 'Sans lisse de rive mais en accord avec le tableau A3
                        If PlanCont.IndexOf(dalle) = 0 And .LappG < 0.05 Then
                            rtf.WriteString(" Voir appui gauche travée suivante.") : rtf.NewLine()
                        ElseIf PlanCont.IndexOf(dalle) = PlanCont.Count - 1 And .LappD < 0.05 Then
                            rtf.WriteString(" Voir appui droit travée précédente.") : rtf.NewLine()
                        Else
                            If .Prefa_etai = True Then
                                If .AppuiG = 1 Then .Prefa_ReposG = 0.03 Else .Prefa_ReposG = 0.045 '= maçonnerie
                                If .AppuiD = 1 Then .Prefa_ReposD = 0.03 Else .Prefa_ReposD = 0.045
                            Else
                                If .AppuiG = 1 Then .Prefa_ReposG = 0.045 Else .Prefa_ReposG = 0.055 '= maçonnerie
                                If .AppuiD = 1 Then .Prefa_ReposD = 0.045 Else .Prefa_ReposD = 0.055
                            End If
                        End If
                    Case 3 'Sans lisse de rive et avec valeur de repos nominal
                        If PlanCont.IndexOf(dalle) = 0 And .LappG < 0.05 Then
                            rtf.WriteString(" Voir appui gauche travée suivante.") : rtf.NewLine()
                        ElseIf PlanCont.IndexOf(dalle) = PlanCont.Count - 1 And .LappD < 0.05 Then
                            rtf.WriteString(" Voir appui droit travée précédente.") : rtf.NewLine()
                        Else
                            If .Prefa_etai = True Then
                                If .AppuiG = 1 Then 'appui béton
                                    If .Prefa_ReposG < 0.03 Then
                                        rtf.WriteString(" Repos nominal appui gauche inférieur au repos nominal autorisé (= 3cm) ") : rtf.NewLine()
                                    End If
                                Else
                                    If .Prefa_ReposG < 0.045 Then
                                        rtf.WriteString(" Repos nominal appui gauche inférieur au repos nominal autorisé (= 4,5cm) ") : rtf.NewLine() '.Prefa_ReposG = 0.045 '= maçonnerie
                                    End If
                                End If
                                If .AppuiD = 1 Then 'appui béton
                                    If .Prefa_ReposD < 0.03 Then
                                        rtf.WriteString(" Repos nominal appui droit inférieur au repos nominal autorisé (= 3cm) ") : rtf.NewLine()
                                    End If
                                Else
                                    If .Prefa_ReposD < 0.045 Then
                                        rtf.WriteString(" Repos nominal appui droit inférieur au repos nominal autorisé (= 4,5cm) ") : rtf.NewLine() '.Prefa_ReposG = 0.045 '= maçonnerie
                                    End If
                                End If
                            Else
                                If .AppuiG = 1 Then 'appui béton
                                    If .Prefa_ReposG < 0.045 Then
                                        rtf.WriteString(" Repos nominal appui gauche inférieur au repos nominal autorisé (= 4,5cm) ") : rtf.NewLine()
                                    End If
                                Else
                                    If .Prefa_ReposG < 0.055 Then
                                        rtf.WriteString(" Repos nominal appui gauche inférieur au repos nominal autorisé (= 5,5cm) ") : rtf.NewLine() '.Prefa_ReposG = 0.045 '= maçonnerie
                                    End If
                                End If
                                If .AppuiD = 1 Then 'appui béton
                                    If .Prefa_ReposD < 0.045 Then
                                        rtf.WriteString(" Repos nominal appui droit inférieur au repos nominal autorisé (= 4,5cm) ") : rtf.NewLine()
                                    End If
                                Else
                                    If .Prefa_ReposD < 0.055 Then
                                        rtf.WriteString(" Repos nominal appui droit inférieur au repos nominal autorisé (= 5,5cm) ") : rtf.NewLine() '.Prefa_ReposG = 0.045 '= maçonnerie
                                    End If
                                End If
                            End If
                        End If
                End Select
                rtf.WriteEndGroup()
            End With
        Next
    End Sub

    ''' <summary>Vérification de l'étaiement </summary>
    '''  <param name="rtf">variable renvoyant le fichier de note de calcul sous format rtf</param>
    Private Sub Verification_etaiement(ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim A1, d, L_etaie As Double
        Dim i_file As Integer
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
        For Each dalle As Plancher In PlanCont
            With dalle
                If PlanCont.IndexOf(dalle) = 0 And .LappG < 0.05 Then

                ElseIf PlanCont.IndexOf(dalle) = PlanCont.Count - 1 And .LappD < 0.05 Then

                Else
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            A1 = aci.As_long(.TS_trav.Item(0).indTS)
                            If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                        Case 2 'barres HA
                            If .LitInf.Count = 2 Then A1 = (.LitInf.Item(0).Aire + .LitInf.Item(1).Aire) / .Ly Else A1 = .LitInf.Item(0).Aire / .Ly
                    End Select
                    d = .dInf
                End If
                If .Prefa_etai = True Then
                    i_file = 1 : L_etaie = 10 'initialisation des données
                    Do While L_etaie > .Prefa_etai_L
                        L_etaie = .Lx / (i_file + 1)
                        i_file = i_file + 1
                    Loop
                    Etaie_Critere_Securite(PlanCont.IndexOf(dalle), L_etaie, i_file - 1, A1, d, rtf)
                Else
                    Etaie_Critere_Securite(PlanCont.IndexOf(dalle), .Lx, 0, A1, d, rtf)
                End If
            End With

        Next
    End Sub
    ''' <summary>calcul de l'étaiement</summary>
    ''' <param name="i1">n° de travée - 1</param>
    ''' <param name="L">distance entre étaie calculée</param>
    '''  <param name="i_file">nbre de files d'étaies</param>
    '''  <param name="rtf">variable renvoyant le fichier de note de calcul sous format rtf</param>
    Private Sub Etaie_Critere_Securite(ByVal i1 As Integer, ByVal L As Double, ByVal i_file As Integer, ByVal A1 As Double, ByVal d As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim Cinf, di, G1, Qc0, Qs, Mu, Mrd As Double
        With PlanCont.Item(i1)
            G1 = .Prefa_h * 0.025
            Qc0 = (.H - .Prefa_h) * 0.024 + 0.007 * 0.024
            Qs = 0.0015 '(150 + 0.6 * 50) / 100000
            Mu = (1.35 * G1 + 1.35 * Qc0 + 1.5 * Qs) * L ^ 2 / 8
            di = d - (.H - .Prefa_h) 'hauteur utile pour la prédalle
            Cinf = .Prefa_h - (di - 1.2 * Sqrt(0.001 ^ 2 + 0.005 ^ 2))
            'calcul du moment résistant positif
            Call Moment_Resistant(1, .Prefa_h, Cinf, A1, Mrd,,, i1 + 1, 3)
            Tab_Verification_etaiement(i1, G1, Qc0, Qs, L, i_file, Mu, Mrd, rtf)
        End With
    End Sub
    ''' <summary>Affichage du calcul de vérification de l'étaiement </summary>
    ''' <param name="i1">n° de travée - 1</param>
    ''' <param name="L">distance entre étaie calculée</param>
    '''  <param name="i_file">nbre de files d'étaies</param>
    '''  <param name="rtf">variable renvoyant le fichier de note de calcul sous format rtf</param>
    Private Sub Tab_Verification_etaiement(ByVal i1 As Integer, ByVal G1 As Double, ByVal Qc0 As Double, ByVal Qs As Double, ByVal L As Double, ByVal i_file As Integer, ByVal Mu As Double, ByVal Mrd As Double,
                                           ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim msg As String
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        If i1 = 0 Then
            'données générales entrées une fois pour toutes
            'Ligne 2
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("G1") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("daN/m²") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((G1 * 100000.0).ToString("#,##0.0 ")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("PP predalle+ PP dalle/predalle + CP avant mise en oeuvre") : rtf.EndCell() : rtf.EndRow()
            'Ligne 2
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Qc0") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("daN/m²") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Qc0 * 100000.0).ToString("#,##0.0 ")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("PP béton coulé sur predalle") : rtf.EndCell() : rtf.EndRow()
            'Ligne 2
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Qs") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("daN/m²") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Qs * 100000.0).ToString("#,##0.0 ")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Charge chantier") : rtf.EndCell() : rtf.EndRow()
        End If
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString("Travée n°" & (i1 + 1).ToString("#")) : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("L étaie max") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(PlanCont(i1).Prefa_etai_L.ToString(" #0.00 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Distance maximale entre étaie demandée") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Nb file étaie") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("U") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(i_file.ToString(" #0  ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Nombre de file d'étaie (hors étaie de rive)") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("L étaie") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(L.ToString(" #0.00 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Distance calculée entre étaie") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("MEd,prov") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000.0).ToString("#,##0.0 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Moment fléchissant en phase provisoire") : rtf.EndCell() : rtf.EndRow()
        'Ligne 21
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mrd") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mrd * 1000).ToString("#0.0")) : rtf.EndCell()

        If Mu <= Mrd Then msg = "MEd < MRd -> OK." Else msg = " MEd > MRd -> NS." : Aff.msg_Avertiss.Add("Vérification étaiement - travée n°" & (i1 + 1).ToString & ": Med>Mrd")
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
    End Sub
    Private Sub Verification_integrite(ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim A1, d, L_etaie As Double
        Dim i_file As Integer
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
        For Each dalle As Plancher In PlanCont
            With dalle
                If PlanCont.IndexOf(dalle) = 0 And .LappG < 0.05 Then

                ElseIf PlanCont.IndexOf(dalle) = PlanCont.Count - 1 And .LappD < 0.05 Then

                Else
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            A1 = aci.As_long(.TS_trav.Item(0).indTS)
                            If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                        Case 2 'barres HA
                            If .LitInf.Count = 2 Then A1 = (.LitInf.Item(0).Aire + .LitInf.Item(1).Aire) / .Ly Else A1 = .LitInf.Item(0).Aire / .Ly
                    End Select
                    d = .dInf
                End If
                If .Prefa_etai = True Then
                    'même procédure que pour le calcul du critère de sécurité
                    i_file = 1 : L_etaie = 10 'initialisation des données  
                    Do While L_etaie > .Prefa_etai_L
                        L_etaie = .Lx / (i_file + 1)
                        i_file = i_file + 1
                    Loop
                    Etaie_Critere_integrite(PlanCont.IndexOf(dalle), L_etaie, A1, d, rtf)
                Else
                    Etaie_Critere_integrite(PlanCont.IndexOf(dalle), .Lx, A1, d, rtf)
                End If
            End With

        Next
    End Sub
    Private Sub Etaie_Critere_integrite(ByVal i1 As Integer, ByVal L As Double, ByVal A1 As Double, ByVal d As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim di, Izz, G1, Qc0, Qs, Ms, Y1, σt As Double
        With PlanCont.Item(i1)
            G1 = .Prefa_h * 0.025
            Qc0 = (.H - .Prefa_h) * 0.024 + 0.007 * 0.024
            Qs = 0.0015 '(150 + 0.6 * 50) / 100000
            Ms = (G1 + Qc0 + Qs) * L ^ 2 / 8
            di = d - (.H - .Prefa_h) 'hauteur utile pour la prédalle
            'calcul Inertie Section homogénéisée
            Y1 = 1 * .Prefa_h ^ 2 / 2 + 15 * A1 * di
            Y1 = Y1 / (1 * .Prefa_h + 15 * A1)
            Izz = 1 * .Prefa_h ^ 3 / 12 + 1 * .Prefa_h * (.Prefa_h / 2 - Y1) ^ 2 + 15 * (A1 * (di - Y1) ^ 2) 'Inertie Section homogénéisée
            σt = Ms / (Izz / (.Prefa_h - Y1))
            Tab_Verification_integrite(i1, Ms, σt, rtf)
        End With
    End Sub
    Private Sub Tab_Verification_integrite(ByVal i1 As Integer, ByVal Ms As Double, ByVal σt As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim msg As String
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString("Travée n°" & (i1 + 1).ToString("#")) : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString("Ms =M[G1 + Qc0 + Qs] - Voir valeurs sur tableau précédent.") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ms") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Ms * 1000.0).ToString("#,##0.0 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Moment fléchissant ELS en phase provisoire") : rtf.EndCell() : rtf.EndRow()
        'Ligne 21
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("σt") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(σt.ToString("#0.00")) : rtf.EndCell()
        If σt <= 3.6 Then msg = "σt < 3,6 MPa -> OK." Else msg = " σt > 3,6 MPa -> -> NS." : Aff.msg_Avertiss.Add("Vérification intégrite - travée n°" & i1.ToString & ": σt > 3,6 MPa")
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
    End Sub
    Private Sub Verification_Deformation(ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim A1, d, L_etaie As Double
        Dim i_file As Integer
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
        For Each dalle As Plancher In PlanCont
            With dalle
                If PlanCont.IndexOf(dalle) = 0 And .LappG < 0.05 Then

                ElseIf PlanCont.IndexOf(dalle) = PlanCont.Count - 1 And .LappD < 0.05 Then

                Else
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            A1 = aci.As_long(.TS_trav.Item(0).indTS)
                            If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                        Case 2 'barres HA
                            If .LitInf.Count = 2 Then A1 = (.LitInf.Item(0).Aire + .LitInf.Item(1).Aire) / .Ly Else A1 = .LitInf.Item(0).Aire / .Ly
                    End Select
                    d = .dInf
                End If
                If .Prefa_etai = True Then
                    'même procédure que pour le calcul du critère de sécurité
                    i_file = 1 : L_etaie = 10 'initialisation des données 
                    Do While L_etaie > .Prefa_etai_L
                        L_etaie = .Lx / (i_file + 1)
                        i_file = i_file + 1
                    Loop
                    Etaie_Critere_deformation(PlanCont.IndexOf(dalle), L_etaie, A1, d, rtf)
                Else
                    Etaie_Critere_deformation(PlanCont.IndexOf(dalle), .Lx, A1, d, rtf)
                End If
            End With

        Next
    End Sub
    Private Sub Etaie_Critere_deformation(ByVal i1 As Integer, ByVal L As Double, ByVal A1 As Double, ByVal d As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim G1, Qc0, Eb, Fl, Izz, y1, di As Double
        With PlanCont.Item(i1)
            G1 = .Prefa_h * 0.025
            Qc0 = (.H - .Prefa_h) * 0.024 + 0.007 * 0.024
            Eb = bet.Ecm
            di = d - (.H - .Prefa_h) 'hauteur utile pour la prédalle
            'calcul Inertie Section homogénéisée
            y1 = 1 * .Prefa_h ^ 2 / 2 + 15 * A1 * di
            y1 = y1 / (1 * .Prefa_h + 15 * A1)
            Izz = 1 * .Prefa_h ^ 3 / 12 + 1 * .Prefa_h * (.Prefa_h / 2 - y1) ^ 2 + 15 * (A1 * (di - y1) ^ 2) 'Inertie Section homogénéisée
            Fl = 5 * (G1 + Qc0) * L ^ 4 / (384 * Eb * Izz)
            Tab_Verification_deformation(i1, Fl, rtf)
        End With
    End Sub
    Private Sub Tab_Verification_deformation(ByVal i1 As Integer, ByVal fl As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim msg As String
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString("Travée n°" & (i1 + 1).ToString("#")) : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("fl") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("mm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((fl * 1000.0).ToString("#,##0.0 ")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Flèche de la prédalle après coulage du béton en oeuvre") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        If fl <= 0.01 Then msg = "fl < 10 mm -> OK." Else msg = " fl > 10 mm -> -> NS." : Aff.msg_Avertiss.Add("Vérification déformation - travée n°" & i1.ToString & ": fl > 10 mm")
        rtf.BeginRow() : rtf.DeclareCell(8700) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()
    End Sub
    Private Sub Manutention_boucle(ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim G, Levage, Levagecor As Double 'rectif, angle
        Dim msg As String
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewParagraph()
        For Each Dalle As Plancher In PlanCont
            With Dalle
                G = .Prefa_Lx.Item(0) * .Prefa_Ly.Item(0) * .Prefa_h * 0.025
                Levage = 1.3 * G / 3
                Levagecor = Levage * 1.41
                rtf.WriteString("Travée n°" & (PlanCont.IndexOf(Dalle) + 1).ToString("#") & " - prédalles n°" & ((PlanCont.IndexOf(Dalle) + 1) * 10 + 1).ToString("#") & " à n°" &
                                ((PlanCont.IndexOf(Dalle) + 1) * 10 + .Prefa_nb).ToString("#") & " :") : rtf.NewLine()
                rtf.WriteString(" - Angle d'élingage : compris entre 60° et 90° soit un coefficient correctif de 1,41 (cas le plus défavorable).") : rtf.NewLine()
                rtf.WriteString(" - Poids de la prédalle la plus lourde :" & (G * 1000).ToString("#") & " KN soit " & (Levage * 1000).ToString("#0.00") & " kN par crochet (coef dynamique 1,5 intégré).") : rtf.NewLine()
                rtf.WriteString(" - effort maximal sur boucle :" & (Levagecor * 1000).ToString("#0.00") & " kN.") : rtf.NewLine()
                msg = "Sélection suivant figure 6 section C §1.2 du CPT : "
                Select Case Levagecor * 1000
                    Case Is <= 15
                        msg = msg & " diamètre de boucle 10 mm autorisé"
                    Case Is <= 22
                        msg = msg & " diamètre de boucle 12 mm autorisé"
                    Case Is <= 30
                        msg = msg & " diamètre de boucle 14 mm autorisé"
                    Case Is <= 40
                        msg = msg & " diamètre de boucle 16 mm autorisé"
                    Case Else
                        msg = msg & " 4 boucles insuffisantes - Passer à 6 boucles."
                        Aff.msg_Avertiss.Add("Manutention boucle - travée n°" & (PlanCont.IndexOf(Dalle) + 1).ToString("#") & ": 4 boucles insuffisantes - Passer à 6 boucles.")
                End Select
                rtf.WriteString(msg) : rtf.NewLine() : rtf.NewLine()
            End With
        Next
        rtf.WriteEndGroup()
    End Sub
#End Region

#Region "Verification Trémie"
    Private Sub Calcul_Tremie_Predalle(ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim i As Integer ', type_chevetre 
        Dim As_coupe_X, As_coupe_Y, Tot, totTr, dM, b_R_Lx As Double
        Dim Chevauchant, Rive_Predal, Bout_Predal, suspente As Boolean
        Dim Eloignement_Rive As Boolean = False '
        Dim msg As String = String.Empty
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        For Each Dalle As Plancher In PlanCont
            With Dalle
                For i = 1 To ._Tremie.Count
                    'Etablissement des données générales de la trémie
                    Chevauchant = False : Rive_Predal = False : Bout_Predal = False : suspente = False ': type_chevetre = 2 'aciers de répartition dans prédalle
                    'calcul aciers coupés
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            'en travée 
                            Tot = aci.As_long(.TS_trav.Item(0).indTS)
                            totTr = aci.As_trans(.TS_trav.Item(0).indTS)
                            If .TS_trav.Count = 2 Then
                                Tot = Tot + aci.As_long(.TS_trav.Item(1).indTS)
                                totTr = totTr + aci.As_trans(.TS_trav.Item(1).indTS)
                            End If
                        Case 2 'HA
                            Tot = (.LitInf.Item(0).Aire / .Ly) '  cm²/ml. 
                            totTr = (.LitInf_Y.Item(0).Aire / .Lx) ' " cm²/ml. 
                    End Select
                    As_coupe_X = Tot * ._Tremie.Item(i - 1)._Ly : As_coupe_Y = totTr * ._Tremie.Item(i - 1)._Lx 'calcul de la section d'acier intercepté par la trémie
                    'calcul éloignement rives non porteuse pour application de l'annexe J
                    If ._Tremie.Item(i - 1)._Ly > 0.6 And ._Tremie.Item(i - 1)._Ly <= .Lcx / 3 And ._Tremie.Item(i - 1)._Lx <= .Lcx / 3 Then Call Verification_eloignement_Rive((PlanCont.IndexOf(Dalle) + 1), i, Eloignement_Rive)
                    rtf.WriteString("Travée n°" & (PlanCont.IndexOf(Dalle) + 1).ToString("#") & ":") : rtf.NewLine()
                    '
                    '--> calcul du renforcement pour trémie de dimension < 30x30 cm
                    If ._Tremie.Item(i - 1)._Lx <= 0.3 And ._Tremie.Item(i - 1)._Ly <= 0.3 Then
                        rtf.WriteString(" - trémie n°" & i.ToString & ": pas de renforts nécessaires. Par construction, 1 HA12 pour chaque coté.") : rtf.NewLine()
                        Call Renforcement_Tremie_barresHA12(i, PlanCont.IndexOf(Dalle) + 1)

                        '
                        '--> calcul du renforcement pour trémie dont une dimension > 30 cm et largeur < 60 cm et longueur < Lx/3
                    ElseIf ._Tremie.Item(i - 1)._Ly <= 0.6 And ._Tremie.Item(i - 1)._Lx <= .Lcx / 3 Then
                        If (._Tremie.Item(i - 1)._Ly / .Lcx) > 0.3 Then
                            'cela signifie que .lcx = 1,80m - Pas d'étude de trémie pour ce cas de figure
                            rtf.WriteString(" - trémie n°" & i.ToString & ": largeur > 1/3 de la portée et portée < 1,80m. Disposition non implémenté. Renforts non calculés.") : rtf.NewLine()
                        Else
                            Verification_Positionnement_Predalle(Dalle, i, Rive_Predal, Chevauchant, Bout_Predal)
                            If Chevauchant = False And Rive_Predal = False And Bout_Predal = False And ._Tremie.Item(i - 1)._Cloison = False Then
                                If ._Tremie.Item(i - 1)._Ly <= ._Tremie.Item(i - 1)._Lx Then
                                    dM = 1 'Pas de majoration du moment - voir annexe J2 - cas 1
                                    b_R_Lx = ._Tremie.Item(i - 1)._Ly / 2
                                Else
                                    dM = 1.1 ' majoration du moment - voir annexe J2 - cas 2
                                    b_R_Lx = ._Tremie.Item(i - 1)._Ly / 3
                                End If
                                Select Case ._Tremie.Item(i - 1)._type_chevetre
                                    Case 1
                                        msg = "armatures disposées sur prédalle"
                                    Case 2
                                        msg = "armatures disposées dans la prédalle"
                                End Select
                                rtf.WriteString(" - trémie n°" & i.ToString & ": renforcement nécessaire par complément acier, " & msg & " et aucun chevêtre créé.") : rtf.NewLine()
                                Call Renforcement_Tremie_ComplementbarresHA(i, PlanCont.IndexOf(Dalle) + 1, As_coupe_X, As_coupe_Y, dM, b_R_Lx, rtf) 'calcul des aciers de renforts - sans chevêtre

                            Else
                                ' = Mise en place d'un chevêtre
                                If Bout_Predal = True Then
                                    If Chevauchant = False And Rive_Predal = False Then
                                        rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en bout de prédalle soit renforts + chevètre type 2.") : rtf.NewLine()
                                        If ._Tremie.Item(i - 1)._type_chevetre <> 1 Then ._Tremie.Item(i - 1)._type_chevetre = 2 'vérification absence de forçage
                                    Else
                                        If Chevauchant = True Then
                                            rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en bout de prédalle et chevauchant 2 prédalles soit renforts + chevètre type 1.") : rtf.NewLine()
                                        Else 'Chevauchant = False   Rive_Predal = true
                                            rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en bout de prédalle et Rive_Predal soit renforts + chevètre type 1.") : rtf.NewLine()
                                        End If
                                        ._Tremie.Item(i - 1)._type_chevetre = 1
                                    End If
                                    If ._Tremie.Item(i - 1)._Ly >= ._Tremie.Item(i - 1)._Lx And ._Tremie.Item(i - 1)._Ly <= .Lcx / 4 Then
                                        ' annexe J3 - cas 1
                                        b_R_Lx = ._Tremie.Item(i - 1)._Ly / 2
                                    Else
                                        ' annexe J3 - cas 2
                                        b_R_Lx = ._Tremie.Item(i - 1)._Ly / 3
                                    End If
                                ElseIf Chevauchant = True Then
                                    rtf.WriteString(" - trémie n°" & i.ToString & ": trémie chevauchant 2 prédalles soit renforts + chevètre type 1.") : rtf.NewLine()
                                    ._Tremie.Item(i - 1)._type_chevetre = 1
                                    b_R_Lx = 0.3
                                ElseIf Rive_Predal = True Then
                                    rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en rive d'une autre prédalle soit renforts + chevètre type 1.") : rtf.NewLine()
                                    ._Tremie.Item(i - 1)._type_chevetre = 1
                                    b_R_Lx = 0.3
                                End If

                                Call Renforcement_Tremie_Chevetre_et_barresHA(i, PlanCont.IndexOf(Dalle) + 1, As_coupe_X, As_coupe_Y, 1, b_R_Lx, rtf) 'calcul des aciers de renforts et du chevêtre
                            End If
                        End If

                        '
                        '--> calcul du renforcement pour trémie dont _Ly > 60 cm  et < Lx/3
                        'Application toujours possible de l'annexe J
                    ElseIf ._Tremie.Item(i - 1)._Ly <= .Lcx / 3 And ._Tremie.Item(i - 1)._Lx <= .Lcx / 3 And Eloignement_Rive = False Then
                        Verification_Positionnement_Predalle(Dalle, i, Rive_Predal, Chevauchant, Bout_Predal)
                        If Bout_Predal = False Then
                            'calcul suivant annexe J2
                            If ._Tremie.Item(i - 1)._Ly <= ._Tremie.Item(i - 1)._Lx Then
                                dM = 1 'Pas de majoration du moment - voir annexe J2 - cas 1
                                b_R_Lx = ._Tremie.Item(i - 1)._Ly / 2
                            Else
                                dM = 1.1 ' majoration du moment - voir annexe J2 - cas 2
                                b_R_Lx = ._Tremie.Item(i - 1)._Ly / 3
                            End If
                            rtf.WriteString(" - trémie n°" & i.ToString & ": renforcement nécessaire par renforts barres HA et chevêtre.") : rtf.NewLine()

                        Else
                            ' = Mise en place d'un chevêtre
                            If Bout_Predal = True Then
                                If Chevauchant = False And Rive_Predal = False Then
                                    rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en bout de prédalle soit renforts + chevètre type 2.") : rtf.NewLine()
                                    If ._Tremie.Item(i - 1)._type_chevetre <> 1 Then ._Tremie.Item(i - 1)._type_chevetre = 2 'vérification absence de forçage
                                Else
                                    If Chevauchant = True Then
                                        rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en bout de prédalle et chevauchant 2 prédalles soit renforts + chevètre type 1.") : rtf.NewLine()
                                    Else 'Chevauchant = False   Rive_Predal = true
                                        rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en bout de prédalle et Rive_Predal soit renforts + chevètre type 1.") : rtf.NewLine()
                                    End If
                                    ._Tremie.Item(i - 1)._type_chevetre = 1
                                End If
                                If ._Tremie.Item(i - 1)._Ly >= ._Tremie.Item(i - 1)._Lx And ._Tremie.Item(i - 1)._Ly <= .Lcx / 4 Then
                                    ' annexe J3 - cas 1
                                    b_R_Lx = ._Tremie.Item(i - 1)._Ly / 2
                                Else
                                    ' annexe J3 - cas 2
                                    b_R_Lx = ._Tremie.Item(i - 1)._Ly / 3
                                End If

                            ElseIf Chevauchant = True Then
                                rtf.WriteString(" - trémie n°" & i.ToString & ": trémie chevauchant 2 prédalles soit renforts + chevètre type 1.") : rtf.NewLine()
                                ._Tremie.Item(i - 1)._type_chevetre = 1
                                b_R_Lx = 0.3
                            ElseIf Rive_Predal = True Then
                                rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en rive d'une autre prédalle soit renforts + chevètre type 1.") : rtf.NewLine()
                                ._Tremie.Item(i - 1)._type_chevetre = 1
                                b_R_Lx = 0.3
                            End If
                            dM = 1
                        End If
                        If b_R_Lx > 0.3 Then b_R_Lx = 0.3
                        Call Renforcement_Tremie_Chevetre_et_barresHA(i, PlanCont.IndexOf(Dalle) + 1, As_coupe_X, As_coupe_Y, dM, b_R_Lx, rtf) 'calcul des aciers de renforts et du chevêtre

                        '
                        '--> calcul du renforcement pour trémie par chevêtre et bandes noyées:
                        '   - au moins une dimension > Lx/3
                        '   - et/ou environnement avec de multiples trémies faisant damiers, cloison lourde en périphérie, éloignement des rives non porteuses insuffisant, ...
                    Else
                        rtf.WriteString(" - trémie n°" & i.ToString & ": en raison des dimensions de la trémie par rapport à la portée, ou de charges lourdes en bordure de trémie ou de sa proximité 
                                        avec une Rive non porteuse, renforts par bandes noyées (Non implémentés dans ce logiciel).") : rtf.NewLine()
                        Call Renforcement_Tremie_Chevetre_BandeNoyé(i, PlanCont.IndexOf(Dalle) + 1, As_coupe_X, As_coupe_Y, rtf)

                    End If

                Next
            End With
        Next
        rtf.WriteEndGroup()
    End Sub
    ''' <summary>
    ''' Vérification que la trémie est suffisament éloigné de la rive non porteur pour appliquer les prescriptions de l'annexe J
    ''' </summary>
    ''' <param name="i_tremie">index de la tremie  </param>
    ''' <param name="j1">n° de la travée  </param>
    ''' <param name="Eloignement_Rive">si trémie à proximité d'une rive Non porteur, Eloignement_Rive = True  </param>
    Private Sub Verification_eloignement_Rive(ByVal j1 As Integer, ByVal i_tremie As Integer, ByRef Eloignement_Rive As Boolean)
        With PlanCont.Item(j1 - 1)
            Eloignement_Rive = False
            'vérification si trémie à proximité de rive Non porteur
            If ._Tremie.Item(i_tremie - 1)._yA < 2 * ._Tremie.Item(i_tremie - 1)._Lx Or 'verif rive haute - Largeur 2b = surface d'influence
                ._Tremie.Item(i_tremie - 1)._yA + ._Tremie.Item(i_tremie - 1)._Ly > .Ly - (2 * ._Tremie.Item(i_tremie - 1)._Lx) Then Eloignement_Rive = True 'verif rive basse

        End With
    End Sub
    Private Sub Verification_Positionnement_Predalle(ByVal dalle As Plancher, ByVal i_tremie As Integer, ByRef Rive As Boolean, ByRef Chevauchant As Boolean, ByRef Bout_Predal As Boolean)
        Dim nb_Predal, i, j As Integer
        Dim X, Y, Y1 As Double
        X = 0 : Y = 0
        With dalle
            'vérification si trémie à proximité appui porteur
            If ._Tremie.Item(i_tremie - 1)._xA < 0.3 Or ._Tremie.Item(i_tremie - 1)._xA + ._Tremie.Item(i_tremie - 1)._Lx > .Lx - 0.3 Then Bout_Predal = True
            'vérification Chevauchant
            Y1 = .Prefa_Ly.Item(0)
            nb_Predal = .Prefa_Ly.Count
            i = 0
            Do
                i = i + 1
                Y = Y + .Prefa_Ly.Item(i - 1) + .Prefa_esp
                If ._Tremie.Item(i_tremie - 1)._yA <= Y And ._Tremie.Item(i_tremie - 1)._yA + ._Tremie.Item(i_tremie - 1)._Ly > Y Then Chevauchant = True : Exit Sub
            Loop While i < nb_Predal - 1
            'vérification Rive
            i = 0
            Do
                i = i + 1
                Y = Y + .Prefa_Ly.Item(i - 1) + .Prefa_esp
                If ._Tremie.Item(i_tremie - 1)._yA <= Y Then Exit Do
            Loop While i < nb_Predal
            If ._Tremie.Item(i_tremie - 1)._yA >= Y - 0.3 Then Rive = True : Exit Sub
            If ._Tremie.Item(i_tremie - 1)._yA <= Y - .Prefa_Ly.Item(i - 1) + 0.3 Then Rive = True
        End With
    End Sub
    Private Sub Calcul_Tremie_DallePleine(ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim i As Integer
        Dim As_coupe_X, As_coupe_Y, Tot, totTr, charge, L, A_R1, A_R2, MuR1, MuR2, R_t1, R_t2, st_susp, Δm1, Δm2 As Double
        Dim fcd, fyd, µu, d, αu, zu, Ainf As Double
        Dim Chevauchant, Rive, Bout_dal, suspente As Boolean

        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        For Each Dalle As Plancher In PlanCont
            With Dalle
                For i = 1 To ._Tremie.Count
                    Chevauchant = False : Rive = False : Bout_dal = False : suspente = False ': type_chevetre = 2 'aciers de répartition dans prédalle
                    'calcul aciers coupés
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            'en travée 
                            Tot = aci.As_long(.TS_trav.Item(0).indTS)
                            totTr = aci.As_trans(.TS_trav.Item(0).indTS)
                            If .TS_trav.Count = 2 Then
                                Tot = Tot + aci.As_long(.TS_trav.Item(1).indTS)
                                totTr = totTr + aci.As_trans(.TS_trav.Item(1).indTS)
                            End If
                        Case 2 'HA
                            Tot = (.LitInf.Item(0).Aire / .Ly) '  cm²/ml. 
                            totTr = (.LitInf_Y.Item(0).Aire / .Lx) ' " cm²/ml. 
                    End Select
                    As_coupe_X = Tot * ._Tremie.Item(i - 1)._Ly : As_coupe_Y = totTr * ._Tremie.Item(i - 1)._Lx
                    rtf.WriteString("Travée n°" & (PlanCont.IndexOf(Dalle) + 1).ToString("#") & ":") : rtf.NewLine()
                    If ._Tremie.Item(i - 1)._Lx <= 0.3 And ._Tremie.Item(i - 1)._Ly <= 0.3 Then
                        rtf.WriteString(" - trémie n°" & i.ToString & ": pas de renforts nécessaires. Par construction, 1 HA12 pour chaque coté.") : rtf.NewLine()
                        Call Renforcement_Tremie_barresHA12(i, PlanCont.IndexOf(Dalle) + 1)
                    ElseIf ._Tremie.Item(i - 1)._Ly <= 0.6 Then
                        Verification_Positionnement_dalle(Dalle, i, Rive, Chevauchant, Bout_dal)
                        If Chevauchant = False And Rive = False And Bout_dal = False Then
                            suspente = False
                            rtf.WriteString(" - trémie n°" & i.ToString & ": renforcement nécessaire par complément acier sur prédalle.") : rtf.NewLine()
                        Else
                            suspente = True
                            If Bout_dal = True Then
                                If Chevauchant = False And Rive = False Then
                                    rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en bout de dalle soit renforts + chevètre.") : rtf.NewLine()
                                Else
                                    If Chevauchant = True Then
                                        rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en bout de prédalle et chevauchant 2 prédalles soit renforts + chevètre.") : rtf.NewLine()
                                    Else
                                        rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en bout de prédalle et rive soit renforts + chevètre.") : rtf.NewLine()
                                    End If
                                End If
                            ElseIf Chevauchant = True Then
                                rtf.WriteString(" - trémie n°" & i.ToString & ": trémie chevauchant 2 prédalles soit renforts + chevètre.") : rtf.NewLine()
                            ElseIf Rive = True Then
                                rtf.WriteString(" - trémie n°" & i.ToString & ": trémie en rive de prédalle soit renforts + chevètre.") : rtf.NewLine()
                            End If
                        End If
                    Else
                        rtf.WriteString(" - trémie n°" & i.ToString & ": renforts + chevètre.") : rtf.NewLine()
                        suspente = True
                    End If
                    charge = 0
                    For Each chge As Charge In PlanCont.Item(PlanCont.IndexOf(Dalle)).Chge
                        If chge.Nature = 1 And chge.schem = 1 Then
                            charge = charge + 1.35 * chge.P1
                        End If
                        If chge.Nature > 1 And chge.Nature < 8 And chge.schem = 1 Then
                            charge = charge + 1.5 * chge.P1
                        End If
                    Next
                    'chevêtre
                    L = Max(._Tremie(i - 1)._xA, .Lx - ._Tremie.Item(i - 1)._Lx - ._Tremie(i - 1)._xA)
                    charge = charge * L / 2 'charge linéaire reporté sur la trémie
                    MuR1 = charge * ._Tremie.Item(i - 1)._Ly ^ 2 / 8
                    fcd = bet.fck / bet.γcd : fyd = aci.fyk / aci.γsd
                    'aciers de répartition dans dalle
                    d = .H - 0.035
                    µu = MuR1 / (0.2 * d ^ 2 * fcd)
                    αu = (1 - (1 - 2 * µu) ^ 0.5) / 0.8
                    zu = d * (1 - 0.8 * αu / 2)
                    Ainf = MuR1 / (zu * fyd)
                    A_R1 = Max(As_coupe_Y / 2, Ainf)
                    rtf.WriteString("  . Chevêtre: aciers de répartition coupés = " & ((As_coupe_Y / 2) * 10000).ToString("#0.00") & "cm² et aciers calculés = " &
                                    (Ainf * 10000).ToString("#0.00") & "cm² soit aciers retenus = " & (A_R1 * 10000).ToString("#0.00") & "cm²") : rtf.NewLine()
                    If suspente = True Then
                        'calcul des aciers de suspente = HA6
                        st_susp = charge / (2 * PI * 0.006 ^ 2 / 4 * fyd) 'espacement entre 2 suspentes
                        If st_susp > 0.2 Then st_susp = 0.2
                        rtf.WriteString("  . Chevêtre: aciers de suspente: cadres HA6 espacés de  " & (st_susp * 100).ToString("#0") & "cm + 1 cadre dans la zone d'ancrage (voir schéma).") : rtf.NewLine()
                    End If
                    L = ._Tremie.Item(i - 1)._Lx + ._Tremie(i - 1)._xA
                    charge = charge * (.Lx - ._Tremie.Item(i - 1)._Lx - ._Tremie(i - 1)._xA) / 2 'charge linéaire reporté sur la trémie
                    R_t2 = charge * ._Tremie.Item(i - 1)._Ly / 2
                    Δm2 = R_t2 * L * ((.Lx - L) / .Lx)
                    R_t1 = charge * ._Tremie(i - 1)._xA / 2 * ._Tremie.Item(i - 1)._Ly / 2
                    Δm1 = R_t1 * ._Tremie(i - 1)._xA * ((.Lx - ._Tremie(i - 1)._xA) / .Lx)
                    MuR2 = Δm1 + Δm2 'valeur par excés
                    µu = MuR2 / (0.3 * d ^ 2 * fcd)
                    αu = (1 - (1 - 2 * µu) ^ 0.5) / 0.8
                    zu = d * (1 - 0.8 * αu / 2)
                    Ainf = MuR2 / (zu * fyd)
                    A_R2 = Max(As_coupe_X / 2, Ainf)
                    rtf.WriteString("  . Renfort: aciers longitudinaux coupés = " & ((As_coupe_X / 2) * 10000).ToString("#0.00") & "cm² et aciers calculés = " &
                                 (Ainf * 10000).ToString("#0.00") & "cm² soit aciers retenus = " & (A_R2 * 10000).ToString("#0.00") & "cm²") : rtf.NewLine()
                Next

            End With
        Next
        rtf.WriteEndGroup()
    End Sub
    Private Sub Verification_Positionnement_dalle(ByVal dalle As Plancher, ByVal i_tremie As Integer, ByRef Rive As Boolean, ByRef Chevauchant As Boolean, ByRef Bout_dal As Boolean)
        With dalle
            'vérification si trémie à proximité appui porteur
            If ._Tremie.Item(i_tremie - 1)._xA < 0.3 Or ._Tremie.Item(i_tremie - 1)._xA + ._Tremie.Item(i_tremie - 1)._Lx > .Lx - 0.3 Then Bout_dal = True
            'vérification Chevauchant
            If ._Tremie.Item(i_tremie - 1)._yA + ._Tremie.Item(i_tremie - 1)._Ly >= .Ly Then Chevauchant = True
            If ._Tremie.Item(i_tremie - 1)._xA + ._Tremie.Item(i_tremie - 1)._Lx >= .Lx Then Chevauchant = True
            'vérification Rive
            If ._Tremie.Item(i_tremie - 1)._yA + ._Tremie.Item(i_tremie - 1)._Ly >= .Ly - 0.3 Then Rive = True
            If ._Tremie.Item(i_tremie - 1)._yA <= 0.3 Then Rive = True
        End With
    End Sub
    ''' <summary>
    ''' calcul du renforcement de petite trémie (+ petit que 30cm)
    ''' </summary>
    ''' <param name="i">index de la tremie  </param>
    ''' <param name="j">n° de travée</param>
    Private Sub Renforcement_Tremie_barresHA12(ByVal i As Integer, ByVal j As Integer)
        Dim Tremie_HA As New Ferraillage_HA
        Tremie_HA.φHA = 0.012
        Dim η1, fctk, fbd, Lbrqd, L0, α1, α2, α3 As Double
        η1 = 0.7
        fctk = bet.fctk05
        If bet.fctk05 > 3.1 Then fctk = 3.1
        fbd = 2.25 * η1 * 1 * fctk / bet.γcd
        Lbrqd = (0.012 / 4) * (0.9 * aci.fyk / fbd)
        α1 = 1 : α2 = 0.7 : α3 = 1
        L0 = α1 * α2 * α3 * Lbrqd '(eq 8.4)
        L0 = Truncate(Max(L0, Max(15 * 0.012, 0.1)) * 100) / 100
        With PlanCont.Item(j - 1)
            Tremie_HA.L13acier = L0
            Tremie_HA.Lacier1 = Truncate((2 * L0 + 0.02 + ._Tremie.Item(i - 1)._Lx) * 100) / 100
            Tremie_HA.nbar = 1
            ._Tremie.Item(i - 1)._HA_Lx = Tremie_HA
            Tremie_HA.Lacier1 = Truncate((2 * L0 + 0.02 + ._Tremie.Item(i - 1)._Ly) * 100) / 100
            ._Tremie.Item(i - 1)._HA_Ly = Tremie_HA
        End With
    End Sub
    ''' <summary>
    ''' calcul du renforcement de petite trémie - Uniquement renforts barres HA en périphérie de la trémie
    ''' </summary>
    ''' <param name="i">n° de la tremie  </param>
    ''' <param name="j">n° de travée</param>
    ''' <param name="As_coupe_X">total aire armatures principales coupées par la trémie  </param>
    ''' <param name="As_coupe_Y">total aire armatures de  répartition coupées par la trémie </param>
    ''' <param name="dM">Majoration du Moment suivant annexe J2 </param>
    ''' <param name="b_R_Lx">dimension de la bande latérale à la trémie, dans le sens de la portée, sur laquelle sera dimensionnée les aciers de renfort</param>
    Private Sub Renforcement_Tremie_ComplementbarresHA(ByVal i As Integer, ByVal j As Integer, ByVal As_coupe_X As Double, ByVal As_coupe_Y As Double, ByVal dM As Double, ByVal b_R_Lx As Double,
                                                       ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim A_R1, A_R2, Ainf, ama, charge, chargeR, d, dm1, dm2, fcd, fyd, L, MuR1, MuR2, R_t1, R_t2, u1 As Double ',   st_susp
        Dim µu, Δm1, Δm2, αu, zu As Double
        Dim i1, i2 As Integer
        Dim η1, fctk, fbd, Lbrqd, L0, α1, α2, α3 As Double
        '
        '--> Détermination des armatures perpendiculaires au sens de portée (= aciers // aux aciers de répartition)
        With PlanCont.Item(j - 1)
            'calcul des charges appliquées sur la dalle
            chargeR = 0
            For Each chge As Charge In .Chge
                If chge.Nature = 1 And chge.schem = 1 Then
                    chargeR = chargeR + 1.35 * chge.P1
                End If
                If chge.Nature > 1 And chge.Nature < 8 And chge.schem = 1 Then
                    chargeR = chargeR + 1.5 * chge.P1
                End If
            Next
            'Renfort // au sens non porteur - Sens Y
            L = Max(._Tremie(i - 1)._xA, .Lx - ._Tremie.Item(i - 1)._Lx - ._Tremie(i - 1)._xA) 'détermination de la longueur reprenant le plus de charge
            charge = chargeR * L / 2 'charge linéaire Maximale reporté sur la trémie côté perpendiculaire à la portée
            MuR1 = charge * ._Tremie.Item(i - 1)._Ly ^ 2 / 8
            fcd = bet.fck / bet.γcd : fyd = aci.fyk / aci.γsd
            If ._Tremie.Item(i - 1)._type_chevetre = 1 Then
                ''aciers de répartition SUR prédalle
                d = .H - .Prefa_h - 0.005
            Else
                'de base sauf indication contraire dans l'entrée des données, aciers de répartition dans prédalle
                d = .H - 0.035
            End If
            ._Tremie.Item(i - 1)._b_Ly = 0.2
            µu = MuR1 / (0.2 * d ^ 2 * fcd)
            αu = (1 - (1 - 2 * µu) ^ 0.5) / 0.8
            zu = d * (1 - 0.8 * αu / 2)
            Ainf = MuR1 / (zu * fyd) 'section d'acier placé le long du coté Lx pour reprendre la charge apportée
            A_R1 = Max(As_coupe_Y / 2, Ainf) 'comparaison avec les aciers coupés
            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.WriteString("   ► Renfort sur coté // au sens NON porteur: aciers de répartition coupés = " & ((As_coupe_Y / 2) * 10000).ToString("#0.00") & "cm² et section calculée = " &
                            (Ainf * 10000).ToString("#0.00") & "cm² soit section retenue = " & (A_R1 * 10000).ToString("#0.00") & "cm² par côté.") : rtf.NewLine()
            'détermination des armatures placées contre la trémie dans le sens Ly
            'calcul sur les sections HA8 à HA14
            ama = 0.1   '1000 cm²
            'recherche de la combinaison des diamètres d'acier la plus proche de la section calculée
            'calcul pour la section x - calcul dans le sens porteur
            For i1 = 2 To 3
                i2 = 0
                Do
                    i2 = i2 + 1
                    If aci.Ha_Lg(i2) = True Then u1 = i1 * PI * aci.Ø_Norm(i2) ^ 2 / 4
                    If i2 + 1 = 5 Then Exit Do 'Limitation à HA14
                Loop While u1 < A_R1
                If u1 < ama Then
                    ama = u1
                    'affectation
                    With ._Tremie.Item(i - 1)._HA_Ly
                        .φHA = aci.Ø_Norm.Item(i2) : .nbar = i1
                    End With
                End If
            Next
            'calcul de l'ancrage de l'acier
            With ._Tremie.Item(i - 1)._HA_Ly
                η1 = 0.7
                fctk = bet.fctk05
                If bet.fctk05 > 3.1 Then fctk = 3.1
                fbd = 2.25 * η1 * 1 * fctk / bet.γcd
                Lbrqd = (.φHA / 4) * (0.9 * aci.fyk / fbd)
                α1 = 1 : α2 = 0.7 : α3 = 1
                L0 = α1 * α2 * α3 * Lbrqd '(eq 8.4)
                L0 = Truncate(Max(L0, Max(15 * 0.012, 0.1)) * 100) / 100
                If L0 > b_R_Lx Then .L13acier = L0 Else .L13acier = b_R_Lx
                PlanCont.Item(j - 1)._Tremie.Item(i - 1)._b_Ly = L0
                .Lacier1 = Truncate((2 * L0 + 0.02 + PlanCont.Item(j - 1)._Tremie.Item(i - 1)._Ly) * 100) / 100
                rtf.WriteString("     Soit " & .nbar.ToString("#0") & "HA" & (.φHA * 1000).ToString("#") & "x" & .Lacier1.ToString("#0.00") & " totalisant " & ((.nbar * PI * .φHA ^ 2 / 4) * 10000).ToString("#0.00") &
                                "cm² par côté (x2 pour la totalité du renfort).") : rtf.NewLine()
                rtf.WriteEndGroup()
            End With
            '
            '--> Détermination des armatures // au sens de portée (= aciers // aux aciers principaux)
            L = ._Tremie.Item(i - 1)._Lx + ._Tremie(i - 1)._xA
            charge = chargeR * (.Lx - ._Tremie.Item(i - 1)._Lx - ._Tremie(i - 1)._xA) / 2 'charge linéaire reporté sur la trémie coté Perpendiculaire au sens porteur
            R_t2 = charge * ._Tremie.Item(i - 1)._Ly / 2
            Δm2 = R_t2 * L * (.Lx - L) / .Lx
            dm2 = R_t2 * ._Tremie(i - 1)._xA * (.Lx - L) / .Lx  'Moment sous R_t1 provoqué par la charge R_t2
            R_t1 = chargeR * ._Tremie(i - 1)._xA / 2 * ._Tremie.Item(i - 1)._Ly / 2
            Δm1 = R_t1 * ._Tremie(i - 1)._xA * ((.Lx - ._Tremie(i - 1)._xA) / .Lx) 'Moment Max sous la charge R_t1
            dm1 = R_t1 * ._Tremie(i - 1)._xA * (1 - L / .Lx) 'Moment sous r_t2 provoqué par la charge R_t1
            MuR2 = Max(Δm1 + dm2, Δm2 + dm1) 'Moment Maxi
            MuR2 = MuR2 * dM
            µu = MuR2 / (PlanCont.Item(j - 1)._Tremie.Item(i - 1)._b_Ly * d ^ 2 * fcd)
            αu = (1 - (1 - 2 * µu) ^ 0.5) / 0.8
            zu = d * (1 - 0.8 * αu / 2)
            Ainf = MuR2 / (zu * fyd)
            A_R2 = Max(As_coupe_X / 2, Ainf)
            rtf.WriteStartGroup()
            rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.WriteString("   ► Renfort: aciers principaux coupés = " & ((As_coupe_X / 2) * 10000).ToString("#0.00") & "cm² et section calculée = " &
                         (Ainf * 10000).ToString("#0.00") & "cm² soit section retenue = " & (A_R2 * 10000).ToString("#0.00") & "cm² par côté.") : rtf.NewLine()
            rtf.WriteEndGroup()
            'détermination des armatures placées contre la trémie dans le sens Ly
            'calcul sur les sections HA8 à HA14
            ama = 0.1   '1000 cm²
            'recherche de la combinaison des diamètres d'acier la plus proche de la section calculée
            'calcul pour la section x - calcul dans le sens porteur
            For i1 = 2 To 3
                i2 = Indice_Acier(._Tremie.Item(i - 1)._HA_Ly.φHA) - 1 'pour ne pas avoir des aciers principaux de diamètre inférieur aux aciers transversaux.
                Do
                    i2 = i2 + 1 'pour ne pas commencer avec un acier de diamètre inférieur
                    If aci.Ha_Lg(i2) = True Then u1 = i1 * PI * aci.Ø_Norm(i2) ^ 2 / 4
                    If i2 + 1 = 5 Then Exit Do 'limitation à HA14
                Loop While u1 < A_R2
                If u1 < ama Then
                    ama = u1
                    'affectation
                    With ._Tremie.Item(i - 1)._HA_Lx
                        .nbar = i1 : .φHA = aci.Ø_Norm.Item(i2)
                    End With
                End If
            Next
            'calcul de l'ancrage de l'acier
            With ._Tremie.Item(i - 1) '
                η1 = 0.7
                fctk = bet.fctk05
                If bet.fctk05 > 3.1 Then fctk = 3.1
                fbd = 2.25 * η1 * 1 * fctk / bet.γcd
                Lbrqd = (._HA_Lx.φHA / 4) * (0.9 * aci.fyk / fbd)
                α1 = 1 : α2 = 0.7 : α3 = 1
                L0 = α1 * α2 * α3 * Lbrqd '(eq 8.4)
                L0 = Truncate(Max(L0, Max(15 * 0.012, 0.1)) * 100) / 100
                'suivant dimension
                If ._Ly <= 0.6 Then
                    'application des prescriptions Thonier voir page 188 Projet BA 1995
                    ._HA_Lx.L13acier = 0
                    ._HA_Lx.Lacier1 = Truncate((2 * L0 + PlanCont.Item(j - 1)._Tremie.Item(i - 1)._Lx + PlanCont.Item(j - 1)._Tremie.Item(i - 1)._Ly + 0.02) * 100) / 100 'acier axé sur trémie
                Else
                    If L0 > Min(PlanCont.Item(j - 1).LappG - PlanCont.Item(j - 1).enrLat, PlanCont.Item(j - 1).LappD - PlanCont.Item(j - 1).enrLat) Then L0 = Min(PlanCont.Item(j - 1).LappG - PlanCont.Item(j - 1).enrLat, PlanCont.Item(j - 1).LappD - PlanCont.Item(j - 1).enrLat)
                    'l'armature est filante sur la longueur de la travée et va s'ancrer à chaque extrémité
                    ._HA_Lx.L13acier = L0
                    ._HA_Lx.Lacier1 = Truncate((2 * L0 + PlanCont.Item(j - 1)._Tremie.Item(i - 1)._Lx) * 100) / 100
                End If
                With ._HA_Lx
                    rtf.WriteString("     Soit " & .nbar.ToString("#0") & "HA" & (.φHA * 1000).ToString("#") & "x" & .Lacier1.ToString("#0.00") & " totalisant " & ((.nbar * PI * .φHA ^ 2 / 4) * 10000).ToString("#0.00") &
                                    "cm² par côté (x2 pour la totalité du renfort).") : rtf.NewLine()
                End With
            End With

        End With

    End Sub
    ''' <summary>
    ''' calcul du renforcement de petite trémie - Chevêtre et barres HA
    ''' </summary>
    ''' <param name="i">index de la tremie  </param>
    ''' <param name="j">n° de travée</param>
    ''' <param name="As_coupe_X">total aire armatures principales coupées par la trémie  </param>
    ''' <param name="As_coupe_Y">total aire armatures de  répartition coupées par la trémie </param>
    ''' <param name="dM">Majoration du Moment suivant annexe J2 </param>
    Private Sub Renforcement_Tremie_Chevetre_et_barresHA(ByVal i As Integer, ByVal j As Integer, ByVal As_coupe_X As Double, ByVal As_coupe_Y As Double, ByVal dM As Double, ByVal b_R_Lx As Double,
                                                       ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim A_R1, A_R2, Ainf, ama, charge, chargeR, Charge_Susp, d, dm1, dm2, fcd, fyd, L, MuR1, MuR2, R_t1, R_t2, st_susp, u1 As Double '
        Dim µu, Δm1, Δm2, αu, zu As Double
        Dim i2, n_susp As Integer
        Dim η1, fctk, fbd, Lbrqd, L0, α1, α2, α3 As Double
        Dim msg As String
        '
        '--> Détermination des armatures perpendiculaires au sens de portée (= aciers // aux aciers de répartition)
        With PlanCont.Item(j - 1)
            'calcul des charges appliquées sur la dalle
            chargeR = 0
            For Each chge As Charge In .Chge
                If chge.Nature = 1 And chge.schem = 1 Then
                    chargeR = chargeR + 1.35 * chge.P1
                End If
                If chge.Nature > 1 And chge.Nature < 8 And chge.schem = 1 Then
                    chargeR = chargeR + 1.5 * chge.P1
                End If
            Next
            'Renfort // au sens non porteur - Sens Y - Sens perpendiculaire au sens porteur
            'réalisation d'un chevêtre
            L = Max(._Tremie(i - 1)._xA, .Lx - ._Tremie.Item(i - 1)._Lx - ._Tremie(i - 1)._xA) 'détermination de la longueur reprenant le plus de charge
            If ._Tremie.Item(i - 1)._Cloison = True Then charge = chargeR + 0.005 'ajout d'un poids linéaire de 5kN/ml, au delà sera traité ailleurs dans le logiciel comme charge linéaire importante
            charge = chargeR * L / 2 'charge linéaire Maximale reporté sur la trémie côté perpendiculaire à la portée
            MuR1 = charge * ._Tremie.Item(i - 1)._Ly ^ 2 / 8
            fcd = bet.fck / bet.γcd : fyd = aci.fyk / aci.γsd
            If ._Tremie.Item(i - 1)._type_chevetre = 1 Then
                ''aciers de répartition SUR prédalle
                d = .H - .Prefa_h - 0.005
            Else
                'de base sauf indication contraire dans l'entrée des données, aciers de répartition dans prédalle
                d = .H - 0.035
            End If
            ._Tremie.Item(i - 1)._b_Ly = 0.2
            µu = MuR1 / (0.2 * d ^ 2 * fcd)
            αu = (1 - (1 - 2 * µu) ^ 0.5) / 0.8
            zu = d * (1 - 0.8 * αu / 2)
            Ainf = MuR1 / (zu * fyd) 'section d'acier placé le long du coté Lx pour reprendre la charge apportée
            A_R1 = Max(As_coupe_Y / 2, Ainf) 'comparaison avec les aciers coupés
            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.WriteString("  . Renfort sur coté // au sens NON porteur: aciers de répartition coupés = " & ((As_coupe_Y / 2) * 10000).ToString("#0.00") & "cm² et aciers calculés = " &
                            (Ainf * 10000).ToString("#0.00") & "cm² soit aciers retenus = " & (A_R1 * 10000).ToString("#0.00") & "cm²") : rtf.NewLine()
            rtf.WriteEndGroup()
            'détermination des armatures placées contre la trémie dans le sens Ly
            'calcul sur les sections HA8 à HA14
            ama = 0.1   '1000 cm²
            'recherche de la combinaison des diamètres d'acier la plus proche de la section calculée
            'calcul pour la section x - calcul dans le sens porteur
            For i = 2 To 3
                i2 = 0
                Do
                    i2 = i2 + 1
                    If aci.Ha_Lg(i2) = True Then u1 = i * PI * aci.Ø_Norm(i2) ^ 2 / 4
                    If i2 + 1 = 5 Then Exit Do 'Limitation à HA14
                Loop While u1 < A_R1
                If u1 < ama Then
                    ama = u1
                    'affectation
                    With ._Tremie.Item(i - 1)._HA_Ly
                        .nbar = i : .φHA = aci.Ø_Norm.Item(i2)
                    End With
                End If
            Next i
            'calcul de l'ancrage de l'acier
            With ._Tremie.Item(i - 1)._HA_Ly
                η1 = 0.7
                fctk = bet.fctk05
                If bet.fctk05 > 3.1 Then fctk = 3.1
                fbd = 2.25 * η1 * 1 * fctk / bet.γcd
                Lbrqd = (.φHA / 4) * (0.9 * aci.fyk / fbd)
                α1 = 1 : α2 = 0.7 : α3 = 1
                L0 = α1 * α2 * α3 * Lbrqd '(eq 8.4)
                L0 = Truncate(Max(L0, Max(15 * 0.012, 0.1)) * 100) / 100
                If L0 > b_R_Lx Then .L13acier = L0 Else .L13acier = b_R_Lx
                PlanCont.Item(j - 1)._Tremie.Item(i - 1)._b_Ly = L0
                .Lacier1 = Truncate((2 * L0 + 0.02 + PlanCont.Item(j - 1)._Tremie.Item(i - 1)._Ly) * 100) / 100
            End With
            'calcul des aciers de suspente
            st_susp = Truncate((0.9 * .H) * 100 + 1) / 100
            n_susp = CInt(Floor(._Tremie.Item(i - 1)._Ly / st_susp))
            If n_susp * st_susp < ._Tremie.Item(i - 1)._Ly Then n_susp = n_susp + 1
            ._Tremie.Item(i - 1)._HA_Ly_Susp.φHA = 0.006
            Charge_Susp = n_susp * 2 * PI * 0.006 ^ 2 / 4 * aci.fyk / aci.γsd
            'vérification que la charge soit bien reprise par les aciers de suspente
            If Charge_Susp < chargeR * L Then
                ._Tremie.Item(i - 1)._HA_Ly_Susp.φHA = 0.008
                Charge_Susp = n_susp * 2 * PI * 0.008 ^ 2 / 4 * aci.fyk / aci.γsd
                If Charge_Susp < chargeR * L Then
                    msg = "Travée n°" & j.ToString & " - trémie n°" & i.ToString & ": les aciers de suspentes en HA8 espacés tous les " & (st_susp * 100).ToString("#0") & "cm sont insuffisant pour suspendre la charge. A revoir."
                    Aff.msg_Avertiss.Add(msg)
                End If
            End If
            For i = 1 To n_susp - 1
                ._Tremie.Item(i - 1)._HA_Ly_Susp.st(i - 1) = st_susp
            Next
            'calcul de la longueur développée des cadres, épingles, étriers
            i = Indice_Acier(._Tremie.Item(i - 1)._HA_Ly_Susp.φHA)
            'Ref: projet BA Thonier page 127
            Dim dm_At() As Single = {0, 0.02, 0.03, 0.04, 0.05, 0.07, 0.1}
            If aci.fyk = 500 Then dm_At(1) = 0.03
            ._Tremie.Item(i - 1)._HA_Ly_Susp.LtCadre = 2 * (.H + 0.02 - 4 * .enrLat + .enrSup + .enrInf) + 1.426 * dm_At(i) / 2 + 16.713 * ._Tremie.Item(i - 1)._HA_Ly_Susp.φHA 'formule THONIER Projet BA
            ._Tremie.Item(i - 1)._HA_Ly_Susp.LtCadre = (Truncate(._Tremie.Item(i - 1)._HA_Ly_Susp.LtCadre + 0.005) * 100) / 100
            ._Tremie.Item(i - 1)._HA_Ly_Susp.nepingle = 0 : ._Tremie.Item(i - 1)._HA_Ly_Susp.LtEpingle = 0
            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.WriteString("  . Chevêtre: aciers de suspente: cadres HA" & (._Tremie.Item(i - 1)._HA_Ly_Susp.φHA * 1000).ToString("#") & " espacés de  " & (st_susp * 100).ToString("#0") & "cm + 1 cadre dans la zone d'ancrage (voir schéma).") : rtf.NewLine()
            rtf.WriteEndGroup()
            '
            '--> Détermination des armatures // au sens de portée (= aciers // aux aciers principaux)
            L = ._Tremie.Item(i - 1)._Lx + ._Tremie(i - 1)._xA
            charge = chargeR * (.Lx - ._Tremie.Item(i - 1)._Lx - ._Tremie(i - 1)._xA) / 2 'charge linéaire reporté sur la trémie coté Perpendiculaire au sens porteur
            R_t2 = charge * ._Tremie.Item(i - 1)._Ly / 2
            Δm2 = R_t2 * L * (.Lx - L) / .Lx
            dm2 = R_t2 * ._Tremie(i - 1)._xA * (.Lx - L) / .Lx  'Moment sous R_t1 provoqué par la charge R_t2
            R_t1 = chargeR * ._Tremie(i - 1)._xA / 2 * ._Tremie.Item(i - 1)._Ly / 2
            Δm1 = R_t1 * ._Tremie(i - 1)._xA * ((.Lx - ._Tremie(i - 1)._xA) / .Lx) 'Moment Max sous la charge R_t1
            dm1 = R_t1 * ._Tremie(i - 1)._xA * (1 - L / .Lx) 'Moment sous r_t2 provoqué par la charge R_t1
            MuR2 = Max(Δm1 + dm2, Δm2 + dm1) 'Moment Maxi
            MuR2 = MuR2 * dM
            µu = MuR2 / (PlanCont.Item(j - 1)._Tremie.Item(i - 1)._b_Ly * d ^ 2 * fcd)
            αu = (1 - (1 - 2 * µu) ^ 0.5) / 0.8
            zu = d * (1 - 0.8 * αu / 2)
            Ainf = MuR2 / (zu * fyd)
            A_R2 = Max(As_coupe_X / 2, Ainf)
            rtf.WriteStartGroup()
            rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.WriteString("  . Renfort: aciers principaux coupés = " & ((As_coupe_X / 2) * 10000).ToString("#0.00") & "cm² et aciers calculés = " &
                         (Ainf * 10000).ToString("#0.00") & "cm² soit aciers retenus = " & (A_R2 * 10000).ToString("#0.00") & "cm²") : rtf.NewLine()
            rtf.WriteEndGroup()
            'détermination des armatures placées contre la trémie dans le sens Ly
            'calcul sur les sections HA8 à HA14
            ama = 0.1   '1000 cm²
            'recherche de la combinaison des diamètres d'acier la plus proche de la section calculée
            'calcul pour la section x - calcul dans le sens porteur
            For i = 2 To 3
                i2 = Indice_Acier(._Tremie.Item(i - 1)._HA_Ly.φHA) - 1
                Do
                    i2 = i2 + 1 'pour ne pas commencer avec un acier de diamètre inférieur
                    If aci.Ha_Lg(i2) = True Then u1 = i * PI * aci.Ø_Norm(i2) ^ 2 / 4
                    If i2 + 1 = 5 Then Exit Do 'limitation à HA14
                Loop While u1 < A_R1
                If u1 < ama Then
                    ama = u1
                    'affectation
                    With ._Tremie.Item(i - 1)._HA_Lx
                        .nbar = i : .φHA = aci.Ø_Norm.Item(i2)
                    End With
                End If
            Next i
            'calcul de l'ancrage de l'acier
            With ._Tremie.Item(i - 1) '
                η1 = 0.7
                fctk = bet.fctk05
                If bet.fctk05 > 3.1 Then fctk = 3.1
                fbd = 2.25 * η1 * 1 * fctk / bet.γcd
                Lbrqd = (._HA_Lx.φHA / 4) * (0.9 * aci.fyk / fbd)
                α1 = 1 : α2 = 0.7 : α3 = 1
                L0 = α1 * α2 * α3 * Lbrqd '(eq 8.4)
                L0 = Truncate(Max(L0, Max(15 * 0.012, 0.1)) * 100) / 100
                If L0 > Min(PlanCont.Item(j - 1).LappG - PlanCont.Item(j - 1).enrLat, PlanCont.Item(j - 1).LappD - PlanCont.Item(j - 1).enrLat) Then L0 = Min(PlanCont.Item(j - 1).LappG - PlanCont.Item(j - 1).enrLat, PlanCont.Item(j - 1).LappD - PlanCont.Item(j - 1).enrLat)
                'l'armature est filante sur la longueur de la travée et va s'ancrer à chaque extrémité
                ._HA_Lx.L13acier = L0
                ._HA_Lx.Lacier1 = Truncate((2 * L0 + PlanCont.Item(j - 1)._Tremie.Item(i - 1)._Lx) * 100) / 100

            End With

        End With

    End Sub
    ''' <summary>
    ''' calcul du renforcement de trémie nécessitant obligatoirement chevêtre et bande noyée
    ''' </summary>
    ''' <param name="i">index de la tremie  </param>
    ''' <param name="j">n° de travée</param>
    Private Sub Renforcement_Tremie_Chevetre_BandeNoyé(ByVal i As Integer, ByVal j As Integer, ByVal As_coupe_X As Double, ByVal As_coupe_Y As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim A_R1, A_R2, Ainf, ama, b_BN, charge, chargeR, Charge_Susp, d, dm1, dm2, fcd, fyd, L, MuR1, MuR2, R_t1, R_t2, st_susp, u1 As Double '
        Dim µu, Δm1, Δm2, αu, zu As Double
        Dim i2, n_susp As Integer
        Dim η1, fctk, fbd, Lbrqd, L0, α1, α2, α3 As Double
        Dim msg As String
        '
        '--> Détermination des armatures perpendiculaires au sens de portée (= aciers // aux aciers de répartition)
        With PlanCont.Item(j - 1)
            'calcul des charges appliquées sur la dalle
            chargeR = 0
            For Each chge As Charge In .Chge
                If chge.Nature = 1 And chge.schem = 1 Then
                    chargeR = chargeR + 1.35 * chge.P1
                End If
                If chge.Nature > 1 And chge.Nature < 8 And chge.schem = 1 Then
                    chargeR = chargeR + 1.5 * chge.P1
                End If
            Next
            'Renfort // au sens non porteur - Sens Y - Sens perpendiculaire au sens porteur
            'réalisation d'un chevêtre
            L = Max(._Tremie(i - 1)._xA, .Lx - ._Tremie.Item(i - 1)._Lx - ._Tremie(i - 1)._xA) 'détermination de la longueur reprenant le plus de charge
            If ._Tremie.Item(i - 1)._Cloison = True Then charge = chargeR + 0.005 'ajout d'un poids linéaire de 5kN/ml, au delà sera traité ailleurs dans le logiciel comme charge linéaire importante
            charge = chargeR * L / 2 'charge linéaire Maximale reporté sur la trémie côté perpendiculaire à la portée
            MuR1 = charge * ._Tremie.Item(i - 1)._Ly ^ 2 / 8
            fcd = bet.fck / bet.γcd : fyd = aci.fyk / aci.γsd
            If ._Tremie.Item(i - 1)._type_chevetre = 1 Then
                ''aciers de répartition SUR prédalle
                d = .H - .Prefa_h - 0.005
            Else
                'de base sauf indication contraire dans l'entrée des données, aciers de répartition dans prédalle
                d = .H - 0.035
            End If
            ._Tremie.Item(i - 1)._b_Ly = 0.2
            µu = MuR1 / (0.2 * d ^ 2 * fcd)
            αu = (1 - (1 - 2 * µu) ^ 0.5) / 0.8
            zu = d * (1 - 0.8 * αu / 2)
            Ainf = MuR1 / (zu * fyd) 'section d'acier placé le long du coté Lx pour reprendre la charge apportée
            A_R1 = Max(As_coupe_Y / 2, Ainf) 'comparaison avec les aciers coupés
            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.WriteString("  . Renfort sur coté // au sens NON porteur: aciers de répartition coupés = " & ((As_coupe_Y / 2) * 10000).ToString("#0.00") & "cm² et aciers calculés = " &
                            (Ainf * 10000).ToString("#0.00") & "cm² soit aciers retenus = " & (A_R1 * 10000).ToString("#0.00") & "cm²") : rtf.NewLine()
            rtf.WriteEndGroup()
            'détermination des armatures placées contre la trémie dans le sens Ly
            'calcul sur les sections HA8 à HA14
            ama = 0.1   '1000 cm²
            'recherche de la combinaison des diamètres d'acier la plus proche de la section calculée
            'calcul pour la section x - calcul dans le sens porteur
            For i = 2 To 3
                i2 = 0
                Do
                    i2 = i2 + 1
                    If aci.Ha_Lg(i2) = True Then u1 = i * PI * aci.Ø_Norm(i2) ^ 2 / 4
                    If i2 + 1 = 5 Then Exit Do 'Limitation à HA14
                Loop While u1 < A_R1
                If u1 < ama Then
                    ama = u1
                    'affectation
                    With ._Tremie.Item(i - 1)._HA_Ly
                        .nbar = i : .φHA = aci.Ø_Norm.Item(i2)
                    End With
                End If
            Next i
            'calcul de l'ancrage de l'acier du chevêtre sur la bande noyé
            '- Voir à la fin du présent sub
            'calcul des aciers de suspente et d'effort tranchant
            '
            'Secteur à reprendre
            '
            st_susp = Truncate((0.9 * .H) * 100 + 1) / 100
            n_susp = CInt(Floor(._Tremie.Item(i - 1)._Ly / st_susp))
            If n_susp * st_susp < ._Tremie.Item(i - 1)._Ly Then n_susp = n_susp + 1
            ._Tremie.Item(i - 1)._HA_Ly_Susp.φHA = 0.006
            Charge_Susp = n_susp * 2 * PI * 0.006 ^ 2 / 4 * aci.fyk / aci.γsd
            'vérification que la charge soit bien reprise par les aciers de suspente
            If Charge_Susp < chargeR * L Then
                ._Tremie.Item(i - 1)._HA_Ly_Susp.φHA = 0.008
                Charge_Susp = n_susp * 2 * PI * 0.008 ^ 2 / 4 * aci.fyk / aci.γsd
                If Charge_Susp < chargeR * L Then
                    msg = "Travée n°" & j.ToString & " - trémie n°" & i.ToString & ": les aciers de suspentes en HA8 espacés tous les " & (st_susp * 100).ToString("#0") & "cm sont insuffisants pour suspendre la charge. A revoir."
                    Aff.msg_Avertiss.Add(msg)
                End If
            End If
            For i = 1 To n_susp - 1
                ._Tremie.Item(i - 1)._HA_Ly_Susp.st(i - 1) = st_susp
            Next
            'calcul de la longueur développée des cadres, épingles, étriers
            i = Indice_Acier(._Tremie.Item(i - 1)._HA_Ly_Susp.φHA)
            'Ref: projet BA Thonier page 127
            Dim dm_At() As Single = {0, 0.02, 0.03, 0.04, 0.05, 0.07, 0.1}
            If aci.fyk = 500 Then dm_At(1) = 0.03
            ._Tremie.Item(i - 1)._HA_Ly_Susp.LtCadre = 2 * (.H + 0.02 - 4 * .enrLat + .enrSup + .enrInf) + 1.426 * dm_At(i) / 2 + 16.713 * ._Tremie.Item(i - 1)._HA_Ly_Susp.φHA 'formule THONIER Projet BA
            ._Tremie.Item(i - 1)._HA_Ly_Susp.LtCadre = (Truncate(._Tremie.Item(i - 1)._HA_Ly_Susp.LtCadre + 0.005) * 100) / 100
            ._Tremie.Item(i - 1)._HA_Ly_Susp.nepingle = 0 : ._Tremie.Item(i - 1)._HA_Ly_Susp.LtEpingle = 0
            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.WriteString("  . Chevêtre: aciers de suspente: cadres HA" & (._Tremie.Item(i - 1)._HA_Ly_Susp.φHA * 1000).ToString("#") & " espacés de  " & (st_susp * 100).ToString("#0") & "cm + 1 cadre dans la zone d'ancrage (voir schéma).") : rtf.NewLine()
            rtf.WriteEndGroup()
            '
            '--> Détermination des armatures // au sens de portée (= aciers // aux aciers principaux)
            L = ._Tremie.Item(i - 1)._Lx + ._Tremie(i - 1)._xA
            charge = chargeR * (.Lx - ._Tremie.Item(i - 1)._Lx - ._Tremie(i - 1)._xA) / 2 'charge linéaire reporté sur la trémie coté Perpendiculaire au sens porteur
            R_t2 = charge * ._Tremie.Item(i - 1)._Ly / 2
            Δm2 = R_t2 * L * (.Lx - L) / .Lx
            dm2 = R_t2 * ._Tremie(i - 1)._xA * (.Lx - L) / .Lx  'Moment sous R_t1 provoqué par la charge R_t2
            R_t1 = chargeR * ._Tremie(i - 1)._xA / 2 * ._Tremie.Item(i - 1)._Ly / 2
            Δm1 = R_t1 * ._Tremie(i - 1)._xA * ((.Lx - ._Tremie(i - 1)._xA) / .Lx) 'Moment Max sous la charge R_t1
            dm1 = R_t1 * ._Tremie(i - 1)._xA * (1 - L / .Lx) 'Moment sous r_t2 provoqué par la charge R_t1
            MuR2 = Max(Δm1 + dm2, Δm2 + dm1) 'Moment Maxi
            b_BN = 0.15
            ._Tremie.Item(i - 1)._HA_Lx.nbar = 0
            Do
                b_BN = b_BN + 0.05 'pas de 5 cm
                MuR2 = MuR2 + (chargeR * b_BN) * .Lcx ^ 2 / 8
                µu = MuR2 / (b_BN * d ^ 2 * fcd)
                αu = (1 - (1 - 2 * µu) ^ 0.5) / 0.8
                zu = d * (1 - 0.8 * αu / 2)
                Ainf = MuR2 / (zu * fyd)
                A_R2 = Max(As_coupe_X / 2, Ainf)
                'détermination des armatures placées contre la trémie dans le sens Lx - Sens porteur
                'calcul sur les sections HA10 à HA20
                ama = 0.1   '1000 cm²
                'recherche de la combinaison des diamètres d'acier la plus proche de la section calculée
                'calcul pour la section x - calcul dans le sens porteur
                For i = 2 To 3
                    i2 = Indice_Acier(._Tremie.Item(i - 1)._HA_Ly.φHA) - 1
                    Do
                        i2 = i2 + 1 'pour ne pas commencer avec un acier de diamètre inférieur
                        If aci.Ha_Lg(i2) = True Then u1 = i * PI * aci.Ø_Norm(i2) ^ 2 / 4
                        If i2 + 1 = 8 Then Exit Do 'limitation à HA20
                    Loop While u1 < A_R2
                    If u1 < ama Then
                        ama = u1
                        'affectation
                        With ._Tremie.Item(i - 1)._HA_Lx
                            .nbar = i : .φHA = aci.Ø_Norm.Item(i2)
                        End With
                    End If
                Next i
                If ._Tremie.Item(i - 1)._HA_Lx.nbar > 1 Then Exit Do
            Loop While b_BN < 0.35 'limitation à 30 cm
            PlanCont.Item(j - 1)._Tremie.Item(i - 1)._b_Lx = b_BN
            If b_BN = 0.35 Then
                With ._Tremie.Item(i - 1)
                    ._HA_Lx.nbar = 3 : ._HA_Lx.φHA = 0.02 : ._b_Lx = 0.3
                End With
                msg = "Travée n°" & j.ToString & " - trémie n°" & i.ToString & ": section bande noyée insuffisant pour reprendre la charge. A revoir."
                Aff.msg_Avertiss.Add(msg)
            End If
            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.WriteString("  . Bande noyée: aciers principaux coupés = " & ((As_coupe_X / 2) * 10000).ToString("#0.00") & "cm² et aciers calculés = " &
                         (Ainf * 10000).ToString("#0.00") & "cm² soit aciers retenus = " & (A_R2 * 10000).ToString("#0.00") & "cm²") : rtf.NewLine()
            rtf.WriteEndGroup()
            'calcul de l'ancrage de l'acier sur appui porteur
            With ._Tremie.Item(i - 1) '
                η1 = 0.7
                fctk = bet.fctk05
                If bet.fctk05 > 3.1 Then fctk = 3.1
                fbd = 2.25 * η1 * 1 * fctk / bet.γcd
                Lbrqd = (._HA_Lx.φHA / 4) * (0.9 * aci.fyk / fbd)
                α1 = 1 : α2 = 0.7 : α3 = 1
                L0 = α1 * α2 * α3 * Lbrqd '(eq 8.4)
                L0 = Truncate(Max(L0, Max(15 * 0.012, 0.1)) * 100) / 100
                If L0 > Min(PlanCont.Item(j - 1).LappG - PlanCont.Item(j - 1).enrLat, PlanCont.Item(j - 1).LappD - PlanCont.Item(j - 1).enrLat) Then L0 = Min(PlanCont.Item(j - 1).LappG - PlanCont.Item(j - 1).enrLat, PlanCont.Item(j - 1).LappD - PlanCont.Item(j - 1).enrLat)
                'l'armature est filante sur la longueur de la travée et va s'ancrer à chaque extrémité
                ._HA_Lx.L13acier = L0
                ._HA_Lx.Lacier1 = Truncate((2 * L0 + PlanCont.Item(j - 1)._Tremie.Item(i - 1)._Lx) * 100) / 100

            End With
            'calcul de l'ancrage de l'acier du chevêtre sur la bande noyé
            With ._Tremie.Item(i - 1)._HA_Ly
                η1 = 0.7
                fctk = bet.fctk05
                If bet.fctk05 > 3.1 Then fctk = 3.1
                fbd = 2.25 * η1 * 1 * fctk / bet.γcd
                Lbrqd = (.φHA / 4) * (0.9 * aci.fyk / fbd)
                α1 = 1 : α2 = 0.7 : α3 = 1
                L0 = α1 * α2 * α3 * Lbrqd '(eq 8.4)
                L0 = Truncate(Max(L0, Max(15 * 0.012, 0.1)) * 100) / 100
                If L0 > b_BN Then .L13acier = L0 Else .L13acier = b_BN
                PlanCont.Item(j - 1)._Tremie.Item(i - 1)._b_Ly = L0
                .Lacier1 = Truncate((2 * L0 + 0.02 + PlanCont.Item(j - 1)._Tremie.Item(i - 1)._Ly) * 100) / 100
            End With
            'Dimensionnement des aciers d'effort tranchant
            '
            'A faire
            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.WriteString("  . Bande noyée: aciers effort tranchant non dimensionné - Calcul non implémenté") : rtf.NewLine()
            rtf.WriteEndGroup()
        End With

    End Sub


#End Region

    Private Sub Armatures_complementaires_predalles()

        Dim G, Q, Q1, L1, Medy, Mus, ηy, Asi(1, PlanCont.Count, 0), Largeur, Reste As Double
        Dim i As Integer

        For Each dalle As Plancher In PlanCont
            With dalle
                G = 0 : Q = 0
                For Each chge As Charge In PlanCont.Item(PlanCont.IndexOf(dalle)).Chge 'Charges
                    If chge.schem = 1 And chge.Nature = 1 Then 'répartie linéaire et Permanente
                        G = G + chge.P1
                    End If
                    If chge.schem = 1 And chge.Nature >= 2 And chge.Nature <= 4 Then 'répartie linéaire et Exploitation
                        Q = Q + chge.P1
                    End If
                Next
                G = G + 0.25 * .H
                Q1 = Q * 0.5
                If Q < 2 * G And Q1 < 0.005 Then
                    'Moment transversal négligeable suivant condition CPT §B3.1.2 page50
                    .Prefa_TS_joint.indTS = 0 'ST10
                    'calcul du recouvrement sur armatures de répartition EN1992-1-1 §8.7.5.2
                    Select Case aci.Øtrans(.TS_trav.Item(0).indTS)
                        Case Is <= 0.006
                            L1 = 0.15
                            If L1 < 2 * aci.Esp_Long(.TS_trav.Item(0).indTS) Then L1 = 2 * aci.Esp_Long(.TS_trav.Item(0).indTS)
                        Case Is <= 0.0085
                            L1 = 0.25
                            If L1 < 3 * aci.Esp_Long(.TS_trav.Item(0).indTS) Then L1 = 3 * aci.Esp_Long(.TS_trav.Item(0).indTS)
                        Case Else
                            L1 = 0.35
                            If L1 < 4 * aci.Esp_Long(.TS_trav.Item(0).indTS) Then L1 = 4 * aci.Esp_Long(.TS_trav.Item(0).indTS)
                    End Select
                    .Prefa_TS_joint.Lacier1 = Truncate((2 * L1 + .Prefa_esp + .Prefa_h + 0.01) * 100) / 100
                Else
                    'Moment transversal NON négligeable suivant condition CPT §B3.1.2 page50 et §1.2.3.3 page 12
                    'calcul au droit du joint
                    ηy = (1 - (.Prefa_h / .H) ^ 3)
                    Medy = 0.02 * ηy * 1.5 * Q * .Lcx ^ 2
                    Mus = bet.fctm * .H ^ 2 / 6
                    Call Calcul_ELU_armature_flexion_poutre_rectangulaire(1, PlanCont.IndexOf(dalle) + 1, 0, Max(Medy, Mus), .H - .Prefa_h - 0.005, Asi)
                    For i = 1 To 6 'st50
                        If Asi(1, PlanCont.IndexOf(dalle) + 1, 0) < aci.As_long.Item(i - 1) Then .Prefa_TS_joint.indTS = i - 1 : Exit For
                    Next
                    'calcul du recouvrement sur armatures principales EN1992-1-1 §8.7.5.1
                    Dim η1, fctk, fbd, Lbmin, Lbrqd, L0, α1, α2, α3, α4, α6 As Double
                    η1 = 0.7
                    fctk = bet.fctk05
                    If bet.fctk05 > 3.1 Then fctk = 3.1
                    fbd = 2.25 * η1 * 1 * fctk / bet.γcd
                    Lbrqd = (aci.Ølong(.Prefa_TS_joint.indTS) / 4) * (0.9 * aci.fyk / fbd)
                    α1 = 1 : α2 = 0.7 : α3 = 1 : α4 = 0.7 : α6 = 1.5
                    L0 = α1 * α2 * α3 * α4 * α6 * Lbrqd '(eq 8.4)
                    Lbmin = Max(0.3 * α6 * Lbrqd, Max(15 * aci.Ølong(.Prefa_TS_joint.indTS), 0.2))
                    If L0 < Lbmin Then L0 = Lbmin
                    .Prefa_TS_joint.Lacier1 = Truncate((2 * L0 + .Prefa_esp + .Prefa_h + 0.01) * 100) / 100
                End If
                'Affectation dans .Prefa_TS_joint
                Select Case .Prefa_TS_joint.indTS
                    Case 0 'ST10
                        Largeur = aci.Long_TS(.Prefa_TS_joint.indTS)
                    Case Else
                        Largeur = aci.Larg_TS(.Prefa_TS_joint.indTS)
                End Select
                .Prefa_TS_joint.LacierY.Clear()
                If Largeur > .Prefa_Lx.Item(0) Then
                    .Prefa_TS_joint.LacierY.Add(.Prefa_Lx.Item(0))
                Else
                    .Prefa_TS_joint.LacierY.Add(Largeur)   '.LacierY.Add(Largeur)
                    Reste = .Lx Mod Largeur
                    .Prefa_TS_joint.nbrePanneaux = CInt(Fix(.Lx / Largeur))
                    'Select Case Reste
                    'Case Is > 2
                    '.Prefa_TS_joint.nbrePanneaux = .Prefa_TS_joint.nbrePanneaux + 1
                    'Case Is < 0.3 ' EspMaxPrincipal = 0.25 pour charge concentrée
                    ''pas de modification du nombre de panneaux
                    'Case Else
                    'ajout d'un modèle
                    .Prefa_TS_joint.LacierY.Add(Truncate(Reste * 100 + 1) / 100) 'arrondi au cm supérieur
                    If .Prefa_TS_joint.LacierY.Item(1) < 0.6 Then .Prefa_TS_joint.LacierY.Item(1) = 0.6
                    'End Select
                End If

                '--> TS sur rive laterale
                .Prefa_TS_RiveLaterale.indTS = .Prefa_TS_joint.indTS
                .Prefa_TS_RiveLaterale.L13acier = Truncate((Max(10 * aci.Ølong(.Prefa_TS_joint.indTS), 0.1) + 0.01) * 100) / 100
                'les valeurs x1 et x2 sont données en valeur relative
                .Prefa_TS_RiveLaterale.X1acier = 0
                .Prefa_TS_RiveLaterale.X2acier = Recouvrement_Trans_TS(.Prefa_TS_RiveLaterale.indTS) 'Valeur recouvrement transversal
                .Prefa_TS_RiveLaterale.Lacier1 = .Prefa_TS_RiveLaterale.X2acier + .Prefa_TS_RiveLaterale.L13acier
                .Prefa_TS_RiveLaterale.Lacier2 = .Prefa_TS_RiveLaterale.Lacier1
                .Prefa_TS_RiveLaterale.nbrePanneaux = .Prefa_TS_joint.nbrePanneaux
                .Prefa_TS_RiveLaterale.LacierY.Clear()
                For i = 1 To .Prefa_TS_joint.LacierY.Count
                    .Prefa_TS_RiveLaterale.LacierY.Add(.Prefa_TS_joint.LacierY.Item(i - 1))
                Next
            End With
        Next

    End Sub
    ''' <summary>
    '''Calcul du métré des armatures
    ''' </summary>
    Private Sub Metre()
        Dim Largeur, Reste, Ly_AppH As Double
        Dim i, i1, i2, j As Integer
        Dim modele_Identique As Boolean = True
        For Each dalle As Plancher In PlanCont
            With dalle
                'Regroupement de prédalles par modèle - Indiquera le nombre de prédalles par modèle soit 3 modèles de prédalles pour la travée 1 avec 2 prédalles du modèle 1, 1 prédalles du modèle 2, etc. ...
                If .Prefa = True Then
                    .Prefa_modele.Clear()
                    j = 1
                    .Prefa_modele.Add(j) 'la prédalle n°1 sera toujours de n° de modèle 1
                    Ly_AppH = .Prefa_Ly.Item(0) + .Prefa_esp
                    For i = 2 To .Prefa_Num.Count
                        modele_Identique = True
                        'vérification sur l'emplacement des trémies - Hypothèse: 2 trémies ne se trouvent pas strictement au même endroit sur 2 prédalles différentes
                        If ._Tremie.Count >= 1 Then
                            For i1 = 1 To ._Tremie.Count
                                If ._Tremie.Item(i1 - 1)._yA > Ly_AppH And ._Tremie.Item(i1 - 1)._yA < Ly_AppH + .Prefa_Ly.Item(i - 1) Then
                                    'la trémie se trouve dans cette prédalle - Création d'un nouveau n° de modèle
                                    j = j + 1
                                    .Prefa_modele.Add(j) 'la prédalle n°i sera de n° de modèle j
                                    modele_Identique = False
                                    Exit For
                                End If
                            Next
                            Ly_AppH = Ly_AppH + .Prefa_Ly.Item(i - 1) + .Prefa_esp
                            If modele_Identique = False Then Continue For 'passage à la prédalle suivante
                        End If
                        'Pas de trémie - comparaison avec toutes les prédalles précédentes sur les termes géométriques (sera à modifier plus tard avec les charges linéaires)
                        i1 = i
                        For i2 = i1 - 1 To 1 Step -1
                            If .Prefa_Ly.Item(i1 - 1) = .Prefa_Ly.Item(i2 - 1) And .Prefa_Lx.Item(i1 - 1) = .Prefa_Lx.Item(i2 - 1) Then
                                modele_Identique = True 'j'ai trouvé un modèle identique dans les prédalles précédentes
                                .Prefa_modele.Add(.Prefa_modele.Item(i2 - 1)) 'la prédalle n°i1 sera de n° de modèle j
                                Exit For 'passage à la prédalle suivante
                            Else
                                modele_Identique = False
                            End If
                        Next
                        If modele_Identique = False Then
                            ' Création d'un nouveau n° de modèle
                            j = j + 1
                            .Prefa_modele.Add(j) 'la prédalle n°i sera de n° de modèle j
                        End If
                    Next
                End If
                'aciers inférieurs
                Select Case .Type_Acier_Travée
                    Case 1 'TS
                        For i = 1 To .TS_trav.Count
                            With .TS_trav
                                Select Case i
                                    Case 1 'lit n°1
                                        If dalle.TS_trav_Portefeuille.Count >= 1 Then
                                            .Item(i - 1).Lacier1 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier
                                            .Item(i - 1).X2acier = aci.Long_TS(.Item(0).indTS) - .Item(i - 1).Lacier1
                                            .Item(i - 1).Lacier2 = aci.Long_TS(.Item(0).indTS)  'normalement égale à Longueur totale du panneau TS
                                            If dalle.Prefa = True Then
                                                .Item(i - 1).Lacier1 = .Item(i - 1).X2acier + dalle.Prefa_ReposG
                                            Else
                                                .Item(i - 1).Lacier1 = .Item(i - 1).X2acier
                                            End If
                                            'appui droit et acier complémentaire
                                            dalle.TS_trav_Portefeuille.Item(0).L21acier = .Item(i - 1).L21acier
                                            dalle.TS_trav_Portefeuille.Item(0).L22acier = .Item(i - 1).L22acier
                                            dalle.TS_trav_Portefeuille.Item(0).L23acier = .Item(i - 1).L23acier
                                            'calcul du recouvrement entre les 2 lits d'acier
                                            dalle.TS_trav_Portefeuille.Item(0).Lacier2 = .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                            dalle.TS_trav_Portefeuille.Item(0).Lacier2 += dalle.Lx - .Item(i - 1).X2acier + Recouvrement_Long_TS(.Item(0).indTS) 'Les 2 TS sont identiques
                                            dalle.TS_trav_Portefeuille.Item(0).X2acier = dalle.Lx
                                            dalle.TS_trav_Portefeuille.Item(0).X1acier = .Item(i - 1).X2acier - Recouvrement_Long_TS(.Item(0).indTS)
                                            If dalle.Prefa = True Then
                                                dalle.TS_trav_Portefeuille.Item(i - 1).Lacier1 = dalle.Lx - .Item(i - 1).X2acier + Recouvrement_Long_TS(.Item(0).indTS) + dalle.Prefa_ReposD
                                            Else
                                                dalle.TS_trav_Portefeuille.Item(i - 1).Lacier1 = dalle.Lx - .Item(i - 1).X2acier + Recouvrement_Long_TS(.Item(0).indTS)
                                            End If
                                            'mise à 0 pour l'acier initial - pour éviter tout pb ultérieur
                                            .Item(i - 1).L21acier = 0 : .Item(i - 1).L22acier = 0 : .Item(i - 1).L23acier = 0
                                        Else
                                            .Item(i - 1).Lacier2 = .Item(i - 1).X2acier - .Item(i - 1).X1acier 'normalement égal à Lx
                                            .Item(i - 1).Lacier1 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).Lacier1
                                        End If
                                    Case 2  'lit n°2
                                        If dalle.Lit2CouvertureTravee = True Then
                                            'le lit n°2 couvre l'intégralité de la dalle
                                            .Item(i - 1).Lacier2 = .Item(i - 1).X2acier - .Item(i - 1).X1acier 'normalement égal à Lx
                                            .Item(i - 1).Lacier1 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier

                                        Else
                                            'le lit n°2 couvre uniquement la partie nécessaire à la reprise de l'effort de flexion
                                            .Item(i - 1).Lacier2 = .Item(i - 1).X2acier - .Item(i - 1).X1acier 'normalement égal à Lx
                                            .Item(i - 1).Lacier1 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier

                                        End If
                                        .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).Lacier1
                                End Select
                            End With
                        Next
                    Case 2 'HA
                        For i = 1 To .LitInf.Count
                            With .LitInf
                                Select Case i
                                    Case 1 'lit n°1
                                        .Item(i - 1).Lacier2 = .Item(i - 1).X2acier - .Item(i - 1).X1acier 'normalement égal à Lx
                                        .Item(i - 1).Lacier1 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                        .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).Lacier1
                                    Case 2  'lit n°2
                                        If dalle.Lit2CouvertureTravee = True Then
                                            'le lit n°2 couvre l'intégralité de la dalle
                                            .Item(i - 1).Lacier2 = .Item(i - 1).X2acier - .Item(i - 1).X1acier 'normalement égal à Lx
                                            .Item(i - 1).Lacier1 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).Lacier1
                                        Else
                                            'le lit n°2 couvre uniquement la partie nécessaire à la reprise de l'effort de flexion
                                            .Item(i - 1).Lacier2 = .Item(i - 1).X2acier - .Item(i - 1).X1acier 'normalement égal à Lx
                                            .Item(i - 1).Lacier1 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).Lacier1
                                        End If
                                End Select
                            End With
                        Next
                End Select

                '--> Détermination du nombre de panneaux TS en armatures inférieures
                If .Prefa = False Then
                    For i = 1 To dalle.TS_trav.Count
                        Largeur = aci.Larg_TS(.TS_trav.Item(i - 1).indTS) 'largeur du panneau TS
                        .TS_trav.Item(i - 1).LacierY.Clear()
                        .TS_trav.Item(i - 1).LacierY.Add(Largeur) 'pour les panneaux de modèle 1
                        .TS_trav.Item(i - 1).RecouvTr = Recouvrement_Trans_TS(.TS_trav.Item(i - 1).indTS) 'recouvrement du panneau TS
                        If .Ly > 2.3 Then
                            Reste = .Ly Mod (Largeur - .TS_trav.Item(i - 1).RecouvTr)
                            Reste += .LapHA * 2
                            .TS_trav.Item(i - 1).nbrePanneaux = 0 'initialisation
                            Select Case Reste
                                Case Is > Largeur
                                    .TS_trav.Item(i - 1).nbrePanneaux = 1
                                    Reste = Reste - Largeur
                                    If Reste < 0.45 Then Reste = 0.45 'pour avoir un panneau de 45cm minimum de large
                                    .TS_trav.Item(i - 1).LacierY.Add(Reste) 'pour les panneaux de modèle 2
                                Case Largeur
                                    .TS_trav.Item(i - 1).nbrePanneaux = 1
                                Case Is < 0.45
                                    .TS_trav.Item(i - 1).nbrePanneaux = 0
                                    Reste = 0.45 'pour avoir un panneau de 45cm minimum de large
                                    .TS_trav.Item(i - 1).LacierY.Add(Reste) 'pour les panneaux de modèle 2
                                Case Else 'reste compris entre 45cm et 240cm
                                    .TS_trav.Item(i - 1).nbrePanneaux = 0
                                    .TS_trav.Item(i - 1).LacierY.Add(Reste) 'pour les panneaux de modèle 2
                            End Select
                            .TS_trav.Item(i - 1).nbrePanneaux = .TS_trav.Item(i - 1).nbrePanneaux + CInt(Fix(.Ly / (Largeur - .TS_trav.Item(i - 1).RecouvTr)))
                        Else
                            'la largeur de la dalle est inférieure à celle du panneau de treillis soudé
                            .TS_trav.Item(i - 1).nbrePanneaux = 1
                            .TS_trav.Item(i - 1).LacierY.Clear()
                            .TS_trav.Item(i - 1).LacierY.Add(.Ly + 2 * .TS_trav.Item(i - 1).L31acier)
                        End If

                    Next
                Else
                    'pour les prédalles, ces dernières ont une largeur < ou = à la largeur du panneaux
                    'donc le nombre de panneaux est égal au nombre de prédalle
                    'Sera à modifier pour les prédalles de longueur > 6m
                    For i = 1 To dalle.Prefa_nb
                        Largeur = dalle.Prefa_Ly.Item(i - 1) 'aci.Larg_TS(.TS_trav.Item(i - 1).indTS) 'largeur du panneau TS
                        For j = 1 To .TS_trav.Count
                            If i = 1 Then .TS_trav.Item(j - 1).LacierY.Clear()
                            .TS_trav.Item(j - 1).LacierY.Add(Largeur) 'pour les panneaux de modèle 1
                        Next
                        'Pour les TS complémentaires
                        For j = 1 To .TS_trav_Portefeuille.Count
                            If i = 1 Then .TS_trav_Portefeuille.Item(j - 1).LacierY.Clear()
                            .TS_trav_Portefeuille.Item(j - 1).LacierY.Add(Largeur) 'pour les panneaux de modèle 1
                        Next
                    Next
                End If


                'Aciers supérieurs
                'calcul des longueurs totales des aciers sur appuis porteurs
                '
                Select Case PlanCont.IndexOf(dalle)
                    Case 0 'travée n°1
                        'Aciers sur appui gauche
                        If dalle.LappG > 0.05 And Aff.Reprise_M0_1_porteur = True Then
                            Select Case .Type_Acier_Appui_Gauche
                                Case 1 'TS
                                    For i = 1 To .TS_AppG.Count
                                        With .TS_AppG
                                            .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier
                                        End With
                                    Next
                                Case 2 'HA
                                    For i = 1 To .LitSupG.Count
                                        With .LitSupG
                                            .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier
                                        End With
                                    Next
                            End Select
                        End If
                        'Acier sur appui droit
                        If PlanCont.Count = 1 Then
                            Select Case .Type_Acier_Appui_Droit
                                Case 1 'TS
                                    For i = 1 To .TS_AppD.Count
                                        With .TS_AppD
                                            .Item(i - 1).Lacier2 = dalle.Lx - .Item(i - 1).X1acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                        End With
                                    Next
                                Case 2 'HA
                                    For i = 1 To .LitSupD.Count
                                        With .LitSupD
                                            .Item(i - 1).Lacier2 = dalle.Lx - .Item(i - 1).X1acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                        End With
                                    Next
                            End Select
                        End If
                    Case PlanCont.Count - 1 'dernière travée
                        'Appui gauche de la dernière travée
                        Select Case .Type_Acier_Appui_Gauche
                            Case 1 'TS
                                For i = 1 To .TS_AppG.Count
                                    With .TS_AppG
                                        .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                        With PlanCont.Item(PlanCont.IndexOf(dalle) - 1)
                                            .TS_AppD.Item(i - 1).Lacier2 = .Lx - .TS_AppD.Item(i - 1).X1acier
                                        End With
                                        .Item(i - 1).Lacier1 = PlanCont.Item(PlanCont.IndexOf(dalle) - 1).TS_AppD.Item(i - 1).Lacier2 + dalle.LappG + .Item(i - 1).Lacier2
                                        PlanCont.Item(PlanCont.IndexOf(dalle) - 1).TS_AppD.Item(i - 1).Lacier1 = .Item(i - 1).Lacier1
                                    End With
                                Next
                            Case 2 'HA
                                For i = 1 To .LitSupG.Count
                                    With .LitSupG
                                        .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                        With PlanCont.Item(PlanCont.IndexOf(dalle) - 1)
                                            .LitSupD.Item(i - 1).Lacier2 = .Lx - .LitSupD.Item(i - 1).X1acier
                                        End With
                                        .Item(i - 1).Lacier1 = PlanCont.Item(PlanCont.IndexOf(dalle) - 1).LitSupD.Item(i - 1).Lacier2 + dalle.LappG + .Item(i - 1).Lacier2
                                        PlanCont.Item(PlanCont.IndexOf(dalle) - 1).LitSupD.Item(i - 1).Lacier1 = .Item(i - 1).Lacier1
                                    End With
                                Next
                        End Select
                        'Appui droit de la dernière travée
                        If dalle.LappD > 0.05 And Aff.Reprise_M0_1_porteur = True Then
                            Select Case .Type_Acier_Appui_Droit
                                Case 1 'TS
                                    For i = 1 To .TS_AppD.Count
                                        With .TS_AppD
                                            .Item(i - 1).Lacier2 = dalle.Lx - .Item(i - 1).X1acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                        End With
                                    Next
                                Case 2 'HA
                                    For i = 1 To .LitSupD.Count
                                        With .LitSupD
                                            .Item(i - 1).Lacier2 = dalle.Lx - .Item(i - 1).X1acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                        End With
                                    Next
                            End Select
                        End If
                    Case Else 'appuis intermédiaires
                        Select Case .Type_Acier_Appui_Gauche
                            Case 1 'TS
                                For i = 1 To .TS_AppG.Count
                                    With .TS_AppG
                                        .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                        With PlanCont.Item(PlanCont.IndexOf(dalle) - 1)
                                            .TS_AppD.Item(i - 1).Lacier2 = .Lx - .TS_AppD.Item(i - 1).X1acier
                                        End With
                                        .Item(i - 1).Lacier1 = PlanCont.Item(PlanCont.IndexOf(dalle) - 1).TS_AppD.Item(i - 1).Lacier2 + dalle.LappG + .Item(i - 1).Lacier2
                                        PlanCont.Item(PlanCont.IndexOf(dalle) - 1).TS_AppD.Item(i - 1).Lacier1 = .Item(i - 1).Lacier1
                                    End With
                                Next
                            Case 2 'HA
                                For i = 1 To .LitSupG.Count
                                    With .LitSupG
                                        .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                        With PlanCont.Item(PlanCont.IndexOf(dalle) - 1)
                                            .LitSupD.Item(i - 1).Lacier2 = .Lx - .LitSupD.Item(i - 1).X1acier
                                        End With
                                        .Item(i - 1).Lacier1 = PlanCont.Item(PlanCont.IndexOf(dalle) - 1).LitSupD.Item(i - 1).Lacier2 + dalle.LappG + .Item(i - 1).Lacier2
                                        PlanCont.Item(PlanCont.IndexOf(dalle) - 1).LitSupD.Item(i - 1).Lacier1 = .Item(i - 1).Lacier1
                                    End With
                                Next
                        End Select
                End Select
                'Aciers supérieurs
                'calcul des longueurs totales des aciers sur appuis NON porteurs
                ' -- Sur appuis haut et bas
                Select Case .Type_Acier_Travée
                    Case 1 'TS
                        ' -- Sur appuis haut
                        If .TS_AppH.Count >= 1 Then
                            With .TS_AppH
                                .Item(0).Lacier2 = .Item(0).X2acier
                                .Item(0).Lacier1 = .Item(0).Lacier2 + .Item(0).L11acier + .Item(0).L12acier + .Item(0).L13acier
                            End With
                        End If
                        ' -- Sur appuis bas
                        If .TS_AppB.Count >= 1 Then
                            With .TS_AppB
                                .Item(0).Lacier2 = .Item(0).X2acier
                                .Item(0).Lacier1 = .Item(0).Lacier2 + .Item(0).L11acier + .Item(0).L12acier + .Item(0).L13acier
                            End With
                        End If
                    Case 2 'HA
                        ' -- Sur appuis haut
                        If .LitSupH.Count >= 1 Then
                            With .LitSupH
                                .Item(0).Lacier2 = .Item(0).X2acier
                                .Item(0).Lacier1 = .Item(0).Lacier2 + .Item(0).L11acier + .Item(0).L12acier + .Item(0).L13acier
                            End With
                        End If
                        ' -- Sur appuis bas
                        If .LitSupB.Count >= 1 Then
                            With .LitSupB
                                .Item(0).Lacier2 = .Item(0).X2acier
                                .Item(0).Lacier1 = .Item(0).Lacier2 + .Item(0).L11acier + .Item(0).L12acier + .Item(0).L13acier
                            End With
                        End If
                End Select
                '
                '--> Détermination du nombre de panneaux sur les appuis
                ' -- Sur appuis d'about et appuis intermédiaires
                Select Case .Type_Acier_Appui_Gauche
                    Case 1 'TS
                        If .LappB > 0.05 Then
                            For i = 1 To .TS_AppG.Count
                                .TS_AppG.Item(i - 1).LacierY.Clear()
                                Largeur = aci.Larg_TS(.TS_AppG.Item(i - 1).indTS)
                                .TS_AppG.Item(i - 1).LacierY.Add(Largeur)
                                Reste = .Ly Mod Largeur
                                .TS_AppG.Item(i - 1).nbrePanneaux = CInt(Fix(.Ly / Largeur))
                                If Reste < 1 Then
                                    Reste = 1
                                    .TS_AppG.Item(i - 1).LacierY.Add(Reste) ' 
                                Else
                                    .TS_AppG.Item(i - 1).LacierY.Add(Truncate(Reste * 100 + 1) / 100) 'arrondi au cm supérieur
                                End If
                            Next
                        End If
                    Case 2 'barres HA

                End Select
                'dernière travée
                Select Case .Type_Acier_Appui_Droit
                    Case 1 'TS
                        If .LappD > 0.05 Then
                            For i = 1 To .TS_AppD.Count
                                .TS_AppD.Item(i - 1).LacierY.Clear()
                                Largeur = aci.Larg_TS(.TS_AppD.Item(i - 1).indTS)
                                .TS_AppD.Item(i - 1).LacierY.Add(Largeur)
                                Reste = .Ly Mod Largeur
                                .TS_AppD.Item(i - 1).nbrePanneaux = CInt(Fix(.Ly / Largeur))
                                If Reste < 1 Then
                                    Reste = 1
                                    .TS_AppD.Item(i - 1).LacierY.Add(Reste) ' 
                                Else
                                    .TS_AppD.Item(i - 1).LacierY.Add(Truncate(Reste * 100 + 1) / 100) 'arrondi au cm supérieur
                                End If
                            Next
                        End If
                    Case 2 'barres HA

                End Select

                ' -- Sur appuis haut et bas
                If .TS_AppH.Count >= 1 Then
                    For i = 1 To .TS_AppH.Count
                        .TS_AppH.Item(i - 1).LacierY.Clear()
                        If .TS_AppH.Item(i - 1).indTS = 0 Then 'ST10 sélectionnée
                            Largeur = aci.Long_TS(.TS_AppH.Item(i - 1).indTS) 'soit 4.8
                        Else
                            Largeur = aci.Larg_TS(.TS_AppH.Item(i - 1).indTS) 'soit 2.4
                        End If
                        .TS_AppH.Item(i - 1).LacierY.Add(Largeur)
                        Reste = .Lx Mod Largeur
                        .TS_AppH.Item(i - 1).nbrePanneaux = CInt(Fix(.Lx / Largeur))
                        .TS_AppH.Item(i - 1).LacierY.Add(Truncate(Reste * 100 + 1) / 100) 'arrondi au cm supérieur
                        If .TS_AppH.Item(i - 1).LacierY.Item(1) < 0.6 Then .TS_AppH.Item(i - 1).LacierY.Item(1) = 0.6 'pour avoir un minimum de 2 mailles
                    Next
                End If
                If .TS_AppB.Count >= 1 Then
                    For i = 1 To .TS_AppB.Count
                        .TS_AppB.Item(i - 1).LacierY.Clear()
                        If .TS_AppB.Item(0).indTS = 0 Then 'ST10 sélectionnée
                            Largeur = aci.Long_TS(.TS_AppB.Item(i - 1).indTS) 'soit 4.8
                        Else
                            Largeur = aci.Larg_TS(.TS_AppB.Item(i - 1).indTS) 'soit 2.4
                        End If
                        .TS_AppB.Item(i - 1).LacierY.Add(Largeur)
                        Reste = .Lx Mod Largeur
                        .TS_AppB.Item(i - 1).nbrePanneaux = CInt(Fix(.Lx / Largeur))
                        .TS_AppB.Item(i - 1).LacierY.Add(Truncate(Reste * 100 + 1) / 100) 'arrondi au cm supérieur
                        If .TS_AppB.Item(i - 1).LacierY.Item(1) < 0.6 Then .TS_AppB.Item(i - 1).LacierY.Item(1) = 0.6 'pour avoir un minimum de 2 mailles
                    Next
                End If
            End With

        Next
    End Sub
    ''' <summary>
    '''Calcul du recouvrement longitudinal entre 2 TS dans le sens de la longueur
    ''' </summary>
    ''' <param name="Indice_TS">n° d'indice du TS - Commence à 0: i=0 pour TS ST10, i=1 pour TS ST20,  ...</param>
    ''' <param name="i_trav">n° de travée</param>
    Public Function Recouvrement_Long_TS(ByVal Indice_TS As Integer, Optional i_trav As Integer = 1) As Double
        '-----------------------------------------------------------------------
        '   longueurs d'ancrage droit total - EN 1992-1-1 A8.4.1 à A8.4.4 -  'longueur de scellement droit des aciers de 6 à 40 mm de diamètre
        '-----------------------------------------------------------------------
        'nlit = index du n° de lit de l'armature = n° de lit - 1 car base = 0 et non 1 !!!!
        'i_trav = n° de travée
        'ELU_ELS = 1 si ELU fondamental, =2 si Accidentel
        'σsd = contrainte sur l'acier, peut être supérieur à fyk / γs si utilisation branche inclinée
        'h = hauteur totale de la dalle
        'h1 = hauteur préfa de la dalle = prédalle - sert à déterminer qualité adhérence
        'enrLat = enrobage latéral
        'enrinf = enrobage inférieur ou supérieur
        'esp = espacement entre armature

        Dim Cd, esp, fbd, fctk, K, Lbd, Lbmin, Lbrqd, Prop_As, α1, α2, α3, α4, α6, η1, η2, γc, φbarre, σsd As Double ', φbarreTrans
        Try
            Recouvrement_Long_TS = 0 'initialisation
            With PlanCont.Item(i_trav - 1)
                σsd = 435 'σsd sur l'acier soit fe/1.15=435MPa
                'armature en travée
                φbarre = aci.Ølong(Indice_TS)
                'condition de bétonnage
                η1 = 0.7 'condition de bétonnage toujours bonne
                η2 = 1 'φbarre<32mm

                fctk = bet.fctk05
                If bet.fctk05 > 3.1 Then fctk = 3.1
                γc = bet.γcd ' : γs = aci.γsd 'combinaison fondamentale

                fbd = 2.25 * η1 * η2 * fctk / γc
                Lbrqd = (φbarre / 4) * (σsd / fbd)
                'forme des barres
                α1 = 1 'scellement droit
                'enrobage
                Cd = Min(.enrInf + 0.01, .enrLat)
                esp = aci.Esp_Long(Indice_TS)
                Cd = Min(esp / 2, Cd)
                α2 = 1 - 0.15 * (Cd - φbarre) / φbarre
                If α2 < 0.7 Then α2 = 0.7
                If α2 > 1 Then α2 = 1
                'Confinement des armatures
                α3 = 1 'armatures soudèes 
                'Confinement des armatures soudès
                α4 = 1 'application de 8.7.5.1
                α6 = 1.5 'je recouvre tout le panneaux
                Prop_As = (PI * aci.Ølong(Indice_TS) ^ 2 / 4) / aci.Esp_Long(Indice_TS) ' art. 8.7.5.1(6)
                Prop_As = Prop_As * 10000 'pour cm²/m
                If Prop_As > 12 Then
                    Aff.msg_Avertiss.Add("Travée n°" & i_trav.ToString & ": rapport As/s > 12cm²/m (=" & Prop_As.ToString("#0.0") & ") - recouvrement sur 2,15xl0")
                    K = 2.15 'pour tenir compte du décalage des TS
                Else
                    K = 1
                End If
                Lbd = α1 * α2 * α3 * α4 * α6 * Lbrqd '(eq 8.10)

            End With

            Lbmin = Max(0.3 * α6 * Lbrqd, 15 * φbarre)
            If Lbmin < 0.2 Then Lbmin = 0.2
            If Lbd < Lbmin Then Lbd = Lbmin
            Recouvrement_Long_TS = Truncate(K * Lbd * 100) / 100 'élimination des chiffres aprés le cm

        Catch ex As Exception
            MessageBox.Show("Erreur calcul" & Environment.NewLine & ex.Message, "Function Recouvrement_Long_TS", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Recouvrement_Long_TS = 0.01 'valeur par défaut  - pour pouvoir continuer le calcul
        End Try
    End Function
    ''' <summary>
    '''Calcul du recouvrement transversal entre 2 TS dans le sens de la largeur
    ''' </summary>
    ''' <param name="Indice_TS">n° d'indice du TS - Commence à 0: i=0 pour TS ST10, i=1 pour TS ST20,  ...</param>
    Private Function Recouvrement_Trans_TS(ByVal Indice_TS As Integer) As Double
        Recouvrement_Trans_TS = 0 'initialisation
        Try
            'suivant tableau 3 de la FT22 de l'ADETS
            Select Case Indice_TS'nom du TS, dans l'ordre: "ST 10", "ST 20", "ST 25", "ST 30", "ST 35", "ST 50", "ST 60", "ST 15C", "ST 25C", "ST 40C", "ST 50C", "ST 65C"
                Case 0, 7
                    Recouvrement_Trans_TS = 0.4
                Case 1, 2, 8
                    Recouvrement_Trans_TS = 0.45
                Case 3, 4, 5, 9, 10
                    Recouvrement_Trans_TS = 0.3
                Case 6, 11
                    Recouvrement_Trans_TS = 0.35
            End Select
            If Recouvrement_Long_TS(Indice_TS) < Recouvrement_Trans_TS Then Recouvrement_Trans_TS = Recouvrement_Long_TS(Indice_TS)
        Catch ex As Exception
            MessageBox.Show("Erreur calcul" & Environment.NewLine & ex.Message, "Function Recouvrement_Trans_TS", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Function
    Public Function Indice_Acier(ByVal phi As Double) As Integer
        Indice_Acier = 0 'initialisation
        Try
            Indice_Acier = aci.Ø_Norm.IndexOf(phi)
        Catch ex As Exception
            MessageBox.Show("Erreur sur recherche diamètre acier" & Environment.NewLine & ex.Message, "Function Indice_Acier ", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End Try
    End Function

#End Region

#Region "Code BA pour plaque sur 4 appuis"
    ''' <summary>Programme général de calcul Plancher continue en béton armé suivant l'Eurocode 2 portant sur 4 appuis</summary>
    '''  <param name="rtf">variable renvoyant le fichier de note de calcul sous format rtf</param>
    ''' <param name="verif"> variable prenant la valeur False si une erreur bloquante s'est produite pendant le calcul</param>
    Public Sub calcul_EC2_4_Appuis(ByRef rtf As To_Rtf.RtfAnsiTextWriter, ByRef verif As Boolean)
        '   
        'Rappel:
        'Les efforts sont adressés en MN et MN.m
        Dim i, tour, Ind1(4), Ind2(4), Num_Chap As Integer ' imax,
        Dim msg As String = String.Empty
        Dim Maitrise_Fissuration As Boolean
        ''-------------------------------------------------------------------------------
        ''   PROGRAMME PRINCIPAL :   Calcul des sections d'aciers et des contraintes
        ''--------------------------------------------------------------------------------   
        ''----->ELU Fondamental:
        ''-----> calcul des aciers de flexion (détermination des armatures et épure),
        ''-----> calcul et vérification des efforts sur appui (bielle, acier de glissement et entrainement)
        rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.NewLine() : rtf.Bold = True
        rtf.Underline = True : rtf.WriteString("3 - Détermination des armatures de flexion") : rtf.Bold = False : rtf.FontSize = 10 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("3 - 1  Calcul des sections d'acier") : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("Les calculs sont réalisés avec les modélisations suivantes:") : rtf.NewLine()
        rtf.WriteString("  - Diagramme simplifié rectangulaire des contraintes de compression du béton suivant art. 3.1.7(3) - figure 3.5") : rtf.NewLine()
        rtf.WriteString("  - Diagramme élasto-plastique parfait pour l'acier suivant art. 3.2.7(2) avec prise en compte éventuelle d'une branche supérieure inclinée suivant " &
                        "valeur  de la déformation de l'acier (voir tableau calcul ci-dessous).") : rtf.NewLine()

        If PlanCont(0).Type_Acier_Travée = 2 Then 'acier HA
            For Each dalle As Plancher In PlanCont
                With dalle
                    Dim EspMaxSecond As Double
                    Select Case dalle.Choix_Esp
                        Case 1 'zone standard
                            If 3 * .H > 0.4 Then .EspMax = 0.4 Else .EspMax = 3 * .H
                            If 3.5 * .H > 0.45 Then EspMaxSecond = 0.45 Else EspMaxSecond = 3.5 * .H
                        Case 2 'zone charge concentrée
                            If 2 * .H > 0.25 Then .EspMax = 0.25 Else .EspMax = 2 * .H
                            If 3 * .H > 0.4 Then EspMaxSecond = 0.4 Else EspMaxSecond = 3 * .H
                        Case 3 'esp max imposé
                            ' .EspMaxPrincipal = .EspMax = déjà renseigné dans le module Ferraillage
                            EspMaxSecond = .EspMax
                    End Select
                    .EspMaxPrincipal = .EspMax : .EspminPrincipal = .Espmin
                    .EspMaxSecondaire = EspMaxSecond : .EspminSecondaire = .Espmin
                End With
            Next
            rtf.WriteString("  - Espacement minimal pour acier principal = " & (PlanCont(0).EspminPrincipal * 100).ToString("#.0") & " cm - Espacement Maximal pour acier principal = " & (PlanCont(0).EspMaxPrincipal * 100).ToString("#.0")) : rtf.NewLine()
            rtf.WriteString("  - Espacement minimal pour acier secondaire = " & (PlanCont(0).EspminSecondaire * 100).ToString("#.0") & " cm - Espacement Maximal pour acier secondaire = " & (PlanCont(0).EspMaxSecondaire * 100).ToString("#.0") & " cm.") : rtf.NewLine()
        End If
        rtf.WriteEndGroup()

        Call Calcul_section_ELU_Fondamental_4_Appuis(verif, rtf)
        If verif = False Then Exit Sub
        '______________________________________________________________________________________________________________________________________________________________________________________
        ''--->ELS: vérification  à l'ELS
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.NewLine() : rtf.Bold = True
        rtf.Underline = True : rtf.WriteString("4 - Vérifications à l'Etat Limite de Service") : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
        '______________________________________________________________________________________________________________________________________________________________________________________
        '----->Calcul des contraintes
        rtf.WriteString("4 - 1  Limitation des contraintes") : rtf.Underline = False : rtf.NewLine()
        rtf.FontSize = 9 : rtf.NewLine()
        i = 0
        For Each dalle As Plancher In PlanCont
            With dalle
                If .ELS_Car.Count = 0 Then
                    rtf.WriteString(" - Travée n°" & (PlanCont.IndexOf(dalle) + 1).ToString & ": Aucune combinaison ELS caractéristique de charges n'a été renseignée - La verification de la limitation des contraintes n'est pas établie.") : rtf.NewLine() : rtf.NewLine()
                    i += 1
                Else
                    If .XD1 = True Or .XD2 = True Or .XD3 = True Or .XS1 = True Then
                        rtf.WriteString(" - Travée n°" & (PlanCont.IndexOf(dalle) + 1).ToString & ": σb ≤ 0,6.fck - Classe d'exposition XD") : rtf.NewLine() : i += 1
                    ElseIf .XS2 = True Or .XS3 = True Then
                        rtf.WriteString(" - Travée n°" & (PlanCont.IndexOf(dalle) + 1).ToString & ": σb ≤ 0,6.fck - Classe d'exposition XS") : rtf.NewLine() : i += 1
                    ElseIf .XF1 = True Or .XF2 = True Or .XF3 = True Or .XF4 = True Then
                        rtf.WriteString(" - Travée n°" & (PlanCont.IndexOf(dalle) + 1).ToString & ": σb ≤ 0,6.fck - Classe d'exposition XF") : rtf.NewLine() : i += 1
                    End If
                End If
            End With
        Next
        If i < 1 Then rtf.WriteString("- Toutes travées : σs ≤ 0,8.fyk") : rtf.NewLine()
        rtf.WriteEndGroup()
        Call Calcul_section_ELS_4appuis(rtf)

        '______________________________________________________________________________________________________________________________________________________________________________________
        '----->Maitrise de la fissuration
        rtf.WriteStartGroup()
        rtf.NewParagraph() : rtf.FontSize = 12
        rtf.Underline = True : rtf.WriteString("4 - 2  Maitrise de la fissuration") : rtf.Underline = False : rtf.NewLine()
        rtf.WriteEndGroup()

        If Aff.ELSQP = 0 Then
            rtf.WriteStartGroup() : rtf.FontSize = 9 : rtf.NewLine()
            rtf.WriteString("Aucune combinaison ELS quasi-permanente de charges n'a été renseignée - La verification de la fissuration n'est pas établie.") : rtf.NewLine() : rtf.NewLine()
            rtf.WriteEndGroup()
        Else
            Maitrise_Fissuration = False : tour = 1
            For Each Dalle As Plancher In PlanCont
                With Dalle
                    If .X0 = True Or .XC1 = True Then
                        rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                        If .X0 = True Then msg = "X0" Else msg = "XC1"
                        rtf.WriteString("Travée n°" & (PlanCont.IndexOf(Dalle) + 1).ToString & ":") : rtf.NewLine()
                        rtf.WriteString("Travée classée" & msg & " - pas de calcul de fissuration demandé, les dispositions constructives suffisent à assurer la maitrise de la fissuration.") : rtf.NewLine()
                        rtf.WriteString("Limite de la fissuration Wmax = 0,40 mm.") : rtf.NewLine()
                        rtf.WriteEndGroup()
                    ElseIf .XC2 = True Or .XC3 = True Or .XC4 = True Then
                        For Each chges As Charge In Dalle.Chge
                            If chges.Nature = 2 Or chges.Nature = 3 Then 'charges catégories A,B,C et D
                                rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                                If .XC2 = True Then
                                    msg = "XC2"
                                ElseIf .XC3 Then
                                    msg = "XC3"
                                Else
                                    msg = "XC4"
                                End If
                                rtf.WriteString("Travée n°" & (PlanCont.IndexOf(Dalle) + 1).ToString & ":") : rtf.NewLine()
                                rtf.WriteString("Travée classée" & msg & " - pas de calcul de fissuration demandé car les charges d'exploitation appartiennent à la catégorie A à D. " &
                                                "Suivant l'Annexe Nationale, les dispositions constructives suffisent à assurer la maitrise de la fissuration.") : rtf.NewLine()
                                rtf.WriteString("Limite de la fissuration Wmax = 0,30 mm.") : rtf.NewLine()
                                rtf.WriteEndGroup()
                                Maitrise_Fissuration = True : Exit For
                            End If
                        Next
                        If Maitrise_Fissuration = False Then

                            rtf.WriteStartGroup() : rtf.FontSize = 9 : rtf.NewLine()
                            rtf.WriteString("Travée n°" & (PlanCont.IndexOf(Dalle) + 1).ToString & ":") : rtf.NewLine()
                            rtf.WriteEndGroup()
                            ' Prefa_Fissuration = False
                            'If .Prefa = True Then
                            '    Call Fissuration_Predalle(Dalle, Mmax, Mmin, m1d, m2d, m3d, rtf, Prefa_Fissuration)
                            'End If
                            If tour = 1 Then 'And Prefa_Fissuration = False
                                rtf.WriteString("Détermination de la fissuration par application du schéma suivant:") : rtf.NewLine()
                                rtf.WriteString("1 - Vérification de la section minimale d'armature - Application de l'article 7.3.2") : rtf.NewLine()
                                rtf.WriteString("2 - Vérification de l'ouverture des fissures soit sans calcul direct soit avec calcul direct") : rtf.NewLine()
                            End If
                            rtf.WriteEndGroup()
                            Call Fissuration_verification_forfaitaire(tour, PlanCont.IndexOf(Dalle) + 1, rtf, verif, Maitrise_Fissuration)
                            If Maitrise_Fissuration = False Then
                                rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                                rtf.NewLine() : rtf.WriteString("La méthode forfaitaire de maitrise de l'ouverture des fissures n'étant pas concluante, il est procédé au calcul direct de l'ouverture des fissures suivant Art. 7.3.4") : rtf.NewLine() : rtf.NewLine()
                                rtf.WriteEndGroup()
                                Call Fissuration_Calcul_Direct_4appuis(PlanCont.IndexOf(Dalle) + 1, rtf, verif, Maitrise_Fissuration)
                            End If
                            tour = tour + 1
                        End If
                    ElseIf .XD1 = True Or .XD2 = True Or .XD3 = True Or .XS1 = True Or .XS2 = True Or .XS3 = True Then
                        'Prefa_Fissuration = False
                        'If .Prefa = True Then
                        '    Call Fissuration_Predalle(Dalle, Mmax, Mmin, m1d, m2d, m3d, rtf, Prefa_Fissuration)
                        'End If
                        'If Prefa_Fissuration = False Then
                        rtf.WriteStartGroup() : rtf.FontSize = 9 : rtf.NewLine()
                        rtf.WriteString("Détermination de la fissuration par application du schéma suivant:") : rtf.NewLine()
                        rtf.WriteString("1 - Vérification de la section minimale d'armature - Application de l'article 7.3.2") : rtf.NewLine()
                        rtf.WriteString("2 - Vérification de l'ouverture des fissures soit sans calcul direct soit avec calcul direct") : rtf.NewLine()
                        rtf.WriteEndGroup()
                        Call Fissuration_verification_forfaitaire(tour, PlanCont.IndexOf(Dalle) + 1, rtf, verif, Maitrise_Fissuration)
                        If Maitrise_Fissuration = False Then
                            rtf.WriteStartGroup()
                            rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                            rtf.NewLine() : rtf.WriteString("La méthode forfaitaire de maitrise de l'ouverture des fissures n'étant pas concluante, il est procédé au calcul direct de l'ouverture des fissures suivant Art. 7.3.4") : rtf.NewLine() : rtf.NewLine()
                            rtf.WriteEndGroup()
                            Call Fissuration_Calcul_Direct_4appuis(PlanCont.IndexOf(Dalle) + 1, rtf, verif, Maitrise_Fissuration)
                        End If
                        'End If
                    End If
                End With
            Next
        End If
        '______________________________________________________________________________________________________________________________________________________________________________________
        '----->Calcul des flèches
        rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.Underline = True
        rtf.WriteString("4 - 3  Détermination des flèches") : rtf.Underline = False : rtf.NewLine() : rtf.FontSize = 9
        rtf.WriteString("Détermination des conditions limites de flèches par le calcul des élancements suivant les dispositions de l'EN1992-1-1 §7.4.2") : rtf.NewLine()
        rtf.WriteString("Si nécessité de calcul, réalisé suivant les dispositions de l'EN1992-1-1 §7.4.3") : rtf.NewLine()
        rtf.WriteEndGroup()
        Dim Fleche(PlanCont.Count) As Boolean '= True
        Call Dispense_Calcul_Fleche(2, Fleche, rtf)
        Call Calcul_Fleche_4appuis(Fleche, rtf) 'calcul flèche poutre
        For i = 1 To PlanCont.Count
            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
            If Fleche(i) = False Then
                rtf.WriteString("Les élancements dépassent les limites autorisées par l'Eurocode, le calcul explicite des flèches est exigé.") : rtf.NewLine()
                If Aff.ELSQP = 0 Then
                    rtf.WriteStartGroup() : rtf.FontSize = 9 : rtf.NewLine()
                    rtf.WriteString("Aucune combinaison ELS quasi-permanente de charges n'a été renseignée - La verification vis à vis de la flèche n'est pas établie.") : rtf.NewLine() : rtf.NewLine()
                    rtf.WriteEndGroup()
                Else

                    rtf.WriteEndGroup()
                    ' 4 = combinaison ELS quasi-permanente, Ind1(4)=1ier numéro de combinaison, Ind2(4)= dernier n° de combinaison

                End If
            Else
                rtf.WriteString("Les élancements ne dépassent pas les limites autorisées par l'Eurocode, les conditions de flèche sont vérifiées.") : rtf.NewLine()
                rtf.WriteEndGroup()
            End If
        Next

        Num_Chap = 5
        '______________________________________________________________________________________________________________________________________________________________________________________
        '----->ELU Effort Tranchant: détermination des cadres
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.NewLine() : rtf.Bold = True
        rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - Dimensionnement de la section de béton et des armatures à l'Effort Tranchant") : rtf.Bold = False : rtf.FontSize = 11 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString(Num_Chap.ToString & " - 1  Clauses générales") : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("Hypothèses générales prises pour le calcul de l'ensemble des travées:") : rtf.NewLine()
        rtf.WriteString("  ■ Angle Θ de la bielle béton comprimé: 45° soit Cot(Θ)= 1") : rtf.NewLine()
        rtf.WriteString("  ■ Angle α des armatures d'effort tranchant (si existe): 90° soit Cot(α)= 0") : rtf.NewLine()
        rtf.WriteString("  ■ [EC2, 6.2.3(5)] - Décalage de l'Effort Tranchant: ")
        If Aff.Decalage_ET = False Then
            rtf.WriteString("Non appliqué") : rtf.NewLine()
        Else
            rtf.WriteString("Pris en compte") : rtf.NewLine()
        End If
        rtf.WriteString("  ■ Minoration pour les charges appliquées près de l'appui pour le calcul de l'Effort Tranchant suivant conditions de l'article  6.2.1(8) de l'EN1992-1-1:  ")
        If Aff.MinorationQ_ET = False Then
            rtf.WriteString("Non appliqué") : rtf.NewLine()
        Else
            rtf.WriteString("Non appliqué") : rtf.NewLine() ' rtf.WriteString("Pris en compte") : rtf.NewLine()  provisoire
        End If
        rtf.WriteString("  ■ Espacement 1ier cadre: St/2") : rtf.NewLine()

        rtf.WriteString("  ■ Ø Maxi de l'armature d'effort tranchant =  Ø de l'armature longitudinale") : rtf.NewLine()
        rtf.WriteEndGroup()
        Call Effort_Tranchant_4appuis(Num_Chap, verif, rtf) : If verif = False Then Exit Sub
        '---------------> Vérification des appuis
        Num_Chap = Num_Chap + 1
        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.Bold = True : rtf.Underline = True
        rtf.WriteString(Num_Chap.ToString & " - Vérification des appuis") : rtf.Bold = False : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteEndGroup()
        Call verification_appui_Bidirectionnel(rtf)
        '---------------> Vérification des trémies
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.NewLine() : rtf.Bold = True : Num_Chap = Num_Chap + 1
        rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - Vérification des trémies") : rtf.Bold = False : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        i = 0
        For Each Dalle As Plancher In PlanCont
            With Dalle
                i = i + ._Tremie.Count
            End With
        Next
        If i = 0 Then
            rtf.WriteString("Aucune trémie") : rtf.NewLine()
            rtf.WriteEndGroup()
        Else
            rtf.WriteEndGroup()
            Select Case PlanCont.Item(0).Prefa
                Case True
                    ' Call Calcul_Tremie_Predalle(rtf)
                Case False
                    Call Calcul_Tremie_DallePleine(rtf)
            End Select
        End If
        '---------------> calcul au feu
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.NewLine() : rtf.Bold = True : Num_Chap = Num_Chap + 1
        rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - Résistance au feu") : rtf.Bold = False : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteEndGroup()
        Call Calcul_au_feu_4appuis(rtf) 'Vérification de la stabilité au feu
        '
        '---------------> calcul du métré
        Call Metre_4appuis()
        '
        '---------------> Affichage des avertissements
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.NewLine() : rtf.Bold = True : Num_Chap = Num_Chap + 1
        rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - Avertissements") : rtf.Bold = False : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        If Aff.msg_Avertiss.Count > 0 Then
            Dim i1 As Integer
            For i1 = 0 To Aff.msg_Avertiss.Count - 1
                rtf.WriteString(Aff.msg_Avertiss.Item(i1).ToString()) : rtf.NewLine()
            Next
        Else
            rtf.WriteString("Aucun avertissement.") : rtf.NewLine()
        End If
        rtf.WriteEndGroup()
        '
        '---------------> Affichage des plans dans la note de calcul
        rtf.WriteStartGroup()
        rtf.NewParagraph() : rtf.FontSize = 12 : rtf.Bold = True : Num_Chap = Num_Chap + 1
        rtf.Underline = True : rtf.WriteString(Num_Chap.ToString & " - Plans") : rtf.Bold = False : rtf.Underline = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("Vous trouverez ci-après les plans d'exécution des ouvrages établis suivant la note de calcul.") : rtf.NewLine()
        rtf.WriteString("Attention: ces plans ne sont pas à l'échelle. Ils servent simplement à illustrer la note de calcul.") : rtf.NewLine()
        rtf.WriteString("Les plans à l'échelle doivent être tirés directement depuis le logiciel via la commande imprimer.") : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("Le dossier de plans se compose de:") : rtf.NewLine() : rtf.NewLine()
        rtf.WriteString("   → Plan Ferraillage - Armatures basses") : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("   → Plan Ferraillage - Armatures hautes") : rtf.NewLine() : rtf.NewLine()
        rtf.WriteEndGroup()
        '
        'affichage des différents plans
        rtf.WriteStartGroup()

        Try
            Dim myImage As Image
            Dim g As Graphics
            '
            'dessin ferraillage inférieur
            myImage = My.Resources.Test1250_890 'Image.FromStream(Img)
            g = Graphics.FromImage(myImage)
            'dessin du ferraillage des armatures inférieures
            Call Affichage_Coffrage_Ferraillage.Affichage_Dalle_Pleine(g, 2, 1, 1250, 890)
            myImage.Save(Directory.GetCurrentDirectory.ToString() & "\test1.bmp") ', ImageFormat.Bmp)
            g.Dispose()
            myImage.RotateFlip(RotateFlipType.Rotate90FlipNone)
            rtf.InsertImage(myImage)
            myImage.Dispose()
            '
            'dessin ferraillage supérieur
            ' msg = "\Images\Test1100_890.png"
            '  Img = New FileStream(Directory.GetCurrentDirectory.ToString() & msg, FileMode.Open)
            myImage = My.Resources.Test1250_890 'Image.FromStream(Img)
            g = Graphics.FromImage(myImage)
            Call Affichage_Coffrage_Ferraillage.Affichage_Dalle_Pleine(g, 2, 2, 1250, 890)
            myImage.Save(Directory.GetCurrentDirectory.ToString() & "\test1.bmp") ', ImageFormat.Bmp)
            g.Dispose()
            myImage.RotateFlip(RotateFlipType.Rotate90FlipNone)
            rtf.InsertImage(myImage)
            ''myImage = Image.FromFile(Directory.GetCurrentDirectory.ToString() & msg)
            'Img.Close()
            My.Computer.FileSystem.DeleteFile(Directory.GetCurrentDirectory.ToString() & "\test1.bmp") 'effacement du fichier
        Catch ex As Exception
            msg = "Le logiciel n'a pas pu charger l'image correspondante." & vbCrLf & "Erreur: " & ex.Message & ex.StackTrace & vbCrLf & "Note de calcul sans image"
            MessageBox.Show(msg, "Plancher BA EC2", MessageBoxButtons.AbortRetryIgnore, MessageBoxIcon.Exclamation)
        End Try
        rtf.WriteEndGroup()
    End Sub
    ''' <summary>
    '''----->ELU Fondamental:
    '''-----> calcul des aciers de flexion (détermination des armatures et épure),
    '''-----> calcul et vérification des efforts sur appui (bielle, acier de glissement et entrainement)
    ''' </summary>
    Private Sub Calcul_section_ELU_Fondamental_4_Appuis(ByRef Verif As Boolean, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        '----->ELU Fondamental:
        '-----> calcul des aciers de flexion (détermination des armatures et épure),
        '-----> calcul et vérification des efforts sur appui (bielle, acier de glissement et entrainement)
        'variables: i1 = n° de travée
        '           courbe_moment = type de courbe du moment enveloppe
        '           x0(ELU_ELS, nbtrav, 2) = point d'intersection entre la courbe enveloppe et l'axe des abscisses
        '                                    indique la valeur de l'abscisse pour un moment nul
        Dim AsteelX, AsteelY, d, Mu As Double 'Ac, Amin,lappui,
        Dim i, i1, j, Message As Integer ' 
        Dim Ainf(PlanCont.Count, 4), AinfC(PlanCont.Count, 4), str(22), σsd(PlanCont.Count, 4) As Double

        ''Dim ferraillage_sup(nbtrav) As Integer
        Dim Fer As New Ferraillage_HA
        ' ELU_ELS = 1 ' = ELU STR
        For i1 = 1 To PlanCont.Count
            Call Valeur_Decalage_Moment(i1)
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.Underline = True : rtf.WriteString("- Travée n°" & i1.ToString) : rtf.Underline = False : rtf.NewLine() : rtf.WriteEndGroup()
            With PlanCont.Item(i1 - 1)
                For j = 1 To 4
                    Select Case j
                        Case 1 'acier inférieur longitudinal en travée
                            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                            rtf.Bold = True : rtf.WriteString("  ■ Armatures inférieures en travée: ") : rtf.Bold = False : rtf.NewLine() : rtf.WriteEndGroup()
                            '-- suivant X
                            'Calcul de l'acier pour le moment maxi en travée x = ELU_Str(0).Mx_Max.x
                            d = .H - .enrInf - 0.007     'les aciers suivant X sont au plus prés de la face
                            Select Case .Repartition
                                Case 1 'répartition uniforme
                                    Call Calcul_ELU_armature_flexion(1, i1, j, .ELU_Str_Env.Mx_max.M, d, Ainf, AinfC(i1, j), σsd(i1, j), str, Message, Verif)
                                    If Verif = False Then Exit Sub
                                    If AinfC(i1, j) = -1000 Then AsteelX = Ainf(i1, j) Else AsteelX = AinfC(i1, j) 'prise en compte soit de l'acier sans la branche inclinée soit de l'acier avec la branche inclinée
                                    'affichage des calculs suivant X
                                    Call Tableau_ELU_Armature_Flexion_4appuis(1, .ELU_Str_Env.Mx_max.M, i1, j, 1, .ELU_Str_Env.Mx_max.x, .ELU_Str_Env.Mx_max.y, d, str, Message, rtf)
                                Case 2 'répartition variable
                                    For i = 1 To .ELU_Str_Env.Mx_Interval_min.Count 'même nombre d'occurence pour Min et Max
                                        With .ELU_Str_Env.Mx_Interval_Max.Item(i - 1)
                                            Call Calcul_ELU_armature_flexion(1, i1, j, .M, d, Ainf, AinfC(i1, j), σsd(i1, j), str, Message, Verif)
                                            If Verif = False Then Exit Sub
                                            If AinfC(i1, j) = -1000 Then AsteelX = Ainf(i1, j) Else AsteelX = AinfC(i1, j)
                                            'affichage des calculs suivant X
                                            Call Tableau_ELU_Armature_Flexion_4appuis_Interval(i, .M, i1, j, 1, .x, .y, i, str, Message, rtf)
                                        End With
                                    Next
                            End Select
                            '-- suivant Y
                            d = .H - .enrInf - 0.014    'les aciers suivant Y sont au dessus de ceux suivant X
                            Select Case .Repartition
                                Case 1 'répartition uniforme
                                    Call Calcul_ELU_armature_flexion(1, i1, j, .ELU_Str_Env.My_Max.M, d, Ainf, AinfC(i1, j), σsd(i1, j), str, Message, Verif)
                                    If Verif = False Then Exit Sub
                                    If AinfC(i1, j) = -1000 Then AsteelY = Ainf(i1, j) Else AsteelY = AinfC(i1, j)
                                    'affichage des calculs suivant Y
                                    Call Tableau_ELU_Armature_Flexion_4appuis(1, .ELU_Str_Env.My_Max.M, i1, j, 2, .ELU_Str_Env.My_Max.x, .ELU_Str_Env.My_Max.y, d, str, Message, rtf)
                                Case 2 'répartition variable
                                    For i = 1 To .ELU_Str_Env.My_Interval_min.Count 'même nombre d'occurence pour Min et Max
                                        With .ELU_Str_Env.My_Interval_Max.Item(i - 1)
                                            Call Calcul_ELU_armature_flexion(1, i1, j, .M, d, Ainf, AinfC(i1, j), σsd(i1, j), str, Message, Verif)
                                            If Verif = False Then Exit Sub
                                            If AinfC(i1, j) = -1000 Then AsteelY = Ainf(i1, j) Else AsteelY = AinfC(i1, j)
                                            'affichage des calculs suivant Y
                                            Call Tableau_ELU_Armature_Flexion_4appuis(i, .M, i1, j, 2, .x, .y, d, str, Message, rtf)
                                        End With
                                    Next
                            End Select
                            'détermination des armatures
                            If .Type_Acier_Travée = 1 Then
                                ' Call Aciers_longitudinaux_par_lit_TS_4appuis(1, i1, j, AsteelX, AsteelY, d, str, Verif)
                                Call Aciers_longitudinaux_par_lit_HA_4appuis(1, i1, j, AsteelX, AsteelY, d, Verif)
                            Else
                                Call Aciers_longitudinaux_par_lit_HA_4appuis(1, i1, j, AsteelX, AsteelY, d, Verif)
                            End If
                            If Verif = False Then Exit Sub
                            .dInf = d
                            Call Ferraillage_Pratique_4Appuis(i1, j, rtf)
                        Case 2 ' acier supérieur longitudinal sur appui gauche
                            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                            rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui gauche:") : rtf.Bold = False : rtf.NewLine()
                            d = .H - .enrSup - 0.007
                            Select Case .Repartition
                                Case 1 'répartition uniforme
                                    If i1 = 1 Then
                                        rtf.WriteString("Reprise moment forfaitaire = 0,15.Mt Max Travée (appui de rive)") : rtf.NewLine() 'traitement des appuis d'about
                                        Mu = 0.15 * .ELU_Str_Env.Mx_max.M
                                    Else
                                        rtf.WriteString("Reprise moment forfaitaire = 0,25.Mt Max Travée") : rtf.NewLine()
                                        Mu = 0.25 * .ELU_Str_Env.Mx_max.M
                                    End If
                                    rtf.WriteEndGroup()
                                    Call Calcul_ELU_armature_flexion(1, i1, j, Mu, d, Ainf, AinfC(i1, j), σsd(i1, j), str, Message, Verif)
                                    If Verif = False Then rtf.WriteEndGroup() : Exit Sub
                                    If AinfC(i1, j) = -1000 Then AsteelX = Ainf(i1, j) Else AsteelX = AinfC(i1, j)
                                Case 2 'répartition par intervalle
                                    If i1 = 1 Then
                                        rtf.WriteString("Reprise moment forfaitaire = 0,15.Mt Max Travée") : rtf.NewLine() 'traitement des appuis d'about
                                        Mu = 0.15 * .ELU_Str_Env.Mx_Interval_Max.Item(0).M 'Provisoire
                                    Else
                                        rtf.WriteString("Reprise moment forfaitaire = 0,25.Mt Max Travée") : rtf.NewLine()
                                        Mu = 0.25 * .ELU_Str_Env.Mx_Interval_Max.Item(0).M 'Provisoire
                                    End If
                                    rtf.WriteEndGroup()
                                    Call Calcul_ELU_armature_flexion(1, i1, j, Mu, d, Ainf, AinfC(i1, j), σsd(i1, j), str, Message, Verif)
                                    If Verif = False Then rtf.WriteEndGroup() : Exit Sub
                                    If AinfC(i1, j) = -1000 Then AsteelX = Ainf(i1, j) Else AsteelX = AinfC(i1, j)
                            End Select

                            If i1 > 1 Then
                                Select Case .Type_Acier_Appui_Gauche
                                    Case 1 'TS
                                        Call Aciers_longitudinaux_par_lit_TS_4appuis(1, i1, j, AsteelX, 0, d, str, Verif) : If Verif = False Then Exit Sub
                                        If aci.nomTS(.TS_AppG.Item(0).indTS) = aci.nomTS(PlanCont(i1 - 2).TS_AppD.Item(0).indTS) And .TS_AppG.Count = PlanCont(i1 - 2).TS_AppD.Count Then
                                            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                                            rtf.WriteString("Ferraillage identique avec le calcul de l'appui droit de la travée précédente car données identiques - Pour les détails du calcul, voir tableau de la travée précédente.") : rtf.NewLine()
                                            rtf.WriteEndGroup()
                                        Else
                                            Call Tableau_ELU_Armature_Flexion_4appuis(1, Mu, i1, j, 1, 0, 0, d, str, Message, rtf)
                                            Call Ferraillage_Pratique_4Appuis(i1, j, rtf)
                                        End If
                                    Case 2 'quadrillage barres HA
                                        Call Aciers_longitudinaux_par_lit_HA_4appuis(1, i1, j, AsteelX, 0, d, Verif) : If Verif = False Then Exit Sub
                                        If .LitSupG.Item(0).nbar = PlanCont(i1 - 2).LitSupD.Item(0).nbar And .LitSupG.Item(0).φHA = PlanCont(i1 - 2).LitSupD.Item(0).φHA Then
                                            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                                            rtf.WriteString("Ferraillage identique avec le calcul de l'appui droit de la travée précédente car données identiques - Pour les détails du calcul, voir tableau de la travée précédente.") : rtf.NewLine()
                                            rtf.WriteEndGroup()
                                        Else
                                            Call Tableau_ELU_Armature_Flexion_4appuis(1, Mu, i1, j, 1, 0, 0, d, str, Message, rtf)
                                            Call Ferraillage_Pratique_4Appuis(i1, j, rtf)
                                        End If
                                End Select

                            Else
                                Select Case .Type_Acier_Appui_Gauche
                                    Case 1 'TS
                                        Call Aciers_longitudinaux_par_lit_TS_4appuis(1, i1, j, AsteelX, 0, d, str, Verif)
                                    Case 2 'quadrillage barres HA
                                        Call Aciers_longitudinaux_par_lit_HA_4appuis(1, i1, j, AsteelX, 0, d, Verif)
                                End Select
                                If Verif = False Then rtf.WriteEndGroup() : Exit Sub
                                .dSupG = d
                                Call Tableau_ELU_Armature_Flexion_4appuis(1, Mu, i1, j, 1, 0, 0, d, str, Message, rtf)
                                Call Ferraillage_Pratique_4Appuis(i1, j, rtf)

                            End If
                        Case 3 ' acier supérieur longitudinal sur appui droit
                            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                            rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui droit:") : rtf.Bold = False : rtf.NewLine()
                            d = .H - .enrSup - 0.007
                            Select Case .Repartition
                                Case 1 'répartition uniforme
                                    If i1 = PlanCont.Count Then
                                        rtf.WriteString("Reprise moment forfaitaire = 0,15.Mt Max Travée") : rtf.NewLine() 'traitement des appuis d'about
                                        Mu = 0.15 * .ELU_Str_Env.Mx_max.M
                                    Else
                                        rtf.WriteString("Reprise moment forfaitaire = 0,25.Mt Max Travée") : rtf.NewLine()
                                        Mu = 0.25 * .ELU_Str_Env.Mx_max.M
                                    End If
                                    rtf.WriteEndGroup()
                                    Call Calcul_ELU_armature_flexion(1, i1, j, Mu, d, Ainf, AinfC(i1, j), σsd(i1, j), str, Message, Verif)
                                    If Verif = False Then rtf.WriteEndGroup() : Exit Sub
                                    If AinfC(i1, j) = -1000 Then AsteelX = Ainf(i1, j) Else AsteelX = AinfC(i1, j)
                                Case 2 'répartition par intervalle
                                    If i1 = PlanCont.Count Then
                                        rtf.WriteString("Reprise moment forfaitaire = 0,15.Mt Max Travée") : rtf.NewLine() 'traitement des appuis d'about
                                        Mu = 0.15 * .ELU_Str_Env.Mx_Interval_Max.Item(0).M 'Provisoire
                                    Else
                                        rtf.WriteString("Reprise moment forfaitaire = 0,25.Mt Max Travée") : rtf.NewLine()
                                        Mu = 0.25 * .ELU_Str_Env.Mx_Interval_Max.Item(0).M 'Provisoire
                                    End If
                                    rtf.WriteEndGroup()
                                    Call Calcul_ELU_armature_flexion(1, i1, j, Mu, d, Ainf, AinfC(i1, j), σsd(i1, j), str, Message, Verif)
                                    If Verif = False Then rtf.WriteEndGroup() : Exit Sub
                                    If AinfC(i1, j) = -1000 Then AsteelX = Ainf(i1, j) Else AsteelX = AinfC(i1, j)
                            End Select

                            Select Case .Type_Acier_Appui_Droit
                                Case 1 'TS
                                    Call Aciers_longitudinaux_par_lit_TS_4appuis(1, i1, j, AsteelX, 0, d, str, Verif)
                                Case 2 'quadrillage barres HA
                                    Call Aciers_longitudinaux_par_lit_HA_4appuis(1, i1, j, AsteelX, 0, d, Verif)
                            End Select
                            If Verif = False Then rtf.WriteEndGroup() : Exit Sub
                            .dSupD = d
                            Call Tableau_ELU_Armature_Flexion_4appuis(1, Mu, i1, j, 1, 0, 0, d, str, Message, rtf)
                            Call Ferraillage_Pratique_4Appuis(i1, j, rtf)
                        Case 4 'aciers supérieurs sur appuis haut et bas
                            rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                            rtf.Bold = True : rtf.WriteString("  ■ Armatures supérieures sur appui haut et appui bas:") : rtf.Bold = False : rtf.NewLine()
                            rtf.WriteString("Moment développé identique pour les 2 appuis.") : rtf.NewLine()
                            rtf.WriteEndGroup()
                            Mu = 0.15 * .ELU_Str_Env.Mx_max.M
                            d = .H - .enrSup - 0.005
                            Call Calcul_ELU_armature_flexion(1, i1, j, Mu, d, Ainf, AinfC(i1, j), σsd(i1, j), str, Message, Verif)
                            If Verif = False Then Exit Sub
                            If AinfC(i1, j) = -1000 Then AsteelX = Ainf(i1, j) Else AsteelX = AinfC(i1, j)
                            Select Case .Type_Acier_Appui_Droit
                                Case 1 'TS
                                    Call Aciers_longitudinaux_par_lit_TS_4appuis(1, i1, j, AsteelX, 0, d, str, Verif)
                                Case 2 'quadrillage barres HA
                                    Call Aciers_longitudinaux_par_lit_HA_4appuis(1, i1, j, AsteelX, 0, d, Verif)
                            End Select
                            If Verif = False Then Exit Sub
                            Call Tableau_ELU_Armature_Flexion_4appuis(1, Mu, i1, j, 1, 0, 0, d, str, Message, rtf)
                            Call Ferraillage_Pratique_4Appuis(i1, j, rtf)

                    End Select


                    ''aciers imposés par le projeteur 
                    'If (j = 1 And .LitInf.Item(1).phiProj = 0) Or (j = 2 And .LitSupG.Item(1).phiProj = 0) Or (j = 3 And .LitSupD.Item(1).phiProj = 0) Then
                    '    '1 seul lit d'acier
                    '    'vérification pour ce cas que la section d'acier soit suffisante sinon risque plantage logiciel
                    '    Select Case j
                    '        Case 1
                    '            If Ainf(1, i1, j) > nbarMax * PI * .LitInf.Item(0).phiProj ^ 2 / 4 Then
                    '                msg = "Travée " & i1.ToString & ": section d'acier est insuffisante - " &
                    '                    ((nbarMax * PI * .LitInf.Item(0).phiProj ^ 2 / 4) * 1000).ToString("#0.00") & " cm² pour " &
                    '                    (Ainf(1, i1, j) * 10000).ToString("#0.00") & " cm² requis" & vbCrLf &
                    '                    "Modifiez le nombre maximum de barres par lit  ou le diamètre de l'acier ou le nombre de lits."
                    '                MessageBox.Show(msg, "EC2: calcul section ELU fondamental", MessageBoxButtons.OK, MessageBoxIcon.Information)
                    '                verif = False
                    '            End If
                    '        Case 2
                    '            If Ainf(1, i1, j) > nbarMax * PI * .LitSupG.Item(0).phiProj ^ 2 / 4 Then
                    '                msg = "Travée " & i1.ToString & ": section d'acier est insuffisante - " &
                    '                    ((nbarMax * PI * .LitSupG.Item(0).phiProj ^ 2 / 4) * 1000).ToString("#0.00") & " cm² pour " &
                    '                    (Ainf(1, i1, j) * 10000).ToString("#0.00") & " cm² requis" & vbCrLf &
                    '                    "Modifiez le nombre maximum de barres par lit  ou le diamètre de l'acier ou le nombre de lits."
                    '                MessageBox.Show(msg, "EC2: calcul section ELU fondamental", MessageBoxButtons.OK, MessageBoxIcon.Information)
                    '                verif = False
                    '            End If
                    '        Case 3
                    '            If Ainf(1, i1, j) > nbarMax * PI * .LitSupD.Item(0).phiProj ^ 2 / 4 Then
                    '                msg = "Travée " & i1.ToString & ": section d'acier est insuffisante - " &
                    '                    ((nbarMax * PI * .LitSupD.Item(0).phiProj ^ 2 / 4) * 1000).ToString("#0.00") & " cm² pour " &
                    '                    (Ainf(1, i1, j) * 10000).ToString("#0.00") & " cm² requis" & vbCrLf &
                    '                    "Modifiez le nombre maximum de barres par lit  ou le diamètre de l'acier ou le nombre de lits."
                    '                MessageBox.Show(msg, "EC2: calcul section ELU fondamental", MessageBoxButtons.OK, MessageBoxIcon.Information)
                    '                verif = False
                    '            End If
                    '    End Select
                    '    If verif = False Then Exit Sub
                    'End If
                    'If Modification_Acier_HA = True Then
                    '    'indique un 2Ième passage car As ELU < As ELS
                    '    If Ainf(1, i1, j) < Ainf(3, i1, j) Then Asteel = Ainf(3, i1, j) Else Asteel = Ainf(1, i1, j)
                    'Else
                    '    Asteel = Ainf(1, i1, j)
                    'End If
                    'Verification_Aciers_longitudinaux_par_lit_HA(i1, j, Asteel, nbarMax, d, verif)
                    'If verif = False Then Exit Sub
                    'Exit Do 'pour éviter de boucler sur des aciers imposés
                Next
            End With
        Next
        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.Underline = True
        rtf.WriteString("2 - 2  Epure des armatures") : rtf.NewLine() : rtf.FontSize = 9 : rtf.NewLine()
        rtf.WriteString("- Clause générale") : rtf.Underline = False : rtf.NewLine()
        rtf.WriteString("EN1992-1-1 A9.2.1.4(2) modifié par AN et commission EC2: FE= [Ved].a1/z + Med/z  (Ned = 0 en flexion simple).") : rtf.NewLine()
        rtf.WriteString("Pour les appuis d'extrémités, le 1ier lit inférieur est ancré totalement quelque soit la valeur de Ved (EN1992-1-1 A9.2.1.4(3)). ") : rtf.NewLine()
        rtf.WriteString("Application de EN1992-1-1 A9.3.1.2(2): armatures supérieures forfaitairement prolongées sur 0,2.L") : rtf.NewLine()
        rtf.WriteEndGroup()
        For i1 = 1 To PlanCont.Count
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.Bold = True
            rtf.Underline = True : rtf.WriteString("- Travée n°" & i1.ToString) : rtf.Bold = False : rtf.Underline = False : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("Valeur du décalage de la courbe enveloppe du moment sollicitant: al = " & PlanCont.Item(i1 - 1).al.ToString("#0.00") & " m (eq. 9.2 de l'EN1992-1-1).") : rtf.NewLine()
            rtf.WriteEndGroup()
            With PlanCont.Item(i1 - 1)
                For j = 1 To 4
                    Select Case j 'détermination de l'épure des armatures et divers suivant j
                        Case 1 'acier en travée
                            Select Case .Type_Acier_Travée
                                Case 1
                                    Call Verification_Acier_sur_4appuis(i1, rtf)
                                'toutes les armatures étant ancrées sur appuis, pas de nécessité de faire appel au calcul d'épure
                                Case 2
                                    Call Verification_Acier_sur_4appuis(i1, rtf)
                                    'toutes les armatures étant ancrées sur appuis, pas de nécessité de faire appel au calcul d'épure
                            End Select
                            Call Tab_Epure_4Appuis(i1, j, rtf)
                        Case 2, 3, 4 'acier sur appui 
                            Call Epure_lit_acierSup_4appuis(i1, j, σsd(i1, j))
                            Call Tab_Epure_4Appuis(i1, j, rtf)
                    End Select
                Next j
                rtf.WriteStartGroup()
                rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                rtf.WriteString("Les Abscisses marquées par une * correspondent aux aciers totalement ancrés sur appui.") : rtf.NewLine()
                rtf.WriteEndGroup()
            End With
        Next i1
        Erase str
    End Sub
    '''<summary>Calcul à l'ELU de la section d'armature en flexion simple</summary>
    '''<param name="ELU_ELS">=1 si ELU Str, =2 si ELU EQ, = si ELU Accidentel</param>
    '''<param name="i1">n° de travée</param>
    '''<param name="j">acier en travée ou sur appui</param>
    '''<param name="AinfC">section d'acier optimisée suite à l'utilisation de la branche inclinée, =-1000 si pas d'utilisation</param>
    '''<param name="fydC">contrainte sur l'acier en cas d'utilisation de la section optimisée</param>
    Private Sub Calcul_ELU_armature_flexion(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal j As Integer, ByVal Mu As Double, ByVal d As Double, ByRef Ainf(,) As Double,
                                            Optional ByRef AinfC As Double = 0, Optional ByRef fydC As Double = 0, Optional ByRef str() As Double = Nothing,
                                             Optional ByRef message As Integer = 0, Optional ByRef verif As Boolean = True)
        '----------------------------------------------------------------------------
        ' Calcul des aciers supérieurs et inférieurs en flexion simple - dalle rectangulaire
        '------------------------------------------------------------------------------

        Dim η, λ, αu, αAB, εspL, εud, εs, εc, εcu3, εc3, µu, µlimu, εuk, γs, γc As Double ' µlim, σcLim,
        Dim fyd, fcd, fud, k, n, zu As Double
        Select Case ELU_ELS
            Case 1
                γc = bet.γcd : γs = aci.γsd 'combinaison fondamentale
            Case 2
                γc = bet.γca : γs = aci.γsa 'combinaison acidentelle
        End Select
        If bet.fck > 50 Then
            η = 1 - ((bet.fck - 50) / 200) : λ = 0.8 - ((bet.fck - 50) / 400) : n = 1.4 + 23.4 * ((90 - bet.fck) / 100) ^ 4
            Select Case bet.fck
                Case 55
                    εcu3 = 3.1 : εc3 = 1.8
                Case 60
                    εcu3 = 2.9 : εc3 = 1.9
                Case 70
                    εcu3 = 2.7 : εc3 = 2
                Case 80
                    εcu3 = 2.6 : εc3 = 2.2
                Case 90
                    εcu3 = 2.6 : εc3 = 2.3
                    'Case Else
                    '    MessageBox.Show("Calcul arrêté." & Environment.NewLine & " Fck béton incorrectement renseigné", "Module Calcul_Flexion_Simple_ELU_Rect", MessageBoxButtons.OK, MessageBoxIcon.Error)
                    '    Exit Sub
            End Select
        Else
            η = 1 : λ = 0.8 : n = 2
            εcu3 = 3.5 : εc3 = 1.75
        End If
        fcd = bet.fck / γc
        fyd = aci.fyk / γs : str(11) = fyd
        'Acier
        Select Case aci.classe
            Case 1 'A
                k = 1.05 : εuk = 25' valeur en pour mille
            Case 2 'B
                k = 1.08 : εuk = 50
            Case 3 'C
                k = 1.15 : εuk = 75
        End Select
        Ainf(i1, j) = 0
        AinfC = -1000 'en initialisation
        With PlanCont.Item(i1 - 1)

            µu = Mu / (1 * d ^ 2 * η * fcd) : str(2) = µu 'largeur de base = 1m
            εud = εuk * 0.9 'EN1992-1-1 Art. 3.2.7
            εspL = fyd / aci.Es : εspL = εspL * 1000
            fud = fyd + fyd * (k - 1) * (εud - εspL) / (εuk - εspL) 'contrainte limite de l'acier sur branche inclinée
            αAB = εcu3 / (εcu3 + εud) ' Voir cours INSA Rennes, identique pour εcu2 car εcu3=εcu2
            µlimu = λ * αAB * (1 - (λ / 2) * αAB) : str(1) = µlimu 'Moment réduit limite entre pivot A et pivot B

            If µu < µlimu Then
                'Pivot A
                αu = (1 - (1 - 2 * µu) ^ 0.5) / λ
                zu = d * (1 - λ * αu / 2)
                Ainf(i1, j) = Mu / (zu * fud)
                εc = εud * αu / (1 - αu) 'déformation sur la fibre béton
                str(10) = fud
                fydC = fyd
                If εc < εc3 Then
                    message = 22 'section béton mal utilisé = béton surabondant' σc = η * fcd * εc / εc3
                Else
                    ' σc = η * fcd
                    ' Tableau_Flexion_Simple_ELU(d, µu, µlimu, αu, zu, Ainf, AinfC, Asup, εc, εud, εspL, εud, fyd, fud, σc, rtf)
                End If
            Else
                'Pivot B
                αu = (1 - (1 - 2 * µu) ^ 0.5) / λ
                zu = d * (1 - λ * αu / 2)
                Ainf(i1, j) = Mu / (zu * fyd)
                εs = εcu3 * (1 - αu) / αu

                If εs < εspL Then
                    'calcul avec acier comprimé
                    message = 21 ' rtf.WriteString("εs < εspL - La poutre nécessite des armatures comprimés - Calcul arrêté !") : rtf.NewLine()
                    MessageBox.Show("Le plancher nécessite des armatures comprimés." & Environment.NewLine & "Calcul arrêté !", "EC2 - module calcul flexion simple ELU:", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                    verif = False
                ElseIf εs > εud Then
                    AinfC = -1000 'Pas de correction - palier plastique
                    fydC = fyd : str(10) = fydC
                Else
                    fydC = fyd + fyd * (k - 1) * (εs - εspL) / (εuk - εspL)
                    AinfC = Mu / (zu * fydC) : str(10) = fydC
                End If
            End If
            str(3) = αu : str(4) = zu : str(5) = Ainf(i1, j) : str(6) = εc : str(7) = εs : str(8) = εspL : str(9) = AinfC
            'vérification section maximale
            If Ainf(i1, j) > 0.04 * (1 * .H) Then
                Dim msg As String
                msg = "Vérification section maximale acier: La section d'acier demandée (" & (Ainf(i1, j) * 10000).ToString("#,##0.00") & " cm²) pour la travée " & i1.ToString &
                Environment.NewLine & "est beaucoup trop importante ! Modifiez la largeur ou la hauteur de la dalle." &
                Environment.NewLine & "La note de calcul a été établi avec la section maximale."
                Aff.msg_Avertiss.Add(msg)
                Ainf(i1, j) = 0.04 * (1 * .H)
            End If
        End With
    End Sub
    '''<summary>Détermination des panneaux TS répondant aux quantités d'acier exigées en flexion bidirectionnelle</summary>
    ''' <param name="i1">n° de travée (et non n° d'index !!!!!!!!!!!!!!!!!!!!!) </param>
    ''' <param name="j">localisation du moment, 1->en travée, 2->sur appui gauche, 3->sur appui droit </param>
    ''' <param name="N_Interval">n° d'intervalle -  obligatoire pour répartition non uniforme des panneaux</param>
    Private Sub Aciers_longitudinaux_par_lit_TS_4appuis(ByVal i1 As Integer, ByVal j As Integer, ByVal AcX As Double, ByVal AcY As Double, ByVal N_Interval As Integer,
                                                        ByRef d As Double, ByRef str() As Double, ByRef Verif As Boolean)

        Dim i, i2, i3, inf As Integer ', nbarS(3, 3), NLit_TS(2),nb,i22, i222,i4, i5, ip1, ip2,iplit,  nblit
        Dim NLit_TS1 As New Ferraillage_TS
        Dim NLit_TS2 As New Ferraillage_TS
        Dim NLit_TS3 As New Ferraillage_TS
        Dim ama(3), da, ev, phiS(3, 3), u1, u2, u3, u4, u5 As Double 'Ac,enrobage,, φn
        Dim Amin As Double 'Aminy,
        Dim Esp As Boolean = True
        Dim sol_Lit As Boolean = False 'indique si une solution a été trouvée
        ev = Max(bet.dg + 0.005, 0.02) 'initialisation espacement vertical entre lits

        With PlanCont.Item(i1 - 1)
            Call Pourcent_mini_acier_flexion(i1, d, Amin) ' % minimun longitudinal des aciers de flexion dans une Dalle

            Select Case j
                Case 1
                    If AcX < Amin Then AcX = Amin
                    If AcY < Amin Then AcY = Amin '% mini
                    str(12) = Amin
                Case 2, 3, 4 'reprise de 0,15M0 ou 0,25M0
                    AcY = 0
                    ' If i1 = 1 Then Aminy = 0 'aminy n'a pas de signification physique dans ce cas là
                    'Pour les autres cas, la section de constructions des TS Adets suffit
                    '  If i1 = PlanCont.Count Then Aminy = 0 'aminy n'a pas de signification physique dans ce cas là
                    'Pour les autres cas, la section de constructions des TS Adets suffit
            End Select

            ama(1) = 0 : ama(2) = 0 : inf = 1
            'calcul sur 1 lit                  
            i2 = -1
            Do
                i2 = i2 + 1 'base 0
                If aci.TS.Item(i2) = True Then
                    ama(1) = aci.As_long(i2)
                    u3 = aci.As_trans(i2)
                End If
            Loop Until (ama(1) > AcX And u3 > AcY) Or i2 + 1 = aci.nomTS.Count  'fin de la table
            If ama(1) < AcX Or u3 < AcY Then
                If (j = 1 And .MaxnbLitTravee = 1) Or (j = 2 Or j = 3 And .MaxnbLitAppui = 1) Then
                    Dim msg As String
                    msg = "Vous avez spécifié une solution à 1 lit, or, aprés calcul, 2 lits de TS sont nécessaires. Calcul effectué avec 2 lits. "
                    Aff.msg_Avertiss.Add(msg)
                    If j = 1 Then .MaxnbLitTravee = 2
                    If j = 2 Or j = 3 Then .MaxnbLitAppui = 2
                End If
                NLit_TS1.indTS = -1
            ElseIf ama(1) > AcX And u3 > AcY Then
                NLit_TS1.indTS = i2 'n° de treillis suivant nomenclature ADETS : inf(i1, j) = 1
                NLit_TS1.Intervalle = N_Interval
            End If
            'calcul de As avec 2 lits
            If (j = 1 And .MaxnbLitTravee = 2) Or (j = 2 Or j = 3 And .MaxnbLitAppui = 2) Then
                i2 = 0 : ama(2) = 0.1 'en m²
                Do
                    i2 = i2 + 1
                    If aci.TS.Item(i2) = True Then
                        u1 = aci.As_long(i2)
                        u4 = aci.As_trans(i2)
                    End If
                    i3 = i2 - 2
                    Do
                        i3 = i3 + 1
                        If aci.TS.Item(i2) = True Then
                            u2 = aci.As_long(i3)
                            u5 = aci.As_trans(i3)
                        End If
                    Loop Until (u1 + u2 > AcX And u4 + u5 > AcY) Or i3 = i2
                    If u1 + u2 > AcX And u4 + u5 > AcY Then
                        sol_Lit = True
                        If u1 + u2 < ama(2) Then 'recherche de la quantité d'acier la + petite
                            ama(2) = u1 + u2
                            NLit_TS2.indTS = i2 : NLit_TS3.indTS = i3 '    NLit_TS(2) = i2 : NLit_TS(3) = i3
                            NLit_TS2.Intervalle = N_Interval : NLit_TS3.Intervalle = N_Interval
                        End If
                    End If
                Loop While i2 + 1 < aci.nomTS.Count
                If sol_Lit = False Then
                    Dim msg As String
                    msg = "Les sectiosn d'acier requise (" & (AcX * 10000).ToString("#,###.00") & " cm² et " & (AcY * 10000).ToString("#,###.00") & " cm²) sont trop importante et ne peuvent être couverte avec 2 lits d'acier."
                    MessageBox.Show(msg, "Module de calcul", MessageBoxButtons.OK, MessageBoxIcon.Error)
                    Verif = False
                    Exit Sub
                End If
                ' comparaison de As avec 2 lits et de As avec 1 lit
                If NLit_TS1.indTS > 0 And ama(1) > ama(2) Then inf = 2 Else inf = 1
                'TS le + important le + prés de la face pour la solution à 2 TS
                If inf = 2 And aci.As_long(NLit_TS3.indTS) > aci.As_long(NLit_TS2.indTS) Then
                    i = NLit_TS2.indTS : NLit_TS2.indTS = NLit_TS3.indTS : NLit_TS3.indTS = i 'inversion pour avoir le TS le plus gros en bas
                    NLit_TS2.Intervalle = N_Interval : NLit_TS3.Intervalle = N_Interval
                End If
            End If

            Select Case inf   ' détermination du nouveau bras de levier avec l'acier réel
                Case 1
                    da = aci.Ølong(NLit_TS1.indTS) / 2
                    Select Case j
                        Case 1
                            .TS_trav.Clear() 'remise à zero
                            .TS_trav.Add(NLit_TS1)
                        Case 2
                            .TS_AppG.Clear() 'remise à zero
                            .TS_AppG.Add(NLit_TS1)
                        Case 3
                            .TS_AppD.Clear() 'remise à zero
                            .TS_AppD.Add(NLit_TS1)
                        Case 4 'acier sur appuis latéraux non porteur
                            .TS_AppH.Clear() 'remise à zero
                            .TS_AppH.Add(NLit_TS1)
                            .TS_AppB.Clear() 'remise à zero
                            .TS_AppB.Add(NLit_TS1)
                    End Select
                Case 2
                    If j = 1 Then
                        Select Case aci.Position_Arma
                            Case 2 ' tous les lits sont espacés
                                ev = Max(ev, aci.Ølong(NLit_TS2.indTS)) : ev = Max(ev, aci.Ølong(NLit_TS3.indTS))  '     ev = Max(ev, aci.Ølong(NLit_TS(2))) : ev = Max(ev, aci.Ølong(NLit_TS(3)))
                                da = (aci.Ølong(NLit_TS2.indTS) / 2 * aci.As_long(NLit_TS2.indTS) + (aci.Ølong(NLit_TS2.indTS) + ev + aci.Ølong(NLit_TS3.indTS) / 2) * aci.As_long(NLit_TS3.indTS)) / (aci.As_long(NLit_TS2.indTS) + aci.As_long(NLit_TS3.indTS))  '     da = (aci.Ølong(NLit_TS(2)) / 2 * aci.As_long(NLit_TS(2)) + (aci.Ølong(NLit_TS(2)) + ev + aci.Ølong(NLit_TS(3)) / 2) * aci.As_long(NLit_TS(3))) / (aci.As_long(NLit_TS(2)) + aci.As_long(NLit_TS(3)))
                            Case 1 '  les lits sont regroupés  
                                da = (aci.Ølong(NLit_TS2.indTS) / 2 * aci.As_long(NLit_TS2.indTS) + (aci.Ølong(NLit_TS2.indTS) + aci.Ølong(NLit_TS3.indTS) / 2) * aci.As_long(NLit_TS3.indTS)) / (aci.As_long(NLit_TS2.indTS) + aci.As_long(NLit_TS3.indTS))     '     da = (aci.Ølong(NLit_TS(2)) / 2 * aci.As_long(NLit_TS(2)) + (aci.Ølong(NLit_TS(2)) + aci.Ølong(NLit_TS(3)) / 2) * aci.As_long(NLit_TS(3))) / (aci.As_long(NLit_TS(2)) + aci.As_long(NLit_TS(3)))
                        End Select
                    Else
                        'Sur appuis, les lits sont regroupés  
                        da = (aci.Ølong(NLit_TS2.indTS) / 2 * aci.As_long(NLit_TS2.indTS) + (aci.Ølong(NLit_TS2.indTS) + aci.Ølong(NLit_TS3.indTS) / 2) * aci.As_long(NLit_TS3.indTS)) / (aci.As_long(NLit_TS2.indTS) + aci.As_long(NLit_TS3.indTS))     '     da = (aci.Ølong(NLit_TS(2)) / 2 * aci.As_long(NLit_TS(2)) + (aci.Ølong(NLit_TS(2)) + aci.Ølong(NLit_TS(3)) / 2) * aci.As_long(NLit_TS(3))) / (aci.As_long(NLit_TS(2)) + aci.As_long(NLit_TS(3)))
                    End If
                    Select Case j
                        Case 1
                            .TS_trav.Clear() 'remise à zero
                            .TS_trav.Add(NLit_TS2)   '  .TS_trav.Item(0).indTS = NLit_TS(2) 'indice du TS retenu
                            .TS_trav.Add(NLit_TS3)   '   .TS_trav.Item(1).indTS = NLit_TS(3) 'indice du TS retenu
                        Case 2
                            .TS_AppG.Clear() 'remise à zero
                            .TS_AppG.Add(NLit_TS2)   ' .TS_AppG.Item(0).indTS = NLit_TS(2) 'indice du TS retenu
                            .TS_AppG.Add(NLit_TS3)  ' .TS_AppG.Item(1).indTS = NLit_TS(3) 'indice du TS retenu
                        Case 3
                            .TS_AppD.Clear() 'remise à zero
                            .TS_AppD.Add(NLit_TS2)    ' .TS_AppD.Item(0).indTS = NLit_TS(2) 'indice du TS retenu
                            .TS_AppD.Add(NLit_TS3)    '  .TS_AppD.Item(1).indTS = NLit_TS(3) 'indice du TS retenu
                        Case 4 'acier sur appuis latéraux non porteur
                            .TS_AppH.Clear() 'remise à zero
                            .TS_AppB.Clear() 'remise à zero
                            .TS_AppH.Add(NLit_TS2) : .TS_AppH.Add(NLit_TS3)
                            .TS_AppB.Add(NLit_TS2) : .TS_AppB.Add(NLit_TS3)
                            '    Dim msg As String
                            '    msg = "Travée n°" & i1.ToString & " Calcul des TS sur appuis Haut et Bas (appuis non porteurs) effectué avec 2 lits. "
                            '    Aff.msg_Avertiss.Add(msg)
                    End Select
            End Select
            'Calcul du nouveau bras de levier
            Select Case j
                Case 1
                    d = .H - .enrInf - da - 0.005  '0.015 correspond au diamètre de l'acier du cadre et aux erreurs de calage 'd(i1, j)
                Case 2, 3
                    d = .H - .enrSup - da - 0.005  '0.015 correspond au diamètre de l'acier du cadre et aux erreurs de calage 'd(i1, j)
            End Select

        End With
        NLit_TS1 = Nothing : NLit_TS2 = Nothing : NLit_TS3 = Nothing 'suppression de la classe
    End Sub
    Private Sub Aciers_longitudinaux_par_lit_HA_4appuis(ByVal ELU_ELS As Integer, ByVal i1 As Integer, ByVal j As Integer, ByVal AcX As Double, ByVal AcY As Double,
                                                ByRef d As Double, ByRef verif As Boolean) 'ByRef str() As Double,
        '-------------------------------------------------------------------------
        '   Détermination des aciers longitudinaux par lits de barres HA
        '            nécessaire à la reprise des efforts
        '-------------------------------------------------------------------------
        'Variables: i1 = n° de travée (et non n° d'index !!!!!!!!!!!!!!!!!!!!!)
        '           j = localisation du moment, 1->en travée, 2->sur appui gauche, 3->sur appui droit

        Dim i, i2, n1, n2 As Integer 'nbarS(3, 3),i22, i222, i4, i5, ip1,, nblit, inf
        Dim ama, amin, Lcalcul, u1 As Double 'ac_Y,da,ev, phiS(3, 3),Ac,enrobage,, φnaminy,ac,
        Dim Esp As Boolean = True
        Dim NLit_HA1 As New Ferraillage_HA
        Dim NLit_HA2 As New Ferraillage_HA
        Dim msg As String

        With PlanCont.Item(i1 - 1)
            ' ac_Y = Ainf * 0.2 ': Str(12) = ac_Y
            Call Pourcent_mini_acier_flexion(i1, d, amin) '% mini dans le sens non porteur
            If AcX < amin Then AcX = amin '% mini
            If AcY < amin Then AcY = amin

            Select Case j
                Case 1
                    '% mini  ac_Y = aminy : Str(12) = ac_Y

                Case 2 'reprise de 0,15M0
                    '      If i1 = 1 Then aminy = 0 'aminy n'a pas de signification physique dans ce cas là
                Case 3
                    '       If i1 = PlanCont.Count Then aminy = 0 'aminy n'a pas de signification physique dans ce cas là
                Case 4 'aciers sur appuis non porteurs
                    '      aminy = 0 'aminy n'a pas de signification physique dans ce cas là
            End Select
            If j <= 3 Then
                n1 = CInt(Int(.Ly / .EspMaxPrincipal)) : n2 = CInt(Int(.Ly / .EspminPrincipal))   'même valeur pour tous les lits
                Lcalcul = .Ly
            Else
                n1 = CInt(Int(.Lx / .EspMaxSecondaire)) : n2 = CInt(Int(.Lx / .EspminSecondaire))   'même valeur pour tous les lits
                Lcalcul = .Lx
            End If

            ama = 0.1   '1000 cm²
            'recherche de la combinaison des diamètres d'acier la plus proche de la section calculée
            'calcul  dans le sens X
            For i = n1 To n2
                i2 = -1 : u1 = 0.2
                Do
                    i2 = i2 + 1
                    If aci.Ha_Lg(i2) = True Then u1 = i * PI * aci.Ø_Norm(i2) ^ 2 / 4
                    If i2 + 1 >= 10 Then Continue For 'nbre total de diamètre
                Loop While u1 < AcX * Lcalcul
                If u1 < ama Then
                    ama = u1
                    With NLit_HA1
                        .nbar = i : .φHA = aci.Ø_Norm.Item(i2)
                        Call Calcul_Espacement(PlanCont.Item(i1 - 1).Ly, i, .S1, .S2, .Scourant)
                    End With
                    'affectation
                    Select Case j
                        Case 1
                            .LitInf.Clear() 'en raison de plusieurs accès successifs
                            .LitInf.Add(NLit_HA1)
                        Case 2
                            .LitSupG.Clear()
                            .LitSupG.Add(NLit_HA1)
                        Case 3
                            .LitSupD.Clear()
                            .LitSupD.Add(NLit_HA1)
                        Case 4
                            .LitSupH.Clear() : .LitSupB.Clear()
                            ' If .LappH > 0.05 Then 'And Aff.EncastPartiel_2_porteur = True 
                            With NLit_HA1
                                    Call Calcul_Espacement(PlanCont.Item(i1 - 1).Lx, i, .S1, .S2, .Scourant)
                                End With
                                .LitSupH.Add(NLit_HA1)
                            '  End If
                            ' If .LappB > 0.05 Then 'And Aff.EncastPartiel_2_porteur = True
                            'With NLit_HA1
                            '        Call Calcul_Espacement(PlanCont.Item(i1 - 1).Lx, i, .S1, .S2, .Scourant)
                            '    End With
                            .LitSupB.Add(NLit_HA1)
                            ' End If
                    End Select
                End If
            Next i
            'vérification qu'un résultat ait été trouvé
            If ama = 0.1 Then
                u1 = n2 * aci.Ø_Norm.Item(i2) ^ 2 / 4 * PI
                msg = "Section" & (AcX * 10000).ToString("#,##0.0") & "cm2 Impossible à satisfaire avec " & n2.ToString & " Barres de " & (aci.Ø_Norm.Item(i2) * 10000).ToString("#") &
                    " mm ( " & (u1 * 10000).ToString("#,##0.0") & " cm2)." & Environment.NewLine & "Cliquez sur la touche OK"
                MessageBox.Show(msg, "Plancher BA : module de calcul", MessageBoxButtons.OK, MessageBoxIcon.Error, MessageBoxDefaultButton.Button1)
                verif = False
                Exit Sub
            End If
            '____________________________________________________________________________________________________________________________________________________________________
            'calcul dans le sens Y

            If j = 4 Then Exit Sub 'Pas de signification pour ce type d'acier

            ama = 0.1   '1000 cm²
            'vérification de la mise en place de ce type d'armatures pour les aciers sur appuis
            Select Case j
                Case 2
                    If i1 = 1 Or Aff.HA_Trans_Appui = False Then Exit Sub
                Case 3
                    If i1 = PlanCont.Count Or Aff.HA_Trans_Appui = False Then Exit Sub ' .LitSupD_Y.Item(0).nbar = 0 : .LitSupD_Y.Item(0).φHA = 0
            End Select

            n1 = CInt(Int(.Lx / .EspMaxPrincipal))
            n2 = CInt(Int(.Lx / .EspminPrincipal))
            For i = n1 To n2
                i2 = -1 'le diamètre peut commencer à 6 mm
                u1 = 0.2 'pour être sûr d'avoir au moins 1 branchement
                Do
                    i2 = i2 + 1
                    If aci.Ha_Lg(i2) = True Then u1 = i * PI * aci.Ø_Norm(i2) ^ 2 / 4
                    If aci.Ø_Norm(i2) >= NLit_HA1.φHA Then Continue For
                Loop While u1 < AcY * .Lx
                If u1 < ama Then
                    ama = u1
                    Select Case j
                        Case 1
                            With NLit_HA2
                                .nbar = i : .φHA = aci.Ø_Norm.Item(i2)
                                Call Calcul_Espacement(PlanCont.Item(i1 - 1).Lx, i, .S1, .S2, .Scourant)
                                If PlanCont.Item(i1 - 1).Ly + .S1 + .S2 > aci.HA_Lg_Max Then
                                    aci.Position_Portefeuille = True ' : .LitInf_Y_nblit = 2
                                Else
                                    aci.Position_Portefeuille = False ': .LitInf_Y_nblit = 1
                                End If
                            End With
                            .LitInf_Y.Clear() 'en raison de plusieurs accès successifs
                            .LitInf_Y.Add(NLit_HA2)
                            If aci.Position_Portefeuille = True Then .LitInf_Y.Add(NLit_HA2)
                        Case 2, 3
                            If Aff.HA_Trans_Appui = True Then
                                msg = "Module acier_longitudinaux_par_lit_HA - aciers transversaux pour armatures supérieures - calcul non implémenté"
                                Aff.msg_Avertiss.Add(msg)
                            End If
                    End Select
                End If
            Next i
            Select Case j
                Case 1
                    d = .H - .enrInf - NLit_HA1.φHA / 2   ' détermination du nouveau bras de levier avec l'acier réel
                Case 2, 3
                    d = .H - .enrSup - NLit_HA1.φHA / 2   ' détermination du nouveau bras de levier avec l'acier réel
            End Select
            NLit_HA1 = Nothing : NLit_HA2 = Nothing
        End With
    End Sub
    Private Sub Tableau_ELU_Armature_Flexion_4appuis(ByVal Entete As Integer, ByVal Mu As Double, ByVal i1 As Integer, ByVal j As Integer, ByVal sens As Integer,
                                             ByVal x As Double, ByVal y As Double, ByVal d As Double, ByVal str() As Double, ByVal Message As Integer, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        'str(1) = µlimu moment réduit limite
        Dim msg As String
        Dim Amin As Double
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9
        If Message = 22 Then
            msg = "Déformation béton εcb < εcb3 - section béton mal utilisé = béton surabondant" '"
            rtf.WriteString(msg) : rtf.NewLine()
            Aff.msg_Avertiss.Add(msg)
        End If
        If Message = 21 Then
            msg = "Contrainte de compression béton > 0,6.fck : Le plancher nécessite des armatures comprimées  "
            rtf.WriteString(msg) : rtf.NewLine() ' & σcLim.ToString("#0.00") & "Mpa > " & (0.6 * bet.fck).ToString("#0.00") & " MPa") : rtf.NewLine()
            Aff.msg_Avertiss.Add(msg)
        End If

        rtf.NewParagraph()
        'Ligne 1
        If Entete = 1 Then
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
            If sens = 1 Then msg = "Sens porteur suivant X" Else msg = "Sens porteur suivant Y"
            rtf.BeginRow() : rtf.DeclareCell(8600) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
        End If
        'Ligne 2
        Select Case j
            Case 1
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True)
                rtf.WriteString("Coordonnées: x= " & (x * 100).ToString("#0.0") & " cm et y= " & (y * 100).ToString("#0.0") & " cm.") : rtf.EndCell() : rtf.EndRow()
            Case 2 'appui gauche
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True)
                If i1 = 1 Then
                    rtf.WriteString(" = 0,15.Mt Max") : rtf.EndCell() : rtf.EndRow()
                Else
                    rtf.WriteString("= 0,25.Mt Max") : rtf.EndCell() : rtf.EndRow()
                End If

            Case 3 'appui droit

                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True)
                If i1 = PlanCont.Count Then
                    rtf.WriteString(" = 0,15.Mt Max") : rtf.EndCell() : rtf.EndRow()
                Else
                    rtf.WriteString("= 0,25.Mt Max") : rtf.EndCell() : rtf.EndRow()
                End If
            Case 4 'armature sur appuis haut et bas
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" = 0,15.Mt Maxi") : rtf.EndCell() : rtf.EndRow()
        End Select
        If det_Ndc.FlexionAs = 1 Then Tableau_ELU_Armature_Flexion_4appuis_Complementaire(d, str, rtf)
        'Ligne 8
        Select Case j
            Case 1
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As inf") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature inférieure ") : rtf.EndCell() : rtf.EndRow()
                Call Pourcent_mini_acier_flexion(i1, d, Amin) ' % minimun longitudinal des aciers de flexion dans une dalle
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As min") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Amin * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section d'acier minimale") : rtf.EndCell() : rtf.EndRow()
            Case 2
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As sup G") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature supérieure sur appui gauche") : rtf.EndCell() : rtf.EndRow()
            Case 3
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As sup Dr") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature supérieure sur appui droit") : rtf.EndCell() : rtf.EndRow()
            Case 4 'Appuis haut et bas
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As sup ") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature supérieure sur appui Haut et/ou Bas") : rtf.EndCell() : rtf.EndRow()
        End Select
        If det_Ndc.FlexionAs = 1 Then
            'Ligne 10
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("εc") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("‰") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(6).ToString("#,##0.00")) : rtf.EndCell() '(εc)
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("déformation maximale béton") : rtf.EndCell() : rtf.EndRow()
            'Ligne 11
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("εs") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("‰") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(7).ToString("#0.00")) : rtf.EndCell() '(εs)
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("déformation maximale acier - Limite plasticité = " & str(8).ToString("#0.00") & "‰") : rtf.EndCell() : rtf.EndRow() 'εspL
        End If
        If str(9) = -1000 Then 'AinfC
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As corrigé") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("S.O.") : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("εs > εuk - utilisation de la branche horizontale") : rtf.EndCell() : rtf.EndRow()
        Else
            'Ligne 12
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("σs") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(10).ToString("#,##0.00")) : rtf.EndCell() 'fydC
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("modélisation avec palier incliné ( à comparer à " & str(11).ToString("#0.0") & "MPa)") : rtf.EndCell() : rtf.EndRow() 'fyd
            'Ligne 13
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As corrigé") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(9) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'AinfC
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature optimisée") : rtf.EndCell() : rtf.EndRow()
        End If
        'If ((i1 = 1 And j = 2 And Aff.Reprise_M0_1_porteur = True) = True Or (i1 = PlanCont.Count And j = 3 And Aff.Reprise_M0_1_porteur = True) = True) = False Then
        '    If ((j = 1) Or (j = 2 Or j = 3 And Aff.HA_Trans_Appui = True)) = True Then
        '        'A réaliser uniquement pour les aciers reprenant les moments sur appui - les armatures reprenant 0.15M0 sont exclues de cette vérification
        '        'ligne
        '        rtf.BeginRow() : rtf.DeclareCell(8600) : rtf.BeginCell(True) : rtf.WriteString("Sens NON porteur suivant Y") : rtf.EndCell() : rtf.EndRow()
        '        'ligne
        '        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As requis") : rtf.EndCell()
        '        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
        '        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(12) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
        '        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature inférieure suivant Y (=20%)") : rtf.EndCell() : rtf.EndRow()
        '    End If
        'End If
        rtf.WriteEndGroup()
    End Sub
    Private Sub Tableau_ELU_Armature_Flexion_4appuis_Complementaire(ByVal d As Double, ByVal str() As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("d") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((d * 100).ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("hauteur utile") : rtf.EndCell() : rtf.EndRow()
        'Ligne 4
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("µ limite") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(1).ToString("#0.0000")) : rtf.EndCell() 'µlimu
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("moment réduit limite") : rtf.EndCell() : rtf.EndRow()
        'Ligne 5
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("µ") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(2).ToString("#0.0000")) : rtf.EndCell() '(µu)
        If str(2) < str(1) Then 'µu < µlimu
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" < µ limite - Pas d'armature comprimée") : rtf.EndCell() : rtf.EndRow()
        Else
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" > µ limite - armature comprimée") : rtf.EndCell() : rtf.EndRow()
        End If
        'Ligne 6
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("α") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(str(3).ToString("#0.0000")) : rtf.EndCell() '(αu)
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Rapport x/d (axe neutre/hauteur utile)") : rtf.EndCell() : rtf.EndRow()
        'Ligne 7
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Zu") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm ") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(4) * 100).ToString("#0.0")) : rtf.EndCell() 'zu 
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("bras de levier ") : rtf.EndCell() : rtf.EndRow()
        '
        rtf.WriteEndGroup()
    End Sub
    Private Sub Tableau_ELU_Armature_Flexion_4appuis_Interval(ByVal Entete As Integer, ByVal Mu As Double, ByVal i1 As Integer, ByVal j As Integer, ByVal sens As Integer,
                                             ByVal x As Double, ByVal y As Double, ByVal branchement As Integer, ByVal str() As Double, ByVal Message As Integer, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        'str(1) = µlimu moment réduit limite
        Dim msg As String

        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9
        If branchement = 1 Then
            If Message = 11 Or Message = 21 Then
                msg = "εs < εspL - Le plancher nécessite des armatures comprimées  "
                rtf.WriteString(msg) : rtf.NewLine()
                Aff.msg_Avertiss.Add(msg)
            End If
            If Message = 12 Or Message = 22 Then
                msg = "Contrainte de compression béton > 0,6.fck : "
                rtf.WriteString(msg) : rtf.NewLine() ' & σcLim.ToString("#0.00") & "Mpa > " & (0.6 * bet.fck).ToString("#0.00") & " MPa") : rtf.NewLine()
                Aff.msg_Avertiss.Add(msg)
                msg = "Des armatures seront placées en partie supérieure pour soulager le béton comprimé. Le calcul est établi aux ELS car la combinaison est plus défavorable."
                rtf.WriteString(msg) : rtf.NewLine() : Aff.msg_Avertiss.Add(msg)
                msg = "Le plancher nécessite des armatures comprimées  "
                rtf.WriteString(msg) : rtf.NewLine() : Aff.msg_Avertiss.Add(msg)
            End If
            'Ligne 1
            If Entete = 1 Then
                rtf.NewParagraph()
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
                If sens = 1 Then msg = "Sens porteur suivant X" Else msg = "Sens porteur suivant Y"
                rtf.BeginRow() : rtf.DeclareCell(8600) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
            End If

        End If

        'Ligne 2
        Select Case j
            Case 1
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True)
                rtf.WriteString("Coordonnées: x= " & (x * 100).ToString("#0.0") & " cm et y= " & (y * 100).ToString("#0.0") & " cm.") : rtf.EndCell() : rtf.EndRow()
            Case 2 'appui gauche
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True)
                If i1 = 1 Then
                    rtf.WriteString(" = 0,15.Mt Max") : rtf.EndCell() : rtf.EndRow()
                Else
                    rtf.WriteString("= 0,25.Mt Max") : rtf.EndCell() : rtf.EndRow()
                End If

            Case 3 'appui droit

                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True)
                If i1 = PlanCont.Count Then
                    rtf.WriteString(" = 0,15.Mt Max") : rtf.EndCell() : rtf.EndRow()
                Else
                    rtf.WriteString("= 0,25.Mt Max") : rtf.EndCell() : rtf.EndRow()
                End If
            Case 4 'armature sur appuis haut et bas
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Mu") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.0")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" = 0,15.Mt Maxi") : rtf.EndCell() : rtf.EndRow()
        End Select
        'Ligne 3
        Select Case j
            Case 1
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As inf") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature inférieure ") : rtf.EndCell() : rtf.EndRow()
            Case 2
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As sup G") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature supérieure sur appui gauche") : rtf.EndCell() : rtf.EndRow()
            Case 3
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As sup Dr") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature supérieure sur appui droit") : rtf.EndCell() : rtf.EndRow()
            Case 4 'Appuis haut et bas
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As sup ") : rtf.EndCell()
                rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(5) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature supérieure sur appui Haut et/ou Bas") : rtf.EndCell() : rtf.EndRow()
        End Select

        If str(9) = -1000 Then 'AinfC
            'rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As corrigé") : rtf.EndCell()
            'rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
            'rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("S.O.") : rtf.EndCell()
            'rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("εs > εuk - utilisation de la branche horizontale") : rtf.EndCell() : rtf.EndRow()
        Else
            'Ligne 13
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As corrigé") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(9) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'AinfC
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature optimisée") : rtf.EndCell() : rtf.EndRow()
        End If
        'If ((i1 = 1 And j = 2 And Aff.Reprise_M0_1_porteur = True) = True Or (i1 = PlanCont.Count And j = 3 And Aff.Reprise_M0_1_porteur = True) = True) = False Then
        '    If ((j = 1) Or (j = 2 Or j = 3 And Aff.HA_Trans_Appui = True)) = True Then
        '        'A réaliser uniquement pour les aciers reprenant les moments sur appui - les armatures reprenant 0.15M0 sont exclues de cette vérification
        '        'ligne
        '        rtf.BeginRow() : rtf.DeclareCell(8600) : rtf.BeginCell(True) : rtf.WriteString("Sens NON porteur suivant Y") : rtf.EndCell() : rtf.EndRow()
        '        'ligne
        '        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As requis") : rtf.EndCell()
        '        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
        '        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((str(12) * 10000).ToString("#,##0.00")) : rtf.EndCell() 'Ainf 
        '        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier armature inférieure suivant Y (=20%)") : rtf.EndCell() : rtf.EndRow()
        '    End If
        'End If
        rtf.WriteEndGroup()
    End Sub
    Private Sub Ferraillage_Pratique_4Appuis(ByVal i1 As Integer, ByVal j As Integer, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim Tot, totTr As Double
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9
        With PlanCont.Item(i1 - 1)
            rtf.NewLine() : rtf.WriteString("Ferraillage retenu (de la génératrice extérieure vers l'intérieur de la dalle):") : rtf.NewLine()
            Select Case j
                Case 1 'en travée 
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            Tot = aci.As_long(.TS_trav.Item(0).indTS)
                            totTr = aci.As_trans(.TS_trav.Item(0).indTS)
                            rtf.WriteString("Lit n°1: TS " & aci.nomTS(.TS_trav.Item(0).indTS)) : rtf.NewLine()
                            If .TS_trav.Count = 2 Then
                                Tot = Tot + aci.As_long(.TS_trav.Item(1).indTS)
                                totTr = totTr + aci.As_trans(.TS_trav.Item(1).indTS)
                                rtf.WriteString("Lit n°2: TS " & aci.nomTS(.TS_trav.Item(1).indTS)) : rtf.NewLine()
                            End If
                            If .Prefa_Lx.Item(0) > 5.8 Then 'toutes les prédalles ont même longueur
                                rtf.WriteString("La portée dépasse la longueur commerciale disponible, le lit n°1 sera doublé et disposé en portefeuille (voir plan de ferraillage).") : rtf.NewLine()

                            End If
                            rtf.WriteString("Total section longitudinale (suivant X):  " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            rtf.WriteString("Total section transversale (suivant Y):  " & (totTr * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                        Case 2 'HA
                            Tot = (.LitInf.Item(0).Aire / .Ly)
                            rtf.WriteString("Longitudinal porteur (suivant X): " & .LitInf.Item(0).nbar.ToString & " HA" & (.LitInf.Item(0).φHA * 1000).ToString("#") & " - " &
                                             "s = " & (.LitInf(0).Scourant * 100).ToString("#0.0") & " cm soit Aire totale = " &
                                              (.LitInf(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            Tot = (.LitInf_Y.Item(0).Aire / .Lx)
                            rtf.WriteString("Transversal porteur (suivant Y):  " & .LitInf_Y.Item(0).nbar.ToString & " HA" & (.LitInf_Y.Item(0).φHA * 1000).ToString("#") & " - " &
                                            "s = " & (.LitInf_Y(0).Scourant * 100).ToString("#0.0") & " cm  soit Aire totale = " &
                                            (.LitInf_Y(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            If .Ly > 11.7 And aci.Position_Portefeuille = True Then
                                rtf.WriteString("La longueur de barre suivant Y dépasse la longueur commerciale disponible, les barres sont doublées et disposées en portefeuille (voir plan de ferraillage).") : rtf.NewLine()
                            End If
                    End Select
                Case 2 'sur appui gauche
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1 'TS
                            Tot = aci.As_long(.TS_AppG.Item(0).indTS)
                            totTr = aci.As_trans(.TS_AppG.Item(0).indTS)
                            rtf.WriteString("Lit n°1: TS " & aci.nomTS(.TS_AppG.Item(0).indTS)) : rtf.NewLine()
                            If .TS_AppG.Count = 2 Then
                                Tot = Tot + aci.As_long(.TS_AppG.Item(1).indTS)
                                rtf.WriteString("Lit n°2: TS " & aci.nomTS(.TS_AppG.Item(1).indTS)) : rtf.NewLine()
                                totTr = totTr + aci.As_trans(.TS_AppG.Item(1).indTS)
                            End If
                            rtf.WriteString("Total section longitudinale (suivant X):  " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            rtf.WriteString("Total section transversale (suivant Y):  " & (totTr * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                        Case 2 'HA
                            rtf.WriteString("Longitudinal porteur (suivant X): Lit n°1: " & .LitSupG.Item(0).nbar.ToString & " HA" & (.LitSupG.Item(0).φHA * 1000).ToString("#") & " - ")
                            rtf.WriteString("s = " & (.LitSupG(0).Scourant * 100).ToString("#0.0") & " cm ")
                            Tot = (.LitSupG.Item(0).Aire / .Ly) : rtf.WriteString(" soit Aire totale = " & (.LitSupG(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            If .LitSupG_Y.Count >= 1 Then
                                rtf.WriteString("Transversal NON porteur (suivant Y): Lit n°2: " & .LitSupG_Y.Item(0).nbar.ToString & " HA" & (.LitSupG_Y.Item(0).φHA * 1000).ToString("#") & "/ - ")
                                ' s = 1 / .LitSupG_Y.Item(0).nbar : rtf.WriteString("s = " & (s * 100).ToString("#0.0") & " cm ")
                                Tot = (.LitSupG_Y.Item(0).Aire) : rtf.WriteString(" soit Aire totale = " & (.LitSupG_Y(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            End If

                    End Select
                Case 3 'sur appui droit
                    Select Case .Type_Acier_Appui_Droit
                        Case 1 'TS
                            Tot = aci.As_long(.TS_AppD.Item(0).indTS)
                            totTr = aci.As_trans(.TS_AppD.Item(0).indTS)
                            rtf.WriteString("Lit n°1: TS " & aci.nomTS(.TS_AppD.Item(0).indTS))
                            If .TS_AppD.Count = 2 Then
                                rtf.WriteString("Lit n°2: TS " & aci.nomTS(.TS_AppD.Item(1).indTS))
                                Tot = Tot + aci.As_long(.TS_AppD.Item(1).indTS)
                                totTr = totTr + aci.As_trans(.TS_AppD.Item(1).indTS)
                            End If
                            rtf.WriteString("Total section longitudinale (suivant X):  " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            rtf.WriteString("Total section transversale (suivant Y):  " & (totTr * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                        Case 2 'HA
                            rtf.WriteString("Longitudinal porteur (suivant X): Lit n°1: " & .LitSupD.Item(0).nbar.ToString & " HA" & (.LitSupD.Item(0).φHA * 1000).ToString("#") & " - ")
                            rtf.WriteString("s = " & (.LitSupD(0).Scourant * 100).ToString("#0.0") & " cm ")
                            Tot = (.LitSupD.Item(0).Aire / .Ly) : rtf.WriteString(" soit Aire totale = " & (.LitSupD(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            If .LitSupD_Y.Count >= 1 Then '.LitSupD_Y.Item(0).nbar >= 1
                                rtf.WriteString("Transversal NON porteur (suivant Y): Lit n°2: " & .LitSupD_Y.Item(0).nbar.ToString & " HA" & (.LitSupD_Y.Item(0).φHA * 1000).ToString("#") & " - ")
                                ' s = 1 / .LitSupD_Y.Item(0).nbar : rtf.WriteString("s = " & (s * 100).ToString("#0.0") & " cm ")
                                Tot = (.LitSupD_Y.Item(0).Aire) : rtf.WriteString(" soit Aire totale = " & (.LitSupD_Y(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            End If
                    End Select
                Case 4 'Aciers supérieurs sur appuis non porteurs
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            Tot = aci.As_long(.TS_AppH.Item(0).indTS)
                            rtf.WriteString("Lit appui haut: TS " & aci.nomTS(.TS_AppH.Item(0).indTS) & " - section = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            rtf.WriteString("Lit appui bas: TS " & aci.nomTS(.TS_AppB.Item(0).indTS) & " - section = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                        Case 2 'HA
                            If .LappH > 0.05 And Aff.Reprise_M0_2_porteur = True Then
                                rtf.WriteString("sur appui Haut " & .LitSupH.Item(0).nbar.ToString & " HA" & (.LitSupH.Item(0).φHA * 1000).ToString("#") & " - ")
                                rtf.WriteString("s = " & (.LitSupH.Item(0).Scourant * 100).ToString("#0.0") & " cm ")
                                Tot = (.LitSupH.Item(0).Aire / .Lx) : rtf.WriteString(" soit Aire totale = " & (.LitSupH.Item(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                            End If
                            If .LappB > 0.05 And Aff.Reprise_M0_2_porteur = True Then
                                If .LappH > 0.05 Then
                                    rtf.WriteString("sur appui Bas: identique à l'appui haut. ") : rtf.NewLine()
                                Else
                                    rtf.WriteString("sur appui Bas " & .LitSupB.Item(0).nbar.ToString & " HA" & (.LitSupB.Item(0).φHA * 1000).ToString("#") & " - ")
                                    rtf.WriteString("s = " & (.LitSupB.Item(0).Scourant * 100).ToString("#0.0") & " cm ")
                                    Tot = .LitSupB.Item(0).Aire / .Lx : rtf.WriteString(" soit Aire totale = " & (.LitSupB.Item(0).Aire * 10000).ToString("#0.00") & " cm² et Aire répartie = " & (Tot * 10000).ToString("#,##0.00") & " cm²/ml.") : rtf.NewLine()
                                End If
                            End If
                    End Select
            End Select
        End With
        rtf.WriteEndGroup()
    End Sub
    Private Sub Tab_Epure_4Appuis(ByVal i1 As Integer, ByVal j As Integer, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim i, total_Lit As Integer ', k
        Dim A_s(3), x1(3), x1s(3), x2(3), x2s(3) As Double
        Dim msg As String
        With PlanCont.Item(i1 - 1)
            Select Case j
                Case 1
                    Select Case .Type_Acier_Travée
                        Case 1
                            With .TS_trav
                                A_s(1) = aci.As_long(.Item(0).indTS)
                                If .Count = 2 Then A_s(2) = aci.As_long(.Item(1).indTS)
                                For i = 1 To .Count
                                    x1(i) = .Item(i - 1).X1acier : x1s(i) = .Item(i - 1).X1acierLS : x2(i) = .Item(i - 1).X2acier
                                    x2s(i) = .Item(i - 1).X2acierLS
                                Next
                                total_Lit = .Count
                            End With
                        Case 2
                            With .LitInf
                                For i = 1 To .Count
                                    A_s(i) = .Item(i - 1).Aire : x1(i) = .Item(i - 1).X1acier : x1s(i) = .Item(i - 1).X1acierLS : x2(i) = .Item(i - 1).X2acier
                                    x2s(i) = .Item(i - 1).X2acierLS
                                Next
                                total_Lit = .Count
                            End With
                    End Select

                Case 2
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1
                            With .TS_AppG
                                A_s(1) = aci.As_long(.Item(0).indTS)
                                If .Count = 2 Then A_s(2) = aci.As_long(.Item(1).indTS)
                                For i = 1 To .Count
                                    x1(i) = .Item(i - 1).X2acier : x1s(i) = .Item(i - 1).X2acierLS : x2(i) = 0 : x2s(i) = 0
                                Next
                                total_Lit = .Count
                            End With
                        Case 2
                            With .LitSupG
                                For i = 1 To .Count
                                    A_s(i) = .Item(i - 1).Aire : x1(i) = .Item(i - 1).X2acier : x1s(i) = .Item(i - 1).X2acierLS : x2(i) = 0 : x2s(i) = 0
                                Next
                                total_Lit = .Count
                            End With
                    End Select
                Case 3
                    Select Case .Type_Acier_Appui_Droit
                        Case 1
                            With .TS_AppD
                                A_s(1) = aci.As_long(.Item(0).indTS)
                                If .Count = 2 Then A_s(2) = aci.As_long(.Item(1).indTS)
                                For i = 1 To .Count
                                    x1(i) = .Item(i - 1).X1acier : x1s(i) = .Item(i - 1).X1acierLS : x2(i) = 0 : x2s(i) = 0
                                Next
                                total_Lit = .Count
                            End With
                        Case 2
                            With .LitSupD
                                For i = 1 To .Count
                                    A_s(i) = .Item(i - 1).Aire : x1(i) = .Item(i - 1).X1acier : x1s(i) = .Item(i - 1).X1acierLS : x2(i) = 0 : x2s(i) = 0
                                Next
                                total_Lit = .Count
                            End With
                    End Select
                Case 4
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1
                            With .TS_AppH
                                A_s(1) = aci.As_long(.Item(0).indTS)
                                If .Count = 2 Then A_s(2) = aci.As_long(.Item(1).indTS)
                                For i = 1 To .Count
                                    x1(i) = .Item(i - 1).X1acier : x1s(i) = .Item(i - 1).X1acierLS : x2(i) = 0 : x2s(i) = 0
                                Next
                                total_Lit = .Count
                            End With
                        Case 2
                            With .LitSupH
                                For i = 1 To .Count
                                    A_s(i) = .Item(i - 1).Aire : x1(i) = .Item(i - 1).X2acier : x1s(i) = .Item(i - 1).X2acierLS : x2(i) = 0 : x2s(i) = 0
                                Next
                                total_Lit = .Count
                            End With
                    End Select
            End Select
        End With

        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9
        rtf.NewParagraph()

        If j = 1 Or (i1 = 1 And j = 3 And PlanCont.Item(0).LappG < 0.05) = True Or (i1 = PlanCont.Count And j = 2 And PlanCont.Item(PlanCont.Count - 1).LappD < 0.05) = True Then
            rtf.Bold = True : rtf.WriteString("  ■ Epure des aciers") : rtf.Bold = False : rtf.NewLine()
            rtf.WriteString("L'origine de l'abscisse est le nu de l'appui gauche (ou l'extrémité de la console gauche pour la 1ière travée dans le cas d'une console).") : rtf.NewLine()
            rtf.NewParagraph()
            rtf.FontSize = 12 : rtf.FontSize = 9
            'Ligne 1
            rtf.BeginRow() : rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString(" ") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Lit n°1") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Lit n°2") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Lit n°3") : rtf.EndCell() : rtf.EndRow()
        End If

        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.Bold = True
        Select Case j
            Case 1
                rtf.WriteString("Section en travée")
            Case 2
                rtf.WriteString("Section sur appui gauche")
            Case 3
                rtf.WriteString("Section sur appui droit")
            Case 4
                rtf.WriteString("Section sur appui haut ou bas = identique")
        End Select
        rtf.Bold = False : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
        If j = 4 Then
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((A_s(1) * 10000 / PlanCont.Item(i1 - 1).Lx).ToString("#0.00")) : rtf.EndCell()
        Else
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((A_s(1) * 10000 / PlanCont.Item(i1 - 1).Ly).ToString("#0.00")) : rtf.EndCell()
        End If
        If total_Lit > 1 Then
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((A_s(2) * 10000 / PlanCont.Item(i1 - 1).Ly).ToString("#0.00")) : rtf.EndCell()
        End If
        rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(6000) : rtf.BeginCell(True)
        If j > 1 Then
            rtf.WriteString("Calcul sur moment forfaitaire - Ancrage sur longueur forfaitaire") : rtf.EndCell()
        Else
            rtf.WriteString("Armatures en travée filant jusqu'aux appuis") : rtf.EndCell()
        End If
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
        If j = 1 Then msg = " *" Else msg = ""
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x1s(1).ToString("#0.00") & msg) : rtf.EndCell()
        If total_Lit > 1 Then
            If x1s(2) = 0 And j = 1 Then msg = " *" Else msg = ""
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x1s(2).ToString("#0.00") & msg) : rtf.EndCell()
        End If
        rtf.EndRow()
        'Ligne 4
        rtf.BeginRow() : rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Abscisse origine de l'ancrage") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x1(1).ToString("#0.00")) : rtf.EndCell()
        If total_Lit > 1 Then
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(x1(2).ToString("#0.00")) : rtf.EndCell()
        End If
        rtf.EndRow()
        rtf.WriteEndGroup()
    End Sub
    Private Sub Epure_lit_acierSup_4appuis(ByVal i1 As Integer, ByVal j As Integer, ByVal σsd As Double)
        '-------------------------------------------------------------------------------
        ' détermination de l'épure des lits d'acier supérieurs sur l'appui gauche  -  ELU  ELS
        '---------------------------------------------------------------------------------  
        'Variables: i1 = n° de travée
        '           j = position acier
        '           courbe_moment = type de courbe de moment
        '           x0(,,) = abcsisse pour laquelle le moment est nul
        Dim i As Integer
        Dim lappui As Double

        With PlanCont.Item(i1 - 1)
            Select Case j
                Case 2
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1
                            If i1 = 1 And Aff.Reprise_M0_2_porteur = True Then 'reprise 0,15.M0
                                lappui = .LappG - .enrLat
                                Call Ancrage_lit_4appuis(0, i1, 1, lappui, j, 1, σsd) 'ancrage total  sur l'appui gauche
                                For i = 0 To .TS_AppG.Count - 1
                                    .TS_AppG.Item(i).X1acier = 0 : .TS_AppG.Item(i).X1acierLS = 0
                                    .TS_AppG.Item(i).X2acier = 0.2 * .Lx
                                    .TS_AppG.Item(i).X2acierLS = .TS_AppG.Item(i).X2acier
                                Next i
                            Else
                                For i = 0 To .TS_AppG.Count - 1
                                    .TS_AppG.Item(i).X1acier = 0 : .TS_AppG.Item(i).X1acierLS = 0
                                    .TS_AppG.Item(i).X2acier = 0.2 * .Lx
                                    .TS_AppG.Item(i).X2acierLS = .TS_AppG.Item(i).X2acier
                                Next i
                            End If
                        Case 2
                            If i1 = 1 And Aff.Reprise_M0_2_porteur = True Then 'reprise 0,15.M0
                                lappui = .LappG - .enrLat
                                Call Ancrage_lit_4appuis(0, i1, 1, lappui, j, 1, σsd) 'ancrage total  sur l'appui gauche
                                For i = 0 To .LitSupG.Count - 1
                                    With .LitSupG
                                        .Item(i).X1acier = 0 : .Item(i).X1acierLS = 0
                                        .Item(i).X2acier = 0.2 * PlanCont.Item(i1 - 1).Lx
                                        .Item(i).X2acierLS = .Item(i).X2acier
                                    End With
                                Next i
                            Else
                                For i = 0 To .LitSupG.Count - 1
                                    With .LitSupG
                                        .Item(i).X1acier = 0 : .Item(i).X1acierLS = 0
                                        .Item(i).X2acier = 0.2 * PlanCont.Item(i1 - 1).Lx
                                        .Item(i).X2acierLS = .Item(i).X2acier
                                    End With
                                Next i
                            End If
                    End Select
                Case 3
                    Select Case .Type_Acier_Appui_Droit
                        Case 1
                            If i1 = PlanCont.Count And Aff.Reprise_M0_2_porteur = True Then 'reprise 0,15.M0
                                lappui = .LappD - .enrLat
                                Call Ancrage_lit_4appuis(0, i1, 1, lappui, j, 1, σsd) 'ancrage total  sur l'appui gauche
                                For i = 0 To .TS_AppD.Count - 1
                                    With .TS_AppD
                                        .Item(i).X1acier = 0.8 * PlanCont.Item(i1 - 1).Lx : .Item(i).X1acierLS = .Item(i).X1acier
                                        .Item(i).X2acier = PlanCont.Item(i1 - 1).Lx : .Item(i).X2acierLS = .Item(i).X2acier
                                    End With
                                Next i
                            Else
                                For i = 0 To .TS_AppD.Count - 1
                                    With .TS_AppD
                                        .Item(i).X1acier = 0.8 * PlanCont.Item(i1 - 1).Lx : .Item(i).X1acierLS = .Item(i).X1acier
                                        .Item(i).X2acier = PlanCont.Item(i1 - 1).Lx : .Item(i).X2acierLS = .Item(i).X2acier
                                    End With
                                Next i
                            End If
                        Case 2
                            If i1 = PlanCont.Count And Aff.Reprise_M0_2_porteur = True Then 'reprise 0,15.M0
                                lappui = .LappD - .enrLat
                                Call Ancrage_lit_4appuis(0, i1, 1, lappui, j, 1, σsd) 'ancrage total  sur l'appui gauche
                                For i = 0 To .LitSupD.Count - 1
                                    With .LitSupD
                                        .Item(i).X1acier = 0.8 * PlanCont.Item(i1 - 1).Lx : .Item(i).X1acierLS = .Item(i).X1acier
                                        .Item(i).X2acier = PlanCont.Item(i1 - 1).Lx : .Item(i).X2acierLS = .Item(i).X2acier
                                    End With
                                Next i
                            Else
                                For i = 0 To .LitSupD.Count - 1
                                    With .LitSupD
                                        .Item(i).X1acier = 0.8 * PlanCont.Item(i1 - 1).Lx : .Item(i).X1acierLS = .Item(i).X1acier
                                        .Item(i).X2acier = PlanCont.Item(i1 - 1).Lx : .Item(i).X2acierLS = .Item(i).X2acier
                                    End With
                                Next i
                            End If
                    End Select
                Case 4
                    If .LappH >= 0.05 Then 'reprise 0,15.M0
                        lappui = .LappH - .enrLat
                        Call Ancrage_lit(0, i1, 1, lappui, 4, 1, σsd, 36, , True) 'ancrage total  sur l'appui haut
                    End If
                    If .LappB >= 0.05 Then 'reprise 0,15.M0
                        lappui = .LappB - .enrLat
                        Call Ancrage_lit(0, i1, 1, lappui, 4, 1, σsd, 36,, True) 'ancrage total  sur l'appui bas
                    End If
                    Select Case .Type_Acier_Appui_Droit
                        Case 1
                            For i = 0 To .TS_AppH.Count - 1
                                With .TS_AppH
                                    .Item(i).X1acier = 0 : .Item(i).X1acierLS = 0
                                    .Item(i).X2acier = 0.2 * PlanCont.Item(i1 - 1).Lx
                                    .Item(i).X2acierLS = .Item(i).X2acier
                                End With
                            Next i
                            For i = 0 To .TS_AppB.Count - 1
                                With .TS_AppB
                                    .Item(i).X1acier = 0 : .Item(i).X1acierLS = 0
                                    .Item(i).X2acier = 0.2 * PlanCont.Item(i1 - 1).Lx
                                    .Item(i).X2acierLS = .Item(i).X2acier
                                End With
                            Next i
                        Case 2
                            For i = 0 To .LitSupH.Count - 1
                                With .LitSupH
                                    .Item(i).X1acier = 0 : .Item(i).X1acierLS = 0
                                    .Item(i).X2acier = 0.2 * PlanCont.Item(i1 - 1).Lx
                                    .Item(i).X2acierLS = .Item(i).X2acier
                                End With
                            Next i
                            For i = 0 To .LitSupB.Count - 1
                                With .LitSupB
                                    .Item(i).X1acier = 0 : .Item(i).X1acierLS = 0
                                    .Item(i).X2acier = 0.2 * PlanCont.Item(i1 - 1).Lx
                                    .Item(i).X2acierLS = .Item(i).X2acier
                                End With
                            Next i
                    End Select
            End Select

        End With

    End Sub
    Private Sub Verification_Acier_sur_4appuis(ByVal i1 As Integer, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        'Variables: i1 = n° de travée ( et non n° d'index !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!)
        '           appui = 1->gauche, 2->droit
        '           j = précise comment se prolonge les aciers sur appui
        '               j=5-> ajout d'acier supplémentaires
        '               j=4->
        '           Amanquant = section d'acier manquant pour satisfaire aux conditions d'appui
        '           cgl = 1,2,3,4  -> caractéristique des aciers sur appui pour la vérification des conditions d'entrainement
        '           utot = somme totale de l'aire des lits d'aciers arrivant sur l'appui
        '           utot1 = (utot) -(somme des lits d'aciers ancrés totalement sur l'appui)

        Dim msg As String = String.Empty
        Dim Appui, Total_Lit As Integer

        Dim agl, angle, FE(4), fyd, Lappui(4), Vapp(2), Z, CotΘ(4) As Double ' 

        Call Effort_Traction_4Appui(i1, CotΘ, FE, Vapp) 'calcul de l'effort horizontal repris par l'appui

        fyd = aci.fyk / aci.γsd
        With PlanCont.Item(i1 - 1)
            Select Case .Type_Acier_Travée
                Case 1
                    With .TS_trav
                        'le lit 1 est toujours ancré sur l'appui par mesure de sécurité en cours de construction
                        .Item(0).X1acier = 0 : .Item(0).X1acierLS = 0
                        .Item(0).X2acier = PlanCont.Item(i1 - 1).Lx : .Item(0).X2acierLS = PlanCont.Item(i1 - 1).Lx
                        If .Count = 2 Then
                            .Item(1).X1acier = 0 : .Item(1).X1acierLS = 0
                            .Item(1).X2acier = PlanCont.Item(i1 - 1).Lx : .Item(1).X2acierLS = PlanCont.Item(i1 - 1).Lx
                            Total_Lit = 2
                            msg = "2  lits inférieurs."
                        Else
                            msg = "1 seul lit inférieur."
                            Total_Lit = 1
                        End If
                    End With
                Case 2
                    With .LitInf
                        'le lit 1 est toujours ancré sur l'appui par mesure de sécurité en cours de construction
                        .Item(0).X1acier = 0 : .Item(0).X1acierLS = 0
                        .Item(0).X2acier = PlanCont.Item(i1 - 1).Lx : .Item(0).X2acierLS = PlanCont.Item(i1 - 1).Lx
                        If .Count = 2 Then
                            .Item(1).X1acier = 0 : .Item(1).X1acierLS = 0
                            .Item(1).X2acier = PlanCont.Item(i1 - 1).Lx : .Item(1).X2acierLS = PlanCont.Item(i1 - 1).Lx
                            msg = "2  lits inférieurs."
                            Total_Lit = 2
                        Else
                            msg = "1 seul lit inférieur."
                            Total_Lit = 1
                        End If
                    End With
            End Select
            Z = 0.9 * .dInf
            rtf.WriteStartGroup()
            rtf.NewParagraph()
            rtf.FontSize = 12 : rtf.FontSize = 9
            rtf.Bold = True : rtf.WriteString("  ■ Vérification de l'ancrage des aciers sur appuis gauche, droit, haut et bas") : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
            rtf.WriteString("La travée comporte " & msg) : rtf.NewLine()
            rtf.NewParagraph()
            'Ligne 1
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
            rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
            'Ligne 1
            rtf.BeginRow() : rtf.DeclareCell(8500) : rtf.BeginCell(True) : rtf.WriteString("Pour appui gauche et droit") : rtf.EndCell() : rtf.EndRow()
            'Ligne 2
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As1") : rtf.EndCell()
            rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
            If .Type_Acier_Travée = 1 Then msg = (aci.As_long(.TS_trav.Item(0).indTS)).ToString("#0.00") Else msg = (.LitInf.Item(0).Aire * 10000 / .Ly).ToString("#0.00")
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Aire répartie du lit inférieur n°1") : rtf.EndCell() : rtf.EndRow()
            If Total_Lit = 2 Then
                'Ligne 2
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As2") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                If .Type_Acier_Travée = 1 Then msg = (aci.As_long(.TS_trav.Item(1).indTS)).ToString("#0.00") Else msg = (.LitInf.Item(1).Aire * 10000 / .Ly).ToString("#0.00")
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Aire répartie du lit inférieur n°2") : rtf.EndCell() : rtf.EndRow()
            End If
            'Ligne 3
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("z") : rtf.EndCell()
            rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("m") : rtf.EndCell()
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Z.ToString("#0.000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(" z = 0.9.d - Identique pour tous les appuis") : rtf.EndCell() : rtf.EndRow()
            rtf.WriteEndGroup()
            For Appui = 1 To 2
                rtf.WriteStartGroup()
                rtf.FontSize = 12 : rtf.FontSize = 9
                'initialisation
                If Appui = 1 Then msg = "Appui gauche" Else msg = "Appui droit"
                'Ligne 5
                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                'Ligne 2
                angle = Atan(1 / CotΘ(Appui)) * 180 / PI
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Θ'") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("°") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(angle.ToString("#0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("angle de la bielle moyenne d'appui") : rtf.EndCell() : rtf.EndRow()
                'Ligne 5
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("|Ved|") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
                If Vapp(1) < 0 Then Vapp(1) = -Vapp(1)
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Vapp(1) * 1000).ToString("#,##0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("au nu de l'appui - En valeur absolue.") : rtf.EndCell() : rtf.EndRow()
                'Ligne 6
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("FE") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "<0" Else msg = (FE(Appui) * 1000).ToString("#,##0.00")
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "Uniquement disposition constructive applicable (voir ci-dessous)" Else msg = " Effort de traction à ancrer sur l'appui"
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                agl = FE(Appui) / fyd
                'Ligne 7
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Agl") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
                If FE(Appui) < 0 Then
                    If .Type_Acier_Travée = 1 Then msg = (aci.As_long(.TS_trav.Item(0).indTS)).ToString("#0.00") Else msg = (.LitInf.Item(0).Aire * 10000).ToString("#0.00")
                    rtf.WriteString(msg)
                    agl = 0
                End If
                msg = (agl * 10000.0).ToString("#,##0.00")
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "1ier lit prolongé sur appui (disp. constructive)" Else msg = "Section minimale pour reprise effort de traction"
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                rtf.WriteEndGroup()
                'ancrage total lit 1 quelque soit le cas de figure
                Lappui(1) = PlanCont.Item(i1 - 1).LappG - PlanCont.Item(i1 - 1).enrLat
                Lappui(2) = PlanCont.Item(i1 - 1).LappD - PlanCont.Item(i1 - 1).enrLat
                Call Ancrage_4appui_About(i1, Appui, agl, fyd, FE, Lappui, rtf)

            Next Appui
            rtf.WriteStartGroup()
            rtf.FontSize = 12 : rtf.FontSize = 9
            '
            '---->   Pour appui haut et bas
            'Ligne 1
            rtf.BeginRow() : rtf.DeclareCell(8500) : rtf.BeginCell(True) : rtf.WriteString("Pour appui haut et bas") : rtf.EndCell() : rtf.EndRow()
            'Ligne 2
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As1") : rtf.EndCell()
            rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
            If .Type_Acier_Travée = 1 Then msg = (aci.As_trans(.TS_trav.Item(0).indTS)).ToString("#0.00") Else msg = (.LitInf_Y.Item(0).Aire * 10000 / .Lx).ToString("#0.00")
            rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Aire répartie du lit inférieur n°1") : rtf.EndCell() : rtf.EndRow()
            If Total_Lit = 2 Then
                'Ligne 2
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As2") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
                If .Type_Acier_Travée = 1 Then msg = (aci.As_trans(.TS_trav.Item(1).indTS)).ToString("#0.00") Else msg = (.LitInf_Y.Item(1).Aire * 10000 / .Lx).ToString("#0.00")
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Aire répartie du lit inférieur n°2") : rtf.EndCell() : rtf.EndRow()
            End If
            rtf.WriteEndGroup()
            For Appui = 3 To 4
                rtf.WriteStartGroup()
                rtf.FontSize = 12 : rtf.FontSize = 9
                'initialisation
                If Appui = 3 Then msg = "Appui haut" Else msg = "Appui bas"
                'Ligne 5
                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                'Ligne 2
                angle = Atan(1 / CotΘ(Appui)) * 180 / PI
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Θ'") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("°") : rtf.EndCell()
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(angle.ToString("#0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("angle de la bielle moyenne d'appui") : rtf.EndCell() : rtf.EndRow()
                'Ligne 5
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("|Ved|") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
                If Vapp(2) < 0 Then Vapp(2) = -Vapp(2)
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Vapp(2) * 1000).ToString("#,##0.00")) : rtf.EndCell()
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("au nu de l'appui - En valeur absolue.") : rtf.EndCell() : rtf.EndRow()
                'Ligne 6
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("FE") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "<0" Else msg = (FE(Appui) * 1000).ToString("#,##0.00")
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "Uniquement disposition constructive applicable (voir ci-dessous)" Else msg = " Effort de traction à ancrer sur l'appui"
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                agl = FE(Appui) / fyd
                'Ligne 7
                rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Agl") : rtf.EndCell()
                rtf.DeclareCell(800) : rtf.BeginCell(True) : rtf.WriteString("cm²") : rtf.EndCell()
                If FE(Appui) < 0 Then
                    If .Type_Acier_Travée = 1 Then msg = (aci.As_trans(.TS_trav.Item(0).indTS)).ToString("#0.00") Else msg = (.LitInf_Y.Item(0).Aire * 10000).ToString("#0.00")
                    agl = 0
                End If
                msg = (agl * 10000.0).ToString("#,##0.00")
                rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell()
                If FE(Appui) < 0 Then msg = "1ier lit prolongé sur appui (disp. constructive)" Else msg = "Section minimale pour reprise effort de traction"
                rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                rtf.WriteEndGroup()
                'ancrage total lit 1 quelque soit le cas de figure
                Lappui(3) = PlanCont.Item(i1 - 1).LappH - PlanCont.Item(i1 - 1).enrLat
                Lappui(4) = PlanCont.Item(i1 - 1).LappB - PlanCont.Item(i1 - 1).enrLat
                Call Ancrage_4appui_About(i1, Appui, agl, fyd, FE, Lappui, rtf)
            Next Appui
        End With
    End Sub
    '''<summary>Calcul à l'effort de traction de l'armature sur appui</summary>
    '''<param name="CotΘ">angle de la bielle</param>
    '''<param name="i1">n° de travée</param>
    '''<param name="FE">effort dans l'armature, =1 appui gauche, =2 appui droit, =3 appui haut, =4 appui bas</param>
    '''<param name="VEd">effort tranchant, =1 appui gauche et droit, =2 appui haut et bas</param>
    Private Sub Effort_Traction_4Appui(ByVal i1 As Integer, ByRef CotΘ() As Double, ByRef FE() As Double, ByRef VEd() As Double)

        Dim z, Lappui(4) As Double ''
        'Θ_Tr = 45° angle de la bielle:  α_tr = 90° acier effort tranchant verticaux        'a1 = d (§9.3.1(4)) soit  a1/z = 1/0.9
        'plancher isostatique
        With PlanCont.Item(i1 - 1)
            z = 0.9 * .dInf
            ' appui gauche
            Lappui(1) = .LappG - .enrLat
            CotΘ(1) = (Lappui(1) + z * Cotgt(.Θ_Tr)) / (2 * z)
            VEd(1) = Max(Abs(.ELU_Str_Env.Vx_min.V), Abs(.ELU_Str_Env.Vx_Max.V))
            FE(1) = Abs(VEd(1)) * CotΘ(1)   'Effort de traction développé sur l'appui
            If FE(1) < 0 Then FE(1) = 0 'pour éviter toute valeur négative
            'appui droit
            Lappui(2) = .LappD - .enrLat
            CotΘ(2) = (Lappui(2) + z * Cotgt(.Θ_Tr)) / (2 * z)
            FE(2) = Abs(VEd(1)) * CotΘ(2)
            'appui haut
            VEd(2) = Max(Abs(.ELU_Str_Env.Vy_min.V), Abs(.ELU_Str_Env.Vy_Max.V))
            Lappui(3) = .LappH - .enrLat
            CotΘ(3) = (Lappui(3) + z * Cotgt(.Θ_Tr)) / (2 * z)
            FE(3) = Abs(VEd(2)) * CotΘ(3)
            'appui bas
            Lappui(4) = .LappB - .enrLat
            CotΘ(4) = (Lappui(4) + z * Cotgt(.Θ_Tr)) / (2 * z)
            FE(4) = Abs(VEd(2)) * CotΘ(4)
        End With
        Erase Lappui
    End Sub
    Private Sub Ancrage_4appui_About(ByVal i1 As Integer, ByVal appui As Integer, ByVal agl As Double, ByVal fyd As Double, ByVal FE() As Double,
                                    ByVal Lappui() As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim Total_Lit As Integer
        Dim msg As String
        Dim LacierTot, u, u1 As Double
        rtf.WriteStartGroup()
        'rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        With PlanCont.Item(i1 - 1)
            If appui > 2 Then
                Select Case .Type_Acier_Travée
                    Case 1
                        u = aci.As_trans(.TS_trav.Item(0).indTS) : Total_Lit = 1
                        If .TS_trav.Count > 1 Then u1 = aci.As_trans(.TS_trav.Item(1).indTS) : Total_Lit = 2
                    Case 2
                        u = .LitInf_Y.Item(0).Aire : Total_Lit = 1
                        If .LitInf_Y.Count = 2 Then u1 = .LitInf_Y.Item(1).Aire : Total_Lit = 2
                End Select

            Else
                Select Case .Type_Acier_Travée
                    Case 1
                        u = aci.As_long(.TS_trav.Item(0).indTS) : Total_Lit = 1
                        If .TS_trav.Count > 1 Then u1 = aci.As_long(.TS_trav.Item(1).indTS) : Total_Lit = 2
                    Case 2
                        u = .LitInf.Item(0).Aire : Total_Lit = 1
                        If .LitInf.Count = 2 Then u1 = .LitInf.Item(1).Aire : Total_Lit = 2
                End Select
            End If

            If agl <= u Then
                fyd = FE(appui) / u
                Call Ancrage_lit_4appuis(appui, i1, 1, Lappui(appui), 1, 1, fyd, LacierTot)
                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Agl < As1  -> Lit n°1 totalement ancré") : rtf.EndCell() : rtf.EndRow()
            ElseIf Total_Lit > 1 Then
                If agl < (u + u1) Then
                    'ancrage total du premier lit sur la limite élastique
                    Call Ancrage_lit_4appuis(appui, i1, 1, Lappui(appui), 1, 1, fyd, LacierTot)
                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(" Agl < As1 + As2 - Ancrage des 2 lits n°1 et n°2") : rtf.EndCell() : rtf.EndRow()
                    'ancrage lit 2  
                    fyd = (FE(appui) - u * fyd) / u1
                    Call Ancrage_lit_4appuis(appui, i1, 2, Lappui(appui), 1, 1, fyd, LacierTot)
                    '___________________________________________________________________________________________________________________________________________
                    ' "Ancrage appui Intermediaire travée n° i1 
                    ' L'ancrage sur appui intemédiaire - nécessite des armatures complémentaires. Disposition non implémentée dans cette version logicielle!"
                End If
            Else
                Dim k As Integer = 0
                Select Case .Type_Acier_Travée
                    Case 1
                        Select Case .TS_trav.Count
                            Case 1 '1 lit
                                'pour permettre la suite du calcul
                                If appui = 1 Then
                                    LacierTot = Truncate(Lappui(1) * 50) / 100
                                    .TS_trav.Item(k).X1acier = 0 : .TS_trav.Item(k).X1acierLS = 0
                                    .TS_trav.Item(k).L13acier = LacierTot : .TS_trav.Item(k).L12acier = 0 : .TS_trav.Item(k).L11acier = 0
                                Else
                                    LacierTot = Truncate(Lappui(2) * 50) / 100
                                    .TS_trav.Item(k).X2acier = .Lx : .TS_trav.Item(k).X2acierLS = .Lx
                                    .TS_trav.Item(k).L23acier = LacierTot : .TS_trav.Item(k).L22acier = 0 : .TS_trav.Item(k).L21acier = 0
                                End If
                                msg = " Agl > As1 et 1 seul lit d'armature dimensionné! "
                            Case 2 ' 2 Lits
                                'pour permettre la suite du calcul
                                If appui = 1 Then
                                    LacierTot = Truncate(Lappui(1) * 50) / 100
                                    For k = 0 To 1
                                        .TS_trav.Item(k).X1acier = 0 : .TS_trav.Item(k).X1acierLS = 0
                                        .TS_trav.Item(k).L13acier = LacierTot : .TS_trav.Item(k).L12acier = 0 : .TS_trav.Item(k).L11acier = 0
                                    Next
                                Else
                                    LacierTot = Truncate(Lappui(2) * 50) / 100
                                    For k = 0 To 1
                                        .TS_trav.Item(k).X2acier = .Lx : .TS_trav.Item(k).X2acierLS = .Lx
                                        .TS_trav.Item(k).L23acier = LacierTot : .TS_trav.Item(k).L22acier = 0 : .TS_trav.Item(k).L21acier = 0
                                    Next
                                End If
                                msg = " Agl > As1 + As2 et 2 lits d'armature dimensionnés! "
                        End Select
                    Case 2
                        If appui > 2 Then
                            Select Case .LitInf_Y.Count
                                Case 1 '1 lit
                                    'pour permettre la suite du calcul
                                    If appui = 3 Then
                                        LacierTot = Truncate(Lappui(3) * 50) / 100
                                        .LitInf_Y.Item(k).X1acier = 0 : .LitInf_Y.Item(k).X1acierLS = 0
                                        .LitInf_Y.Item(k).L13acier = LacierTot : .LitInf_Y.Item(k).L12acier = 0 : .LitInf_Y.Item(k).L11acier = 0
                                    Else
                                        LacierTot = Truncate(Lappui(4) * 50) / 100
                                        .LitInf_Y.Item(k).X2acier = .Ly : .LitInf_Y.Item(k).X2acierLS = .Ly
                                        .LitInf_Y.Item(k).L23acier = LacierTot : .LitInf_Y.Item(k).L22acier = 0 : .LitInf_Y.Item(k).L21acier = 0
                                    End If
                                    msg = " Agl > As1 et 1 seul lit d'armature dimensionné! "
                                Case 2 ' 2 Lits
                                    'pour permettre la suite du calcul
                                    If appui = 1 Then
                                        LacierTot = Truncate(Lappui(1) * 50) / 100
                                        For k = 0 To 1
                                            .LitInf_Y.Item(k).X1acier = 0 : .LitInf_Y.Item(k).X1acierLS = 0
                                            .LitInf_Y.Item(k).L13acier = LacierTot : .LitInf_Y.Item(k).L12acier = 0 : .LitInf_Y.Item(k).L11acier = 0
                                        Next
                                    Else
                                        LacierTot = Truncate(Lappui(2) * 50) / 100
                                        For k = 0 To 1
                                            With .LitInf_Y
                                                .Item(k).X2acier = PlanCont.Item(i1 - 1).Lx : .Item(k).X2acierLS = PlanCont.Item(i1 - 1).Lx
                                                .Item(k).L23acier = LacierTot : .Item(k).L22acier = 0 : .Item(k).L21acier = 0
                                            End With
                                        Next
                                    End If
                                    msg = " Agl > As1 + As2 et 2 lits d'armature dimensionnés! "
                            End Select
                        Else
                            Select Case .LitInf.Count
                                Case 1 '1 lit
                                    'pour permettre la suite du calcul
                                    If appui = 1 Then
                                        LacierTot = Truncate(Lappui(1) * 50) / 100
                                        .LitInf.Item(k).X1acier = 0 : .LitInf.Item(k).X1acierLS = 0
                                        .LitInf.Item(k).L13acier = LacierTot : .LitInf.Item(k).L12acier = 0 : .LitInf.Item(k).L11acier = 0
                                    Else
                                        LacierTot = Truncate(Lappui(2) * 50) / 100
                                        .LitInf.Item(k).X2acier = .Lx : .LitInf.Item(k).X2acierLS = .Lx
                                        .LitInf.Item(k).L23acier = LacierTot : .LitInf.Item(k).L22acier = 0 : .LitInf.Item(k).L21acier = 0
                                    End If
                                    msg = " Agl > As1 et 1 seul lit d'armature dimensionné! "
                                Case 2 ' 2 Lits
                                    'pour permettre la suite du calcul
                                    If appui = 1 Then
                                        LacierTot = Truncate(Lappui(1) * 50) / 100
                                        For k = 0 To 1
                                            .LitInf.Item(k).X1acier = 0 : .LitInf.Item(k).X1acierLS = 0
                                            .LitInf.Item(k).L13acier = LacierTot : .LitInf.Item(k).L12acier = 0 : .LitInf.Item(k).L11acier = 0
                                        Next
                                    Else
                                        LacierTot = Truncate(Lappui(2) * 50) / 100
                                        For k = 0 To 1
                                            .LitInf.Item(k).X2acier = .Lx : .LitInf.Item(k).X2acierLS = .Lx
                                            .LitInf.Item(k).L23acier = LacierTot : .LitInf.Item(k).L22acier = 0 : .LitInf.Item(k).L21acier = 0
                                        Next
                                    End If
                                    msg = " Agl > As1 + As2 et 2 lits d'armature dimensionnés! "
                            End Select
                        End If
                End Select

                rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
                Select Case appui
                    Case 1
                        msg = "gauche"
                    Case 2
                        msg = "droit"
                    Case 3
                        msg = "haut"
                    Case 4
                        msg = "bas"
                End Select
                msg = "Ancrage appui About travée n°" & i1.ToString & " :" & Environment.NewLine &
                    " L'ancrage sur appui de rive " & msg & " nécessite soit des armatures complémentaires ou soit l'ancrage du 2ième lit. Disposition non implémentée dans cette version logicielle!"
                Aff.msg_Avertiss.Add(msg)

            End If
        End With
        rtf.WriteEndGroup()
    End Sub
    ''' <summary>Détermination des longueurs d'ancrage sur les 4 appuis, en partie inférieure et en partie supérieure</summary>
    '''  <param name="appui">n° d'appui pour les aciers inférieurs, 1= appui gauche, 2 = app.droit, 3=app. haut, 4=app. bas</param>
    '''  <param name="i1">n° de travée</param>
    '''  <param name="calcul_Ancrage">calcul_Ancrage = numéro de lit;  calcul_Ancrage = indique si des aciers supplémentaires sont rajoutées pour satisfaire aux conditions d'appui (5) ou si des aciers des lits en travée sont simplement prolongés sur l'appui </param>
    '''  <param name="lappui">Longueur de l'appui </param>
    '''  <param name="j">n°  </param>
    '''  <param name="ELU_ELS"> e</param>
    '''  <param name="σsd">contrainte de l'acier</param>
    '''  <param name="LacierTot"> e</param>
    '''  <param name="Partie_Courante">Permet de déterminer la valeur du coef α4 pour les TS, réduit si vrai sinon pris égal à 1 </param>
    Private Sub Ancrage_lit_4appuis(ByVal appui As Integer, ByVal i1 As Integer, ByVal calcul_Ancrage As Integer, ByVal lappui As Double, ByVal j As Integer,
                            ByVal ELU_ELS As Integer, ByVal σsd As Double, Optional ByRef LacierTot As Double = 0, Optional ByVal Partie_Courante As Boolean = False)
        '------------------------------------------------------------------
        '                    ancrage total du lit
        '------------------------------------------------------------------
        'Variables: appui=1->appuigauche, 2->droit
        '           i1=n° de travée ( et non n° d'index !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!)
        '           nlit = calcul_Ancrage = numéro de lit
        '           j = type de lit (inf, sup gauche, sup droit)
        '           calcul_Ancrage = indique si des aciers supplémentaires sont rajoutées pour satisfaire aux conditions d'appui (5)  
        '                             ou si des aciers des lits en travée sont simplement prolongés sur l'appui
        Dim l1, l2, l3 As Double
        Dim j1, appui1 As Integer
        l1 = 0 : l2 = 0 : l3 = 0
        j1 = j : appui1 = 0
        If j = 1 And appui > 2 Then j1 = 7 : appui1 = 3
        If Lscel_droit(calcul_Ancrage - 1, j1, i1, ELU_ELS, σsd, Partie_Courante) > lappui Then '---------------->  SI ANCRAGE COURBE
            Call Ancrage_courbe(lappui, calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, l1, l2, l3, True, appui1)  'rapatrie les valeurs l1,l2,l3
        Else  '---------------->  ALORS ANCRAGE DROIT
            l1 = 0 : l2 = 0 : l3 = -10
        End If
        'détermination des abscisses des armatures
        With PlanCont.Item(i1 - 1)
            Select Case j
                Case 1 'Armatures inférieures: ancrage total du premier, deuxieme,troisième lit en travée
                    Select Case appui
                        Case 1 'appui gauche
                            Select Case .Type_Acier_Travée
                                Case 1
                                    .TS_trav.Item(calcul_Ancrage - 1).X1acier = 0 : .TS_trav.Item(calcul_Ancrage - 1).X1acierLS = 0
                                    If l3 = -10 Then
                                        .TS_trav.Item(calcul_Ancrage - 1).L13acier = Lscel_droit(calcul_Ancrage - 1, 1, i1, ELU_ELS, σsd, False)
                                    Else
                                        .TS_trav.Item(calcul_Ancrage - 1).L13acier = l3
                                    End If
                                    .TS_trav.Item(calcul_Ancrage - 1).L12acier = l2 : .TS_trav.Item(calcul_Ancrage - 1).L11acier = l1
                                    LacierTot = .TS_trav.Item(calcul_Ancrage - 1).L13acier + .TS_trav.Item(calcul_Ancrage - 1).L12acier + .TS_trav.Item(calcul_Ancrage - 1).L11acier
                                Case 2
                                    .LitInf.Item(calcul_Ancrage - 1).X1acier = 0 : .LitInf.Item(calcul_Ancrage - 1).X1acierLS = 0
                                    If l3 = -10 Then
                                        .LitInf.Item(calcul_Ancrage - 1).L13acier = Lscel_droit(calcul_Ancrage - 1, 1, i1, ELU_ELS, σsd, False)
                                    Else
                                        .LitInf.Item(calcul_Ancrage - 1).L13acier = l3
                                    End If
                                    .LitInf.Item(calcul_Ancrage - 1).L12acier = l2 : .LitInf.Item(calcul_Ancrage - 1).L11acier = l1
                                    LacierTot = .LitInf.Item(calcul_Ancrage - 1).L13acier + .LitInf.Item(calcul_Ancrage - 1).L12acier + .LitInf.Item(calcul_Ancrage - 1).L11acier
                            End Select
                        Case 2 'appui droit
                            Select Case .Type_Acier_Travée
                                Case 1
                                    With .TS_trav
                                        .Item(calcul_Ancrage - 1).X2acier = PlanCont.Item(i1 - 1).Lx
                                        .Item(calcul_Ancrage - 1).X2acierLS = PlanCont.Item(i1 - 1).Lx
                                        If l3 = -10 Then
                                            .Item(calcul_Ancrage - 1).L23acier = Lscel_droit(calcul_Ancrage - 1, 1, i1, ELU_ELS, σsd, False)
                                        Else
                                            .Item(calcul_Ancrage - 1).L23acier = l3
                                        End If
                                        .Item(calcul_Ancrage - 1).L22acier = l2 : .Item(calcul_Ancrage - 1).L21acier = l1
                                        LacierTot = .Item(calcul_Ancrage - 1).L23acier + .Item(calcul_Ancrage - 1).L22acier + .Item(calcul_Ancrage - 1).L21acier
                                    End With
                                Case 2
                                    With .LitInf
                                        .Item(calcul_Ancrage - 1).X2acier = PlanCont.Item(i1 - 1).Lx
                                        .Item(calcul_Ancrage - 1).X2acierLS = PlanCont.Item(i1 - 1).Lx
                                        If l3 = -10 Then
                                            .Item(calcul_Ancrage - 1).L23acier = Lscel_droit(calcul_Ancrage - 1, 1, i1, ELU_ELS, σsd, False)
                                        Else
                                            .Item(calcul_Ancrage - 1).L23acier = l3
                                        End If
                                        .Item(calcul_Ancrage - 1).L22acier = l2 : .Item(calcul_Ancrage - 1).L21acier = l1
                                        LacierTot = .Item(calcul_Ancrage - 1).L23acier + .Item(calcul_Ancrage - 1).L22acier + .Item(calcul_Ancrage - 1).L21acier
                                    End With
                            End Select
                        Case 3, 4 'Appuis haut et bas
                            Select Case .Type_Acier_Travée
                                Case 1
                                    With .TS_trav
                                        If l3 = -10 Then
                                            .Item(calcul_Ancrage - 1).L33acier = Lscel_droit(calcul_Ancrage - 1, 7, i1, ELU_ELS, σsd, False)
                                        Else
                                            .Item(calcul_Ancrage - 1).L33acier = l3
                                        End If
                                        .Item(calcul_Ancrage - 1).L32acier = l2 : .Item(calcul_Ancrage - 1).L31acier = l1
                                    End With
                                Case 2
                                    With .LitInf_Y
                                        .Item(calcul_Ancrage - 1).X1acier = 0 : .Item(calcul_Ancrage - 1).X1acierLS = 0
                                        .Item(calcul_Ancrage - 1).X2acier = PlanCont.Item(i1 - 1).Ly : .Item(calcul_Ancrage - 1).X2acierLS = PlanCont.Item(i1 - 1).Ly
                                        If l3 = -10 Then
                                            .Item(calcul_Ancrage - 1).L13acier = Lscel_droit(calcul_Ancrage - 1, 7, i1, ELU_ELS, σsd, False)
                                            .Item(calcul_Ancrage - 1).L23acier = Lscel_droit(calcul_Ancrage - 1, 7, i1, ELU_ELS, σsd, False)
                                        Else
                                            .Item(calcul_Ancrage - 1).L13acier = l3 : .Item(calcul_Ancrage - 1).L23acier = l3
                                        End If
                                        .Item(calcul_Ancrage - 1).L12acier = l2 : .Item(calcul_Ancrage - 1).L11acier = l1
                                        .Item(calcul_Ancrage - 1).L22acier = l2 : .Item(calcul_Ancrage - 1).L21acier = l1
                                        LacierTot = .Item(calcul_Ancrage - 1).L23acier + .Item(calcul_Ancrage - 1).L22acier + .Item(calcul_Ancrage - 1).L21acier
                                    End With
                            End Select
                    End Select
                Case 2 'ancrage total du premier lit sur appui gauche pour la travée 1
                    Select Case .Type_Acier_Appui_Gauche
                        Case 1
                            With .TS_AppG
                                .Item(calcul_Ancrage - 1).X1acier = 0 : .Item(calcul_Ancrage - 1).X1acierLS = 0
                                .Item(calcul_Ancrage - 1).X2acier = Truncate(0.2 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100
                                .Item(calcul_Ancrage - 1).X2acierLS = .Item(calcul_Ancrage - 1).X2acier - Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True)
                                If l3 = -10 Then
                                    .Item(calcul_Ancrage - 1).L13acier = Truncate(Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True) * 100) / 100
                                Else
                                    .Item(calcul_Ancrage - 1).L13acier = Truncate(l3 * 100) / 100 'en mètre
                                End If
                                .Item(calcul_Ancrage - 1).L12acier = Truncate(l2 * 100) / 100 : .Item(calcul_Ancrage - 1).L11acier = Truncate(l1 * 100) / 100
                            End With
                        Case 2
                            .LitSupG.Item(calcul_Ancrage - 1).X1acier = 0
                            .LitSupG.Item(calcul_Ancrage - 1).X1acierLS = 0
                            .LitSupG.Item(calcul_Ancrage - 1).X2acier = Truncate(0.2 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100
                            .LitSupG.Item(calcul_Ancrage - 1).X2acierLS = .LitSupG.Item(calcul_Ancrage - 1).X2acier - Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, False)
                            If l3 = -10 Then
                                .LitSupG.Item(calcul_Ancrage - 1).L13acier = Truncate(Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True) * 100) / 100
                            Else
                                .LitSupG.Item(calcul_Ancrage - 1).L13acier = Truncate(l3 * 100) / 100 'en mètre
                            End If
                            .LitSupG.Item(calcul_Ancrage - 1).L12acier = Truncate(l2 * 100) / 100 : .LitSupG.Item(calcul_Ancrage - 1).L11acier = Truncate(l1 * 100) / 100
                    End Select
                Case 3 'ancrage total du premier lit sur appui droit pour la dernière travée
                    'appui droit légèrement encastré ->0,15xM0
                    Select Case .Type_Acier_Appui_Droit
                        Case 1
                            With .TS_AppD
                                .Item(calcul_Ancrage - 1).X2acier = PlanCont.Item(i1 - 1).Lx  '+ 0.02 + la(2)
                                .Item(calcul_Ancrage - 1).X2acierLS = PlanCont.Item(i1 - 1).Lx
                                .Item(calcul_Ancrage - 1).X1acier = Truncate(0.8 * PlanCont.Item(i1 - 1).Lx * 100 - 1) / 100
                                .Item(calcul_Ancrage - 1).X1acierLS = .Item(calcul_Ancrage - 1).X1acier + Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True)
                                If l3 = -10 Then
                                    .Item(calcul_Ancrage - 1).L23acier = Truncate(Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True) * 100) / 100
                                Else
                                    .Item(calcul_Ancrage - 1).L23acier = Truncate(l3 * 100) / 100 'en mètre
                                End If
                                .Item(calcul_Ancrage - 1).L22acier = Truncate(l2 * 100) / 100 : .Item(calcul_Ancrage - 1).L21acier = Truncate(l1 * 100) / 100
                            End With
                        Case 2
                            .LitSupD.Item(calcul_Ancrage - 1).X2acier = .Lx  '+ 0.02 + la(2)
                            .LitSupD.Item(calcul_Ancrage - 1).X2acierLS = .Lx
                            .LitSupD.Item(calcul_Ancrage - 1).X1acier = Truncate(0.8 * PlanCont.Item(i1 - 1).Lx * 100 - 1) / 100
                            .LitSupD.Item(calcul_Ancrage - 1).X1acierLS = .LitSupD.Item(calcul_Ancrage - 1).X1acier + Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, False)
                            If l3 = -10 Then
                                .LitSupD.Item(calcul_Ancrage - 1).L23acier = Truncate(Lscel_droit(calcul_Ancrage - 1, j, i1, ELU_ELS, σsd, True) * 100) / 100
                            Else
                                .LitSupD.Item(calcul_Ancrage - 1).L23acier = Truncate(l3 * 100) / 100 'en mètre
                            End If
                            .LitSupD.Item(calcul_Ancrage - 1).L22acier = Truncate(l2 * 100) / 100
                            .LitSupD.Item(calcul_Ancrage - 1).L21acier = Truncate(l1 * 100) / 100
                    End Select
                Case 5 'acier de glissement supplémentaire appui gauche
                    With .LitAppG
                        .X1acier = 0
                        .X1acierLS = 0
                        If l3 = -10 Then
                            .L13acier = Lscel_droit(0, 5, i1, ELU_ELS, σsd, False)
                        Else
                            .L13acier = l3
                        End If
                        .L12acier = l2
                        .L11acier = l1
                        LacierTot = .L13acier
                    End With
                Case 6 'acier de glissement supplémentaire appui droit
                    With .LitAppD
                        .X2acier = PlanCont.Item(i1 - 1).Lx
                        .X2acierLS = PlanCont.Item(i1 - 1).Lx
                        If l3 = -10 Then
                            .L23acier = Lscel_droit(0, 6, i1, ELU_ELS, σsd, False)
                        Else
                            .L23acier = l3
                        End If
                        .L22acier = l2
                        .L21acier = l1
                        LacierTot = .L23acier
                    End With
                Case 40, 41 ' aciers sur appuis haut et bas
                    If j = 4 Then
                        Select Case .Type_Acier_Travée
                            Case 1
                                If .TS_AppH.Count >= 1 Then
                                    With .TS_AppH.Item(0)
                                        .X2acier = Truncate(0.1 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100
                                        .X2acierLS = Truncate((.X2acier - Lscel_droit(0, j, i1, ELU_ELS, σsd, True)) * 100) / 100
                                        If l3 = -10 Then
                                            .L13acier = Lscel_droit(0, j, i1, ELU_ELS, σsd, True) 'en m
                                        Else
                                            .L13acier = Truncate((l3 - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        End If
                                        .L12acier = Truncate(l2 * 100) / 100 'en m
                                        .L11acier = Truncate(l1 * 100) / 100 'en m
                                    End With
                                End If
                                If .TS_AppB.Count >= 1 Then
                                    With .TS_AppB.Item(0)
                                        .X2acier = Truncate(0.1 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100
                                        .X2acierLS = Truncate((.X2acier - Lscel_droit(0, j, i1, ELU_ELS, σsd, True)) * 100) / 100
                                        If l3 = -10 Then
                                            .L13acier = Lscel_droit(0, j, i1, ELU_ELS, σsd, True) 'en m
                                        Else
                                            .L13acier = Truncate((l3 - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        End If
                                        .L12acier = Truncate(l2 * 100) / 100 'en m
                                        .L11acier = Truncate(l1 * 100) / 100 'en m
                                    End With
                                End If
                            Case 2 'HA
                                If .LitSupH.Count >= 1 Then
                                    With .LitSupH.Item(0)
                                        .X2acier = Truncate(0.1 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100
                                        .X2acierLS = Truncate((.X2acier - Lscel_droit(0, j, i1, ELU_ELS, σsd, False)) * 100) / 100
                                        If l3 = -10 Then
                                            .L13acier = Truncate((Lscel_droit(0, 4, i1, 1, 0.8 * aci.fyk, False) - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        Else
                                            .L13acier = Truncate((l3 - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        End If
                                        .L12acier = Truncate(l2 * 100) / 100 'en m
                                        .L11acier = Truncate(l1 * 100) / 100 'en m
                                    End With
                                End If
                                If .LitSupB.Count >= 1 Then
                                    With .LitSupB.Item(0)
                                        .X2acier = Truncate(0.1 * PlanCont.Item(i1 - 1).Lx * 100 + 1) / 100
                                        .X2acierLS = Truncate((.X2acier - Lscel_droit(0, j, i1, ELU_ELS, σsd, False)) * 100) / 100
                                        If l3 = -10 Then
                                            .L13acier = Truncate((Lscel_droit(0, 4, i1, 1, 0.8 * aci.fyk, False) - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        Else
                                            .L13acier = Truncate((l3 - (PlanCont.Item(i1 - 1).enrSup + 0.01) * Cotgt(PlanCont.Item(i1 - 1).Θ_Tr)) * 100) / 100 'en m
                                        End If
                                        .L12acier = Truncate(l2 * 100) / 100 'en m
                                        .L11acier = Truncate(l1 * 100) / 100 'en m
                                    End With
                                End If
                        End Select
                    End If
            End Select
        End With
    End Sub
    ''' <summary>    '''--->ELS: vérification des aciers de flexion à l'ELS et recalcul si nécessaire</summary>
    '''  <param name="rtf">variable renvoyant le fichier de note de calcul sous format rtf</param>
    Private Sub Calcul_section_ELS_4appuis(ByRef rtf As To_Rtf.RtfAnsiTextWriter)

        Dim i, i1 As Integer 'boucle,courbe_moment
        Dim As_ELS, As_ELU, Ifiss, Mu, MsQP, n, y, σbc, σsc, σs, φeff As Double '
        Dim msg As String = String.Empty
        Dim msg1 As String = String.Empty
        ' ELU_ELS = 3 'Combinaison ELS caractéristiques
        'le calcul se fait en 2 passes:
        ' passe 1 = Ok, le logiciel ecrit directement la note de calcul
        ' si passe 1 = not OK:
        '   * contrainte béton trop élevé: section insuffisante le logiciel adresse un avertissement et ecrit directement la note de calcul
        '  * contrainte acier trop élevée: le logiciel redimensionne les aciers de flexion

        For i1 = 1 To PlanCont.Count
            With PlanCont.Item(i1 - 1)
                For i = 1 To 2
                    'armatures en travée en travée
                    ' i=1 calcul suivant X
                    'i=2 calcul suivant Y
                    Select Case i
                        Case 1
                            Select Case .Repartition
                                Case 1 'Uniforme
                                    Mu = .ELS_Car_Env.Mx_max.M 'Calcul de l'acier pour le moment maxi en travée
                                    MsQP = .ELS_QP_Env.Mx_max.M
                                Case 2 'variable
                                    'Mu = .ELU_Str(0).Mx_Max.M 'Calcul de l'acier pour le moment maxi en travée
                                    'MsQP = .ELS_QP(0).Mx_Max.M
                            End Select
                        Case 2
                            Select Case .Repartition
                                Case 1 'Uniforme
                                    Mu = .ELS_Car_Env.My_Max.M 'Calcul de l'acier pour le moment maxi en travée
                                    MsQP = .ELS_QP_Env.My_Max.M
                                Case 2 'variable
                                    'Mu = .ELU_Str(0).My_Max.M 'Calcul de l'acier pour le moment maxi en travée
                                    'MsQP = .ELS_QP(0).My_Max.M
                            End Select
                    End Select
                    'Les diverses sections d'aciers de la poutre sont toujours calculées  en première partie à l'ELU puis sont vérifiés à l'ELS. 
                    'Si la vérification montre que la section d'acier est insuffisante, le logiciel recalcule les nouvelles sections d'aciers nécessaire
                    Select Case aci.n_Forcer_Cont
                        Case True
                            If aci.n_Cont < 2 Then
                                aci.n_Cont = 15
                                Aff.msg_Avertiss.Add("Coefficient d'équivalence n: non renseigné - Pris égal à 15 par défaut")
                            End If
                            n = aci.n_Cont
                        Case False
                            Dim Eceff, φinf As Double
                            Call Calcul_Coef_Fluage(Aff.t0, Aff.RH, Aff.tinf, φinf, rtf)
                            φeff = φinf * (MsQP / Mu) 'suivant Recommandations Professionnelles
                            Eceff = bet.Ecm / (1 + φeff) 'eq 7.20
                            n = 200000 / Eceff
                    End Select
                    If .Prefa = True Then n = 15
                    Call Verification_armature_flexion_ELS(i1, 1, Mu, n, y, Ifiss, σbc, σsc, σs, msg,,, i)
                    If msg <> String.Empty Then
                        msg1 = "Travée " & i1.ToString & " en travée - "
                        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                        rtf.WriteString(msg1 & msg) : rtf.NewLine()
                        rtf.WriteEndGroup()
                    End If
                    If σs > 0.8 * aci.fyk Then
                        Calcul_armature_flexion_ELS(i1, 1, n, Mu, As_ELS)
                        Select Case i
                            Case 1
                                Select Case .Type_Acier_Travée
                                    Case 1 'TS
                                        As_ELU = aci.As_long(.TS_trav.Item(0).indTS)
                                        If .TS_trav.Count = 2 Then As_ELU = As_ELU + aci.As_long(.TS_trav.Item(1).indTS)
                                    Case 2 'barres HA
                                        As_ELU = (.LitInf.Item(0).Aire) / .Ly
                                        If .LitInf.Count = 2 Then As_ELU += (.LitInf.Item(1).Aire) / .Ly
                                End Select
                            Case 2
                                Select Case .Type_Acier_Travée
                                    Case 1 'TS
                                        As_ELU = aci.As_trans(.TS_trav.Item(0).indTS)
                                        If .TS_trav.Count = 2 Then As_ELU = As_ELU + aci.As_trans(.TS_trav.Item(1).indTS)
                                    Case 2 'barres HA
                                        As_ELU = .LitInf_Y.Item(0).Aire / .Lx
                                        If .LitInf_Y.Count = 2 Then As_ELU += (.LitInf_Y.Item(1).Aire) / .Lx
                                End Select
                        End Select
                        If As_ELS > As_ELU Then
                            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
                            rtf.WriteString(" - travée" & i1.ToString & " - armatures en travée:  As(ELU) = " & (As_ELU * 10000).ToString("#0.00") & " cm² < " &
                                                    (As_ELS * 10000).ToString("#0.00") & " cm² - Modification des lits d'armatures:") : rtf.NewLine()
                            rtf.WriteEndGroup()
                            msg = "Calcul section ELS - armatures en travée -  travée" & i1.ToString & msg & " : σs > 0,8.fyk car As(ELS) (= " & (As_ELS * 10000.0).ToString("#0.00") & " cm²) > As(ELU)(= " &
                                 (As_ELU * 10000.0).ToString("#0.00") & " cm²)"
                            Aff.msg_Avertiss.Add(msg)
                        Else
                            msg = " - armatures en travée - "
                            msg = "Calcul section ELS - travée" & i1.ToString & msg & "incohérence dans le calcul: σs > 0,8.fyk et As(ELU)>As(ELS) - voir module de calcul Private Sub Calcul_section_ELS"
                            Aff.msg_Avertiss.Add(msg)
                        End If
                    End If
                    Tab_section_ELS_4appuis(i1, i, Mu, y, Ifiss, σbc, σs, rtf) 'Passe 2 - Ecriture de la note de calcul
                Next
            End With
        Next i1

        'si modification d'armature, les calculs sont automatiquement repris dans leur intégralité avec les nouvelles sections. 
        'Le logiciel va donc revenir au début du programme d'ou l'utilisation du Goto
        '  If Modification_Acier_HA = True Then Exit Sub
    End Sub
    Private Sub Tab_section_ELS_4appuis(ByVal i1 As Integer, ByVal i As Integer, ByVal Mu As Double, ByVal y As Double, ByVal Ifiss As Double, ByVal σbc As Double,
                                        ByVal σs As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim A1 As Double
        'Passe 2 - Ecriture de la note de calcul
        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        Dim msg As String = String.Empty
        With PlanCont.Item(i1 - 1)
            'armatures en travée en travée  
            ' i=1 calcul suivant X
            'i=2 calcul suivant Y
            rtf.BeginRow() : rtf.DeclareCell(8750) : rtf.BeginCell(True) : rtf.Bold = True : rtf.WriteString("Travée n°" & i1.ToString) : rtf.Bold = False : rtf.EndCell() : rtf.EndRow()
            If i = 1 Then
                msg = "Vérification contraintes de flexion en travée suivant X"
            Else
                msg = "Vérification contraintes de flexion en travée suivant Y"
            End If
            rtf.BeginRow() : rtf.DeclareCell(8750) : rtf.BeginCell(True) : rtf.WriteString(msg) : rtf.EndCell() : rtf.EndRow()
            'Ligne 2
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ms") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("kN.m") : rtf.EndCell()
            rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((Mu * 1000).ToString("#,##0.00")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("moment Maxi sous combinaisons caractéristiques") : rtf.EndCell() : rtf.EndRow()
            'Ligne 4
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("d") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
            rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((.dInf * 100).ToString("#,##0.0")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("hauteur utile") : rtf.EndCell() : rtf.EndRow()
            'Ligne 5
            Select Case i
                Case 1
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            A1 = aci.As_long(.TS_trav.Item(0).indTS)
                            If .TS_trav.Count = 2 Then A1 += aci.As_long(.TS_trav.Item(1).indTS)
                        Case 2 'barres HA
                            A1 = (.LitInf.Item(0).Aire) / .Ly
                            If .LitInf.Count = 2 Then A1 += (.LitInf.Item(1).Aire) / .Ly
                    End Select
                Case 2
                    Select Case .Type_Acier_Travée
                        Case 1 'TS
                            A1 = aci.As_trans(.TS_trav.Item(0).indTS)
                            If .TS_trav.Count = 2 Then A1 += aci.As_trans(.TS_trav.Item(1).indTS)
                        Case 2 'barres HA
                            A1 = .LitInf_Y.Item(0).Aire / .Lx
                            If .LitInf_Y.Count = 2 Then A1 += (.LitInf_Y.Item(1).Aire) / .Lx
                    End Select
            End Select
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("As") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm²/ml") : rtf.EndCell()
            rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((A1 * 10000).ToString("#,##0.00")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("section acier répartie  ELU armature inférieure") : rtf.EndCell() : rtf.EndRow()
            'Ligne 6
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("y") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
            rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((y * 100).ToString("#0.0")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("axe neutre") : rtf.EndCell() : rtf.EndRow()
            'Ligne 7
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("If") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("dm4") : rtf.EndCell()
            rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString((Ifiss * 10000).ToString("#0.0000")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("inertie fissurée") : rtf.EndCell() : rtf.EndRow()
            'Ligne 8
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("σb") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
            rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString(σbc.ToString("#0.00 ")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True)
            If σbc > 0.6 * bet.fck Then
                If .XD1 = True Or .XD2 = True Or .XD3 = True Or .XS1 = True Or .XS2 = True Or .XS3 = True Or .XF1 = True Or .XF2 = True Or .XF3 = True _
                     Or .XF4 = True Then
                    rtf.WriteString("contrainte béton - σb > 0,6.fck = " & (0.6 * bet.fck).ToString("#0.00") & " MPa - NS") : rtf.EndCell() : rtf.EndRow()
                    msg = "Vérification des contraintes à l'ELS - Travée " & i1.ToString & " : contrainte béton trop forte, modifiez la section."
                    Aff.msg_Avertiss.Add(msg)
                Else
                    rtf.WriteString("contrainte béton - σb > 0,6.fck = " & (0.6 * bet.fck).ToString("#0.00") & " MPa ") : rtf.EndCell() : rtf.EndRow()
                End If
            Else
                rtf.WriteString("contrainte béton - σb < 0,6.fck = " & (0.6 * bet.fck).ToString("#0.00") & " MPa - OK") : rtf.EndCell() : rtf.EndRow()
            End If
            'Ligne 9
            rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("σs") : rtf.EndCell()
            rtf.DeclareCell(900) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
            rtf.DeclareCell(1150) : rtf.BeginCell(True) : rtf.WriteString(σs.ToString("#0.00 ")) : rtf.EndCell()
            rtf.DeclareCell(5500) : rtf.BeginCell(True)
            If σs > 0.8 * aci.fyk Then
                rtf.WriteString("contrainte acier - σs > 0,8.fyk = " & (0.8 * aci.fyk).ToString("#0.0") & " MPa - NS") : rtf.EndCell() : rtf.EndRow()
            Else
                rtf.WriteString("contrainte acier - σs < 0,8.fyk = " & (0.8 * aci.fyk).ToString("#0.0") & " MPa - OK") : rtf.EndCell() : rtf.EndRow()
            End If

        End With
        rtf.WriteEndGroup()
    End Sub
    Private Sub Fissuration_Calcul_Direct_4appuis(ByVal i1 As Integer, ByRef rtf As To_Rtf.RtfAnsiTextWriter, ByRef verif As Boolean, ByRef Maitrise_Fissuration As Boolean)

        Dim EcmLt, Fcteff, kt, Lim_Fiss, MS, nLt, Srmax, wk, x, αe, ϕ, σs, εsm_cm, ρpeff As Double
        Dim i, boucle As Integer ' 
        'ELU_ELS = 4 'ELS quasi permanent
        Fcteff = bet.fctm
        rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        'Ligne 2
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Fct,eff") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("MPa") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Fcteff.ToString("#0.00")) : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("valeur moyenne de la résistance en traction du béton") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("k1 - K2 - K4") : rtf.EndCell()
        rtf.DeclareCell(1600) : rtf.BeginCell(True) : rtf.WriteString("= 0.8 :  = 0.5 :  = 0.425") : rtf.EndCell()
        rtf.DeclareCell(5500) : rtf.BeginCell(True) : rtf.WriteString("valeurs k1, K2 et K4 pour eq. 7.11") : rtf.EndCell() : rtf.EndRow()
        rtf.WriteEndGroup()

        With PlanCont.Item(i1 - 1)
            If .XD1 = True Or .XD2 = True Or .XD3 = True Or .XS1 = True Or .XS2 = True Or .XS3 = True Then
                Lim_Fiss = 0.0002
            ElseIf .XC2 = True Or .XC3 = True Or .XC4 = True Then
                Lim_Fiss = 0.0003
            Else
                Lim_Fiss = 0.0004
            End If
            'calcul du coefficient d'équivalence Acier/béton n
            If aci.n_Forcer_Fl = True Then
                'Calcul avec un coefficient n forcé
                nLt = aci.n_Fl '  : ϕ = (aci.n_Fl * bet.Ecm / aci.Es) - 1EcmLt = bet.Ecm / (1 + ϕ)
            Else
                'Suivant §7.4.3 et suivant article CSTC = conforme à l'EN1992-1-1
                Calcul_Coef_Fluage(Aff.t0_Fl, Aff.RH_Fl, Aff.tinf_Fl, ϕ, rtf)
                EcmLt = bet.Ecm / (1 + ϕ) : nLt = aci.Es / EcmLt     '  
            End If
            rtf.WriteStartGroup() : rtf.NewParagraph() : rtf.FontSize = 12 : rtf.FontSize = 9
            'acier inférieur longitudinal en travée
            rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Travée n°" & i1.ToString) : rtf.EndCell() : rtf.EndRow()
            rtf.WriteEndGroup()
            For i = 1 To 2
                'i=1 suivant X
                'i=2  suivant Y
                boucle = 1
                'Calcul de l'acier pour le moment maxi en travée
                If i = 1 Then
                    MS = .ELS_QP.Item(0).Mx_max.M
                    rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(" Ouverture de fissures de flexion en travée sous efforts suivant X") : rtf.EndCell() : rtf.EndRow()
                    rtf.WriteEndGroup()
                Else
                    MS = .ELS_QP.Item(0).My_Max.M
                    rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString(" Ouverture de fissures de flexion en travée sous efforts suivant Y") : rtf.EndCell() : rtf.EndRow()
                    rtf.WriteEndGroup()
                End If
                If MS = 0 Then 'And frmDon.MessagesInformationToolStripMenuItem.Checked = False
                    rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9
                    rtf.BeginRow() : rtf.DeclareCell(8300) : rtf.BeginCell(True) : rtf.WriteString("Sans objet - Moment nul") : rtf.EndCell() : rtf.EndRow() : rtf.WriteEndGroup()
                    boucle = 0
                Else
                    Call Verification_armature_flexion_ELS(i1, 1, MS, nLt, x, 0, 0, 0, σs,,,, i)
                    SrmaxEQ711_714(i1, 1, x, ρpeff, Srmax, rtf, i) 'calcul de Srmax suivant EQ7.11 ou  Eq7.14
                End If
                If boucle = 1 Then
                    kt = 0.4 'charge de longue durée
                    αe = aci.Es / bet.Ecm
                    εsm_cm = (σs - kt * (Fcteff / ρpeff) * (1 + αe * ρpeff)) / 200000
                    If εsm_cm < 0.6 * σs / 200000 Then εsm_cm = 0.6 * σs / 200000
                    wk = Srmax * εsm_cm
                    Call Tab_Ouverture_Fissure2(kt, MS, wk, σs, εsm_cm, αe, Lim_Fiss, rtf)
                End If
            Next
        End With
    End Sub
    ''' <summary>Calcul explicite de la flèche</summary>
    '''  <param name="rtf">variable renvoyant le fichier de note de calcul sous format rtf</param>
    ''' <param name="Fleche">variable prenant la valeur False si un calcul explicite de flèche est à réaliser, fleche(i) avec i n° de travée</param>
    Private Sub Calcul_Fleche_4appuis(ByVal Fleche() As Boolean, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        'ByVal ELU_ELS As Integer, ByVal Ind1() As Integer, ByVal Ind2() As Integer, ByVal M(,,) As Double, ByVal EIf(,,) As Double,
        'Calcul de la flèche suivant les conditions de l'article 7.4.3.  
        Dim i, i1, k, Comb As Integer ' x_Fl
        Dim A1, delta, EcmLt, d, FlMax, fctmFl, Fl_Tot, FlI, FlII, Iltnc, Ictnc, Iltc, Ictc, Imc, Mc, Mqp, nCt, nLt, S, XX1, XX2, XX3, x_Ictnc, x_Iltnc, x_Ictc, x_Iltc, xn, ϕ, β, ξ As Double
        β = 0.5
        If aci.n_Forcer_Fl = True Then
            'Calcul avec un coefficient n forcé
            nCt = aci.n_Fl : nLt = aci.n_Fl : ϕ = (aci.n_Fl * bet.Ecm / aci.Es) - 1
            EcmLt = bet.Ecm / (1 + ϕ)
            Call Tab_Initial_Fleche(True, ϕ, β, EcmLt, nCt, nLt, rtf)
        Else
            'Suivant §7.4.3 et suivant article CSTC = conforme à l'EN1992-1-1
            nCt = aci.Es / bet.Ecm
            Calcul_Coef_Fluage(Aff.t0_Fl, Aff.RH_Fl, Aff.tinf_Fl, ϕ, rtf)
            EcmLt = bet.Ecm / (1 + ϕ) : nLt = aci.Es / EcmLt
            Call Tab_Initial_Fleche(False, ϕ, β, EcmLt, nCt, nLt, rtf)
        End If

        For i1 = 1 To PlanCont.Count
            With PlanCont.Item(i1 - 1)
                If Fleche(i1) = False Then
                    rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                    rtf.WriteString("Travée n°" & i1.ToString & ": les élancements dépassent les limites autorisées par l'Eurocode, le calcul explicite des flèches est exigé.") : rtf.NewLine()
                    rtf.WriteEndGroup()
                    If Aff.ELSQP = 0 Then
                        rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                        rtf.WriteString("Aucune combinaison ELS quasi-permanente de charges n'a été renseignée - La verification vis à vis de la flèche n'est pas établie.") : rtf.NewLine() : rtf.NewLine()
                        rtf.WriteEndGroup()
                    Else

                        If i1 = 1 And .LappG < 0.05 Then
                            Select Case .Type_Acier_Appui_Droit
                                Case 1 'TS
                                    A1 = aci.As_long(.TS_AppD.Item(0).indTS)
                                    If .TS_AppD.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppD.Item(1).indTS)
                                Case 2 'barres HA
                                    If .LitSupD.Count = 2 Then A1 = (.LitSupD.Item(0).Aire + .LitSupD.Item(1).Aire) / .Ly Else A1 = .LitSupD.Item(0).Aire / .Ly
                            End Select
                            d = .dSupD
                        ElseIf i1 = PlanCont.Count And .LappD < 0.05 Then
                            Select Case .Type_Acier_Appui_Gauche
                                Case 1 'TS
                                    A1 = aci.As_long(.TS_AppG.Item(0).indTS)
                                    If .TS_AppG.Count = 2 Then A1 = A1 + aci.As_long(.TS_AppG.Item(1).indTS)
                                Case 2 'barres HA
                                    If .LitSupG.Count = 2 Then A1 = (.LitSupG.Item(0).Aire + .LitSupG.Item(1).Aire) / .Ly Else A1 = .LitSupG.Item(0).Aire / .Ly
                            End Select
                            d = .dSupG
                        Else
                            Select Case .Type_Acier_Travée
                                Case 1 'TS
                                    A1 = aci.As_long(.TS_trav.Item(0).indTS)
                                    If .TS_trav.Count = 2 Then A1 = A1 + aci.As_long(.TS_trav.Item(1).indTS)
                                Case 2 'barres HA
                                    If .LitInf.Count = 2 Then A1 = (.LitInf.Item(0).Aire + .LitInf.Item(1).Aire) / .Ly Else A1 = .LitInf.Item(0).Aire / .Ly
                            End Select
                            d = .dInf
                        End If
                        'plancher sur une largeur de 1 m.
                        '-> non fissurée
                        S = 1 * .H + (nCt - 1) * A1
                        x_Ictnc = (1 * .H ^ 2 / 2 + (nCt - 1) * A1 * d) / S
                        Ictnc = 1 * .H ^ 3 / 12 + 1 * .H * (.H / 2 - x_Ictnc) ^ 2 + (nCt - 1) * A1 * (d - x_Ictnc) ^ 2
                        S = 1 * .H + (nLt - 1) * A1
                        x_Iltnc = (1 * .H ^ 2 / 2 + (nLt - 1) * A1 * d) / S
                        Iltnc = 1 * .H ^ 3 / 12 + 1 * .H * (.H / 2 - x_Iltnc) ^ 2 + (nLt - 1) * A1 * (d - x_Iltnc) ^ 2
                        '->  fissurée
                        XX1 = 1 / 2
                        XX2 = nCt * A1
                        XX3 = -nCt * A1 * d
                        delta = XX2 ^ 2 - 4 * XX1 * XX3 : delta = Sqrt(delta)
                        x_Ictc = (-XX2 + delta) / (2 * XX1)
                        Ictc = 1 * x_Ictc ^ 3 / 3 + nCt * A1 * (d - x_Ictc) ^ 2
                        XX2 = nLt * A1
                        XX3 = -nLt * A1 * d
                        delta = XX2 ^ 2 - 4 * XX1 * XX3 : delta = Sqrt(delta)
                        x_Iltc = (-XX2 + delta) / (2 * XX1)
                        Iltc = 1 * x_Iltc ^ 3 / 3 + nLt * A1 * (d - x_Iltc) ^ 2

                        fctmFl = Max((1.6 - .H) * bet.fctm, bet.fctm)
                        'calcul de Imc suivant commentaires Paillé page 362 sauf pour la formule qui est fausse (voir CSTC)
                        xn = (1 * .H ^ 2 / 2 + nLt * A1 * d) / (1 * .H + nLt * A1)
                        Imc = 1 * .H ^ 3 / 12 + 1 * .H * (.H / 2 - xn) ^ 2 + nLt * A1 * (d - xn) ^ 2
                        Mc = fctmFl * Imc / (.H - xn)
                        ' Recherche de la flèche Max sour ELS QuasiPermanent
                        FlMax = 0 : Mqp = 0
                        For i = 1 To .ELS_QP.Count
                            If FlMax < Abs(.ELS_QP(i - 1).W_fl_min.wfl) Then
                                FlMax = Abs(.ELS_QP(i - 1).W_fl_min.wfl) : Comb = i - 1
                                For k = 1 To .ELS_QP(i - 1).Mx.Count
                                    If .ELS_QP(i - 1).W_fl_min.x = .ELS_QP(i - 1).W_fl.Item(k - 1).x And .ELS_QP(i - 1).W_fl_min.y = .ELS_QP(i - 1).W_fl.Item(k - 1).y Then Mqp = Abs(.ELS_QP(i - 1).Mx.Item(k - 1).M)
                                Next
                            End If
                        Next
                        If Mqp > Mc Then
                            ξ = 1 - β * (Mc / Mqp) ^ 2
                        Else
                            ξ = 0
                        End If
                        Select Case β
                            Case 1 'chargement unique de courte durée
                        '  FlII = EIFlMax / (bet.Ecm * Ictc) : FlI = EIFlMax / (bet.Ecm * Ictnc)
                            Case 0.5 'chargement prolongé
                                FlII = FlMax * (Iltnc / Iltc) : FlI = FlMax
                        End Select
                        Fl_Tot = ξ * FlII + (1 - ξ) * FlI
                        Call Tab_Intermediaire_Fleche(i1, Ictnc, x_Ictnc, Iltnc, x_Iltnc, Ictc, x_Ictc, Iltc, x_Iltc, Mc, Mqp, -10, Comb, ξ, FlI, FlII, Fl_Tot, .Lcx, rtf)
                    End If
                Else
                    rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewLine()
                    rtf.WriteString("Travée n°" & i1.ToString & ": Les élancements ne dépassent pas les limites autorisées par l'Eurocode, les conditions de flèche sont vérifiées.") : rtf.NewLine()
                    rtf.WriteEndGroup()
                End If
            End With
        Next

    End Sub

#Region "Calcul effort tranchant"
    Private Sub Effort_Tranchant_4appuis(ByVal Num_Chap As Integer, ByRef verif As Boolean, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim Crdc, d, fcd, fywd, k, V1, V2, Vrdc, Vrdc1, Vrdcmin, VEdMax, ν, ν1, νmin, ρ1, ρwmin As Double
        Dim VrdMax As Double ', xx At, AtSt, Atstmin, Stmax, Stmaxx,
        Dim i, i1, j As Integer
        Dim x1, y1, x2, y2 As Double 'i,imin, iMax,i2,i3,ik, ik1, ik2, phiDiamMax,sism, X,
        ' Dim msg As String
        'détermination des variables globales à l'ensemble des travées pour le calcul à l'effort tranchant
        fywd = aci.fyk / aci.γsd
        Crdc = 0.18 / bet.γcd
        ν = 0.6 * (1 - bet.fck / 250)
        ν1 = ν
        fcd = bet.fck / bet.γcd
        ρwmin = (0.08 * bet.fck ^ 0.5) / aci.fyk

        For i1 = 1 To PlanCont.Count
            rtf.WriteStartGroup()
            rtf.NewParagraph() :
            rtf.FontSize = 11 : rtf.Underline = True
            rtf.WriteString(Num_Chap.ToString & " - " & (i1 + 1).ToString & "  Travée n°" & i1.ToString) : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine() : rtf.Bold = True
            rtf.WriteString("Vérification de l'effort tranchant résistant de calcul") : rtf.NewLine() : rtf.Underline = False : rtf.Bold = False
            rtf.WriteEndGroup()
            'Vérification de la compression des bielles
            With PlanCont.Item(i1 - 1)
                'Vérification sur effort tranchant Maximum
                'Suivant x
                With .ELU_Str_Env
                    V1 = Abs(.Vx_Max.V) : x1 = .Vx_Max.x : y1 = .Vx_Max.y
                    If V1 < Abs(.Vx_min.V) Then V1 = Abs(.Vx_min.V) : x1 = .Vx_min.x : y1 = .Vx_min.y
                End With
                With .ELU_Str_Env 'suivant y
                    V2 = Abs(.Vy_Max.V) : x2 = .Vy_Max.x : y2 = .Vy_Max.y
                    If V2 < Abs(.Vy_min.V) Then V2 = Abs(.Vy_min.V) : x2 = .Vy_min.x : y2 = .Vy_min.y
                End With
                d = .dInf
                If .Type_Acier_Travée = 1 Then ρ1 = aci.As_long(.TS_trav.Item(0).indTS) Else ρ1 = .LitInf.Item(0).Aire / .Ly
                VEdMax = Max(V1, V2) 'effort tranchant maximum
                k = 1 + (200 / (d * 1000)) ^ 0.5
                If k > 2 Then k = 2
                νmin = (0.34 / 1.5) * bet.fck ^ 0.5
                ρ1 = ρ1 / (1 * d)
                If ρ1 > 0.02 Then ρ1 = 0.02
                Vrdcmin = (νmin * 1000 * d * 1000) / 1000.0 'unité: N -> KN
                Vrdc1 = (Crdc * k * (100 * ρ1 * bet.fck) ^ 1 / 3 * 1000 * (d * 1000)) / 1000 'unité: N -> KN
                Vrdc = Max(Vrdcmin, Vrdc1)
                VrdMax = 1 * 0.9 * .dInf * ν1 * fcd / (Cotgt(.Θ_Tr) + Tan(.Θ_Tr))
                Call Tab_Tranchant_4appuis(i1, V1, x1, y1, V2, x2, y2, d, Vrdc1, Vrdcmin, VrdMax, rtf)
                If V1 > VrdMax Or V2 > VrdMax Then
                    rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewParagraph()
                    rtf.WriteString("L'épaisseur de la dalle est insuffisante pour reprendre l'effort tranchant. Veuillez modifier la géométrie.") : rtf.NewLine()
                    rtf.WriteEndGroup()
                ElseIf V1 * 1000 > Max(Vrdc1, Vrdcmin) Or V2 * 1000 > Max(Vrdc1, Vrdcmin) Then
                    Call Calcul_Ferraillage_Tranchant_4appuis(i1, VEdMax, fywd, ρwmin, rtf) ' ║Ved║ > Vrdc -> nécessité armature d'effort tranchant")
                End If
            End With
        Next
    End Sub
    Private Sub Tab_Tranchant_4appuis(ByVal i1 As Integer, ByVal V1 As Double, ByVal x1 As Double, ByVal y1 As Double, ByVal V2 As Double, ByVal x2 As Double, ByVal y2 As Double, ByVal d As Double,
                                      ByVal Vrdc1 As Double, ByVal Vrdcmin As Double, ByVal VrdMax As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)

        rtf.WriteStartGroup()
        rtf.NewParagraph()
        rtf.FontSize = 12 : rtf.FontSize = 9
        'Ligne 1
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Variables") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("Unité") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString("Valeur") : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Observations") : rtf.EndCell() : rtf.EndRow()
        'Ligne 10
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("d") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("cm") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((d * 100).ToString("#0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("bras de levier pour calcul eq. 6.2a 6.2b et armatures effort tranchant") : rtf.EndCell() : rtf.EndRow()
        'Ligne 3
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ved_X") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((V1 * 1000).ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("║Ved║ suivant X aux coord. x= " & x1.ToString("#0.000") & " y= " & y1.ToString("#0.000") & "m.") : rtf.EndCell() : rtf.EndRow()
        'Ligne 5
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Ved_Y") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((V2 * 1000).ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("║Ved║ suivant Y aux coord. x= " & x2.ToString("#0.000") & " y= " & y2.ToString("#0.000") & "m.") : rtf.EndCell() : rtf.EndRow()
        'Ligne 7
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vrdc min") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Vrdcmin.ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Effort tranchant résistant de calcul - valeur minimale Vrdc eq 6.2b") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vrdc eq6.2a") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString(Vrdc1.ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Effort tranchant résistant de calcul - Vrdc suivant eq 6.2a") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vrdc") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((Max(Vrdc1, Vrdcmin)).ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True) : rtf.WriteString("Valeur retenue pour Effort tranchant résistant de calcul Vrdc") : rtf.EndCell() : rtf.EndRow()
        'Ligne 8
        rtf.BeginRow() : rtf.DeclareCell(8800) : rtf.BeginCell(True)
        If V1 * 1000 > Max(Vrdc1, Vrdcmin) Or V2 * 1000 > Max(Vrdc1, Vrdcmin) Then
            rtf.WriteString(" ║Ved║ > Vrdc -> nécessité armature d'effort tranchant")
        Else
            rtf.WriteString(" ║Ved║ < Vrdc - Plancher sans armature d'effort tranchant")
        End If
        rtf.EndCell() : rtf.EndRow()
        'Ligne 9
        rtf.BeginRow() : rtf.DeclareCell(1200) : rtf.BeginCell(True) : rtf.WriteString("Vrdmax") : rtf.EndCell()
        rtf.DeclareCell(600) : rtf.BeginCell(True) : rtf.WriteString("kN") : rtf.EndCell()
        rtf.DeclareCell(1000) : rtf.BeginCell(True) : rtf.WriteString((VrdMax * 1000).ToString("#,##0.0")) : rtf.EndCell()
        rtf.DeclareCell(6000) : rtf.BeginCell(True)
        'vérification de la compression des bielles
        If V1 > VrdMax Or V2 > VrdMax Then
            rtf.WriteString("VrdMax eq 6.9 - ║Ved║ > Vrdmax -> section béton insuffisante") : rtf.EndCell() : rtf.EndRow()
        Else
            rtf.WriteString("VrdMax eq 6.9 - ║Ved║ < Vrdmax -> compression bielle OK") : rtf.EndCell() : rtf.EndRow()
        End If
        rtf.WriteEndGroup()
    End Sub
    Private Sub Calcul_Ferraillage_Tranchant_4appuis(ByVal i1 As Integer, VEdMax As Double, fywd As Double, ρwmin As Double, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim i As Integer ' , sism, X, x1, x2  
        Dim msg As String = String.Empty
        Dim At, AtSt, n1, nbar, nbar1, Stmax_Long, Stmax_trans, xx, St As Double
        With PlanCont.Item(i1 - 1)
            'Détermination du diamètre des armatures d'effort tranchant - Ce diamètre reste valable sur l'ensemble du plancher
            'calcul de la valeur At/St
            AtSt = VEdMax / (0.9 * .dInf * fywd * Cotgt(.Θ_Tr)) 'EN1992-1- eq6.8
            If .Prefa = True Then AtSt = 1.15 * AtSt 'de base avant d'aller plus loin
            .HATr.d_Tr = .dInf
            'Espacement maximal transversal
            Stmax_trans = Truncate(1.5 * .HATr.d_Tr * 100) / 100
            nbar = 1 / Stmax_trans
            .HATr.φHA = aci.Ø_Norm(0) 'mini HA6
            'Détermination de l'espacement maximal entre cadre
            i = 0
            Do
                .HATr.φHA = aci.Ø_Norm(i) 'mini HA6
                '->longitudinal
                Stmax_Long = 0.75 * .HATr.d_Tr
                Stmax_Long = Min((nbar * .HATr.Aire) / (ρwmin * 1), Stmax_Long)
                Stmax_Long = Truncate(Stmax_Long * 100) / 100
                nbar1 = 1 / Stmax_Long
                At = nbar1 * nbar * .HATr.Aire
                St = At / AtSt 'espacement entre cadre dans le sens longitudinal
                i += 1
                If i = 3 Then msg = "Calcul armature effort tranchant - travée n°" & i1.ToString & " : Ø trop élevé - Calcul continué mais faux!" : Aff.msg_Avertiss.Add(msg) : Exit Do
            Loop While St > Stmax_Long
            'calcul de la répartition des cadres: depuis le nu gauche puis depuis le nu droit
            With .HATr
                .st_Long = Stmax_Long
                .st_Trans = Stmax_trans
                n1 = PlanCont.Item(i1 - 1).Lx / .st_Long
                St = PlanCont.Item(i1 - 1).Lx - n1 * .st_Long
                .st_Long_about = Truncate(St * 100) / 100
                n1 = PlanCont.Item(i1 - 1).Ly / .st_Trans
                St = PlanCont.Item(i1 - 1).Ly - n1 * .st_Trans
                .st_Trans_About = Truncate(St * 100) / 100
            End With
            'calcul de la longueur développée des cadres, épingles, étriers
            i = Indice_Acier(.HATr.φHA)
            'Ref: projet BA Thonier page 127
            .HATr.LtCadre = 0.1 + 2 * (.HATr.d_Tr - .enrSup) + .HATr.st_Trans + 1.426 * aci.dmT(i)
            .HATr.LtCadre = (Truncate(.HATr.LtCadre + 0.005) * 100) / 100
            'détermination du nbre de modèle
            '--> dans le sens lx
            i = CInt(Truncate(.Lx / .HATr.st_Long) + 1)
            '--> dans le sens ly
            xx = CDec(Truncate(.Ly / .HATr.st_Trans))
            xx = CInt(Math.Round(xx))
            'nbre total de modèle réparti sur la surface totale de la dalle
            .HATr.nModele = CInt(i * xx)
            rtf.WriteStartGroup()
            rtf.FontSize = 12 : rtf.FontSize = 10 : rtf.Underline = True : rtf.Bold = True
            rtf.WriteString(" Détermination des armatures") : rtf.Underline = False : rtf.Bold = False : rtf.NewLine()
            rtf.WriteString("Armatures d'effort tranchant retenues: " & .HATr.nModele.ToString & " modèles n°1 avec HA" & (.HATr.φHA * 1000).ToString()) : rtf.NewLine()
            rtf.WriteString("Répartition: " & i.ToString & " modèles dnas le sens x et " & xx.ToString & " modèles dans le sens y soit un total de " & .HATr.nModele.ToString & " modèles.") : rtf.NewLine()
            rtf.WriteString("Espacement sens X: " & (.HATr.st_Long_about * 100).ToString("#0.0") & " cm en about et " & (.HATr.st_Long * 100).ToString("#0.0") & " cm en partie courante.") : rtf.NewLine()
            rtf.WriteString("Espacement sens Y: " & (.HATr.st_Trans_About * 100).ToString("#0.0") & " cm en about et " & (.HATr.st_Trans * 100).ToString("#0.0") & " cm en partie courante.") : rtf.NewLine()
            rtf.WriteEndGroup()
        End With
    End Sub
#End Region

#Region "Verification Appuis"
    Private Sub verification_appui_Bidirectionnel(ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim i, i1, ELU_ELS As Integer ' k,
        ELU_ELS = 1 'provisoire
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 11 : rtf.NewLine()
        rtf.Underline = True : rtf.Bold = True : rtf.WriteString("5 - 1  Dimensions des appuis") : rtf.Underline = False : rtf.Bold = False : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine()
        rtf.WriteEndGroup()
        For i1 = 1 To PlanCont.Count
            With PlanCont.Item(i1 - 1)
                If .LappG > 0.05 Then Call Appui_About_4appuis(i1, 1, rtf) 'about gauche
                If .LappD > 0.05 Then Call Appui_About_4appuis(i1, 2, rtf) 'about droit
                If .LappH > 0.05 Then Call Appui_About_4appuis(i1, 3, rtf) 'about haut
                If .LappB > 0.05 Then Call Appui_About_4appuis(i1, 4, rtf) 'about bas

            End With
        Next
        '------------------------------------------------------------------------------
        'Affichage de la réacton d'appui pour chaque appui et pour chaque combinaison ELU
        '------------------------------------------------------------------------------
        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9
        rtf.Underline = True : rtf.Bold = True : rtf.WriteString("5 - 2  Valeurs des réactions d'appuis") : rtf.Underline = False : rtf.Bold = False : rtf.NewLine() : rtf.NewLine()
        rtf.NewParagraph()
        rtf.WriteString("(voir chapitre effort Tranchant)") : rtf.NewLine()
        rtf.WriteEndGroup()
    End Sub
    Private Sub Appui_About_4appuis(ByVal i1 As Integer, ByVal appui As Integer, ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim a1, a2, Comp, FEd, Lbielle, LappuiMin, VMax, Z As Double
        Dim CtΘ, Θa, σRd1, σRd2, σRdMax As Double
        Dim i As Integer

        rtf.WriteStartGroup()
        rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.Underline = True
        Select Case appui
            Case 1
                rtf.FontSize = 10 : rtf.WriteString("Travée " & i1.ToString & " :") : rtf.FontSize = 9 : rtf.NewLine() : rtf.NewLine() : rtf.WriteString("Appui gauche :")
            Case 2
                rtf.NewLine() : rtf.WriteString("Appui droit :")
            Case 3
                rtf.NewLine() : rtf.WriteString("Appui haut :")
            Case 4
                rtf.NewLine() : rtf.WriteString("Appui bas :")
            Case Else
                Aff.msg_Avertiss.Add("Travée " & i1.ToString & ": aucune valeur d'appui définie - calcul non effectué.")
                rtf.WriteEndGroup() : Exit Sub
        End Select
        rtf.Underline = False : rtf.NewLine() : rtf.Bold = True
        rtf.WriteString("  ■ Détermination de l'angle de la bielle d'about") : rtf.Bold = False : rtf.NewLine()
        rtf.WriteString("calcul de l'angle suivant disposition EN1992-1- Art.9.2.1.4(2)") : rtf.NewLine()

        With PlanCont.Item(i1 - 1)
            Z = 0.9 * .dInf
            Select Case appui
                Case 1
                    CtΘ = (.LappG + Z * Cotgt(.Θ_Tr) - .enrLat) / (2 * Z)
                Case 2
                    CtΘ = (.LappD + Z * Cotgt(.Θ_Tr) - .enrLat) / (2 * Z)
                Case 3
                    CtΘ = (.LappH + Z * Cotgt(.Θ_Tr) - .enrLat) / (2 * Z)
                Case 4
                    CtΘ = (.LappB + Z * Cotgt(.Θ_Tr) - .enrLat) / (2 * Z)
            End Select
            Θa = Atan(1 / CtΘ)
            rtf.WriteString("Angle de la bielle d'about Θ': " & (Θa * 180 / PI).ToString("#0.00") & "° (suivant eq.9.17 de l'AN)") : rtf.NewLine() : rtf.NewLine()
            rtf.Bold = True : rtf.WriteString("  ■ Vérification de la bielle en zone courante") : rtf.Bold = False : rtf.NewLine()
            rtf.WriteString("Application de la clause 6.5.2(2) de l'EN1992-1-1") : rtf.NewLine()
            σRdMax = 0.6 * (1 - bet.fck / 250) * bet.fck / bet.γcd
            rtf.WriteString("Contrainte maximale autorisée: σRdMax = 0.6.ν'.fcd (eq 6.56) soit: " & σRdMax.ToString("#0.00") & " MPa.") : rtf.NewLine()
            Select Case appui
                Case 1, 2
                    With .ELU_Str_Env
                        VMax = Max(Abs(.Vx_Max.V), Abs(.Vx_min.V))
                    End With
                Case 3, 4
                    With .ELU_Str_Env
                        VMax = Max(Abs(.Vy_Max.V), Abs(.Vy_min.V))
                    End With
            End Select
            FEd = VMax * CtΘ
            Comp = Sqrt(FEd ^ 2 + VMax ^ 2)
            Lbielle = Comp / 1 / σRdMax
            rtf.WriteString("Effort de compression dans la bielle : " & (Comp * 1000).ToString("#,##0.00") & " kN.") : rtf.NewLine()
            rtf.WriteString("Dimension transversale minimale de la bielle : " & (Lbielle * 100).ToString("#0.0") & " cm.") : rtf.NewLine() : rtf.NewLine()
            rtf.Bold = True : rtf.WriteString("  ■ Vérification du noeud") : rtf.Bold = False : rtf.NewLine()
            rtf.WriteString("Application de la clause 6.5.4(4).b) de l'EN1992-1-1: appui avec compression et traction et tirant dans une seule direction.") : rtf.NewLine()
            rtf.WriteString("La dimension de l'appui perpendiculairement au plancher est, à minima, égale à la dimension du plancher.") : rtf.NewLine()
            σRdMax = 0.85 * (1 - bet.fck / 250) * bet.fck / bet.γcd
            rtf.WriteString("Contrainte maximale autorisée: σRdMax = 0.85.ν'.fcd (eq 6.61) soit: " & σRdMax.ToString("#0.00") & " MPa.") : rtf.NewLine()
            rtf.WriteString("Effort vertical maximal: " & (VMax * 1000).ToString("#,##0.00") & " kN ( soit VEd au nu de l'appui - voir § Effort Tranchant).") : rtf.NewLine()
            Select Case appui
                Case 1 'appui gauche
                    a1 = .LappG - .enrLat  'fig 9.102 Pr NF EN 1992-1-1/NA: 2015
                Case 2 'appui droit
                    a1 = .LappD - .enrLat 'fig 9.102 Pr NF EN 1992-1-1/NA: 2015
                Case 3 'appui haut
                    a1 = .LappH - .enrLat  'fig 9.102 Pr NF EN 1992-1-1/NA: 2015
                Case 4 'appui bas
                    a1 = .LappB - .enrLat  'fig 9.102 Pr NF EN 1992-1-1/NA: 2015
            End Select
            a2 = (a1 + Cotgt(Θa) * (.enrInf + 0.01)) * Sin(Θa)
            rtf.WriteString("longueur d'appui a1: " & (a1 * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
            rtf.WriteString("longueur d'appui a2: " & (a2 * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
            σRd1 = VMax / (1 * a1) : σRd2 = VMax / (1 * a2 * Sin(Θa))
            rtf.WriteString("Facette appui: σRd1 (=Ved/bw.a1): " & σRd1.ToString("#0.00") & " MPa ")
            If σRd1 > σRdMax Then
                rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                LappuiMin = VMax / 1 / σRdMax
                rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
            Else
                rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
            End If
            rtf.WriteString("Facette poutre: σRd2 (=Ved/bw.a2.SinΘ'): " & σRd2.ToString("#0.00") & " MPa ")
            If σRd2 > σRdMax Then
                rtf.WriteString("> σRdMax - Pb: dimension de l'appui à revoir.") : rtf.NewLine()
                LappuiMin = VMax / 1 / σRdMax
                rtf.WriteString("Longueur minimale nécessaire de l'appui : " & (LappuiMin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
            Else
                rtf.WriteString("< σRdMax - OK") : rtf.NewLine()
            End If

        End With
        rtf.WriteEndGroup()
    End Sub
#End Region
    Private Sub Calcul_au_feu_4appuis(ByRef rtf As To_Rtf.RtfAnsiTextWriter)
        Dim da, enrfeu, hmin As Double
        Dim i As Integer
        rtf.WriteStartGroup() : rtf.FontSize = 12 : rtf.FontSize = 9 : rtf.NewParagraph()
        rtf.WriteString(" Calcul de stabilité au feu établi suivant méthode des valeurs tabulées tableau 5.8 §5.7.2 de l'EN1992-1-2.") : rtf.NewLine()
        '2 sens porteurs
        For i = 1 To PlanCont.Count
            With PlanCont.Item(i - 1)
                Select Case .Feu_REI 'tableau 5.8 EN1992-1-2
                    Case 1 '0.5h
                        hmin = 0.06 : enrfeu = 0.01
                    Case 2 '1h
                        hmin = 0.08
                        If .Ly / .Lx < 1.5 Then enrfeu = 0.01 Else enrfeu = 0.015
                    Case 3 '1.5h
                        hmin = 0.1 : If .Ly / .Lx < 1.5 Then enrfeu = 0.015 Else enrfeu = 0.02
                    Case 4 '2h
                        hmin = 0.12 : If .Ly / .Lx < 1.5 Then enrfeu = 0.02 Else enrfeu = 0.025
                    Case 5 '3h
                        hmin = 0.15 : If .Ly / .Lx < 1.5 Then enrfeu = 0.03 Else enrfeu = 0.04
                    Case 6 '4h
                        hmin = 0.175 : If .Ly / .Lx < 1.5 Then enrfeu = 0.04 Else enrfeu = 0.05
                End Select

                rtf.WriteString("- Travée " & i.ToString & ": ") : rtf.NewLine()
                '--> Vérification de la condition d'épaisseur de dalle
                If (.H + .feu_EpCarrelage) < hmin Then 'And Moment_Flechissant_Appui = False
                    rtf.WriteString("  - hauteur dalle insuffisante, doit être supérieure à = " & (hmin * 100).ToString("#0.0") & " cm minimum (l'épaisseur d'un revêtement de sol incombustible peut compter) - NS") : rtf.NewLine()
                    Aff.msg_Avertiss.Add("Calcul au feu - hauteur dalle insuffisante: pb sur travée " & i.ToString)
                ElseIf (.H + .feu_EpCarrelage) >= hmin Then
                    rtf.WriteString("  - hauteur dalle suffisante (+ ep carrelage éventuel), h minimal = " & (hmin * 100).ToString("#0.0") & " cm.") : rtf.NewLine()
                End If
                '--> Vérification de la condition de distance d'entr'axe
                da = .H - .dInf 'Quelque soit le type d'acier TS ou HA
                If da < enrfeu Then
                    rtf.WriteString("  - distance à l'axe de l'armature inférieure insuffisante, doit être supérieure à = " & (enrfeu * 100).ToString("#0.0") & " cm minimum - NS") : rtf.NewLine()
                    Aff.msg_Avertiss.Add("Calcul au feu - distance à l'axe de l'armature inférieure: pb sur travée " & i.ToString)
                Else
                    rtf.WriteString("  - distance à l'axe de l'armature inférieure suffisante, distance minimale = " & (enrfeu * 100).ToString("#0.0") & " cm - OK") : rtf.NewLine()
                End If
            End With
        Next
        rtf.WriteEndGroup()
    End Sub
    Private Sub Metre_4appuis()
        Dim Largeur, Reste As Double
        Dim i As Integer
        Dim modele_Identique As Boolean = True
        Dim msg As String = String.Empty
        For Each dalle As Plancher In PlanCont
            With dalle
                'aciers inférieurs
                Select Case .Type_Acier_Travée
                    Case 1 'TS
                        Call Repartition_Transversale_TS_Travée(dalle)
                        Call Repartition_Longitudinale_TS_Travée(dalle)
                    Case 2 'HA
                        Select Case .Repartition
                            Case 1 'uniforme
                                Call Repartition_Uniforme_HA_Travée(dalle)
                            Case 2 'variable
                                '  Call Repartition_Variable_HA_Travée(dalle)
                        End Select

                End Select
                'Aciers supérieurs
                'calcul des longueurs totales des aciers sur appuis porteurs
                '
                Select Case PlanCont.IndexOf(dalle)
                    Case 0 'travée n°1
                        If dalle.LappG > 0.05 And Aff.Reprise_M0_2_porteur = True Then
                            Select Case .Type_Acier_Appui_Gauche
                                Case 1 'TS
                                    For i = 1 To .TS_AppG.Count
                                        With .TS_AppG
                                            .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier
                                        End With
                                    Next
                                Case 2 'HA
                                    For i = 1 To .LitSupG.Count
                                        With .LitSupG
                                            .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier
                                        End With
                                    Next
                            End Select
                        End If
                    Case PlanCont.Count - 1 'dernière travée
                        'Appui gauche de la dernière travée
                        Select Case .Type_Acier_Appui_Gauche
                            Case 1 'TS
                                For i = 1 To .TS_AppG.Count
                                    With .TS_AppG
                                        .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                        With PlanCont.Item(PlanCont.IndexOf(dalle) - 1)
                                            .TS_AppD.Item(i - 1).Lacier2 = .Lx - .TS_AppD.Item(i - 1).X1acier
                                        End With
                                        .Item(i - 1).Lacier1 = PlanCont.Item(PlanCont.IndexOf(dalle) - 1).TS_AppD.Item(i - 1).Lacier2 + dalle.LappG + .Item(i - 1).Lacier2
                                        PlanCont.Item(PlanCont.IndexOf(dalle) - 1).TS_AppD.Item(i - 1).Lacier1 = .Item(i - 1).Lacier1
                                    End With
                                Next
                            Case 2 'HA
                                For i = 1 To .LitSupG.Count
                                    With .LitSupG
                                        .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                        With PlanCont.Item(PlanCont.IndexOf(dalle) - 1)
                                            .LitSupD.Item(i - 1).Lacier2 = .Lx - .LitSupD.Item(i - 1).X1acier
                                        End With
                                        .Item(i - 1).Lacier1 = PlanCont.Item(PlanCont.IndexOf(dalle) - 1).LitSupD.Item(i - 1).Lacier2 + dalle.LappG + .Item(i - 1).Lacier2
                                        PlanCont.Item(PlanCont.IndexOf(dalle) - 1).LitSupD.Item(i - 1).Lacier1 = .Item(i - 1).Lacier1
                                    End With
                                Next
                        End Select
                        'Appui droit de la dernière travée
                        If dalle.LappD > 0.05 And Aff.Reprise_M0_2_porteur = True Then
                            Select Case .Type_Acier_Appui_Droit
                                Case 1 'TS
                                    For i = 1 To .TS_AppD.Count
                                        With .TS_AppD
                                            .Item(i - 1).Lacier2 = dalle.Lx - .Item(i - 1).X1acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                        End With
                                    Next
                                Case 2 'HA
                                    For i = 1 To .LitSupD.Count
                                        With .LitSupD
                                            .Item(i - 1).Lacier2 = dalle.Lx - .Item(i - 1).X1acier
                                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2 + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                        End With
                                    Next
                            End Select
                        End If
                    Case Else 'appuis intermédiaires
                        Select Case .Type_Acier_Appui_Gauche
                            Case 1 'TS
                                For i = 1 To .TS_AppG.Count
                                    With .TS_AppG
                                        .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                        With PlanCont.Item(PlanCont.IndexOf(dalle) - 1)
                                            .TS_AppD.Item(i - 1).Lacier2 = .Lx - .TS_AppD.Item(i - 1).X1acier
                                        End With
                                        .Item(i - 1).Lacier1 = PlanCont.Item(PlanCont.IndexOf(dalle) - 1).TS_AppD.Item(i - 1).Lacier2 + dalle.LappG + .Item(i - 1).Lacier2
                                        PlanCont.Item(PlanCont.IndexOf(dalle) - 1).TS_AppD.Item(i - 1).Lacier1 = .Item(i - 1).Lacier1
                                    End With
                                Next
                            Case 2 'HA
                                For i = 1 To .LitSupG.Count
                                    With .LitSupG
                                        .Item(i - 1).Lacier2 = .Item(i - 1).X2acier
                                        With PlanCont.Item(PlanCont.IndexOf(dalle) - 1)
                                            .LitSupD.Item(i - 1).Lacier2 = .Lx - .LitSupD.Item(i - 1).X1acier
                                        End With
                                        .Item(i - 1).Lacier1 = PlanCont.Item(PlanCont.IndexOf(dalle) - 1).LitSupD.Item(i - 1).Lacier2 + dalle.LappG + .Item(i - 1).Lacier2
                                        PlanCont.Item(PlanCont.IndexOf(dalle) - 1).LitSupD.Item(i - 1).Lacier1 = .Item(i - 1).Lacier1
                                    End With
                                Next
                        End Select
                End Select
                'Aciers supérieurs
                'calcul des longueurs totales des aciers sur appuis NON porteurs
                ' -- Sur appuis haut et bas
                Select Case .Type_Acier_Travée
                    Case 1 'TS
                        ' -- Sur appuis haut
                        If .TS_AppH.Count >= 1 Then
                            With .TS_AppH
                                .Item(0).Lacier2 = .Item(0).X2acier
                                .Item(0).Lacier1 = .Item(0).Lacier2 + .Item(0).L11acier + .Item(0).L12acier + .Item(0).L13acier
                            End With
                        End If
                        ' -- Sur appuis bas
                        If .TS_AppB.Count >= 1 Then
                            With .TS_AppB
                                .Item(0).Lacier2 = .Item(0).X2acier
                                .Item(0).Lacier1 = .Item(0).Lacier2 + .Item(0).L11acier + .Item(0).L12acier + .Item(0).L13acier
                            End With
                        End If
                    Case 2 'HA
                        ' -- Sur appuis haut
                        If .LitSupH.Count >= 1 Then
                            With .LitSupH
                                .Item(0).Lacier2 = .Item(0).X2acier
                                .Item(0).Lacier1 = .Item(0).Lacier2 + .Item(0).L11acier + .Item(0).L12acier + .Item(0).L13acier
                            End With
                        End If
                        ' -- Sur appuis bas
                        If .LitSupB.Count >= 1 Then
                            With .LitSupB
                                .Item(0).Lacier2 = .Item(0).X2acier
                                .Item(0).Lacier1 = .Item(0).Lacier2 + .Item(0).L11acier + .Item(0).L12acier + .Item(0).L13acier
                            End With
                        End If
                End Select
                '
                '--> Détermination du nombre de panneaux sur les appuis
                ' -- Sur appuis d'about et appuis intermédiaires
                Select Case .Type_Acier_Appui_Gauche
                    Case 1 'TS
                        If .LappB > 0.05 Then
                            .TS_AppG.Item(0).LacierY.Clear()
                            For i = 1 To .TS_AppG.Count
                                Select Case i
                                    Case 1
                                        Largeur = aci.Larg_TS(.TS_AppG.Item(0).indTS)
                                        .TS_AppG.Item(0).LacierY.Add(Largeur)
                                        Reste = .Ly Mod Largeur
                                        .TS_AppG.Item(0).nbrePanneaux = CInt(Fix(.Ly / Largeur))
                                        Select Case Reste
                                            Case Is > 2
                                                .TS_AppG.Item(0).nbrePanneaux = .TS_AppG.Item(0).nbrePanneaux + 1
                                            Case Is < 0.3 ' EspMaxPrincipal = 0.25 pour charge concentrée
                                                'pas de modification du nombre de panneaux
                                            Case Else
                                                'ajout d'un modèle
                                                .TS_AppG.Item(0).LacierY.Add(Truncate(Reste * 100 + 1) / 100) 'arrondi au cm supérieur
                                        End Select
                                    Case 2
                                        Largeur = aci.Larg_TS(.TS_AppG.Item(1).indTS)
                                        .TS_AppG.Item(1).LacierY.Add(Largeur)
                                        Reste = .Ly Mod Largeur
                                        .TS_AppG.Item(1).nbrePanneaux = CInt(Fix(.Ly / Largeur))
                                        Select Case Reste
                                            Case Is > 2 '2 mètres
                                                .TS_AppG.Item(1).nbrePanneaux = .TS_AppG.Item(1).nbrePanneaux + 1
                                            Case Is < 0.3 ' EspMaxPrincipal = 0.25 pour charge concentrée
                                                'pas de modification du nombre de panneaux
                                            Case Else
                                                'ajout d'un modèle
                                                .TS_AppG.Item(1).LacierY.Add(Truncate(Reste * 100 + 1) / 100) 'arrondi au cm supérieur
                                        End Select
                                End Select
                            Next
                        End If
                    Case 2 'barres HA

                End Select
                'dernière travée
                If PlanCont.IndexOf(dalle) = PlanCont.Count - 1 Then
                    Select Case .Type_Acier_Appui_Droit
                        Case 1 'TS
                            If .LappD > 0.05 Then
                                .TS_AppD.Item(0).LacierY.Clear()
                                For i = 1 To .TS_AppD.Count
                                    Select Case i
                                        Case 1
                                            Largeur = aci.Larg_TS(.TS_AppD.Item(0).indTS)
                                            .TS_AppD.Item(0).LacierY.Add(Largeur)
                                            Reste = .Ly Mod Largeur
                                            .TS_AppD.Item(0).nbrePanneaux = CInt(Fix(.Ly / Largeur))
                                            Select Case Reste
                                                Case Is > 2
                                                    .TS_AppD.Item(0).nbrePanneaux = .TS_AppD.Item(0).nbrePanneaux + 1
                                                Case Is < 0.3 ' EspMaxPrincipal = 0.25 pour charge concentrée
                                                    'pas de modification du nombre de panneaux
                                                Case Else
                                                    'ajout d'un modèle
                                                    .TS_AppD.Item(0).LacierY.Add(Truncate(Reste * 100 + 1) / 100) 'arrondi au cm supérieur
                                            End Select
                                        Case 2
                                            Largeur = aci.Larg_TS(.TS_AppD.Item(1).indTS)
                                            .TS_AppD.Item(1).LacierY.Add(Largeur)
                                            Reste = .Ly Mod Largeur
                                            .TS_AppD.Item(1).nbrePanneaux = CInt(Fix(.Ly / Largeur))
                                            Select Case Reste
                                                Case Is > 2
                                                    .TS_AppD.Item(1).nbrePanneaux = .TS_AppD.Item(1).nbrePanneaux + 1
                                                Case Is < 0.3 ' EspMaxPrincipal = 0.25 pour charge concentrée
                                                    'pas de modification du nombre de panneaux
                                                Case Else
                                                    'ajout d'un modèle
                                                    .TS_AppD.Item(1).LacierY.Add(Truncate(Reste * 100 + 1) / 100) 'arrondi au cm supérieur
                                            End Select
                                    End Select
                                Next
                            End If
                        Case 2 'barres HA

                    End Select
                End If
                ' -- Sur appuis haut et bas
                If .TS_AppH.Count >= 1 Then
                    .TS_AppH.Item(0).LacierY.Clear()
                    If .TS_AppH.Item(0).indTS = 0 Then 'ST10 sélectionnée
                        Largeur = aci.Long_TS(.TS_AppH.Item(0).indTS) 'soit 4.8
                        .TS_AppH.Item(0).LacierY.Add(Largeur)
                        Reste = .Lx Mod Largeur
                        .TS_AppH.Item(0).nbrePanneaux = CInt(Fix(.Lx / Largeur))
                        Select Case Reste
                            Case Is > 0.8 * Largeur
                                .TS_AppH.Item(0).nbrePanneaux = .TS_AppH.Item(0).nbrePanneaux + 1
                            Case Is < 0.3 ' EspMaxPrincipal = 0.25 pour charge concentrée
                                'pas de modification du nombre de panneaux
                            Case Else
                                'ajout d'un modèle
                                .TS_AppH.Item(0).LacierY.Add(Truncate(Reste * 100 + 1) / 100) 'arrondi au cm supérieur
                        End Select
                    End If
                End If
                If .TS_AppB.Count >= 1 Then
                    .TS_AppB.Item(0).LacierY.Clear()
                    If .TS_AppB.Item(0).indTS = 0 Then 'ST10 sélectionnée
                        Largeur = aci.Long_TS(.TS_AppB.Item(0).indTS) 'soit 4.8
                        .TS_AppB.Item(0).LacierY.Add(Largeur)
                        Reste = .Lx Mod Largeur
                        .TS_AppB.Item(0).nbrePanneaux = CInt(Fix(.Lx / Largeur))
                        Select Case Reste
                            Case Is > 0.8 * Largeur
                                .TS_AppB.Item(0).nbrePanneaux = .TS_AppB.Item(0).nbrePanneaux + 1
                            Case Is < 0.3 ' EspMaxPrincipal = 0.25 pour charge concentrée
                                'pas de modification du nombre de panneaux
                            Case Else
                                'ajout d'un modèle
                                .TS_AppB.Item(0).LacierY.Add(Truncate(Reste * 100 + 1) / 100) 'arrondi au cm supérieur
                        End Select
                    End If
                End If
            End With

        Next
    End Sub
    Private Sub Repartition_Longitudinale_TS_Travée(ByRef dalle As Plancher)
        Dim i As Integer
        With dalle
            For i = 1 To .TS_trav.Count
                With .TS_trav.Item(i - 1)
                    Select Case .Position
                        Case 1 'lit n°1
                            .Lacier2 = .L11acier + .L12acier + .L13acier + .L21acier + .L22acier + .L23acier
                            .Lacier2 = .Lacier2 + .X2acier - .X1acier
                            .Lacier1 = .Lacier2
                        Case 2  'lit n°2
                            If dalle.Lit2CouvertureTravee = True Then
                                'le lit n°2 couvre l'intégralité de la dalle
                                .Lacier2 = .L11acier + .L12acier + .L13acier + .L21acier + .L22acier + .L23acier
                                .Lacier2 = .Lacier2 + dalle.Lx 'pas de distinguo entre dalle et prédalle
                                .Lacier1 = .Lacier2
                            Else
                                'le lit n°2 couvre uniquement la partie nécessaire à la reprise de l'effort de flexion
                                .Lacier2 = .L11acier + .L12acier + .L13acier + .L21acier + .L22acier + .L23acier
                                .Lacier2 = .Lacier2 + .X2acier - .X1acier
                                .Lacier1 = .Lacier2
                            End If
                    End Select
                End With
            Next
        End With
    End Sub
    Private Sub Repartition_Transversale_TS_Travée(ByRef dalle As Plancher)
        Dim i, j As Integer
        With dalle
            For j = 1 To .Interval_Ainf.Count
                For i = 1 To .TS_trav.Count
                    With .TS_trav.Item(i - 1)
                        Select Case .Position
                            Case 1 'lit n°1
                                .Lacier2 = .L11acier + .L12acier + .L13acier + .L21acier + .L22acier + .L23acier
                                .Lacier2 = .Lacier2 + .X2acier - .X1acier
                                .Lacier1 = .Lacier2
                            Case 2  'lit n°2
                                If dalle.Lit2CouvertureTravee = True Then
                                    'le lit n°2 couvre l'intégralité de la dalle
                                    .Lacier2 = .L11acier + .L12acier + .L13acier + .L21acier + .L22acier + .L23acier
                                    .Lacier2 = .Lacier2 + dalle.Lx 'pas de distinguo entre dalle et prédalle
                                    .Lacier1 = .Lacier2
                                Else
                                    'le lit n°2 couvre uniquement la partie nécessaire à la reprise de l'effort de flexion
                                    .Lacier2 = .L11acier + .L12acier + .L13acier + .L21acier + .L22acier + .L23acier
                                    .Lacier2 = .Lacier2 + .X2acier - .X1acier
                                    .Lacier1 = .Lacier2
                                End If
                        End Select
                    End With
                Next
            Next

        End With
    End Sub
    Private Sub Repartition_Uniforme_HA_Travée(ByRef Dalle As Plancher)
        Dim i As Integer
        With Dalle
            For i = 1 To .LitInf.Count
                With .LitInf
                    'acier inférieur dans le sens longitudinal
                    Select Case i
                        Case 1 'lit n°1
                            .Item(i - 1).Lacier2 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                            .Item(i - 1).Lacier2 = .Item(i - 1).Lacier2 + .Item(i - 1).X2acier - .Item(i - 1).X1acier
                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2
                        Case 2  'lit n°2
                            If Dalle.Lit2CouvertureTravee = True Then
                                'le lit n°2 couvre l'intégralité de la dalle
                                .Item(i - 1).Lacier2 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                .Item(i - 1).Lacier2 = .Item(i - 1).Lacier2 + Dalle.Lx 'pas de distinguo entre dalle et prédalle
                                .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2
                            Else
                                'le lit n°2 couvre uniquement la partie nécessaire à la reprise de l'effort de flexion
                                .Item(i - 1).Lacier2 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                .Item(i - 1).Lacier2 = .Item(i - 1).Lacier2 + .Item(i - 1).X2acier - .Item(i - 1).X1acier
                                .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2
                            End If
                    End Select
                End With
            Next
            For i = 1 To .LitInf_Y.Count
                'acier inférieur dans le sens transversal
                With .LitInf_Y
                    Select Case i
                        Case 1 'lit n°1
                            .Item(i - 1).Lacier2 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                            .Item(i - 1).Lacier2 = .Item(i - 1).Lacier2 + .Item(i - 1).X2acier - .Item(i - 1).X1acier
                            .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2
                        Case 2  'lit n°2
                            If Dalle.Lit2CouvertureTravee = True Then
                                'le lit n°2 couvre l'intégralité de la dalle
                                .Item(i - 1).Lacier2 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                .Item(i - 1).Lacier2 = .Item(i - 1).Lacier2 + Dalle.Lx 'pas de distinguo entre dalle et prédalle
                                .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2
                            Else
                                'le lit n°2 couvre uniquement la partie nécessaire à la reprise de l'effort de flexion
                                .Item(i - 1).Lacier2 = .Item(i - 1).L11acier + .Item(i - 1).L12acier + .Item(i - 1).L13acier + .Item(i - 1).L21acier + .Item(i - 1).L22acier + .Item(i - 1).L23acier
                                .Item(i - 1).Lacier2 = .Item(i - 1).Lacier2 + .Item(i - 1).X2acier - .Item(i - 1).X1acier
                                .Item(i - 1).Lacier1 = .Item(i - 1).Lacier2
                            End If
                    End Select
                End With
            Next
            If .Ly > 11.7 And aci.Position_Portefeuille = True Then
                If .LitInf_Y.Count = 1 Then
                    Dim lit As New Ferraillage_HA
                    lit.nbar = .LitInf_Y.Item(0).nbar : lit.φHA = .LitInf_Y.Item(0).φHA : lit.Scourant = .LitInf_Y(0).Scourant
                    .LitInf_Y.Item(0).Lacier2 = 12 : .LitInf_Y.Item(0).L13acier = 0.05 : lit.L13acier = 0.05
                    lit.Lacier2 = .Ly - 11.95 + 40 * .LitInf_Y.Item(0).φHA
                    lit.Lacier2 = (Truncate(.LitInf_Y.Item(1).Lacier2 * 100) + 1) / 100 'arrondi au cm supérieur
                    .LitInf_Y.Add(lit)
                    lit = Nothing
                Else
                    Aff.msg_Avertiss.Add("Module Métré: ajout d'un 2ième lit transversal à un 2ième lit transversal = disposition non implémenté!")
                End If
            End If
        End With
    End Sub
#End Region

    Private Sub Poubelle()
        '        Private Sub Vérification_armature_construction(ByVal i1 As Integer, ByVal nbarre As Integer, ByVal phi1 As Double, ByVal A_appui As Double,
        '                                            ByVal nbre_lit As Integer, ByRef in2 As Integer)
        '    '-----------------------------------------------------------------------
        '    '    Modification des armatures de construction
        '    '------------------------------------------------------------------------
        '    Dim in1 As Integer
        '    'With PoutCont.Item(i1 - 1)
        '    '    If .HAConstruc.φHA <= phi1 Then   'phi1=diam. de la barre 2 ou 3
        '    '        If .HAConstruc.nbar <= nbarre Then
        '    '            in1 = Indice_Acier(phi1) : in2 = Indice_Acier(.HAConstruc.φHA)
        '    '            'si le nbre de lit=1 et le diam. des aciers reprenant 0.15*M0 à maximun 3 diamètre au-dessus des aciers de construction alors on augmente phi et
        '    '            'nbar pour compléter
        '    '            If nbre_lit = 1 And in1 < 6 And in1 < in2 + 3 Then 'And nbar(i1, 4, 1) + 2 >= nbarre
        '    '                .HAConstruc.nbar = nbarre ' si phi<phi+3 et nbar<nbar+2 alors on
        '    '                .HAConstruc.φHA = phi1     'augmente phi et nbar pour compléter
        '    '                .HAConstruc.Type = 1
        '    '            Else
        '    '                If .HAConstruc.Aire > 0.7 * A_appui Then '30% d'acier de const. à rajouter
        '    '                    For in2 = 1 To 8
        '    '                        If nbarre * PI * dio(in2) ^ 2 / 4 > A_appui Then Exit For
        '    '                    Next in2
        '    '                    .HAConstruc.Type = 1
        '    '                Else 'sinon les aciers de construction vont compléter les aciers sur appuis
        '    '                    .HAConstruc.nbar = nbarre
        '    '                    .HAConstruc.φHA = phi1
        '    '                    .HAConstruc.Type = 3
        '    '                    in2 = Indice_Acier(phi1)
        '    '                End If
        '    '            End If
        '    '        End If
        '    '    End If
        '    'End With

        'End Sub
        '        Private Sub Prolongement_Barre(ByVal i1 As Integer, ByRef Verif As Boolean, ByVal courbe_moment As Integer,
        '                                   ByVal x0(,,) As Integer, ByRef acier_construction() As Boolean)
        '    'Dim Mu, u1, u2, u3 As Double
        '    ''Vérifications que les limites du logiciel soient respectées
        '    'Select Case geom
        '    '    Case 2 'poutre rectangulaire
        '    '        If b(i1) <> b(i1 + 1) Or h(i1) <> h(i1 + 1) Then
        '    '            msg = "Le logiciel ne permet pas l'étude d'une poutre continue à inertie variable " &  Environment.NewLine & _
        '    '            "d'une travée à l'autre comportant une ou plusieurs travées soulevées"
        '    '            MsgBox(msg, MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "BAEL: Sub Prolongement_Barre")
        '    '            Verif = False : Exit Sub
        '    '        End If
        '    '    Case 4 'poutre en Té
        '    '        If b(i1) <> b(i1 + 1) Or h(i1) <> h(i1 + 1) _
        '    '        Or b0(i1) <> b0(i1 + 1) Or h0(i1) <> h0(i1 + 1) Then
        '    '            msg = "Le logiciel ne permet pas l'étude d'une poutre continue à inertie variable " & vbCrLf & _
        '    '            "d'une travée à l'autre comportant une ou plusieurs travées soulevées"
        '    '            MsgBox(msg, MsgBoxStyle.Critical Or MsgBoxStyle.OkOnly, "BAEL: Sub Prolongement_Barre")
        '    '            Verif = False : Exit Sub
        '    '        End If
        '    'End Select

        '    'If Ainf(ELU_ELS, i1, 2) > Ainf(ELU_ELS, i1, 3) Then
        '    '    'par souci de simplification, tous les aciers nécessaires sont prolongés au delà de l'appui
        '    '    inf(i1, 3) = 0
        '    '    phi(i1, 3, 1) = 0 : nbar(i1, 3, 1) = 0
        '    '    phi(i1, 3, 2) = 0 : nbar(i1, 3, 2) = 0
        '    '    phi(i1, 3, 3) = 0 : nbar(i1, 3, 3) = 0
        '    '    X2acier(i1, 2, 1) = l(i1) : X2acierLS(i1, 2, 1) = l(i1)
        '    '    L3acier(i1, 2, 1, 2) = Lap(i1 + 1 - consolG) : L2acier(i1, 2, 1, 2) = 0 : L1acier(i1, 2, 1, 2) = 0
        '    '    LacierTotal(i1, 2, 1, 2) = L3acier(i1, 2, 1, 2)
        '    '    Select Case inf(i1, 2)
        '    '        Case 2 '2 lits
        '    '            u2 = nbar(i1, 2, 2) * PI * phi(i1, 2, 2) ^ 2 / 4
        '    '            If Ainf(ELU_ELS, i1, 2) - u2 < Ainf(ELU_ELS, i1, 3) Then
        '    '                'le 2ieme lit doit être prolongé au delà de l'appui
        '    '                X2acier(i1, 2, 2) = l(i1) : X2acierLS(i1, 2, 2) = l(i1)
        '    '                L3acier(i1, 2, 2, 2) = Lap(i1 + 1 - consolG) : L2acier(i1, 2, 2, 2) = 0 : L1acier(i1, 2, 2, 2) = 0
        '    '                LacierTotal(i1, 2, 2, 2) = L3acier(i1, 2, 2, 2)
        '    '            Else
        '    '                'le 2ieme lit peut être arrëté avant l'appui
        '    '                'pas de modification des valeurs de X2acier et X2acierLS
        '    '            End If
        '    '        Case 3 ' 3 lits
        '    '            u3 = nbar(i1, 2, 3) * PI * phi(i1, 2, 3) ^ 2 / 4
        '    '            u2 = nbar(i1, 2, 2) * PI * phi(i1, 2, 2) ^ 2 / 4
        '    '            If Ainf(ELU_ELS, i1, 2) - u3 < Ainf(ELU_ELS, i1, 3) Then
        '    '                'le 3ieme lit doit être prolongé au delà de l'appui
        '    '                X2acier(i1, 2, 3) = l(i1) : X2acierLS(i1, 2, 3) = l(i1)
        '    '                L3acier(i1, 2, 2, 3) = Lap(i1 + 1 - consolG) : L2acier(i1, 2, 3, 2) = 0 : L1acier(i1, 2, 3, 2) = 0
        '    '                LacierTotal(i1, 2, 3, 2) = L3acier(i1, 2, 3, 2)
        '    '            Else
        '    '                'le 3ieme lit peut être arrëté avant l'appui
        '    '                'pas de modification des valeurs de X2acier et X2acierLS
        '    '                If Ainf(ELU_ELS, i1, 2) - u3 - u2 < Ainf(ELU_ELS, i1, 3) Then
        '    '                    'le 2ieme lit doit être prolongé au delà de l'appui
        '    '                    X2acier(i1, 2, 2) = l(i1) : X2acierLS(i1, 2, 2) = l(i1)
        '    '                    L3acier(i1, 2, 2, 2) = Lap(i1 + 1 - consolG) : L2acier(i1, 2, 2, 2) = 0 : L1acier(i1, 2, 2, 2) = 0
        '    '                    LacierTotal(i1, 2, 2, 2) = L3acier(i1, 2, 2, 2)
        '    '                Else
        '    '                    'le 3ieme et 2ieme lit peuvent être arrëtés avant l'appui
        '    '                    'pas de modification des valeurs de X2acier et X2acierLS déterminés à partir du moment de l'appui 1
        '    '                End If
        '    '                'le 2ieme lit peut être arrëté avant l'appui
        '    '                'pas de modification des valeurs de X2acier et X2acierLS déterminés à partir du moment de l'appui 1
        '    '            End If
        '    '    End Select
        '    'Else
        '    '    'Section d'acier appui 1< section d'acier appui 2

        '    '    Call Moment_resistant(i1, 3, Mu) 'calcul du moment repris par chaque lit d'acier
        '    '    Call Epure_lit_acier_appui_droit(Verif, i1, 3, courbe_moment, x0, acier_construction) : If Verif = False Then Exit Sub
        '    '    Call Entrainement_armature_haute(i1, Verif, 3) : If Verif = False Then Exit Sub 'vérification de l'entrainement des armatures hautes sur appuis pour les poutres Hyperstatiques
        '    '    u2 = X1acier(i1, 3, 1) : u3 = X1acierLS(i1, 3, 1)
        '    '    Stop
        '    '    'reste à faire
        '    '    inf(i1 - 1, 3) = inf(i1, 3)
        '    '    phi(i1 - 1, 3, 1) = phi(i1, 3, 1) : nbar(i1 - 1, 3, 1) = nbar(i1, 3, 1)
        '    '    phi(i1 - 1, 3, 2) = phi(i1, 3, 2) : nbar(i1 - 1, 3, 2) = nbar(i1, 3, 2)
        '    '    phi(i1 - 1, 3, 3) = phi(i1, 3, 3) : nbar(i1 - 1, 3, 3) = nbar(i1, 3, 3)

        '    '    Select Case inf(i1, 3)
        '    '        Case 2 '2 lits
        '    '            u2 = nbar(i1, 3, 2) * PI * phi(i1, 3, 2) ^ 2 / 4
        '    '            If Ainf(ELU_ELS, i1, 2) < Ainf(ELU_ELS, i1, 3) - u2 Then
        '    '                'le 2ieme lit peut être arrëté avant l'appui 1
        '    '                'pas de modification des valeurs de X1acier et X1acierLS
        '    '            Else
        '    '                'le 2ieme lit doit être prolongé au delà de l'appui 1
        '    '                X1acier(i1, 2, 2) = 0 : X1acierLS(i1, 2, 2) = 0
        '    '                L3acier(i1, 2, 2, 2) = Lap(i1 + 1 - consolG) : L2acier(i1, 2, 2, 2) = 0 : L1acier(i1, 2, 2, 2) = 0
        '    '                LacierTotal(i1, 2, 2, 2) = L3acier(i1, 2, 2, 2)
        '    '            End If
        '    '        Case 3 ' 3 lits
        '    '            u3 = nbar(i1, 2, 3) * PI * phi(i1, 2, 3) ^ 2 / 4
        '    '            u2 = nbar(i1, 2, 2) * PI * phi(i1, 2, 2) ^ 2 / 4
        '    '            If Ainf(ELU_ELS, i1, 2) - u3 < Ainf(ELU_ELS, i1, 3) Then
        '    '                'le 3ieme lit doit être prolongé au delà de l'appui
        '    '                X2acier(i1, 2, 3) = l(i1) : X2acierLS(i1, 2, 3) = l(i1)
        '    '                L3acier(i1, 2, 2, 3) = Lap(i1 + 1 - consolG) : L2acier(i1, 2, 3, 2) = 0 : L1acier(i1, 2, 3, 2) = 0
        '    '                LacierTotal(i1, 2, 3, 2) = L3acier(i1, 2, 3, 2)
        '    '            Else
        '    '                'le 3ieme lit peut être arrëté avant l'appui
        '    '                'pas de modification des valeurs de X2acier et X2acierLS
        '    '                If Ainf(ELU_ELS, i1, 2) - u3 - u2 < Ainf(ELU_ELS, i1, 3) Then
        '    '                    'le 2ieme lit doit être prolongé au delà de l'appui
        '    '                    X2acier(i1, 2, 2) = l(i1) : X2acierLS(i1, 2, 2) = l(i1)
        '    '                    L3acier(i1, 2, 2, 2) = Lap(i1 + 1 - consolG) : L2acier(i1, 2, 2, 2) = 0 : L1acier(i1, 2, 2, 2) = 0
        '    '                    LacierTotal(i1, 2, 2, 2) = L3acier(i1, 2, 2, 2)
        '    '                Else
        '    '                    'le 3ieme et 2ieme lit peuvent être arrëtés avant l'appui
        '    '                    'pas de modification des valeurs de X2acier et X2acierLS déterminés à partir du moment de l'appui 1
        '    '                End If
        '    '                'le 2ieme lit peut être arrëté avant l'appui
        '    '                'pas de modification des valeurs de X2acier et X2acierLS déterminés à partir du moment de l'appui 1
        '    '            End If
        '    '    End Select
        '    '    '    Case 1 '1 seul lit
        '    '    '        phi(i1, 2, 1) = phi(i1 - 1, 3, 1) : nbar(i1, 2, 1) = nbar(i1 - 1, 3, 1)
        '    '    '        phi(i1, 2, 2) = 0 : nbar(i1, 2, 2) = 0
        '    '    '        phi(i1, 2, 3) = 0 : nbar(i1, 2, 3) = 0
        '    '    '    Case 2 '2 lits
        '    '    '        phi(i1, 2, 1) = phi(i1 - 1, 3, 1) : nbar(i1, 2, 1) = nbar(i1 - 1, 3, 1)
        '    '    '        phi(i1, 2, 2) = phi(i1 - 1, 3, 2) : nbar(i1, 2, 2) = nbar(i1 - 1, 3, 2)
        '    '    '        phi(i1, 2, 3) = 0 : nbar(i1, 2, 3) = 0
        '    '    '    Case 3 ' 3 lits
        '    '    '        phi(i1, 2, 1) = phi(i1 - 1, 3, 1) : nbar(i1, 2, 1) = nbar(i1 - 1, 3, 1)
        '    '    '        phi(i1, 2, 2) = phi(i1 - 1, 3, 2) : nbar(i1, 2, 2) = nbar(i1 - 1, 3, 2)
        '    '    '        phi(i1, 2, 3) = phi(i1 - 1, 3, 3) : nbar(i1, 2, 3) = nbar(i1 - 1, 3, 3)
        '    '    'End Select
        '    '    inf(i1, 2) = inf(i1 - 1, 3)
        '    '    Mu = -m2d(ELU_ELS, i1, 0) 'Calcul de l'acier pour le moment maxi sur appui gauche
        '    '    Call Moment_resistant(i1, 2, Mu)  'calcul du moment repris par chaque lit d'acier
        '    '    Call Epure_lit_acier_appui_gauche(Verif, i1, 2, courbe_moment, x0, acier_construction) : If Verif = False Then Exit Sub
        '    '    Call Entrainement_armature_haute(i1, Verif, 2)  'vérification de l'entrainement des armatures hautes sur appuis pour les poutres Hyperstatiques
        '    'End If
        'End Sub


        '     Private Sub barre_construction(ByVal i1 As Integer)
        '    '----------------------------------------------------------
        '    ' Barres de construction (suivant COIN)
        '    '----------------------------------------------------------
        '    'nomenclature:
        '    'construction=1 signifie les aciers de construction sont suffisant pour reprendre les moments
        '    'construction=3 signifie les aciers de construction sont insuffisant pour reprendre les moments
        '    'il est alors rajouté des barres complémentaires
        '    '
        '    Dim appui, i, i2, in1, in2, j, nbarre As Integer 'i4,
        '    Dim a30, aconstr, A_appui, A_manquant, la(2) As Double
        '    Dim msg As String
        '    With PlanCont.Item(i1 - 1)
        '        MsgBox("Accès au module barre_construction Non Implémenté")
        '        'a30 = .LitInf.Item(0).Aire / 30 ' (nbar(i1, 1, 1) * PI * phi(i1, 1, 1) ^ 2 / 4) / 30
        '        'i2 = Indice_Acier(.HATr.φHA)
        '        'nbarre = .LitInf.Item(0).nbar
        '        'If i1 = 1 And .LappG <= 0.05 Then nbarre = .LitSupD.Item(0).nbar
        '        'If i1 = PoutCont.Count And .LappD <= 0.05 Then nbarre = .LitSupG.Item(0).nbar
        '        'For i = i2 To 8
        '        '    aconstr = nbarre * PI * dio(i) ^ 2 / 4
        '        '    If aconstr > a30 Then Exit For
        '        '    If i = 8 Then
        '        '        msg = "Acier de construction: problème de ferraillage, le diamètre des aciers de construction est supérieur à HA25. Revoir le dimensionnement de la" &
        '        '            " poutre."
        '        '        Aff.msg_Avertiss.Add(msg)
        '        '    End If
        '        'Next i
        '        '.HAConstruc.nbar = nbarre '  nbar(i1, 4, 1) = nbar(i1, j, 1)
        '        '.HAConstruc.φHA = dio(i) 'phi(i1, 4, 1) = dio(i)
        '        '.HAConstruc.X1acier = 0 ' X1acier(i1, 4, 1) = 0
        '        '.HAConstruc.X2acier = .L ' X2acier(i1, 4, 1) = l(i1)
        '        'A_appui = 0
        '        'Select Case i1
        '        '    Case 1
        '        '        'If acier_construction(1) = False Then
        '        '        '    'cas particulier ou les aciers sur l'appui droit sont prolongés jusqu'à l'appui gauche
        '        '        '    .HAConstruc.nbar = 0
        '        '        '    .HAConstruc.Type = 0 'pas d'acier de construction
        '        '        '    '            A_appui = 0 : inf(1, 4) = 0
        '        '        '    '            For i = 1 To inf(i1, 2) : A_appui = A_appui + nbar(i1, 2, i) * PI * phi(i1, 2, i) ^ 2 / 4 : Next i
        '        '        '    '            Dim A_appui1 As Double
        '        '        '    '            A_appui1 = nbar(i1, 3, 1) * PI * phi(i1, 3, 1) ^ 2 / 4
        '        '        '    '            If A_appui1 > A_appui Then
        '        '        '    '                inf(1, 2) = 0
        '        '        '    '            Else
        '        '        '    '                msg = "Problème de ferraillage d'acier sur appui!!!"
        '        '        '    '                MsgBox(msg, vbCritical Or vbOKOnly, "BAEL: barre_construction")
        '        '        '    '                Stop ': Exit Sub
        '        '        '    '            End If
        '        '        'Else
        '        '        If .LappG > 0.05 Then
        '        '            If PoutCont.Count = 1 Then 'travée unique sans console
        '        '                If Aff.Reprise_M0 = True Then 'appui légèrement encastré
        '        '                    A_appui = 0
        '        '                    If .HAConstruc.Aire >= .LitSupG.Item(0).Aire + .LitSupG.Item(1).Aire + .LitSupG.Item(2).Aire And
        '        '                        .HAConstruc.Aire >= .LitSupD.Item(0).Aire + .LitSupD.Item(1).Aire + .LitSupD.Item(2).Aire Then
        '        '                        .LitSupG_nblit = 0 : .LitSupD_nblit = 0
        '        '                        .HAConstruc.X1acier = 0
        '        '                        .HAConstruc.X2acier = .L
        '        '                        appui = 1 : .HAConstruc.Type = 1
        '        '                        Call Ancrage_total_lit(appui, i1, 1, .LappG - .enrLat, 4, 1, 0.8 * aci.fyk, 36, 40)
        '        '                        appui = 2
        '        '                        Call Ancrage_total_lit(appui, i1, 1, .LappD - .enrLat, 4, 1, 0.8 * aci.fyk, 36, 40)
        '        '                        .HAConstruc.Lacier2 = .HAConstruc.L11acier + .HAConstruc.L12acier + .HAConstruc.L13acier + .HAConstruc.X2acier + .HAConstruc.L21acier + .HAConstruc.L22acier + .HAConstruc.L23acier
        '        '                    Else 'Aciers de construction initiaux insuffisant pour reprendre 0.15M0
        '        '                        '-->  APPUI  GAUCHE
        '        '                        A_appui = .LitSupG.Item(0).Aire + .LitSupG.Item(1).Aire + .LitSupG.Item(2).Aire
        '        '                        Call Vérification_armature_construction(i1, .LitSupG.Item(0).nbar, .LitSupG.Item(0).φHA, A_appui, .LitSupG_nblit, in2) ' in1, in2)
        '        '                        If .HAConstruc.Type = 3 Then
        '        '                            'litsupG modifié: 1 seul lit avec des barres plus grosses et en complément des aciers de construction
        '        '                            .LitSupG_nblit = 1
        '        '                            A_manquant = A_appui - .HAConstruc.Aire
        '        '                            For i = in2 To 8
        '        '                                If .LitSupG.Item(0).nbar * PI * dio(in2) ^ 2 / 4 > A_manquant Then Exit For
        '        '                            Next i
        '        '                            .LitSupG.Item(0).φHA = dio(in2)
        '        '                            appui = 1 : j = 2
        '        '                            Call Ancrage_total_lit(appui, i1, 1, .LappG - .enrLat, 2, 1, 0.8 * aci.fyk, 38, 40)
        '        '                            .LitSupG.Item(0).Lacier2 = .LitSupG.Item(0).Lacier2 + .LitSupG.Item(0).X2acier
        '        '                            .LitSupG.Item(0).Lacier1 = .LitSupG.Item(0).Lacier2
        '        '                        Else
        '        '                            .LitSupG_nblit = 0 ' inf(i1, 2) = 0
        '        '                        End If
        '        '                        .HAConstruc.X1acier = 0 : .HAConstruc.X2acier = .L
        '        '                        appui = 1
        '        '                        Call Ancrage_total_lit(appui, i1, 1, .LappG - .enrLat, 4, 1, 0.8 * aci.fyk, 36, 40)
        '        '                        '--->  APPUI  DROIT
        '        '                        A_appui = .LitSupD.Item(0).Aire + .LitSupD.Item(1).Aire + .LitSupD.Item(2).Aire
        '        '                        Call Vérification_armature_construction(i1, .LitSupD.Item(0).nbar, .LitSupD.Item(0).φHA, A_appui, .LitSupD_nblit, in2)
        '        '                        If .HAConstruc.Type = 3 Then
        '        '                            'appui droit:'litsupG modifié: 1 seul lit avec des barres plus grosses et en complément des aciers de construction
        '        '                            .LitSupD_nblit = 1
        '        '                            A_manquant = A_appui - .HAConstruc.Aire
        '        '                            For i = in2 To 8
        '        '                                If .LitSupD.Item(0).nbar * PI * dio(in2) ^ 2 / 4 > A_manquant Then Exit For
        '        '                            Next i
        '        '                            .LitSupD.Item(0).φHA = dio(in2)
        '        '                            appui = 2 : j = 3
        '        '                            Call Ancrage_total_lit(appui, i1, 1, .LappD - .enrLat, j, 1, 0.8 * aci.fyk, 37, 40)
        '        '                            .LitSupD.Item(0).Lacier2 = .LitSupD.Item(0).Lacier2 + .LitSupD.Item(0).X2acier
        '        '                            .LitSupD.Item(0).Lacier1 = .LitSupD.Item(0).Lacier2
        '        '                        Else
        '        '                            .LitSupD_nblit = 0    '                        inf(i1, 3) = 0
        '        '                        End If
        '        '                        appui = 2 : j = 4
        '        '                        Call Ancrage_total_lit(appui, i1, 1, .LappD - .enrLat, j, 1, 0.8 * aci.fyk, 36, 40)
        '        '                        .HAConstruc.Lacier2 = .HAConstruc.L11acier + .HAConstruc.L12acier + .HAConstruc.L13acier + .HAConstruc.X2acier + .HAConstruc.L21acier + .HAConstruc.L22acier + .HAConstruc.L23acier
        '        '                    End If
        '        '                Else
        '        '                    'Par règle de bonne construction, les aciers de construction sont prolongés sur appui mais
        '        '                    'aucune vérification d'ancrage n'est faite
        '        '                    .HAConstruc.Type = 1
        '        '                    .HAConstruc.L13acier = .LappG - .enrLat
        '        '                    .HAConstruc.L23acier = .LappD - .enrLat
        '        '                    .HAConstruc.Lacier2 = .L + .LappG + .LappD - 2 * .enrLat
        '        '                    .HAConstruc.Lacier1 = .HAConstruc.Lacier2
        '        '                End If
        '        '            Else 'si la poutre comporte plusieurs travées et pas de console gauche
        '        '                If Aff.Reprise_M0 = True Then
        '        '                    If .HAConstruc.Aire >= .LitSupG.Item(0).Aire + .LitSupG.Item(1).Aire + .LitSupG.Item(2).Aire Then
        '        '                        .LitSupG_nblit = 0
        '        '                        .HAConstruc.X1acier = 0
        '        '                        .HAConstruc.X2acier = .L
        '        '                        .HAConstruc.L23acier = Truncate(.LappD * 100 / 2) / 100 'en m
        '        '                        .HAConstruc.L21acier = 0 : .HAConstruc.L22acier = 0
        '        '                        appui = 1 : .HAConstruc.Type = 1
        '        '                        Call Ancrage_total_lit(appui, i1, 1, .LappG - .enrLat, 4, 1, 0.8 * aci.fyk, 36, 40)
        '        '                        .HAConstruc.Lacier2 = .HAConstruc.Lacier2 + .HAConstruc.X2acier + .HAConstruc.L23acier
        '        '                    Else 'Aciers de construction initiaux insuffisant pour reprendre 0.15M0
        '        '                        A_appui = .LitSupG.Item(0).Aire + .LitSupG.Item(1).Aire + .LitSupG.Item(2).Aire
        '        '                        Call Vérification_armature_construction(i1, .LitSupG.Item(0).nbar, .LitSupG.Item(0).φHA, A_appui, .LitSupG_nblit, in2) ' in1, in2)
        '        '                        If .HAConstruc.Type = 3 Then
        '        '                            'litsupG modifié: 1 seul lit avec des barres plus grosses et en complément des aciers de construction
        '        '                            .LitSupG_nblit = 1
        '        '                            A_manquant = A_appui - .HAConstruc.Aire
        '        '                            For i = in2 To 8
        '        '                                If .LitSupG.Item(0).nbar * PI * dio(in2) ^ 2 / 4 > A_manquant Then Exit For
        '        '                            Next i
        '        '                            .LitSupG.Item(0).φHA = dio(in2)
        '        '                            appui = 1 : j = 2
        '        '                            Call Ancrage_total_lit(appui, i1, 1, .LappG - .enrLat, 2, 1, 0.8 * aci.fyk, 38, 40)
        '        '                            .LitSupG.Item(0).Lacier2 = .LitSupG.Item(0).Lacier2 + .LitSupG.Item(0).X2acier
        '        '                            .LitSupG.Item(0).Lacier1 = .LitSupG.Item(0).Lacier2
        '        '                        Else
        '        '                            .LitSupG_nblit = 0 ' inf(i1, 2) = 0
        '        '                        End If
        '        '                        .HAConstruc.X1acier = 0
        '        '                        .HAConstruc.X2acier = .L
        '        '                        .HAConstruc.L23acier = 0 'en m - Pour plus de commodité, on arrête l'acier au nu
        '        '                        .HAConstruc.L21acier = 0 : .HAConstruc.L22acier = 0
        '        '                        appui = 1
        '        '                        Call Ancrage_total_lit(appui, i1, 1, .LappG - .enrLat, 4, 1, 0.8 * aci.fyk, 36, 40)
        '        '                        .HAConstruc.Lacier2 = .HAConstruc.Lacier2 + .HAConstruc.X2acier + .HAConstruc.L23acier
        '        '                    End If
        '        '                Else
        '        '                    .LitSupG_nblit = 0
        '        '                    .HAConstruc.Type = 1
        '        '                    .HAConstruc.X1acier = 0
        '        '                    .HAConstruc.X2acier = .L
        '        '                    .HAConstruc.L23acier = 0 'en m - Pour plus de commodité, on arrête l'acier au nu
        '        '                    .HAConstruc.L21acier = 0 : .HAConstruc.L22acier = 0
        '        '                    .HAConstruc.L13acier = Truncate(.LappG * 100 / 2) / 100 'en m
        '        '                    .HAConstruc.L11acier = 0 : .HAConstruc.L12acier = 0
        '        '                    .HAConstruc.Lacier2 = .HAConstruc.L13acier + .HAConstruc.X2acier + .HAConstruc.L23acier
        '        '                    '                    Lacier1(i1, 4, 1) = Lacier2(i1, 4, 1)
        '        '                End If
        '        '            End If
        '        '        Else 'si la première travée est une console
        '        '            .HAConstruc.Type = 2
        '        '            'lit d'acier en partie basse
        '        '            .HAConstruc.X1acier = .enrLat
        '        '            .HAConstruc.X2acier = .L
        '        '            .HAConstruc.L23acier = (Truncate(.LappD * 100 / 2)) / 100
        '        '            .HAConstruc.L13acier = 0 : .HAConstruc.L12acier = 0 : .HAConstruc.L11acier = 0
        '        '            .HAConstruc.L22acier = 0 : .HAConstruc.L21acier = 0
        '        '            .HAConstruc.Lacier2 = .HAConstruc.L23acier + .HAConstruc.X2acier
        '        '        End If
        '        '    Case Is < PoutCont.Count
        '        '        .HAConstruc.Type = 1
        '        '        .HAConstruc.X1acier = 0
        '        '        .HAConstruc.L13acier = Truncate(.LappG * 100 / 2) / 100 'en m
        '        '        .HAConstruc.L11acier = 0 : .HAConstruc.L12acier = 0
        '        '        .HAConstruc.X2acier = .L
        '        '        .HAConstruc.L23acier = Truncate(.LappD * 100 / 2) / 100 'en m
        '        '        .HAConstruc.L21acier = 0 : .HAConstruc.L22acier = 0
        '        '        .HAConstruc.Lacier2 = .HAConstruc.L13acier + .HAConstruc.X2acier + .HAConstruc.L23acier
        '        '        .HAConstruc.Lacier1 = .HAConstruc.Lacier2
        '        '    Case PoutCont.Count
        '        '        'If acier_construction(PoutCont.Count) = False Then
        '        '        '    'cas particulier ou les aciers sur l'appui droit sont prolongés jusqu'à l'appui gauche
        '        '        '    .HAConstruc.nbar = 0
        '        '        '    .HAConstruc.Type = 0 'pas d'acier de construction
        '        '        '    'A_appui = 0 : inf(nbtrav, 4) = 0
        '        '        '    'For i = 1 To inf(i1, 3) : A_appui = A_appui + nbar(i1, 3, i) * PI * phi(i1, 3, i) ^ 2 / 4 : Next i
        '        '        '    'Dim A_appui1 As Double
        '        '        '    'A_appui1 = nbar(i1, 2, 1) * PI * phi(i1, 2, 1) ^ 2 / 4
        '        '        '    'If A_appui1 > A_appui Then
        '        '        '    '    inf(i1, 3) = 0
        '        '        '    'Else
        '        '        '    '    msg = "Problème de ferraillage d'acier sur appui!!!"
        '        '        '    '    MsgBox(msg, vbCritical Or vbOKOnly, "BAEL: barre_construction")
        '        '        '    '    Stop ': Exit Sub
        '        '        '    'End If
        '        '        'Else
        '        '        If .LappD > 0.05 Then
        '        '            If Aff.Reprise_M0 = True Then 'appui légèrement encastré
        '        '                If .HAConstruc.Aire > .LitSupD.Item(0).Aire + .LitSupD.Item(1).Aire + .LitSupD.Item(2).Aire Then
        '        '                    .LitSupD_nblit = 0
        '        '                    .HAConstruc.X1acier = 0
        '        '                    .HAConstruc.X2acier = .L
        '        '                    .HAConstruc.L13acier = Truncate(.LappG * 100 / 2) / 100 'en m
        '        '                    .HAConstruc.L11acier = 0 : .HAConstruc.L12acier = 0
        '        '                    appui = 2 : .HAConstruc.Type = 1
        '        '                    Call Ancrage_total_lit(appui, i1, 1, .LappD - .enrLat, 4, 1, 0.8 * aci.fyk, 36, 40)
        '        '                    .HAConstruc.Lacier2 = .HAConstruc.Lacier2 + .HAConstruc.X2acier + .HAConstruc.L23acier
        '        '                Else 'Aciers de construction initiaux insuffisant pour reprendre 0.15M0
        '        '                    A_appui = .LitSupD.Item(0).Aire + .LitSupD.Item(1).Aire + .LitSupD.Item(2).Aire
        '        '                    Call Vérification_armature_construction(i1, .LitSupD.Item(0).nbar, .LitSupD.Item(0).φHA, A_appui, .LitSupD_nblit, in2)
        '        '                    If .HAConstruc.Type = 3 Then
        '        '                        'appui droit:'litsupG modifié: 1 seul lit avec des barres plus grosses et en complément des aciers de construction
        '        '                        .LitSupD_nblit = 1
        '        '                        A_manquant = A_appui - .HAConstruc.Aire
        '        '                        For i = in2 To 8
        '        '                            If .LitSupD.Item(0).nbar * PI * dio(in2) ^ 2 / 4 > A_manquant Then Exit For
        '        '                        Next i
        '        '                        .LitSupD.Item(0).φHA = dio(in2)
        '        '                        appui = 2 : j = 3
        '        '                        Call Ancrage_total_lit(appui, i1, 1, .LappD - .enrLat, j, 1, 0.8 * aci.fyk, 37, 40)
        '        '                        .LitSupD.Item(0).Lacier2 = .LitSupD.Item(0).Lacier2 + .LitSupD.Item(0).X2acier
        '        '                        .LitSupD.Item(0).Lacier1 = .LitSupD.Item(0).Lacier2
        '        '                    Else
        '        '                        .LitSupD_nblit = 0    '                        inf(i1, 3) = 0
        '        '                    End If
        '        '                    .HAConstruc.X1acier = 0
        '        '                    .HAConstruc.X2acier = .L
        '        '                    .HAConstruc.L13acier = 0 'en m - Pour plus de commodité, on arrête l'acier au nu
        '        '                    .HAConstruc.L11acier = 0 : .HAConstruc.L12acier = 0
        '        '                    appui = 2 : j = 4
        '        '                    Call Ancrage_total_lit(appui, i1, 1, .LappD - .enrLat, j, 1, 0.8 * aci.fyk, 36, 40)
        '        '                    .HAConstruc.Lacier2 = .HAConstruc.Lacier2 + .HAConstruc.X2acier + .HAConstruc.L23acier
        '        '                End If
        '        '            Else
        '        '                .LitSupD_nblit = 0
        '        '                .HAConstruc.Type = 1
        '        '                .HAConstruc.X1acier = 0
        '        '                .HAConstruc.X2acier = .L
        '        '                .HAConstruc.L23acier = Truncate(.LappD * 100 / 2) / 100 'en m
        '        '                .HAConstruc.L21acier = 0 : .HAConstruc.L22acier = 0
        '        '                .HAConstruc.L13acier = 0 'en m - Pour plus de commodité, on arrête l'acier au nu
        '        '                .HAConstruc.L11acier = 0 : .HAConstruc.L12acier = 0
        '        '                .HAConstruc.Lacier2 = .HAConstruc.L13acier + .HAConstruc.X2acier + .HAConstruc.L23acier
        '        '            End If
        '        '        Else  'si la dernière travée est une console
        '        '            .HAConstruc.Type = 2
        '        '            'lit d'acier en partie basse
        '        '            .HAConstruc.X1acier = 0
        '        '            .HAConstruc.X2acier = .L - .enrLat
        '        '            .HAConstruc.L13acier = (Truncate(.LappG * 100 / 2)) / 100
        '        '            .HAConstruc.L11acier = 0 : .HAConstruc.L12acier = 0
        '        '            .HAConstruc.L21acier = 0 : .HAConstruc.L22acier = 0 : .HAConstruc.L23acier = 0
        '        '            .HAConstruc.Lacier2 = .HAConstruc.L13acier + .HAConstruc.X2acier
        '        '        End If
        '        'End Select
        '    End With

        '    ''a10 = ama(inf(i1, 1), i1, 1) / 10
        '    'j = 4
        '    'Ainf(1, i1, 4) = nbar(i1, 4, 1) * dio(i) ^ 2 * PI / 4

        '    'If geom = 2 And couture = 2 Then 'acier de couture de table pour poutre en Té
        '    '    nbar(i1, 4, 2) = 4
        '    '    phi(i1, 4, 2) = phi(i1, 4, 1)
        '    '    Lacier2(i1, 4, 2) = l(i1) + 0.04
        '    'End If
        'End Sub
    End Sub
End Module
